<?php
/**
 * Custom meta fields | Fields
 *
 * @package Betheme
 * @author Muffin group
 * @link https://muffingroup.com
 */

if( ! defined( 'ABSPATH' ) ){
	exit; // Exit if accessed directly
}

if( ! class_exists('Mfn_Builder_Fields') )
{
  class Mfn_Builder_Fields {

		private $options;

    private $sliders;
    private $animations;

    private $section;
    private $wrap;
    private $items;

    private $advanced;

    private $vb;

    /**
      * Constructor
    */

    public function __construct( $vb = false, $limited = false ) {

			// BeBlocks get items only

			if( 'items' == $limited ) {

        $this->sliders = array(
          'layer' => Mfn_Builder_Helper::get_sliders('layer'),
          'rev' => Mfn_Builder_Helper::get_sliders('rev'),
        );

				$this->options = Mfn_Builder_Helper::get_options();

				$this->set_items();

				return;

			}

			// Default constructor

    	$this->vb = $vb;

			$this->sliders = array(
        'layer' => Mfn_Builder_Helper::get_sliders('layer'),
        'rev' => Mfn_Builder_Helper::get_sliders('rev'),
      );

      $this->options = Mfn_Builder_Helper::get_options();

      $this->set_animations();

      $this->set_advanced();

      $this->set_section();
      $this->set_wrap();
      $this->set_items();

      if( ! $this->vb ){
				$this->get_advanced( $limited );
			}

    }

    /**
     * SET advanced fields
     */

    public function get_advanced( $limited = false ){

 			if( 'admin' == $limited && mfn_opts_get('builder-blocks') ){

 				$items_exclude = array('placeholder');

 				foreach ($this->items as $i=>$it) {
 	    		foreach($this->advanced as $adv){
 	    			if( !in_array($i, $items_exclude) ){
 	    				$this->items[$i]['attr'][] = $adv;
 	    			}
 	    		}
 	    	}

 			}

 			return $this->advanced;

    }

    /**
     * GET section fields
     */

    public function get_section(){

      return $this->section;

    }

    /**
     * GET wrap fields
     */

    public function get_wrap(){

      return $this->wrap;

    }

    /**
     * GET items
     */

    public function get_items(){

      return $this->items;

    }

    /**
     * GET item fields
     */

    public function get_item_fields( $item_type ){

      return $this->items[$item_type];

    }

		/**
		 * GET entrance animations
		 */

		public function get_animations(){

			return $this->animations;

		}

		/**
		 * GET placeholder image
		 */

		public function get_placeholder(){

			return get_theme_file_uri( '/muffin-options/svg/placeholders/image.svg' );

		}

		/**
		 * GET column editor
		 */

		public function get_column_editor(){

			$column_editor = 'textarea'; // codemirror

			if( $this->options['column-visual'] ){
				$column_editor = 'visual';
			}

			return $column_editor;

		}

    /**
     * SET section fields
   	 */

    private function set_section(){

  		$this->section = array(

  			/**
  			 * HEADER additional fields
  			 */

				array(
					'type' => 'html',
					'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
				),

				array(
					'type' => 'header',
					'class' => 'visible-header-builder',
					'title' => __('Header options', 'mfn-opts'),
				),

				array(
  				'id' => 'scroll-visibility',
  				'type' => 'switch',
  				'class' => 'visible-header-builder',
  				'title' => __('Scroll visibility', 'mfn-opts'),
  				'desc' => __('Sections hidden on the scroll should be at the top of the header. It takes effect with fixed type of Header', 'mfn-opts'),
  				'options' => array(
  					'show' => __('Default', 'mfn-opts'),
  					'hide' => __('Hide', 'mfn-opts'),
  				),
  				'std' => 'show'
  			),

				array(
  				'id' => 'closeable',
  				'attr_id' => 'closable-field',
  				'type' => 'switch',
  				'class' => 'visible-header-builder',
  				'title' => __('Closeable', 'mfn-opts'),
  				'options' => array(
  					'0' => __('Disabled', 'mfn-opts'),
  					'1' => __('Enabled', 'mfn-opts'),
  				),
  				'std' => '0'
  			),

  			array(
  				'id' => 'closeable-time',
  				'condition' => array( 'id' => 'closable-field', 'opt' => 'is', 'val' => '1' ),
  				'type' => 'text',
  				'class' => 'visible-header-builder narrow',
  				'title' => __('Show again after', 'mfn-opts'),
  				'desc' => __('How many days should it stay hidden after closing?', 'mfn-opts'),
  				'std' => '0',
  				'after' => 'days',
					'param' => 'number',
					'preview' => 'number',
  			),

  			array(
  				'id' => 'closeable-x',
  				'condition' => array( 'id' => 'closable-field', 'opt' => 'is', 'val' => '1' ),
  				'type' => 'switch',
  				'title' => __('Close button position', 'mfn-opts'),
  				'std' => 'left',
  				'options' => array(
  					'left' => __('Left', 'mfn-opts'),
  					'right' => __('Right', 'mfn-opts'),
  				),
  			),

  			array(
					'id' => 'style:.mcb-section-mfnuidelement .close-closeable-section .icon:color',
					'condition' => array( 'id' => 'closable-field', 'opt' => 'is', 'val' => '1' ),
					'type' => 'color',
					'title' => __('Close icon color', 'mfn-opts'),
				),

  			array(
  				'id' => 'title',
  				'class' => 'hidden-header-builder',
  				'type' => 'text',
  				'title' => __('Title', 'mfn-opts'),
  				'desc' => __('Label in admin panel only', 'mfn-opts'),
  			),

  			array(
  				'type' => 'header',
  				'class' => 'hidden-header-builder',
					'title' => __('Type', 'mfn-opts'),
				),

  			array(
  				'id' => 'type',
  				'attr_id' => 'section_type',
  				'class' => 'hidden-header-builder mfn-loop-switcher',
  				'type' => 'switch',
  				're_render' => true,
  				'title' => __('Type', 'mfn-opts'),
  				'desc' => __('Query loop displays section&apos;s content in designed loop', 'mfn-opts'),
  				'options' => array(
  					'' => __('Default', 'mfn-opts'),
  					'query' => __('Query loop', 'mfn-opts'),
  				),
  				'std' => ''
  			),

  			/* query type */

  			array(
  				'id' => 'query_type',
  				'attr_id' => 'query_type',
  				're_render' => true,
  				'class' => 'hidden-header-builder hidden-blog-builder hidden-portfolio-builder',
  				'condition' => array( 'id' => 'section_type', 'opt' => 'is', 'val' => 'query' ),
  				'type' => 'select',
  				'title' => __('Query type', 'mfn-opts'),
  				'options' => array(
  					'' => __('Select', 'mfn-opts'),
  					'posts' => __('Posts', 'mfn-opts'),
  					'terms' => __('Terms', 'mfn-opts')
  				),
  				'std' => ''
  			),

  			/* posts query builder */

  			array(
  				'id' => 'query_post_type',
  				'attr_id' => 'query_post_type',
  				'class' => 'hidden-header-builder hidden-blog-builder hidden-portfolio-builder',
  				're_render' => true,
  				'condition' => array( 'id' => 'query_type', 'opt' => 'is', 'val' => 'posts' ),
  				'type' => 'select',
  				'title' => __('Post type', 'mfn-opts'),
  				'options' => array(
  					'' => __('- Select -', 'mfn-opts'),
  					'post' => __('Post', 'mfn-opts'),
  					'page' => __('Page', 'mfn-opts'),
  					'portfolio' => __('Portfolio', 'mfn-opts'),
  					'offer' => __('Offer', 'mfn-opts'),
  					'client' => __('Clients', 'mfn-opts'),
  					'slide' => __('Slides', 'mfn-opts'),
  					'product' => __('Product', 'mfn-opts'),
  					'testimonial' => __('Testimonial', 'mfn-opts'),
  				),
  				'std' => ''
  			),

  			array(
					'id' => 'query_post_type_testimonial',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'testimonial' ),
					're_render' => true,
					'type' => 'multiselect',
					'title' => __('Categories included', 'mfn-opts'),
					//'options' => mfn_get_categories('testimonial-types'),
   				'js_hierarchical_options' => 'testimonial_types',
					'preview' => 'category',
				),

  			array(
					'id' => 'query_post_type_product',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'product' ),
					're_render' => true,
					'type' => 'multiselect',
					'title' => __('Categories included', 'mfn-opts'),
					//'hierarchical_options' => mfn_hierarchical_taxonomy('product_cat'),
					'js_hierarchical_options' => 'product_cat',
					'opt_append' => array(
						'0-current' => 'Current'
					),
					'preview' => 'category',
				),

				array(
					'id' => 'query_post_type_post',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'post' ),
					're_render' => true,
					'type' => 'multiselect',
					'title' => __('Categories included', 'mfn-opts'),
					'class' => 'hidden-blog-builder hidden-portfolio-builder',
					//'hierarchical_options' => mfn_get_categories('category'),
					'js_hierarchical_options' => 'blog_categories',
					'opt_append' => array(
						'0-current' => 'Current'
					),
					'preview' => 'category',
				),

				array(
					'id' => 'query_post_type_portfolio',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'portfolio' ),
					're_render' => true,
					'type' => 'multiselect',
					'title' => __('Categories included', 'mfn-opts'),
					//'hierarchical_options' => mfn_get_categories('portfolio-types'),
					'js_hierarchical_options' => 'portfolio_types',
					'opt_append' => array(
						'0-current' => 'Current'
					),
					'preview' => 'category',
				),

				array(
					'id' => 'query_post_type_client',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'client' ),
					're_render' => true,
					'type' => 'multiselect',
					'title' => __('Categories included', 'mfn-opts'),
					//'hierarchical_options' => mfn_get_categories('client-types'),
					'js_hierarchical_options' => 'client_types',
					'preview' => 'category',
				),

				array(
					'id' => 'query_post_type_offer',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'offer' ),
					're_render' => true,
					'type' => 'multiselect',
					'title' => __('Categories included', 'mfn-opts'),
					// 'hierarchical_options' => mfn_get_categories('offer-types'),
					'js_hierarchical_options' => 'offer_types',
					'preview' => 'category',
				),

				array(
					'id' => 'query_post_type_slide',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'slide' ),
					're_render' => true,
					'type' => 'multiselect',
					'title' => __('Categories included', 'mfn-opts'),
					// 'hierarchical_options' => mfn_get_categories('slide-types'),
					'js_hierarchical_options' => 'slide_types',
					'preview' => 'category',
				),










				array(
					'id' => 'query_post_type_testimonial_exclude',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'testimonial' ),
					're_render' => true,
					'type' => 'multiselect',
					'title' => __('Categories excluded', 'mfn-opts'),
					//'options' => mfn_get_categories('testimonial-types'),
   				'js_hierarchical_options' => 'testimonial_types',
					'preview' => 'category',
				),

  			array(
					'id' => 'query_post_type_product_exclude',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'product' ),
					're_render' => true,
					'type' => 'multiselect',
					'title' => __('Categories excluded', 'mfn-opts'),
					//'hierarchical_options' => mfn_hierarchical_taxonomy('product_cat'),
					'js_hierarchical_options' => 'product_cat',
					'opt_append' => array(
						'0-current' => 'Current'
					),
					'preview' => 'category',
				),

				array(
					'id' => 'query_post_type_post_exclude',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'post' ),
					're_render' => true,
					'class' => 'hidden-blog-builder hidden-portfolio-builder',
					'type' => 'multiselect',
					'title' => __('Categories excluded', 'mfn-opts'),
					//'hierarchical_options' => mfn_get_categories('category'),
					'js_hierarchical_options' => 'blog_categories',
					'opt_append' => array(
						'0-current' => 'Current'
					),
					'preview' => 'category',
				),

				array(
					'id' => 'query_post_type_portfolio_exclude',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'portfolio' ),
					're_render' => true,
					'type' => 'multiselect',
					'title' => __('Categories excluded', 'mfn-opts'),
					//'hierarchical_options' => mfn_get_categories('portfolio-types'),
					'js_hierarchical_options' => 'portfolio_types',
					'opt_append' => array(
						'0-current' => 'Current'
					),
					'preview' => 'category',
				),

				array(
					'id' => 'query_post_type_client_exclude',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'client' ),
					're_render' => true,
					'type' => 'multiselect',
					'title' => __('Categories excluded', 'mfn-opts'),
					//'hierarchical_options' => mfn_get_categories('client-types'),
					'js_hierarchical_options' => 'client_types',
					'preview' => 'category',
				),

				array(
					'id' => 'query_post_type_offer_exclude',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'offer' ),
					're_render' => true,
					'type' => 'multiselect',
					'title' => __('Categories excluded', 'mfn-opts'),
					// 'hierarchical_options' => mfn_get_categories('offer-types'),
					'js_hierarchical_options' => 'offer_types',
					'preview' => 'category',
				),

				array(
					'id' => 'query_post_type_slide_exclude',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'slide' ),
					're_render' => true,
					'type' => 'multiselect',
					'title' => __('Categories excluded', 'mfn-opts'),
					// 'hierarchical_options' => mfn_get_categories('slide-types'),
					'js_hierarchical_options' => 'slide_types',
					'preview' => 'category',
				),

				array(
					'id' => 'query_post_type_product_order',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'product' ),
					're_render' => true,
					'type' => 'select',
					'title' => __('Display', 'mfn-opts'),
					'desc' => __('This option turns off standard order settings', 'mfn-opts'),
					'options' => array(
						'' => __('Default', 'mfn-opts'),
						'on_sale' => __('On sale', 'mfn-opts'),
						'best_selling' => __('Best selling', 'mfn-opts'),
						'top_rated' => __('Top rated', 'mfn-opts'),
					),
					'std' => '',
				),

  			array(
  				'id' => 'query_post_orderby',
  				'class' => 'hidden-header-builder',
  				're_render' => true,
  				'condition' => array( 'id' => 'query_type', 'opt' => 'is', 'val' => 'posts' ),
  				'type' => 'select',
  				'title' => __('Order by', 'mfn-opts'),
  				'options' => array(
  					'' => __('None', 'mfn-opts'),
  					'ID' => __('ID', 'mfn-opts'),
  					'date' => __('Date', 'mfn-opts'),
  					'modified' => __('Modified', 'mfn-opts'),
  					'menu_order' => __('Menu order', 'mfn-opts'),
  					'title' => __('Title', 'mfn-opts'),
  					'rand' => __('Rand', 'mfn-opts'),
  				),
  				'std' => 'date'
  			),

  			array(
  				'id' => 'query_post_order',
  				're_render' => true,
  				'class' => 'hidden-header-builder',
  				'condition' => array( 'id' => 'query_type', 'opt' => 'is', 'val' => 'posts' ),
  				'type' => 'select',
  				'title' => __('Order', 'mfn-opts'),
  				'options' => array(
  					'ASC' => __('Ascending', 'mfn-opts'),
  					'DESC' => __('Descending', 'mfn-opts'),
  				),
  				'std' => 'ASC'
  			),

  			array(
  				'id' => 'query_post_per_page',
  				're_render' => true,
  				'class' => 'hidden-header-builder',
  				'condition' => array( 'id' => 'query_type', 'opt' => 'is', 'val' => 'posts' ),
  				'type' => 'text',
  				'title' => __('Limit', 'mfn-opts'),
  				'desc' => __('By default limit from Admin > Settings > Reading > Blog pages show at most', 'mfn-opts'),
  				'param' => 'number',
					'preview' => 'number',
					'std' => '0',
					'js_std' => 'per_page',
  			),

  			array(
  				'id' => 'query_post_offset',
  				're_render' => true,
  				'class' => 'hidden-header-builder',
  				'condition' => array( 'id' => 'query_type', 'opt' => 'is', 'val' => 'posts' ),
  				'type' => 'text',
  				'title' => __('Offset', 'mfn-opts'),
  				'desc' => __('This option omits provided quantity of posts (from the beginning)', 'mfn-opts'),
  				'param' => 'number',
					'preview' => 'number',
					'std' => '0',
  			),

  			array(
  				'id' => 'query_post_pagination',
  				'attr_id' => 'query_post_pagination',
  				're_render' => true,
  				'class' => 'hidden-header-builder',
  				'condition' => array( 'id' => 'query_type', 'opt' => 'is', 'val' => 'posts' ),
  				'type' => 'select',
  				'title' => __('Pagination', 'mfn-opts'),
  				'options' => array(
  					'' => __('Hidden', 'mfn-opts'),
  					'numbers' => __('Numbers + links', 'mfn-opts'),
  					'dots' => __('Dots + links', 'mfn-opts'),
  					'prevnext' => __('Prev / Next links', 'mfn-opts'),
  					'loadmore' => __('Load more', 'mfn-opts'),
  					'infiniteload' => __('Infinite load', 'mfn-opts'),
  				),
  				'std' => ''
  			),

  			/* terms query builder */

  			array(
  				'id' => 'query_terms_taxonomy',
  				'attr_id' => 'query_terms_taxonomy',
  				're_render' => true,
  				'class' => 'hidden-header-builder',
  				'condition' => array( 'id' => 'query_type', 'opt' => 'is', 'val' => 'terms' ),
  				'type' => 'select',
  				'title' => __('Taxonomy', 'mfn-opts'),
  				'options' => array(
  					'' => __('- Select -', 'mfn-opts'),
  					'category' => __('Post category', 'mfn-opts'),
  					'post_tag' => __('Post tag', 'mfn-opts'),
  					'portfolio_types' => __('Portfolio category', 'mfn-opts'),
  					'offer_types' => __('Offer category', 'mfn-opts'),
  					'testimonial_types' => __('Testimonial category', 'mfn-opts'),
  					'product_cat' => __('Product category', 'mfn-opts'),
  				),
  				'std' => 'category'
  			),

  			array(
  				'id' => 'query_terms_excludes_post_tag',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'post_tag' ),
  				're_render' => true,
  				'type' => 'multiselect',
  				'title' => __('Excludes', 'mfn-opts'),
  				'desc' => __('IDs of taxonomies to exclude', 'mfn-opts'),
  				//'options' => mfn_get_categories('post_tag'),
   				'js_hierarchical_options' => 'blog_tags',
  			),

  			array(
  				'id' => 'query_terms_excludes_client_types',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'client_types' ),
  				're_render' => true,
  				'type' => 'multiselect',
  				'title' => __('Excludes', 'mfn-opts'),
  				'desc' => __('IDs of taxonomies to exclude', 'mfn-opts'),
  				//'options' => mfn_get_categories('client-types'),
   				'js_hierarchical_options' => 'client_types',
  			),

  			array(
  				'id' => 'query_terms_excludes_portfolio_types',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'portfolio_types' ),
  				're_render' => true,
  				'type' => 'multiselect',
  				'title' => __('Excludes', 'mfn-opts'),
  				'desc' => __('IDs of taxonomies to exclude', 'mfn-opts'),
  				//'options' => mfn_get_categories('portfolio-types'),
   				'js_hierarchical_options' => 'portfolio_types',
  			),

				array(
  				'id' => 'query_terms_excludes_category',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'category' ),
  				're_render' => true,
  				'type' => 'multiselect',
  				'title' => __('Excludes', 'mfn-opts'),
  				'desc' => __('IDs of taxonomies to exclude', 'mfn-opts'),
  				//'hierarchical_options' => mfn_get_categories('category'),
					'js_hierarchical_options' => 'blog_categories',
  			),

  			array(
  				'id' => 'query_terms_excludes_offer_types',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'offer_types' ),
  				're_render' => true,
  				'type' => 'multiselect',
  				'title' => __('Excludes', 'mfn-opts'),
  				'desc' => __('IDs of taxonomies to exclude', 'mfn-opts'),
  				//'options' => mfn_get_categories('offer-types'),
   				'js_hierarchical_options' => 'offer_types',
  			),

  			array(
  				'id' => 'query_terms_excludes_testimonial_types',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'testimonial_types' ),
  				're_render' => true,
  				'type' => 'multiselect',
  				'title' => __('Excludes', 'mfn-opts'),
  				'desc' => __('IDs of taxonomies to exclude', 'mfn-opts'),
  				//'options' => mfn_get_categories('testimonial-types'),
   				'js_hierarchical_options' => 'testimonial_types',
  			),

  			array(
  				'id' => 'query_terms_excludes_product_cat',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'product_cat' ),
  				're_render' => true,
  				'type' => 'multiselect',
  				'title' => __('Excludes', 'mfn-opts'),
  				'desc' => __('IDs of taxonomies to exclude', 'mfn-opts'),
  				//'hierarchical_options' => mfn_hierarchical_taxonomy('product_cat'),
					'js_hierarchical_options' => 'product_cat',
  			),

  			array(
  				'id' => 'query_terms_includes_post_tag',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'post_tag' ),
  				're_render' => true,
  				'type' => 'multiselect',
  				'title' => __('Includes', 'mfn-opts'),
  				'desc' => __('IDs of taxonomies to include', 'mfn-opts'),
  				//'options' => mfn_get_categories('post_tag'),
   				'js_hierarchical_options' => 'blog_tags',
  			),

  			array(
  				'id' => 'query_terms_includes_client_types',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'client_types' ),
  				're_render' => true,
  				'type' => 'multiselect',
  				'title' => __('Includes', 'mfn-opts'),
  				'desc' => __('IDs of taxonomies to include', 'mfn-opts'),
  				//'options' => mfn_get_categories('client-types'),
   				'js_hierarchical_options' => 'client_types',
  			),

  			array(
  				'id' => 'query_terms_includes_portfolio_types',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'portfolio_types' ),
  				're_render' => true,
  				'type' => 'multiselect',
  				'title' => __('Includes', 'mfn-opts'),
  				'desc' => __('IDs of taxonomies to include', 'mfn-opts'),
  				//'options' => mfn_get_categories('portfolio-types'),
   				'js_hierarchical_options' => 'portfolio_types',
  			),

				array(
  				'id' => 'query_terms_includes_category',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'category' ),
  				're_render' => true,
  				'type' => 'multiselect',
  				'title' => __('Includes', 'mfn-opts'),
  				'desc' => __('IDs of taxonomies to include', 'mfn-opts'),
  				//'hierarchical_options' => mfn_get_categories('category'),
					'js_hierarchical_options' => 'blog_categories',
  			),

  			array(
  				'id' => 'query_terms_includes_offer_types',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'offer_types' ),
  				're_render' => true,
  				'type' => 'multiselect',
  				'title' => __('Includes', 'mfn-opts'),
  				'desc' => __('IDs of taxonomies to include', 'mfn-opts'),
  				//'options' => mfn_get_categories('offer-types'),
   				'js_hierarchical_options' => 'offer_types',
  			),

  			array(
  				'id' => 'query_terms_includes_testimonial_types',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'testimonial_types' ),
  				're_render' => true,
  				'type' => 'multiselect',
  				'title' => __('Includes', 'mfn-opts'),
  				'desc' => __('IDs of taxonomies to include', 'mfn-opts'),
  				//'options' => mfn_get_categories('testimonial-types'),
   				'js_hierarchical_options' => 'testimonial_types',
  			),

  			array(
  				'id' => 'query_terms_includes_product_cat',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'product_cat' ),
  				're_render' => true,
  				'type' => 'multiselect',
  				'title' => __('Includes', 'mfn-opts'),
  				'desc' => __('IDs of taxonomies to exclude', 'mfn-opts'),
  				//'hierarchical_options' => mfn_hierarchical_taxonomy('product_cat'),
					'js_hierarchical_options' => 'product_cat',
					'opt_append' => array(
						'0-current' => 'Current'
					),
  			),

  			array(
  				'id' => 'query_terms_child_of_product_cat',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'product_cat' ),
  				're_render' => true,
  				'type' => 'select',
  				'title' => __('Child of', 'mfn-opts'),
					'js_hierarchical_options' => 'product_cat',
					'opt_append' => array(
						'0-current' => 'Current'
					),
  			),

  			array(
  				'id' => 'query_terms_orderby',
  				're_render' => true,
  				'class' => 'hidden-header-builder',
  				'condition' => array( 'id' => 'query_type', 'opt' => 'is', 'val' => 'terms' ),
  				'type' => 'select',
  				'title' => __('Order by', 'mfn-opts'),
  				'options' => array(
  					'' => __('None', 'mfn-opts'),
  					'term_order' => __('Term order', 'mfn-opts'),
  					'term_id' => __('ID', 'mfn-opts'),
  					'name' => __('Name', 'mfn-opts'),
  					'count' => __('Count', 'mfn-opts'),
  				),
  				'std' => 'ID'
  			),

  			array(
  				'id' => 'query_terms_order',
  				're_render' => true,
  				'class' => 'hidden-header-builder',
  				'condition' => array( 'id' => 'query_type', 'opt' => 'is', 'val' => 'terms' ),
  				'type' => 'select',
  				'title' => __('Order', 'mfn-opts'),
  				'options' => array(
  					'ASC' => __('Ascending', 'mfn-opts'),
  					'DESC' => __('Descending', 'mfn-opts'),
  				),
  				'std' => 'ASC'
  			),

  			array(
  				'id' => 'query_terms_hide_empty',
  				're_render' => true,
  				'class' => 'hidden-header-builder',
  				'condition' => array( 'id' => 'query_type', 'opt' => 'is', 'val' => 'terms' ),
  				'type' => 'select',
  				'title' => __('Hide empty', 'mfn-opts'),
  				'options' => array(
  					'' => __('False', 'mfn-opts'),
  					'1' => __('True', 'mfn-opts'),
  				),
  				'std' => ''
  			),

  			array(
  				'id' => 'query_terms_number',
  				're_render' => true,
  				'class' => 'hidden-header-builder',
  				'condition' => array( 'id' => 'query_type', 'opt' => 'is', 'val' => 'terms' ),
  				'type' => 'text',
  				'title' => __('Limit', 'mfn-opts'),
  				'param' => 'number',
					'preview' => 'number',
					'std' => '0',
  			),

  			/* END posts query builder */

  			array(
  				'type' => 'header',
  				'class' => 'hidden-header-builder',
					'title' => __('Style', 'mfn-opts'),
				),

				array(
  				'class' => 'hidden-header-builder',
  				'id' => 'style',
  				'type' => 'checkbox_pseudo',
  				'title' => __('Style', 'mfn-opts'),
  				'desc' => __('<strong>Notice!</strong> Options grayed out and marked with OLD tag are deprecated and will be removed in the future. Please use <a href="https://support.muffingroup.com/documentation/muffin-live-builder/positioning/" target="_blank">Positioning</a> instead.', 'mfn-opts'),
  				'options' => mfna_section_style(),
  			),

				// shape dividers

				array(
					'type' => 'header',
					'class' => 'hidden-header-builder',
					'title' => __('Shape divider', 'mfn-opts'),
				),

				array(
					'type' => 'html',
					'html' => '<div class="mfn-form-row hidden-header-builder mfn-sidebar-fields-tabs mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Top</a></li><li><a href="#hover" data-tab="hover">Bottom</a></li></ul><div class="mfn-sft mfn-sft-normal">',
				),

				array(
					'id' => 'shape_divider_type_top',
					'class' => 'hidden-header-builder',
					'attr_id' => 'shape_divider_type_top',
					'type' => 'select',
					'title' => __('Type', 'mfn-opts'),
					'options' => Mfn_Builder_Helper::get_shape_divider('options'),
					'std' => 0,
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-shape-divider-top svg:--mfn-shape-divider',
					'class' => 'hidden-header-builder',
					'type' => 'color',
					'title' => __('Color', 'mfn-opts'),
					'condition' => array( 'id' => 'shape_divider_type_top', 'opt' => 'isnt', 'val' => 0 ),
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-shape-divider-top svg:width',
					'class' => 'hidden-header-builder',
					'type' => 'sliderbar',
					'title' => __('Width %', 'mfn-opts'),
					'param' => array(
						'min' => '100',
						'max' => '300',
						'step' => '1',
						'unit' => '%',
					),
					'condition' => array( 'id' => 'shape_divider_type_top', 'opt' => 'isnt', 'val' => 0 ),
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-shape-divider-top svg:height',
					'type' => 'sliderbar',
					'title' => __('Height px', 'mfn-opts'),
					'param' => array(
						'min' => '0',
						'max' => '500',
						'step' => '1',
						'unit' => 'px',
					),
					'condition' => array( 'id' => 'shape_divider_type_top', 'opt' => 'isnt', 'val' => 0 ),
					'responsive' => 'desktop',
					'class' => 'hidden-header-builder',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-shape-divider-top svg:height_laptop',
					'type' => 'sliderbar',
					'title' => __('Height px', 'mfn-opts'),
					'param' => array(
						'min' => '0',
						'max' => '500',
						'step' => '1',
						'unit' => 'px',
					),
					'condition' => array( 'id' => 'shape_divider_type_top', 'opt' => 'isnt', 'val' => 0 ),
					'responsive' => 'laptop',
					'class' => 'hidden-header-builder',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-shape-divider-top svg:height_tablet',
					'type' => 'sliderbar',
					'title' => __('Height px', 'mfn-opts'),
					'param' => array(
						'min' => '0',
						'max' => '500',
						'step' => '1',
						'unit' => 'px',
					),
					'condition' => array( 'id' => 'shape_divider_type_top', 'opt' => 'isnt', 'val' => 0 ),
					'responsive' => 'tablet',
					'class' => 'hidden-header-builder',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-shape-divider-top svg:height_mobile',
					'type' => 'sliderbar',
					'title' => __('Height px', 'mfn-opts'),
					'param' => array(
						'min' => '0',
						'max' => '500',
						'step' => '1',
						'unit' => 'px',
					),
					'condition' => array( 'id' => 'shape_divider_type_top', 'opt' => 'isnt', 'val' => 0 ),
					'responsive' => 'mobile',
					'class' => 'hidden-header-builder',
				),

				array(
					'id' => 'shape_divider_flip_top',
					'type' => 'switch',
					'class' => 'hidden-header-builder',
					'title' => __('Flip', 'mfn-opts'),
					'options' => [
						0 => __('Disable', 'mfn-opts'),
						1 => __('Enable', 'mfn-opts'),
					],
					'std' => 0,
					'condition' => array( 'id' => 'shape_divider_type_top', 'opt' => 'isnt', 'val' => 0 ),
				),

				array(
					'id' => 'shape_divider_invert_top',
					'type' => 'switch',
					'class' => 'hidden-header-builder',
					'title' => __('Invert', 'mfn-opts'),
					'options' => [
						0 => __('Disable', 'mfn-opts'),
						1 => __('Enable', 'mfn-opts'),
					],
					'std' => 0,
					'condition' => array( 'id' => 'shape_divider_type_top', 'opt' => 'is', 'val' => Mfn_Builder_Helper::get_shape_divider('invert') ),
				),

				array(
					'id' => 'shape_divider_bring_front_top',
					'type' => 'switch',
					'class' => 'hidden-header-builder',
					'title' => __('Bring to front', 'mfn-opts'),
					'options' => [
						0 => __('Disable', 'mfn-opts'),
						1 => __('Enable', 'mfn-opts'),
					],
					'std' => 0,
					'condition' => array( 'id' => 'shape_divider_type_top', 'opt' => 'isnt', 'val' => 0 ),
				),

				array(
					'type' => 'html',
					'html' => '</div><div class="mfn-sft mfn-sft-hover">',
				),

				array(
					'class' => 'hidden-header-builder',
					'id' => 'shape_divider_type_bottom',
					'attr_id' => 'shape_divider_type_bottom',
					'type' => 'select',
					'title' => __('Type', 'mfn-opts'),
					'options' => Mfn_Builder_Helper::get_shape_divider('options'),
					'std' => 0,
				),

				array(
					'class' => 'hidden-header-builder',
					'id' => 'style:.mcb-section-mfnuidelement .mfn-shape-divider-bottom svg:--mfn-shape-divider',
					'type' => 'color',
					'title' => __('Color', 'mfn-opts'),
					'std' => '',
					'condition' => array( 'id' => 'shape_divider_type_bottom', 'opt' => 'isnt', 'val' => 0 ),
				),

				array(
					'class' => 'hidden-header-builder',
					'id' => 'style:.mcb-section-mfnuidelement .mfn-shape-divider-bottom svg:width',
					'type' => 'sliderbar',
					'title' => __('Width %', 'mfn-opts'),
					'param' => array(
						'min' => '100',
						'max' => '300',
						'step' => '1',
						'unit' => '%',
					),
					'condition' => array( 'id' => 'shape_divider_type_bottom', 'opt' => 'isnt', 'val' => 0 ),
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-shape-divider-bottom svg:height',
					'type' => 'sliderbar',
					'title' => __('Height px', 'mfn-opts'),
					'param' => array(
						'min' => '0',
						'max' => '500',
						'step' => '1',
						'unit' => 'px',
					),
					'condition' => array( 'id' => 'shape_divider_type_bottom', 'opt' => 'isnt', 'val' => 0 ),
					'responsive' => 'desktop',
					'class' => 'hidden-header-builder',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-shape-divider-bottom svg:height_laptop',
					'type' => 'sliderbar',
					'title' => __('Height px', 'mfn-opts'),
					'param' => array(
						'min' => '0',
						'max' => '500',
						'step' => '1',
						'unit' => 'px',
					),
					'condition' => array( 'id' => 'shape_divider_type_bottom', 'opt' => 'isnt', 'val' => 0 ),
					'responsive' => 'laptop',
					'class' => 'hidden-header-builder',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-shape-divider-bottom svg:height_tablet',
					'type' => 'sliderbar',
					'title' => __('Height px', 'mfn-opts'),
					'param' => array(
						'min' => '0',
						'max' => '500',
						'step' => '1',
						'unit' => 'px',
					),
					'condition' => array( 'id' => 'shape_divider_type_bottom', 'opt' => 'isnt', 'val' => 0 ),
					'responsive' => 'tablet',
					'class' => 'hidden-header-builder',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-shape-divider-bottom svg:height_mobile',
					'type' => 'sliderbar',
					'title' => __('Height px', 'mfn-opts'),
					'param' => array(
						'min' => '0',
						'max' => '500',
						'step' => '1',
						'unit' => 'px',
					),
					'condition' => array( 'id' => 'shape_divider_type_bottom', 'opt' => 'isnt', 'val' => 0 ),
					'responsive' => 'mobile',
					'class' => 'hidden-header-builder',
				),

				array(
					'id' => 'shape_divider_flip_bottom',
					'type' => 'switch',
					'class' => 'hidden-header-builder',
					'title' => __('Flip', 'mfn-opts'),
					'options' => [
						0 => __('Disable', 'mfn-opts'),
						1 => __('Enable', 'mfn-opts'),
					],
					'std' => 0,
					'condition' => array( 'id' => 'shape_divider_type_bottom', 'opt' => 'isnt', 'val' => 0 ),
				),

				array(
					'id' => 'shape_divider_invert_bottom',
					'type' => 'switch',
					'class' => 'hidden-header-builder',
					'title' => __('Invert', 'mfn-opts'),
					'options' => [
						0 => __('Disable', 'mfn-opts'),
						1 => __('Enable', 'mfn-opts'),
					],
					'std' => 0,
					'condition' => array( 'id' => 'shape_divider_type_bottom', 'opt' => 'is', 'val' => Mfn_Builder_Helper::get_shape_divider('invert') ),
				),

				array(
					'id' => 'shape_divider_bring_front_bottom',
					'type' => 'switch',
					'class' => 'hidden-header-builder',
					'title' => __('Bring to front', 'mfn-opts'),
					'options' => [
						0 => __('Disable', 'mfn-opts'),
						1 => __('Enable', 'mfn-opts'),
					],
					'std' => 0,
					'condition' => array( 'id' => 'shape_divider_type_bottom', 'opt' => 'isnt', 'val' => 0 ),
				),


				array(
					'type' => 'html',
					'html' => '</div></div>',
				),

  			// decoration

  			array(
  				'type' => 'header',
  				'class' => 'hidden-header-builder',
  				'title' => __('Background decoration', 'mfn-opts'),
  			),

  			array(
  				'class' => 'hidden-header-builder',
  				'id' => 'divider',
  				'type' => 'select',
  				'title' => __('Pattern', 'mfn-opts'),
  				'desc' => __('Please select background color in Advanced tab<br />Does <b>not</b> work with parallax and some section styles', 'mfn-opts'),
  				'options' => array(
  					'' => __('None', 'mfn-opts'),
  					'circle up' => __('Circle up', 'mfn-opts'),
  					'square up' => __('Square up', 'mfn-opts'),
  					'triangle up' => __('Triangle up', 'mfn-opts'),
  					'triple-triangle up' => __('Triple triangle up', 'mfn-opts'),
  					'circle down' => __('Circle down', 'mfn-opts'),
  					'square down' => __('Square down', 'mfn-opts'),
  					'triangle down' => __('Triangle down', 'mfn-opts'),
  					'triple-triangle down' => __('Triple triangle down', 'mfn-opts'),
  				),
  			),

  			array(
  				'class' => 'hidden-header-builder',
  				'id' => 'decor_top',
  				'type' => 'upload',
  				'title' => __('Image top', 'mfn-opts'),
  				'desc' => __('for images from <b> Media library</b><br/>Recommended width: 1920px', 'mfn-opts'),
  			),

  			array(
  				'class' => 'hidden-header-builder',
  				'id' => 'decor_bottom',
  				'type' => 'upload',
  				'title' => __('Image bottom', 'mfn-opts'),
					'desc' => __('for images from <b> Media library</b><br/>Recommended width: 1920px', 'mfn-opts'),
  			),

  			// options

  			array(
  				'type' => 'header',
  				'class' => 'hidden-header-builder',
  				'title' => __('Options', 'mfn-opts'),
  			),

				array(
					'class' => 'hidden-header-builder',
  				'id' => 'navigation',
  				'type' => 'select',
  				'title' => __('Navigation', 'mfn-opts'),
  				'options' => array(
  					'' => __('None', 'mfn-opts'),
  					'arrows' => __('Arrows', 'mfn-opts'),
  				),
  			),

				// hidden attributes

  			array(
  				'id' => 'hide',
  				'type' => 'text',
  				'title' => __('Hide', 'mfn-opts'),
  				'row_class' => 'hidden',
  				'class' => 'mfn-hidden-form-row',
  			),

  			array(
  				'id' => 'collapse',
  				'type' => 'text',
  				'title' => __('Collapse', 'mfn-opts'),
  				'row_class' => 'hidden',
  				'class' => 'mfn-hidden-form-row',
  			),

				// deprecated

  			array(
  				'type' => 'header',
  				'class' => 'mfn-deprecated',
  				'title' => __('Deprecated', 'mfn-opts'),
  			),

  			array(
  				'id' => 'bg_color',
  				'type' => 'color',
  				'class' => 'mfn-deprecated',
  				'title' => __('Background color', 'mfn-opts'),
  				'alpha' => true,
  			),

  			array(
  				'id' => 'bg_image',
  				'class' => 'mfn-deprecated',
  				'type' => 'upload',
  				'title' => __('Background image', 'mfn-opts'),
  				'desc' => __('Recommended image size <b>1920px x 1080px</b>', 'mfn-opts'),
  			),

  			array(
  				'id' => 'bg_position',
  				'class' => 'mfn-deprecated',
  				'type' => 'select',
  				'title' => __('Background position', 'mfn-opts'),
  				'desc' => __('iOS does <b>not</b> support background-position: fixed<br/>For parallax required background image size is at least 1920px x 1080px', 'mfn-opts'),
  				'options' => mfna_bg_position(),
  			),

  			array(
  				'id' => 'bg_size',
  				'class' => 'mfn-deprecated',
  				'type' => 'select',
  				'title' => __('Background size', 'mfn-opts'),
  				'desc' => __('Does <b>not</b> work with position fixed or parallax', 'mfn-opts'),
  				'options' => mfna_bg_size(),
  			),

  			array(
  				'id' => 'padding_top',
  				'class' => 'mfn-deprecated',
  				'type' => 'text',
  				'title' => __('Padding top', 'mfn-opts'),
  				'after' => 'px',
  			),

  			array(
  				'id' => 'padding_bottom',
  				'class' => 'mfn-deprecated',
  				'type' => 'text',
  				'title' => __('Padding bottom', 'mfn-opts'),
  				'after' => 'px',
  			),

  			array(
  				'id' => 'padding_horizontal',
  				'class' => 'mfn-deprecated',
  				'type' => 'text',
  				'title' => __('Padding left & right', 'mfn-opts'),
  				'desc' => __('Use <b>px</b> or <b>%</b>', 'mfn-opts'),
  			),

  			array(
  				'id' => 'visibility',
  				'class' => 'mfn-deprecated',
  				'type' => 'select',
  				'title' => __('Responsive visibility', 'mfn-opts'),
  				'options' => array(
  					'' => __('- Default -', 'mfn-opts'),
  					'hide-desktop' => __('Hide on Desktop | 960px +', 'mfn-opts'),			// 960 +
  					'hide-tablet' => __('Hide on Tablet | 768px - 959px', 'mfn-opts'),		// 768 - 959
  					'hide-mobile' => __('Hide on Mobile | - 768px', 'mfn-opts'),			// - 768
  					'hide-desktop hide-tablet' => __('Hide on Desktop & Tablet', 'mfn-opts'),
  					'hide-desktop hide-mobile' => __('Hide on Desktop & Mobile', 'mfn-opts'),
  					'hide-tablet hide-mobile' => __('Hide on Tablet & Mobile', 'mfn-opts'),
  				),
  			),

  			array(
  				'id' => 'class',
  				'class' => 'mfn-deprecated',
  				'type' => 'pills',
  				'title' => __('Section classes', 'mfn-opts'),
  			),

  			array(
  				'id' => 'section_id',
  				'class' => 'mfn-deprecated',
  				'type' => 'text',
  				'title' => __('Section ID', 'mfn-opts'),
  				'desc' => __('Use this option to create One Page sites<br />Example: Your <b>Section ID</b> is <b>offer</b> and you want to open this section, please use link: <b>your-url/#offer</b>', 'mfn-opts'),
					'param' => 'id',
  			),

				array(
					'type' => 'helper',
					'class' => 'mfn-deprecated',
					'title' => __('Need help', 'mfn-opts'),
					'link' => 'https://support.muffingroup.com/faq/what-deprecated-in-the-bebuilder-means/',
				),

  			array(
					'type' => 'html',
					'html' => '</div>', // end: modalbox-card
				),

				array(
					'type' => 'html',
					'html' => '<div class="modalbox-card modalbox-card-style">',
				),


				array(
					'type' => 'header',
					'class' => 'toggled_header row-header',
					'title' => __('Container', 'mfn-opts'),
				),

				array(
  				'id' => 'query_display',
  				'attr_id' => 'query_display',
  				'class' => 'mfn-toggled',
  				're_render' => true,
  				'type' => 'switch',
  				'title' => __('Display', 'mfn-opts'),
  				'options' => array(
  					'' => __('Default', 'mfn-opts'),
  					'slider' => __('Slider', 'mfn-opts'),
  				),
  				'std' => ''
  			),

  			array(
  				'id' => 'query_slider_columns',
  				'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
  				'class' => 'narrow mfn-toggled',
  				'responsive' => 'desktop',
  				'type' => 'text',
  				're_render' => true,
  				'title' => __('Columns', 'mfn-opts'),
  				'param' => 'number',
					'preview' => 'number',
					'std' => '1',
  			),

  			array(
  				'id' => 'query_slider_columns_laptop',
  				'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
  				'class' => 'narrow mfn-toggled',
  				'responsive' => 'laptop',
  				'type' => 'text',
  				're_render' => true,
  				'title' => __('Columns', 'mfn-opts'),
  				'param' => 'number',
					'preview' => 'number',
					'std' => '1',
  			),

  			array(
  				'id' => 'query_slider_columns_tablet',
  				'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
  				'class' => 'narrow mfn-toggled',
  				'responsive' => 'tablet',
  				'type' => 'text',
  				're_render' => true,
  				'title' => __('Columns', 'mfn-opts'),
  				'param' => 'number',
					'preview' => 'number',
					'std' => '1',
  			),

  			array(
  				'id' => 'query_slider_columns_mobile',
  				'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
  				'class' => 'narrow mfn-toggled',
  				'responsive' => 'mobile',
  				'type' => 'text',
  				're_render' => true,
  				'title' => __('Columns', 'mfn-opts'),
  				'param' => 'number',
					'preview' => 'number',
					'std' => '1',
  			),

  			/*array(
  				'id' => 'query_slider_animation',
  				'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
  				'class' => 'mfn-toggled',
  				'type' => 'switch',
  				'title' => __('Animation', 'mfn-opts'),
  				'options' => array(
  					'' => __('Slide', 'mfn-opts'),
  					'fade' => __('Fade', 'mfn-opts'),
  				),
  				'std' => ''
  			),*/

  			array(
  				'id' => 'query_slider_infinity',
  				'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
  				'class' => 'mfn-toggled',
  				'type' => 'switch',
  				'title' => __('Infinite loop', 'mfn-opts'),
  				'options' => array(
  					'' => __('Off', 'mfn-opts'),
  					'1' => __('On', 'mfn-opts'),
  				),
  				'std' => ''
  			),

  			array(
  				'id' => 'query_slider_autoplay',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'default_unit' => '',
					'class' => 'mfn-toggled',
					'title' => __('Autoplay speed', 'mfn-opts'),
					'desc' => __('Leave empty to disallow autoplay', 'mfn-opts'),
					'param' => array(
						'min' => '0',
						'max' => '10000',
						'step' => '100',
						'unit' => '',
					)
  			),

  			array(
  				'id' => 'query_slider_arrows',
  				'attr_id' => 'query_slider_arrows',
  				'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
  				'class' => 'mfn-toggled',
  				'type' => 'switch',
  				'title' => __('Arrows', 'mfn-opts'),
  				'options' => array(
  					'0' => __('Hide', 'mfn-opts'),
  					'1' => __('Show', 'mfn-opts'),
  				),
  				'std' => '0'
  			),

  			array(
  				'id' => 'query_slider_arrows_style',
  				'attr_id' => 'query_slider_arrows_style',
  				'condition' => array( 'id' => 'query_slider_arrows', 'opt' => 'is', 'val' => '1' ),
  				'class' => 'mfn-toggled',
  				'type' => 'switch',
  				'title' => __('Arrows style', 'mfn-opts'),
  				'options' => array(
  					'' => __('Standard', 'mfn-opts'),
  					'overlay' => __('Overlay', 'mfn-opts'),
  					'custom' => __('Custom', 'mfn-opts'),
  				),
  				'std' => ''
  			),

  			array(
  				'id' => 'query_slider_dots',
  				'attr_id' => 'query_slider_dots',
  				'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
  				'class' => 'mfn-toggled',
  				'type' => 'switch',
  				'title' => __('Dots', 'mfn-opts'),
  				'options' => array(
  					'0' => __('Hide', 'mfn-opts'),
  					'1' => __('Show', 'mfn-opts'),
  				),
  				'std' => '0'
  			),

  			array(
  				'id' => 'query_slider_dots_style',
  				'attr_id' => 'query_slider_dots_style',
  				'condition' => array( 'id' => 'query_slider_dots', 'opt' => 'is', 'val' => '1' ),
  				'class' => 'mfn-toggled',
  				'type' => 'switch',
  				'title' => __('Dots style', 'mfn-opts'),
  				'options' => array(
  					'' => __('Standard', 'mfn-opts'),
  					'overlay' => __('Overlay', 'mfn-opts'),
  					'custom' => __('Custom', 'mfn-opts'),
  				),
  				'std' => ''
  			),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .mcb-section-inner .swiper-pagination:transformtranslatex',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'type' => 'sliderbar',
					'class' => 'mfn-toggled',
					'title' => __('Translate(X)', 'mfn-opts'),
					'param' => array(
						'min' => '-100',
						'max' => '100',
						'step' => '1',
						'unit' => '%',
					)
				),

  			array(
  				'id' => 'query_slider_mousewheel',
  				'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
  				'class' => 'mfn-toggled',
  				'type' => 'switch',
  				'title' => __('Mousewheel', 'mfn-opts'),
  				'options' => array(
  					'' => __('Disabled', 'mfn-opts'),
  					'1' => __('Enabled', 'mfn-opts'),
  				),
  				'std' => ''
  			),

  			array(
  				'id' => 'query_slider_centered',
  				'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
  				'class' => 'mfn-toggled',
  				're_render' => true,
  				'type' => 'switch',
  				'title' => __('Center mode', 'mfn-opts'),
  				'options' => array(
  					'' => __('Disabled', 'mfn-opts'),
  					//'1' => __('Center', 'mfn-opts'),
  					'2' => __('Wrapper offset', 'mfn-opts'),
  				),
  				'std' => ''
  			),

  			array(
  				'id' => 'query_display_style',
  				'condition' => array( 'id' => 'query_display', 'opt' => 'isnt', 'val' => 'slider' ),
  				'class' => 'mfn-toggled',
  				// 're_render' => true,
  				'type' => 'switch',
  				'title' => __('Style', 'mfn-opts'),
  				'options' => array(
  					'' => __('Default', 'mfn-opts'),
  					'masonry' => __('Masonry', 'mfn-opts'),
  				),
  				'std' => ''
  			),

  			array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper.mfn-ql-item-default:width',
					'condition' => array( 'id' => 'query_display', 'opt' => 'isnt', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'default_unit' => 'px',
					'responsive' => 'desktop',
					'class' => 'mfn-toggled query_item_default_width sliderbar-units',
					'title' => __('Item width', 'mfn-opts'),
					'units' => array(
						'px' 	=> array('min' => 0, 'max' => 1920, 'step' => 1),
						'%' 	=> array('min' => 0, 'max' => 100, 'step' => 0.1),
						'vw' 	=> array('min' => 0, 'max' => 100, 'step' => 0.1),
					),
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper.mfn-ql-item-default:width_laptop',
					'condition' => array( 'id' => 'query_display', 'opt' => 'isnt', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'default_unit' => 'px',
					'responsive' => 'laptop',
					'class' => 'mfn-toggled query_item_default_width sliderbar-units',
					'title' => __('Item width', 'mfn-opts'),
					'units' => array(
						'px' 	=> array('min' => 0, 'max' => 1920, 'step' => 1),
						'%' 	=> array('min' => 0, 'max' => 100, 'step' => 1),
						'vw' 	=> array('min' => 0, 'max' => 100, 'step' => 1),
					),
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper.mfn-ql-item-default:width_tablet',
					'condition' => array( 'id' => 'query_display', 'opt' => 'isnt', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'default_unit' => 'px',
					'responsive' => 'tablet',
					'class' => 'mfn-toggled query_item_default_width sliderbar-units',
					'title' => __('Item width', 'mfn-opts'),
					'units' => array(
						'px' 	=> array('min' => 0, 'max' => 1920, 'step' => 1),
						'%' 	=> array('min' => 0, 'max' => 100, 'step' => 1),
						'vw' 	=> array('min' => 0, 'max' => 100, 'step' => 1),
					),
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper.mfn-ql-item-default:width_mobile',
					'condition' => array( 'id' => 'query_display', 'opt' => 'isnt', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'default_unit' => 'px',
					'responsive' => 'mobile',
					'class' => 'mfn-toggled query_item_default_width sliderbar-units',
					'title' => __('Item width', 'mfn-opts'),
					'units' => array(
						'px' 	=> array('min' => 0, 'max' => 1920, 'step' => 1),
						'%' 	=> array('min' => 0, 'max' => 100, 'step' => 0.1),
						'vw' 	=> array('min' => 0, 'max' => 100, 'step' => 0.1),
					),
				),

  			array(
  				'type' => 'header',
  				'class' => 'toggled_header row-header',
					'title' => __('Item', 'mfn-opts'),
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:min-height',
					'type' => 'text',
					'default_unit' => 'px',
					'title' => __('Min height', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:min-height_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'title' => __('Min height', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:min-height_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'title' => __('Min height', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:min-height_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'title' => __('Min height', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:align-content',
					'type' => 'switch',
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'title' => __('Vertical align', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'flex-start' => __('Top', 'mfn-opts'),
						'center' => __('Center', 'mfn-opts'),
						'flex-end' => __('Bottom', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:align-content_laptop',
					'type' => 'switch',
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'title' => __('Vertical align', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'flex-start' => __('Top', 'mfn-opts'),
						'center' => __('Center', 'mfn-opts'),
						'flex-end' => __('Bottom', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:align-content_tablet',
					'type' => 'switch',
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'title' => __('Vertical align', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'flex-start' => __('Top', 'mfn-opts'),
						'center' => __('Center', 'mfn-opts'),
						'flex-end' => __('Bottom', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:align-content_mobile',
					'type' => 'switch',
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'title' => __('Vertical align', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'flex-start' => __('Top', 'mfn-opts'),
						'center' => __('Center', 'mfn-opts'),
						'flex-end' => __('Bottom', 'mfn-opts'),
					],
				),






				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:justify-content',
					'type' => 'select',
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'title' => __('Horizontal align', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'flex-start' => __('Left', 'mfn-opts'),
						'center' => __('Center', 'mfn-opts'),
						'flex-end' => __('Right', 'mfn-opts'),
						'space-around' => __('Space around', 'mfn-opts'),
						'space-between' => __('Space between', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:justify-content_laptop',
					'type' => 'select',
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'title' => __('Horizontal align', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'flex-start' => __('Left', 'mfn-opts'),
						'center' => __('Center', 'mfn-opts'),
						'flex-end' => __('Right', 'mfn-opts'),
						'space-around' => __('Space around', 'mfn-opts'),
						'space-between' => __('Space between', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:justify-content_tablet',
					'type' => 'select',
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'title' => __('Horizontal align', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'flex-start' => __('Left', 'mfn-opts'),
						'center' => __('Center', 'mfn-opts'),
						'flex-end' => __('Right', 'mfn-opts'),
						'space-around' => __('Space around', 'mfn-opts'),
						'space-between' => __('Space between', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:justify-content_mobile',
					'type' => 'select',
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'title' => __('Horizontal align', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'flex-start' => __('Left', 'mfn-opts'),
						'center' => __('Center', 'mfn-opts'),
						'flex-end' => __('Right', 'mfn-opts'),
						'space-around' => __('Space around', 'mfn-opts'),
						'space-between' => __('Space between', 'mfn-opts'),
					],
				),






				array(
					'type' => 'subheader',
					'class' => 'mfn-builder-subheader row-header mfn-toggled',
					'title' => __('Spacing', 'mfn-opts'),
				),

				array(
  				'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:padding',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Padding', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'padding',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:padding_laptop',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Padding', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'css_attr' => 'padding',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:padding_tablet',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Padding', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'padding',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:padding_mobile',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Padding', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'padding',
  			),

				array(
  				'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:margin',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Margin', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'margin',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:margin_laptop',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Margin', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'css_attr' => 'margin',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:margin_tablet',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Margin', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'margin',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:margin_mobile',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Margin', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'margin',
  			),

  			array(
  				'type' => 'subheader',
					'class' => 'mfn-builder-subheader row-header mfn-toggled',
					'title' => __('Border', 'mfn-opts'),
				),

  			array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:border-style',
					'attr_id' => 'border_style_ql',
					'type' => 'select',
					'class' => 'mfn-toggled',
					'title' => __('Border style', 'mfn-opts'),
					'options' => [
						'none' => __('None', 'mfn-opts'),
						'solid' => __('Solid', 'mfn-opts'),
						'dashed' => __('Dashed', 'mfn-opts'),
						'dotted' => __('Dotted', 'mfn-opts'),
						'double' => __('Double', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:border-color',
					'condition' => array( 'id' => 'border_style_ql', 'opt' => 'isnt', 'val' => 'none' ),
					'type' => 'color',
					'class' => 'mfn-toggled',
					'title' => __('Border color', 'mfn-opts'),
				),

				array(
  				'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:border-width',
  				'condition' => array( 'id' => 'border_style_ql', 'opt' => 'isnt', 'val' => 'none' ),
  				'type' => 'dimensions',
  				'title' => __('Border width', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-width',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:border-width_laptop',
  				'condition' => array( 'id' => 'border_style_ql', 'opt' => 'isnt', 'val' => 'none' ),
  				'type' => 'dimensions',
  				'title' => __('Border width', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-width',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:border-width_tablet',
  				'condition' => array( 'id' => 'border_style_ql', 'opt' => 'isnt', 'val' => 'none' ),
  				'type' => 'dimensions',
  				'title' => __('Border width', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-width',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:border-width_mobile',
  				'condition' => array( 'id' => 'border_style_ql', 'opt' => 'isnt', 'val' => 'none' ),
  				'type' => 'dimensions',
  				'title' => __('Border width', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-width',
  			),

				array(
  				'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:border-radius',
  				'type' => 'dimensions',
  				'title' => __('Border radius', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-radius',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:border-radius_laptop',
  				'type' => 'dimensions',
  				'title' => __('Border radius', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-radius',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:border-radius_tablet',
  				'type' => 'dimensions',
  				'title' => __('Border radius', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-radius',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:border-radius_mobile',
  				'type' => 'dimensions',
  				'title' => __('Border radius', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-radius',
  			),

  			array(
		  				'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:box-shadow',
  				'type' => 'box_shadow',
  				'title' => __('Box shadow', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'box-shadow'
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:box-shadow_tablet',
  				'type' => 'box_shadow',
  				'title' => __('Box shadow', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'box-shadow',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:box-shadow_mobile',
  				'type' => 'box_shadow',
  				'title' => __('Box shadow', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'box-shadow',
  			),

  			array(
  				'type' => 'subheader',
					'class' => 'mfn-builder-subheader row-header mfn-toggled',
					'title' => __('Background', 'mfn-opts'),
				),

  			array(
					'id' => 'background_switcher_ql',
					'attr_id' => 'background_switcher_ql',
					'type' => 'switch',
					'class' => 'mfn-fields-switcher mfn-toggled disable-history',
					'title' => __('Background type', 'mfn-opts'),
					'options' => array(
  					'default' => __('Default', 'mfn-opts'),
  					'gradient' 	=> __('Gradient', 'mfn-opts'),
  				),
					'visual_options' => array(
  					'default' => '',
  					'gradient' 	=> '',
  				),
  				'std' => 'default',
				),

				array(
  				'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:gradient',
  				'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'gradient' ),
  				'type' => 'gradient',
  				'title' => __('Gradient', 'mfn-opts'),
  				'class' => 'mfn-toggled',
					'css_attr' => 'gradient',
  			),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:background-color',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'color',
					'class' => 'mfn-toggled',
					'title' => __('Background color', 'mfn-opts'),
				),

				array(
  				'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:background-image',
  				'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
  				'oldid' => 'bg_color',
  				'dynamic_data' => 'featured_image',
  				'type' => 'upload',
  				'responsive' => 'desktop',
  				'class' => 'mfn-toggled',
  				'title' => __('Image', 'mfn-opts'),
					'css_attr' => 'background-image',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:background-image_laptop',
  				'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
  				'oldid' => 'bg_color',
  				'dynamic_data' => 'featured_image',
  				'type' => 'upload',
  				'responsive' => 'laptop',
  				'class' => 'mfn-toggled',
  				'title' => __('Image', 'mfn-opts'),
					'css_attr' => 'background-image',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:background-image_tablet',
  				'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
  				'oldid' => 'bg_color',
  				'dynamic_data' => 'featured_image',
  				'type' => 'upload',
  				'responsive' => 'tablet',
  				'class' => 'mfn-toggled',
  				'title' => __('Image', 'mfn-opts'),
					'css_attr' => 'background-image',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:background-image_mobile',
  				'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
  				'oldid' => 'bg_color',
  				'dynamic_data' => 'featured_image',
  				'type' => 'upload',
  				'responsive' => 'mobile',
  				'class' => 'mfn-toggled',
  				'title' => __('Image', 'mfn-opts'),
					'css_attr' => 'background-image',
  			),

  			array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:background-repeat',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'class' => 'mfn-toggled',
					'css_attr' => 'background-repeat',
					'responsive' => 'desktop',
					'title' => __('Repeat', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'repeat' => __('Repeat', 'mfn-opts'),
						'no-repeat' => __('No repeat', 'mfn-opts'),
						'repeat-x' => __('Repeat X', 'mfn-opts'),
						'repeat-y' => __('Repeat Y', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:background-repeat_laptop',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'class' => 'mfn-toggled',
					'css_attr' => 'background-repeat',
					'responsive' => 'laptop',
					'title' => __('Repeat', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'repeat' => __('Repeat', 'mfn-opts'),
						'no-repeat' => __('No repeat', 'mfn-opts'),
						'repeat-x' => __('Repeat X', 'mfn-opts'),
						'repeat-y' => __('Repeat Y', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:background-repeat_tablet',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'class' => 'mfn-toggled',
					'css_attr' => 'background-repeat',
					'responsive' => 'tablet',
					'title' => __('Repeat', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'repeat' => __('Repeat', 'mfn-opts'),
						'no-repeat' => __('No repeat', 'mfn-opts'),
						'repeat-x' => __('Repeat X', 'mfn-opts'),
						'repeat-y' => __('Repeat Y', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:background-repeat_mobile',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'class' => 'mfn-toggled',
					'css_attr' => 'background-repeat',
					'responsive' => 'mobile',
					'title' => __('Repeat', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'repeat' => __('Repeat', 'mfn-opts'),
						'no-repeat' => __('No repeat', 'mfn-opts'),
						'repeat-x' => __('Repeat X', 'mfn-opts'),
						'repeat-y' => __('Repeat Y', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:background-position',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_position',
					'type' => 'select',
					'class' => 'mfn-toggled',
					'responsive' => 'desktop',
					'css_attr' => 'background-position',
					'title' => __('Position', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'center' => __('Center Center', 'mfn-opts'),
						'center left' => __('Center Left', 'mfn-opts'),
						'center right' => __('Center Right', 'mfn-opts'),
						'top left' => __('Top Left', 'mfn-opts'),
						'top right' => __('Top Right', 'mfn-opts'),
						'top center' => __('Top Center', 'mfn-opts'),
						'bottom left' => __('Bottom Left', 'mfn-opts'),
						'bottom right' => __('Bottom Right', 'mfn-opts'),
						'bottom center' => __('Bottom Center', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:background-position_laptop',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_position',
					'type' => 'select',
					'class' => 'mfn-toggled',
					'responsive' => 'laptop',
					'css_attr' => 'background-position',
					'title' => __('Position', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'center' => __('Center Center', 'mfn-opts'),
						'center left' => __('Center Left', 'mfn-opts'),
						'center right' => __('Center Right', 'mfn-opts'),
						'top left' => __('Top Left', 'mfn-opts'),
						'top right' => __('Top Right', 'mfn-opts'),
						'top center' => __('Top Center', 'mfn-opts'),
						'bottom left' => __('Bottom Left', 'mfn-opts'),
						'bottom right' => __('Bottom Right', 'mfn-opts'),
						'bottom center' => __('Bottom Center', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:background-position_tablet',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_position',
					'type' => 'select',
					'class' => 'mfn-toggled',
					'responsive' => 'tablet',
					'css_attr' => 'background-position',
					'title' => __('Position', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'center' => __('Center Center', 'mfn-opts'),
						'center left' => __('Center Left', 'mfn-opts'),
						'center right' => __('Center Right', 'mfn-opts'),
						'top left' => __('Top Left', 'mfn-opts'),
						'top right' => __('Top Right', 'mfn-opts'),
						'top center' => __('Top Center', 'mfn-opts'),
						'bottom left' => __('Bottom Left', 'mfn-opts'),
						'bottom right' => __('Bottom Right', 'mfn-opts'),
						'bottom center' => __('Bottom Center', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:background-position_mobile',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_position',
					'type' => 'select',
					'class' => 'mfn-toggled',
					'responsive' => 'mobile',
					'css_attr' => 'background-position',
					'title' => __('Position', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'center' => __('Center Center', 'mfn-opts'),
						'center left' => __('Center Left', 'mfn-opts'),
						'center right' => __('Center Right', 'mfn-opts'),
						'top left' => __('Top Left', 'mfn-opts'),
						'top right' => __('Top Right', 'mfn-opts'),
						'top center' => __('Top Center', 'mfn-opts'),
						'bottom left' => __('Bottom Left', 'mfn-opts'),
						'bottom right' => __('Bottom Right', 'mfn-opts'),
						'bottom center' => __('Bottom Center', 'mfn-opts'),
					],
				),


				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:background-attachment',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'class' => 'mfn-toggled',
					'responsive' => 'desktop',
					'css_attr' => 'background-attachment',
					'title' => __('Attachment', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'scroll' => __('Scroll', 'mfn-opts'),
						'fixed' => __('Fixed', 'mfn-opts'),
						'parallax' => __('Parallax', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:background-attachment_laptop',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'class' => 'mfn-toggled',
					'responsive' => 'laptop',
					'css_attr' => 'background-attachment',
					'title' => __('Attachment', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'scroll' => __('Scroll', 'mfn-opts'),
						'fixed' => __('Fixed', 'mfn-opts'),
						'parallax' => __('Parallax', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:background-attachment_tablet',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'class' => 'mfn-toggled',
					'responsive' => 'tablet',
					'css_attr' => 'background-attachment',
					'title' => __('Attachment', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'scroll' => __('Scroll', 'mfn-opts'),
						'fixed' => __('Fixed', 'mfn-opts'),
						'parallax' => __('Parallax', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:background-attachment_mobile',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'class' => 'mfn-toggled',
					'responsive' => 'mobile',
					'css_attr' => 'background-attachment',
					'title' => __('Attachment', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'scroll' => __('Scroll', 'mfn-opts'),
						'fixed' => __('Fixed', 'mfn-opts'),
						'parallax' => __('Parallax', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:background-size',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_size',
					'class' => 'mfn-toggled',
					'responsive' => 'desktop',
					'type' => 'select',
					'css_attr' => 'background-size',
					'title' => __('Size', 'mfn-opts'),
					'options' => mfna_bg_size()
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:background-size_laptop',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_size',
					'class' => 'mfn-toggled',
					'responsive' => 'laptop',
					'type' => 'select',
					'css_attr' => 'background-size',
					'title' => __('Size', 'mfn-opts'),
					'options' => mfna_bg_size()
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:background-size_tablet',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_size',
					'class' => 'mfn-toggled',
					'responsive' => 'tablet',
					'type' => 'select',
					'css_attr' => 'background-size',
					'title' => __('Size', 'mfn-opts'),
					'options' => mfna_bg_size()
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-section-inner .mfn-queryloop-item-wrapper:background-size_mobile',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_size',
					'class' => 'mfn-toggled',
					'responsive' => 'mobile',
					'type' => 'select',
					'css_attr' => 'background-size',
					'title' => __('Size', 'mfn-opts'),
					'options' => mfna_bg_size()
				),

				array(
					'type' => 'header',
					'class' => 'toggled_header row-header',
					'title' => __('Slider dots', 'mfn-opts'),
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .mcb-section-inner .swiper-pagination:top',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Top', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled dots-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .mcb-section-inner .swiper-pagination:bottom',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Bottom', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled dots-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .mcb-section-inner .swiper-pagination:left',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Left', 'mfn-opts'),
						'responsive' => 'desktop',
					'class' => 'mfn-toggled dots-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .mcb-section-inner .swiper-pagination:right',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Right', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled dots-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .mcb-section-inner .swiper-pagination:top_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Top', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled dots-custom-style',
				),


				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .mcb-section-inner .swiper-pagination:top_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Top', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled dots-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .mcb-section-inner .swiper-pagination:bottom_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Bottom', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled dots-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .mcb-section-inner .swiper-pagination:bottom_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Bottom', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled dots-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .mcb-section-inner .swiper-pagination:left_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Left', 'mfn-opts'),
						'responsive' => 'laptop',
					'class' => 'mfn-toggled dots-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .mcb-section-inner .swiper-pagination:left_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Left', 'mfn-opts'),
						'responsive' => 'tablet',
					'class' => 'mfn-toggled dots-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .mcb-section-inner .swiper-pagination:right_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Right', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled dots-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .mcb-section-inner .swiper-pagination:right_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Right', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled dots-custom-style',
				),



				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .mcb-section-inner .swiper-pagination:top_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Top', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled dots-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .mcb-section-inner .swiper-pagination:bottom_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Bottom', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled dots-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .mcb-section-inner .swiper-pagination:left_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Left', 'mfn-opts'),
						'responsive' => 'mobile',
					'class' => 'mfn-toggled dots-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .mcb-section-inner .swiper-pagination:right_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Right', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled dots-custom-style',
				),



				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .swiper-pagination:--mfn-swiper-pagination-spacing',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'title' => __('Offset', 'mfn-opts'),
					'desc' => __('Works with Standard and Overlay pagination mode', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .swiper-pagination:--mfn-swiper-pagination-spacing_laptop',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'title' => __('Offset', 'mfn-opts'),
					'desc' => __('Works with Standard and Overlay pagination mode', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .swiper-pagination:--mfn-swiper-pagination-spacing_tablet',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'title' => __('Offset', 'mfn-opts'),
					'desc' => __('Works with Standard and Overlay pagination mode', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .swiper-pagination:--mfn-swiper-pagination-spacing_mobile',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'title' => __('Offset', 'mfn-opts'),
					'desc' => __('Works with Standard and Overlay pagination mode', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .swiper-pagination-bullet:--mfn-swiper-pagination-bullet-width',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'title' => __('Width', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '100',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .swiper-pagination-bullet:--mfn-swiper-pagination-bullet-width_laptop',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'title' => __('Width', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '100',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .swiper-pagination-bullet:--mfn-swiper-pagination-bullet-width_tablet',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'title' => __('Width', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '100',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .swiper-pagination-bullet:--mfn-swiper-pagination-bullet-width_mobile',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'title' => __('Width', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '100',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .swiper-pagination-bullet:--mfn-swiper-pagination-bullet-height',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'title' => __('Height', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '100',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .swiper-pagination-bullet:--mfn-swiper-pagination-bullet-height_laptop',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'title' => __('Height', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '100',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .swiper-pagination-bullet:--mfn-swiper-pagination-bullet-height_tablet',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'title' => __('Height', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '100',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .swiper-pagination-bullet:--mfn-swiper-pagination-bullet-height_mobile',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'title' => __('Height', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '100',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .swiper-pagination-bullet:border-radius',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'class' => 'mfn-toggled',
					'title' => __('Border radius', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '100',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .swiper-pagination-bullet:opacity',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'class' => 'mfn-toggled',
					'title' => __('Opacity', 'mfn-opts'),
					'param' => array(
						'min' => '0',
						'max' => '1',
						'step' => '0.01',
					),
					'std' => '0',
				),

				array(
					'type' => 'html',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .swiper-pagination .swiper-pagination-bullet:--swiper-pagination-bullet-inactive-color',
					'type' => 'color',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'class' => 'mfn-toggled',
					'title' => __('Color', 'mfn-opts'),
				),

				array(
					'type' => 'html',
					'html' => '</div><div class="mfn-sft mfn-sft-hover">',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .swiper-pagination .swiper-pagination-bullet-active:--mfn-swiper-pagination-bullet-active-color',
					'type' => 'color',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'class' => 'mfn-toggled',
					'title' => __('Active color', 'mfn-opts'),
				),

				array(
					'type' => 'html',
					'html' => '</div></div>',
				),

				array(
					'type' => 'header',
					'class' => 'toggled_header row-header',
					'title' => __('Arrows', 'mfn-opts'),
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
				),

				array(
					'type' => 'subheader',
  				'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'class' => 'mfn-builder-subheader row-header mfn-toggled',
					'title' => __('Prev arrow custom style', 'mfn-opts'),
				),

  			array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-prev.mfn-swiper-arrow:top',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Top', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-prev.mfn-swiper-arrow:bottom',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Bottom', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-prev.mfn-swiper-arrow:left',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Left', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-prev.mfn-swiper-arrow:right',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Right', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-prev.mfn-swiper-arrow:top_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Top', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-prev.mfn-swiper-arrow:top_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Top', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-prev.mfn-swiper-arrow:bottom_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Bottom', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-prev.mfn-swiper-arrow:bottom_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Bottom', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-prev.mfn-swiper-arrow:left_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Left', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-prev.mfn-swiper-arrow:left_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Left', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-prev.mfn-swiper-arrow:right_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Right', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-prev.mfn-swiper-arrow:right_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Right', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-prev.mfn-swiper-arrow:top_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Top', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-prev.mfn-swiper-arrow:bottom_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Bottom', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-prev.mfn-swiper-arrow:left_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Left', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-prev.mfn-swiper-arrow:right_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Right', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'type' => 'subheader',
  				'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'class' => 'mfn-builder-subheader row-header mfn-toggled',
					'title' => __('Next arrow custom style', 'mfn-opts'),
				),

  			array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-next.mfn-swiper-arrow:top',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Top', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-next.mfn-swiper-arrow:bottom',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Bottom', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-next.mfn-swiper-arrow:left',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Left', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-next.mfn-swiper-arrow:right',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Right', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-next.mfn-swiper-arrow:top_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Top', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-next.mfn-swiper-arrow:bottom_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Bottom', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-next.mfn-swiper-arrow:left_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Left', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-next.mfn-swiper-arrow:right_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Right', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled arrows-custom-style',
				),


				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-next.mfn-swiper-arrow:top_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Top', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-next.mfn-swiper-arrow:bottom_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Bottom', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-next.mfn-swiper-arrow:left_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Left', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-next.mfn-swiper-arrow:right_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Right', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-next.mfn-swiper-arrow:top_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Top', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-next.mfn-swiper-arrow:bottom_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Bottom', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-next.mfn-swiper-arrow:left_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Left', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .swiper-button-next.mfn-swiper-arrow:right_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Right', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mcb-section-inner .mfn-swiper-arrow:transformtranslatey',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'type' => 'sliderbar',
					'class' => 'mfn-toggled',
					'title' => __('Translate(Y)', 'mfn-opts'),
					'desc' => __('Moves navigation arrows along the Y-axis by the percent height', 'mfn-opts'),
					'default_unit' => '%',
					'after' => '%',
					'param' => array(
						'min' => '-100',
						'max' => '100',
						'step' => '1',
						'unit' => '%',
					)
				),

				array(
					'type' => 'subheader',
					'class' => 'mfn-builder-subheader row-header mfn-toggled',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'title' => __('Custom arrows', 'mfn-opts'),
				),

				array(
					'id' => 'query_display_slider_arrow_prev',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'icon',
					'class' => 'mfn-toggled',
					'title' => __('Prev arrow', 'mfn-opts'),
					'preview' => 'icon',
					'std' => 'icon-left-open-big'
				),

				array(
					'id' => 'query_display_slider_arrow_next',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'icon',
					'class' => 'mfn-toggled',
					'title' => __('Next arrow', 'mfn-opts'),
					'preview' => 'icon',
					'std' => 'icon-right-open-big'
				),

				array(
					'type' => 'subheader',
					'class' => 'mfn-builder-subheader row-header mfn-toggled',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'title' => __('Size', 'mfn-opts'),
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow:--mfn-swiper-arrow-width',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'title' => __('Width', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow:--mfn-swiper-arrow-height',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'title' => __('Height', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow i:--mfn-swiper-arrow-size',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'title' => __('Size', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow:--mfn-swiper-arrow-spacing',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'title' => __('Offset', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '-300',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow:--mfn-swiper-arrow-width_tablet',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'title' => __('Width', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow:--mfn-swiper-arrow-height_tablet',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'title' => __('Height', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow i:--mfn-swiper-arrow-size_tablet',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'title' => __('Size', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow:--mfn-swiper-arrow-spacing_tablet',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'title' => __('Offset', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '-300',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow:--mfn-swiper-arrow-width_laptop',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'title' => __('Width', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow:--mfn-swiper-arrow-height_laptop',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'title' => __('Height', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow i:--mfn-swiper-arrow-size_laptop',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'title' => __('Size', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow:--mfn-swiper-arrow-spacing_laptop',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'title' => __('Offset', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '-300',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),



				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow:--mfn-swiper-arrow-width_mobile',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'title' => __('Width', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow:--mfn-swiper-arrow-height_mobile',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'title' => __('Height', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow i:--mfn-swiper-arrow-size_mobile',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'title' => __('Size', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow:--mfn-swiper-arrow-spacing_mobile',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'title' => __('Offset', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '-300',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'type' => 'header',
					'class' => 'mfn-builder-subheader row-header mfn-toggled',
					'title' => __('Border', 'mfn-opts'),
				),

  			array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow:border-style',
					'attr_id' => 'border_style_qlarr',
					'type' => 'select',
					'class' => 'mfn-toggled',
					'title' => __('Border style', 'mfn-opts'),
					'options' => [
						'none' => __('None', 'mfn-opts'),
						'solid' => __('Solid', 'mfn-opts'),
						'dashed' => __('Dashed', 'mfn-opts'),
						'dotted' => __('Dotted', 'mfn-opts'),
						'double' => __('Double', 'mfn-opts'),
					],
				),

				array(
  				'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow:border-width',
  				'condition' => array( 'id' => 'border_style_qlarr', 'opt' => 'isnt', 'val' => 'none' ),
  				'type' => 'dimensions',
  				'title' => __('Border width', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-width',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow:border-width_laptop',
  				'condition' => array( 'id' => 'border_style_qlarr', 'opt' => 'isnt', 'val' => 'none' ),
  				'type' => 'dimensions',
  				'title' => __('Border width', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-width',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow:border-width_tablet',
  				'condition' => array( 'id' => 'border_style_qlarr', 'opt' => 'isnt', 'val' => 'none' ),
  				'type' => 'dimensions',
  				'title' => __('Border width', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-width',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow:border-width_mobile',
  				'condition' => array( 'id' => 'border_style_qlarr', 'opt' => 'isnt', 'val' => 'none' ),
  				'type' => 'dimensions',
  				'title' => __('Border width', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-width',
  			),

				array(
  				'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow:border-radius',
  				'type' => 'dimensions',
  				'title' => __('Border radius', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-radius',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow:border-radius_laptop',
  				'type' => 'dimensions',
  				'title' => __('Border radius', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-radius',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow:border-radius_tablet',
  				'type' => 'dimensions',
  				'title' => __('Border radius', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-radius',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow:border-radius_mobile',
  				'type' => 'dimensions',
  				'title' => __('Border radius', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-radius',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow:box-shadow',
  				'type' => 'box_shadow',
  				'title' => __('Box shadow', 'mfn-opts'),
					'class' => 'mfn-toggled',
					'css_attr' => 'box-shadow',
  			),

				array(
					'type' => 'subheader',
					'class' => 'mfn-builder-subheader row-header mfn-toggled',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'title' => __('Color', 'mfn-opts'),
				),


				array(
					'type' => 'html',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow:--swiper-navigation-color',
					'type' => 'color',
					'class' => 'mfn-toggled',
					'title' => __('Color', 'mfn-opts'),
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow:background-color',
					'type' => 'color',
					'class' => 'mfn-toggled',
					'title' => __('Background', 'mfn-opts'),
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow:border-color',
					'condition' => array( 'id' => 'border_style_qlarr', 'opt' => 'isnt', 'val' => 'none' ),
					'type' => 'color',
					'class' => 'mfn-toggled',
					'title' => __('Border color', 'mfn-opts'),
				),

				array(
					'type' => 'html',
					'html' => '</div><div class="mfn-sft mfn-sft-hover">',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow|hover:--swiper-navigation-color',
					'type' => 'color',
					'class' => 'mfn-toggled',
					'title' => __('Color', 'mfn-opts'),
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow|hover:background-color',
					'type' => 'color',
					'class' => 'mfn-toggled',
					'title' => __('Background', 'mfn-opts'),
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.mfn-looped-items-slider-wrapper .mcb-section-inner .mfn-swiper-arrow|hover:border-color',
					'condition' => array( 'id' => 'border_style_qlarr', 'opt' => 'isnt', 'val' => 'none' ),
					'type' => 'color',
					'class' => 'mfn-toggled',
					'title' => __('Border color', 'mfn-opts'),
				),

				array(
					'type' => 'html',
					'html' => '</div></div>',
				),














				array(
  				'type' => 'header',
  				'condition' => array( 'id' => 'query_post_pagination', 'opt' => 'isnt', 'val' => '' ),
  				'class' => 'toggled_header row-header',
					'title' => __('Pagination', 'mfn-opts'),
				),

				array(
					'type' => 'subheader',
					'class' => 'mfn-builder-subheader row-header mfn-toggled',
					'title' => __('Container', 'mfn-opts'),
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination:background-color',
					'type' => 'color',
					'class' => 'mfn-toggled',
					'title' => __('Background', 'mfn-opts'),
				),

				array(
  				'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination:padding',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Padding', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'padding',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination:padding_laptop',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Padding', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'css_attr' => 'padding',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination:padding_tablet',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Padding', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'padding',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination:padding_mobile',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Padding', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'padding',
  			),

				array(
  				'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination:margin',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Margin', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'margin',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination:margin_laptop',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Margin', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'css_attr' => 'margin',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination:margin_tablet',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Margin', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'margin',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination:margin_mobile',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Margin', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'margin',
  			),

				array(
					'type' => 'subheader',
					'class' => 'mfn-builder-subheader row-header mfn-toggled',
					'title' => __('Items', 'mfn-opts'),
				),



				array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers:gap',
					'type' => 'sliderbar',
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'title' => __('Gap', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '50',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers:gap_laptop',
					'type' => 'sliderbar',
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'title' => __('Gap', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '50',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers:gap_tablet',
					'type' => 'sliderbar',
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'title' => __('Gap', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '50',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers:gap_mobile',
					'type' => 'sliderbar',
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'title' => __('Gap', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '50',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers li .page-numbers:min-width',
					'type' => 'sliderbar',
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'title' => __('Min width', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '200',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers li .page-numbers:min-width_laptop',
					'type' => 'sliderbar',
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'title' => __('Min width', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '200',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers li .page-numbers:min-width_tablet',
					'type' => 'sliderbar',
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'title' => __('Min width', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '200',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers li .page-numbers:min-width_mobile',
					'type' => 'sliderbar',
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'title' => __('Min width', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '200',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
  				'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers .page-numbers:padding',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Padding', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'padding',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers .page-numbers:padding_laptop',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Padding', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'css_attr' => 'padding',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers .page-numbers:padding_tablet',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Padding', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'padding',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers .page-numbers:padding_mobile',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Padding', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'padding',
  			),

  			array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers .page-numbers:border-style',
					'attr_id' => 'border_style_pag',
					'type' => 'select',
					'class' => 'mfn-toggled',
					'title' => __('Border style', 'mfn-opts'),
					'options' => [
						'none' => __('None', 'mfn-opts'),
						'solid' => __('Solid', 'mfn-opts'),
						'dashed' => __('Dashed', 'mfn-opts'),
						'dotted' => __('Dotted', 'mfn-opts'),
						'double' => __('Double', 'mfn-opts'),
					],
				),

				array(
  				'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers .page-numbers:border-width',
  				'condition' => array( 'id' => 'border_style_pag', 'opt' => 'isnt', 'val' => 'none' ),
  				'type' => 'dimensions',
  				'title' => __('Border width', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-width',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers .page-numbers:border-width_laptop',
  				'condition' => array( 'id' => 'border_style_pag', 'opt' => 'isnt', 'val' => 'none' ),
  				'type' => 'dimensions',
  				'title' => __('Border width', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-width',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers .page-numbers:border-width_tablet',
  				'condition' => array( 'id' => 'border_style_pag', 'opt' => 'isnt', 'val' => 'none' ),
  				'type' => 'dimensions',
  				'title' => __('Border width', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-width',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers .page-numbers:border-width_mobile',
  				'condition' => array( 'id' => 'border_style_pag', 'opt' => 'isnt', 'val' => 'none' ),
  				'type' => 'dimensions',
  				'title' => __('Border width', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-width',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers .page-numbers:border-radius',
  				'type' => 'dimensions',
  				'title' => __('Border radius', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-radius',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers .page-numbers:border-radius_laptop',
  				'type' => 'dimensions',
  				'title' => __('Border radius', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-radius',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers .page-numbers:border-radius_tablet',
  				'type' => 'dimensions',
  				'title' => __('Border radius', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-radius',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers .page-numbers:border-radius_mobile',
  				'type' => 'dimensions',
  				'title' => __('Border radius', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-radius',
  			),

				array(
					'type' => 'html',
					'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers li .page-numbers:--mfn-pagination-items-color',
					'type' => 'color',
					'class' => 'mfn-toggled',
					'title' => __('Color', 'mfn-opts'),
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers li .page-numbers:background-color',
					'type' => 'color',
					'class' => 'mfn-toggled',
					'title' => __('Background', 'mfn-opts'),
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers .page-numbers:border-color',
					'condition' => array( 'id' => 'border_style_pag', 'opt' => 'isnt', 'val' => 'none' ),
					'type' => 'color',
					'class' => 'mfn-toggled',
					'title' => __('Border color', 'mfn-opts'),
				),

				array(
					'type' => 'html',
					'html' => '</div><div class="mfn-sft mfn-sft-hover">',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers a.page-numbers|hover:--mfn-pagination-items-color',
					'type' => 'color',
					'class' => 'mfn-toggled',
					'title' => __('Color', 'mfn-opts'),
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers a.page-numbers|hover:background-color',
					'type' => 'color',
					'class' => 'mfn-toggled',
					'title' => __('Background', 'mfn-opts'),
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers .page-numbers|hover:border-color',
					'condition' => array( 'id' => 'border_style_pag', 'opt' => 'isnt', 'val' => 'none' ),
					'type' => 'color',
					'class' => 'mfn-toggled',
					'title' => __('Border color', 'mfn-opts'),
				),

				array(
					'type' => 'html',
					'html' => '</div><div class="mfn-sft mfn-sft-active">',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers li .current:--mfn-pagination-items-color',
					'type' => 'color',
					'class' => 'mfn-toggled',
					'title' => __('Color', 'mfn-opts'),
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers li .current:background-color',
					'type' => 'color',
					'class' => 'mfn-toggled',
					'title' => __('Background', 'mfn-opts'),
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mfn-query-pagination ul.page-numbers li .current:border-color',
					'condition' => array( 'id' => 'border_style_pag', 'opt' => 'isnt', 'val' => 'none' ),
					'type' => 'color',
					'class' => 'mfn-toggled',
					'title' => __('Border color', 'mfn-opts'),
				),

        array(
					'type' => 'html',
					'html' => '</div></div>',
				),

  			array(
					'type' => 'html',
					'html' => '</div>', // end: modalbox-card
				),

				array(
					'type' => 'html',
					'html' => '<div class="modalbox-card modalbox-card-advanced">',
				),

				array(
					'type' => 'header',
					'class' => 'toggled_header row-header',
					'title' => __('Dimensions', 'mfn-opts'),
				),

				array(
					'id' => 'width_switcher',
					'attr_id' => 'sect_width_switcher',
					'type' => 'select',
					'class' => 'mfn-toggled mfn-layout-modifier',
					'title' => __('Width', 'mfn-opts'),
					'options' => array(
  					'default' => __('Default', 'mfn-opts'),
  					'full' 		=> __('Full width', 'mfn-opts'),
  					'custom' 	=> __('Custom', 'mfn-opts'),
  				),
  				'std' => 'default',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement.custom-width .section_wrapper:max-width',
					'condition' => array( 'id' => 'sect_width_switcher', 'opt' => 'is', 'val' => 'custom' ),
					'type' => 'text',
					'default_unit' => 'px',
					'class' => 'mfn-toggled narrow mfn-slider-input mfn-layout-modifier',
					'title' => __('Custom width', 'mfn-opts')
				),

				array(
					'id' => 'height_switcher',
					'attr_id' => 'sect_height_switcher',
					'type' => 'select',
					'class' => 'mfn-toggled',
					'title' => __('Height', 'mfn-opts'),
					'options' => array(
  					'default' => __('Default', 'mfn-opts'),
  					'full-screen' => __('Full screen', 'mfn-opts'),
  					'custom' 	=> __('Custom', 'mfn-opts'),
  				),
  				'std' => 'default',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:height',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'sect_height_switcher', 'opt' => 'is', 'val' => 'custom' ),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled narrow mfn-slider-input',
					'title' => __('Custom height', 'mfn-opts')
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:height_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'sect_height_switcher', 'opt' => 'is', 'val' => 'custom' ),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled narrow mfn-slider-input',
					'title' => __('Custom height', 'mfn-opts')
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:height_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'sect_height_switcher', 'opt' => 'is', 'val' => 'custom' ),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled narrow mfn-slider-input',
					'title' => __('Custom height', 'mfn-opts')
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:height_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'sect_height_switcher', 'opt' => 'is', 'val' => 'custom' ),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled narrow mfn-slider-input',
					'title' => __('Custom height', 'mfn-opts')
				),

  			array(
  				'type' => 'header',
  				'class' => 'toggled_header row-header',
					'title' => __('Spacing', 'mfn-opts'),
				),

				array(
  				'id' => 'style:.mcb-section-mfnuidelement:padding',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Padding', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'padding',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement:padding_laptop',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Padding', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'css_attr' => 'padding',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement:padding_tablet',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Padding', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'padding',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement:padding_mobile',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Padding', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'padding',
  			),

				array(
  				'id' => 'style:.mcb-section-mfnuidelement:margin',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Margin', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'margin',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement:margin_laptop',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Margin', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'css_attr' => 'margin',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement:margin_tablet',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Margin', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'margin',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement:margin_mobile',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Margin', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'margin',
  			),

  			array(
  				'type' => 'header',
  				'class' => 'toggled_header hide_if_ql row-header',
					'title' => __('Positioning', 'mfn-opts'),
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .section_wrapper:align-content',
					'type' => 'radio_img',
					'alias' => 'section-flex-align-content',
					'responsive' => 'desktop',
					'class' => 'form-content-full-width small mfn-toggled',
					'title' => __('Section content position', 'mfn-opts'),
					'options' => array(
						'' => 'Default',
						'flex-start' => 'Top',
						'flex-end' => 'Bottom',
						'center' => 'Center',
					),
					'std' => '',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .section_wrapper:align-content_laptop',
					'type' => 'radio_img',
					'alias' => 'section-flex-align-content',
					'responsive' => 'laptop',
					'class' => 'form-content-full-width small mfn-toggled',
					'title' => __('Section content position', 'mfn-opts'),
					'options' => array(
						'' => 'Default',
						'flex-start' => 'Top',
						'flex-end' => 'Bottom',
						'center' => 'Center',
					),
					'std' => '',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .section_wrapper:align-content_tablet',
					'type' => 'radio_img',
					'alias' => 'section-flex-align-content',
					'responsive' => 'tablet',
					'class' => 'form-content-full-width small mfn-toggled',
					'title' => __('Section content position', 'mfn-opts'),
					'options' => array(
						'' => 'Default',
						'flex-start' => 'Top',
						'flex-end' => 'Bottom',
						'center' => 'Center',
					),
					'std' => '',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .section_wrapper:align-content_mobile',
					'type' => 'radio_img',
					'alias' => 'section-flex-align-content',
					'responsive' => 'mobile',
					'class' => 'form-content-full-width small mfn-toggled',
					'title' => __('Section content position', 'mfn-opts'),
					'options' => array(
						'' => 'Default',
						'flex-start' => 'Top',
						'flex-end' => 'Bottom',
						'center' => 'Center',
					),
					'std' => '',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .section_wrapper:align-items',
					'type' => 'radio_img',
					'alias' => 'section-flex-align',
					'responsive' => 'desktop',
					'class' => 'form-content-full-width small mfn-toggled adv_align-items_section',
					'title' => __('Wraps vertical spacing', 'mfn-opts'),
					'options' => array(
						'flex-start' => 'Top',
						'flex-end' => 'Bottom',
						'center' => 'Center',
						'stretch' => 'Stretch',
					),
					'std' => '',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .section_wrapper:align-items_laptop',
					'type' => 'radio_img',
					'alias' => 'section-flex-align',
					'responsive' => 'laptop',
					'class' => 'form-content-full-width small mfn-toggled adv_align-items_section',
					'title' => __('Wraps vertical spacing', 'mfn-opts'),
					'options' => array(
						'flex-start' => 'Top',
						'flex-end' => 'Bottom',
						'center' => 'Center',
						'stretch' => 'Stretch',
					),
					'std' => '',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .section_wrapper:align-items_tablet',
					'type' => 'radio_img',
					'alias' => 'section-flex-align',
					'responsive' => 'tablet',
					'class' => 'form-content-full-width small mfn-toggled adv_align-items_section',
					'title' => __('Wraps vertical spacing', 'mfn-opts'),
					'options' => array(
						'flex-start' => 'Top',
						'flex-end' => 'Bottom',
						'center' => 'Center',
						'stretch' => 'Stretch',
					),
					'std' => '',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .section_wrapper:align-items_mobile',
					'type' => 'radio_img',
					'alias' => 'section-flex-align',
					'responsive' => 'mobile',
					'class' => 'form-content-full-width small mfn-toggled adv_align-items_section',
					'title' => __('Wraps vertical spacing', 'mfn-opts'),
					'options' => array(
						'flex-start' => 'Top',
						'flex-end' => 'Bottom',
						'center' => 'Center',
						'stretch' => 'Stretch',
					),
					'std' => '',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .section_wrapper:justify-content',
					'type' => 'radio_img',
					'alias' => 'section-flex-justify',
					'responsive' => 'desktop',
					'class' => 'form-content-full-width small mfn-toggled',
					'title' => __('Horizontal align', 'mfn-opts'),
					'options' => [
						'flex-start' => __('Left', 'mfn-opts'),
						'center' => __('Center', 'mfn-opts'),
						'flex-end' => __('Right', 'mfn-opts'),
						'space-between' => __('Space between', 'mfn-opts'),
						'space-around' => __('Space around', 'mfn-opts'),
					],
					'std' => '',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .section_wrapper:justify-content_laptop',
					'type' => 'radio_img',
					'alias' => 'section-flex-justify',
					'responsive' => 'laptop',
					'class' => 'form-content-full-width small mfn-toggled',
					'title' => __('Horizontal align', 'mfn-opts'),
					'options' => [
						'flex-start' => __('Left', 'mfn-opts'),
						'center' => __('Center', 'mfn-opts'),
						'flex-end' => __('Right', 'mfn-opts'),
						'space-between' => __('Space between', 'mfn-opts'),
						'space-around' => __('Space around', 'mfn-opts'),
					],
					'std' => '',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .section_wrapper:justify-content_tablet',
					'type' => 'radio_img',
					'alias' => 'section-flex-justify',
					'responsive' => 'tablet',
					'class' => 'form-content-full-width small mfn-toggled',
					'title' => __('Horizontal align', 'mfn-opts'),
					'options' => [
						'flex-start' => __('Left', 'mfn-opts'),
						'center' => __('Center', 'mfn-opts'),
						'flex-end' => __('Right', 'mfn-opts'),
						'space-between' => __('Space between', 'mfn-opts'),
						'space-around' => __('Space around', 'mfn-opts'),
					],
					'std' => '',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .section_wrapper:justify-content_mobile',
					'type' => 'radio_img',
					'alias' => 'section-flex-justify',
					'responsive' => 'mobile',
					'class' => 'form-content-full-width small mfn-toggled',
					'title' => __('Horizontal align', 'mfn-opts'),
					'options' => [
						'flex-start' => __('Left', 'mfn-opts'),
						'center' => __('Center', 'mfn-opts'),
						'flex-end' => __('Right', 'mfn-opts'),
						'space-between' => __('Space between', 'mfn-opts'),
						'space-around' => __('Space around', 'mfn-opts'),
					],
					'std' => '',
				),

				array(
					'type' => 'helper',
					'class' => 'mfn-toggled',
					'title' => __('Need help', 'mfn-opts'),
					'link' => 'https://support.muffingroup.com/documentation/bebuilder/positioning/#section',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:z-index',
					'type' => 'text',
					'title' => __('Z-index', 'mfn-opts'),
					'param' => 'number',
					'class' => 'narrow mfn-toggled',
					'preview' => 'number',
				),

  			array(
  				'type' => 'header',
  				'class' => 'toggled_header row-header',
					'title' => __('Background', 'mfn-opts'),
				),

				array(
					'type' => 'html',
					'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li class="visible-header-builder"><a href="#scroll" data-tab="scroll">Scroll</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
				),

				array(
					'id' => 'background_switcher',
					'attr_id' => 'background_switcher',
					'type' => 'switch',
					'class' => 'mfn-fields-switcher disable-history',
					'title' => __('Background type', 'mfn-opts'),
					'options' => array(
  					'default' => __('Default', 'mfn-opts'),
  					'video' => __('Video', 'mfn-opts'),
  					'gradient' 	=> __('Gradient', 'mfn-opts'),
  				),
					'visual_options' => array(
  					'default' => '',
  					'video' 	=> '',
  					'gradient' 	=> '',
  				),
  				'std' => 'default',
				),

				array(
  				'id' => 'style:.mcb-section-mfnuidelement:gradient',
  				'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'gradient' ),
  				'type' => 'gradient',
  				'title' => __('Gradient', 'mfn-opts'),
  				'class' => 'mfn-toggled',
					'css_attr' => 'gradient',
  			),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:background-color',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'color',
					'title' => __('Color', 'mfn-opts'),
				),

				array(
  				'id' => 'style:.mcb-section-mfnuidelement:background-image',
  				'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
  				'oldid' => 'bg_color',
  				'dynamic_data' => 'featured_image',
  				'type' => 'upload',
  				'title' => __('Image', 'mfn-opts'),
  				'responsive' => 'desktop',

					'css_attr' => 'background-image',
  			),

				array(
  				'id' => 'bg_video_mp4',
  				'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'video' ),
  				'class' => 'hidden-header-builder',
  				'type' => 'upload',
  				'title' => __('Video MP4', 'mfn-opts'),
  				'desc' => __('Image will be used as placeholder before video loads and on mobile devices', 'mfn-opts'),
  				'data' => 'video',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement:background-image_laptop',
  				'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
  				'oldid' => 'bg_image',
  				'dynamic_data' => 'featured_image',
  				'type' => 'upload',
  				'title' => __('Image', 'mfn-opts'),
  				'responsive' => 'laptop',

					'css_attr' => 'background-image',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement:background-image_tablet',
  				'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
  				'oldid' => 'bg_image',
  				'dynamic_data' => 'featured_image',
  				'type' => 'upload',
  				'title' => __('Image', 'mfn-opts'),
  				'responsive' => 'tablet',

					'css_attr' => 'background-image',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement:background-image_mobile',
  				'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
  				'type' => 'upload',
  				'dynamic_data' => 'featured_image',
  				'title' => __('Image', 'mfn-opts'),
  				'responsive' => 'mobile',

					'css_attr' => 'background-image',
  			),

  			array(
					'id' => 'style:.mcb-section-mfnuidelement:background-repeat',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'desktop',

					'css_attr' => 'background-repeat',
					'title' => __('Repeat', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'repeat' => __('Repeat', 'mfn-opts'),
						'no-repeat' => __('No repeat', 'mfn-opts'),
						'repeat-x' => __('Repeat X', 'mfn-opts'),
						'repeat-y' => __('Repeat Y', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:background-repeat_laptop',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'laptop',

					'css_attr' => 'background-repeat',
					'title' => __('Repeat', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'repeat' => __('Repeat', 'mfn-opts'),
						'no-repeat' => __('No repeat', 'mfn-opts'),
						'repeat-x' => __('Repeat X', 'mfn-opts'),
						'repeat-y' => __('Repeat Y', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:background-repeat_tablet',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'tablet',

					'css_attr' => 'background-repeat',
					'title' => __('Repeat', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'repeat' => __('Repeat', 'mfn-opts'),
						'no-repeat' => __('No repeat', 'mfn-opts'),
						'repeat-x' => __('Repeat X', 'mfn-opts'),
						'repeat-y' => __('Repeat Y', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:background-repeat_mobile',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'mobile',

					'css_attr' => 'background-repeat',
					'title' => __('Repeat', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'repeat' => __('Repeat', 'mfn-opts'),
						'no-repeat' => __('No repeat', 'mfn-opts'),
						'repeat-x' => __('Repeat X', 'mfn-opts'),
						'repeat-y' => __('Repeat Y', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:background-position',
					'attr_id' => 'bgposopt',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_position',
					'type' => 'select',
					'responsive' => 'desktop',
					'class' => 'bgposopt',
					'css_attr' => 'background-position',
					'title' => __('Position', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'center' => __('Center Center', 'mfn-opts'),
						'center left' => __('Center Left', 'mfn-opts'),
						'center right' => __('Center Right', 'mfn-opts'),
						'top left' => __('Top Left', 'mfn-opts'),
						'top right' => __('Top Right', 'mfn-opts'),
						'top center' => __('Top Center', 'mfn-opts'),
						'bottom left' => __('Bottom Left', 'mfn-opts'),
						'bottom right' => __('Bottom Right', 'mfn-opts'),
						'bottom center' => __('Bottom Center', 'mfn-opts'),
						'custom' => __('Custom', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:background-position_laptop',
					'attr_id' => 'bgposopt_laptop',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_position',
					'type' => 'select',
					'responsive' => 'laptop',
					'class' => 'bgposopt',
					'css_attr' => 'background-position',
					'title' => __('Position', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'center' => __('Center Center', 'mfn-opts'),
						'center left' => __('Center Left', 'mfn-opts'),
						'center right' => __('Center Right', 'mfn-opts'),
						'top left' => __('Top Left', 'mfn-opts'),
						'top right' => __('Top Right', 'mfn-opts'),
						'top center' => __('Top Center', 'mfn-opts'),
						'bottom left' => __('Bottom Left', 'mfn-opts'),
						'bottom right' => __('Bottom Right', 'mfn-opts'),
						'bottom center' => __('Bottom Center', 'mfn-opts'),
						'custom' => __('Custom', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:background-position_tablet',
					'attr_id' => 'bgposopt_tablet',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_position',
					'type' => 'select',
					'responsive' => 'tablet',
					'class' => 'bgposopt',
					'css_attr' => 'background-position',
					'title' => __('Position', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'center' => __('Center Center', 'mfn-opts'),
						'center left' => __('Center Left', 'mfn-opts'),
						'center right' => __('Center Right', 'mfn-opts'),
						'top left' => __('Top Left', 'mfn-opts'),
						'top right' => __('Top Right', 'mfn-opts'),
						'top center' => __('Top Center', 'mfn-opts'),
						'bottom left' => __('Bottom Left', 'mfn-opts'),
						'bottom right' => __('Bottom Right', 'mfn-opts'),
						'bottom center' => __('Bottom Center', 'mfn-opts'),
						'custom' => __('Custom', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:background-position_mobile',
					'attr_id' => 'bgposopt_mobile',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_position',
					'type' => 'select',
					'responsive' => 'mobile',
					'class' => 'bgposopt',
					'css_attr' => 'background-position',
					'title' => __('Position', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'center' => __('Center Center', 'mfn-opts'),
						'center left' => __('Center Left', 'mfn-opts'),
						'center right' => __('Center Right', 'mfn-opts'),
						'top left' => __('Top Left', 'mfn-opts'),
						'top right' => __('Top Right', 'mfn-opts'),
						'top center' => __('Top Center', 'mfn-opts'),
						'bottom left' => __('Bottom Left', 'mfn-opts'),
						'bottom right' => __('Bottom Right', 'mfn-opts'),
						'bottom center' => __('Bottom Center', 'mfn-opts'),
						'custom' => __('Custom', 'mfn-opts'),
					],
				),











				array(
					'id' => 'style:.mcb-section-mfnuidelement:background-position_v2',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'bgposopt', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Custom background position', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled mfn-custom-bg-pos',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:background-position_laptop_v2',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'bgposopt_laptop', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Custom background position', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled mfn-custom-bg-pos',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:background-position_tablet_v2',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'bgposopt_tablet', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Custom background position', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled mfn-custom-bg-pos',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:background-position_mobile_v2',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'bgposopt_mobile', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Custom background position', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled mfn-custom-bg-pos',
				),






























				array(
					'id' => 'style:.mcb-section-mfnuidelement:background-attachment',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'desktop',

					'css_attr' => 'background-attachment',
					'title' => __('Attachment', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'scroll' => __('Scroll', 'mfn-opts'),
						'fixed' => __('Fixed', 'mfn-opts'),
						'parallax' => __('Parallax', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:background-attachment_laptop',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'laptop',

					'css_attr' => 'background-attachment',
					'title' => __('Attachment', 'mfn-opts'),
					'desc' => __('iOS does <b>not</b> support background-attachment: fixed', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'scroll' => __('Scroll', 'mfn-opts'),
						'fixed' => __('Fixed', 'mfn-opts'),
						'parallax' => __('Parallax', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:background-attachment_tablet',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'tablet',

					'css_attr' => 'background-attachment',
					'title' => __('Attachment', 'mfn-opts'),
					'desc' => __('iOS does <b>not</b> support background-attachment: fixed', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'scroll' => __('Scroll', 'mfn-opts'),
						'fixed' => __('Fixed', 'mfn-opts'),
						'parallax' => __('Parallax', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:background-attachment_mobile',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'mobile',

					'css_attr' => 'background-attachment',
					'title' => __('Attachment', 'mfn-opts'),
					'desc' => __('iOS does <b>not</b> support background-attachment: fixed', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'scroll' => __('Scroll', 'mfn-opts'),
						'fixed' => __('Fixed', 'mfn-opts'),
						'parallax' => __('Parallax', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:background-size',
					'attr_id' => 'custombgsize',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_size',
					'type' => 'select',
					'responsive' => 'desktop',
					'class' => 'custombgsize',
					'css_attr' => 'background-size',
					'title' => __('Size', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'auto' => __('Auto', 'mfn-opts'),
						'contain' => __('Contain', 'mfn-opts'),
						'cover' => __('Cover', 'mfn-opts'),
						'cover-ultrawide' => __('Cover, on ultrawide screens only > 1920px', 'mfn-opts'),
						'custom' => __('Custom', 'mfn-opts'),
					],
					//'options' => mfna_bg_size()
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:background-size_laptop',
					'attr_id' => 'custombgsize_laptop',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_size',
					'type' => 'select',
					'responsive' => 'laptop',
					'class' => 'custombgsize',
					'css_attr' => 'background-size',
					'title' => __('Size', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'auto' => __('Auto', 'mfn-opts'),
						'contain' => __('Contain', 'mfn-opts'),
						'cover' => __('Cover', 'mfn-opts'),
						'cover-ultrawide' => __('Cover, on ultrawide screens only > 1920px', 'mfn-opts'),
						'custom' => __('Custom', 'mfn-opts'),
					],
					//'options' => mfna_bg_size()
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:background-size_tablet',
					'attr_id' => 'custombgsize_tablet',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_size',
					'type' => 'select',
					'responsive' => 'tablet',
					'class' => 'custombgsize',
					'css_attr' => 'background-size',
					'title' => __('Size', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'auto' => __('Auto', 'mfn-opts'),
						'contain' => __('Contain', 'mfn-opts'),
						'cover' => __('Cover', 'mfn-opts'),
						'cover-ultrawide' => __('Cover, on ultrawide screens only > 1920px', 'mfn-opts'),
						'custom' => __('Custom', 'mfn-opts'),
					],
					//'options' => mfna_bg_size()
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:background-size_mobile',
					'attr_id' => 'custombgsize_mobile',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_size',
					'type' => 'select',
					'responsive' => 'mobile',
					'class' => 'custombgsize',
					'css_attr' => 'background-size',
					'title' => __('Size', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'auto' => __('Auto', 'mfn-opts'),
						'contain' => __('Contain', 'mfn-opts'),
						'cover' => __('Cover', 'mfn-opts'),
						'cover-ultrawide' => __('Cover, on ultrawide screens only > 1920px', 'mfn-opts'),
						'custom' => __('Custom', 'mfn-opts'),
					],
					//'options' => mfna_bg_size()
				),


				array(
					'id' => 'style:.mcb-section-mfnuidelement:background-size_v2',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'custombgsize', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Custom background size', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled mfn-custom-bg-size',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:background-size_laptop_v2',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'custombgsize_laptop', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Custom background size', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled mfn-custom-bg-size',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:background-size_tablet_v2',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'custombgsize_tablet', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Custom background size', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled mfn-custom-bg-size',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:background-size_mobile_v2',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'custombgsize_mobile', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Custom background size', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled mfn-custom-bg-size',
				),



				array(
					'type' => 'html',
					'html' => '</div><div class="mfn-sft mfn-sft-hover">',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:transition',
					'type' => 'sliderbar',
					'title' => __('Transition duration', 'mfn-opts'),
					'param' => array(
						'min' => '0',
						'max' => '3',
						'step' => '0.1',
						'unit' => 's',
					)
				),

				array(
					'id' => 'background_switcher_hover',
					'attr_id' => 'background_switcher_hover',
					'type' => 'switch',
					'class' => 'mfn-fields-switcher disable-history',
					'title' => __('Background type', 'mfn-opts'),
					'options' => array(
  					'default' => __('Default', 'mfn-opts'),
  					'gradient' 	=> __('Gradient', 'mfn-opts'),
  				),
					'visual_options' => array(
  					'default' => '',
  					'gradient' 	=> '',
  				),
  				'std' => 'default',
				),

				array(
  				'id' => 'style:.mcb-section-mfnuidelement|hover:gradient',
  				'condition' => array( 'id' => 'background_switcher_hover', 'opt' => 'is', 'val' => 'gradient' ),
  				'type' => 'gradient',
  				'title' => __('Gradient', 'mfn-opts'),
  				'class' => 'mfn-toggled',
					'css_attr' => 'gradient',
  			),

				array(
					'id' => 'style:.mcb-section-mfnuidelement|hover:background-color',
					'condition' => array( 'id' => 'background_switcher_hover', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'color',
					'title' => __('Color', 'mfn-opts'),
				),

				array(
  				'id' => 'style:.mcb-section-mfnuidelement|hover:background-image',
  				'condition' => array( 'id' => 'background_switcher_hover', 'opt' => 'is', 'val' => 'default' ),
  				'oldid' => 'bg_color',
  				'dynamic_data' => 'featured_image',
  				'type' => 'upload',
  				'title' => __('Image', 'mfn-opts'),
  				'responsive' => 'desktop',

					'css_attr' => 'background-image',
  			),

  			array(
					'type' => 'html',
					'html' => '</div><div class="mfn-sft mfn-sft-scroll">',
				),

				array(
					'id' => 'style:.mfn-header-tmpl .mcb-section-mfnuidelement:transition',
					'type' => 'sliderbar',
					'style_prefix' => 'background ',
					'title' => __('Transition duration', 'mfn-opts'),
					'param' => array(
						'min' => '0',
						'max' => '3',
						'step' => '0.1',
						'unit' => 's',
					)
				),

				array(
					'id' => 'background_switcher_scroll',
					'attr_id' => 'background_switcher_scroll',
					'type' => 'switch',
					'class' => 'mfn-fields-switcher disable-history',
					'title' => __('Background type', 'mfn-opts'),
					'options' => array(
  					'default' => __('Default', 'mfn-opts'),
  					'gradient' 	=> __('Gradient', 'mfn-opts'),
  				),
					'visual_options' => array(
  					'default' => '',
  					'gradient' 	=> '',
  				),
  				'std' => 'default',
				),

				array(
  				'id' => 'style:.mfn-header-scrolled .mfn-header-tmpl .mcb-section-mfnuidelement:gradient',
  				'condition' => array( 'id' => 'background_switcher_scroll', 'opt' => 'is', 'val' => 'gradient' ),
  				'type' => 'gradient',
  				'title' => __('Gradient', 'mfn-opts'),
  				'class' => 'mfn-toggled',
					'css_attr' => 'gradient',
  			),

				array(
					'id' => 'style:.mfn-header-scrolled .mfn-header-tmpl .mcb-section-mfnuidelement:background-color',
					'condition' => array( 'id' => 'background_switcher_scroll', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'color',
					'title' => __('Color', 'mfn-opts'),
				),

				array(
  				'id' => 'style:.mfn-header-scrolled .mfn-header-tmpl .mcb-section-mfnuidelement:background-image',
  				'condition' => array( 'id' => 'background_switcher_scroll', 'opt' => 'is', 'val' => 'default' ),
  				'oldid' => 'bg_color',
  				'type' => 'upload',
  				'title' => __('Image', 'mfn-opts'),
  				'responsive' => 'desktop',

					'css_attr' => 'background-image',
  			),

				array(
					'type' => 'html',
					'html' => '</div></div>', // end: mfn-sidebar-fields-tabs
				),

				// Background Overlay

				array(
					'type' => 'header',
  				'class' => 'toggled_header row-header',
					'title' => __('Background overlay', 'mfn-opts'),
				),

				array(
					'type' => 'html',
					'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-background-overlay:opacity',
					'type' => 'sliderbar',
					'title' => __('Opacity', 'mfn-opts'),
					'param' => array(
						'min' => '0',
						'max' => '1',
						'step' => '0.01',
					),
					'std' => '0',
				),

				array(
					'id' => 'background_overlay_switcher',
					'attr_id' => 'background_overlay_switcher',
					'type' => 'switch',
					'class' => 'mfn-fields-switcher disable-history',
					'title' => __('Background type', 'mfn-opts'),
					'options' => array(
  					'default' => __('Default', 'mfn-opts'),
  					'gradient' 	=> __('Gradient', 'mfn-opts'),
  				),
					'visual_options' => array(
  					'default' => '',
  					'gradient' 	=> '',
  				),
  				'std' => 'default',
				),

				array(
  				'id' => 'style:.mcb-section-mfnuidelement .mcb-background-overlay:gradient',
  				'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'gradient' ),
  				'type' => 'gradient',
  				'title' => __('Gradient', 'mfn-opts'),
  				'class' => 'mfn-toggled',
					'css_attr' => 'gradient',
  			),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-background-overlay:background-color',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'color',
					'title' => __('Color', 'mfn-opts'),
				),

				array(
  				'id' => 'style:.mcb-section-mfnuidelement .mcb-background-overlay:background-image',
  				'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
  				'oldid' => 'bg_color',
  				'type' => 'upload',
  				'title' => __('Image', 'mfn-opts'),
  				'responsive' => 'desktop',

					'css_attr' => 'background-image',
  			),

  			array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-background-overlay:background-image_laptop',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_image',
					'type' => 'upload',
					'title' => __('Image', 'mfn-opts'),
					'responsive' => 'laptop',

					'css_attr' => 'background-image',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-background-overlay:background-image_tablet',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_image',
					'type' => 'upload',
					'title' => __('Image', 'mfn-opts'),
					'responsive' => 'tablet',

					'css_attr' => 'background-image',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-background-overlay:background-image_mobile',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'upload',
					'title' => __('Image', 'mfn-opts'),
					'responsive' => 'mobile',

					'css_attr' => 'background-image',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-background-overlay:background-repeat',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'desktop',

					'css_attr' => 'background-repeat',
					'title' => __('Repeat', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'repeat' => __('Repeat', 'mfn-opts'),
						'no-repeat' => __('No repeat', 'mfn-opts'),
						'repeat-x' => __('Repeat X', 'mfn-opts'),
						'repeat-y' => __('Repeat Y', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-background-overlay:background-repeat_laptop',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'laptop',

					'css_attr' => 'background-repeat',
					'title' => __('Repeat', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'repeat' => __('Repeat', 'mfn-opts'),
						'no-repeat' => __('No repeat', 'mfn-opts'),
						'repeat-x' => __('Repeat X', 'mfn-opts'),
						'repeat-y' => __('Repeat Y', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-background-overlay:background-repeat_tablet',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'tablet',

					'css_attr' => 'background-repeat',
					'title' => __('Repeat', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'repeat' => __('Repeat', 'mfn-opts'),
						'no-repeat' => __('No repeat', 'mfn-opts'),
						'repeat-x' => __('Repeat X', 'mfn-opts'),
						'repeat-y' => __('Repeat Y', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-background-overlay:background-repeat_mobile',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'mobile',

					'css_attr' => 'background-repeat',
					'title' => __('Repeat', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'repeat' => __('Repeat', 'mfn-opts'),
						'no-repeat' => __('No repeat', 'mfn-opts'),
						'repeat-x' => __('Repeat X', 'mfn-opts'),
						'repeat-y' => __('Repeat Y', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-background-overlay:background-position',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_position',
					'type' => 'select',
					'responsive' => 'desktop',

					'css_attr' => 'background-position',
					'title' => __('Position', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'center' => __('Center Center', 'mfn-opts'),
						'center left' => __('Center Left', 'mfn-opts'),
						'center right' => __('Center Right', 'mfn-opts'),
						'top left' => __('Top Left', 'mfn-opts'),
						'top right' => __('Top Right', 'mfn-opts'),
						'top center' => __('Top Center', 'mfn-opts'),
						'bottom left' => __('Bottom Left', 'mfn-opts'),
						'bottom right' => __('Bottom Right', 'mfn-opts'),
						'bottom center' => __('Bottom Center', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-background-overlay:background-position_laptop',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_position',
					'type' => 'select',
					'responsive' => 'laptop',

					'css_attr' => 'background-position',
					'title' => __('Position', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'center' => __('Center Center', 'mfn-opts'),
						'center left' => __('Center Left', 'mfn-opts'),
						'center right' => __('Center Right', 'mfn-opts'),
						'top left' => __('Top Left', 'mfn-opts'),
						'top right' => __('Top Right', 'mfn-opts'),
						'top center' => __('Top Center', 'mfn-opts'),
						'bottom left' => __('Bottom Left', 'mfn-opts'),
						'bottom right' => __('Bottom Right', 'mfn-opts'),
						'bottom center' => __('Bottom Center', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-background-overlay:background-position_tablet',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_position',
					'type' => 'select',
					'responsive' => 'tablet',

					'css_attr' => 'background-position',
					'title' => __('Position', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'center' => __('Center Center', 'mfn-opts'),
						'center left' => __('Center Left', 'mfn-opts'),
						'center right' => __('Center Right', 'mfn-opts'),
						'top left' => __('Top Left', 'mfn-opts'),
						'top right' => __('Top Right', 'mfn-opts'),
						'top center' => __('Top Center', 'mfn-opts'),
						'bottom left' => __('Bottom Left', 'mfn-opts'),
						'bottom right' => __('Bottom Right', 'mfn-opts'),
						'bottom center' => __('Bottom Center', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-background-overlay:background-position_mobile',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_position',
					'type' => 'select',
					'responsive' => 'mobile',

					'css_attr' => 'background-position',
					'title' => __('Position', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'center' => __('Center Center', 'mfn-opts'),
						'center left' => __('Center Left', 'mfn-opts'),
						'center right' => __('Center Right', 'mfn-opts'),
						'top left' => __('Top Left', 'mfn-opts'),
						'top right' => __('Top Right', 'mfn-opts'),
						'top center' => __('Top Center', 'mfn-opts'),
						'bottom left' => __('Bottom Left', 'mfn-opts'),
						'bottom right' => __('Bottom Right', 'mfn-opts'),
						'bottom center' => __('Bottom Center', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-background-overlay:background-attachment',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'desktop',

					'css_attr' => 'background-attachment',
					'title' => __('Attachment', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'scroll' => __('Scroll', 'mfn-opts'),
						'fixed' => __('Fixed', 'mfn-opts'),
						'parallax' => __('Parallax', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-background-overlay:background-attachment_laptop',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'laptop',

					'css_attr' => 'background-attachment',
					'title' => __('Attachment', 'mfn-opts'),
					'desc' => __('iOS does <b>not</b> support background-attachment: fixed', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'scroll' => __('Scroll', 'mfn-opts'),
						'fixed' => __('Fixed', 'mfn-opts'),
						'parallax' => __('Parallax', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-background-overlay:background-attachment_tablet',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'tablet',

					'css_attr' => 'background-attachment',
					'title' => __('Attachment', 'mfn-opts'),
					'desc' => __('iOS does <b>not</b> support background-attachment: fixed', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'scroll' => __('Scroll', 'mfn-opts'),
						'fixed' => __('Fixed', 'mfn-opts'),
						'parallax' => __('Parallax', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-background-overlay:background-attachment_mobile',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'mobile',

					'css_attr' => 'background-attachment',
					'title' => __('Attachment', 'mfn-opts'),
					'desc' => __('iOS does <b>not</b> support background-attachment: fixed', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'scroll' => __('Scroll', 'mfn-opts'),
						'fixed' => __('Fixed', 'mfn-opts'),
						'parallax' => __('Parallax', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-background-overlay:background-size',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_size',
					'type' => 'select',
					'responsive' => 'desktop',

					'css_attr' => 'background-size',
					'title' => __('Size', 'mfn-opts'),
					'options' => mfna_bg_size()
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-background-overlay:background-size_laptop',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_size',
					'type' => 'select',
					'responsive' => 'laptop',

					'css_attr' => 'background-size',
					'title' => __('Size', 'mfn-opts'),
					'options' => mfna_bg_size()
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-background-overlay:background-size_tablet',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_size',
					'type' => 'select',
					'responsive' => 'tablet',

					'css_attr' => 'background-size',
					'title' => __('Size', 'mfn-opts'),
					'options' => mfna_bg_size()
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-background-overlay:background-size_mobile',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_size',
					'type' => 'select',
					'responsive' => 'mobile',

					'css_attr' => 'background-size',
					'title' => __('Size', 'mfn-opts'),
					'options' => mfna_bg_size()
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-background-overlay:filter',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'css_filters',
					//'label_after' => '<a class="reset-css-filters" name="reset-css-filters" href="#">RES</a><a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
					'label_after' => '<a name="reset-css-filters" href="#" class="mfn-option-btn mfn-option-blank btn-medium reset-big-field reset-css-filters" data-tooltip="Reset"><span class="mfn-icon mfn-icon-reset"></span></a><a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
					'class' => 'mfn-toggled toggle_fields',
					'title' => __('CSS Filters', 'mfn-opts'),
				),

				array(
					'type' => 'html',
					'html' => '</div><div class="mfn-sft mfn-sft-hover">',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement .mcb-background-overlay:transition',
					'type' => 'sliderbar',
					'title' => __('Transition duration', 'mfn-opts'),
					'param' => array(
						'min' => '0',
						'max' => '3',
						'step' => '0.1',
						'unit' => 's',
					)
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement|hover .mcb-background-overlay:opacity',
					'type' => 'sliderbar',
					'title' => __('Opacity', 'mfn-opts'),
					'param' => array(
						'min' => '0',
						'max' => '1',
						'step' => '0.01',
					),
					//'std' => '1',
				),

				array(
					'id' => 'background_switcher_hover',
					'attr_id' => 'background_switcher_hover',
					'type' => 'switch',
					'class' => 'mfn-fields-switcher disable-history',
					'title' => __('Background type', 'mfn-opts'),
					'options' => array(
						'default' => __('Default', 'mfn-opts'),
						'gradient' 	=> __('Gradient', 'mfn-opts'),
					),
					'visual_options' => array(
						'default' => '',
						'gradient' 	=> '',
					),
					'std' => 'default',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement|hover .mcb-background-overlay:gradient',
					'condition' => array( 'id' => 'background_switcher_hover', 'opt' => 'is', 'val' => 'gradient' ),
					'type' => 'gradient',
					'title' => __('Gradient', 'mfn-opts'),
					'class' => 'mfn-toggled',
					'css_attr' => 'gradient',
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement|hover .mcb-background-overlay:background-color',
					'condition' => array( 'id' => 'background_switcher_hover', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'color',
					'title' => __('Color', 'mfn-opts'),
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement|hover .mcb-background-overlay:background-image',
					'condition' => array( 'id' => 'background_switcher_hover', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_color',
					'type' => 'upload',
					'title' => __('Image', 'mfn-opts'),
					'responsive' => 'desktop',

					'css_attr' => 'background-image',
				),

				array(
					'type' => 'html',
					'html' => '</div></div>', // end: mfn-sidebar-fields-tabs
				),

				array(
					'type' => 'header',
					'class' => 'toggled_header row-header',
					'title' => __('Border', 'mfn-opts'),
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:border-style',
					'attr_id' => 'border_style_sect',
					'type' => 'select',
					'class' => 'mfn-toggled',
					'title' => __('Border style', 'mfn-opts'),
					'options' => [
						'none' => __('None', 'mfn-opts'),
						'solid' => __('Solid', 'mfn-opts'),
						'dashed' => __('Dashed', 'mfn-opts'),
						'dotted' => __('Dotted', 'mfn-opts'),
						'double' => __('Double', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section-mfnuidelement:border-color',
					'condition' => array( 'id' => 'border_style_sect', 'opt' => 'isnt', 'val' => 'none' ),
					'type' => 'color',
					'class' => 'mfn-toggled',
					'title' => __('Border color', 'mfn-opts'),
				),

				array(
  				'id' => 'style:.mcb-section-mfnuidelement:border-width',
  				'condition' => array( 'id' => 'border_style_sect', 'opt' => 'isnt', 'val' => 'none' ),
  				'type' => 'dimensions',
  				'title' => __('Border width', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-width',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement:border-width_laptop',
  				'condition' => array( 'id' => 'border_style_sect', 'opt' => 'isnt', 'val' => 'none' ),
  				'type' => 'dimensions',
  				'title' => __('Border width', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-width',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement:border-width_tablet',
  				'condition' => array( 'id' => 'border_style_sect', 'opt' => 'isnt', 'val' => 'none' ),
  				'type' => 'dimensions',
  				'title' => __('Border width', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-width',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement:border-width_mobile',
  				'condition' => array( 'id' => 'border_style_sect', 'opt' => 'isnt', 'val' => 'none' ),
  				'type' => 'dimensions',
  				'title' => __('Border width', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-width',
  			),

				array(
  				'id' => 'style:.mcb-section-mfnuidelement:border-radius',
  				'type' => 'dimensions',
  				'title' => __('Border radius', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-radius',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement:border-radius_laptop',
  				'type' => 'dimensions',
  				'title' => __('Border radius', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-radius',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement:border-radius_tablet',
  				'type' => 'dimensions',
  				'title' => __('Border radius', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-radius',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement:border-radius_mobile',
  				'type' => 'dimensions',
  				'title' => __('Border radius', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-radius',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement:box-shadow',
  				'type' => 'box_shadow',
  				'title' => __('Box shadow', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'box-shadow'
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement:box-shadow_laptop',
  				'type' => 'box_shadow',
  				'title' => __('Box shadow', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'css_attr' => 'box-shadow',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement:box-shadow_tablet',
  				'type' => 'box_shadow',
  				'title' => __('Box shadow', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'box-shadow',
  			),

  			array(
  				'id' => 'style:.mcb-section-mfnuidelement:box-shadow_mobile',
  				'type' => 'box_shadow',
  				'title' => __('Box shadow', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'box-shadow',
  			),


  			array(
  				'type' => 'header',
					'class' => 'toggled_header row-header',
					'title' => __('Responsive', 'mfn-opts'),
				),

				array(
  				'id' => 'visibility',
  				'iconinfo' => 'https://support.muffingroup.com/video-tutorials/section-responsive-visibility/',
  				'type' => 'switch',
  				'invert' => true,
  				'version' => 'multiple',
  				'class' => 'mfn-toggled',
  				'title' => __('Responsive visibility', 'mfn-opts'),
  				'active_tooltip' => array(
  					'hide-desktop' => __('Show on Desktop', 'mfn-opts'),
  					'hide-laptop' => __('Show on Laptop', 'mfn-opts'),
  					'hide-tablet' => __('Show on Tablet', 'mfn-opts'),
  					'hide-mobile' => __('Show on Mobile', 'mfn-opts'),
  				),
  				'options' => array(
  					'hide-desktop' => __('Hide on Desktop', 'mfn-opts'),			// 960 +
  					'hide-laptop' => __('Hide on Laptop', 'mfn-opts'),
  					'hide-tablet' => __('Hide on Tablet', 'mfn-opts'),		// 768 - 959
  					'hide-mobile' => __('Hide on Mobile', 'mfn-opts'),			// - 768
  				),
  				'visual_options' => array(
  					'hide-desktop' => '<svg id="Layer_1" data-name="Layer 1" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 22 22"><defs><style>.cls-1{fill:none;stroke:#304050;stroke-linecap:round;stroke-linejoin:round;stroke-width:1.5px;}.cls-2{fill:#304050;}</style></defs><path class="cls-1" d="M4.94,5H17.06a.93.93,0,0,1,.94.91v7.18a.93.93,0,0,1-.94.91H4.94A.93.93,0,0,1,4,13.09V5.91A.93.93,0,0,1,4.94,5Z"/><path class="cls-1" d="M7,17h8"/><path class="cls-2" d="M13,15.51a1.58,1.58,0,0,0,.7,1.26L14,17H8l.3-.23A1.58,1.58,0,0,0,9,15.51V14h4Z"/></svg>',			// 960 +
  					'hide-laptop' => '<svg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"viewBox="0 0 22 22" style="enable-background:new 0 0 22 22;" xml:space="preserve"> <style type="text/css"> .st0{fill:none;stroke:currentColor;stroke-width:1.5;stroke-linecap:round;stroke-linejoin:round;} </style> <path class="st0" d="M5,15V6.9C5,6.4,5.4,5.9,5.9,6h10.1c0.5,0,0.9,0.4,0.9,0.9V15"/> <path class="st0" d="M4,17h14"/> </svg>',		// 768 - 959
  					'hide-tablet' => '<svg id="Layer_1" data-name="Layer 1" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 22 22"><defs><style>.cls-1{fill:none;stroke:#304050;stroke-linecap:round;stroke-linejoin:round;stroke-width:1.5px;}</style></defs><path class="cls-1" d="M7,15.83V7.17A1.21,1.21,0,0,1,8.25,6h5.5A1.21,1.21,0,0,1,15,7.17v8.66A1.21,1.21,0,0,1,13.75,17H8.25A1.21,1.21,0,0,1,7,15.83Z"/></svg>',		// 768 - 959
  					'hide-mobile' => '<svg id="Layer_1" data-name="Layer 1" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 22 22"><defs><style>.cls-1{fill:none;stroke:#304050;stroke-linecap:round;stroke-linejoin:round;stroke-width:1.5px;}.cls-2{fill:#304050;}</style></defs><path class="cls-1" d="M8,15.82V8.18A1.08,1.08,0,0,1,8.94,7h4.12A1.08,1.08,0,0,1,14,8.18v7.64A1.08,1.08,0,0,1,13.06,17H8.94A1.08,1.08,0,0,1,8,15.82Z"/><circle class="cls-2" cx="11" cy="14" r="1"/></svg>',			// - 768
  				),
  			),

  			array(
  				'id' => 'reverse_order',
  				'class' => 'mfn-toggled hide_if_ql',
  				'type' => 'switch',
  				'iconinfo' => 'https://support.muffingroup.com/how-to/how-to-reverse-the-order-of-elements-on-mobile/',
  				'title' => __('Order on mobile', 'mfn-opts'),
					'options' => [
						0 => __('Default', 'mfn-opts'),
						1 => __('Reverse rows', 'mfn-opts'),
						2 => __('Reverse columns', 'mfn-opts'),
					],
					'std' => 0,
  			),

  			// Conditions

				array(
					'form' => 'advanced',
					'type' => 'header',
					'class' => 'toggled_header row-header',
					'title' => __('Conditional logic', 'mfn-opts'),
				),

				array(
					'form' => 'advanced',
  				'id' => 'conditional_logic',
  				'class' => 'mfn-toggled',
  				'type' => 'logic',
  				'title' => __('Conditional logic', 'mfn-opts'),
  			),

				array(
					'type' => 'header',
					'class' => 'toggled_header row-header',
					'title' => __('Custom', 'mfn-opts'),
				),

				array(
  				'id' => 'classes',
  				'oldid' => 'class',
  				'class' => 'mfn-toggled',
  				'type' => 'pills',
  				'title' => __('Classes', 'mfn-opts'),
  			),

  			array(
  				'id' => 'custom_id',
  				'oldid' => 'section_id',
  				'class' => 'mfn-toggled',
  				'type' => 'text',
  				'title' => __('ID', 'mfn-opts'),
  			),

  			array(
  				'id' => 'custom_css',
  				'oldid' => 'style',
  				'class' => 'mfn-toggled',
  				'type' => 'textarea',
  				'title' => __('Additional CSS', 'mfn-opts'),
  			),

			  array(
			  	'type' => 'header',
					'id' => 'global_sections',
					'class' => 'toggled_header global_sections row-header',
					'title' => __('Global Section', 'mfn-opts'),
				),

				array(
					'id' => 'global_sections_select',
					'attr_id' => 'global_sections_select',
					'class' => 'mfn-toggled global_sections',
					'type' => 'select',
					'title' => __('Select template', 'mfn-opts'),
					'std' => '',
					'js_options' => 'global_sections',
					'php_options' => mfna_templates('section'),
				),

  			array(
					'type' => 'html',
					'html' => '</div>', // end: modalbox-card
				)

  		);

  	}

    /**
     * SET wrap fields
   	 */

    private function set_wrap()
  	{
  		$this->wrap = array(

  			/**
  			 * HEADER additional fields
  			* */

				array(
					'type' => 'html',
					'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
				),

				// options

				array(
					'type' => 'header',
					'class' => 'hidden-header-builder',
					'title' => __('Options', 'mfn-opts'),
				),

  			array(
  				'id' => 'sticky',
  				'attr_id' => 'wrap_sticky_desktop',
  				'class' => 'hidden-header-builder mfn-hidden-nested-wrap',
  				'type' => 'switch',
  				'responsive' => 'desktop',
  				'title' => __('Sticky', 'mfn-opts'),
  				'desc' => __('Does <b>not</b> work with Move up and Parallax', 'mfn-opts'),
					'options' => [
						0 => __('Disable', 'mfn-opts'),
						1 => __('Enable', 'mfn-opts'),
					],
					'std' => 0,
  			),

  			array(
  				'id' => 'laptop_sticky',
  				'attr_id' => 'wrap_sticky_laptop',
  				'class' => 'hidden-header-builder mfn-hidden-nested-wrap',
  				'type' => 'switch',
  				'responsive' => 'laptop',
  				'title' => __('Sticky', 'mfn-opts'),
  				'desc' => __('Does <b>not</b> work with Move up and Parallax', 'mfn-opts'),
					'options' => [
						0 => __('Disable', 'mfn-opts'),
						1 => __('Enable', 'mfn-opts'),
					],
					'std' => 0,
  			),

  			array(
  				'id' => 'tablet_sticky',
  				'attr_id' => 'wrap_sticky_tablet',
  				'class' => 'hidden-header-builder mfn-hidden-nested-wrap',
  				'type' => 'switch',
  				'responsive' => 'tablet',
  				'title' => __('Sticky', 'mfn-opts'),
  				'desc' => __('Does <b>not</b> work with Move up and Parallax', 'mfn-opts'),
					'options' => [
						0 => __('Disable', 'mfn-opts'),
						1 => __('Enable', 'mfn-opts'),
					],
					'std' => 0,
  			),

  			array(
  				'id' => 'mobile_sticky',
  				'attr_id' => 'wrap_sticky_mobile',
  				'class' => 'hidden-header-builder mfn-hidden-nested-wrap',
  				'type' => 'switch',
  				'responsive' => 'mobile',
  				'title' => __('Sticky', 'mfn-opts'),
  				'desc' => __('Does <b>not</b> work with Move up and Parallax', 'mfn-opts'),
					'options' => [
						0 => __('Disable', 'mfn-opts'),
						1 => __('Enable', 'mfn-opts'),
					],
					'std' => 0,
  			),


  			array(
  				'id' => 'sticky_offset',
					'condition' => array( 'id' => 'wrap_sticky_desktop', 'opt' => 'is', 'val' => '1' ),
					'type' => 'sliderbar',
					'class' => 'hidden-header-builder mfn-hidden-nested-wrap',
					'responsive' => 'desktop',
					'title' => __('Sticky offset', 'mfn-opts'),
					'desc' => __('Offset from top during scrolling', 'mfn-opts'),
					'param' => array(
						'min' => '0',
						'max' => '1000',
						'step' => '1',
						'unit' => '',
					)
  			),

  			array(
  				'id' => 'sticky_offset_laptop',
					'condition' => array( 'id' => 'wrap_sticky_laptop', 'opt' => 'is', 'val' => '1' ),
					'type' => 'sliderbar',
					'class' => 'hidden-header-builder mfn-hidden-nested-wrap',
					'responsive' => 'laptop',
					'title' => __('Sticky offset', 'mfn-opts'),
					'desc' => __('Offset from top during scrolling', 'mfn-opts'),
					'param' => array(
						'min' => '0',
						'max' => '1000',
						'step' => '1',
						'unit' => '',
					)
  			),

  			array(
  				'id' => 'sticky_offset_tablet',
					'condition' => array( 'id' => 'wrap_sticky_tablet', 'opt' => 'is', 'val' => '1' ),
					'type' => 'sliderbar',
					'class' => 'hidden-header-builder mfn-hidden-nested-wrap',
					'responsive' => 'tablet',
					'title' => __('Sticky offset', 'mfn-opts'),
					'desc' => __('Offset from top during scrolling', 'mfn-opts'),
					'param' => array(
						'min' => '0',
						'max' => '1000',
						'step' => '1',
						'unit' => '',
					)
  			),

  			array(
  				'id' => 'sticky_offset_mobile',
					'condition' => array( 'id' => 'wrap_sticky_mobile', 'opt' => 'is', 'val' => '1' ),
					'type' => 'sliderbar',
					'class' => 'hidden-header-builder mfn-hidden-nested-wrap',
					'responsive' => 'mobile',
					'title' => __('Sticky offset', 'mfn-opts'),
					'desc' => __('Offset from top during scrolling', 'mfn-opts'),
					'param' => array(
						'min' => '0',
						'max' => '1000',
						'step' => '1',
						'unit' => '',
					)
  			),

  			array(
					'id' => 'info',
					'class' => 'visible-header-builder mfn-alert',
					'type' => 'info',
					'title' => __('This element has no attributes. Please check <b>advanced</b> tab for more customisation options.', 'mfn-opts'),
				),












  			array(
  				'id' => 'type',
  				're_render' => true,
  				'attr_id' => 'wrap_type',
  				'class' => 'mfn-hidden-nested-wrap hidden-header-builder mfn-loop-switcher',
  				'type' => 'switch',
  				'title' => __('Type', 'mfn-opts'),
  				'desc' => __('Query loop displays wrap&apos;s content in designed loop', 'mfn-opts'),
  				'options' => array(
  					'' => __('Default', 'mfn-opts'),
  					'query' => __('Query loop', 'mfn-opts'),
  				),
  				'std' => ''
  			),


  			/* query type */

  			array(
  				'id' => 'query_type',
  				're_render' => true,
  				'attr_id' => 'query_type',
  				'class' => 'mfn-hidden-nested-wrap hidden-header-builder hidden-blog-builder hidden-portfolio-builder',
  				'condition' => array( 'id' => 'wrap_type', 'opt' => 'is', 'val' => 'query' ),
  				'type' => 'select',
  				'title' => __('Query type', 'mfn-opts'),
  				'options' => array(
  					'' => __('Select', 'mfn-opts'),
  					'posts' => __('Posts', 'mfn-opts'),
  					'terms' => __('Terms', 'mfn-opts')
  				),
  				'std' => ''
  			),

  			/* posts query builder */

  			array(
  				'id' => 'query_post_type',
  				're_render' => true,
  				'attr_id' => 'query_post_type',
  				'class' => 'mfn-hidden-nested-wrap hidden-header-builder hidden-blog-builder hidden-portfolio-builder',
  				'condition' => array( 'id' => 'query_type', 'opt' => 'is', 'val' => 'posts' ),
  				'type' => 'select',
  				'title' => __('Post type', 'mfn-opts'),
  				'options' => array(
  					'' => __('- Select -', 'mfn-opts'),
  					'post' => __('Post', 'mfn-opts'),
  					'page' => __('Page', 'mfn-opts'),
  					'portfolio' => __('Portfolio', 'mfn-opts'),
  					'offer' => __('Offer', 'mfn-opts'),
  					'slide' => __('Slides', 'mfn-opts'),
  					'product' => __('Product', 'mfn-opts'),
  					'testimonial' => __('Testimonial', 'mfn-opts'),
  				),
  				'std' => 'post'
  			),

  			array(
					'id' => 'query_post_type_product',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'product' ),
					're_render' => true,
					'class' => 'mfn-hidden-nested-wrap mfn-reset-if-term',
					'type' => 'multiselect',
					'title' => __('Categories included', 'mfn-opts'),
					//'hierarchical_options' => mfn_hierarchical_taxonomy('product_cat'),
					'js_hierarchical_options' => 'product_cat',
					'opt_append' => array(
						'0-current' => 'Current'
					),
					'preview' => 'category',
				),

				array(
					'id' => 'query_post_type_testimonial',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'testimonial' ),
					're_render' => true,
					'class' => 'mfn-hidden-nested-wrap mfn-reset-if-term',
					'type' => 'multiselect',
					'title' => __('Categories included', 'mfn-opts'),
					//'options' => mfn_get_categories('testimonial-types'),
   				'js_hierarchical_options' => 'testimonial_types',
					'preview' => 'category',
				),

				array(
					'id' => 'query_post_type_post',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'post' ),
					're_render' => true,
					'class' => 'mfn-hidden-nested-wrap mfn-reset-if-term hidden-blog-builder hidden-portfolio-builder',
					'type' => 'multiselect',
					'title' => __('Categories included', 'mfn-opts'),
					//'hierarchical_options' => mfn_get_categories('category'),
					'js_hierarchical_options' => 'blog_categories',
					'opt_append' => array(
						'0-current' => 'Current'
					),
					'preview' => 'category',
				),

				array(
					'id' => 'query_post_type_portfolio',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'portfolio' ),
					're_render' => true,
					'class' => 'mfn-hidden-nested-wrap mfn-reset-if-term hidden-blog-builder hidden-portfolio-builder',
					'type' => 'multiselect',
					'title' => __('Categories included', 'mfn-opts'),
					//'hierarchical_options' => mfn_get_categories('portfolio-types'),
					'js_hierarchical_options' => 'portfolio_types',
					'opt_append' => array(
						'0-current' => 'Current'
					),
					'preview' => 'category',
				),

				array(
					'id' => 'query_post_type_client',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'client' ),
					're_render' => true,
					'class' => 'mfn-hidden-nested-wrap mfn-reset-if-term',
					'type' => 'multiselect',
					'title' => __('Categories included', 'mfn-opts'),
					//'hierarchical_options' => mfn_get_categories('client-types'),
					'js_hierarchical_options' => 'client_types',
					'preview' => 'category',
				),

				array(
					'id' => 'query_post_type_offer',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'offer' ),
					're_render' => true,
					'class' => 'mfn-hidden-nested-wrap mfn-reset-if-term',
					'type' => 'multiselect',
					'title' => __('Categories included', 'mfn-opts'),
					//'hierarchical_options' => mfn_get_categories('offer-types'),
					'js_hierarchical_options' => 'offer_types',
					'preview' => 'category',
				),

				array(
					'id' => 'query_post_type_slide',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'slide' ),
					're_render' => true,
					'class' => 'mfn-hidden-nested-wrap mfn-reset-if-term',
					'type' => 'multiselect',
					'title' => __('Categories included', 'mfn-opts'),
					//'hierarchical_options' => mfn_get_categories('slide-types'),
					'js_hierarchical_options' => 'slide_types',
					'preview' => 'category',
				),










				array(
					'id' => 'query_post_type_product_exclude',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'product' ),
					're_render' => true,
					'class' => 'mfn-hidden-nested-wrap mfn-reset-if-term',
					'type' => 'multiselect',
					'title' => __('Categories excluded', 'mfn-opts'),
					//'hierarchical_options' => mfn_hierarchical_taxonomy('product_cat'),
					'js_hierarchical_options' => 'product_cat',
					'opt_append' => array(
						'0-current' => 'Current'
					),
					'preview' => 'category',
				),

				array(
					'id' => 'query_post_type_testimonial_exclude',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'testimonial' ),
					're_render' => true,
					'class' => 'mfn-hidden-nested-wrap mfn-reset-if-term',
					'type' => 'multiselect',
					'title' => __('Categories excluded', 'mfn-opts'),
					//'options' => mfn_get_categories('testimonial-types'),
   				'js_hierarchical_options' => 'testimonial_types',
					'preview' => 'category',
				),

				array(
					'id' => 'query_post_type_post_exclude',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'post' ),
					're_render' => true,
					'class' => 'mfn-hidden-nested-wrap mfn-reset-if-term hidden-blog-builder hidden-portfolio-builder',
					'type' => 'multiselect',
					'title' => __('Categories excluded', 'mfn-opts'),
					//'hierarchical_options' => mfn_get_categories('category'),
					'js_hierarchical_options' => 'blog_categories',
					'opt_append' => array(
						'0-current' => 'Current'
					),
					'preview' => 'category',
				),

				array(
					'id' => 'query_post_type_portfolio_exclude',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'portfolio' ),
					're_render' => true,
					'class' => 'mfn-hidden-nested-wrap mfn-reset-if-term hidden-blog-builder hidden-portfolio-builder',
					'type' => 'multiselect',
					'title' => __('Categories excluded', 'mfn-opts'),
					//'hierarchical_options' => mfn_get_categories('portfolio-types'),
					'js_hierarchical_options' => 'portfolio_types',
					'opt_append' => array(
						'0-current' => 'Current'
					),
					'preview' => 'category',
				),

				array(
					'id' => 'query_post_type_client_exclude',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'client' ),
					're_render' => true,
					'class' => 'mfn-hidden-nested-wrap mfn-reset-if-term',
					'type' => 'multiselect',
					'title' => __('Categories excluded', 'mfn-opts'),
					//'hierarchical_options' => mfn_get_categories('client-types'),
					'js_hierarchical_options' => 'client_types',
					'preview' => 'category',
				),

				array(
					'id' => 'query_post_type_offer_exclude',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'offer' ),
					're_render' => true,
					'class' => 'mfn-hidden-nested-wrap mfn-reset-if-term',
					'type' => 'multiselect',
					'title' => __('Categories excluded', 'mfn-opts'),
					//'hierarchical_options' => mfn_get_categories('offer-types'),
					'js_hierarchical_options' => 'offer_types',
					'preview' => 'category',
				),

				array(
					'id' => 'query_post_type_slide_exclude',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'slide' ),
					're_render' => true,
					'class' => 'mfn-hidden-nested-wrap mfn-reset-if-term',
					'type' => 'multiselect',
					'title' => __('Categories excluded', 'mfn-opts'),
					//'hierarchical_options' => mfn_get_categories('slide-types'),
					'js_hierarchical_options' => 'slide_types',
					'preview' => 'category',
				),








				array(
					'id' => 'query_post_type_product_order',
					'condition' => array( 'id' => 'query_post_type', 'opt' => 'is', 'val' => 'product' ),
					're_render' => true,
					'type' => 'select',
					'class' => 'mfn-hidden-nested-wrap',
					'desc' => __('This option turns off standard order settings', 'mfn-opts'),
					'title' => __('Display', 'mfn-opts'),
					'options' => array(
						'' => __('Default', 'mfn-opts'),
						'on_sale' => __('On sale', 'mfn-opts'),
						'best_selling' => __('Best selling', 'mfn-opts'),
						'top_rated' => __('Top rated', 'mfn-opts'),
					),
					'std' => '',
				),

  			array(
  				'id' => 'query_post_orderby',
  				're_render' => true,
  				'class' => 'mfn-hidden-nested-wrap hidden-header-builder',
  				'condition' => array( 'id' => 'query_type', 'opt' => 'is', 'val' => 'posts' ),
  				'type' => 'select',
  				'title' => __('Order by', 'mfn-opts'),
  				'options' => array(
  					'' => __('None', 'mfn-opts'),
  					'ID' => __('ID', 'mfn-opts'),
  					'date' => __('Date', 'mfn-opts'),
  					'modified' => __('Modified', 'mfn-opts'),
  					'menu_order' => __('Menu order', 'mfn-opts'),
  					'title' => __('Title', 'mfn-opts'),
  					'rand' => __('Rand', 'mfn-opts'),
  				),
  				'std' => 'date'
  			),

  			array(
  				'id' => 'query_post_order',
  				're_render' => true,
  				'class' => 'mfn-hidden-nested-wrap hidden-header-builder',
  				'condition' => array( 'id' => 'query_type', 'opt' => 'is', 'val' => 'posts' ),
  				'type' => 'select',
  				'title' => __('Order', 'mfn-opts'),
  				'options' => array(
  					'ASC' => __('Ascending', 'mfn-opts'),
  					'DESC' => __('Descending', 'mfn-opts'),
  				),
  				'std' => 'ASC'
  			),

  			array(
  				'id' => 'query_post_per_page',
  				're_render' => true,
  				'class' => 'mfn-hidden-nested-wrap hidden-header-builder',
  				'condition' => array( 'id' => 'query_type', 'opt' => 'is', 'val' => 'posts' ),
  				'type' => 'text',
  				'title' => __('Limit', 'mfn-opts'),
  				'desc' => __('By default limit from Admin > Settings > Reading > Blog pages show at most', 'mfn-opts'),
  				'param' => 'number',
					'preview' => 'number',
					'std' => '0',
					'js_std' => 'per_page',
  			),

  			array(
  				'id' => 'query_post_offset',
  				're_render' => true,
  				'class' => 'mfn-hidden-nested-wrap hidden-header-builder',
  				'condition' => array( 'id' => 'query_type', 'opt' => 'is', 'val' => 'posts' ),
  				'type' => 'text',
  				'title' => __('Offset', 'mfn-opts'),
  				'desc' => __('This option omits provided quantity of posts (from the beginning)', 'mfn-opts'),
  				'param' => 'number',
					'preview' => 'number',
					'std' => '0',
  			),

  			/*array(
  				'id' => 'query_post_pagination',
  				'attr_id' => 'query_post_pagination',
  				're_render' => true,
  				'class' => 'hidden-header-builder',
  				'condition' => array( 'id' => 'query_type', 'opt' => 'is', 'val' => 'posts' ),
  				'type' => 'select',
  				'title' => __('Pagination', 'mfn-opts'),
  				'options' => array(
  					'' => __('Hidden', 'mfn-opts'),
  					'numbers' => __('Numbers + links', 'mfn-opts'),
  					'dots' => __('Dots + links', 'mfn-opts'),
  					'prevnext' => __('Prev / Next links', 'mfn-opts'),
  					'loadmore' => __('Load more', 'mfn-opts'),
  					'infiniteload' => __('Infinite load', 'mfn-opts'),
  				),
  				'std' => ''
  			),*/

  			/* terms query builder */

  			array(
  				'id' => 'query_terms_taxonomy',
  				'attr_id' => 'query_terms_taxonomy',
  				're_render' => true,
  				'class' => 'mfn-hidden-nested-wrap hidden-header-builder',
  				'condition' => array( 'id' => 'query_type', 'opt' => 'is', 'val' => 'terms' ),
  				'type' => 'select',
  				'title' => __('Taxonomy', 'mfn-opts'),
  				'options' => array(
  					'' => __('- Select -', 'mfn-opts'),
  					'category' => __('Post category', 'mfn-opts'),
  					'post_tag' => __('Post tag', 'mfn-opts'),
  					'portfolio_types' => __('Portfolio category', 'mfn-opts'),
  					'offer_types' => __('Offer category', 'mfn-opts'),
  					'product_cat' => __('Product category', 'mfn-opts'),
  					'testimonial_types' => __('Testimonial category', 'mfn-opts'),
  				),
  				'std' => 'category'
  			),

  			array(
  				'id' => 'query_terms_excludes_post_tag',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'post_tag' ),
  				're_render' => true,
  				'type' => 'multiselect',
  				'class' => 'mfn-hidden-nested-wrap',
  				'title' => __('Excludes', 'mfn-opts'),
  				'desc' => __('IDs of taxonomies to exclude', 'mfn-opts'),
  				//'options' => mfn_get_categories('post_tag'),
   				'js_hierarchical_options' => 'blog_tags',
  			),

  			array(
  				'id' => 'query_terms_excludes_client_types',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'client_types' ),
  				're_render' => true,
  				'class' => 'mfn-hidden-nested-wrap',
  				'type' => 'multiselect',
  				'title' => __('Excludes', 'mfn-opts'),
  				'desc' => __('IDs of taxonomies to exclude', 'mfn-opts'),
  				//'options' => mfn_get_categories('client-types'),
   				'js_hierarchical_options' => 'client_types',
  			),

  			array(
  				'id' => 'query_terms_excludes_portfolio_types',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'portfolio_types' ),
  				're_render' => true,
  				'type' => 'multiselect',
  				'class' => 'mfn-hidden-nested-wrap',
  				'title' => __('Excludes', 'mfn-opts'),
  				'desc' => __('IDs of taxonomies to exclude', 'mfn-opts'),
  				//'options' => mfn_get_categories('portfolio-types'),
   				'js_hierarchical_options' => 'portfolio_types',
  			),

				array(
  				'id' => 'query_terms_excludes_category',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'category' ),
  				're_render' => true,
  				'type' => 'multiselect',
  				'class' => 'mfn-hidden-nested-wrap',
  				'title' => __('Excludes', 'mfn-opts'),
  				'desc' => __('IDs of taxonomies to exclude', 'mfn-opts'),
  				//'hierarchical_options' => mfn_get_categories('category'),
					'js_hierarchical_options' => 'blog_categories',
  			),

  			array(
  				'id' => 'query_terms_excludes_offer_types',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'offer_types' ),
  				're_render' => true,
  				'type' => 'multiselect',
  				'class' => 'mfn-hidden-nested-wrap',
  				'title' => __('Excludes', 'mfn-opts'),
  				'desc' => __('IDs of taxonomies to exclude', 'mfn-opts'),
  				//'options' => mfn_get_categories('offer-types'),
   				'js_hierarchical_options' => 'offer_types',
  			),

  			array(
  				'id' => 'query_terms_excludes_testimonial_types',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'testimonial_types' ),
  				're_render' => true,
  				'type' => 'multiselect',
  				'class' => 'mfn-hidden-nested-wrap',
  				'title' => __('Excludes', 'mfn-opts'),
  				'desc' => __('IDs of taxonomies to exclude', 'mfn-opts'),
  				//'options' => mfn_get_categories('testimonial-types'),
   				'js_hierarchical_options' => 'testimonial_types',
  			),

  			array(
  				'id' => 'query_terms_excludes_product_cat',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'product_cat' ),
  				're_render' => true,
  				'type' => 'multiselect',
  				'class' => 'mfn-hidden-nested-wrap',
  				'title' => __('Excludes', 'mfn-opts'),
  				'desc' => __('IDs of taxonomies to exclude', 'mfn-opts'),
  				//'hierarchical_options' => mfn_hierarchical_taxonomy('product_cat'),
					'js_hierarchical_options' => 'product_cat',
  			),

  			array(
  				'id' => 'query_terms_includes_post_tag',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'post_tag' ),
  				're_render' => true,
  				'type' => 'multiselect',
  				'class' => 'mfn-hidden-nested-wrap',
  				'title' => __('Includes', 'mfn-opts'),
  				'desc' => __('IDs of taxonomies to include', 'mfn-opts'),
  				//'options' => mfn_get_categories('post_tag'),
   				'js_hierarchical_options' => 'blog_tags',
  			),

  			array(
  				'id' => 'query_terms_includes_client_types',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'client_types' ),
  				're_render' => true,
  				'type' => 'multiselect',
  				'class' => 'mfn-hidden-nested-wrap',
  				'title' => __('Includes', 'mfn-opts'),
  				'desc' => __('IDs of taxonomies to include', 'mfn-opts'),
  				//'options' => mfn_get_categories('client-types'),
   				'js_hierarchical_options' => 'client_types',
  			),

  			array(
  				'id' => 'query_terms_includes_portfolio_types',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'portfolio_types' ),
  				're_render' => true,
  				'type' => 'multiselect',
  				'class' => 'mfn-hidden-nested-wrap',
  				'title' => __('Includes', 'mfn-opts'),
  				'desc' => __('IDs of taxonomies to include', 'mfn-opts'),
  				//'options' => mfn_get_categories('portfolio-types'),
   				'js_hierarchical_options' => 'portfolio_types',
  			),

				array(
  				'id' => 'query_terms_includes_category',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'category' ),
  				're_render' => true,
  				'type' => 'multiselect',
  				'class' => 'mfn-hidden-nested-wrap',
  				'title' => __('Includes', 'mfn-opts'),
  				'desc' => __('IDs of taxonomies to include', 'mfn-opts'),
  				//'hierarchical_options' => mfn_get_categories('category'),
					'js_hierarchical_options' => 'blog_categories',
  			),

  			array(
  				'id' => 'query_terms_includes_offer_types',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'offer_types' ),
  				're_render' => true,
  				'type' => 'multiselect',
  				'class' => 'mfn-hidden-nested-wrap',
  				'title' => __('Includes', 'mfn-opts'),
  				'desc' => __('IDs of taxonomies to include', 'mfn-opts'),
  				//'options' => mfn_get_categories('offer-types'),
   				'js_hierarchical_options' => 'offer_types',
  			),

  			array(
  				'id' => 'query_terms_includes_testimonial_types',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'testimonial_types' ),
  				're_render' => true,
  				'type' => 'multiselect',
  				'class' => 'mfn-hidden-nested-wrap',
  				'title' => __('Includes', 'mfn-opts'),
  				'desc' => __('IDs of taxonomies to include', 'mfn-opts'),
  				//'options' => mfn_get_categories('testimonial-types'),
   				'js_hierarchical_options' => 'testimonial_types',
  			),

  			array(
  				'id' => 'query_terms_includes_product_cat',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'product_cat' ),
  				're_render' => true,
  				'type' => 'multiselect',
  				'class' => 'mfn-hidden-nested-wrap',
  				'title' => __('Includes', 'mfn-opts'),
  				'desc' => __('IDs of taxonomies to include', 'mfn-opts'),
  				//'hierarchical_options' => mfn_hierarchical_taxonomy('product_cat'),
					'js_hierarchical_options' => 'product_cat',
					'opt_append' => array(
						'0-current' => 'Current'
					),
  			),

  			array(
  				'id' => 'query_terms_child_of_product_cat',
  				'condition' => array( 'id' => 'query_terms_taxonomy', 'opt' => 'is', 'val' => 'product_cat' ),
  				're_render' => true,
  				'type' => 'select',
  				'class' => 'mfn-hidden-nested-wrap',
  				'title' => __('Child of', 'mfn-opts'),
					'js_hierarchical_options' => 'product_cat',
					'opt_append' => array(
						'0-current' => 'Current'
					),
  			),

  			array(
  				'id' => 'query_terms_orderby',
  				're_render' => true,
  				'class' => 'hidden-header-builder mfn-hidden-nested-wrap',
  				'condition' => array( 'id' => 'query_type', 'opt' => 'is', 'val' => 'terms' ),
  				'type' => 'select',
  				'title' => __('Order by', 'mfn-opts'),
  				'options' => array(
  					'' => __('None', 'mfn-opts'),
  					'term_order' => __('Term order', 'mfn-opts'),
  					'term_id' => __('ID', 'mfn-opts'),
  					'name' => __('Name', 'mfn-opts'),
  					'count' => __('Count', 'mfn-opts'),
  				),
  				'std' => 'ID'
  			),

  			array(
  				'id' => 'query_terms_order',
  				're_render' => true,
  				'class' => 'hidden-header-builder mfn-hidden-nested-wrap',
  				'condition' => array( 'id' => 'query_type', 'opt' => 'is', 'val' => 'terms' ),
  				'type' => 'select',
  				'title' => __('Order', 'mfn-opts'),
  				'options' => array(
  					'ASC' => __('Ascending', 'mfn-opts'),
  					'DESC' => __('Descending', 'mfn-opts'),
  				),
  				'std' => 'ASC'
  			),

  			array(
  				'id' => 'query_terms_hide_empty',
  				'class' => 'hidden-header-builder mfn-hidden-nested-wrap',
  				're_render' => true,
  				'condition' => array( 'id' => 'query_type', 'opt' => 'is', 'val' => 'terms' ),
  				'type' => 'select',
  				'title' => __('Hide empty', 'mfn-opts'),
  				'options' => array(
  					'' => __('False', 'mfn-opts'),
  					'1' => __('True', 'mfn-opts'),
  				),
  				'std' => ''
  			),

  			array(
  				'id' => 'query_terms_number',
  				're_render' => true,
  				'class' => 'hidden-header-builder mfn-hidden-nested-wrap',
  				'condition' => array( 'id' => 'query_type', 'opt' => 'is', 'val' => 'terms' ),
  				'type' => 'text',
  				'title' => __('Limit', 'mfn-opts'),
  				'param' => 'number',
					'preview' => 'number',
					'std' => '0',
  			),





				// deprecated

				array(
					'type' => 'header',
					'class' => 'hidden-header-builder mfn-deprecated',
					'title' => __('Deprecated', 'mfn-opts'),
				),

				array(
  				'id' => 'bg_color',
  				'class' => 'mfn-deprecated',
  				'type' => 'color',
  				'title' => __('Background color', 'mfn-opts'),
  				'alpha' => true,
  			),

  			array(
  				'id' => 'bg_image',
  				'class' => 'mfn-deprecated',
  				'type' => 'upload',
  				'title' => __('Background image', 'mfn-opts'),
  				'desc' => __('Recommended image width <b>320px - 1920px</b> depending on size of the wrap', 'mfn-opts'),
  			),

  			array(
  				'id' => 'bg_position',
  				'class' => 'mfn-deprecated',
  				'type' => 'select',
  				'title' => __('Background position', 'mfn-opts'),
					'desc' => __('iOS does <b>not</b> support background-position: fixed<br/>For parallax required background image size is at least 1920px x 1080px', 'mfn-opts'),
  				'options' => mfna_bg_position(),
  			),

  			array(
  				'id' => 'bg_size',
  				'class' => 'mfn-deprecated',
  				'type' => 'select',
  				'title' => __('Background size', 'mfn-opts'),
					'desc' => __('Does <b>not</b> work with position fixed or parallax', 'mfn-opts'),
  				'options' => mfna_bg_size(),
  			),

  			array(
  				'id' => 'move_up',
  				'class' => 'mfn-deprecated',
  				'type' => 'text',
  				'title' => __('Move up', 'mfn-opts'),
  				'desc' => __('Move this wrap to overflow on previous section. Does <b>not</b> work with parallax', 'mfn-opts'),
  				'after' => 'px',
  			),

  			array(
  				'id' => 'padding',
  				'class' => 'mfn-deprecated',
  				'type' => 'text',
  				'title' => __('Padding', 'mfn-opts'),
  				'desc' => __('Use value with <b>px</b> or <b>%</b><br />Example: 20px or 20px 10px 20px 10px or 20px 1%', 'mfn-opts'),
  			),

  			array(
  				'id' => 'column_margin',
  				'class' => 'mfn-deprecated',
  				'type' => 'select',
  				'title' => __('Elements margin bottom', 'mfn-opts'),
  				'options' => array(
  					'' => __('- Default -', 'mfn-opts'),
  					'0px' => '0px',
  					'10px' => '10px',
  					'20px' => '20px',
  					'30px' => '30px',
  					'40px' => '40px',
  					'50px' => '50px',
  				),
  			),

				array(
  				'id' => 'vertical_align',
  				'class' => 'mfn-deprecated',
  				'type' => 'select',
  				'title' => __('Elements vertical align', 'mfn-opts'),
  				'desc' => __('for Section style: <b>Equal height of wraps</b>', 'mfn-opts'),
  				'options' => array(
  					'' => __('Default', 'mfn-opts'),
  					'top' => __('Top', 'mfn-opts'),
  					'middle' => __('Middle', 'mfn-opts'),
  					'bottom' => __('Bottom', 'mfn-opts'),
  				),
  			),

				array(
					'id' => 'class',
					'type' => 'pills',
					'title' => __('Wrap classes', 'mfn-opts'),
					'class' => 'mfn-deprecated',
				),

				array(
					'id' => 'style',
					'type' => 'textarea',
					'title' => __('Wrap inline CSS', 'mfn-opts'),
					'desc' => __('Example: <b>border: 1px solid #999;</b>', 'mfn-opts'),
					'class' => 'mfn-deprecated',
				),

				array(
					'type' => 'helper',
					'class' => 'mfn-deprecated',
					'title' => __('Need help', 'mfn-opts'),
					'link' => 'https://support.muffingroup.com/faq/what-deprecated-in-the-bebuilder-means/',
				),

				array(
					'type' => 'html',
					'html' => '</div>',
				),















				array(
					'type' => 'html',
					'html' => '<div class="modalbox-card modalbox-card-style">',
				),


				array(
					'type' => 'header',
					'class' => 'toggled_header row-header',
					'title' => __('Container', 'mfn-opts'),
				),

				array(
  				'id' => 'query_display',
  				'attr_id' => 'query_display',
  				're_render' => true,
  				'class' => 'mfn-toggled',
  				'type' => 'switch',
  				'title' => __('Display', 'mfn-opts'),
  				'options' => array(
  					'' => __('Default', 'mfn-opts'),
  					'slider' => __('Slider', 'mfn-opts'),
  				),
  				'std' => ''
  			),

  			array(
  				'id' => 'query_slider_columns',
  				'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
  				'class' => 'narrow mfn-toggled',
  				'responsive' => 'desktop',
  				're_render' => true,
  				'type' => 'text',
  				'title' => __('Columns', 'mfn-opts'),
  				'param' => 'number',
					'preview' => 'number',
					'std' => '1',
  			),

  			array(
  				'id' => 'query_slider_columns_laptop',
  				'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
  				'class' => 'narrow mfn-toggled',
  				'responsive' => 'laptop',
  				'type' => 'text',
  				're_render' => true,
  				'title' => __('Columns', 'mfn-opts'),
  				'param' => 'number',
					'preview' => 'number',
					'std' => '1',
  			),

  			array(
  				'id' => 'query_slider_columns_tablet',
  				'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
  				'class' => 'narrow mfn-toggled',
  				'responsive' => 'tablet',
  				'type' => 'text',
  				're_render' => true,
  				'title' => __('Columns', 'mfn-opts'),
  				'param' => 'number',
					'preview' => 'number',
					'std' => '1',
  			),

  			array(
  				'id' => 'query_slider_columns_mobile',
  				'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
  				'class' => 'narrow mfn-toggled',
  				'responsive' => 'mobile',
  				'type' => 'text',
  				're_render' => true,
  				'title' => __('Columns', 'mfn-opts'),
  				'param' => 'number',
					'preview' => 'number',
					'std' => '1',
  			),

  			/*array(
  				'id' => 'query_slider_animation',
  				'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
  				'class' => 'mfn-toggled',
  				'type' => 'switch',
  				'title' => __('Animation', 'mfn-opts'),
  				'options' => array(
  					'' => __('Slide', 'mfn-opts'),
  					'fade' => __('Fade', 'mfn-opts'),
  				),
  				'std' => ''
  			),*/

  			array(
  				'id' => 'query_slider_infinity',
  				'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
  				'class' => 'mfn-toggled',
  				'type' => 'switch',
  				'title' => __('Infinite loop', 'mfn-opts'),
  				'options' => array(
  					'' => __('Off', 'mfn-opts'),
  					'1' => __('On', 'mfn-opts'),
  				),
  				'std' => ''
  			),

  			array(
  				'id' => 'query_slider_autoplay',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'default_unit' => '',
					'class' => 'mfn-toggled',
					'title' => __('Autoplay speed', 'mfn-opts'),
					'desc' => __('Leave empty to disallow autoplay', 'mfn-opts'),
					'param' => array(
						'min' => '0',
						'max' => '10000',
						'step' => '100',
						'unit' => '',
					)
  			),

  			array(
  				'id' => 'query_slider_arrows',
  				'attr_id' => 'query_slider_arrows',
  				'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
  				'class' => 'mfn-toggled',
  				'type' => 'switch',
  				'title' => __('Arrows', 'mfn-opts'),
  				'options' => array(
  					'0' => __('Hide', 'mfn-opts'),
  					'1' => __('Show', 'mfn-opts'),
  				),
  				'std' => '0'
  			),

  			array(
  				'id' => 'query_slider_arrows_style',
  				'attr_id' => 'query_slider_arrows_style',
  				'condition' => array( 'id' => 'query_slider_arrows', 'opt' => 'is', 'val' => '1' ),
  				'class' => 'mfn-toggled',
  				'type' => 'switch',
  				'title' => __('Arrows style', 'mfn-opts'),
  				'options' => array(
  					'' => __('Standard', 'mfn-opts'),
  					'overlay' => __('Overlay', 'mfn-opts'),
  					'custom' => __('Custom', 'mfn-opts'),
  				),
  				'std' => ''
  			),

  			array(
  				'id' => 'query_slider_dots',
  				'attr_id' => 'query_slider_dots',
  				'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
  				'class' => 'mfn-toggled',
  				'type' => 'switch',
  				'title' => __('Dots', 'mfn-opts'),
  				'options' => array(
  					'0' => __('Hide', 'mfn-opts'),
  					'1' => __('Show', 'mfn-opts'),
  				),
  				'std' => '0'
  			),

  			array(
  				'id' => 'query_slider_dots_style',
  				'attr_id' => 'query_slider_dots_style',
  				'condition' => array( 'id' => 'query_slider_dots', 'opt' => 'is', 'val' => '1' ),
  				'class' => 'mfn-toggled',
  				'type' => 'switch',
  				'title' => __('Dots style', 'mfn-opts'),
  				'options' => array(
  					'' => __('Standard', 'mfn-opts'),
  					'overlay' => __('Overlay', 'mfn-opts'),
  					'custom' => __('Custom', 'mfn-opts'),
  				),
  				'std' => ''
  			),

  			array(
  				'id' => 'query_slider_mousewheel',
  				'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
  				'class' => 'mfn-toggled',
  				'type' => 'switch',
  				'title' => __('Mousewheel', 'mfn-opts'),
  				'options' => array(
  					'' => __('Disabled', 'mfn-opts'),
  					'1' => __('Enabled', 'mfn-opts'),
  				),
  				'std' => ''
  			),

  			array(
  				'id' => 'query_display_style',
  				'condition' => array( 'id' => 'query_display', 'opt' => 'isnt', 'val' => 'slider' ),
  				'class' => 'mfn-toggled',
  				'type' => 'switch',
  				'title' => __('Style', 'mfn-opts'),
  				'options' => array(
  					'' => __('Default', 'mfn-opts'),
  					'masonry' => __('Masonry', 'mfn-opts'),
  				),
  				'std' => ''
  			),

  			array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper.mfn-ql-item-default:width',
					'condition' => array( 'id' => 'query_display', 'opt' => 'isnt', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'default_unit' => 'px',
					'responsive' => 'desktop',
					'class' => 'mfn-toggled query_item_default_width sliderbar-units',
					'title' => __('Item width', 'mfn-opts'),
					'units' => array(
						'px' 	=> array('min' => 0, 'max' => 1920, 'step' => 1),
						'%' 	=> array('min' => 0, 'max' => 100, 'step' => 0.1),
						'vw' 	=> array('min' => 0, 'max' => 100, 'step' => 0.1),
					),
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper.mfn-ql-item-default:width_laptop',
					'condition' => array( 'id' => 'query_display', 'opt' => 'isnt', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'default_unit' => 'px',
					'responsive' => 'laptop',
					'class' => 'mfn-toggled query_item_default_width sliderbar-units',
					'title' => __('Item width', 'mfn-opts'),
					'units' => array(
						'px' 	=> array('min' => 0, 'max' => 1920, 'step' => 1),
						'%' 	=> array('min' => 0, 'max' => 100, 'step' => 1),
						'vw' 	=> array('min' => 0, 'max' => 100, 'step' => 1),
					),
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper.mfn-ql-item-default:width_tablet',
					'condition' => array( 'id' => 'query_display', 'opt' => 'isnt', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'default_unit' => 'px',
					'responsive' => 'tablet',
					'class' => 'mfn-toggled query_item_default_width sliderbar-units',
					'title' => __('Item width', 'mfn-opts'),
					'units' => array(
						'px' 	=> array('min' => 0, 'max' => 1920, 'step' => 1),
						'%' 	=> array('min' => 0, 'max' => 100, 'step' => 1),
						'vw' 	=> array('min' => 0, 'max' => 100, 'step' => 1),
					),
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper.mfn-ql-item-default:width_mobile',
					'condition' => array( 'id' => 'query_display', 'opt' => 'isnt', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'default_unit' => 'px',
					'responsive' => 'mobile',
					'class' => 'mfn-toggled query_item_default_width sliderbar-units',
					'title' => __('Item width', 'mfn-opts'),
					'units' => array(
						'px' 	=> array('min' => 0, 'max' => 1920, 'step' => 1),
						'%' 	=> array('min' => 0, 'max' => 100, 'step' => 0.1),
						'vw' 	=> array('min' => 0, 'max' => 100, 'step' => 0.1),
					),
				),

  			array(
  				'type' => 'header',
  				'class' => 'toggled_header row-header',
					'title' => __('Item', 'mfn-opts'),
				),


				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:min-height',
					'type' => 'text',
					'default_unit' => 'px',
					'title' => __('Min height', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:min-height_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'title' => __('Min height', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:min-height_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'title' => __('Min height', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:min-height_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'title' => __('Min height', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:align-content',
					'type' => 'switch',
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'title' => __('Vertical align', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'flex-start' => __('Top', 'mfn-opts'),
						'center' => __('Center', 'mfn-opts'),
						'flex-end' => __('Bottom', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:align-content_laptop',
					'type' => 'switch',
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'title' => __('Vertical align', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'flex-start' => __('Top', 'mfn-opts'),
						'center' => __('Center', 'mfn-opts'),
						'flex-end' => __('Bottom', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:align-content_tablet',
					'type' => 'switch',
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'title' => __('Vertical align', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'flex-start' => __('Top', 'mfn-opts'),
						'center' => __('Center', 'mfn-opts'),
						'flex-end' => __('Bottom', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:align-content_mobile',
					'type' => 'switch',
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'title' => __('Vertical align', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'flex-start' => __('Top', 'mfn-opts'),
						'center' => __('Center', 'mfn-opts'),
						'flex-end' => __('Bottom', 'mfn-opts'),
					],
				),




				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:justify-content',
					'type' => 'select',
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'title' => __('Horizontal align', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'flex-start' => __('Left', 'mfn-opts'),
						'center' => __('Center', 'mfn-opts'),
						'flex-end' => __('Right', 'mfn-opts'),
						'space-around' => __('Space around', 'mfn-opts'),
						'space-between' => __('Space between', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:justify-content_laptop',
					'type' => 'select',
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'title' => __('Horizontal align', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'flex-start' => __('Left', 'mfn-opts'),
						'center' => __('Center', 'mfn-opts'),
						'flex-end' => __('Right', 'mfn-opts'),
						'space-around' => __('Space around', 'mfn-opts'),
						'space-between' => __('Space between', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:justify-content_tablet',
					'type' => 'select',
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'title' => __('Horizontal align', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'flex-start' => __('Left', 'mfn-opts'),
						'center' => __('Center', 'mfn-opts'),
						'flex-end' => __('Right', 'mfn-opts'),
						'space-around' => __('Space around', 'mfn-opts'),
						'space-between' => __('Space between', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:justify-content_mobile',
					'type' => 'select',
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'title' => __('Horizontal align', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'flex-start' => __('Left', 'mfn-opts'),
						'center' => __('Center', 'mfn-opts'),
						'flex-end' => __('Right', 'mfn-opts'),
						'space-around' => __('Space around', 'mfn-opts'),
						'space-between' => __('Space between', 'mfn-opts'),
					],
				),





				array(
					'type' => 'subheader',
					'class' => 'mfn-builder-subheader row-header mfn-toggled',
					'title' => __('Spacing', 'mfn-opts'),
				),

				array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:padding',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Padding', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'padding',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:padding_laptop',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Padding', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'css_attr' => 'padding',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:padding_tablet',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Padding', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'padding',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:padding_mobile',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Padding', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'padding',
  			),

				array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:margin',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Margin', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'margin',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:margin_laptop',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Margin', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'css_attr' => 'margin',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:margin_tablet',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Margin', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'margin',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:margin_mobile',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Margin', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'margin',
  			),









  			array(
  				'type' => 'subheader',
					'class' => 'mfn-builder-subheader row-header mfn-toggled',
					'title' => __('Border', 'mfn-opts'),
				),

  			array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:border-style',
					'attr_id' => 'border_style_ql',
					'type' => 'select',
					'class' => 'mfn-toggled',
					'title' => __('Border style', 'mfn-opts'),
					'options' => [
						'none' => __('None', 'mfn-opts'),
						'solid' => __('Solid', 'mfn-opts'),
						'dashed' => __('Dashed', 'mfn-opts'),
						'dotted' => __('Dotted', 'mfn-opts'),
						'double' => __('Double', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:border-color',
					'condition' => array( 'id' => 'border_style_ql', 'opt' => 'isnt', 'val' => 'none' ),
					'type' => 'color',
					'title' => __('Border color', 'mfn-opts'),
				),

				array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:border-width',
  				'condition' => array( 'id' => 'border_style_ql', 'opt' => 'isnt', 'val' => 'none' ),
  				'type' => 'dimensions',
  				'title' => __('Border width', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-width',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:border-width_laptop',
  				'condition' => array( 'id' => 'border_style_ql', 'opt' => 'isnt', 'val' => 'none' ),
  				'type' => 'dimensions',
  				'title' => __('Border width', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-width',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:border-width_tablet',
  				'condition' => array( 'id' => 'border_style_ql', 'opt' => 'isnt', 'val' => 'none' ),
  				'type' => 'dimensions',
  				'title' => __('Border width', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-width',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:border-width_mobile',
  				'condition' => array( 'id' => 'border_style_ql', 'opt' => 'isnt', 'val' => 'none' ),
  				'type' => 'dimensions',
  				'title' => __('Border width', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-width',
  			),

				array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:border-radius',
  				'type' => 'dimensions',
  				'title' => __('Border radius', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-radius',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:border-radius_laptop',
  				'type' => 'dimensions',
  				'title' => __('Border radius', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-radius',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:border-radius_tablet',
  				'type' => 'dimensions',
  				'title' => __('Border radius', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-radius',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:border-radius_mobile',
  				'type' => 'dimensions',
  				'title' => __('Border radius', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-radius',
  			),

  			array(
		  		'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:box-shadow',
  				'type' => 'box_shadow',
  				'title' => __('Box shadow', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'box-shadow'
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:box-shadow_tablet',
  				'type' => 'box_shadow',
  				'title' => __('Box shadow', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'box-shadow',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:box-shadow_mobile',
  				'type' => 'box_shadow',
  				'title' => __('Box shadow', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'box-shadow',
  			),

  			array(
  				'type' => 'subheader',
					'class' => 'mfn-builder-subheader row-header mfn-toggled',
					'title' => __('Background', 'mfn-opts'),
				),

  			array(
					'id' => 'background_switcher_ql',
					'attr_id' => 'background_switcher_ql',
					'type' => 'switch',
					'class' => 'mfn-fields-switcher mfn-toggled disable-history',
					'title' => __('Background type', 'mfn-opts'),
					'options' => array(
  					'default' => __('Default', 'mfn-opts'),
  					'gradient' 	=> __('Gradient', 'mfn-opts'),
  				),
					'visual_options' => array(
  					'default' => '',
  					'gradient' 	=> '',
  				),
  				'std' => 'default',
				),

				array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:gradient',
  				'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'gradient' ),
  				'type' => 'gradient',
  				'title' => __('Gradient', 'mfn-opts'),
  				'class' => 'mfn-toggled',
					'css_attr' => 'gradient',
  			),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:background-color',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'color',
					'class' => 'mfn-toggled',
					'title' => __('Background color', 'mfn-opts'),
				),

				array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:background-image',
  				'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
  				'oldid' => 'bg_color',
  				'class' => 'mfn-toggled',
  				'dynamic_data' => 'featured_image',
  				'type' => 'upload',
  				'responsive' => 'desktop',
  				'title' => __('Image', 'mfn-opts'),
					'css_attr' => 'background-image',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:background-image_laptop',
  				'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
  				'oldid' => 'bg_color',
  				'class' => 'mfn-toggled',
  				'dynamic_data' => 'featured_image',
  				'type' => 'upload',
  				'responsive' => 'laptop',
  				'title' => __('Image', 'mfn-opts'),
					'css_attr' => 'background-image',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:background-image_tablet',
  				'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
  				'oldid' => 'bg_color',
  				'class' => 'mfn-toggled',
  				'dynamic_data' => 'featured_image',
  				'type' => 'upload',
  				'responsive' => 'tablet',
  				'title' => __('Image', 'mfn-opts'),
					'css_attr' => 'background-image',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:background-image_mobile',
  				'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
  				'oldid' => 'bg_color',
  				'class' => 'mfn-toggled',
  				'dynamic_data' => 'featured_image',
  				'type' => 'upload',
  				'responsive' => 'mobile',
  				'title' => __('Image', 'mfn-opts'),
					'css_attr' => 'background-image',
  			),

  			array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:background-repeat',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'class' => 'mfn-toggled',
					'responsive' => 'desktop',
					'css_attr' => 'background-repeat',
					'title' => __('Repeat', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'repeat' => __('Repeat', 'mfn-opts'),
						'no-repeat' => __('No repeat', 'mfn-opts'),
						'repeat-x' => __('Repeat X', 'mfn-opts'),
						'repeat-y' => __('Repeat Y', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:background-repeat_laptop',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'class' => 'mfn-toggled',
					'responsive' => 'laptop',
					'css_attr' => 'background-repeat',
					'title' => __('Repeat', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'repeat' => __('Repeat', 'mfn-opts'),
						'no-repeat' => __('No repeat', 'mfn-opts'),
						'repeat-x' => __('Repeat X', 'mfn-opts'),
						'repeat-y' => __('Repeat Y', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:background-repeat_tablet',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'class' => 'mfn-toggled',
					'responsive' => 'tablet',
					'css_attr' => 'background-repeat',
					'title' => __('Repeat', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'repeat' => __('Repeat', 'mfn-opts'),
						'no-repeat' => __('No repeat', 'mfn-opts'),
						'repeat-x' => __('Repeat X', 'mfn-opts'),
						'repeat-y' => __('Repeat Y', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:background-repeat_mobile',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'class' => 'mfn-toggled',
					'responsive' => 'mobile',
					'css_attr' => 'background-repeat',
					'title' => __('Repeat', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'repeat' => __('Repeat', 'mfn-opts'),
						'no-repeat' => __('No repeat', 'mfn-opts'),
						'repeat-x' => __('Repeat X', 'mfn-opts'),
						'repeat-y' => __('Repeat Y', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:background-position',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_position',
					'type' => 'select',
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'background-position',
					'title' => __('Position', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'center' => __('Center Center', 'mfn-opts'),
						'center left' => __('Center Left', 'mfn-opts'),
						'center right' => __('Center Right', 'mfn-opts'),
						'top left' => __('Top Left', 'mfn-opts'),
						'top right' => __('Top Right', 'mfn-opts'),
						'top center' => __('Top Center', 'mfn-opts'),
						'bottom left' => __('Bottom Left', 'mfn-opts'),
						'bottom right' => __('Bottom Right', 'mfn-opts'),
						'bottom center' => __('Bottom Center', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:background-position_laptop',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_position',
					'type' => 'select',
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'css_attr' => 'background-position',
					'title' => __('Position', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'center' => __('Center Center', 'mfn-opts'),
						'center left' => __('Center Left', 'mfn-opts'),
						'center right' => __('Center Right', 'mfn-opts'),
						'top left' => __('Top Left', 'mfn-opts'),
						'top right' => __('Top Right', 'mfn-opts'),
						'top center' => __('Top Center', 'mfn-opts'),
						'bottom left' => __('Bottom Left', 'mfn-opts'),
						'bottom right' => __('Bottom Right', 'mfn-opts'),
						'bottom center' => __('Bottom Center', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:background-position_tablet',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_position',
					'type' => 'select',
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'background-position',
					'title' => __('Position', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'center' => __('Center Center', 'mfn-opts'),
						'center left' => __('Center Left', 'mfn-opts'),
						'center right' => __('Center Right', 'mfn-opts'),
						'top left' => __('Top Left', 'mfn-opts'),
						'top right' => __('Top Right', 'mfn-opts'),
						'top center' => __('Top Center', 'mfn-opts'),
						'bottom left' => __('Bottom Left', 'mfn-opts'),
						'bottom right' => __('Bottom Right', 'mfn-opts'),
						'bottom center' => __('Bottom Center', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:background-position_mobile',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_position',
					'type' => 'select',
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'background-position',
					'title' => __('Position', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'center' => __('Center Center', 'mfn-opts'),
						'center left' => __('Center Left', 'mfn-opts'),
						'center right' => __('Center Right', 'mfn-opts'),
						'top left' => __('Top Left', 'mfn-opts'),
						'top right' => __('Top Right', 'mfn-opts'),
						'top center' => __('Top Center', 'mfn-opts'),
						'bottom left' => __('Bottom Left', 'mfn-opts'),
						'bottom right' => __('Bottom Right', 'mfn-opts'),
						'bottom center' => __('Bottom Center', 'mfn-opts'),
					],
				),


				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:background-attachment',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'class' => 'mfn-toggled',
					'responsive' => 'desktop',
					'css_attr' => 'background-attachment',
					'title' => __('Attachment', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'scroll' => __('Scroll', 'mfn-opts'),
						'fixed' => __('Fixed', 'mfn-opts'),
						'parallax' => __('Parallax', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:background-attachment_laptop',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'class' => 'mfn-toggled',
					'responsive' => 'laptop',
					'css_attr' => 'background-attachment',
					'title' => __('Attachment', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'scroll' => __('Scroll', 'mfn-opts'),
						'fixed' => __('Fixed', 'mfn-opts'),
						'parallax' => __('Parallax', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:background-attachment_tablet',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'class' => 'mfn-toggled',
					'responsive' => 'tablet',
					'css_attr' => 'background-attachment',
					'title' => __('Attachment', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'scroll' => __('Scroll', 'mfn-opts'),
						'fixed' => __('Fixed', 'mfn-opts'),
						'parallax' => __('Parallax', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:background-attachment_mobile',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'class' => 'mfn-toggled',
					'responsive' => 'mobile',
					'css_attr' => 'background-attachment',
					'title' => __('Attachment', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'scroll' => __('Scroll', 'mfn-opts'),
						'fixed' => __('Fixed', 'mfn-opts'),
						'parallax' => __('Parallax', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:background-size',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_size',
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'type' => 'select',
					'css_attr' => 'background-size',
					'title' => __('Size', 'mfn-opts'),
					'options' => mfna_bg_size()
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:background-size_laptop',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_size',
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'type' => 'select',
					'css_attr' => 'background-size',
					'title' => __('Size', 'mfn-opts'),
					'options' => mfna_bg_size()
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:background-size_tablet',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_size',
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'type' => 'select',
					'css_attr' => 'background-size',
					'title' => __('Size', 'mfn-opts'),
					'options' => mfna_bg_size()
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner .mfn-queryloop-item-wrapper:background-size_mobile',
					'condition' => array( 'id' => 'background_switcher_ql', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_size',
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'type' => 'select',
					'css_attr' => 'background-size',
					'title' => __('Size', 'mfn-opts'),
					'options' => mfna_bg_size()
				),

				array(
					'type' => 'header',
					'class' => 'toggled_header row-header',
					'title' => __('Pagination', 'mfn-opts'),
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .swiper-pagination:top',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Top', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled dots-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .swiper-pagination:bottom',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Bottom', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled dots-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .swiper-pagination:left',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Left', 'mfn-opts'),
						'responsive' => 'desktop',
					'class' => 'mfn-toggled dots-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .swiper-pagination:right',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Right', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled dots-custom-style',
				),




				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .swiper-pagination:top_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Top', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled dots-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .swiper-pagination:bottom_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Bottom', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled dots-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .swiper-pagination:left_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Left', 'mfn-opts'),
						'responsive' => 'tablet',
					'class' => 'mfn-toggled dots-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .swiper-pagination:right_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Right', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled dots-custom-style',
				),



				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .swiper-pagination:top_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Top', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled dots-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .swiper-pagination:bottom_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Bottom', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled dots-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .swiper-pagination:left_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Left', 'mfn-opts'),
						'responsive' => 'laptop',
					'class' => 'mfn-toggled dots-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .swiper-pagination:right_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Right', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled dots-custom-style',
				),


				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .swiper-pagination:top_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Top', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled dots-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .swiper-pagination:bottom_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Bottom', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled dots-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .swiper-pagination:left_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Left', 'mfn-opts'),
						'responsive' => 'mobile',
					'class' => 'mfn-toggled dots-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-dots-custom .swiper-pagination:right_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_dots_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Right', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled dots-custom-style',
				),



				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .swiper-pagination:--mfn-swiper-pagination-spacing',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'title' => __('Offset', 'mfn-opts'),
					'desc' => __('Works with Standard and Overlay pagination mode', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .swiper-pagination:--mfn-swiper-pagination-spacing_laptop',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'title' => __('Offset', 'mfn-opts'),
					'desc' => __('Works with Standard and Overlay pagination mode', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .swiper-pagination:--mfn-swiper-pagination-spacing_tablet',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'title' => __('Offset', 'mfn-opts'),
					'desc' => __('Works with Standard and Overlay pagination mode', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .swiper-pagination:--mfn-swiper-pagination-spacing_mobile',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'title' => __('Offset', 'mfn-opts'),
					'desc' => __('Works with Standard and Overlay pagination mode', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .swiper-pagination .swiper-pagination-bullet:--swiper-pagination-bullet-inactive-color',
					'type' => 'color',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'class' => 'mfn-toggled',
					'title' => __('Color', 'mfn-opts'),
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .swiper-pagination .swiper-pagination-bullet-active:--mfn-swiper-pagination-bullet-active-color',
					'type' => 'color',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'class' => 'mfn-toggled',
					'title' => __('Active color', 'mfn-opts'),
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .swiper-pagination-bullet:--mfn-swiper-pagination-bullet-width',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'title' => __('Width', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '100',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .swiper-pagination-bullet:--mfn-swiper-pagination-bullet-width_laptop',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'title' => __('Width', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '100',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .swiper-pagination-bullet:--mfn-swiper-pagination-bullet-width_tablet',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'title' => __('Width', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '100',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .swiper-pagination-bullet:--mfn-swiper-pagination-bullet-width_mobile',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'title' => __('Width', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '100',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .swiper-pagination-bullet:--mfn-swiper-pagination-bullet-height',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'title' => __('Height', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '100',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .swiper-pagination-bullet:--mfn-swiper-pagination-bullet-height_laptop',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'title' => __('Height', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '100',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .swiper-pagination-bullet:--mfn-swiper-pagination-bullet-height_tablet',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'title' => __('Height', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '100',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .swiper-pagination-bullet:--mfn-swiper-pagination-bullet-height_mobile',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'title' => __('Height', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '100',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .swiper-pagination-bullet:border-radius',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'class' => 'mfn-toggled',
					'title' => __('Border radius', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '100',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .swiper-pagination-bullet:opacity',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'class' => 'mfn-toggled',
					'title' => __('Opacity', 'mfn-opts'),
					'param' => array(
						'min' => '0',
						'max' => '1',
						'step' => '0.01',
					),
					'std' => '0',
				),








				array(
					'type' => 'header',
					'class' => 'toggled_header row-header',
					'title' => __('Arrows', 'mfn-opts'),
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
				),

				array(
					'type' => 'subheader',
  				'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'class' => 'mfn-builder-subheader row-header mfn-toggled',
					'title' => __('Prev arrow custom style', 'mfn-opts'),
				),

  			array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-prev.mfn-swiper-arrow:top',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Top', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-prev.mfn-swiper-arrow:bottom',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Bottom', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-prev.mfn-swiper-arrow:left',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Left', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-prev.mfn-swiper-arrow:right',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Right', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled arrows-custom-style',
				),


				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-prev.mfn-swiper-arrow:top_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Top', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-prev.mfn-swiper-arrow:bottom_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Bottom', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-prev.mfn-swiper-arrow:left_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Left', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-prev.mfn-swiper-arrow:right_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Right', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled arrows-custom-style',
				),


				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-prev.mfn-swiper-arrow:top_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Top', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-prev.mfn-swiper-arrow:bottom_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Bottom', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-prev.mfn-swiper-arrow:left_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Left', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-prev.mfn-swiper-arrow:right_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Right', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled arrows-custom-style',
				),



				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-prev.mfn-swiper-arrow:top_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Top', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-prev.mfn-swiper-arrow:bottom_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Bottom', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-prev.mfn-swiper-arrow:left_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Left', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-prev.mfn-swiper-arrow:right_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Right', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled arrows-custom-style',
				),



				array(
					'type' => 'subheader',
  				'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'class' => 'mfn-builder-subheader row-header mfn-toggled',
					'title' => __('Next arrow custom style', 'mfn-opts'),
				),

  			array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-next.mfn-swiper-arrow:top',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Top', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-next.mfn-swiper-arrow:bottom',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Bottom', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-next.mfn-swiper-arrow:left',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Left', 'mfn-opts'),
						'responsive' => 'desktop',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-next.mfn-swiper-arrow:right',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Right', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled arrows-custom-style',
				),



				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-next.mfn-swiper-arrow:top_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Top', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-next.mfn-swiper-arrow:bottom_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Bottom', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-next.mfn-swiper-arrow:left_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Left', 'mfn-opts'),
						'responsive' => 'tablet',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-next.mfn-swiper-arrow:right_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Right', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled arrows-custom-style',
				),



				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-next.mfn-swiper-arrow:top_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Top', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-next.mfn-swiper-arrow:bottom_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Bottom', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-next.mfn-swiper-arrow:left_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Left', 'mfn-opts'),
						'responsive' => 'laptop',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-next.mfn-swiper-arrow:right_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Right', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled arrows-custom-style',
				),



				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-next.mfn-swiper-arrow:top_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Top', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-next.mfn-swiper-arrow:bottom_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Bottom', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-next.mfn-swiper-arrow:left_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Left', 'mfn-opts'),
						'responsive' => 'mobile',
					'class' => 'mfn-toggled arrows-custom-style',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .swiper-button-next.mfn-swiper-arrow:right_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Right', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled arrows-custom-style',
				),



				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper.mfn-arrows-custom .mfn-swiper-arrow:transformtranslatey',
					'condition' => array( 'id' => 'query_slider_arrows_style', 'opt' => 'is', 'val' => 'custom' ),
					'type' => 'sliderbar',
					'class' => 'mfn-toggled',
					'title' => __('Translate(Y)', 'mfn-opts'),
					'desc' => __('Moves navigation arrows along the Y-axis by the percent height', 'mfn-opts'),
					'after' => '%',
					'param' => array(
						'min' => '-100',
						'max' => '100',
						'step' => '1',
						'unit' => '%',
					)
				),

				array(
					'type' => 'subheader',
					'class' => 'mfn-builder-subheader row-header mfn-toggled',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'title' => __('Custom arrows', 'mfn-opts'),
				),

				array(
					'id' => 'query_display_slider_arrow_prev',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'icon',
					'class' => 'mfn-toggled',
					'title' => __('Prev arrow', 'mfn-opts'),
					'preview' => 'icon',
					'std' => 'icon-left-open-big'
				),

				array(
					'id' => 'query_display_slider_arrow_next',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'icon',
					'class' => 'mfn-toggled',
					'title' => __('Next arrow', 'mfn-opts'),
					'preview' => 'icon',
					'std' => 'icon-right-open-big'
				),

				array(
					'type' => 'subheader',
					'class' => 'mfn-builder-subheader row-header mfn-toggled',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'title' => __('Size', 'mfn-opts'),
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow:--mfn-swiper-arrow-width',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'desktop',
					'after' => 'px',
					'class' => 'mfn-toggled',
					'title' => __('Width', 'mfn-opts'),
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow:--mfn-swiper-arrow-height',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'desktop',
					'after' => 'px',
					'class' => 'mfn-toggled',
					'title' => __('Height', 'mfn-opts'),
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow i:--mfn-swiper-arrow-size',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'title' => __('Size', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow:--mfn-swiper-arrow-spacing',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'title' => __('Offset', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '-300',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),




				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow:--mfn-swiper-arrow-width_tablet',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'title' => __('Width', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow:--mfn-swiper-arrow-height_tablet',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'title' => __('Height', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow i:--mfn-swiper-arrow-size_tablet',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'title' => __('Size', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow:--mfn-swiper-arrow-spacing_tablet',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'title' => __('Offset', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '-300',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),







				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow:--mfn-swiper-arrow-width_laptop',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'title' => __('Width', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow:--mfn-swiper-arrow-height_laptop',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'title' => __('Height', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow i:--mfn-swiper-arrow-size_laptop',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'title' => __('Size', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow:--mfn-swiper-arrow-spacing_laptop',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'title' => __('Offset', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '-300',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),




				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow:--mfn-swiper-arrow-width_mobile',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'title' => __('Width', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow:--mfn-swiper-arrow-height_mobile',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'title' => __('Height', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow i:--mfn-swiper-arrow-size_mobile',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'title' => __('Size', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '0',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow:--mfn-swiper-arrow-spacing_mobile',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'type' => 'sliderbar',
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'title' => __('Offset', 'mfn-opts'),
					'after' => 'px',
					'param' => array(
						'min' => '-300',
						'max' => '300',
						'step' => '1',
						'unit' => 'px',
					)
				),





				array(
					'type' => 'subheader',
					'class' => 'mfn-builder-subheader row-header mfn-toggled',
					'title' => __('Border', 'mfn-opts'),
				),

  			array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow:border-style',
					'attr_id' => 'border_style_qlarr',
					'type' => 'select',
					'class' => 'mfn-toggled',
					'title' => __('Border style', 'mfn-opts'),
					'options' => [
						'none' => __('None', 'mfn-opts'),
						'solid' => __('Solid', 'mfn-opts'),
						'dashed' => __('Dashed', 'mfn-opts'),
						'dotted' => __('Dotted', 'mfn-opts'),
						'double' => __('Double', 'mfn-opts'),
					],
				),

				array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow:border-width',
  				'condition' => array( 'id' => 'border_style_qlarr', 'opt' => 'isnt', 'val' => 'none' ),
  				'type' => 'dimensions',
  				'title' => __('Border width', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-width',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow:border-width_laptop',
  				'condition' => array( 'id' => 'border_style_qlarr', 'opt' => 'isnt', 'val' => 'none' ),
  				'type' => 'dimensions',
  				'title' => __('Border width', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-width',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow:border-width_tablet',
  				'condition' => array( 'id' => 'border_style_qlarr', 'opt' => 'isnt', 'val' => 'none' ),
  				'type' => 'dimensions',
  				'title' => __('Border width', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-width',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow:border-width_mobile',
  				'condition' => array( 'id' => 'border_style_qlarr', 'opt' => 'isnt', 'val' => 'none' ),
  				'type' => 'dimensions',
  				'title' => __('Border width', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-width',
  			),

				array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow:border-radius',
  				'type' => 'dimensions',
  				'title' => __('Border radius', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-radius',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow:border-radius_laptop',
  				'type' => 'dimensions',
  				'title' => __('Border radius', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-radius',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow:border-radius_tablet',
  				'type' => 'dimensions',
  				'title' => __('Border radius', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-radius',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow:border-radius_mobile',
  				'type' => 'dimensions',
  				'title' => __('Border radius', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-radius',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow:box-shadow',
  				'type' => 'box_shadow',
  				'title' => __('Box shadow', 'mfn-opts'),
					'class' => 'mfn-toggled',
					'css_attr' => 'box-shadow',
  			),

				array(
					'type' => 'subheader',
					'class' => 'mfn-builder-subheader row-header mfn-toggled',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'title' => __('Color', 'mfn-opts'),
				),


				array(
					'type' => 'html',
					'condition' => array( 'id' => 'query_display', 'opt' => 'is', 'val' => 'slider' ),
					'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow:--swiper-navigation-color',
					'type' => 'color',
					'class' => 'mfn-toggled',
					'title' => __('Color', 'mfn-opts'),
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow:background-color',
					'type' => 'color',
					'class' => 'mfn-toggled',
					'title' => __('Background', 'mfn-opts'),
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow:border-color',
					'condition' => array( 'id' => 'border_style_qlarr', 'opt' => 'isnt', 'val' => 'none' ),
					'type' => 'color',
					'class' => 'mfn-toggled',
					'title' => __('Border color', 'mfn-opts'),
				),

				array(
					'type' => 'html',
					'html' => '</div><div class="mfn-sft mfn-sft-hover">',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow|hover:--swiper-navigation-color',
					'type' => 'color',
					'class' => 'mfn-toggled',
					'title' => __('Color', 'mfn-opts'),
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow|hover:background-color',
					'type' => 'color',
					'class' => 'mfn-toggled',
					'title' => __('Background', 'mfn-opts'),
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement.mfn-looped-items-slider-wrapper .mfn-swiper-arrow|hover:border-color',
					'condition' => array( 'id' => 'border_style_qlarr', 'opt' => 'isnt', 'val' => 'none' ),
					'type' => 'color',
					'class' => 'mfn-toggled',
					'title' => __('Border color', 'mfn-opts'),
				),

				array(
					'type' => 'html',
					'html' => '</div></div>',
				),


  			array(
					'type' => 'html',
					'html' => '</div>', // end: modalbox-card
				),























				array(
					'type' => 'html',
					'html' => '<div class="modalbox-card modalbox-card-advanced">',
				),

				array(
					'type' => 'header',
					'class' => 'toggled_header row-header',
					'title' => __('Dimensions', 'mfn-opts'),
				),

				array(
					'id' => 'width_switcher',
					'attr_id' => 'wrap_width_switcher',
					'type' => 'select',
					'class' => 'mfn-toggled mfn-layout-modifier',
					'title' => __('Width', 'mfn-opts'),
					'options' => array(
  					'default' => __('Default', 'mfn-opts'),
  					'custom' 	=> __('Custom', 'mfn-opts'),
  				),
  				'std' => 'default',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement:flex',
					'type' => 'text',
					'condition' => array( 'id' => 'wrap_width_switcher', 'opt' => 'is', 'val' => 'custom' ),
					'responsive' => 'desktop',
					'default_unit' => 'px',
					'class' => 'mfn-toggled advanced_flex mfn-slider-input mfn-layout-modifier',
					'title' => __('Width', 'mfn-opts'),
					'desc' => __('Use px, %, vw, vh or auto to set content width', 'mfn-opts'),
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement:flex_laptop',
					'type' => 'text',
					'condition' => array( 'id' => 'wrap_width_switcher', 'opt' => 'is', 'val' => 'custom' ),
					'responsive' => 'laptop',
					'default_unit' => 'px',
					'class' => 'mfn-toggled advanced_flex mfn-slider-input mfn-layout-modifier',
					'title' => __('Width', 'mfn-opts'),
					'desc' => __('Use px, %, vw, vh or auto to set content width', 'mfn-opts'),
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement:flex_tablet',
					'type' => 'text',
					'condition' => array( 'id' => 'wrap_width_switcher', 'opt' => 'is', 'val' => 'custom' ),
					'responsive' => 'tablet',
					'default_unit' => 'px',
					'class' => 'mfn-toggled advanced_flex mfn-slider-input mfn-layout-modifier',
					'title' => __('Width', 'mfn-opts'),
					'desc' => __('Use px, %, vw, vh or auto to set content width', 'mfn-opts'),
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement:flex_mobile',
					'type' => 'text',
					'condition' => array( 'id' => 'wrap_width_switcher', 'opt' => 'is', 'val' => 'custom' ),
					'responsive' => 'mobile',
					'default_unit' => 'px',
					'class' => 'mfn-toggled advanced_flex mfn-slider-input mfn-layout-modifier',
					'title' => __('Width', 'mfn-opts'),
					'desc' => __('Use px, %, vw, vh or auto to set content width', 'mfn-opts'),
				),

				array(
					'id' => 'height_switcher',
					'attr_id' => 'wrap_height_switcher',
					'type' => 'select',
					'class' => 'mfn-toggled',
					'title' => __('Height', 'mfn-opts'),
					'options' => array(
  					'default' => __('Default', 'mfn-opts'),
  					'custom' 	=> __('Custom', 'mfn-opts'),
  				),
  				'std' => 'default',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:height',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'wrap_height_switcher', 'opt' => 'is', 'val' => 'custom' ),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled adv_height_wrap mfn-slider-input',
					'title' => __('Height', 'mfn-opts')
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:height_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'wrap_height_switcher', 'opt' => 'is', 'val' => 'custom' ),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled adv_height_wrap mfn-slider-input',
					'title' => __('Height', 'mfn-opts')
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:height_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'wrap_height_switcher', 'opt' => 'is', 'val' => 'custom' ),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled adv_height_wrap mfn-slider-input',
					'title' => __('Height', 'mfn-opts')
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:height_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'wrap_height_switcher', 'opt' => 'is', 'val' => 'custom' ),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled adv_height_wrap mfn-slider-input',
					'title' => __('Height', 'mfn-opts')
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:min-height',
					'type' => 'text',
					'default_unit' => 'px',
					'responsive' => 'desktop',
					'class' => 'mfn-toggled mfn-slider-input',
					'title' => __('Min height', 'mfn-opts')
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:min-height_laptop',
					'type' => 'text',
					'default_unit' => 'px',
					'responsive' => 'laptop',
					'class' => 'mfn-toggled mfn-slider-input',
					'title' => __('Min height', 'mfn-opts')
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:min-height_tablet',
					'type' => 'text',
					'default_unit' => 'px',
					'responsive' => 'tablet',
					'class' => 'mfn-toggled mfn-slider-input',
					'title' => __('Min height', 'mfn-opts')
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:min-height_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'responsive' => 'mobile',
					'class' => 'mfn-toggled mfn-slider-input',
					'title' => __('Min height', 'mfn-opts')
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement:flex-grow',
					'type' => 'select',
					'condition' => array( 'id' => 'wrap_width_switcher', 'opt' => 'is', 'val' => 'default' ),
					'class' => 'mfn-toggled visible-header-builder',
					'responsive' => 'desktop',
					'title' => __('Fit container', 'mfn-opts'),
					'options' => array(
						'unset' => __('Default', 'mfn-opts'),
  					'1' => __('Fit', 'mfn-opts'),
  				),
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement:flex-grow_laptop',
					'type' => 'select',
					'condition' => array( 'id' => 'wrap_width_switcher', 'opt' => 'is', 'val' => 'default' ),
					'class' => 'mfn-toggled visible-header-builder',
					'responsive' => 'laptop',
					'title' => __('Fit container', 'mfn-opts'),
					'options' => array(
						'unset' => __('Default', 'mfn-opts'),
  					'1' => __('Fit', 'mfn-opts'),
  				),
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement:flex-grow_tablet',
					'type' => 'select',
					'condition' => array( 'id' => 'wrap_width_switcher', 'opt' => 'is', 'val' => 'default' ),
					'class' => 'mfn-toggled visible-header-builder',
					'responsive' => 'tablet',
					'title' => __('Fit container', 'mfn-opts'),
					'options' => array(
						'unset' => __('Default', 'mfn-opts'),
  					'1' => __('Fit', 'mfn-opts'),
  				),
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement:flex-grow_mobile',
					'type' => 'select',
					'condition' => array( 'id' => 'wrap_width_switcher', 'opt' => 'is', 'val' => 'default' ),
					'class' => 'mfn-toggled visible-header-builder',
					'responsive' => 'mobile',
					'title' => __('Fit container', 'mfn-opts'),
					'options' => array(
						'unset' => __('Default', 'mfn-opts'),
  					'1' => __('Fit', 'mfn-opts'),
  				),
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:flex-wrap',
					'type' => 'select',
					'class' => 'mfn-toggled visible-header-builder',
					'responsive' => 'desktop',
					'title' => __('Wrap elements', 'mfn-opts'),
					'options' => array(
						'nowrap' => __('Nowrap', 'mfn-opts'),
  					'wrap' => __('Wrap', 'mfn-opts'),
  				),
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:flex-wrap_laptop',
					'type' => 'select',
					'class' => 'mfn-toggled visible-header-builder',
					'responsive' => 'laptop',
					'title' => __('Wrap elements', 'mfn-opts'),
					'options' => array(
						'nowrap' 	=> __('Nowrap', 'mfn-opts'),
  					'wrap' => __('Wrap', 'mfn-opts'),
  				),
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:flex-wrap_tablet',
					'type' => 'select',
					'class' => 'mfn-toggled visible-header-builder',
					'responsive' => 'tablet',
					'title' => __('Wrap elements', 'mfn-opts'),
					'options' => array(
						'nowrap' 	=> __('Nowrap', 'mfn-opts'),
  					'wrap' => __('Wrap', 'mfn-opts'),
  				),
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:flex-wrap_mobile',
					'type' => 'select',
					'class' => 'mfn-toggled visible-header-builder',
					'responsive' => 'mobile',
					'title' => __('Wrap elements', 'mfn-opts'),
					'options' => array(
						'nowrap' 	=> __('Nowrap', 'mfn-opts'),
  					'wrap' => __('Wrap', 'mfn-opts'),
  				),
				),

  			array(
  				'type' => 'header',
  				'class' => 'toggled_header row-header',
						'title' => __('Spacing', 'mfn-opts'),
					),

					array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:padding',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Padding', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'padding',
  			),

				array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:padding_laptop',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Padding', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'css_attr' => 'padding',
  			),


  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:padding_tablet',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Padding', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'padding',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:padding_mobile',
  				'type' => 'dimensions',
  				'version' => 'separated-fields',
  				'title' => __('Padding', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'padding',
  			),

				array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:margin',
  				'oldid' => 'move_up',
  				'version' => 'separated-fields',
  				'type' => 'dimensions',
  				'title' => __('Margin', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled advanced_margin',
					'css_attr' => 'margin',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:margin_tablet',
  				'oldid' => 'move_up',
  				'version' => 'separated-fields',
  				'type' => 'dimensions',
  				'title' => __('Margin', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled advanced_margin',
					'css_attr' => 'margin',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:margin_laptop',
  				'oldid' => 'move_up',
  				'version' => 'separated-fields',
  				'type' => 'dimensions',
  				'title' => __('Margin', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled advanced_margin',
					'css_attr' => 'margin',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:margin_mobile',
  				'oldid' => 'move_up',
  				'version' => 'separated-fields',
  				'type' => 'dimensions',
  				'title' => __('Margin', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled advanced_margin',
					'css_attr' => 'margin',
  			),

  			array(
  				'type' => 'header',
  				'class' => 'toggled_header hide_if_ql row-header',
					'title' => __('Positioning', 'mfn-opts'),
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement:align-self',
					'condition' => array( 'id' => 'wrap_sticky_desktop', 'opt' => 'is', 'val' => '0' ),
					'type' => 'radio_img',
					'alias' => 'flex-align-self',
					'responsive' => 'desktop',
					'class' => 'form-content-full-width small mfn-toggled adv_alignself_wrap',
					'title' => __('Wrap position', 'mfn-opts'),
					'options' => array(
						'' => __('Default', 'mfn-opts'),
						'flex-start' => 'Top',
						'flex-end' => 'Bottom',
						'center' => 'Center',
						'stretch' => 'Stretch',
					),
					'std' => '',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement:align-self_laptop',
					'condition' => array( 'id' => 'wrap_sticky_laptop', 'opt' => 'is', 'val' => '0' ),
					'type' => 'radio_img',
					'alias' => 'flex-align-self',
					'responsive' => 'laptop',
					'class' => 'form-content-full-width small mfn-toggled',
					'title' => __('Wrap position', 'mfn-opts'),
					'options' => array(
						'' => __('Default', 'mfn-opts'),
						'flex-start' => 'Top',
						'flex-end' => 'Bottom',
						'center' => 'Center',
						'stretch' => 'Stretch',
					),
					'std' => '',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement:align-self_tablet',
					'condition' => array( 'id' => 'wrap_sticky_tablet', 'opt' => 'is', 'val' => '0' ),
					'type' => 'radio_img',
					'alias' => 'flex-align-self',
					'responsive' => 'tablet',
					'class' => 'form-content-full-width small mfn-toggled',
					'title' => __('Wrap position', 'mfn-opts'),
					'options' => array(
						'' => __('Default', 'mfn-opts'),
						'flex-start' => 'Top',
						'flex-end' => 'Bottom',
						'center' => 'Center',
						'stretch' => 'Stretch',
					),
					'std' => '',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement:align-self_mobile',
					'condition' => array( 'id' => 'wrap_sticky_mobile', 'opt' => 'is', 'val' => '0' ),
					'type' => 'radio_img',
					'alias' => 'flex-align-self',
					'responsive' => 'mobile',
					'class' => 'form-content-full-width small mfn-toggled',
					'title' => __('Wrap position', 'mfn-opts'),
					'options' => array(
						'' => __('Default', 'mfn-opts'),
						'flex-start' => 'Top',
						'flex-end' => 'Bottom',
						'center' => 'Center',
						'stretch' => 'Stretch',
					),
					'std' => '',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:align-content',
					'oldid' => 'vertical_align',
					'type' => 'radio_img',
					'alias' => 'flex-align-content',
					'responsive' => 'desktop',
					'class' => 'form-content-full-width small mfn-toggled',
					'title' => __('Wrap content position', 'mfn-opts'),
					'desc' => __('Works with <b>Stretch</b> only', 'mfn-opts'),
					'options' => array(
						'' => __('Default', 'mfn-opts'),
						'flex-start' => 'Top',
						'flex-end' => 'Bottom',
						'center' => 'Center',
						'stretch' => 'Stretch',
						'space-between' => 'Space between',
						'space-around' => 'Space around',
						'space-evenly' => 'Space evenly'
					),
					'std' => '',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:align-content_laptop',
					'oldid' => 'vertical_align',
					'type' => 'radio_img',
					'alias' => 'flex-align-content',
					'responsive' => 'laptop',
					'class' => 'form-content-full-width small mfn-toggled',
					'title' => __('Wrap content position', 'mfn-opts'),
					'desc' => __('Works with <b>Stretch</b> only', 'mfn-opts'),
					'options' => array(
						'' => __('Default', 'mfn-opts'),
						'flex-start' => 'Top',
						'flex-end' => 'Bottom',
						'center' => 'Center',
						'stretch' => 'Stretch',
						'space-between' => 'Space between',
						'space-around' => 'Space around',
						'space-evenly' => 'Space evenly'
					),
					'std' => '',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:align-content_tablet',
					'oldid' => 'vertical_align',
					'type' => 'radio_img',
					'alias' => 'flex-align-content',
					'responsive' => 'tablet',
					'class' => 'form-content-full-width small mfn-toggled',
					'title' => __('Wrap content position', 'mfn-opts'),
					'desc' => __('Works with <b>Stretch</b> only', 'mfn-opts'),
					'options' => array(
						'' => __('Default', 'mfn-opts'),
						'flex-start' => 'Top',
						'flex-end' => 'Bottom',
						'center' => 'Center',
						'stretch' => 'Stretch',
						'space-between' => 'Space between',
						'space-around' => 'Space around',
						'space-evenly' => 'Space evenly'
					),
					'std' => '',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:align-content_mobile',
					'oldid' => 'vertical_align',
					'type' => 'radio_img',
					'alias' => 'flex-align-content',
					'responsive' => 'mobile',
					'class' => 'form-content-full-width small mfn-toggled',
					'title' => __('Wrap content position', 'mfn-opts'),
					'desc' => __('Works with <b>Stretch</b> only', 'mfn-opts'),
					'options' => array(
						'' => __('Default', 'mfn-opts'),
						'flex-start' => 'Top',
						'flex-end' => 'Bottom',
						'center' => 'Center',
						'stretch' => 'Stretch',
						'space-between' => 'Space between',
						'space-around' => 'Space around',
						'space-evenly' => 'Space evenly'
					),
					'std' => '',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:align-items',
					'type' => 'radio_img',
					'alias' => 'flex-align-items',
					'class' => 'form-content-full-width small mfn-toggled',
					'responsive' => 'desktop',
					'title' => __('Elements vertical align', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'flex-start' => 'Top',
						'flex-end' => 'Bottom',
						'center' => 'Center',
						'stretch' => 'Stretch columns',
					],
					'std' => '',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:align-items_laptop',
					'type' => 'radio_img',
					'alias' => 'flex-align-items',
					'class' => 'form-content-full-width small mfn-toggled',
					'responsive' => 'laptop',
					'title' => __('Elements vertical align', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'flex-start' => 'Top',
						'flex-end' => 'Bottom',
						'center' => 'Center',
						'stretch' => 'Stretch columns',
					],
					'std' => '',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:align-items_tablet',
					'type' => 'radio_img',
					'alias' => 'flex-align-items',
					'class' => 'form-content-full-width small mfn-toggled',
					'responsive' => 'tablet',
					'title' => __('Elements vertical align', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'flex-start' => 'Top',
						'flex-end' => 'Bottom',
						'center' => 'Center',
						'stretch' => 'Stretch columns',
					],
					'std' => '',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:align-items_mobile',
					'type' => 'radio_img',
					'alias' => 'flex-align-items',
					'class' => 'form-content-full-width small mfn-toggled',
					'responsive' => 'mobile',
					'title' => __('Elements vertical align', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'flex-start' => 'Top',
						'flex-end' => 'Bottom',
						'center' => 'Center',
						'stretch' => 'Stretch columns',
					],
					'std' => '',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:justify-content',
					'type' => 'radio_img',
					'alias' => 'flex-justify',
					'class' => 'form-content-full-width small mfn-toggled',
					'responsive' => 'desktop',
					'title' => __('Elements horizontal align', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'flex-start' => __('Left', 'mfn-opts'),
						'center' => __('Center', 'mfn-opts'),
						'flex-end' => __('Right', 'mfn-opts'),
						'space-between' => __('Space between', 'mfn-opts'),
						'space-around' => __('Space around', 'mfn-opts'),
					],
					'std' => '',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:justify-content_laptop',
					'type' => 'radio_img',
					'alias' => 'flex-justify',
					'class' => 'form-content-full-width small mfn-toggled',
					'responsive' => 'laptop',
					'title' => __('Elements horizontal align', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'flex-start' => __('Left', 'mfn-opts'),
						'center' => __('Center', 'mfn-opts'),
						'flex-end' => __('Right', 'mfn-opts'),
						'space-between' => __('Space between', 'mfn-opts'),
						'space-around' => __('Space around', 'mfn-opts'),
					],
					'std' => '',
				),


				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:justify-content_tablet',
					'type' => 'radio_img',
					'alias' => 'flex-justify',
					'class' => 'form-content-full-width small mfn-toggled',
					'responsive' => 'tablet',
					'title' => __('Elements horizontal align', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'flex-start' => __('Left', 'mfn-opts'),
						'center' => __('Center', 'mfn-opts'),
						'flex-end' => __('Right', 'mfn-opts'),
						'space-between' => __('Space between', 'mfn-opts'),
						'space-around' => __('Space around', 'mfn-opts'),
					],
					'std' => '',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:justify-content_mobile',
					'type' => 'radio_img',
					'alias' => 'flex-justify',
					'class' => 'form-content-full-width small mfn-toggled',
					'responsive' => 'mobile',
					'title' => __('Elements horizontal align', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'flex-start' => __('Left', 'mfn-opts'),
						'center' => __('Center', 'mfn-opts'),
						'flex-end' => __('Right', 'mfn-opts'),
						'space-between' => __('Space between', 'mfn-opts'),
						'space-around' => __('Space around', 'mfn-opts'),
					],
					'std' => '',
				),

				array(
					'type' => 'helper',
					'class' => 'mfn-toggled',
					'title' => __('Need help', 'mfn-opts'),
					'link' => 'https://support.muffingroup.com/documentation/bebuilder/positioning/#wrap',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement:z-index',
					'type' => 'text',
					'title' => __('Z-index', 'mfn-opts'),
					'param' => 'number',
					'class' => 'narrow mfn-toggled',
					'preview' => 'number',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement:order',
					'type' => 'text',
					'title' => __('Order', 'mfn-opts'),
					'param' => 'number',
						'class' => 'narrow mfn-toggled',
						'responsive' => 'desktop',
					'preview' => 'number',
				),


				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement:order_laptop',
					'type' => 'text',
					'title' => __('Order', 'mfn-opts'),
					'param' => 'number',
						'class' => 'narrow mfn-toggled',
						'responsive' => 'laptop',
					'preview' => 'number',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement:order_tablet',
					'type' => 'text',
					'title' => __('Order', 'mfn-opts'),
					'param' => 'number',
						'class' => 'narrow mfn-toggled',
						'responsive' => 'tablet',
					'preview' => 'number',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement:order_mobile',
					'type' => 'text',
					'title' => __('Order', 'mfn-opts'),
					'param' => 'number',
						'class' => 'narrow mfn-toggled',
						'responsive' => 'mobile',
					'preview' => 'number',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:position',
					'attr_id' => 'wrap_position',
					'type' => 'select',
					'class' => 'mfn-toggled custom-element-position',
					'title' => __('Position', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'relative' => __('Offset', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:top',
					'type' => 'text',
					'default_unit' => 'px',
					'responsive' => 'desktop',
					'condition' => array( 'id' => 'wrap_position', 'opt' => 'isnt', 'val' => '' ),
					'title' => __('Top', 'mfn-opts'),
					'class' => 'mfn-toggled custom-abs-offset-values',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:bottom',
					'type' => 'text',
					'default_unit' => 'px',
					'responsive' => 'desktop',
					'condition' => array( 'id' => 'wrap_position', 'opt' => 'isnt', 'val' => '' ),
					'title' => __('Bottom', 'mfn-opts'),
					'class' => 'mfn-toggled custom-abs-offset-values',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:left',
					'type' => 'text',
					'default_unit' => 'px',
					'responsive' => 'desktop',
					'condition' => array( 'id' => 'wrap_position', 'opt' => 'isnt', 'val' => '' ),
					'title' => __('Left', 'mfn-opts'),
					'class' => 'mfn-toggled custom-abs-offset-values',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:right',
					'type' => 'text',
					'default_unit' => 'px',
					'responsive' => 'desktop',
					'condition' => array( 'id' => 'wrap_position', 'opt' => 'isnt', 'val' => '' ),
					'title' => __('Right', 'mfn-opts'),
					'class' => 'mfn-toggled custom-abs-offset-values',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:top_tablet',
					'type' => 'text',
					'responsive' => 'tablet',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'wrap_position', 'opt' => 'isnt', 'val' => '' ),
					'title' => __('Top', 'mfn-opts'),
					'class' => 'mfn-toggled custom-abs-offset-values',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:bottom_tablet',
					'type' => 'text',
					'responsive' => 'tablet',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'wrap_position', 'opt' => 'isnt', 'val' => '' ),
					'title' => __('Bottom', 'mfn-opts'),
					'class' => 'mfn-toggled custom-abs-offset-values',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:left_tablet',
					'type' => 'text',
					'responsive' => 'tablet',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'wrap_position', 'opt' => 'isnt', 'val' => '' ),
					'title' => __('Left', 'mfn-opts'),
					'class' => 'mfn-toggled custom-abs-offset-values',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:right_tablet',
					'type' => 'text',
					'responsive' => 'tablet',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'wrap_position', 'opt' => 'isnt', 'val' => '' ),
					'title' => __('Right', 'mfn-opts'),
					'class' => 'mfn-toggled custom-abs-offset-values',
				),






				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:top_laptop',
					'type' => 'text',
					'responsive' => 'laptop',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'wrap_position', 'opt' => 'isnt', 'val' => '' ),
					'title' => __('Top', 'mfn-opts'),
					'class' => 'mfn-toggled custom-abs-offset-values',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:bottom_laptop',
					'type' => 'text',
					'responsive' => 'laptop',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'wrap_position', 'opt' => 'isnt', 'val' => '' ),
					'title' => __('Bottom', 'mfn-opts'),
					'class' => 'mfn-toggled custom-abs-offset-values',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:left_laptop',
					'type' => 'text',
					'responsive' => 'laptop',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'wrap_position', 'opt' => 'isnt', 'val' => '' ),
					'title' => __('Left', 'mfn-opts'),
					'class' => 'mfn-toggled custom-abs-offset-values',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:right_laptop',
					'type' => 'text',
					'responsive' => 'laptop',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'wrap_position', 'opt' => 'isnt', 'val' => '' ),
					'title' => __('Right', 'mfn-opts'),
					'class' => 'mfn-toggled custom-abs-offset-values',
				),



				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:top_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'responsive' => 'mobile',
					'condition' => array( 'id' => 'wrap_position', 'opt' => 'isnt', 'val' => '' ),
					'title' => __('Top', 'mfn-opts'),
					'class' => 'mfn-toggled custom-abs-offset-values',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:bottom_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'responsive' => 'mobile',
					'condition' => array( 'id' => 'wrap_position', 'opt' => 'isnt', 'val' => '' ),
					'title' => __('Bottom', 'mfn-opts'),
					'class' => 'mfn-toggled custom-abs-offset-values',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:left_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'responsive' => 'mobile',
					'condition' => array( 'id' => 'wrap_position', 'opt' => 'isnt', 'val' => '' ),
					'title' => __('Left', 'mfn-opts'),
					'class' => 'mfn-toggled custom-abs-offset-values',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:right_mobile',
					'type' => 'text',
					'default_unit' => 'px',
					'responsive' => 'mobile',
					'condition' => array( 'id' => 'wrap_position', 'opt' => 'isnt', 'val' => '' ),
					'title' => __('Right', 'mfn-opts'),
					'class' => 'mfn-toggled custom-abs-offset-values',
				),

  			array(
  				'type' => 'header',
  				'class' => 'toggled_header row-header',
					'title' => __('Background', 'mfn-opts'),
				),

				array(
					'type' => 'html',
					'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
				),

				array(
					'id' => 'background_switcher',
					'attr_id' => 'background_switcher',
					'type' => 'switch',
					'class' => 'mfn-fields-switcher disable-history',
					'title' => __('Background type', 'mfn-opts'),
					'options' => array(
  					'default' => __('Default', 'mfn-opts'),
  					'gradient' 	=> __('Gradient', 'mfn-opts'),
  				),
					'visual_options' => array(
  					'default' => '',
  					'gradient' 	=> '',
  				),
  				'std' => 'default',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-color',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'color',
					'title' => __('Color', 'mfn-opts'),
				),

				array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:gradient',
  				'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'gradient' ),
  				'type' => 'gradient',
  				'title' => __('Gradient', 'mfn-opts'),
  				'class' => 'mfn-toggled',
					'css_attr' => 'gradient',
  			),

				array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-image',
  				'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
  				'oldid' => 'bg_color',
  				'dynamic_data' => 'featured_image',
  				'type' => 'upload',
  				'title' => __('Image', 'mfn-opts'),
  				'responsive' => 'desktop',

					'css_attr' => 'background-image',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-image_laptop',
  				'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
  				'oldid' => 'bg_image',
  				'dynamic_data' => 'featured_image',
  				'type' => 'upload',
  				'title' => __('Image', 'mfn-opts'),
  				'responsive' => 'laptop',

					'css_attr' => 'background-image',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-image_tablet',
  				'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
  				'oldid' => 'bg_image',
  				'dynamic_data' => 'featured_image',
  				'type' => 'upload',
  				'title' => __('Image', 'mfn-opts'),
  				'responsive' => 'tablet',

					'css_attr' => 'background-image',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-image_mobile',
  				'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
  				'type' => 'upload',
  				'dynamic_data' => 'featured_image',
  				'title' => __('Image', 'mfn-opts'),
  				'responsive' => 'mobile',

					'css_attr' => 'background-image',
  			),

  			array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-repeat',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'desktop',

					'css_attr' => 'background-repeat',
					'title' => __('Repeat', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'repeat' => __('Repeat', 'mfn-opts'),
						'no-repeat' => __('No repeat', 'mfn-opts'),
						'repeat-x' => __('Repeat X', 'mfn-opts'),
						'repeat-y' => __('Repeat Y', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-repeat_laptop',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'laptop',

					'css_attr' => 'background-repeat',
					'title' => __('Repeat', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'repeat' => __('Repeat', 'mfn-opts'),
						'no-repeat' => __('No repeat', 'mfn-opts'),
						'repeat-x' => __('Repeat X', 'mfn-opts'),
						'repeat-y' => __('Repeat Y', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-repeat_tablet',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'tablet',

					'css_attr' => 'background-repeat',
					'title' => __('Repeat', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'repeat' => __('Repeat', 'mfn-opts'),
						'no-repeat' => __('No repeat', 'mfn-opts'),
						'repeat-x' => __('Repeat X', 'mfn-opts'),
						'repeat-y' => __('Repeat Y', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-repeat_mobile',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'mobile',

					'css_attr' => 'background-repeat',
					'title' => __('Repeat', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'repeat' => __('Repeat', 'mfn-opts'),
						'no-repeat' => __('No repeat', 'mfn-opts'),
						'repeat-x' => __('Repeat X', 'mfn-opts'),
						'repeat-y' => __('Repeat Y', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-position',
					'attr_id' => 'bgposopt',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_position',
					'type' => 'select',
					'responsive' => 'desktop',
					'class' => 'bgposopt',
					'css_attr' => 'background-position',
					'title' => __('Position', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'center' => __('Center Center', 'mfn-opts'),
						'center left' => __('Center Left', 'mfn-opts'),
						'center right' => __('Center Right', 'mfn-opts'),
						'top left' => __('Top Left', 'mfn-opts'),
						'top right' => __('Top Right', 'mfn-opts'),
						'top center' => __('Top Center', 'mfn-opts'),
						'bottom left' => __('Bottom Left', 'mfn-opts'),
						'bottom right' => __('Bottom Right', 'mfn-opts'),
						'bottom center' => __('Bottom Center', 'mfn-opts'),
						'custom' => __('Custom', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-position_laptop',
					'attr_id' => 'bgposopt_laptop',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_position',
					'type' => 'select',
					'responsive' => 'laptop',
					'class' => 'bgposopt',
					'css_attr' => 'background-position',
					'title' => __('Position', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'center' => __('Center Center', 'mfn-opts'),
						'center left' => __('Center Left', 'mfn-opts'),
						'center right' => __('Center Right', 'mfn-opts'),
						'top left' => __('Top Left', 'mfn-opts'),
						'top right' => __('Top Right', 'mfn-opts'),
						'top center' => __('Top Center', 'mfn-opts'),
						'bottom left' => __('Bottom Left', 'mfn-opts'),
						'bottom right' => __('Bottom Right', 'mfn-opts'),
						'bottom center' => __('Bottom Center', 'mfn-opts'),
						'custom' => __('Custom', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-position_tablet',
					'attr_id' => 'bgposopt_tablet',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_position',
					'type' => 'select',
					'responsive' => 'tablet',
					'class' => 'bgposopt',
					'css_attr' => 'background-position',
					'title' => __('Position', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'center' => __('Center Center', 'mfn-opts'),
						'center left' => __('Center Left', 'mfn-opts'),
						'center right' => __('Center Right', 'mfn-opts'),
						'top left' => __('Top Left', 'mfn-opts'),
						'top right' => __('Top Right', 'mfn-opts'),
						'top center' => __('Top Center', 'mfn-opts'),
						'bottom left' => __('Bottom Left', 'mfn-opts'),
						'bottom right' => __('Bottom Right', 'mfn-opts'),
						'bottom center' => __('Bottom Center', 'mfn-opts'),
						'custom' => __('Custom', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-position_mobile',
					'attr_id' => 'bgposopt_mobile',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_position',
					'type' => 'select',
					'responsive' => 'mobile',
					'class' => 'bgposopt',
					'css_attr' => 'background-position',
					'title' => __('Position', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'center' => __('Center Center', 'mfn-opts'),
						'center left' => __('Center Left', 'mfn-opts'),
						'center right' => __('Center Right', 'mfn-opts'),
						'top left' => __('Top Left', 'mfn-opts'),
						'top right' => __('Top Right', 'mfn-opts'),
						'top center' => __('Top Center', 'mfn-opts'),
						'bottom left' => __('Bottom Left', 'mfn-opts'),
						'bottom right' => __('Bottom Right', 'mfn-opts'),
						'bottom center' => __('Bottom Center', 'mfn-opts'),
						'custom' => __('Custom', 'mfn-opts'),
					],
				),






















				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-position_v2',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'bgposopt', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Custom background position', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled mfn-custom-bg-pos',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-position_laptop_v2',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'bgposopt_laptop', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Custom background position', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled mfn-custom-bg-pos',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-position_tablet_v2',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'bgposopt_tablet', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Custom background position', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled mfn-custom-bg-pos',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-position_mobile_v2',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'bgposopt_mobile', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Custom background position', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled mfn-custom-bg-pos',
				),




























				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-attachment',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'desktop',

					'css_attr' => 'background-attachment',
					'title' => __('Attachment', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'scroll' => __('Scroll', 'mfn-opts'),
						'fixed' => __('Fixed', 'mfn-opts'),
						'parallax' => __('Parallax', 'mfn-opts'),
					],
				),


				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-attachment_laptop',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'laptop',

					'css_attr' => 'background-attachment',
					'title' => __('Attachment', 'mfn-opts'),
					'desc' => __('iOS does <b>not</b> support background-attachment: fixed', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'scroll' => __('Scroll', 'mfn-opts'),
						'fixed' => __('Fixed', 'mfn-opts'),
						'parallax' => __('Parallax', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-attachment_tablet',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'tablet',

					'css_attr' => 'background-attachment',
					'title' => __('Attachment', 'mfn-opts'),
					'desc' => __('iOS does <b>not</b> support background-attachment: fixed', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'scroll' => __('Scroll', 'mfn-opts'),
						'fixed' => __('Fixed', 'mfn-opts'),
						'parallax' => __('Parallax', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-attachment_mobile',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'mobile',

					'css_attr' => 'background-attachment',
					'title' => __('Attachment', 'mfn-opts'),
					'desc' => __('iOS does <b>not</b> support background-attachment: fixed', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'scroll' => __('Scroll', 'mfn-opts'),
						'fixed' => __('Fixed', 'mfn-opts'),
						'parallax' => __('Parallax', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-size',
					'attr_id' => 'custombgsize',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_size',
					'type' => 'select',
					'responsive' => 'desktop',
					'class' => 'custombgsize',
					'css_attr' => 'background-size',
					'title' => __('Size', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'auto' => __('Auto', 'mfn-opts'),
						'contain' => __('Contain', 'mfn-opts'),
						'cover' => __('Cover', 'mfn-opts'),
						'cover-ultrawide' => __('Cover, on ultrawide screens only > 1920px', 'mfn-opts'),
						'custom' => __('Custom', 'mfn-opts'),
					],
					//'options' => mfna_bg_size()
				),


				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-size_laptop',
					'attr_id' => 'custombgsize_laptop',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_size',
					'type' => 'select',
					'responsive' => 'laptop',
					'class' => 'custombgsize',
					'css_attr' => 'background-size',
					'title' => __('Size', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'auto' => __('Auto', 'mfn-opts'),
						'contain' => __('Contain', 'mfn-opts'),
						'cover' => __('Cover', 'mfn-opts'),
						'cover-ultrawide' => __('Cover, on ultrawide screens only > 1920px', 'mfn-opts'),
						'custom' => __('Custom', 'mfn-opts'),
					],
					//'options' => mfna_bg_size()
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-size_tablet',
					'attr_id' => 'custombgsize_tablet',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_size',
					'type' => 'select',
					'responsive' => 'tablet',
					'class' => 'custombgsize',
					'css_attr' => 'background-size',
					'title' => __('Size', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'auto' => __('Auto', 'mfn-opts'),
						'contain' => __('Contain', 'mfn-opts'),
						'cover' => __('Cover', 'mfn-opts'),
						'cover-ultrawide' => __('Cover, on ultrawide screens only > 1920px', 'mfn-opts'),
						'custom' => __('Custom', 'mfn-opts'),
					],
					//'options' => mfna_bg_size()
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-size_mobile',
					'attr_id' => 'custombgsize_mobile',
					'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_size',
					'type' => 'select',
					'responsive' => 'mobile',
					'class' => 'custombgsize',
					'css_attr' => 'background-size',
					'title' => __('Size', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'auto' => __('Auto', 'mfn-opts'),
						'contain' => __('Contain', 'mfn-opts'),
						'cover' => __('Cover', 'mfn-opts'),
						'cover-ultrawide' => __('Cover, on ultrawide screens only > 1920px', 'mfn-opts'),
						'custom' => __('Custom', 'mfn-opts'),
					],
					//'options' => mfna_bg_size()
				),



				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-size_v2',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'custombgsize', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Custom background size', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled mfn-custom-bg-size',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-size_laptop_v2',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'custombgsize_laptop', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Custom background size', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled mfn-custom-bg-size',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-size_tablet_v2',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'custombgsize_tablet', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Custom background size', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled mfn-custom-bg-size',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:background-size_mobile_v2',
					'type' => 'text',
					'default_unit' => 'px',
					'condition' => array( 'id' => 'custombgsize_mobile', 'opt' => 'is', 'val' => 'custom' ),
					'title' => __('Custom background size', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled mfn-custom-bg-size',
				),



				array(
					'type' => 'html',
					'html' => '</div><div class="mfn-sft mfn-sft-hover">',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:transition',
					'type' => 'sliderbar',
					'title' => __('Transition duration', 'mfn-opts'),
					'param' => array(
						'min' => '0',
						'max' => '3',
						'step' => '0.1',
						'unit' => 's',
					)
				),

				array(
					'id' => 'background_switcher_hover',
					'attr_id' => 'background_switcher_hover',
					'type' => 'switch',
					'class' => 'mfn-fields-switcher disable-history',
					'title' => __('Background type', 'mfn-opts'),
					'options' => array(
  					'default' => __('Default', 'mfn-opts'),
  					'gradient' 	=> __('Gradient', 'mfn-opts'),
  				),
					'visual_options' => array(
  					'default' => '',
  					'gradient' 	=> '',
  				),
  				'std' => 'default',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner|hover:background-color',
					'condition' => array( 'id' => 'background_switcher_hover', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'color',
					'title' => __('Color', 'mfn-opts'),
				),

				array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner|hover:gradient',
  				'condition' => array( 'id' => 'background_switcher_hover', 'opt' => 'is', 'val' => 'gradient' ),
  				'type' => 'gradient',
  				'title' => __('Gradient', 'mfn-opts'),
  				'class' => 'mfn-toggled',
					'css_attr' => 'gradient',
  			),

				array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner|hover:background-image',
  				'condition' => array( 'id' => 'background_switcher_hover', 'opt' => 'is', 'val' => 'default' ),
  				'oldid' => 'bg_color',
  				'dynamic_data' => 'featured_image',
  				'type' => 'upload',
  				'title' => __('Image', 'mfn-opts'),
  				'responsive' => 'desktop',

					'css_attr' => 'background-image',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner|hover:background-image_laptop',
  				'condition' => array( 'id' => 'background_switcher_hover', 'opt' => 'is', 'val' => 'default' ),
  				'oldid' => 'bg_color',
  				'dynamic_data' => 'featured_image',
  				'type' => 'upload',
  				'title' => __('Image', 'mfn-opts'),
  				'responsive' => 'laptop',

					'css_attr' => 'background-image',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner|hover:background-image_tablet',
  				'condition' => array( 'id' => 'background_switcher_hover', 'opt' => 'is', 'val' => 'default' ),
  				'oldid' => 'bg_color',
  				'dynamic_data' => 'featured_image',
  				'type' => 'upload',
  				'title' => __('Image', 'mfn-opts'),
  				'responsive' => 'tablet',

					'css_attr' => 'background-image',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner|hover:background-image_mobile',
  				'condition' => array( 'id' => 'background_switcher_hover', 'opt' => 'is', 'val' => 'default' ),
  				'oldid' => 'bg_color',
  				'dynamic_data' => 'featured_image',
  				'type' => 'upload',
  				'title' => __('Image', 'mfn-opts'),
  				'responsive' => 'mobile',

					'css_attr' => 'background-image',
  			),

				array(
					'type' => 'html',
					'html' => '</div></div>',
				),

				// Background Overlay WRAP

				array(
  				'type' => 'header',
  				'class' => 'toggled_header row-header',
					'title' => __('Background Overlay', 'mfn-opts'),
				),

				array(
					'type' => 'html',
					'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner > .mcb-wrap-background-overlay:opacity',
					'type' => 'sliderbar',
					'title' => __('Opacity', 'mfn-opts'),
					'param' => array(
						'min' => '0',
						'max' => '1',
						'step' => '0.01',
					),
					'std' => '0',
				),

				array(
					'id' => 'background_overlay_switcher',
					'attr_id' => 'background_overlay_switcher',
					'type' => 'switch',
					'class' => 'mfn-fields-switcher',
					'title' => __('Background type', 'mfn-opts'),
					'options' => array(
  					'default' => __('Default', 'mfn-opts'),
  					'gradient' 	=> __('Gradient', 'mfn-opts'),
  				),
					'visual_options' => array(
  					'default' => '',
  					'gradient' 	=> '',
  				),
  				'std' => 'default',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner > .mcb-wrap-background-overlay:background-color',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'color',
					'title' => __('Color', 'mfn-opts'),
				),

				array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner > .mcb-wrap-background-overlay:gradient',
  				'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'gradient' ),
  				'type' => 'gradient',
  				'title' => __('Gradient', 'mfn-opts'),
  				'class' => 'mfn-toggled',
					'css_attr' => 'gradient',
  			),

				array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner > .mcb-wrap-background-overlay:background-image',
  				'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
  				'oldid' => 'bg_color',
  				'type' => 'upload',
  				'title' => __('Image', 'mfn-opts'),
  				'responsive' => 'desktop',

					'css_attr' => 'background-image',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner > .mcb-wrap-background-overlay:background-image_laptop',
  				'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
  				'oldid' => 'bg_image',
  				'type' => 'upload',
  				'title' => __('Image', 'mfn-opts'),
  				'responsive' => 'laptop',

					'css_attr' => 'background-image',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner > .mcb-wrap-background-overlay:background-image_tablet',
  				'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
  				'oldid' => 'bg_image',
  				'type' => 'upload',
  				'title' => __('Image', 'mfn-opts'),
  				'responsive' => 'tablet',

					'css_attr' => 'background-image',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner > .mcb-wrap-background-overlay:background-image_mobile',
  				'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
  				'type' => 'upload',
  				'title' => __('Image', 'mfn-opts'),
  				'responsive' => 'mobile',

					'css_attr' => 'background-image',
  			),

  			array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner > .mcb-wrap-background-overlay:background-repeat',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'desktop',

					'css_attr' => 'background-repeat',
					'title' => __('Repeat', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'repeat' => __('Repeat', 'mfn-opts'),
						'no-repeat' => __('No repeat', 'mfn-opts'),
						'repeat-x' => __('Repeat X', 'mfn-opts'),
						'repeat-y' => __('Repeat Y', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner > .mcb-wrap-background-overlay:background-repeat_laptop',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'laptop',

					'css_attr' => 'background-repeat',
					'title' => __('Repeat', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'repeat' => __('Repeat', 'mfn-opts'),
						'no-repeat' => __('No repeat', 'mfn-opts'),
						'repeat-x' => __('Repeat X', 'mfn-opts'),
						'repeat-y' => __('Repeat Y', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner > .mcb-wrap-background-overlay:background-repeat_tablet',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'tablet',

					'css_attr' => 'background-repeat',
					'title' => __('Repeat', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'repeat' => __('Repeat', 'mfn-opts'),
						'no-repeat' => __('No repeat', 'mfn-opts'),
						'repeat-x' => __('Repeat X', 'mfn-opts'),
						'repeat-y' => __('Repeat Y', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner > .mcb-wrap-background-overlay:background-repeat_mobile',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'mobile',

					'css_attr' => 'background-repeat',
					'title' => __('Repeat', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'repeat' => __('Repeat', 'mfn-opts'),
						'no-repeat' => __('No repeat', 'mfn-opts'),
						'repeat-x' => __('Repeat X', 'mfn-opts'),
						'repeat-y' => __('Repeat Y', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner > .mcb-wrap-background-overlay:background-position',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_position',
					'type' => 'select',
					'responsive' => 'desktop',

					'css_attr' => 'background-position',
					'title' => __('Position', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'center' => __('Center Center', 'mfn-opts'),
						'center left' => __('Center Left', 'mfn-opts'),
						'center right' => __('Center Right', 'mfn-opts'),
						'top left' => __('Top Left', 'mfn-opts'),
						'top right' => __('Top Right', 'mfn-opts'),
						'top center' => __('Top Center', 'mfn-opts'),
						'bottom left' => __('Bottom Left', 'mfn-opts'),
						'bottom right' => __('Bottom Right', 'mfn-opts'),
						'bottom center' => __('Bottom Center', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner > .mcb-wrap-background-overlay:background-position_laptop',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_position',
					'type' => 'select',
					'responsive' => 'laptop',

					'css_attr' => 'background-position',
					'title' => __('Position', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'center' => __('Center Center', 'mfn-opts'),
						'center left' => __('Center Left', 'mfn-opts'),
						'center right' => __('Center Right', 'mfn-opts'),
						'top left' => __('Top Left', 'mfn-opts'),
						'top right' => __('Top Right', 'mfn-opts'),
						'top center' => __('Top Center', 'mfn-opts'),
						'bottom left' => __('Bottom Left', 'mfn-opts'),
						'bottom right' => __('Bottom Right', 'mfn-opts'),
						'bottom center' => __('Bottom Center', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner > .mcb-wrap-background-overlay:background-position_tablet',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_position',
					'type' => 'select',
					'responsive' => 'tablet',

					'css_attr' => 'background-position',
					'title' => __('Position', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'center' => __('Center Center', 'mfn-opts'),
						'center left' => __('Center Left', 'mfn-opts'),
						'center right' => __('Center Right', 'mfn-opts'),
						'top left' => __('Top Left', 'mfn-opts'),
						'top right' => __('Top Right', 'mfn-opts'),
						'top center' => __('Top Center', 'mfn-opts'),
						'bottom left' => __('Bottom Left', 'mfn-opts'),
						'bottom right' => __('Bottom Right', 'mfn-opts'),
						'bottom center' => __('Bottom Center', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner > .mcb-wrap-background-overlay:background-position_mobile',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_position',
					'type' => 'select',
					'responsive' => 'mobile',

					'css_attr' => 'background-position',
					'title' => __('Position', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'center' => __('Center Center', 'mfn-opts'),
						'center left' => __('Center Left', 'mfn-opts'),
						'center right' => __('Center Right', 'mfn-opts'),
						'top left' => __('Top Left', 'mfn-opts'),
						'top right' => __('Top Right', 'mfn-opts'),
						'top center' => __('Top Center', 'mfn-opts'),
						'bottom left' => __('Bottom Left', 'mfn-opts'),
						'bottom right' => __('Bottom Right', 'mfn-opts'),
						'bottom center' => __('Bottom Center', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner > .mcb-wrap-background-overlay:background-attachment',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'desktop',

					'css_attr' => 'background-attachment',
					'title' => __('Attachment', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'scroll' => __('Scroll', 'mfn-opts'),
						'fixed' => __('Fixed', 'mfn-opts'),
						'parallax' => __('Parallax', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner > .mcb-wrap-background-overlay:background-attachment_laptop',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'laptop',

					'css_attr' => 'background-attachment',
					'title' => __('Attachment', 'mfn-opts'),
					'desc' => __('iOS does <b>not</b> support background-attachment: fixed', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'scroll' => __('Scroll', 'mfn-opts'),
						'fixed' => __('Fixed', 'mfn-opts'),
						'parallax' => __('Parallax', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner > .mcb-wrap-background-overlay:background-attachment_tablet',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'tablet',

					'css_attr' => 'background-attachment',
					'title' => __('Attachment', 'mfn-opts'),
					'desc' => __('iOS does <b>not</b> support background-attachment: fixed', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'scroll' => __('Scroll', 'mfn-opts'),
						'fixed' => __('Fixed', 'mfn-opts'),
						'parallax' => __('Parallax', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner > .mcb-wrap-background-overlay:background-attachment_mobile',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'select',
					'responsive' => 'mobile',

					'css_attr' => 'background-attachment',
					'title' => __('Attachment', 'mfn-opts'),
					'desc' => __('iOS does <b>not</b> support background-attachment: fixed', 'mfn-opts'),
					'options' => [
						'' => __('Default', 'mfn-opts'),
						'scroll' => __('Scroll', 'mfn-opts'),
						'fixed' => __('Fixed', 'mfn-opts'),
						'parallax' => __('Parallax', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner > .mcb-wrap-background-overlay:background-size',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_size',
					'type' => 'select',
					'responsive' => 'desktop',

					'css_attr' => 'background-size',
					'title' => __('Size', 'mfn-opts'),
					'options' => mfna_bg_size()
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner > .mcb-wrap-background-overlay:background-size_laptop',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_size',
					'type' => 'select',
					'responsive' => 'laptop',

					'css_attr' => 'background-size',
					'title' => __('Size', 'mfn-opts'),
					'options' => mfna_bg_size()
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner > .mcb-wrap-background-overlay:background-size_tablet',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_size',
					'type' => 'select',
					'responsive' => 'tablet',

					'css_attr' => 'background-size',
					'title' => __('Size', 'mfn-opts'),
					'options' => mfna_bg_size()
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner > .mcb-wrap-background-overlay:background-size_mobile',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'oldid' => 'bg_size',
					'type' => 'select',
					'responsive' => 'mobile',

					'css_attr' => 'background-size',
					'title' => __('Size', 'mfn-opts'),
					'options' => mfna_bg_size()
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner > .mcb-wrap-background-overlay:filter',
					'condition' => array( 'id' => 'background_overlay_switcher', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'css_filters',
					'label_after' => '<a name="reset-css-filters" href="#" class="mfn-option-btn mfn-option-blank btn-medium reset-big-field reset-css-filters" data-tooltip="Reset"><span class="mfn-icon mfn-icon-reset"></span></a><a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
					'class' => 'mfn-toggled toggle_fields',
					'title' => __('CSS Filters', 'mfn-opts'),
				),

				array(
					'type' => 'html',
					'html' => '</div><div class="mfn-sft mfn-sft-hover">',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner > .mcb-wrap-background-overlay:transition',
					'type' => 'sliderbar',
					'title' => __('Transition duration', 'mfn-opts'),
					'param' => array(
						'min' => '0',
						'max' => '3',
						'step' => '0.1',
						'unit' => 's',
					)
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner|hover > .mcb-wrap-background-overlay:opacity',
					'type' => 'sliderbar',
					'title' => __('Opacity', 'mfn-opts'),
					'param' => array(
						'min' => '0',
						'max' => '1',
						'step' => '0.01',
					),
					//'std' => '1',
				),

				array(
					'id' => 'background_switcher_hover',
					'attr_id' => 'background_switcher_hover',
					'type' => 'switch',
					'class' => 'mfn-fields-switcher',
					'title' => __('Background type', 'mfn-opts'),
					'options' => array(
  					'default' => __('Default', 'mfn-opts'),
  					'gradient' 	=> __('Gradient', 'mfn-opts'),
  				),
					'visual_options' => array(
  					'default' => '',
  					'gradient' 	=> '',
  				),
  				'std' => 'default',
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner|hover > .mcb-wrap-background-overlay:background-color',
					'condition' => array( 'id' => 'background_switcher_hover', 'opt' => 'is', 'val' => 'default' ),
					'type' => 'color',
					'title' => __('Color', 'mfn-opts'),
				),

				array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner|hover > .mcb-wrap-background-overlay:gradient',
  				'condition' => array( 'id' => 'background_switcher_hover', 'opt' => 'is', 'val' => 'gradient' ),
  				'type' => 'gradient',
  				'title' => __('Gradient', 'mfn-opts'),
  				'class' => 'mfn-toggled',
					'css_attr' => 'gradient',
  			),

				array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner|hover > .mcb-wrap-background-overlay:background-image',
  				'condition' => array( 'id' => 'background_switcher_hover', 'opt' => 'is', 'val' => 'default' ),
  				//'oldid' => 'bg_color',
  				'type' => 'upload',
  				'title' => __('Image', 'mfn-opts'),
  				'responsive' => 'desktop',

					'css_attr' => 'background-image',
  			),

				array(
					'type' => 'html',
					'html' => '</div></div>',
				),

				array(
					'type' => 'header',
					'class' => 'toggled_header row-header',
						'title' => __('Border', 'mfn-opts'),
					),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:border-style',
					'attr_id' => 'border_style_wrap',
					'type' => 'select',
					'class' => 'mfn-toggled',
					'title' => __('Border style', 'mfn-opts'),
					'options' => [
						'none' => __('None', 'mfn-opts'),
						'solid' => __('Solid', 'mfn-opts'),
						'dashed' => __('Dashed', 'mfn-opts'),
						'dotted' => __('Dotted', 'mfn-opts'),
						'double' => __('Double', 'mfn-opts'),
					],
				),

				array(
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:border-color',
					'condition' => array( 'id' => 'border_style_wrap', 'opt' => 'isnt', 'val' => 'none' ),
					'type' => 'color',
					'class' => 'mfn-toggled',
					'title' => __('Border color', 'mfn-opts'),
				),

				array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:border-width',
  				'condition' => array( 'id' => 'border_style_wrap', 'opt' => 'isnt', 'val' => 'none' ),
  				'type' => 'dimensions',
  				'title' => __('Border width', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-width',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:border-width_laptop',
  				'condition' => array( 'id' => 'border_style_wrap', 'opt' => 'isnt', 'val' => 'none' ),
  				'type' => 'dimensions',
  				'title' => __('Border width', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-width',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:border-width_tablet',
  				'condition' => array( 'id' => 'border_style_wrap', 'opt' => 'isnt', 'val' => 'none' ),
  				'type' => 'dimensions',
  				'title' => __('Border width', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-width',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:border-width_mobile',
  				'condition' => array( 'id' => 'border_style_wrap', 'opt' => 'isnt', 'val' => 'none' ),
  				'type' => 'dimensions',
  				'title' => __('Border width', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-width',
  			),

				array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:border-radius',
  				'type' => 'dimensions',
  				'title' => __('Border radius', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-radius',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:border-radius_laptop',
  				'type' => 'dimensions',
  				'title' => __('Border radius', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-radius',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:border-radius_tablet',
  				'type' => 'dimensions',
  				'title' => __('Border radius', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-radius',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:border-radius_mobile',
  				'type' => 'dimensions',
  				'title' => __('Border radius', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'border-radius',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:box-shadow',
  				'type' => 'box_shadow',
  				'title' => __('Box shadow', 'mfn-opts'),
					'responsive' => 'desktop',
					'class' => 'mfn-toggled',
					'css_attr' => 'box-shadow',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:box-shadow_laptop',
  				'type' => 'box_shadow',
  				'title' => __('Box shadow', 'mfn-opts'),
					'responsive' => 'laptop',
					'class' => 'mfn-toggled',
					'css_attr' => 'box-shadow',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:box-shadow_tablet',
  				'type' => 'box_shadow',
  				'title' => __('Box shadow', 'mfn-opts'),
					'responsive' => 'tablet',
					'class' => 'mfn-toggled',
					'css_attr' => 'box-shadow',
  			),

  			array(
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement .mcb-wrap-inner:box-shadow_mobile',
  				'type' => 'box_shadow',
  				'title' => __('Box shadow', 'mfn-opts'),
					'responsive' => 'mobile',
					'class' => 'mfn-toggled',
					'css_attr' => 'box-shadow',
  			),

				array(
					'type' => 'header',
					'class' => 'toggled_header row-header',
					'title' => __('Responsive', 'mfn-opts'),
				),

				array(
  				'id' => 'visibility',
  				'iconinfo' => 'https://support.muffingroup.com/video-tutorials/section-responsive-visibility/',
  				'type' => 'switch',
  				'invert' => true,
  				'version' => 'multiple',
  				'class' => 'mfn-toggled',
  				'title' => __('Responsive visibility', 'mfn-opts'),
  				'options' => array(
  					'hide-desktop' => __('Hide on Desktop', 'mfn-opts'),
  					'hide-laptop' => __('Hide on Laptop', 'mfn-opts'),			// 960 +
  					'hide-tablet' => __('Hide on Tablet', 'mfn-opts'),		// 768 - 959
  					'hide-mobile' => __('Hide on Mobile', 'mfn-opts'),			// - 768
  				),
  				'active_tooltip' => array(
  					'hide-desktop' => __('Show on Desktop', 'mfn-opts'),
  					'hide-laptop' => __('Show on Laptop', 'mfn-opts'),
  					'hide-tablet' => __('Show on Tablet', 'mfn-opts'),
  					'hide-mobile' => __('Show on Mobile', 'mfn-opts'),
  				),
  				'visual_options' => array(
  					'hide-desktop' => '<svg id="Layer_1" data-name="Layer 1" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 22 22"><defs><style>.cls-1{fill:none;stroke:#304050;stroke-linecap:round;stroke-linejoin:round;stroke-width:1.5px;}.cls-2{fill:#304050;}</style></defs><path class="cls-1" d="M4.94,5H17.06a.93.93,0,0,1,.94.91v7.18a.93.93,0,0,1-.94.91H4.94A.93.93,0,0,1,4,13.09V5.91A.93.93,0,0,1,4.94,5Z"/><path class="cls-1" d="M7,17h8"/><path class="cls-2" d="M13,15.51a1.58,1.58,0,0,0,.7,1.26L14,17H8l.3-.23A1.58,1.58,0,0,0,9,15.51V14h4Z"/></svg>',			// 960 +
  					'hide-laptop' => '<svg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"viewBox="0 0 22 22" style="enable-background:new 0 0 22 22;" xml:space="preserve"> <style type="text/css"> .st0{fill:none;stroke:currentColor;stroke-width:1.5;stroke-linecap:round;stroke-linejoin:round;} </style> <path class="st0" d="M5,15V6.9C5,6.4,5.4,5.9,5.9,6h10.1c0.5,0,0.9,0.4,0.9,0.9V15"/> <path class="st0" d="M4,17h14"/> </svg>',		// 768 - 959
  					'hide-tablet' => '<svg id="Layer_1" data-name="Layer 1" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 22 22"><defs><style>.cls-1{fill:none;stroke:#304050;stroke-linecap:round;stroke-linejoin:round;stroke-width:1.5px;}</style></defs><path class="cls-1" d="M7,15.83V7.17A1.21,1.21,0,0,1,8.25,6h5.5A1.21,1.21,0,0,1,15,7.17v8.66A1.21,1.21,0,0,1,13.75,17H8.25A1.21,1.21,0,0,1,7,15.83Z"/></svg>',		// 768 - 959
  					'hide-mobile' => '<svg id="Layer_1" data-name="Layer 1" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 22 22"><defs><style>.cls-1{fill:none;stroke:#304050;stroke-linecap:round;stroke-linejoin:round;stroke-width:1.5px;}.cls-2{fill:#304050;}</style></defs><path class="cls-1" d="M8,15.82V8.18A1.08,1.08,0,0,1,8.94,7h4.12A1.08,1.08,0,0,1,14,8.18v7.64A1.08,1.08,0,0,1,13.06,17H8.94A1.08,1.08,0,0,1,8,15.82Z"/><circle class="cls-2" cx="11" cy="14" r="1"/></svg>',			// - 768
  				),
  			),

  			array(
  				'id' => 'reverse_order',
  				'class' => 'mfn-toggled hide_if_ql',
  				'type' => 'switch',
  				'iconinfo' => 'https://support.muffingroup.com/how-to/how-to-reverse-the-order-of-elements-on-mobile/',
  				'title' => __('Order on mobile', 'mfn-opts'),
					'options' => [
						0 => __('Default', 'mfn-opts'),
						1 => __('Reverse rows', 'mfn-opts'),
						2 => __('Reverse columns', 'mfn-opts'),
					],
					'std' => 0,
  			),

  			array(
					'type' => 'subheader',
					'class' => 'mfn-builder-subheader row-header mfn-toggled',
					'title' => __('Custom', 'mfn-opts'),
				),

				array(
					'form' => 'advanced',
  				'id' => 'custom-responsive',
  				'attr_id' => 'custom-responsive',
  				'type' => 'switch',
  				'class' => 'mfn-toggled',
  				'title' => __('Custom visibility', 'mfn-opts'),
  				'options' => array(
  					'hide' => __('Hide under', 'mfn-opts'),
  					'show' => __('Show under', 'mfn-opts'),
  				),
  				'std' => 'hide'
  			),

				array(
  				'form' => 'advanced',
  				'condition' => array( 'id' => 'custom-responsive', 'opt' => 'is', 'val' => 'hide' ),
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement:hide_under_custom',
  				'type' => 'sliderbar',
  				'class' => 'mfn-toggled responsive-custom-visibility',
					'title' => __('Hide under', 'mfn-opts'),
					'param' => array(
						'min' => '400',
						'max' => '1920',
						'step' => '1',
						'unit' => 'px',
					),
  			),

  			array(
  				'form' => 'advanced',
  				'condition' => array( 'id' => 'custom-responsive', 'opt' => 'is', 'val' => 'show' ),
  				'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement:show_under_custom',
  				'type' => 'sliderbar',
  				'title' => __('Show under', 'mfn-opts'),
					'class' => 'mfn-toggled responsive-custom-visibility',
					'param' => array(
						'min' => '400',
						'max' => '1920',
						'step' => '1',
						'unit' => 'px',
					),
  			),

  			// Conditions

				array(
					'form' => 'advanced',
					'type' => 'header',
					'class' => 'toggled_header row-header',
					'title' => __('Conditional logic', 'mfn-opts'),
				),

				array(
					'form' => 'advanced',
  				'id' => 'conditional_logic',
  				'class' => 'mfn-toggled',
  				'type' => 'logic',
  				'title' => __('Conditional logic', 'mfn-opts'),
  			),

				array(
					'type' => 'header',
					'class' => 'toggled_header row-header',
					'title' => __('Custom', 'mfn-opts'),
				),

				array(
  				'id' => 'classes',
  				'oldid' => 'class',
  				'class' => 'mfn-toggled',
  				'type' => 'pills',
  				'title' => __('Classes', 'mfn-opts'),
  			),

  			array(
  				'id' => 'custom_id',
  				'class' => 'mfn-toggled',
  				'type' => 'text',
  				'title' => __('ID', 'mfn-opts'),
  			),

  			array(
  				'id' => 'custom_css',
  				'oldid' => 'style',
  				'class' => 'mfn-toggled',
  				'type' => 'textarea',
  				'title' => __('Additional CSS', 'mfn-opts'),
  			),

			// animation

				array(
					'form' => 'advanced',
					'class' => 'toggled_header row-header',
					'type' => 'header',
					'title' => __('Animation', 'mfn-opts'),
				),

				array(
					'form' => 'advanced',
					'id' => 'animate',
					'attr_id' => 'wrap-animate',
					'type' => 'select',
					'title' => __('Animation', 'mfn-opts'),
					'desc' => __('Entrance animation', 'mfn-opts'),
					'class' => 'mfn-toggled',
					'options' => $this->get_animations(),
				),

				array(
					'form' => 'advanced',
					'id' => 'style:.mcb-section .mcb-wrap-mfnuidelement:animation-delay',
					'condition' => array( 'id' => 'wrap-animate', 'opt' => 'isnt', 'val' => '' ),
					'type' => 'sliderbar',
					'class' => 'mfn-toggled',
					'title' => __('Animation delay', 'mfn-opts'),
					'after' => 'ms',
					'param' => array(
						'min' => '0',
						'max' => '3000',
						'step' => '1',
						'unit' => 'ms'
					),
				),

				// global wraps

			array(
				'id' => 'global_wraps',
				'type' => 'header',
				'class' => 'toggled_header global_wraps row-header mfn-hidden-nested-wrap',
				'title' => __('Global Wraps', 'mfn-opts'),
			),

			array(
				'id' => 'global_wraps_select',
				'attr_id' => 'global_wraps_select',
				'class' => 'mfn-toggled mfn-hidden-nested-wrap',
				'type' => 'select',
				'title' => __('Select template', 'mfn-opts'),
				'std' => '',
				'js_options' => 'global_wraps',
				'php_options' => mfna_templates('wrap'),
			),

			array(
				'type' => 'html',
				'html' => '</div>',
			),


  		);

  	}

    /**
     * SET items and their fields
   	 */

   	private function set_items(){

   		$this->items = array(

   			// ALIAS: Blog Heading ----------------------------------------------------

   			'archive_heading' => array(
   				'type' => 'heading',
   				'title' => __('Title', 'mfn-opts'),
   				'cat' => 'archive',
   			),

   			// ALIAS: Blog image ----------------------------------------------------

   			'archive_image' => array(
   				'type' => 'image',
   				'title' => __('Archive image', 'mfn-opts'),
   				'cat' => 'archive',
   			),

   			// ALIAS: Blog readmore ----------------------------------------------------

   			'archive_read_more' => array(
   				'type' => 'button',
   				'title' => __('Read more', 'mfn-opts'),
   				'cat' => 'archive',
   			),

   			// ALIAS: Blog Desc ----------------------------------------------------

   			'archive_content' => array(
   				'type' => 'plain_text',
   				'title' => __('Description', 'mfn-opts'),
   				'cat' => 'archive',
   			),

   			'archive_blog_categories' => array(
   				'type' => 'tag_cloud',
   				'title' => __('Categories', 'mfn-opts'),
   				'cat' => 'blog',
   			),

   			'archive_portfolio_categories' => array(
   				'type' => 'tag_cloud',
   				'title' => __('Categories', 'mfn-opts'),
   				'cat' => 'portfolio',
   			),

   			// ALIAS: Single Post Heading ----------------------------------------------------

   			'post_heading' => array(
   				'type' => 'heading',
   				'title' => __('Post title', 'mfn-opts'),
   				'cat' => 'single-post',
   			),

   			// ALIAS: Single Post Image ----------------------------------------------------

   			'post_image' => array(
   				'type' => 'image',
   				'title' => __('Featured image', 'mfn-opts'),
   				'cat' => 'single-post',
   			),

   			// ALIAS: Single Post Author -----------------------------------------------------

   			'post_author' => array(
   				'type' => 'icon_box_2',
   				'title' => __('Post author', 'mfn-opts'),
   				'cat' => 'single-post',
   			),

   			// ALIAS: Single Post date -------------------------------------------------------

   			'post_date' => array(
   				'type' => 'heading',
   				'title' => __('Post date', 'mfn-opts'),
   				'cat' => 'single-post',
   			),

   			// ALIAS: Single Post related ----------------------------------------------------

   			'post_blog_related' => array(
   				'type' => 'blog',
   				'title' => __('Related posts', 'mfn-opts'),
   				'cat' => 'single-blog-post',
   			),

   			// ALIAS: Single Portfolio related ----------------------------------------------------

   			'post_portfolio_related' => array(
   				'type' => 'portfolio',
   				'title' => __('Related posts', 'mfn-opts'),
   				'cat' => 'single-portfolio-post',
   			),

   			// ALIAS: Single Post categories -------------------------------------------------

   			'post_blog_categories' => array(
   				'type' => 'tag_cloud',
   				'title' => __('Post categories', 'mfn-opts'),
   				'cat' => 'single-blog-post',
   			),

   			// ALIAS: Single Post categories -------------------------------------------------

   			'post_portfolio_categories' => array(
   				'type' => 'tag_cloud',
   				'title' => __('Post categories', 'mfn-opts'),
   				'cat' => 'single-portfolio-post',
   			),

   			// ALIAS: Single Post categories -------------------------------------------------

   			'post_blog_tags' => array(
   				'type' => 'tag_cloud',
   				'title' => __('Post tags', 'mfn-opts'),
   				'cat' => 'single-blog-post',
   			),

   			// ALIAS: Single Post categories -------------------------------------------------

   			'post_excerpt' => array(
   				'type' => 'plain_text',
   				'title' => __('Excerpt', 'mfn-opts'),
   				'cat' => 'single-post',
   			),

   			// ALIAS: Single Post love it -------------------------------------------------

   			'post_love' => array(
   				'type' => 'icon_box_2',
   				'title' => __('Post love', 'mfn-opts'),
   				'cat' => 'single-post',
   			),

   			// Single Post Comments ----------------------------------------------------

   			'post_comments' => array(
   				'type' => 'post_comments',
   				'title' => __('Post comments', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'single-post',
   				'attr' => array(

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'id' => 'info',
							'class' => 'mfn-alert',
							'type' => 'info',
							'title' => __('This element has no attributes. Please check <b>advanced</b> tab for more customisation options.', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render modalbox-card-style-mfnuidhere">',
						),

						array(
							'id' => 'info',
							'class' => 'mfn-alert',
							'type' => 'info',
							'title' => __('This element has no attributes. Please check <b>advanced</b> tab for more customisation options.', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

					),
   			),

   			// Single Post Content ----------------------------------------------------

   			'post_content' => array(
   				'type' => 'post_content',
   				'title' => __('Post content', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'single-post',
   				'attr' => array(

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'type' => 'header',
							'title' => __('Content', 'mfn-opts'),
						),

						array(
							'id' => 'content',
							're_render' => true,
							'type' => 'select',
							'title' => __('Source', 'mfn-opts'),
							'options' => array(
								'' => __('Default', 'mfn-opts'),
								'mfn' => __('BeBuilder', 'mfn-opts'),
							),
							'std' => '',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render modalbox-card-style-mfnuidhere">',
						),

						array(
							'id' => 'info',
							'class' => 'mfn-alert',
							'type' => 'info',
							'title' => __('This element has no attributes. Please check <b>advanced</b> tab for more customisation options.', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

					),
   			),

   			// Menu ----------------------------------------------------

   			'sidemenu_menu' => array(
   				'type' => 'sidemenu_menu',
   				'title' => __('Menu', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'sidemenu',
   				'attr' => array(
   				//'fields' => array(

						/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'type' => 'header',
							'title' => __('Menu', 'mfn-opts'),
						),

   					array(
   						'id' => 'tabs',
   						'type' => 'tabs',
   						'title' => __('Menu', 'mfn-opts'),
   						'desc' => __('Add multiple menus easily in tabs. You can add one menu in single tab as well.', 'mfn-opts'),
							'options' => [
								'title' => [
									'input',
									__('Title', 'mfn-opts'),
									__('Menu title', 'mfn-opts'),
								],
								'menu' => [
									'select',
									__('Menu to display', 'mfn-opts'),
									'',
									'menus',
								],
							],
							'std' => [
								0 => [
									'title' => __('Menu 1', 'mfn-opts'),
									'menu' => '',
								]
							],
							'preview' => 'tabs',
							'primary' => 'title', // default
   					),

   					array(
   						'type' => 'header',
							'title' => __('Icon', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-menu a .menu-icon:display',
							'attr_id' => 'icon_sm_visibility',
							'type' => 'switch',
							'title' => __('Icon visibility', 'mfn-opts'),
							'options' => array(
								'block' => 'Show',
								'none' 	=> 'Hide',
							),
							'std' => 'block',
						),

   					array(
   						'type' => 'header',
							'title' => __('Submenu', 'mfn-opts'),
						),

   					array(
							'id' => 'submenu',
							'attr_id' => 'submenu',
							're_render' => true,
							'type' => 'switch',
							'title' => __('Submenu', 'mfn-opts'),
							'options' => array(
								'on' => 'Show',
								'off' => 'Hide',
							),
							'std' => 'on',
						),

						array(
							'id' => 'submenu_on',
							'attr_id' => 'submenu_on',
							'condition' => array( 'id' => 'submenu', 'opt' => 'is', 'val' => 'on' ),
							're_render' => true,
							'type' => 'select',
							'title' => __('Submenu visibility', 'mfn-opts'),
							'options' => array(
								'visible' => 'Visible',
								'toggled' => 'Toggled',
								'replace' => 'Replace current',
							),
							'std' => 'visible',
						),

						array(
							'id' => 'submenu_icon_display',
							'condition' => array( 'id' => 'submenu', 'opt' => 'is', 'val' => 'on' ),
							're_render' => true,
							'type' => 'switch',
							'title' => __('Submenu icon visibility', 'mfn-opts'),
							'options' => array(
								'on' => 'Show',
								'off' => 'Hide',
							),
							'std' => 'on',
						),

						array(
							'id' => 'submenu_icon',
							'condition' => array( 'id' => 'submenu', 'opt' => 'is', 'val' => 'on' ),
							'type' => 'icon',
							'title' => __('Submenu icon', 'mfn-opts'),
							'preview' => 'icon',
							'std' => 'fas fa-arrow-down'
						),

						array(
   						'id' => 'submenu_icon_animation',
   						'type' => 'select',
   						'condition' => array( 'id' => 'submenu', 'opt' => 'is', 'val' => 'on' ),
   						'title' => __('Icon Animation', 'mfn-opts'),
							'options' => array(
								'' => __('None', 'mfn-opts'),
								'rotate' => 'Rotate',
								'zoom' => 'Zoom'
							),
   					),

   					array(
   						'type' => 'header',
							'title' => __('Desc', 'mfn-opts'),
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-menu a .menu-desc:display',
							'type' => 'switch',
							'title' => __('Desc visibility', 'mfn-opts'),
							'options' => array(
								'block' => 'Show',
								'none' 	=> 'Hide',
							),
							'std' => 'block',
						),



   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render modalbox-card-style-mfnuidhere">',
						),

   					array(
							'class' => 'toggled_header row-header',
							'type' => 'header',
   						'title' => __('Menu', 'mfn-opts'),
   					),




   					array(
   						'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Container', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper:border-style',
							'attr_id' => 'border_sidemenu_tabs_content',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper:border-width',
		  				'condition' => array( 'id' => 'border_sidemenu_tabs_content', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper:border-width_laptop',
		  				'condition' => array( 'id' => 'border_sidemenu_tabs_content', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper:border-width_tablet',
		  				'condition' => array( 'id' => 'border_sidemenu_tabs_content', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper:border-width_mobile',
		  				'condition' => array( 'id' => 'border_sidemenu_tabs_content', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper:border-color',
							'condition' => array( 'id' => 'border_sidemenu_tabs_content', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),



   					array(
   						'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Items', 'mfn-opts'),
					),

					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li a.mfn-menu-link:justify-content',
							'type' => 'switch',
							'responsive' => 'desktop',
							'title' => __('Alignment', 'mfn-opts'),
							'options' => array(
								'' => 'Left',
								'center' => 'Center',
								'flex-end' => 'Right',
							),
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li a.mfn-menu-link:justify-content_tablet',
							'type' => 'switch',
							'responsive' => 'tablet',
							'title' => __('Alignment', 'mfn-opts'),
							'options' => array(
								'' => __('Inherited', 'mfn-opts'),
								'flex-start' => 'Left',
								'center' => 'Center',
								'flex-end' => 'Right',
							),
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li a.mfn-menu-link:justify-content_mobile',
							'type' => 'switch',
							'responsive' => 'mobile',
							'title' => __('Alignment', 'mfn-opts'),
							'options' => array(
								'' => __('Inherited', 'mfn-opts'),
								'flex-start' => 'Left',
								'center' => 'Center',
								'flex-end' => 'Right',
							),
							'std' => '',
						),










   					array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li a.mfn-menu-link:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li a.mfn-menu-link:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li a.mfn-menu-link:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li a.mfn-menu-link:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li a.mfn-menu-link:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li a.mfn-menu-link:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li a.mfn-menu-link:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li a.mfn-menu-link:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li a.mfn-menu-link:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),


						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li a.mfn-menu-link:border-style',
							'attr_id' => 'border_sidemenu_tabs_ulli',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li a.mfn-menu-link:border-width',
		  				'condition' => array( 'id' => 'border_sidemenu_tabs_ulli', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li a.mfn-menu-link:border-width_laptop',
		  				'condition' => array( 'id' => 'border_sidemenu_tabs_ulli', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li a.mfn-menu-link:border-width_tablet',
		  				'condition' => array( 'id' => 'border_sidemenu_tabs_ulli', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li a.mfn-menu-link:border-width_mobile',
		  				'condition' => array( 'id' => 'border_sidemenu_tabs_ulli', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li a.mfn-menu-link:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li a.mfn-menu-link:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li a.mfn-menu-link:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li a.mfn-menu-link:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),


						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li a.mfn-menu-link:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li a.mfn-menu-link:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li a.mfn-menu-link:border-color',
							'condition' => array( 'id' => 'border_sidemenu_tabs_ulli', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li a.mfn-menu-link|hover:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li a.mfn-menu-link|hover:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li a.mfn-menu-link|hover:border-color',
							'condition' => array( 'id' => 'border_sidemenu_tabs_ulli', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li.current-menu-item > a.mfn-menu-link:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li.current-menu-item > a.mfn-menu-link:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li.current-menu-item > a.mfn-menu-link:border-color',
							'condition' => array( 'id' => 'border_sidemenu_tabs_ulli', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),














						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Icon', 'mfn-opts'),
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-menu li:--mfn-sidemenu-icon-gap',
							'type' => 'sliderbar',
							//'condition' => array( 'id' => 'icon_visibility', 'opt' => 'is', 'val' => 'block' ),
							'responsive' => 'desktop',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							),
							'std' => '5px'
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-menu li:--mfn-sidemenu-icon-gap_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-menu li:--mfn-sidemenu-icon-gap_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-menu li:--mfn-sidemenu-icon-gap_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-menu li:--mfn-sidemenu-icon-width',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-menu li:--mfn-sidemenu-icon-width_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-menu li:--mfn-sidemenu-icon-width_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-menu li:--mfn-sidemenu-icon-width_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-menu .menu-icon i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-menu a|hover .menu-icon i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-menu li.current-menu-item > a .menu-icon i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Submenu icon', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-menu li:--mfn-sidemenu-submenu-icon-width',
							//'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-menu li:--mfn-sidemenu-submenu-icon-width_laptop',
							//'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-menu li:--mfn-sidemenu-submenu-icon-width_tablet',
							//'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-menu li:--mfn-sidemenu-submenu-icon-width_mobile',
							//'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-menu li:--mfn-sidemenu-submenu-icon-size',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-menu li:--mfn-sidemenu-submenu-icon-size_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-menu li:--mfn-sidemenu-submenu-icon-size_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-menu li:--mfn-sidemenu-submenu-icon-size_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-menu li .outer-menu-sub i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-menu li .outer-menu-sub:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-menu li|hover > .outer-menu-sub i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-menu li|hover > .outer-menu-sub:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-menu li.current-menu-item > .outer-menu-sub i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-menu li.current-menu-item > .outer-menu-sub:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'class' => 'toggled_header row-header',
							'condition' => array( 'id' => 'submenu_on', 'opt' => 'isnt', 'val' => 'replace' ),
							'type' => 'header',
   						'title' => __('Submenu', 'mfn-opts'),
   					),

   					array(
   						'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Container', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper li ul:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper li ul:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper li ul:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper li ul:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper li ul:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper li ul:border-style',
							'attr_id' => 'border_sidemenu_tabs_content',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper li ul:border-width',
		  				'condition' => array( 'id' => 'border_sidemenu_tabs_content', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper li ul:border-width_laptop',
		  				'condition' => array( 'id' => 'border_sidemenu_tabs_content', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper li ul:border-width_tablet',
		  				'condition' => array( 'id' => 'border_sidemenu_tabs_content', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper li ul:border-width_mobile',
		  				'condition' => array( 'id' => 'border_sidemenu_tabs_content', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper li ul:border-color',
							'condition' => array( 'id' => 'border_sidemenu_tabs_content', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper li ul:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper li ul:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper li ul:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-wrapper li ul:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),



   					array(
   						'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Items', 'mfn-opts'),
						),


   					array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li ul li a.mfn-menu-link:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li ul li a.mfn-menu-link:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li ul li a.mfn-menu-link:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li ul li a.mfn-menu-link:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li ul li a.mfn-menu-link:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li ul li a.mfn-menu-link:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li ul li a.mfn-menu-link:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li ul li a.mfn-menu-link:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li ul li a.mfn-menu-link:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),


						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li ul li a.mfn-menu-link:border-style',
							'attr_id' => 'border_sidemenu_tabs_ulli',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li ul li a.mfn-menu-link:border-width',
		  				'condition' => array( 'id' => 'border_sidemenu_tabs_ulli', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li ul li a.mfn-menu-link:border-width_laptop',
		  				'condition' => array( 'id' => 'border_sidemenu_tabs_ulli', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li ul li a.mfn-menu-link:border-width_tablet',
		  				'condition' => array( 'id' => 'border_sidemenu_tabs_ulli', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li ul li a.mfn-menu-link:border-width_mobile',
		  				'condition' => array( 'id' => 'border_sidemenu_tabs_ulli', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li ul li a.mfn-menu-link:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li ul li a.mfn-menu-link:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li ul li a.mfn-menu-link:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li ul li a.mfn-menu-link:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),


						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li ul li a.mfn-menu-link:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li ul li a.mfn-menu-link:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li ul li a.mfn-menu-link:border-color',
							'condition' => array( 'id' => 'border_sidemenu_tabs_ulli', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li ul li a.mfn-menu-link|hover:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li ul li a.mfn-menu-link|hover:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li ul li a.mfn-menu-link|hover:border-color',
							'condition' => array( 'id' => 'border_sidemenu_tabs_ulli', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li ul li.current-menu-item > a.mfn-menu-link:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li ul li.current-menu-item > a.mfn-menu-link:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.mfn-sidemenu-menu li ul li.current-menu-item > a.mfn-menu-link:border-color',
							'condition' => array( 'id' => 'border_sidemenu_tabs_ulli', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),


   					array(
							'class' => 'toggled_header row-header',
							'type' => 'header',
   						'title' => __('Tabs', 'mfn-opts'),
   					),


   					array(
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'type' => 'subheader',
   						'title' => __('Nav', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav:--mfn-sidemenu-menu-tabber-padding',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Padding', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							),
							'std' => '5px',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav:--mfn-sidemenu-menu-tabber-padding_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Padding', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav:--mfn-sidemenu-menu-tabber-padding_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Padding', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav:--mfn-sidemenu-menu-tabber-padding_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Padding', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav:margin-bottom',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Margin bottom', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							),
							'std' => '5px',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav:margin-bottom_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Margin bottom', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav:margin-bottom_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Margin bottom', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav:margin-bottom_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Margin bottom', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav:--mfn-sidemenu-menu-tabber-bg',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav:border-style',
							'attr_id' => 'border_sidemenu_tabs_ul',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav:border-width',
		  				'condition' => array( 'id' => 'border_sidemenu_tabs_ul', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav:border-width_laptop',
		  				'condition' => array( 'id' => 'border_sidemenu_tabs_ul', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav:border-width_tablet',
		  				'condition' => array( 'id' => 'border_sidemenu_tabs_ul', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav:border-width_mobile',
		  				'condition' => array( 'id' => 'border_sidemenu_tabs_ul', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav:border-color',
							'condition' => array( 'id' => 'border_sidemenu_tabs_ul', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

   					array(
   						'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Nav items', 'mfn-opts'),
						),


   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav li:--mfn-sidemenu-menu-tabber-tab-spacing',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							),
							'std' => '5px',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav li:--mfn-sidemenu-menu-tabber-tab-spacing_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav li:--mfn-sidemenu-menu-tabber-tab-spacing_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav li:--mfn-sidemenu-menu-tabber-tab-spacing_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav li a:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav li a:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav li a:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav li a:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav li a:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),


						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav li a:border-style',
							'attr_id' => 'border_sidemenu_tabs_ulli',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav li a:border-width',
		  				'condition' => array( 'id' => 'border_sidemenu_tabs_ulli', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav li a:border-width_laptop',
		  				'condition' => array( 'id' => 'border_sidemenu_tabs_ulli', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav li a:border-width_tablet',
		  				'condition' => array( 'id' => 'border_sidemenu_tabs_ulli', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav li a:border-width_mobile',
		  				'condition' => array( 'id' => 'border_sidemenu_tabs_ulli', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav li a:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav li a:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav li a:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav li a:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),


						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav li a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav li a:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav li a:border-color',
							'condition' => array( 'id' => 'border_sidemenu_tabs_ulli', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav li a|hover:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav li a|hover:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav li a|hover:border-color',
							'condition' => array( 'id' => 'border_sidemenu_tabs_ulli', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav li.active a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav li.active a:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-tabs-wrapper ul.mfn-menu-tabs-nav li.active a:border-color',
							'condition' => array( 'id' => 'border_sidemenu_tabs_ulli', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'class' => 'toggled_header row-header',
							'type' => 'header',
							'condition' => array( 'id' => 'submenu_on', 'opt' => 'is', 'val' => 'replace' ),
   						'title' => __('Breadcrumbs', 'mfn-opts'),
   					),


   					array(
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'condition' => array( 'id' => 'submenu_on', 'opt' => 'is', 'val' => 'replace' ),
							'type' => 'subheader',
   						'title' => __('Links', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-breadcrumbs a:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-breadcrumbs a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-breadcrumbs a|hover:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

   					array(
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'type' => 'subheader',
   						'title' => __('Separator', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-sidemenu-breadcrumbs .mfn-sidemenu-breadcrumbs-separator:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

						// advanced here

   				),
   			),



   			// Popup Exit popup exit_popup exit-button ----------------------------------------------------

   			'popup_exit' => array(
   				'type' => 'popup_exit',
   				'title' => __('Exit link', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'popup',
   				'fields' => array(

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
   						'id' => 'label',
   						'edit_tag' => '.exit-mfn-popup | span.button_label',
   						'edit_position' => 'append',
   						'type' => 'text',
   						'title' => __('Label', 'mfn-opts'),
   						'std' => __('Close popup', 'mfn-opts'),
   					),

						array(
							'id' => 'icon',
							'type' => 'icon',
							'title' => __('Icon', 'mfn-opts'),
							'preview' => 'icon',
							'std' => 'icon-cancel'
						),

						array(
   						'id' => 'image',
   						'type' => 'upload',
   						'title' => __('Image', 'mfn-opts'),
   						'desc' => __('Overrides icon', 'mfn-opts'),
							'preview' => 'image',
   					),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render modalbox-card-style-mfnuidhere">',
						),

						array(
							'class' => 'toggled_header row-header',
							'type' => 'header',
   						'title' => __('Link', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:text-align',
							'type' => 'switch',
							'class' => 'mfn-toggled',
							'title' => __('Alignment', 'mfn-opts'),
							'options' => array(
								'left' => 'Left',
								'center' => 'Center',
								'right' => 'Right',
							),
							'std' => 'left',
						),


						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner a.exit-mfn-popup:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner a.exit-mfn-popup:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner a.exit-mfn-popup:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner a.exit-mfn-popup:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner a.exit-mfn-popup:--mfn-exitbutton-item-size',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner a.exit-mfn-popup:--mfn-exitbutton-item-size_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner a.exit-mfn-popup:--mfn-exitbutton-item-size_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner a.exit-mfn-popup:--mfn-exitbutton-item-size_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner a.exit-mfn-popup:border-style',
							'attr_id' => 'border_exit_popup',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner a.exit-mfn-popup:border-width',
		  				'condition' => array( 'id' => 'border_exit_popup', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner a.exit-mfn-popup:border-width_laptop',
		  				'condition' => array( 'id' => 'border_exit_popup', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner a.exit-mfn-popup:border-width_tablet',
		  				'condition' => array( 'id' => 'border_exit_popup', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner a.exit-mfn-popup:border-width_mobile',
		  				'condition' => array( 'id' => 'border_exit_popup', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner a.exit-mfn-popup:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner a.exit-mfn-popup:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner a.exit-mfn-popup:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner a.exit-mfn-popup:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),






						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner a.exit-mfn-popup:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner a.exit-mfn-popup:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner a.exit-mfn-popup:border-color',
							'condition' => array( 'id' => 'border_exit_popup', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner a.exit-mfn-popup|hover:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner a.exit-mfn-popup|hover:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner a.exit-mfn-popup|hover:border-color',
							'condition' => array( 'id' => 'border_exit_popup', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

						// advanced here

   				),
   			),

   			// Footer Logo ----------------------------------------------------

   			'footer_logo' => array(
   				'type' => 'footer_logo',
   				'title' => __('Logo', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'footer',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
   						'id' => 'image',
   						'type' => 'upload',
   						'title' => __('Logo', 'mfn-opts'),
   						'desc' => __('Recommended svg', 'mfn-opts'),
							'std' => mfn_opts_get('logo-img') ? mfn_opts_get('logo-img') : $this->get_placeholder(),
							'preview' => 'image',
   					),

   					array(
							'id' => 'link',
							'type' => 'text',
							'title' => __('Link', 'mfn-opts'),
							'std' => '/',
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'id' => 'info',
							'type' => 'info',
							'class' => 'mfn-alert',
							'title' => __('This element has no attributes. Please check <b>advanced</b> tab for more customisation options.', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

						// advanced here

   				),
   			),

				// Footer Menu ----------------------------------------------------

   			'footer_menu' => array(
   				'type' => 'footer_menu',
   				'title' => __('Menu', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'footer',
   				'attr' => array(
   				//'fields' => array(

						/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'type' => 'header',
							'title' => __('Menu', 'mfn-opts'),
						),

						array(
   						'id' => 'menu_display',
   						'type' => 'select',
   						're_render' => true,
   						'title' => __('Menu to display', 'mfn-opts'),
							//'options' => mfna_menu(),
							'js_options' => 'menus',
   					),

   					array(
   						'id' => 'menu_style',
   						'attr_id' => 'menu_style',
   						'type' => 'select',
   						're_render' => true,
   						'title' => __('Style', 'mfn-opts'),
							'options' => array(
								'vertical' => __('Vertical', 'mfn-opts'),
								'horizontal' => __('Horizontal', 'mfn-opts'),
							),
							'std' => 'vertical'
   					),


   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner ul.mfn-footer-menu-style-vertical:text-align',
							'condition' => array( 'id' => 'menu_style', 'opt' => 'is', 'val' => 'vertical' ),
							'attr_id' => 'menu-items-alignment',
							'type' => 'select',
							'responsive' => 'desktop',

							'title' => __('Alignment', 'mfn-opts'),
							'options' => array(
								'left' => 'Left',
								'center' => 'Center',
								'right' => 'Right',
							),
							'std' => 'center',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner ul.mfn-footer-menu-style-vertical:text-align_laptop',
							'condition' => array( 'id' => 'menu_style', 'opt' => 'is', 'val' => 'vertical' ),
							'attr_id' => 'menu-items-alignment-laptop',
							'type' => 'select',
							'responsive' => 'laptop',

							'title' => __('Alignment', 'mfn-opts'),
							'options' => array(
								'left' => 'Left',
								'center' => 'Center',
								'right' => 'Right',
							),
							'std' => 'center',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner ul.mfn-footer-menu-style-vertical:text-align_tablet',
							'condition' => array( 'id' => 'menu_style', 'opt' => 'is', 'val' => 'vertical' ),
							'attr_id' => 'menu-items-alignment-tablet',
							'type' => 'select',
							'responsive' => 'tablet',

							'title' => __('Alignment', 'mfn-opts'),
							'options' => array(
								'left' => 'Left',
								'center' => 'Center',
								'right' => 'Right',
							),
							'std' => 'center',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner ul.mfn-footer-menu-style-vertical:text-align_mobile',
							'condition' => array( 'id' => 'menu_style', 'opt' => 'is', 'val' => 'vertical' ),
							'attr_id' => 'menu-items-alignment-mobile',
							'type' => 'select',
							'responsive' => 'mobile',

							'title' => __('Alignment', 'mfn-opts'),
							'options' => array(
								'left' => 'Left',
								'center' => 'Center',
								'right' => 'Right',
							),
							'std' => 'center',
						),


   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner ul.mfn-footer-menu-style-horizontal:justify-content',
							'condition' => array( 'id' => 'menu_style', 'opt' => 'is', 'val' => 'horizontal' ),
							'attr_id' => 'menu-items-alignment',
							'type' => 'select',
							'responsive' => 'desktop',

							'title' => __('Alignment', 'mfn-opts'),
							'options' => array(
								'flex-start' => 'Left',
								'center' => 'Center',
								'flex-end' => 'Right',
								'space-between' => 'Space between',
								'space-around' => 'Space around',
								'space-evenly' => 'Space evenly',
							),
							'std' => 'center',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner ul.mfn-footer-menu-style-horizontal:justify-content_laptop',
							'condition' => array( 'id' => 'menu_style', 'opt' => 'is', 'val' => 'horizontal' ),
							'attr_id' => 'menu-items-alignment-laptop',
							'type' => 'select',
							'responsive' => 'laptop',

							'title' => __('Alignment', 'mfn-opts'),
							'options' => array(
								'flex-start' => 'Left',
								'center' => 'Center',
								'flex-end' => 'Right',
								'space-between' => 'Space between',
								'space-around' => 'Space around',
								'space-evenly' => 'Space evenly',
							),
							'std' => 'center',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner ul.mfn-footer-menu-style-horizontal:justify-content_tablet',
							'condition' => array( 'id' => 'menu_style', 'opt' => 'is', 'val' => 'horizontal' ),
							'attr_id' => 'menu-items-alignment-tablet',
							'type' => 'select',
							'responsive' => 'tablet',

							'title' => __('Alignment', 'mfn-opts'),
							'options' => array(
								'flex-start' => 'Left',
								'center' => 'Center',
								'flex-end' => 'Right',
								'space-between' => 'Space between',
								'space-around' => 'Space around',
								'space-evenly' => 'Space evenly',
							),
							'std' => 'center',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner ul.mfn-footer-menu-style-horizontal:justify-content_mobile',
							'condition' => array( 'id' => 'menu_style', 'opt' => 'is', 'val' => 'horizontal' ),
							'attr_id' => 'menu-items-alignment-mobile',
							'type' => 'select',
							'responsive' => 'mobile',

							'title' => __('Alignment', 'mfn-opts'),
							'options' => array(
								'flex-start' => 'Left',
								'center' => 'Center',
								'flex-end' => 'Right',
								'space-between' => 'Space between',
								'space-around' => 'Space around',
								'space-evenly' => 'Space evenly',
							),
							'std' => 'center',
						),



						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Menu', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul a:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul a:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul a:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul a:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul a:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul a|hover:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul .current-menu-item > a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

					),
   			),

   			// Mega menu Menu ----------------------------------------------------

   			'megamenu_menu' => array(
   				'type' => 'megamenu_menu',
   				'title' => __('Menu', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'megamenu',
   				'attr' => array(
   				//'fields' => array(

						/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'type' => 'header',
							'title' => __('Menu', 'mfn-opts'),
						),

						array(
   						'id' => 'menu_display',
   						'type' => 'select',
   						're_render' => true,
   						'title' => __('Menu to display', 'mfn-opts'),
							//'options' => mfna_menu(),
							'js_options' => 'menus',
   					),

   					array(
   						'id' => 'menu_style',
   						'attr_id' => 'menu_style',
   						'type' => 'select',
   						're_render' => true,
   						'title' => __('Style', 'mfn-opts'),
							'options' => array(
								'vertical' => __('Vertical', 'mfn-opts'),
								'horizontal' => __('Horizontal', 'mfn-opts'),
							),
							'std' => 'vertical'
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu.mfn-mm-menu-horizontal:justify-content',
							'attr_id' => 'menu-items-alignment',
							'condition' => array( 'id' => 'menu_style', 'opt' => 'is', 'val' => 'horizontal' ),
							'type' => 'select',
							'responsive' => 'desktop',

							'title' => __('Alignment', 'mfn-opts'),
							'options' => array(
								'flex-start' => 'Left',
								'center' => 'Center',
								'flex-end' => 'Right',
								'space-between' => 'Space between',
								'space-around' => 'Space around',
								'space-evenly' => 'Space evenly',
							),
							'std' => 'flex-start',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu.mfn-mm-menu-horizontal:justify-content_laptop',
							'attr_id' => 'menu-items-alignment-laptop',
							'condition' => array( 'id' => 'menu_style', 'opt' => 'is', 'val' => 'horizontal' ),
							'type' => 'select',
							'responsive' => 'laptop',

							'title' => __('Alignment', 'mfn-opts'),
							'options' => array(
								'flex-start' => 'Left',
								'center' => 'Center',
								'flex-end' => 'Right',
								'space-between' => 'Space between',
								'space-around' => 'Space around',
								'space-evenly' => 'Space evenly',
							),
							'std' => 'flex-start',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu.mfn-mm-menu-horizontal:justify-content_tablet',
							'attr_id' => 'menu-items-alignment-tablet',
							'condition' => array( 'id' => 'menu_style', 'opt' => 'is', 'val' => 'horizontal' ),
							'type' => 'select',
							'responsive' => 'tablet',

							'title' => __('Alignment', 'mfn-opts'),
							'options' => array(
								'flex-start' => 'Left',
								'center' => 'Center',
								'flex-end' => 'Right',
								'space-between' => 'Space between',
								'space-around' => 'Space around',
								'space-evenly' => 'Space evenly',
							),
							'std' => 'flex-start',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu.mfn-mm-menu-horizontal:justify-content_mobile',
							'attr_id' => 'menu-items-alignment-mobile',
							'condition' => array( 'id' => 'menu_style', 'opt' => 'is', 'val' => 'horizontal' ),
							'type' => 'select',
							'responsive' => 'mobile',

							'title' => __('Alignment', 'mfn-opts'),
							'options' => array(
								'flex-start' => 'Left',
								'center' => 'Center',
								'flex-end' => 'Right',
								'space-between' => 'Space between',
								'space-around' => 'Space around',
								'space-evenly' => 'Space evenly',
							),
							'std' => 'flex-start',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu.mfn-mm-menu-horizontal > li:flex-grow',
							'condition' => array( 'id' => 'menu_style', 'opt' => 'is', 'val' => 'horizontal' ),
							'type' => 'select',
							'responsive' => 'desktop',

							'title' => __('Fit container', 'mfn-opts'),
							'options' => [
								'unset' => __('Default', 'mfn-opts'),
								'1' => __('Fit', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu.mfn-mm-menu-horizontal > li:flex-grow_laptop',
							'condition' => array( 'id' => 'menu_style', 'opt' => 'is', 'val' => 'horizontal' ),
							'type' => 'select',
							'responsive' => 'laptop',

							'title' => __('Fit container', 'mfn-opts'),
							'options' => [
								'unset' => __('Default', 'mfn-opts'),
								'1' => __('Fit', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu.mfn-mm-menu-horizontal > li:flex-grow_tablet',
							'condition' => array( 'id' => 'menu_style', 'opt' => 'is', 'val' => 'horizontal' ),
							'type' => 'select',
							'responsive' => 'tablet',

							'title' => __('Fit container', 'mfn-opts'),
							'options' => [
								'unset' => __('Default', 'mfn-opts'),
								'1' => __('Fit', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu.mfn-mm-menu-horizontal > li:flex-grow_mobile',
							'condition' => array( 'id' => 'menu_style', 'opt' => 'is', 'val' => 'horizontal' ),
							'type' => 'select',
							'responsive' => 'mobile',

							'title' => __('Fit container', 'mfn-opts'),
							'options' => [
								'unset' => __('Default', 'mfn-opts'),
								'1' => __('Fit', 'mfn-opts'),
							],
						),

						array(
							'id' => 'submenu_on',
							'attr_id' => 'submenu_on',
							're_render' => true,
							'condition' => array( 'id' => 'menu_style', 'opt' => 'is', 'val' => 'vertical' ),
							'type' => 'switch',
							'title' => __('Submenu visibility', 'mfn-opts'),
							'options' => array(
								'visible' => 'Visible',
								'toggled' => 'Toggled',
							),
							'std' => 'visible',
						),

						array(
							'id' => 'submenu_hori_on',
							're_render' => true,
							'condition' => array( 'id' => 'menu_style', 'opt' => 'is', 'val' => 'horizontal' ),
							'type' => 'switch',
							'title' => __('Submenu visibility', 'mfn-opts'),
							'options' => array(
								'hover' => 'on Hover',
								'click' => 'on Click',
							),
							'std' => 'hover',
						),

   					array(
   						'type' => 'header',
							'title' => __('Icon', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu a .menu-icon:display',
							'attr_id' => 'icon_mm_visibility',
							'type' => 'switch',
							'title' => __('Icon visibility', 'mfn-opts'),
							'options' => array(
								'block' => 'Show',
								'none' 	=> 'Hide',
							),
							'std' => 'block',
						),

						array(
   						'id' => 'icon_align',
   						//'condition' => array( 'id' => 'icon_mm_visibility', 'opt' => 'is', 'val' => 'block' ),
   						'type' => 'switch',
   						'title' => __('Icon align', 'mfn-opts'),
							'options' => array(
								'left' => 'Left',
								'top' => 'Top',
								'right' => 'Right'
							),
							'std' => 'left'
   					),

						array(
   						'id' => 'icon_animation',
   						//'condition' => array( 'id' => 'icon_mm_visibility', 'opt' => 'is', 'val' => 'block' ),
   						'type' => 'select',
   						'title' => __('Icon animation', 'mfn-opts'),
							'options' => array(
								'' => __('None', 'mfn-opts'),
								'rotate' => 'Rotate',
								'zoom' => 'Zoom'
							),
   					),

   					array(
   						'type' => 'header',
							'title' => __('Submenu', 'mfn-opts'),
						),

   					array(
							'id' => 'submenu',
							'attr_id' => 'submenu',
							're_render' => true,
							'type' => 'switch',
							'title' => __('Submenu', 'mfn-opts'),
							'options' => array(
								'on' => 'Show',
								'off' => 'Hide',
							),
							'std' => 'on',
						),

						array(
   						'id' => 'submenu_animation',
   						'condition' => array( 'id' => 'submenu', 'opt' => 'is', 'val' => 'on' ),
   						'type' => 'select',
   						'title' => __('Submenu Animation', 'mfn-opts'),
							'options' => array(
								'' => __('None', 'mfn-opts'),
								'fade-in' => 'Fade In',
								'fade-up' => 'Fade In Up'
							),
   					),

						array(
							'id' => 'submenu_icon',
							'condition' => array( 'id' => 'submenu', 'opt' => 'is', 'val' => 'on' ),
							'type' => 'icon',
							'title' => __('Submenu icon', 'mfn-opts'),
							'preview' => 'icon',
							'std' => 'fas fa-arrow-down'
						),

						array(
   						'id' => 'submenu_icon_animation',
   						'type' => 'select',
   						'condition' => array( 'id' => 'submenu', 'opt' => 'is', 'val' => 'on' ),
   						'title' => __('Icon Animation', 'mfn-opts'),
							'options' => array(
								'' => __('None', 'mfn-opts'),
								'rotate' => 'Rotate',
								'zoom' => 'Zoom'
							),
   					),

   					array(
   						'type' => 'header',
							'title' => __('Desc', 'mfn-opts'),
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu a .menu-desc:display',
							'type' => 'switch',
							'title' => __('Desc visibility', 'mfn-opts'),
							'options' => array(
								'block' => 'Show',
								'none' 	=> 'Hide',
							),
							'std' => 'block',
						),

						array(
							'type' => 'header',
							'title' => __('Decoration icon', 'mfn-opts'),
						),

						array(
							'id' => 'decoration_icon',
							'type' => 'icon',
							'title' => __('Decoration icon', 'mfn-opts'),
							'preview' => 'icon',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Menu', 'mfn-opts'),
   					),

   					array(
   						'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Items', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu a:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu a:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu a:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu a:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu a:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li > a:border-style',
							'attr_id' => 'border_style_header_mm_menu_link',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li > a:border-width',
		  				'condition' => array( 'id' => 'border_style_header_mm_menu_link', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li > a:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_header_mm_menu_link', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li > a:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_header_mm_menu_link', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li > a:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_header_mm_menu_link', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li > a:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li > a:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li > a:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li > a:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu a:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li > a:border-color',
							'condition' => array( 'id' => 'border_style_header_mm_menu_link', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu a|hover:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu a|hover:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li > a|hover:border-color',
							'condition' => array( 'id' => 'border_style_header_mm_menu_link', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu .current-menu-item > a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu .current-menu-item > a:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu .current-menu-item > li > a:border-color',
							'condition' => array( 'id' => 'border_style_header_mm_menu_link', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Icon', 'mfn-opts'),
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li:--mfn-megamenu-menu-icon-gap',
							'type' => 'sliderbar',
							//'condition' => array( 'id' => 'icon_visibility', 'opt' => 'is', 'val' => 'block' ),
							'responsive' => 'desktop',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li:--mfn-megamenu-menu-icon-gap_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li:--mfn-megamenu-menu-icon-gap_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li:--mfn-megamenu-menu-icon-gap_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li:--mfn-megamenu-menu-icon-size',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),


						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li:--mfn-megamenu-menu-icon-size_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li:--mfn-megamenu-menu-icon-size_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li:--mfn-megamenu-menu-icon-size_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu .menu-icon > i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu a|hover > .menu-icon > i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li.current-menu-item > a > .menu-icon i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Submenu icon', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li:--mfn-megamenu-menu-submenu-icon-gap',
							//'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li:--mfn-megamenu-menu-submenu-icon-gap_laptop',
							//'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li:--mfn-megamenu-menu-submenu-icon-gap_tablet',
							//'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li:--mfn-megamenu-menu-submenu-icon-gap_mobile',
							//'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li .menu-sub i:--mfn-megamenu-submenu-icon-size',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li .menu-sub i:--mfn-megamenu-submenu-icon-size_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li .menu-sub i:--mfn-megamenu-submenu-icon-size_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li .menu-sub i:--mfn-megamenu-submenu-icon-size_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li > a .menu-sub i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li > a|hover .menu-sub i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li.current-menu-item > a .menu-sub i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'condition' => array( 'id' => 'submenu', 'opt' => 'is', 'val' => 'on' ),
   						'title' => __('Submenu', 'mfn-opts'),
   					),

   					array(
   						'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Container', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li > ul:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li > ul:border-style',
							'attr_id' => 'border_style_header_submenu',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li ul:border-width',
		  				'condition' => array( 'id' => 'border_style_header_submenu', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li ul:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_header_submenu', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li ul:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_header_submenu', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li ul:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_header_submenu', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li ul:border-color',
							'condition' => array( 'id' => 'border_style_header_submenu', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li ul:box-shadow',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow'
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li ul:--mfn-megamenu-submenu-border-radius',
							'type' => 'dimensions',
							'title' => __('Border radius', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'version' => 'separated-fields',
							'css_attr' => 'border-radius',
						),

   					array(
   						'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Items', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li ul a:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li ul li a:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li ul li a:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li ul li a:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li ul li a:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li ul a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li ul li a:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li ul a|hover:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li ul .current-menu-item > a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Icon', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li:--mfn-megamenu-submenu-subicon-gap',
							//'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li:--mfn-megamenu-submenu-subicon-gap_laptop',
							//'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li:--mfn-megamenu-submenu-subicon-gap_tablet',
							//'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li:--mfn-megamenu-submenu-subicon-gap_mobile',
							//'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li ul li .menu-icon i:--mfn-megamenu-submenu-subicon-size',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li ul li .menu-icon i:--mfn-megamenu-submenu-subicon-size_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li ul li .menu-icon i:--mfn-megamenu-submenu-subicon-size_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li ul li .menu-icon i:--mfn-megamenu-submenu-subicon-size_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li > a .menu-sub i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li > a|hover .menu-sub i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu > li.current-menu-item > a .menu-sub i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Submenu icon', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li ul li .menu-sub i:--mfn-megamenu-submenu-submenu-icon-size',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li ul li .menu-sub i:--mfn-megamenu-submenu-submenu-icon-size_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li ul li .menu-sub i:--mfn-megamenu-submenu-submenu-icon-size_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li ul li .menu-sub i:--mfn-megamenu-submenu-submenu-icon-size_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li ul > li > a .menu-sub i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li ul > li > a|hover .menu-sub i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li ul > li.current-menu-item > a .menu-sub i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'header',
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Decoration icon', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li a .decoration-icon i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li|hover > a > .decoration-icon i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li.current-menu-item > a .decoration-icon i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),



						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li a .decoration-icon i:font-size',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),


						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li a .decoration-icon i:font-size_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li a .decoration-icon i:font-size_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-megamenu-menu li a .decoration-icon i:font-size_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),
						array(
							'type' => 'html',
							'html' => '</div>',
						),

					),
   			),

   			// Logo ----------------------------------------------------

   			'header_logo' => array(
   				'type' => 'header_logo',
   				'title' => __('Logo', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'header',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
   						'id' => 'image',
   						'type' => 'upload',
   						'dynamic_data' => 'featured_image',
   						'title' => __('Logo', 'mfn-opts'),
   						'desc' => __('Recommended svg', 'mfn-opts'),
							'std' => mfn_opts_get('logo-img') ? mfn_opts_get('logo-img') : $this->get_placeholder(),
							'preview' => 'image',
   					),

   					array(
							'id' => 'link',
							'type' => 'text',
							'title' => __('Link', 'mfn-opts'),
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Image', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .logo-wrapper:align-items',
							'type' => 'switch',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'title' => __('Vertical align', 'mfn-opts'),
							'options' => array(
								'flex-start' => 'Top',
								'center' => 'Center',
								'flex-end' => 'Bottom',
							),
							'std' => 'center',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .logo-wrapper:align-items_laptop',
							'type' => 'switch',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'title' => __('Vertical align', 'mfn-opts'),
							'options' => array(
								'flex-start' => 'Top',
								'flex-end' => 'Bottom',
								'center' => 'Center',
							),
							'std' => 'center',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .logo-wrapper:align-items_tablet',
							'type' => 'switch',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'title' => __('Vertical align', 'mfn-opts'),
							'options' => array(
								'flex-start' => 'Top',
								'flex-end' => 'Bottom',
								'center' => 'Center',
							),
							'std' => 'center',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .logo-wrapper:align-items_mobile',
							'type' => 'switch',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'title' => __('Vertical align', 'mfn-opts'),
							'options' => array(
								'flex-start' => 'Top',
								'flex-end' => 'Bottom',
								'center' => 'Center',
							),
							'std' => 'center',
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

						// advanced here

   				),
   			),

			// Menu ----------------------------------------------------

   			'header_menu' => array(
   				'type' => 'header_menu',
   				'title' => __('Menu', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'header',
   				'attr' => array(
   				//'fields' => array(

						/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'type' => 'header',
							'title' => __('Menu', 'mfn-opts'),
						),

						array(
   						'id' => 'menu_display',
   						'type' => 'select',
   						're_render' => true,
   						'title' => __('Menu to display', 'mfn-opts'),
							//'options' => mfna_menu(),
							'js_options' => 'menus',
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu:justify-content',
							'attr_id' => 'menu-items-alignment',
							'type' => 'select',
							'responsive' => 'desktop',

							'title' => __('Alignment', 'mfn-opts'),
							'options' => array(
								'flex-start' => 'Left',
								'center' => 'Center',
								'flex-end' => 'Right',
								'space-between' => 'Space between',
								'space-around' => 'Space around',
								'space-evenly' => 'Space evenly',
							),
							'std' => 'flex-end',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu:justify-content_laptop',
							'attr_id' => 'menu-items-alignment-laptop',
							'type' => 'select',
							'responsive' => 'laptop',

							'title' => __('Alignment', 'mfn-opts'),
							'options' => array(
								'flex-start' => 'Left',
								'center' => 'Center',
								'flex-end' => 'Right',
								'space-between' => 'Space between',
								'space-around' => 'Space around',
								'space-evenly' => 'Space evenly',
							),
							'std' => 'center',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu:justify-content_tablet',
							'attr_id' => 'menu-items-alignment-tablet',
							'type' => 'select',
							'responsive' => 'tablet',

							'title' => __('Alignment', 'mfn-opts'),
							'options' => array(
								'flex-start' => 'Left',
								'center' => 'Center',
								'flex-end' => 'Right',
								'space-between' => 'Space between',
								'space-around' => 'Space around',
								'space-evenly' => 'Space evenly',
							),
							'std' => 'center',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu:justify-content_mobile',
							'attr_id' => 'menu-items-alignment-mobile',
							'type' => 'select',
							'responsive' => 'mobile',

							'title' => __('Alignment', 'mfn-opts'),
							'options' => array(
								'flex-start' => 'Left',
								'center' => 'Center',
								'flex-end' => 'Right',
								'space-between' => 'Space between',
								'space-around' => 'Space around',
								'space-evenly' => 'Space evenly',
							),
							'std' => 'center',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li:flex-grow',
							'condition' => array( 'id' => 'menu-items-alignment', 'opt' => 'is', 'val' => array('space-between', 'space-around', 'space-evenly') ),
							'type' => 'select',
							'responsive' => 'desktop',

							'title' => __('Fit container', 'mfn-opts'),
							'options' => [
								'unset' => __('Default', 'mfn-opts'),
								'1' => __('Fit', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li:flex-grow_laptop',
							'condition' => array( 'id' => 'menu-items-alignment-laptop', 'opt' => 'is', 'val' => array('space-between', 'space-around', 'space-evenly') ),
							'type' => 'select',
							'responsive' => 'laptop',

							'title' => __('Fit container', 'mfn-opts'),
							'options' => [
								'unset' => __('Default', 'mfn-opts'),
								'1' => __('Fit', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li:flex-grow_tablet',
							'condition' => array( 'id' => 'menu-items-alignment-tablet', 'opt' => 'is', 'val' => array('space-between', 'space-around', 'space-evenly') ),
							'type' => 'select',
							'responsive' => 'tablet',

							'title' => __('Fit container', 'mfn-opts'),
							'options' => [
								'unset' => __('Default', 'mfn-opts'),
								'1' => __('Fit', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li:flex-grow_mobile',
							'condition' => array( 'id' => 'menu-items-alignment-mobile', 'opt' => 'is', 'val' => array('space-between', 'space-around', 'space-evenly') ),
							'type' => 'select',
							'responsive' => 'mobile',

							'title' => __('Fit container', 'mfn-opts'),
							'options' => [
								'unset' => __('Default', 'mfn-opts'),
								'1' => __('Fit', 'mfn-opts'),
							],
						),

   					array(
   						'id' => 'animation',
   						'attr_id' => 'header_menu_animation',
   						'type' => 'select',
   						'title' => __('Item Animation', 'mfn-opts'),
							'options' => array(
								'' => __('None', 'mfn-opts'),
								'text-line-bottom' => __('Text line bottom', 'mfn-opts'),
								'text-toggle-line-bottom' => __('Text toggle line bottom', 'mfn-opts'),
								'toggle-line-bottom' => __('Toggle line bottom', 'mfn-opts'),
								'toggle-line-top' => __('Toggle line top', 'mfn-opts'),
								'text-bg-line' => __('Text bg line', 'mfn-opts'),
								'bg-left' => __('Bg left', 'mfn-opts'),
							),
   					),

   					array(
   						'id' => 'separator',
   						'attr_id' => 'menu_separator',
   						'type' => 'switch',
   						'title' => __('Separator', 'mfn-opts'),
							'options' => array(
								'off' => 'Off',
								'on' => 'On'
							),
							'std' => 'off'
   					),

   					array(
   						'id' => 'submenu_display',
   						'type' => 'switch',
   						'title' => __('Display submenu on', 'mfn-opts'),
							'options' => array(
								'hover' => 'Hover',
								'click' => 'Click'
							),
							'std' => 'hover'
   					),

   					array(
   						'id' => 'submenu_icon_display',
   						'attr_id' => 'submenu_icon_display',
   						'type' => 'switch',
   						'title' => __('Submenu Icon', 'mfn-opts'),
							'options' => array(
								'' => 'Off',
								'on' => 'On'
							),
							'std' => 'on'
   					),

   					array(
							'id' => 'submenu_icon',
							'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
							'type' => 'icon',
							'title' => __('Submenu icon', 'mfn-opts'),
							'preview' => 'icon',
							'std' => 'fas fa-arrow-down'
						),

   					array(
   						'type' => 'header',
							'title' => __('Icon', 'mfn-opts'),
						),

						array(
   						'id' => 'icon_align',
   						'type' => 'switch',
   						'title' => __('Icon align', 'mfn-opts'),
							'options' => array(
								'left' => 'Left',
								'top' => 'Top',
								'right' => 'Right'
							),
							'std' => 'left'
   					),

						array(
   						'id' => 'icon_animation',
   						'type' => 'select',
   						'title' => __('Icon animation', 'mfn-opts'),
							'options' => array(
								'' => __('None', 'mfn-opts'),
								'rotate' => 'Rotate',
								'zoom' => 'Zoom'
							),
   					),

						array(
							'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
							'title' => __('Submenu', 'mfn-opts'),
						),

   					array(
							'id' => 'submenu_subicon',
							'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
							'type' => 'icon',
							'title' => __('Submenu icon', 'mfn-opts'),
							'preview' => 'icon',
							'std' => 'fas fa-arrow-right'
						),

						array(
   						'id' => 'submenu_icon_animation',
   						'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
   						'type' => 'select',
   						'title' => __('Icon Animation', 'mfn-opts'),
							'options' => array(
								'' => __('None', 'mfn-opts'),
								'rotate' => 'Rotate',
								'zoom' => 'Zoom'
							),
   					),

   					array(
   						'id' => 'submenu_animation',
   						'type' => 'select',
   						'title' => __('Submenu Animation', 'mfn-opts'),
							'options' => array(
								'' => __('None', 'mfn-opts'),
								'fade-in' => 'Fade In',
								'fade-up' => 'Fade In Up'
							),
   					),

   					array(
							'id' => 'submenu_fold_to_right',
							'type' => 'switch',
							'title' => __('Submenu fold last two items to left', 'mfn-opts'),
							'options' => array(
								0 => __('Disable', 'mfn-opts'),
								1 => __('Enable', 'mfn-opts'),
							),
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Menu', 'mfn-opts'),
						),

						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Items', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li:--mfn-header-menu-gap',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li:--mfn-header-menu-gap_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li:--mfn-header-menu-gap_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li:--mfn-header-menu-gap_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li > a.mfn-menu-link:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li > a.mfn-menu-link:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li > a.mfn-menu-link:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li > a.mfn-menu-link:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li > a.mfn-menu-link:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li > a.mfn-menu-link:border-style',
							'attr_id' => 'border_style_header_menu_link',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li > a.mfn-menu-link:border-width',
		  				'condition' => array( 'id' => 'border_style_header_menu_link', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li > a.mfn-menu-link:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_header_menu_link', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li > a.mfn-menu-link:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_header_menu_link', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li > a.mfn-menu-link:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_header_menu_link', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li > a.mfn-menu-link:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li > a.mfn-menu-link:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li > a.mfn-menu-link:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li > a.mfn-menu-link:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li > a.mfn-menu-link:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li > a.mfn-menu-link:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li > a.mfn-menu-link:border-color',
							'condition' => array( 'id' => 'border_style_header_menu_link', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li > a.mfn-menu-link:box-shadow',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow'
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li > a.mfn-menu-link:box-shadow_laptop',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li > a.mfn-menu-link:box-shadow_tablet',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li > a.mfn-menu-link:box-shadow_mobile',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li|hover > a.mfn-menu-link:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li|hover > a.mfn-menu-link:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li|hover > a.mfn-menu-link:border-color',
							'condition' => array( 'id' => 'border_style_header_menu_link', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li|hover > a.mfn-menu-link:box-shadow',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow'
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li|hover > a.mfn-menu-link:box-shadow_laptop',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li|hover > a.mfn-menu-link:box-shadow_tablet',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li|hover > a.mfn-menu-link:box-shadow_mobile',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.current-menu-item > a.mfn-menu-link:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.current-menu-item > a.mfn-menu-link:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.current-menu-item > a.mfn-menu-link:border-color',
							'condition' => array( 'id' => 'border_style_header_menu_link', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.current-menu-item > a.mfn-menu-link:box-shadow',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow'
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.current-menu-item > a.mfn-menu-link:box-shadow_laptop',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.current-menu-item > a.mfn-menu-link:box-shadow_tablet',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.current-menu-item > a.mfn-menu-link:box-shadow_mobile',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Icon', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li:--mfn-header-menu-icon-gap',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li:--mfn-header-menu-icon-gap_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li:--mfn-header-menu-icon-gap_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li:--mfn-header-menu-icon-gap_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li:--mfn-header-menu-icon-size',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li:--mfn-header-menu-icon-size_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li:--mfn-header-menu-icon-size_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li:--mfn-header-menu-icon-size_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu .mfn-menu-item-icon > i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu a.mfn-menu-link|hover > .mfn-menu-item-icon > i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.current-menu-item.mfn-menu-li > a.mfn-menu-link > .mfn-menu-item-icon i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Submenu icon', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li:--mfn-header-menu-submenu-icon-gap',
							'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li:--mfn-header-menu-submenu-icon-gap_laptop',
							'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li:--mfn-header-menu-submenu-icon-gap_tablet',
							'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li:--mfn-header-menu-submenu-icon-gap_mobile',
							'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li .mfn-menu-subicon i:--mfn-header-submenu-icon-size',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li .mfn-menu-subicon i:--mfn-header-submenu-icon-size_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li .mfn-menu-subicon i:--mfn-header-submenu-icon-size_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li .mfn-menu-subicon i:--mfn-header-submenu-icon-size_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li > a.mfn-menu-link .mfn-menu-subicon i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li > a.mfn-menu-link|hover .mfn-menu-subicon i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.current-menu-item.mfn-menu-li > a.mfn-menu-link .mfn-menu-subicon i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),



						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Submenu', 'mfn-opts'),
						),

						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Container', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li > ul.mfn-submenu:width',
							'type' => 'text',
							'class' => 'mfn-toggled',
							'responsive' => 'desktop',
							'default_unit' => 'px',
							'title' => __('Width', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li > ul.mfn-submenu:width_laptop',
							'type' => 'text',
							'class' => 'mfn-toggled',
							'responsive' => 'laptop',
							'default_unit' => 'px',
							'title' => __('Width', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li > ul.mfn-submenu:width_tablet',
							'type' => 'text',
							'class' => 'mfn-toggled',
							'responsive' => 'tablet',
							'default_unit' => 'px',
							'title' => __('Width', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li > ul.mfn-submenu:width_mobile',
							'type' => 'text',
							'class' => 'mfn-toggled',
							'responsive' => 'mobile',
							'default_unit' => 'px',
							'title' => __('Width', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li > ul.mfn-submenu:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li > ul.mfn-submenu:border-style',
							'attr_id' => 'border_style_header_submenu',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu:border-width',
		  				'condition' => array( 'id' => 'border_style_header_submenu', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_header_submenu', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_header_submenu', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_header_submenu', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu:border-color',
							'condition' => array( 'id' => 'border_style_header_submenu', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu:box-shadow',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow'
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu:--mfn-header-submenu-border-radius',
							'type' => 'dimensions',
							'title' => __('Border radius', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'version' => 'separated-fields',
							'css_attr' => 'border-radius',
						),

						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Items', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li a.mfn-menu-link:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li a.mfn-menu-link:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li a.mfn-menu-link:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li a.mfn-menu-link:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),




		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li a.mfn-menu-link:border-style',
							'attr_id' => 'border_style_header_submenu_links',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li a.mfn-menu-link:border-width',
		  				'condition' => array( 'id' => 'border_style_header_submenu_links', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li a.mfn-menu-link:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_header_submenu_links', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li a.mfn-menu-link:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_header_submenu_links', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li a.mfn-menu-link:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_header_submenu_links', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li a.mfn-menu-link:border-color',
							'condition' => array( 'id' => 'border_style_header_submenu_links', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),







		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li a.mfn-menu-link:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li a.mfn-menu-link:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li a.mfn-menu-link:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li a.mfn-menu-link|hover:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li a.mfn-menu-link|hover:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.current-menu-item.mfn-menu-li > a.mfn-menu-link:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.current-menu-item.mfn-menu-li > a.mfn-menu-link:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Icon', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li:--mfn-header-submenu-subicon-gap',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li:--mfn-header-submenu-subicon-gap_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li:--mfn-header-submenu-subicon-gap_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li:--mfn-header-submenu-subicon-gap_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li:--mfn-header-submenu-subicon-size',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li:--mfn-header-submenu-subicon-size_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li:--mfn-header-submenu-subicon-size_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li:--mfn-header-submenu-subicon-size_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li .mfn-menu-item-icon > i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li a.mfn-menu-link:border-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li|hover > a.mfn-menu-link > .mfn-menu-item-icon > i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li|hover > a.mfn-menu-link:border-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu > li.current-menu-item.mfn-menu-li > a.mfn-menu-link > .mfn-menu-item-icon i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu > li.current-menu-item.mfn-menu-li > a.mfn-menu-link:border-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Submenu icon', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li .mfn-menu-sub-subicon i:--mfn-header-submenu-submenu-icon-size',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li .mfn-menu-sub-subicon i:--mfn-header-submenu-submenu-icon-size_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li .mfn-menu-sub-subicon i:--mfn-header-submenu-submenu-icon-size_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu li.mfn-menu-li .mfn-menu-sub-subicon i:--mfn-header-submenu-submenu-icon-size_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu > li.mfn-menu-li > a.mfn-menu-link .mfn-menu-sub-subicon i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu > li.mfn-menu-li > a.mfn-menu-link|hover .mfn-menu-sub-subicon i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu li.mfn-menu-li ul.mfn-submenu > li.current-menu-item.mfn-menu-li > a.mfn-menu-link .mfn-menu-sub-subicon i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'condition' => array( 'id' => 'menu_separator', 'opt' => 'is', 'val' => 'on' ),
							'title' => __('Separator', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li:--mfn-header-menu-sep',
							'condition' => array( 'id' => 'menu_separator', 'opt' => 'is', 'val' => 'on' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'condition' => array( 'id' => 'header_menu_animation', 'opt' => 'isnt', 'val' => '' ),
							'title' => __('Animation', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li:--mfn-header-menu-animation-color',
							'condition' => array( 'id' => 'header_menu_animation', 'opt' => 'isnt', 'val' => '' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li:--mfn-header-menu-animation-height',
							'type' => 'sliderbar',
							'condition' => array( 'id' => 'header_menu_animation', 'opt' => 'is', 'val' => array('text-line-bottom', 'text-toggle-line-bottom', 'toggle-line-bottom', 'toggle-line-top') ),
							'title' => __('Height', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '1',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Dropdown pointer', 'mfn-opts'),
						),

						array(
   						'id' => 'dropdown_pointer',
   						'attr_id' => 'dropdown_pointer',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Visibility', 'mfn-opts'),
							'options' => array(
								'' => 'Hide',
								'1' => 'Show',
							),
							'std' => ''
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li:--mfn-header-submenu-dropdown-size',
							'type' => 'sliderbar',
							'condition' => array( 'id' => 'dropdown_pointer', 'opt' => 'is', 'val' => '1' ),
							'title' => __('Size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '1',
								'max' => '40',
								'step' => '1',
								'unit' => 'px',
							)
						),

   					array(
   						'id' => 'dropdown_alignment',
   						'condition' => array( 'id' => 'dropdown_pointer', 'opt' => 'is', 'val' => '1' ),
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Icon align', 'mfn-opts'),
							'options' => array(
								'left' => 'Left',
								'center' => 'Center',
								'right' => 'Right',
							),
							'std' => 'left'
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li:--mfn-header-submenu-dropdown-offset',
							'type' => 'sliderbar',
							'condition' => array( 'id' => 'dropdown_pointer', 'opt' => 'is', 'val' => '1' ),
							'title' => __('Offset', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '1',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu > li.mfn-menu-li:--mfn-header-submenu-dropdown-color',
							'condition' => array( 'id' => 'dropdown_pointer', 'opt' => 'is', 'val' => '1' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

						// advanced here

   				),
   			),


				// Header menu burger ----------------------------------------------------

   			'header_burger' => array(
   				'type' => 'header_burger',
   				'title' => __('Menu burger', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'header',
   				'attr' => array(
   				//'fields' => array(

						/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						// icon

						array(
							'type' => 'header',
							'title' => __('Icon', 'mfn-opts'),
						),

						array(
							'id' => 'icon',
							'attr_id' => 'header_icon_font',
							'type' => 'icon',
							'title' => __('Icon', 'mfn-opts'),
							'preview' => 'icon',
						),

						array(
							'id' => 'image',
							'type' => 'upload',
							'title' => __('Image', 'mfn-opts'),
							'desc' => __('Image instead of an icon', 'mfn-opts'),
							'preview' => 'image',
						),

						array(
							'id' => 'desc',
							'type' => 'text',
							'edit_tag' => '.mfn-header-menu-burger | div.desc-wrapper',
   						'edit_position' => 'append',
							'title' => __('Desc', 'mfn-opts'),
						),

						array(
							'id' => 'link_title',
							'type' => 'text',
							'title' => __('Link title', 'mfn-opts'),
						),

						// sidebar

						array(
							'type' => 'header',
							'title' => __('Nav', 'mfn-opts'),
						),

						array(
   						'id' => 'sidebar_type',
   						'attr_id' => 'sidebar_type',
   						'type' => 'select',
   						're_render' => true,
   						'title' => __('Nav type', 'mfn-opts'),
							'js_options' => 'sidemenus',
							'opt_append' => array(
								'classic' => __('Classic menu', 'mfn-opts'),
							),
							'std' => '0'
   					),

   					array(
							'type' => 'subheader',
							'condition' => array( 'id' => 'sidebar_type', 'opt' => 'is', 'val' => array('0', 'classic') ),
							'class' => 'mfn-builder-subheader row-header',
							'title' => __('Menu', 'mfn-opts'),
						),

						array(
   						'id' => 'menu_display',
   						'type' => 'select',
   						'condition' => array( 'id' => 'sidebar_type', 'opt' => 'is', 'val' => array('0', 'classic') ),
   						're_render' => true,
   						'title' => __('Menu to display', 'mfn-opts'),
							'js_options' => 'menus',
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-tmpl-menu-sidebar-wrapper:align-items',
							'type' => 'select',
							'condition' => array( 'id' => 'sidebar_type', 'opt' => 'is', 'val' => '0' ),
							'title' => __('Vertical align', 'mfn-opts'),
							'options' => array(
								'flex-start' => 'Top',
								'center' => 'Center',
								'flex-end' => 'Bottom',
							),
							'std' => 'center'
						),

   					array(
   						'id' => 'menu_pos',
   						'type' => 'select',
   						'condition' => array( 'id' => 'sidebar_type', 'opt' => 'is', 'val' => '0' ),
   						're_render' => true,
   						'title' => __('Sidebar position', 'mfn-opts'),
							'options' => array(
								'left' => __('Left', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							),
							'std' => 'right'
   					),

   					array(
   						'id' => 'animation',
   						'condition' => array( 'id' => 'sidebar_type', 'opt' => 'is', 'val' => array('0', 'classic') ),
   						'attr_id' => 'header_menu_animation',
   						'type' => 'select',
   						'title' => __('Item Animation', 'mfn-opts'),
							'options' => array(
								'' => __('None', 'mfn-opts'),
								'text-line-bottom' => __('Text line bottom', 'mfn-opts'),
								'text-toggle-line-bottom' => __('Text toggle line bottom', 'mfn-opts'),
								'toggle-line-bottom' => __('Toggle line bottom', 'mfn-opts'),
								'toggle-line-top' => __('Toggle line top', 'mfn-opts'),
								'text-bg-line' => __('Text bg line', 'mfn-opts'),
								'bg-left' => __('Bg left', 'mfn-opts'),
							),
   					),

   					array(
   						'id' => 'submenu_icon_display',
   						'condition' => array( 'id' => 'sidebar_type', 'opt' => 'is', 'val' => array('0', 'classic') ),
   						'attr_id' => 'submenu_icon_display',
   						'type' => 'switch',
   						'title' => __('Submenu Icon', 'mfn-opts'),
							'options' => array(
								'' => 'Off',
								'on' => 'On'
							),
							'std' => 'on'
   					),

   					array(
   						'id' => 'submenu_animation',
   						//'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
   						'condition' => array( 'id' => 'sidebar_type', 'opt' => 'is', 'val' => array('0', 'classic') ),
   						'type' => 'select',
   						'title' => __('Submenu Animation', 'mfn-opts'),
							'options' => array(
								'' => __('None', 'mfn-opts'),
								'fade-in' => 'Fade In',
								'fade-up' => 'Fade In Up'
							),
   					),

   					array(
							'id' => 'submenu_icon',
							//'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
							'condition' => array( 'id' => 'sidebar_type', 'opt' => 'is', 'val' => array('0', 'classic') ),
							'type' => 'icon',
							'title' => __('Submenu icon', 'mfn-opts'),
							'preview' => 'icon',
							'std' => 'fas fa-arrow-down'
						),

						array(
   						'id' => 'menu_icon_align',
   						'condition' => array( 'id' => 'sidebar_type', 'opt' => 'is', 'val' => array('0', 'classic') ),
   						'attr_id' => 'menu_icon_align',
   						'type' => 'switch',
   						'title' => __('Icon align', 'mfn-opts'),
							'options' => array(
								'left' => 'Left',
								'top' => 'Top',
								'right' => 'Right'
							),
							'std' => 'left'
   					),

						array(
   						'id' => 'icon_animation',
   						'type' => 'select',
   						'condition' => array( 'id' => 'sidebar_type', 'opt' => 'is', 'val' => array('0', 'classic') ),
   						'title' => __('Icon animation', 'mfn-opts'),
							'options' => array(
								'' => __('None', 'mfn-opts'),
								'rotate' => 'Rotate',
								'zoom' => 'Zoom'
							),
   					),

						array(
   						'condition' => array( 'id' => 'sidebar_type', 'opt' => 'is', 'val' => array('0', 'classic') ),
   						//'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header',
							'title' => __('Submenu', 'mfn-opts'),
						),

   					array(
							'id' => 'submenu_subicon',
							'condition' => array( 'id' => 'sidebar_type', 'opt' => 'is', 'val' => array('0', 'classic') ),
							//'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
							'type' => 'icon',
							'title' => __('Submenu icon', 'mfn-opts'),
							'preview' => 'icon',
							'std' => 'fas fa-arrow-right'
						),

						array(
   						'id' => 'submenu_icon_animation',
   						'condition' => array( 'id' => 'sidebar_type', 'opt' => 'is', 'val' => array('0', 'classic') ),
   						//'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
   						'type' => 'select',
   						'title' => __('Icon Animation', 'mfn-opts'),
							'options' => array(
								'' => __('None', 'mfn-opts'),
								'rotate' => 'Rotate',
								'zoom' => 'Zoom'
							),
   					),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Wrapper', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu-burger:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu-burger:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu-burger:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu-burger:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu-burger:border-style',
							'attr_id' => 'border_header_burger_wrapper',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu-burger:border-width',
							'condition' => array( 'id' => 'border_header_burger_wrapper', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'dimensions',
							'title' => __('Border width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu-burger:border-radius',
							'type' => 'dimensions',
							'title' => __('Border radius', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
						),






						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu-burger:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu-burger:border-color',
							'condition' => array( 'id' => 'border_header_burger_wrapper', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu-burger:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-menu-burger:border-color',
							'condition' => array( 'id' => 'border_header_burger_wrapper', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),






						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Icon', 'mfn-opts'),
						),

						array(
							'id' => 'icon_position',
							'type' => 'switch',
							'title' => __('Icon position', 'mfn-opts'),
							'options' => array(
								'top' => __('Top', 'mfn-opts'),
								'bottom' => __('Bottom', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							),
							'std' => 'top',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'icon_position_laptop',
							'type' => 'switch',
							'title' => __('Icon position', 'mfn-opts'),
							'options' => array(
								'top' => __('Top', 'mfn-opts'),
								'bottom' => __('Bottom', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
								'' => __('Default', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'icon_position_tablet',
							'type' => 'switch',
							'title' => __('Icon position', 'mfn-opts'),
							'options' => array(
								'top' => __('Top', 'mfn-opts'),
								'bottom' => __('Bottom', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
								'' => __('Default', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'icon_position_mobile',
							'type' => 'switch',
							'title' => __('Icon position', 'mfn-opts'),
							'options' => array(
								'top' => __('Top', 'mfn-opts'),
								'bottom' => __('Bottom', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
								'' => __('Default', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'icon_align',
							'type' => 'switch',
							'condition' => array( 'id' => 'header-icon-desc', 'opt' => 'isnt', 'val' => '' ),
							'title' => __('Icon alignment', 'mfn-opts'),
							'options' => array(
								'start' => __('Start', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'end' => __('End', 'mfn-opts'),
							),
							'std' => 'center',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'icon_align_laptop',
							'condition' => array( 'id' => 'header-icon-desc', 'opt' => 'isnt', 'val' => '' ),
							'type' => 'switch',
							'title' => __('Icon alignment', 'mfn-opts'),
							'options' => array(
								'start' => __('Start', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'end' => __('End', 'mfn-opts'),
								'' => __('Default', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'icon_align_tablet',
							'condition' => array( 'id' => 'header-icon-desc', 'opt' => 'isnt', 'val' => '' ),
							'type' => 'switch',
							'title' => __('Icon alignment', 'mfn-opts'),
							'options' => array(
								'start' => __('Start', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'end' => __('End', 'mfn-opts'),
								'' => __('Default', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'icon_align_mobile',
							'condition' => array( 'id' => 'header-icon-desc', 'opt' => 'isnt', 'val' => '' ),
							'type' => 'switch',
							'title' => __('Icon alignment', 'mfn-opts'),
							'options' => array(
								'start' => __('Start', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'end' => __('End', 'mfn-opts'),
								'' => __('Default', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:--mfn-header-menu-icon-size',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '150',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:--mfn-header-menu-icon-size_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '150',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:--mfn-header-menu-icon-size_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '150',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:--mfn-header-menu-icon-size_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '150',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:width',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '150',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:width_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '150',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:width_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '150',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:width_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '150',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:height',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Height', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '150',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:height_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Height', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '150',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:height_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Height', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '150',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:height_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Height', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '150',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:border-style',
							'attr_id' => 'border_style_header_icon',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:border-color',
							'condition' => array( 'id' => 'border_style_header_icon', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:border-width',
							'condition' => array( 'id' => 'border_style_header_icon', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'dimensions',
							'title' => __('Border width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:border-radius',
							'type' => 'dimensions',
							'title' => __('Border radius', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Icon color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box|hover .icon-wrapper:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box|hover .icon-wrapper:border-color',
							'condition' => array( 'id' => 'border_style_header_icon', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box|hover .icon-wrapper i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Icon color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Desc', 'mfn-opts'),
						),

						array(
		  				'id' => 'header_icon_desc_visibility',
		  				'iconinfo' => 'https://support.muffingroup.com/video-tutorials/section-responsive-visibility/',
		  				'type' => 'switch',
		  				'invert' => true,
		  				'version' => 'multiple',
		  				'class' => 'mfn-toggled',
		  				'title' => __('Responsive visibility', 'mfn-opts'),
		  				'options' => array(
		  					'hide-desktop' => __('Hide on Desktop', 'mfn-opts'),
		  					'hide-laptop' => __('Hide on Laptop', 'mfn-opts'),
		  					'hide-tablet' => __('Hide on Tablet', 'mfn-opts'),
		  					'hide-mobile' => __('Hide on Mobile', 'mfn-opts'),
		  				),
		  				'active_tooltip' => array(
		  					'hide-desktop' => __('Show on Desktop', 'mfn-opts'),
		  					'hide-laptop' => __('Show on Laptop', 'mfn-opts'),
		  					'hide-tablet' => __('Show on Tablet', 'mfn-opts'),
		  					'hide-mobile' => __('Show on Mobile', 'mfn-opts'),
		  				),
		  				'visual_options' => array(
		  					'hide-desktop' => '<svg id="Layer_1" data-name="Layer 1" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 22 22"><defs><style>.cls-1{fill:none;stroke:#304050;stroke-linecap:round;stroke-linejoin:round;stroke-width:1.5px;}.cls-2{fill:#304050;}</style></defs><path class="cls-1" d="M4.94,5H17.06a.93.93,0,0,1,.94.91v7.18a.93.93,0,0,1-.94.91H4.94A.93.93,0,0,1,4,13.09V5.91A.93.93,0,0,1,4.94,5Z"/><path class="cls-1" d="M7,17h8"/><path class="cls-2" d="M13,15.51a1.58,1.58,0,0,0,.7,1.26L14,17H8l.3-.23A1.58,1.58,0,0,0,9,15.51V14h4Z"/></svg>',			// 960 +
		  					'hide-laptop' => '<svg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"viewBox="0 0 22 22" style="enable-background:new 0 0 22 22;" xml:space="preserve"> <style type="text/css"> .st0{fill:none;stroke:currentColor;stroke-width:1.5;stroke-linecap:round;stroke-linejoin:round;} </style> <path class="st0" d="M5,15V6.9C5,6.4,5.4,5.9,5.9,6h10.1c0.5,0,0.9,0.4,0.9,0.9V15"/> <path class="st0" d="M4,17h14"/> </svg>',		// 768 - 959
		  					'hide-tablet' => '<svg id="Layer_1" data-name="Layer 1" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 22 22"><defs><style>.cls-1{fill:none;stroke:#304050;stroke-linecap:round;stroke-linejoin:round;stroke-width:1.5px;}</style></defs><path class="cls-1" d="M7,15.83V7.17A1.21,1.21,0,0,1,8.25,6h5.5A1.21,1.21,0,0,1,15,7.17v8.66A1.21,1.21,0,0,1,13.75,17H8.25A1.21,1.21,0,0,1,7,15.83Z"/></svg>',		// 768 - 959
		  					'hide-mobile' => '<svg id="Layer_1" data-name="Layer 1" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 22 22"><defs><style>.cls-1{fill:none;stroke:#304050;stroke-linecap:round;stroke-linejoin:round;stroke-width:1.5px;}.cls-2{fill:#304050;}</style></defs><path class="cls-1" d="M8,15.82V8.18A1.08,1.08,0,0,1,8.94,7h4.12A1.08,1.08,0,0,1,14,8.18v7.64A1.08,1.08,0,0,1,13.06,17H8.94A1.08,1.08,0,0,1,8,15.82Z"/><circle class="cls-2" cx="11" cy="14" r="1"/></svg>',			// - 768
		  				),
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc-wrapper:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),


		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .desc-wrapper:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box|hover .desc-wrapper:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'header',
							'condition' => array( 'id' => 'sidebar_type', 'opt' => 'is', 'val' => '0,classic' ),
							'class' => 'toggled_header row-header',
							'title' => __('Nav', 'mfn-opts'),
						),


						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Container', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar:--mfn-header-menu-sidebar-width',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'condition' => array( 'id' => 'sidebar_type', 'opt' => 'is', 'val' => '0' ),
							'title' => __('Width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '100',
								'max' => '1920',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar:--mfn-header-menu-sidebar-width_laptop',
							'type' => 'sliderbar',
							'condition' => array( 'id' => 'sidebar_type', 'opt' => 'is', 'val' => '0' ),
							'responsive' => 'laptop',
							'title' => __('Width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '100',
								'max' => '1920',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar:--mfn-header-menu-sidebar-width_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'condition' => array( 'id' => 'sidebar_type', 'opt' => 'is', 'val' => '0' ),
							'title' => __('Width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '100',
								'max' => '1920',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar:--mfn-header-menu-sidebar-width_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'condition' => array( 'id' => 'sidebar_type', 'opt' => 'is', 'val' => '0' ),
							'title' => __('Width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '100',
								'max' => '1920',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-tmpl-menu-sidebar-wrapper:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-tmpl-menu-sidebar-wrapper:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-tmpl-menu-sidebar-wrapper:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-tmpl-menu-sidebar-wrapper:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

						array(
							'id' => 'background_switcher',
							'attr_id' => 'bg_menu-tmpl-sidebar',
							'type' => 'switch',
							'class' => 'mfn-toggled mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar:background-color',
							'condition' => array( 'id' => 'bg_menu-tmpl-sidebar', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar:gradient',
		  				'condition' => array( 'id' => 'bg_menu-tmpl-sidebar', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar:background-image',
		  				'condition' => array( 'id' => 'bg_menu-tmpl-sidebar', 'opt' => 'is', 'val' => 'default' ),
		  				'oldid' => 'bg_color',
		  				'type' => 'upload',
		  				'title' => __('Image', 'mfn-opts'),
		  				'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-image',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar:background-image_laptop',
		  				'condition' => array( 'id' => 'bg_menu-tmpl-sidebar', 'opt' => 'is', 'val' => 'default' ),
		  				'oldid' => 'bg_image',
		  				'type' => 'upload',
		  				'title' => __('Image', 'mfn-opts'),
		  				'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-image',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar:background-image_tablet',
		  				'condition' => array( 'id' => 'bg_menu-tmpl-sidebar', 'opt' => 'is', 'val' => 'default' ),
		  				'oldid' => 'bg_image',
		  				'type' => 'upload',
		  				'title' => __('Image', 'mfn-opts'),
		  				'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-image',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar:background-image_mobile',
		  				'condition' => array( 'id' => 'bg_menu-tmpl-sidebar', 'opt' => 'is', 'val' => 'default' ),
		  				'type' => 'upload',
		  				'title' => __('Image', 'mfn-opts'),
		  				'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-image',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar:background-repeat',
							'condition' => array( 'id' => 'bg_menu-tmpl-sidebar', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'select',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-repeat',
							'title' => __('Repeat', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'repeat' => __('Repeat', 'mfn-opts'),
								'no-repeat' => __('No repeat', 'mfn-opts'),
								'repeat-x' => __('Repeat X', 'mfn-opts'),
								'repeat-y' => __('Repeat Y', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar:background-repeat_laptop',
							'condition' => array( 'id' => 'bg_menu-tmpl-sidebar', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'select',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-repeat',
							'title' => __('Repeat', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'repeat' => __('Repeat', 'mfn-opts'),
								'no-repeat' => __('No repeat', 'mfn-opts'),
								'repeat-x' => __('Repeat X', 'mfn-opts'),
								'repeat-y' => __('Repeat Y', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar:background-repeat_tablet',
							'condition' => array( 'id' => 'bg_menu-tmpl-sidebar', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'select',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-repeat',
							'title' => __('Repeat', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'repeat' => __('Repeat', 'mfn-opts'),
								'no-repeat' => __('No repeat', 'mfn-opts'),
								'repeat-x' => __('Repeat X', 'mfn-opts'),
								'repeat-y' => __('Repeat Y', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar:background-repeat_mobile',
							'condition' => array( 'id' => 'bg_menu-tmpl-sidebar', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'select',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-repeat',
							'title' => __('Repeat', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'repeat' => __('Repeat', 'mfn-opts'),
								'no-repeat' => __('No repeat', 'mfn-opts'),
								'repeat-x' => __('Repeat X', 'mfn-opts'),
								'repeat-y' => __('Repeat Y', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar:background-position',
							'condition' => array( 'id' => 'bg_menu-tmpl-sidebar', 'opt' => 'is', 'val' => 'default' ),
							'oldid' => 'bg_position',
							'type' => 'select',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-position',
							'title' => __('Position', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'center' => __('Center Center', 'mfn-opts'),
								'center left' => __('Center Left', 'mfn-opts'),
								'center right' => __('Center Right', 'mfn-opts'),
								'top left' => __('Top Left', 'mfn-opts'),
								'top right' => __('Top Right', 'mfn-opts'),
								'top center' => __('Top Center', 'mfn-opts'),
								'bottom left' => __('Bottom Left', 'mfn-opts'),
								'bottom right' => __('Bottom Right', 'mfn-opts'),
								'bottom center' => __('Bottom Center', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar:background-position_laptop',
							'condition' => array( 'id' => 'bg_menu-tmpl-sidebar', 'opt' => 'is', 'val' => 'default' ),
							'oldid' => 'bg_position',
							'type' => 'select',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-position',
							'title' => __('Position', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'center' => __('Center Center', 'mfn-opts'),
								'center left' => __('Center Left', 'mfn-opts'),
								'center right' => __('Center Right', 'mfn-opts'),
								'top left' => __('Top Left', 'mfn-opts'),
								'top right' => __('Top Right', 'mfn-opts'),
								'top center' => __('Top Center', 'mfn-opts'),
								'bottom left' => __('Bottom Left', 'mfn-opts'),
								'bottom right' => __('Bottom Right', 'mfn-opts'),
								'bottom center' => __('Bottom Center', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar:background-position_tablet',
							'condition' => array( 'id' => 'bg_menu-tmpl-sidebar', 'opt' => 'is', 'val' => 'default' ),
							'oldid' => 'bg_position',
							'type' => 'select',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-position',
							'title' => __('Position', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'center' => __('Center Center', 'mfn-opts'),
								'center left' => __('Center Left', 'mfn-opts'),
								'center right' => __('Center Right', 'mfn-opts'),
								'top left' => __('Top Left', 'mfn-opts'),
								'top right' => __('Top Right', 'mfn-opts'),
								'top center' => __('Top Center', 'mfn-opts'),
								'bottom left' => __('Bottom Left', 'mfn-opts'),
								'bottom right' => __('Bottom Right', 'mfn-opts'),
								'bottom center' => __('Bottom Center', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar:background-position_mobile',
							'condition' => array( 'id' => 'bg_menu-tmpl-sidebar', 'opt' => 'is', 'val' => 'default' ),
							'oldid' => 'bg_position',
							'type' => 'select',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-position',
							'title' => __('Position', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'center' => __('Center Center', 'mfn-opts'),
								'center left' => __('Center Left', 'mfn-opts'),
								'center right' => __('Center Right', 'mfn-opts'),
								'top left' => __('Top Left', 'mfn-opts'),
								'top right' => __('Top Right', 'mfn-opts'),
								'top center' => __('Top Center', 'mfn-opts'),
								'bottom left' => __('Bottom Left', 'mfn-opts'),
								'bottom right' => __('Bottom Right', 'mfn-opts'),
								'bottom center' => __('Bottom Center', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar:background-attachment',
							'condition' => array( 'id' => 'bg_menu-tmpl-sidebar', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'select',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-attachment',
							'title' => __('Attachment', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'scroll' => __('Scroll', 'mfn-opts'),
								'fixed' => __('Fixed', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar:background-attachment_laptop',
							'condition' => array( 'id' => 'bg_menu-tmpl-sidebar', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'select',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-attachment',
							'title' => __('Attachment', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'scroll' => __('Scroll', 'mfn-opts'),
								'fixed' => __('Fixed', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar:background-attachment_tablet',
							'condition' => array( 'id' => 'bg_menu-tmpl-sidebar', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'select',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-attachment',
							'title' => __('Attachment', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'scroll' => __('Scroll', 'mfn-opts'),
								'fixed' => __('Fixed', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar:background-attachment_mobile',
							'condition' => array( 'id' => 'bg_menu-tmpl-sidebar', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'select',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-attachment',
							'title' => __('Attachment', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'scroll' => __('Scroll', 'mfn-opts'),
								'fixed' => __('Fixed', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar:background-size',
							'condition' => array( 'id' => 'bg_menu-tmpl-sidebar', 'opt' => 'is', 'val' => 'default' ),
							'oldid' => 'bg_size',
							'type' => 'select',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-size',
							'title' => __('Size', 'mfn-opts'),
							'options' => mfna_bg_size()
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar:background-size_laptop',
							'condition' => array( 'id' => 'bg_menu-tmpl-sidebar', 'opt' => 'is', 'val' => 'default' ),
							'oldid' => 'bg_size',
							'type' => 'select',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-size',
							'title' => __('Size', 'mfn-opts'),
							'options' => mfna_bg_size()
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar:background-size_tablet',
							'condition' => array( 'id' => 'bg_menu-tmpl-sidebar', 'opt' => 'is', 'val' => 'default' ),
							'oldid' => 'bg_size',
							'type' => 'select',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-size',
							'title' => __('Size', 'mfn-opts'),
							'options' => mfna_bg_size()
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar:background-size_mobile',
							'condition' => array( 'id' => 'bg_menu-tmpl-sidebar', 'opt' => 'is', 'val' => 'default' ),
							'oldid' => 'bg_size',
							'type' => 'select',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-size',
							'title' => __('Size', 'mfn-opts'),
							'options' => mfna_bg_size()
						),
						array(
							'type' => 'subheader',
							'condition' => array( 'id' => 'sidebar_type', 'opt' => 'is', 'val' => '0' ),
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Close', 'mfn-opts'),
						),

						array(
							'id' => 'sidebar-menu-close-icon-position',
							'type' => 'switch',
							'condition' => array( 'id' => 'sidebar_type', 'opt' => 'is', 'val' => '0' ),
							'class' => 'mfn-toggled',
							'title' => __('Position', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),










						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-close-icon:font-size',
							'type' => 'sliderbar',
							'condition' => array( 'id' => 'sidebar_type', 'opt' => 'is', 'val' => '0' ),
							'title' => __('Size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '40',
								'step' => '1',
								'unit' => 'px',
							),
							'std' => '20px'
						),












						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu-toggle .icon:color',
							'type' => 'color',
							'condition' => array( 'id' => 'sidebar_type', 'opt' => 'is', 'val' => '0' ),
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Content overlay', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement.mfn-header-tmpl-menu-active|before:display',
							'type' => 'switch',
							'title' => __('Visibility', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'options' => [
								'' => __('Hidden', 'mfn-opts'),
								'block' => __('Visible', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement.mfn-header-tmpl-menu-active|before:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'condition' => array( 'id' => 'sidebar_type', 'opt' => 'is', 'val' => array('0', 'classic') ),
							'class' => 'toggled_header row-header',
							'title' => __('Menu', 'mfn-opts'),
						),

						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Items', 'mfn-opts'),
						),

						array(
							'id' => 'items_align',
							'condition' => array( 'id' => 'menu_icon_align', 'opt' => 'isnt', 'val' => 'top' ),
							'type' => 'switch',
							'title' => __('Alignment', 'mfn-opts'),
							'options' => array(
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							),
							'std' => 'top',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'items_align_laptop',
							'condition' => array( 'id' => 'menu_icon_align', 'opt' => 'isnt', 'val' => 'top' ),
							'type' => 'switch',
							'title' => __('Alignment', 'mfn-opts'),
							'options' => array(
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'items_align_tablet',
							'condition' => array( 'id' => 'menu_icon_align', 'opt' => 'isnt', 'val' => 'top' ),
							'type' => 'switch',
							'title' => __('Alignment', 'mfn-opts'),
							'options' => array(
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'items_align_mobile',
							'condition' => array( 'id' => 'menu_icon_align', 'opt' => 'isnt', 'val' => 'top' ),
							'type' => 'switch',
							'title' => __('Alignment', 'mfn-opts'),
							'options' => array(
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li:--mfn-header-menu-gap',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li:--mfn-header-menu-gap_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li:--mfn-header-menu-gap_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li:--mfn-header-menu-gap_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li > a:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li > a:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li > a:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li > a:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li > a:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li > a:border-style',
							'attr_id' => 'border_style_header_menu_link',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li > a:border-width',
		  				'condition' => array( 'id' => 'border_style_header_menu_link', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li > a:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_header_menu_link', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li > a:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_header_menu_link', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li > a:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_header_menu_link', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li > a:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li > a:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li > a:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li > a:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li > a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li > a:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li > a:border-color',
							'condition' => array( 'id' => 'border_style_header_menu_link', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li > a:box-shadow',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow'
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li > a:box-shadow_laptop',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li > a:box-shadow_tablet',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li > a:box-shadow_mobile',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li|hover > a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li|hover > a:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li|hover > a:border-color',
							'condition' => array( 'id' => 'border_style_header_menu_link', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li|hover > a:box-shadow',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow'
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li|hover > a:box-shadow_laptop',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li|hover > a:box-shadow_tablet',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li|hover > a:box-shadow_mobile',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li.current-menu-item > a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li.current-menu-item > a:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li.current-menu-item > a:border-color',
							'condition' => array( 'id' => 'border_style_header_menu_link', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li.current-menu-item > a:box-shadow',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow'
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li.current-menu-item > a:box-shadow_laptop',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li.current-menu-item > a:box-shadow_tablet',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li.current-menu-item > a:box-shadow_mobile',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Animation', 'mfn-opts'),
							'condition' => array( 'id' => 'header_menu_animation', 'opt' => 'isnt', 'val' => '' ),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-menu-li > .mfn-menu-link .label-wrapper|before:--mfn-header-menu-animation-color',
							'condition' => array( 'id' => 'header_menu_animation', 'opt' => 'isnt', 'val' => '' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Icon', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li:--mfn-header-menu-icon-gap',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li:--mfn-header-menu-icon-gap_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li:--mfn-header-menu-icon-gap_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li:--mfn-header-menu-icon-gap_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li:--mfn-header-menu-icon-size',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li:--mfn-header-menu-icon-size_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li:--mfn-header-menu-icon-size_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li:--mfn-header-menu-icon-size_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu .menu-icon > i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu a|hover > .menu-icon > i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li.current-menu-item > a > .menu-icon i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Submenu icon', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li:--mfn-header-menu-submenu-icon-gap',
							'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li:--mfn-header-menu-submenu-icon-gap_laptop',
							'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li:--mfn-header-menu-submenu-icon-gap_tablet',
							'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li:--mfn-header-menu-submenu-icon-gap_mobile',
							'condition' => array( 'id' => 'submenu_icon_display', 'opt' => 'is', 'val' => 'on' ),
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Gap', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .menu-sub i:--mfn-header-submenu-icon-size',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .menu-sub i:--mfn-header-submenu-icon-size_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .menu-sub i:--mfn-header-submenu-icon-size_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .menu-sub i:--mfn-header-submenu-icon-size_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Icon size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li > a .menu-sub i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li > a|hover .menu-sub i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu > li.current-menu-item > a .menu-sub i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'condition' => array( 'id' => 'sidebar_type', 'opt' => 'is', 'val' => '0,classic' ),
							'title' => __('Submenu', 'mfn-opts'),
						),

						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Container', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu li a:justify-content',
							'type' => 'switch',
							'responsive' => 'desktop',
							'title' => __('Alignment', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'flex-start' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'flex-end' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu li a:justify-content_laptop',
							'type' => 'switch',
							'responsive' => 'laptop',
							'title' => __('Alignment', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'flex-start' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'flex-end' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu li a:justify-content_tablet',
							'type' => 'switch',
							'responsive' => 'tablet',
							'title' => __('Alignment', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'flex-start' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'flex-end' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu li a:justify-content_mobile',
							'type' => 'switch',
							'responsive' => 'mobile',
							'title' => __('Alignment', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'flex-start' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'flex-end' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),


		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),





		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu:border-style',
							'attr_id' => 'border_style_header_submenu_link',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu:border-width',
		  				'condition' => array( 'id' => 'border_style_header_submenu_link', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_header_submenu_link', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_header_submenu_link', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_header_submenu_link', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu:border-color',
							'condition' => array( 'id' => 'border_style_header_submenu_link', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Items', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu li a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu li a:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu li|hover > a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu li|hover > a:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu li.current-menu-item > a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-header-tmpl-menu-sidebar .mfn-header-menu li .mfn-submenu li.current-menu-item > a:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						// advanced here

   				),
   			),

				// Header icon ----------------------------------------------------

   			'header_icon' => array(
   				'type' => 'header_icon',
   				'title' => __('Icon', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'header',
   				'attr' => array(
   				//'fields' => array(

						/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						// icon

						array(
							'type' => 'header',
							'title' => __('Icon', 'mfn-opts'),
						),

						array(
							'id' => 'type',
							'attr_id' => 'header_icon_type',
							're_render' => true,
							'type' => 'select',
							'title' => __('Icon type', 'mfn-opts'),
							'options' => array(
								'tel' => 'Phone',
								'mail' => 'E-mail',
								'search' => 'Search',
								'default' => 'Custom',
								'cart' => 'Shop cart',
								'account' => 'My account',
								'wishlist' => 'Wishlist',
							),
							'desc' => __('For icons: Shop cart, My account and Wishlist, woocommerce plugin is required', 'mfn-opts'),
							'std' => 'tel',
						),

						array(
							'id' => 'icon',
							'attr_id' => 'header_icon_font',
							'type' => 'icon',
							'title' => __('Icon', 'mfn-opts'),
							'preview' => 'icon',
						),

						array(
							'id' => 'image',
							'type' => 'upload',
							'dynamic_data' => 'featured_image',
							'title' => __('Image', 'mfn-opts'),
							'desc' => __('Image instead of an icon', 'mfn-opts'),
							'preview' => 'image',
						),

						// link

						array(
							'type' => 'header',
							'condition' => array( 'id' => 'header_icon_type', 'opt' => 'is', 'val' => 'tel,mail,default,cart,wishlist,search' ),
							'title' => __('Additional', 'mfn-opts'),
						),

						array(
							'id' => 'desc',
							'attr_id' => 'header-icon-desc',
							'type' => 'text',
							'edit_tag' => '.mfn-header-icon-box | div.desc-wrapper',
   						'edit_position' => 'append',
   						'dynamic_data' => 'title',
							'title' => __('Desc', 'mfn-opts'),
						),

						array(
							'id' => 'cart_total',
							'condition' => array( 'id' => 'header_icon_type', 'opt' => 'is', 'val' => 'cart' ),
							'type' => 'switch',
							're_render' => true,
							'title' => __('Cart total ', 'mfn-opts'),
							'options' => array(
								'0' => 'Hide',
								'1' => 'Show',
							),
							'std' => '1',
						),

						array(
							'id' => 'count',
							'attr_id' => 'header_icon_count',
							'condition' => array( 'id' => 'header_icon_type', 'opt' => 'is', 'val' => array('wishlist', 'cart') ),
							'type' => 'switch',
							're_render' => true,
							'title' => __('Icon count', 'mfn-opts'),
							'options' => array(
								'0' => 'Hide',
								'1' => 'Show',
							),
							'std' => '1',
						),

						array(
							'id' => 'count_if_zero',
							'condition' => array( 'id' => 'header_icon_type', 'opt' => 'is', 'val' => array('wishlist', 'cart') ),
							'type' => 'switch',
							're_render' => true,
							'title' => __('Icon count if 0', 'mfn-opts'),
							'options' => array(
								'1' => 'Hide',
								'' => 'Show',
							),
							'std' => '',
						),

						array(
							'id' => 'link',
							'condition' => array( 'id' => 'header_icon_type', 'opt' => 'is', 'val' => 'tel,mail,default' ),
							'type' => 'text',
							'title' => __('Link', 'mfn-opts'),
						),

						array(
							'id' => 'target',
							'condition' => array( 'id' => 'header_icon_type', 'opt' => 'is', 'val' => 'tel,mail,default' ),
							'type' => 'select',
							'title' => __('Target', 'mfn-opts'),
							'options' => array(
								0 => __('Default | _self', 'mfn-opts'),
								1 => __('New tab or window | _blank', 'mfn-opts'),
							),
						),

						array(
							'id' => 'link_title',
							'type' => 'text',
							'title' => __('Link title', 'mfn-opts'),
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Icon', 'mfn-opts'),
						),

						array(
							'id' => 'icon_position',
							//'condition' => array( 'id' => 'header-icon-desc', 'opt' => 'isnt', 'val' => '' ),
							'type' => 'switch',
							'title' => __('Icon position', 'mfn-opts'),
							'desc' => __('Work with desc only', 'mfn-opts'),
							'options' => array(
								'top' => __('Top', 'mfn-opts'),
								'bottom' => __('Bottom', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							),
							'std' => 'top',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'icon_position_tablet',
							//'condition' => array( 'id' => 'header-icon-desc', 'opt' => 'isnt', 'val' => '' ),
							'type' => 'switch',
							'title' => __('Icon position', 'mfn-opts'),
							'desc' => __('Work with desc only', 'mfn-opts'),
							'options' => array(
								'top' => __('Top', 'mfn-opts'),
								'bottom' => __('Bottom', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
								'' => __('Default', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'icon_position_laptop',
							//'condition' => array( 'id' => 'header-icon-desc', 'opt' => 'isnt', 'val' => '' ),
							'type' => 'switch',
							'title' => __('Icon position', 'mfn-opts'),
							'desc' => __('Work with desc only', 'mfn-opts'),
							'options' => array(
								'top' => __('Top', 'mfn-opts'),
								'bottom' => __('Bottom', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
								'' => __('Default', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'icon_position_mobile',
							//'condition' => array( 'id' => 'header-icon-desc', 'opt' => 'isnt', 'val' => '' ),
							'type' => 'switch',
							'title' => __('Icon position', 'mfn-opts'),
							'desc' => __('Work with desc only', 'mfn-opts'),
							'options' => array(
								'top' => __('Top', 'mfn-opts'),
								'bottom' => __('Bottom', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
								'' => __('Default', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'icon_align',
							'type' => 'switch',
							'condition' => array( 'id' => 'header-icon-desc', 'opt' => 'isnt', 'val' => '' ),
							'title' => __('Icon alignment', 'mfn-opts'),
							'options' => array(
								'start' => __('Start', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'end' => __('End', 'mfn-opts'),
							),
							'std' => 'center',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'icon_align_laptop',
							'condition' => array( 'id' => 'header-icon-desc', 'opt' => 'isnt', 'val' => '' ),
							'type' => 'switch',
							'title' => __('Icon alignment', 'mfn-opts'),
							'options' => array(
								'start' => __('Start', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'end' => __('End', 'mfn-opts'),
								'' => __('Default', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'icon_align_tablet',
							'condition' => array( 'id' => 'header-icon-desc', 'opt' => 'isnt', 'val' => '' ),
							'type' => 'switch',
							'title' => __('Icon alignment', 'mfn-opts'),
							'options' => array(
								'start' => __('Start', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'end' => __('End', 'mfn-opts'),
								'' => __('Default', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'icon_align_mobile',
							'condition' => array( 'id' => 'header-icon-desc', 'opt' => 'isnt', 'val' => '' ),
							'type' => 'switch',
							'title' => __('Icon alignment', 'mfn-opts'),
							'options' => array(
								'start' => __('Start', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'end' => __('End', 'mfn-opts'),
								'' => __('Default', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:--mfn-header-menu-icon-size',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '150',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:--mfn-header-menu-icon-size_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '150',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:--mfn-header-menu-icon-size_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '150',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:--mfn-header-menu-icon-size_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '150',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:width',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '150',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:width_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '150',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:width_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '150',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:width_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '150',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:height',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Height', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '150',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:height_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Height', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '150',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:height_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Height', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '150',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:height_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Height', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '150',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:border-style',
							'attr_id' => 'border_style_header_icon',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:border-color',
							'condition' => array( 'id' => 'border_style_header_icon', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:border-width',
							'condition' => array( 'id' => 'border_style_header_icon', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'dimensions',
							'title' => __('Border width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper:border-radius',
							'type' => 'dimensions',
							'title' => __('Border radius', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:--mfn-header-icon-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Icon color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box|hover .icon-wrapper:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box|hover .icon-wrapper:border-color',
							'condition' => array( 'id' => 'border_style_header_icon', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box|hover .icon-wrapper:--mfn-header-icon-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Icon color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'condition' => array( 'id' => 'header_icon_count', 'opt' => 'is', 'val' => '1' ),
							'title' => __('Icon count', 'mfn-opts'),
						),

						array(
							'id' => 'icon_count_posv',
							'attr_id' => 'icon_count_posv',
							'condition' => array( 'id' => 'header_icon_count', 'opt' => 'is', 'val' => '1' ),
							'type' => 'switch',
							'class' => 'mfn-toggled',
							'title' => __('Vertical offset', 'mfn-opts'),
							'options' => array(
								'0' => __('Top', 'mfn-opts'),
								'1' => __('Bottom', 'mfn-opts'),
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-cart-count,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-wishlist-count:top',
							'condition' => array( 'id' => 'icon_count_posv', 'opt' => 'is', 'val' => '0' ),
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Top', 'mfn-opts'),
							'class' => 'mfn-toggled header-icon-vert-count',
							'param' => array(
								'min' => '-50',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							),
							'default_value' => '-9px'
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-cart-count,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-wishlist-count:top_laptop',
							'condition' => array( 'id' => 'icon_count_posv', 'opt' => 'is', 'val' => '0' ),
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Top', 'mfn-opts'),
							'class' => 'mfn-toggled header-icon-vert-count',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-cart-count,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-wishlist-count:top_tablet',
							'condition' => array( 'id' => 'icon_count_posv', 'opt' => 'is', 'val' => '0' ),
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Top', 'mfn-opts'),
							'class' => 'mfn-toggled header-icon-vert-count',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-cart-count,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-wishlist-count:top_mobile',
							'condition' => array( 'id' => 'icon_count_posv', 'opt' => 'is', 'val' => '0' ),
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Top', 'mfn-opts'),
							'class' => 'mfn-toggled header-icon-vert-count',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-cart-count,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-wishlist-count:bottom',
							'condition' => array( 'id' => 'icon_count_posv', 'opt' => 'is', 'val' => '1' ),
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Bottom', 'mfn-opts'),
							'class' => 'mfn-toggled header-icon-vert-count',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-cart-count,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-wishlist-count:bottom_laptop',
							'condition' => array( 'id' => 'icon_count_posv', 'opt' => 'is', 'val' => '1' ),
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Bottom', 'mfn-opts'),
							'class' => 'mfn-toggled header-icon-vert-count',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-cart-count,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-wishlist-count:bottom_tablet',
							'condition' => array( 'id' => 'icon_count_posv', 'opt' => 'is', 'val' => '1' ),
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Bottom', 'mfn-opts'),
							'class' => 'mfn-toggled header-icon-vert-count',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-cart-count,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-wishlist-count:bottom_mobile',
							'condition' => array( 'id' => 'icon_count_posv', 'opt' => 'is', 'val' => '1' ),
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Bottom', 'mfn-opts'),
							'class' => 'mfn-toggled header-icon-vert-count',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'icon_count_posh',
							'attr_id' => 'icon_count_posh',
							'condition' => array( 'id' => 'header_icon_count', 'opt' => 'is', 'val' => '1' ),
							'type' => 'switch',
							'class' => 'mfn-toggled',
							'title' => __('Horizontal offset', 'mfn-opts'),
							'options' => array(
								'0' => __('Left', 'mfn-opts'),
								'1' => __('Right', 'mfn-opts'),
							),
							'std' => '1'
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-cart-count,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-wishlist-count:right',
							'condition' => array( 'id' => 'icon_count_posh', 'opt' => 'is', 'val' => '1' ),
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Right', 'mfn-opts'),
							'class' => 'mfn-toggled header-icon-hori-count',
							'param' => array(
								'min' => '-50',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							),
							'default_value' => '-11px'
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-cart-count,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-wishlist-count:right_laptop',
							'condition' => array( 'id' => 'icon_count_posh', 'opt' => 'is', 'val' => '1' ),
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Right', 'mfn-opts'),
							'class' => 'mfn-toggled header-icon-hori-count',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-cart-count,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-wishlist-count:right_tablet',
							'condition' => array( 'id' => 'icon_count_posh', 'opt' => 'is', 'val' => '1' ),
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Right', 'mfn-opts'),
							'class' => 'mfn-toggled header-icon-hori-count',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-cart-count,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-wishlist-count:right_mobile',
							'condition' => array( 'id' => 'icon_count_posh', 'opt' => 'is', 'val' => '1' ),
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Right', 'mfn-opts'),
							'class' => 'mfn-toggled header-icon-hori-count',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-cart-count,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-wishlist-count:left',
							'condition' => array( 'id' => 'icon_count_posh', 'opt' => 'is', 'val' => '0' ),
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Left', 'mfn-opts'),
							'class' => 'mfn-toggled header-icon-hori-count',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-cart-count,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-wishlist-count:left_laptop',
							'condition' => array( 'id' => 'icon_count_posh', 'opt' => 'is', 'val' => '0' ),
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Left', 'mfn-opts'),
							'class' => 'mfn-toggled header-icon-hori-count',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-cart-count,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-wishlist-count:left_tablet',
							'condition' => array( 'id' => 'icon_count_posh', 'opt' => 'is', 'val' => '0' ),
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Left', 'mfn-opts'),
							'class' => 'mfn-toggled header-icon-hori-count',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-cart-count,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-wishlist-count:left_mobile',
							'condition' => array( 'id' => 'icon_count_posh', 'opt' => 'is', 'val' => '0' ),
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Left', 'mfn-opts'),
							'class' => 'mfn-toggled header-icon-hori-count',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-cart-count,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-wishlist-count:color',
							'condition' => array( 'id' => 'header_icon_count', 'opt' => 'is', 'val' => '1' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-cart-count,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-wishlist-count:background-color',
							'condition' => array( 'id' => 'header_icon_count', 'opt' => 'is', 'val' => '1' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),



						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-cart-count,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .icon-wrapper .header-wishlist-count:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),


		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Desc', 'mfn-opts'),
						),

						array(
		  				'id' => 'header_icon_desc_visibility',
		  				'iconinfo' => 'https://support.muffingroup.com/video-tutorials/section-responsive-visibility/',
		  				'type' => 'switch',
		  				'invert' => true,
		  				'version' => 'multiple',
		  				'class' => 'mfn-toggled',
		  				'title' => __('Responsive visibility', 'mfn-opts'),
		  				'options' => array(
		  					'hide-desktop' => __('Hide on Desktop', 'mfn-opts'),
		  					'hide-tablet' => __('Hide on Tablet', 'mfn-opts'),
		  					'hide-mobile' => __('Hide on Mobile', 'mfn-opts'),
		  				),
		  				'active_tooltip' => array(
		  					'hide-desktop' => __('Show on Desktop', 'mfn-opts'),
		  					'hide-tablet' => __('Show on Tablet', 'mfn-opts'),
		  					'hide-mobile' => __('Show on Mobile', 'mfn-opts'),
		  				),
		  				'visual_options' => array(
		  					'hide-desktop' => '<svg id="Layer_1" data-name="Layer 1" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 22 22"><defs><style>.cls-1{fill:none;stroke:#304050;stroke-linecap:round;stroke-linejoin:round;stroke-width:1.5px;}.cls-2{fill:#304050;}</style></defs><path class="cls-1" d="M4.94,5H17.06a.93.93,0,0,1,.94.91v7.18a.93.93,0,0,1-.94.91H4.94A.93.93,0,0,1,4,13.09V5.91A.93.93,0,0,1,4.94,5Z"/><path class="cls-1" d="M7,17h8"/><path class="cls-2" d="M13,15.51a1.58,1.58,0,0,0,.7,1.26L14,17H8l.3-.23A1.58,1.58,0,0,0,9,15.51V14h4Z"/></svg>',			// 960 +
		  					'hide-tablet' => '<svg id="Layer_1" data-name="Layer 1" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 22 22"><defs><style>.cls-1{fill:none;stroke:#304050;stroke-linecap:round;stroke-linejoin:round;stroke-width:1.5px;}</style></defs><path class="cls-1" d="M7,15.83V7.17A1.21,1.21,0,0,1,8.25,6h5.5A1.21,1.21,0,0,1,15,7.17v8.66A1.21,1.21,0,0,1,13.75,17H8.25A1.21,1.21,0,0,1,7,15.83Z"/></svg>',		// 768 - 959
		  					'hide-mobile' => '<svg id="Layer_1" data-name="Layer 1" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 22 22"><defs><style>.cls-1{fill:none;stroke:#304050;stroke-linecap:round;stroke-linejoin:round;stroke-width:1.5px;}.cls-2{fill:#304050;}</style></defs><path class="cls-1" d="M8,15.82V8.18A1.08,1.08,0,0,1,8.94,7h4.12A1.08,1.08,0,0,1,14,8.18v7.64A1.08,1.08,0,0,1,13.06,17H8.94A1.08,1.08,0,0,1,8,15.82Z"/><circle class="cls-2" cx="11" cy="14" r="1"/></svg>',			// - 768
		  				),
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc-wrapper:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),


		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .desc-wrapper:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box|hover .desc-wrapper:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),






















						array(
							'type' => 'header',
							'condition' => array( 'id' => 'header_icon_type', 'opt' => 'is', 'val' => 'search' ),
							'class' => 'toggled_header row-header',
							'title' => __('Search results', 'mfn-opts'),
						),

						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Container', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-live-search-box:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-live-search-box:border-style',
							'attr_id' => 'border_live-search_results',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-live-search-box:border-width',
							'condition' => array( 'id' => 'border_live-search_results', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'dimensions',
							'title' => __('Border width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-live-search-box:border-color',
							'type' => 'color',
							'condition' => array( 'id' => 'border_live-search_results', 'opt' => 'isnt', 'val' => 'none' ),
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Label', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-live-search-box .mfn-live-search-heading:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-live-search-box .mfn-live-search-heading:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-live-search-box .mfn-live-search-heading:opacity',
							'type' => 'sliderbar',
							'class' => 'mfn-toggled',
							'title' => __('Opacity', 'mfn-opts'),
							'param' => array(
								'min' => '0',
								'max' => '1',
								'step' => '0.1',
							),
							'std' => '0.6'
						),


						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Items', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-live-search-box .mfn-live-search-texts:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-live-search-box li a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Link color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-live-search-box .mfn-live-search-texts span.mfn-ls-price:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Price color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-live-search-box .mfn-live-search-texts:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-live-search-box .mfn-live-search-texts p:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Desc typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-live-search-box .mfn-live-search-texts span.mfn-ls-price:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Price typography', 'mfn-opts'),
						),


						array(
							'type' => 'html',
							'html' => '</div>',
						),

						// advanced here

   				),
   			),

			// Header search ----------------------------------------------------

   			'header_search' => array(
   				'type' => 'header_search',
   				'title' => __('Search field', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'header',
   				'attr' => array(
   				//'fields' => array(

						/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'type' => 'header',
							'title' => __('Input', 'mfn-opts'),
						),

						array(
							'id' => 'placeholder',
							'type' => 'text',
							'title' => __('Placeholder', 'mfn-opts'),
							'std' => __('Enter your search', 'betheme'),
						),

						array(
   						'id' => 'icon',
   						'type' => 'icon',
   						'title' => __('Icon', 'mfn-opts'),
   					),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Input', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.field:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input:border-style',
							'attr_id' => 'border_style_header_search',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.field:border-width',
							'condition' => array( 'id' => 'border_style_header_search', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'dimensions',
							'title' => __('Border width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.field:border-radius',
							'type' => 'dimensions',
							'title' => __('Border radius', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.field:box-shadow',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow'
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.field:box-shadow_laptop',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.field:box-shadow_tablet',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.field:box-shadow_mobile',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),


		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.field:height',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Height', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '15',
								'max' => '80',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.field:height_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Height', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '15',
								'max' => '80',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.field:height_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Height', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '15',
								'max' => '80',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.field:height_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Height', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '15',
								'max' => '80',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.field:font-size',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Font size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '70',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.field:font-size_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Font size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '70',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.field:font-size_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Font size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '70',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.field:font-size_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Font size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '70',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.field:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.field:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.field:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.field:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.field:--mfn-header-search-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.field:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.field:border-color',
							'condition' => array( 'id' => 'border_style_header_search', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.field|focus:--mfn-header-search-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.field|focus:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.field|focus:border-color',
							'condition' => array( 'id' => 'border_style_header_search', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Icon', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .search_wrapper:--mfn-header-search-icon-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .search_wrapper:--mfn-header-search-icon-size',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '150',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .search_wrapper:--mfn-header-search-icon-size_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '150',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .search_wrapper:--mfn-header-search-icon-size_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '150',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .search_wrapper:--mfn-header-search-icon-size_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '5',
								'max' => '150',
								'step' => '1',
								'unit' => 'px',
							)
						),


						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .search_wrapper .icon_search:left',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Left offset', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '50',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .search_wrapper .icon_search:left_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Left offset', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '50',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .search_wrapper .icon_search:left_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Left offset', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '50',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .search_wrapper .icon_search:left_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Left offset', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '50',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

						// advanced here

   				),
   			),

				// Header Promo Bar ----------------------------------------------------

   			'header_promo_bar' => array(
   				'type' => 'header_promo_bar',
   				'title' => __('Promo bar', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'header',
   				'attr' => array(
   				//'fields' => array(

						/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
   						'id' => 'tabs',
   						'type' => 'tabs',
   						'title' => __('Slides', 'mfn-opts'),
							'options' => [
								'title' => [
									'textarea',
									__('Content', 'mfn-opts'),
									'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Mauris eu massa orci.',
								],
							],
							'std' => [
								0 => [
									'title' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Mauris eu massa orci.',
								],
							],
							'preview' => 'tabs',
							'primary' => 'title', // default
   					),

   					array(
   						'id' => 'slider_speed',
   						're_render' => true,
   						'type' => 'text',
   						'title' => __('Autoplay speed', 'mfn-opts'),
   						'std' => '3',
   						'after' => 's',
   						'param' => 'number',
   						'class' => 'narrow',
							'preview' => 'number',
   					),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Container', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:text-align',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'desktop',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:text-align_laptop',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'laptop',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:text-align_tablet',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'tablet',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:text-align_mobile',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'mobile',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:color',
							'type' => 'color',
							'class' => 'mfn-toggled main-color',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner a:color',
							'type' => 'color',
							'class' => 'mfn-toggled main-color',
							'title' => __('Link color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner a|hover:color',
							'type' => 'color',
							'class' => 'mfn-toggled main-color',
							'title' => __('Link color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

						// advanced here

   				),
   			),

				// Shop Products ----------------------------------------------------

   			'shop_products' => array(
   				'type' => 'shop_products',
   				'title' => __('Shop products', 'mfn-opts'),
   				'desc' => __('List of products from woocommerce', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'shop-archive',
   				'attr' => array(
   				//'fields' => array(

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
   						'id' => 'products',
   						'type' => 'text',
   						're_render' => true,
   						'title' => __('Products per page', 'mfn-opts'),
   						'std' => '12',
   						'after' => 'products',
   						'param' => 'number',
   						'class' => 'narrow',
							'preview' => 'number',
   					),

   					array(
   						'id' => 'layout',
   						'type' => 'select',
   						're_render' => true,
   						'title' => __('Columns', 'mfn-opts'),
							'options' => array(
								'grid' => __('Grid, 3 columns', 'mfn-opts'),
								'grid col-4' => __('Grid, 4 columns', 'mfn-opts'),
								'masonry' => __('Masonry, 3 columns', 'mfn-opts'),
								'list' => __('List', 'mfn-opts'),
							),
   						'std' => 'grid',
   					),

   					array(
   						'id' => 'ordering',
   						'type' => 'select',
   						're_render' => true,
   						'title' => __('Order', 'mfn-opts'),
							'options' => array(
								'' => __('Default', 'mfn-opts'),
								'menu_order' => __('Menu order', 'mfn-opts'),
								'popularity' => __('Sales', 'mfn-opts'),
								'rating' => __('Average rating', 'mfn-opts'),
								'date' => __('Most recent', 'mfn-opts'),
								'price' => __('Lowest price', 'mfn-opts'),
								'price-desc' => __('Highest price', 'mfn-opts'),
							),
   						'std' => '',
   					),

						// options

   					array(
   						'type' => 'header',
   						'title' => __('Options', 'mfn-opts'),
   					),

						array(
							'id' => 'description',
							're_render' => true,
							'type' => 'switch',
							'title' => __('Description', 'mfn-opts'),
							'options' => array(
								'0' => __('Hide', 'mfn-opts'),
								'1' => __('Show', 'mfn-opts'),
                'list' => __('List layout only', 'mfn-opts'),
							),
							'std' => '0'
						),

						array(
							'id' => 'button',
							're_render' => true,
							'type' => 'switch',
							'title' => __('Add to cart button', 'mfn-opts'),
							'desc' => __('Required for some plugins', 'mfn-opts'),
							'options' => array(
								'0' => __('Hide', 'mfn-opts'),
								'1' => __('Show', 'mfn-opts'),
                'list' => __('List layout only', 'mfn-opts'),
							),
							'std' => '0',
						),

						array(
							'id' => 'title_tag',
							'type' => 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => [
								'h1' => 'H1',
								'h2' => 'H2',
								'h3' => 'H3',
								'h4' => 'H4',
								'h5' => 'H5',
								'h6' => 'H6',
								'p' => 'p',
								'span' => 'span',
							],
							'std' => 'h2',
						),

						array(
							'id' => 'equal_heights',
							'attr_id' => 'equal_heights',
							're_render' => true,
							'type' => 'switch',
							'title' => __('Products List Standardization', 'mfn-opts'),
							'options' => [
								'0' => __('Default', 'mfn-opts'),
								'1' => __('Equal heights', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'equal_heights_last_el_class',
							'condition' => array( 'id' => 'equal_heights', 'opt' => 'is', 'val' => '1' ),
							'type' => 'select',
							're_render' => true,
							'title' => __('Align to the bottom from', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'image' => __('Image', 'mfn-opts'),
								'title' => __('Title', 'mfn-opts'),
								'price' => __('Price', 'mfn-opts'),
								'variations' => __('Variations', 'mfn-opts'),
								'description' => __('Description', 'mfn-opts'),
								'button' => __('Button', 'mfn-opts'),
							],
							'std' => '',
						),


						array(
   						'type' => 'header',
   						'title' => __('Filters', 'mfn-opts'),
   					),

						array(
							'id' => 'shop-list-active-filters',
							'type' => 'switch',
							'class' => 'shop-products-filters-switcher',
							'title' => __('Active filters', 'mfn-opts'),
							'options' => array(
								'0' => __('Hide', 'mfn-opts'),
								'1' => __('Show', 'mfn-opts'),
							),
							'std' => mfn_opts_get('shop-list-active-filters'),
						),

						array(
							'id' => 'shop-list-perpage',
							'type' => 'switch',
							'class' => 'shop-products-filters-switcher',
							'title' => __('Products per page', 'mfn-opts'),
							'options' => array(
								'0' => __('Hide', 'mfn-opts'),
								'1' => __('Show', 'mfn-opts'),
							),
							'std' => mfn_opts_get('shop-list-perpage'),
						),

						array(
							'id' => 'shop-list-layout',
							'class' => 'shop-products-filters-switcher',
							'type' => 'switch',
							'title' => __('Layout switch', 'mfn-opts'),
							'options' => array(
								'0' => __('Hide', 'mfn-opts'),
								'1' => __('Show', 'mfn-opts'),
							),
							'std' => mfn_opts_get('shop-list-layout'),
						),

						array(
							'id' => 'shop-list-sorting',
							'class' => 'shop-products-filters-switcher',
							'type' => 'switch',
							'title' => __('List sorting', 'mfn-opts'),
							'options' => array(
								'0' => __('Hide', 'mfn-opts'),
								'1' => __('Show', 'mfn-opts'),
							),
							'std' => mfn_opts_get('shop-list-sorting'),
						),

						array(
							'id' => 'shop-list-results-count',
							'class' => 'shop-products-filters-switcher',
							'type' => 'switch',
							'title' => __('Results count', 'mfn-opts'),
							'options' => array(
								'0' => __('Hide', 'mfn-opts'),
								'1' => __('Show', 'mfn-opts'),
							),
							'std' => mfn_opts_get('shop-list-results-count'),
						),


						// order

						array(
							'type' => 'header',
							'title' => __('Order', 'mfn-opts'),
						),

						array(
							'id' => 'order',
							'type' => 'order',
							're_render' => true,
							'title' => __('Order', 'mfn-opts'),
							'std' => 'image,title,price,description,button',
						),

						// HTML style

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

   					// HTML end: style

   					array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Container', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:text-align',
							'oldid' => 'style:li.product:text-align',
							'type' => 'switch',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'title' => __('Text align', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:text-align_laptop',
							'oldid' => 'style:li.product:text-align',
							'type' => 'switch',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'title' => __('Text align', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:text-align_tablet',
							'oldid' => 'style:li.product:text-align',
							'type' => 'switch',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'title' => __('Text align', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:text-align_mobile',
							'oldid' => 'style:li.product:text-align',
							'type' => 'switch',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'title' => __('Text align', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-style',
							'attr_id' => 'border_style_shop_li',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-color',
							'condition' => array( 'id' => 'border_style_shop_li', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-width',
		  				'condition' => array( 'id' => 'border_style_shop_li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_shop_li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_shop_li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_shop_li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Image', 'mfn-opts'),
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-style',
							'attr_id' => 'border_style_shopimg',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-color',
							'condition' => array( 'id' => 'border_style_shopimg', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-width',
		  				'condition' => array( 'id' => 'border_style_shopimg', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_shopimg', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_shopimg', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_shopimg', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			// title

						array(
							'title' => __('Title', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .title:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .title:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'title' => __('Price', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .price:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .price:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .price:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .price:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .price:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .price:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'title' => __('Desc', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .excerpt:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .excerpt:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .excerpt:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .excerpt:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .excerpt:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .excerpt:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Attributes', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-variations-wrapper-loop .mfn-vr ul.mfn-vr-options li a:border-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-variations-wrapper-loop .mfn-vr ul.mfn-vr-options li a|hover:border-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color on Hover', 'mfn-opts'),
						),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Sale label', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .onsale:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .onsale:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .onsale:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .onsale:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .onsale:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .onsale:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Button', 'mfn-opts'),
   					),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-style',
							'attr_id' => 'border_style_bsb',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-width',
		  				'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-color',
							'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher',
							'attr_id' => 'background_shop_button',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:background-color',
							'condition' => array( 'id' => 'background_shop_button', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:gradient',
		  				'condition' => array( 'id' => 'background_shop_button', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button|hover:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button|hover:border-color',
							'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher_hover',
							'attr_id' => 'background_hover_shop_button',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button|hover:background',
							'condition' => array( 'id' => 'background_hover_shop_button', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button|hover:gradient',
		  				'condition' => array( 'id' => 'background_hover_shop_button', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),


						array(
							'title' => __('List options container', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),


						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters:border-style',
							'attr_id' => 'border_style_shop_filters',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters:border-width',
		  				'condition' => array( 'id' => 'border_style_shop_filters', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_shop_filters', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_shop_filters', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_shop_filters', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters:border-color',
							'condition' => array( 'id' => 'border_style_shop_filters', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),


						array(
							'title' => __('Result count', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-result-count:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-result-count:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'title' => __('Sorting', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters .woocommerce-ordering select:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters .woocommerce-ordering select:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters .woocommerce-ordering select:background',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters .woocommerce-ordering select:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters .woocommerce-ordering select:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters .woocommerce-ordering select:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters .woocommerce-ordering select:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters .woocommerce-ordering select:border-style',
							'attr_id' => 'border_style_shop_order',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters .woocommerce-ordering select:border-width',
		  				'condition' => array( 'id' => 'border_style_shop_order', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters .woocommerce-ordering select:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_shop_order', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters .woocommerce-ordering select:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_shop_order', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters .woocommerce-ordering select:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_shop_order', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters .woocommerce-ordering select:border-color',
							'condition' => array( 'id' => 'border_style_shop_order', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters .woocommerce-ordering select:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters .woocommerce-ordering select:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters .woocommerce-ordering select:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters .woocommerce-ordering select:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'title' => __('List options', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters .mfn-woo-list-options form.mfn-before-products-list-form:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters .mfn-woo-list-options form.mfn-before-products-list-form:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'title' => __('Layout options', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters .mfn-woo-list-options ul li svg:width',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Size', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters .mfn-woo-list-options ul li svg:width_laptop',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Size', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters .mfn-woo-list-options ul li svg:width_tablet',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Size', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters .mfn-woo-list-options ul li svg:width_mobile',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Size', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters a.open-filters svg .path, .mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters .mfn-woo-list-options ul li:--mfn-woo-icon-option',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'title' => __('Rating', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .star-rating:display',
							'attr_id' => 'shop-list-rating',
							'type' => 'switch',
							'class' => 'mfn-toggled',
							'title' => __('Visibility', 'mfn-opts'),
							'options' => [
								'' => __('Visible', 'mfn-opts'),
								'none' => __('Hidden', 'mfn-opts'),
							],
							'std' => ''
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .star-rating span:color',
							'condition' => array( 'id' => 'shop-list-rating', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'title' => __('Filters switcher', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters a.open-filters:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters a.open-filters:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters a.open-filters svg:width',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Icon size', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters a.open-filters svg:width_laptop',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Icon size', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters a.open-filters svg:width_tablet',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Icon size', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop-filters a.open-filters svg:width_mobile',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Icon size', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),


						array(
   						'type' => 'html',
   						'html' => '</div>',
   					),

   				),
   			),

   			// Shop Title ----------------------------------------------------

   			'shop_title' => array(
   				'type' => 'shop_title',
   				'title' => __('Shop title', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'shop-archive',
   				'attr' => array(
   				//'fields' => array(

						array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
							'id' => 'title_tag',
							'type' => 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => [
								'h1' => 'H1',
								'h2' => 'H2',
								'h3' => 'H3',
								'h4' => 'H4',
								'h5' => 'H5',
								'h6' => 'H6',
								'p' => 'p',
								'span' => 'span',
							],
							'std' => 'h1',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						// title

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Title', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .page-title:text-align',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'desktop',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .page-title:text-align_laptop',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'laptop',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .page-title:text-align_tablet',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'tablet',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .page-title:text-align_mobile',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'mobile',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .page-title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .page-title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

   					// HTML end: style

						array(
   						'type' => 'html',
   						'html' => '</div>',
   					),

   				),
   			),


				// ALIAS: category description ----------------------------------------------------

   			'shop_cat_desc' => array(
   				'type' => 'plain_text',
   				'title' => __('Category Description', 'mfn-opts'),
   				'cat' => 'shop-archive',
   			),

   			// ALIAS: category top description ----------------------------------------------------

   			'shop_cat_top_desc' => array(
   				'type' => 'plain_text',
   				'title' => __('Category <br>Top Content', 'mfn-opts'),
   				'cat' => 'shop-archive',
   			),

   			// ALIAS: category bottom description ----------------------------------------------------

   			'shop_cat_bottom_desc' => array(
   				'type' => 'plain_text',
   				'title' => __('Category <br>Bottom Content', 'mfn-opts'),
   				'cat' => 'shop-archive',
   			),






















   			// Product Title ----------------------------------------------------

   			'product_title' => array(
   				'type' => 'product_title',
   				'title' => __('Product title', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'single-product',
   				'attr' => array(
   				//'fields' => array(

						array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
							'id' => 'title_tag',
							'type' => 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => [
								'h1' => 'H1',
								'h2' => 'H2',
								'h3' => 'H3',
								'h4' => 'H4',
								'h5' => 'H5',
								'h6' => 'H6',
								'p' => 'p',
								'span' => 'span',
							],
							'std' => 'h1',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						// title

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Title', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:text-align',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'desktop',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:text-align_laptop',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'laptop',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:text-align_tablet',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'tablet',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:text-align_mobile',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'mobile',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

   					// HTML end: style

						array(
   						'type' => 'html',
   						'html' => '</div>',
   					),

   				),
   			),

   			// Product images ----------------------------------------------------

   			'product_images' => array(
   				'type' => 'product_images',
   				'title' => __('Product images', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'single-product',
   				'attr' => array(
   				//'fields' => array(

						array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						// options

   					array(
   						'type' => 'header',
   						'title' => __('Options', 'mfn-opts'),
   					),

   					array(
   						'id' => 'zoom',
   						'type' => 'switch',
   						'title' => __('Zoom effect', 'mfn-opts'),
							'options' => array(
								'0' => __('Disable', 'mfn-opts'),
								'1' => __('Enable', 'mfn-opts'),
							),
   						'std' => '1',
   					),

   					array(
   						'type' => 'header',
   						'title' => __('Arrows', 'mfn-opts'),
   					),

   					array(
   						'id' => 'thumbnail_arrows',
   						'attr_id' => 'thumbnail_arrows',
   						'type' => 'switch',
   						'title' => __('Arrows', 'mfn-opts'),
   						'desc' => __('Works with thumbnail horizontal styles', 'mfn-opts'),
							'options' => array(
								'' => __('Disable', 'mfn-opts'),
								'1' => __('Enable', 'mfn-opts'),
							),
   					),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						// image

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Image', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-has-gallery .flex-viewport, .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-hasnt-gallery .woocommerce-product-gallery__image:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-has-gallery .flex-viewport, .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-hasnt-gallery .woocommerce-product-gallery__image:border-style',
							'attr_id' => 'border_style_prodimg',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-has-gallery .flex-viewport, .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-hasnt-gallery .woocommerce-product-gallery__image:border-color',
							'condition' => array( 'id' => 'border_style_prodimg', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-has-gallery .flex-viewport, .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-hasnt-gallery .woocommerce-product-gallery__image:border-width',
		  				'condition' => array( 'id' => 'border_style_prodimg', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-has-gallery .flex-viewport, .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-hasnt-gallery .woocommerce-product-gallery__image:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_prodimg', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-has-gallery .flex-viewport, .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-hasnt-gallery .woocommerce-product-gallery__image:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_prodimg', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-has-gallery .flex-viewport, .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-hasnt-gallery .woocommerce-product-gallery__image:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_prodimg', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-has-gallery .flex-viewport, .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-hasnt-gallery .woocommerce-product-gallery__image:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-has-gallery .flex-viewport, .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-hasnt-gallery .woocommerce-product-gallery__image:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-has-gallery .flex-viewport, .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-hasnt-gallery .woocommerce-product-gallery__image:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-has-gallery .flex-viewport, .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-hasnt-gallery .woocommerce-product-gallery__image:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

						// thumbnails

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Thumbnails', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flex-control-thumbs li img:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),


						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flex-control-thumbs li:border-style',
							'attr_id' => 'border_style_prodthumbs',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flex-control-thumbs li:border-color',
							'condition' => array( 'id' => 'border_style_prodthumbs', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flex-control-thumbs li:border-width',
		  				'condition' => array( 'id' => 'border_style_prodthumbs', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flex-control-thumbs li:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_prodthumbs', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flex-control-thumbs li:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_prodthumbs', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flex-control-thumbs li:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_prodthumbs', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flex-control-thumbs li:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flex-control-thumbs li:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flex-control-thumbs li:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flex-control-thumbs li:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'condition' => array( 'id' => 'thumbnail_arrows', 'opt' => 'is', 'val' => '1' ),
							'title' => __('Arrows', 'mfn-opts'),
						),
		
						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:--mfn-swiper-arrow-size',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Size', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:--mfn-swiper-arrow-size_laptop',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Size', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:--mfn-swiper-arrow-size_tablet',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Size', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:--mfn-swiper-arrow-size_mobile',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Size', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-product-gallery:--mfn-swiper-arrow-offset',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Offset', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-product-gallery:--mfn-swiper-arrow-offset_laptop',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Offset', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-product-gallery:--mfn-swiper-arrow-offset_tablet',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Offset', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-product-gallery:--mfn-swiper-arrow-offset_mobile',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Offset', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-swiper-arrow i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-swiper-arrow|hover i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

   					// HTML end: style

						array(
   						'type' => 'html',
   						'html' => '</div>',
   					),

   				),
   			),

   			// Product price ----------------------------------------------------

   			'product_price' => array(
   				'type' => 'product_price',
   				'title' => __('Product price', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'single-product',
   				'attr' => array(
   				//'fields' => array(

						array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'id' => 'info',
							'class' => 'mfn-alert',
   						'type' => 'info',
   						'title' => __('This element has no attributes. Please check <b>style</b> tab for more customisation options.', 'mfn-opts'),
   					),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Price', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .price:text-align',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'desktop',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .price:text-align_laptop',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'laptop',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .price:text-align_tablet',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'tablet',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .price:text-align_mobile',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'mobile',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .price:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .price:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Old price', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .price > del:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .price > del:color',
							'oldid' => 'style:.price > del:color',
							'type' => 'color',
							'class' => 'sales-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
							'desc' => __('Old price on sale', 'mfn-opts'),
						),

   					// HTML end: style

						array(
   						'type' => 'html',
   						'html' => '</div>',
   					),

   				),
   			),

   			// Product add to cart ----------------------------------------------------

   			'product_cart_button' => array(
   				'type' => 'product_cart_button',
   				'title' => __('Add to cart', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'single-product',
   				'attr' => array(
   				//'fields' => array(

						array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'id' => 'cart_button_text',
							'type' => 'text',
							'title' => __('Button text', 'mfn-opts'),
							'std' => __('Add to cart', 'woocommerce'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Attributes', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-variations-wrapper .mfn-vr label:color',
							'type' => 'color',
							'class' => 'sales-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

   					array(
							'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-variations-wrapper .mfn-vr label:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-variations-wrapper .mfn-vr ul.mfn-vr-options li a:border-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-variations-wrapper .mfn-vr ul.mfn-vr-options li a|hover:border-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color on Hover', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-variations-wrapper .mfn-vr:border-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Line separator', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Button', 'mfn-opts'),
   					),

		  			array(
		  				'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-add-to-cart .button:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-add-to-cart .button:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-add-to-cart .button:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-add-to-cart .button:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-add-to-cart .button:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-add-to-cart .button:border-style',
							'attr_id' => 'border_style_bsb',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-add-to-cart .button:border-width',
		  				'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-add-to-cart .button:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-add-to-cart .button:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-add-to-cart .button:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-add-to-cart .button:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-add-to-cart .button:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-add-to-cart .button:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-add-to-cart .button:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-add-to-cart .button:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-add-to-cart .button:border-color',
							'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher',
							'attr_id' => 'background_shop_button',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-add-to-cart .button:background-color',
							'condition' => array( 'id' => 'background_shop_button', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-add-to-cart .button:gradient',
		  				'condition' => array( 'id' => 'background_shop_button', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-add-to-cart .button|hover:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-add-to-cart .button|hover:border-color',
							'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher_hover',
							'attr_id' => 'background_hover_shop_button',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-add-to-cart .button|hover:background',
							'condition' => array( 'id' => 'background_hover_shop_button', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-product-add-to-cart .button|hover:gradient',
		  				'condition' => array( 'id' => 'background_hover_shop_button', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Quantity', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .quantity input.qty:background',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .quantity input.qty:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .quantity input.qty:border-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .quantity a.quantity-change:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Plus/Minus color', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Variation', 'mfn-opts'),
   					),

						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Price', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-variation-price:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-variation-price:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-variation-price:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-variation-price:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
							'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-variation-price .price:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-variation-price .price:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Product breadcrumbs ----------------------------------------------------

   			'product_breadcrumbs' => array(
   				'type' => 'product_breadcrumbs',
   				'title' => __('Breadcrumbs', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'single-product',
   				'attr' => array(
   				//'fields' => array(

						array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'type' => 'header',
							'title' => __('Custom', 'mfn-opts'),
						),

						array(
							'id' => 'breadcrumb_delimiter',
							'type' => 'text',
							're_render' => true,
							'title' => __('Delimiter', 'mfn-opts'),
							'std' => '/',
						),

						array(
							'id' => 'breadcrumb_home',
							'type' => 'switch',
							're_render' => true,
							'title' => __('Home page', 'mfn-opts'),
							'options' => [
								'0' => 'No',
								'1' => 'Include',
							],
							'std' => '1',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),


						array(
							'title' => __('Breadcrumbs', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-breadcrumb:text-align',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'desktop',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-breadcrumb:text-align_laptop',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'laptop',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),


						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-breadcrumb:text-align_tablet',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'tablet',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-breadcrumb:text-align_mobile',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'mobile',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-breadcrumb:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-breadcrumb:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'title' => __('Delimiter', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-breadcrumb span:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-breadcrumb span:opacity',
							'type' => 'sliderbar',
							'class' => 'mfn-toggled',
							'title' => __('Opacity', 'mfn-opts'),
							'param' => array(
								'min' => '0',
								'max' => '1',
								'step' => '0.1',
							),
							'std' => '0.2'
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Product reviews ----------------------------------------------------

   			'product_reviews' => array(
   				'type' => 'product_reviews',
   				'title' => __('Product reviews', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'single-product',
   				'attr' => array(
   				//'fields' => array(

						array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'id' => 'info',
							'class' => 'mfn-alert',
   						'type' => 'info',
   						'title' => __('This item can only be used once per page (including product tab reviews)', 'mfn-opts'),
   					),

						array(
							'id' => 'info',
							'class' => 'mfn-alert',
   						'type' => 'info',
   						'title' => __('This element has no attributes. Please check <b>style</b> tab for more customisation options.', 'mfn-opts'),
   					),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'title' => __('Title', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-Reviews-title:text-align',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'desktop',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),


						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-Reviews-title:text-align_laptop',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'laptop',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-Reviews-title:text-align_tablet',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'tablet',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-Reviews-title:text-align_mobile',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'mobile',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-Reviews-title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-Reviews-title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'title' => __('Container', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #reviews #comments ol.commentlist li .comment-text:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #reviews #comments ol.commentlist li .comment-text p.meta .woocommerce-review__author:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Author name color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #reviews #comments ol.commentlist li .comment-text p.meta .woocommerce-review__published-date,.mcb-section .mcb-wrap .mcb-item-mfnuidelement #reviews #comments ol.commentlist li .comment-text p.meta .woocommerce-review__dash:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Date color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #reviews #comments ol.commentlist li .comment-text .description:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Comment color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #reviews #comments ol.commentlist li .comment-text p.meta:border-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Line color', 'mfn-opts'),
						),

						array(
							'title' => __('Stars', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .star-rating span,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .comment-form-rating p.stars a|before:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'title' => __('Form title', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #respond .comment-reply-title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #respond .comment-reply-title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'title' => __('Your rating', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #reviews #review_form_wrapper .comment-form label:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Label', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .comment-form-rating p.stars a:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Number', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .comment-form-rating p.stars a:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Textarea', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #reviews #review_form_wrapper .comment-form .comment-form-comment textarea:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #reviews #review_form_wrapper .comment-form .comment-form-comment textarea,.mcb-section .mcb-wrap .mcb-item-mfnuidelement #reviews #review_form_wrapper .comment-form .comment-form-comment textarea|focus:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #reviews #review_form_wrapper .comment-form .comment-form-comment textarea:border-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Button', 'mfn-opts'),
   					),

		  			array(
		  				'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement #review_form #respond .form-submit input:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement #review_form #respond .form-submit input:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement #review_form #respond .form-submit input:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement #review_form #respond .form-submit input:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement #review_form #respond .form-submit input:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement #review_form #respond .form-submit input:border-style',
							'attr_id' => 'border_style_prodrev',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement #review_form #respond .form-submit input:border-width',
		  				'condition' => array( 'id' => 'border_style_prodrev', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement #review_form #respond .form-submit input:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_prodrev', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement #review_form #respond .form-submit input:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_prodrev', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement #review_form #respond .form-submit input:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_prodrev', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement #review_form #respond .form-submit input:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement #review_form #respond .form-submit input:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement #review_form #respond .form-submit input:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement #review_form #respond .form-submit input:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement #review_form #respond .form-submit input:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement #review_form #respond .form-submit input:border-color',
							'condition' => array( 'id' => 'border_style_prodrev', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher',
							'attr_id' => 'background_shop_button',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement #review_form #respond .form-submit input:background-color',
							'condition' => array( 'id' => 'background_shop_button', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement #review_form #respond .form-submit input:gradient',
		  				'condition' => array( 'id' => 'background_shop_button', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement #review_form #respond .form-submit input|hover:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement #review_form #respond .form-submit input|hover:border-color',
							'condition' => array( 'id' => 'border_style_prodrev', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher_hover',
							'attr_id' => 'background_hover_shop_button',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement #review_form #respond .form-submit input|hover:background',
							'condition' => array( 'id' => 'background_hover_shop_button', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.woocommerce #Content .mcb-section .mcb-wrap .mcb-item-mfnuidelement #review_form #respond .form-submit input|hover:gradient',
		  				'condition' => array( 'id' => 'background_hover_shop_button', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

   					// HTML end: style

						array(
   						'type' => 'html',
   						'html' => '</div>',
   					),

   				),
   			),

   			// Product rating ----------------------------------------------------

   			'product_rating' => array(
   				'type' => 'product_rating',
   				'title' => __('Product rating', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'single-product',
   				'attr' => array(
   				//'fields' => array(

						array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'id' => 'info',
							'class' => 'mfn-alert',
   						'type' => 'info',
   						'title' => __('This element has no attributes. Please check <b>style</b> tab for more customisation options.', 'mfn-opts'),
   					),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'title' => __('Rating', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-review-link:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .star-rating span:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Active stars color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .star-rating|before:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Inactive stars color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-product-rating:text-align',
							'class' => 'mfn-toggled',
							'responsive' => 'desktop',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-product-rating:text-align_laptop',
							'class' => 'mfn-toggled',
							'responsive' => 'laptop',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-product-rating:text-align_tablet',
							'class' => 'mfn-toggled',
							'responsive' => 'tablet',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-product-rating:text-align_mobile',
							'class' => 'mfn-toggled',
							'responsive' => 'mobile',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
   						'type' => 'html',
   						'html' => '</div>',
   					),

   				),
   			),


   			// Product stock ----------------------------------------------------

   			'product_stock' => array(
   				'type' => 'product_stock',
   				'title' => __('Product stock', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'single-product',
   				'attr' => array(
   				//'fields' => array(

						array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'id' => 'info',
							'class' => 'mfn-alert',
   						'type' => 'info',
   						'title' => __('To display stock, please enable stock management at product level and set its quantity. This element has no attributes. Please check <b>style</b> tab for more customisation options.', 'mfn-opts'),
   					),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'title' => __('Product stock', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .stock:text-align',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'desktop',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .stock:text-align_laptop',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'laptop',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .stock:text-align_tablet',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'tablet',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .stock:text-align_mobile',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'mobile',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .stock:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .stock.in-stock:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('In stock color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .stock.out-of-stock:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Out of stock color', 'mfn-opts'),
						),

   					// HTML end: style

						array(
   						'type' => 'html',
   						'html' => '</div>',
   					),

   				),
   			),

   			// Product meta ----------------------------------------------------

   			'product_meta' => array(
   				'type' => 'product_meta',
   				'title' => __('Product meta', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'single-product',
   				'attr' => array(
   				//'fields' => array(

						array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'type' => 'header',
							'title' => __('Options', 'mfn-opts'),
						),

						array(
							'id' => 'layout',
							'attr_id' => 'prod_meta_layout',
							'type' => 'switch',
							're_render' => true,
							'title' => __('Layout', 'mfn-opts'),
							'options' => [
								'inline' => __('Inline', 'mfn-opts'),
								'stacked' => __('Stacked', 'mfn-opts'),
								'table' => __('Table', 'mfn-opts'),
							],
							'std' => 'inline',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'title' => __('Container', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .product_meta, .mcb-section .mcb-wrap .mcb-item-mfnuidelement .product_meta th, .mcb-section .mcb-wrap .mcb-item-mfnuidelement .product_meta td:text-align',
							'oldid' => 'style:.product_meta, .product_meta th, .product_meta td:text-align',
							'type' => 'switch',
							'class' => 'mfn-toggled',
							'title' => __('Text align', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .product_meta .stacked-meta li:border-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Stacked line color', 'mfn-opts'),
						),

						array(
							'title' => __('Label', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn_product_meta span.tagged_as,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn_product_meta span.posted_in,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn_product_meta span.stacked-meta-title,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .table-meta th:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn_product_meta span.tagged_as,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn_product_meta span.posted_in,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn_product_meta span.stacked-meta-title,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .table-meta th:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'title' => __('Value', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .table-meta td,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .table-meta td,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .table-meta td a,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn_product_meta .stacked-categories span.stacked-meta-value a,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn_product_meta span.posted_in a:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Links color', 'mfn-opts'),
						),

						array(
							'title' => __('Tags', 'mfn-opts'),
							'condition' => array( 'id' => 'prod_meta_layout', 'opt' => 'isnt', 'val' => 'table' ),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .product_meta .stacked-meta li.stacked-tags .stacked-meta-value a,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .tagged_as a:color',
							'condition' => array( 'id' => 'prod_meta_layout', 'opt' => 'isnt', 'val' => 'table' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .product_meta .stacked-meta li.stacked-tags .stacked-meta-value a,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .tagged_as a:background',
							'condition' => array( 'id' => 'prod_meta_layout', 'opt' => 'isnt', 'val' => 'table' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'title' => __('SKU', 'mfn-opts'),
							'condition' => array( 'id' => 'prod_meta_layout', 'opt' => 'isnt', 'val' => 'table' ),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .stacked-sku .stacked-meta-value,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn_product_meta .sku_wrapper:color',
							'condition' => array( 'id' => 'prod_meta_layout', 'opt' => 'isnt', 'val' => 'table' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .stacked-sku .stacked-meta-value,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn_product_meta .sku_wrapper:background',
							'condition' => array( 'id' => 'prod_meta_layout', 'opt' => 'isnt', 'val' => 'table' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .stacked-sku .stacked-meta-value,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn_product_meta .sku_wrapper:border-color',
							'condition' => array( 'id' => 'prod_meta_layout', 'opt' => 'isnt', 'val' => 'table' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

   					// HTML end: style

						array(
   						'type' => 'html',
   						'html' => '</div>',
   					),

   				),
   			),

   			// Product short_description ----------------------------------------------------

   			'product_short_description' => array(
   				'type' => 'product_short_description',
   				'title' => __('Short description', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'single-product',
   				'attr' => array(
   				//'fields' => array(

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'id' => 'info',
							'class' => 'mfn-alert',
   						'type' => 'info',
   						'title' => __('This element has no attributes. Please check <b>style</b> tab for more customisation options.', 'mfn-opts'),
   					),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'title' => __('Description', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-product-details__short-description:text-align',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'desktop',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-product-details__short-description:text-align_laptop',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'laptop',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-product-details__short-description:text-align_tablet',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'tablet',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-product-details__short-description:text-align_mobile',
							'oldid' => 'style:.page-title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'mobile',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-product-details__short-description:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-product-details__short-description:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

   					// HTML end: style

						array(
   						'type' => 'html',
   						'html' => '</div>',
   					),

   				),
   			),

   			// Product tabs ----------------------------------------------------

   			'product_tabs' => array(
   				'type' => 'product_tabs',
   				'title' => __('Product tabs', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'single-product',
   				'attr' => array(
   				//'fields' => array(

						array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'type' => 'header',
							'title' => __('Navigation', 'mfn-opts'),
						),

						array(
							'id' => 'nav',
							'attr_id' => 'nav',
							'type' => 'select',
							'title' => __('Position', 'mfn-opts'),
							'options' => [
								'' => __('Top', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs.mfn-woocommerce-tabs-nav-top .mfn-woocommerce-tabs-nav:justify-content',
							'type' => 'select',
							'condition' => array( 'id' => 'nav', 'opt' => 'is', 'val' => '' ),
							'title' => __('Align', 'mfn-opts'),
							'options' => [
								'' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'flex-end' => __('Right', 'mfn-opts'),
								'space-around' => __('Space around', 'mfn-opts'),
								'space-between' => __('Space between', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Navigation', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav:border-style',
							'attr_id' => 'border_tabs_nav_ul',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav:border-color',
							'condition' => array( 'id' => 'border_tabs_nav_ul', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav:border-width',
		  				'condition' => array( 'id' => 'border_tabs_nav_ul', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav:border-width_laptop',
		  				'condition' => array( 'id' => 'border_tabs_nav_ul', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav:border-width_tablet',
		  				'condition' => array( 'id' => 'border_tabs_nav_ul', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav:border-width_mobile',
		  				'condition' => array( 'id' => 'border_tabs_nav_ul', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Nav items', 'mfn-opts'),
						),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav li a:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav li a:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav li a:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav li a:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav li:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav li:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav li:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav li:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav li a:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav li a:border-style',
							'attr_id' => 'border_tabs_nav_ul_li',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav li a:border-width',
		  				'condition' => array( 'id' => 'border_tabs_nav_ul_li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav li a:border-width_laptop',
		  				'condition' => array( 'id' => 'border_tabs_nav_ul_li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav li a:border-width_tablet',
		  				'condition' => array( 'id' => 'border_tabs_nav_ul_li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav li a:border-width_mobile',
		  				'condition' => array( 'id' => 'border_tabs_nav_ul_li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav li a:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav li a:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav li a:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav li a:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),


		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav li a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav li a:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav li a:border-color',
							'condition' => array( 'id' => 'border_tabs_nav_ul_li', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav li a|hover:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav li a|hover:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav li a|hover:border-color',
							'condition' => array( 'id' => 'border_tabs_nav_ul_li', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav li.active a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav li.active a:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-nav li.active a:border-color',
							'condition' => array( 'id' => 'border_tabs_nav_ul_li', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),



						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Content', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-content:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-content:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-content:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-content:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-content:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-content:border-style',
							'attr_id' => 'border_tabs_content',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-content:border-width',
		  				'condition' => array( 'id' => 'border_tabs_content', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-content:border-width_laptop',
		  				'condition' => array( 'id' => 'border_tabs_content', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-content:border-width_tablet',
		  				'condition' => array( 'id' => 'border_tabs_content', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-content:border-width_mobile',
		  				'condition' => array( 'id' => 'border_tabs_content', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-content:border-color',
							'condition' => array( 'id' => 'border_tabs_content', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-content:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-content:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-content:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-woocommerce-tabs .mfn-woocommerce-tabs-content:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),




















						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Product content ----------------------------------------------------

   			'product_content' => array(
   				'type' => 'product_content',
   				'title' => __('Product content', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'single-product',
   				'attr' => array(
   				//'fields' => array(

						array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'id' => 'info',
							'class' => 'mfn-alert',
							'type' => 'info',
							'title' => __('This element has no attributes. Please check <b>advanced</b> tab for more customisation options.', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Product additional information ----------------------------------------------------

   			'product_additional_information' => array(
   				'type' => 'product_additional_information',
   				'title' => __('Additional information', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'single-product',
   				'attr' => array(
   				//'fields' => array(

						array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'type' => 'header',
							'title' => __('Options', 'mfn-opts'),
						),

						array(
							'id' => 'title',
							'attr_id' => 'title',
							'type' => 'switch',
							're_render' => true,
							'title' => __('Title', 'mfn-opts'),
							'options' => [
								'0' => __('Hide', 'mfn-opts'),
								'1' => __('Show', 'mfn-opts'),
							],
							'std' => '1',
						),

						array(
							'id' => 'title_tag',
							'condition' => array( 'id' => 'title', 'opt' => 'is', 'val' => '1' ),
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h3',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'title' => __('Item title', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:text-align',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'title' => __('Content', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-product-attributes th, .mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-product-attributes td:text-align',
							'oldid' => 'style:.woocommerce-product-attributes th, .woocommerce-product-attributes td:text-align',
							'type' => 'switch',
							'class' => 'mfn-toggled',
							'title' => __('Text align', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-product-attributes th:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Label color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-product-attributes td,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce-product-attributes td a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Value color', 'mfn-opts'),
						),

						array(
							'title' => __('Line', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement table.woocommerce-product-attributes td span|before:background',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						// HTML end: style

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Product related ----------------------------------------------------

   			'product_related' => array(
   				'type' => 'product_related',
   				'title' => __('Product related', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'single-product',
   				'attr' => array(
   				//'fields' => array(

						array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'id' => 'heading_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Heading tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h3',
						),

						array(
   						'id' => 'products',
   						'type' => 'text',
   						're_render' => true,
   						'title' => __('Products per page', 'mfn-opts'),
   						'std' => '4',
   						'after' => 'products',
   						'param' => 'number',
   						'class' => 'narrow',
   					),

						array(
   						'id' => 'columns',
   						'type' => 'text',
   						're_render' => true,
   						'title' => __('Columns', 'mfn-opts'),
   						'std' => '4',
   						'after' => 'columns',
   						'param' => 'number',
   						'class' => 'narrow',
   					),

						// options

   					array(
   						'type' => 'header',
   						'title' => __('Options', 'mfn-opts'),
   					),

						array(
							'id' => 'button',
							'type' => 'switch',
							're_render' => true,
							'title' => __('Add to cart button', 'mfn-opts'),
							'desc' => __('Required for some plugins', 'mfn-opts'),
							'options' => array(
								'0' => __('Hide', 'mfn-opts'),
								'1' => __('Show', 'mfn-opts'),
							),
							'std' => '0',
						),

						array(
							'id' => 'description',
							'type' => 'switch',
							're_render' => true,
							'title' => __('Description', 'mfn-opts'),
							'options' => array(
								'0' => __('Hide', 'mfn-opts'),
								'1' => __('Show', 'mfn-opts'),
							),
							'std' => '0'
						),

						array(
							'type' => 'header',
							'title' => __('Order', 'mfn-opts'),
						),

						array(
							'id' => 'order',
							'type' => 'order',
							're_render' => true,
							'title' => __('Order', 'mfn-opts'),
							'std' => 'image,title,price,description,button',
						),

						array(
							'id' => 'title_tag',
							'type' => 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => [
								'h1' => 'H1',
								'h2' => 'H2',
								'h3' => 'H3',
								'h4' => 'H4',
								'h5' => 'H5',
								'h6' => 'H6',
								'p' => 'p',
								'span' => 'span',
							],
							'std' => 'h2',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'title' => __('Item title', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-related > .title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'desktop',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-related > .title:text-align_laptop',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'laptop',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-related > .title:text-align_tablet',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'tablet',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-related > .title:text-align_mobile',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'mobile',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-related > .title:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-related > .title:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-related > .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-related > .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-related > .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-related > .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Container', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:text-align',
							'oldid' => 'style:li.product:text-align',
							'type' => 'switch',
							'class' => 'mfn-toggled',
							'title' => __('Text align', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-style',
							'attr_id' => 'border_style_shop_li',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-color',
							'condition' => array( 'id' => 'border_style_shop_li', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-width',
		  				'condition' => array( 'id' => 'border_style_shop_li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_shop_li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_shop_li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_shop_li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Image', 'mfn-opts'),
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-style',
							'attr_id' => 'border_style_shopimg',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-color',
							'condition' => array( 'id' => 'border_style_shopimg', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-width',
		  				'condition' => array( 'id' => 'border_style_shopimg', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_shopimg', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_shopimg', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_shopimg', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			// title

						array(
							'title' => __('Title', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .title:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .title:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'title' => __('Price', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .price:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .price:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .price:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .price:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .price:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .price:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'title' => __('Desc', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .excerpt:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .excerpt:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .excerpt:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .excerpt:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .excerpt:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .excerpt:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Attributes', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-variations-wrapper-loop .mfn-vr ul.mfn-vr-options li a:border-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-variations-wrapper-loop .mfn-vr ul.mfn-vr-options li a|hover:border-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color on Hover', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-variations-wrapper-loop .mfn-vr ul.mfn-vr-options li a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Label color', 'mfn-opts'),
						),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Sale label', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .onsale:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .onsale:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .onsale:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .onsale:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .onsale:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .onsale:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Button', 'mfn-opts'),
   					),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-style',
							'attr_id' => 'border_style_bsb',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-width',
		  				'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-color',
							'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher',
							'attr_id' => 'background_shop_button',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:background-color',
							'condition' => array( 'id' => 'background_shop_button', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:gradient',
		  				'condition' => array( 'id' => 'background_shop_button', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button|hover:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button|hover:border-color',
							'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher_hover',
							'attr_id' => 'background_hover_shop_button',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button|hover:background',
							'condition' => array( 'id' => 'background_hover_shop_button', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button|hover:gradient',
		  				'condition' => array( 'id' => 'background_hover_shop_button', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'title' => __('Rating', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .star-rating:display',
							'attr_id' => 'shop-list-rating',
							'type' => 'switch',
							'class' => 'mfn-toggled',
							'title' => __('Visibility', 'mfn-opts'),
							'options' => [
								'' => __('Visible', 'mfn-opts'),
								'none' => __('Hidden', 'mfn-opts'),
							],
							'std' => ''
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .star-rating span:color',
							'condition' => array( 'id' => 'shop-list-rating', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

   					// HTML end: style

						array(
   						'type' => 'html',
   						'html' => '</div>',
   					),

   				),
   			),

   			// Product upsells ----------------------------------------------------

   			'product_upsells' => array(
   				'type' => 'product_upsells',
   				'title' => __('Product upsells', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'single-product',
   				'attr' => array(
   				//'fields' => array(

						array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'id' => 'heading_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Heading tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h3',
						),

						array(
   						'id' => 'products',
   						're_render' => true,
   						'type' => 'text',
   						'title' => __('Products per page', 'mfn-opts'),
   						'std' => '4',
   						'after' => 'products',
   						'param' => 'number',
   						'class' => 'narrow',
   					),

						array(
   						'id' => 'columns',
   						're_render' => true,
   						'type' => 'text',
   						'title' => __('Columns', 'mfn-opts'),
   						'std' => '4',
   						'after' => 'columns',
   						'param' => 'number',
   						'class' => 'narrow',
   					),

						// options

   					array(
   						'type' => 'header',
   						'title' => __('Options', 'mfn-opts'),
   					),

						array(
							'id' => 'button',
							'type' => 'switch',
							're_render' => true,
							'title' => __('Add to cart button', 'mfn-opts'),
							'desc' => __('Required for some plugins', 'mfn-opts'),
							'options' => array(
								'0' => __('Hide', 'mfn-opts'),
								'1' => __('Show', 'mfn-opts'),
							),
							'std' => '0',
						),

						array(
							'id' => 'description',
							'type' => 'switch',
							're_render' => true,
							'title' => __('Description', 'mfn-opts'),
							'options' => array(
								'0' => __('Hide', 'mfn-opts'),
								'1' => __('Show', 'mfn-opts'),
							),
							'std' => '0'
						),

						array(
							'type' => 'header',
							'title' => __('Order', 'mfn-opts'),
						),

						array(
							'id' => 'order',
							'type' => 'order',
							're_render' => true,
							'title' => __('Order', 'mfn-opts'),
							'std' => 'image,title,price,description,button',
						),

						array(
							'id' => 'title_tag',
							'type' => 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => [
								'h1' => 'H1',
								'h2' => 'H2',
								'h3' => 'H3',
								'h4' => 'H4',
								'h5' => 'H5',
								'h6' => 'H6',
								'p' => 'p',
								'span' => 'span',
							],
							'std' => 'h2',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'title' => __('Item title', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-upsells > .title:text-align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'desktop',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-upsells > .title:text-align_laptop',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'laptop',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-upsells > .title:text-align_tablet',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'tablet',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-upsells > .title:text-align_mobile',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'mobile',
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-upsells > .title:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-upsells > .title:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-upsells > .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-upsells > .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-upsells > .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-upsells > .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Container', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:text-align',
							'oldid' => 'style:li.product:text-align',
							'type' => 'switch',
							'class' => 'mfn-toggled',
							'title' => __('Text align', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							],
							'std' => '',
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-style',
							'attr_id' => 'border_style_shop_li',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-color',
							'condition' => array( 'id' => 'border_style_shop_li', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-width',
		  				'condition' => array( 'id' => 'border_style_shop_li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_shop_li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_shop_li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_shop_li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Image', 'mfn-opts'),
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-style',
							'attr_id' => 'border_style_shopimg',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-color',
							'condition' => array( 'id' => 'border_style_shopimg', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-width',
		  				'condition' => array( 'id' => 'border_style_shopimg', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_shopimg', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_shopimg', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_shopimg', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .image_frame:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			// title

						array(
							'title' => __('Title', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .title:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .title:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'title' => __('Price', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .price:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .price:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .price:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .price:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .price:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .price:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'title' => __('Desc', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .excerpt:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .excerpt:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .excerpt:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .excerpt:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .excerpt:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .excerpt:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Attributes', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-variations-wrapper-loop .mfn-vr ul.mfn-vr-options li a:border-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-variations-wrapper-loop .mfn-vr ul.mfn-vr-options li a|hover:border-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color on Hover', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-variations-wrapper-loop .mfn-vr ul.mfn-vr-options li a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Label color', 'mfn-opts'),
						),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Sale label', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .onsale:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .onsale:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .onsale:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .onsale:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .onsale:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .onsale:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Button', 'mfn-opts'),
   					),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-style',
							'attr_id' => 'border_style_bsb',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-width',
		  				'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:border-color',
							'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher',
							'attr_id' => 'background_shop_button',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:background-color',
							'condition' => array( 'id' => 'background_shop_button', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button:gradient',
		  				'condition' => array( 'id' => 'background_shop_button', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button|hover:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button|hover:border-color',
							'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher_hover',
							'attr_id' => 'background_hover_shop_button',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button|hover:background',
							'condition' => array( 'id' => 'background_hover_shop_button', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .button|hover:gradient',
		  				'condition' => array( 'id' => 'background_hover_shop_button', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'title' => __('Rating', 'mfn-opts'),
							'type' => 'header',
							'class' => 'toggled_header row-header',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .star-rating:display',
							'attr_id' => 'shop-list-rating',
							'type' => 'switch',
							'class' => 'mfn-toggled',
							'title' => __('Visibility', 'mfn-opts'),
							'options' => [
								'' => __('Visible', 'mfn-opts'),
								'none' => __('Hidden', 'mfn-opts'),
							],
							'std' => ''
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.products li.product .star-rating span:color',
							'condition' => array( 'id' => 'shop-list-rating', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

   					// HTML end: style

						array(
   						'type' => 'html',
   						'html' => '</div>',
   					),

   				),
   			),

   			/* END WOO */

				// Divider NEW ------------------------------------------------------

				'divider_2' => array(
   				'type' => 'divider_2',
   				'title' => '&bull; '. __('Divider', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'other',
   				'attr' => array(
   				//'fields' => array(

						/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

						array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'id' => 'type',
							'attr_id' => 'divider-2-type',
							'type' => 'select',
							'title' => __('Type', 'mfn-opts'),
							're_render' => true,
							'options' => array(
								'' => 'Empty',

								'#optgroup-border' => 'Border',
									'border-solid' => 'Solid',
									'border-dotted' => 'Dotted',
									'border-dashed' => 'Dashed',
									'border-double' => 'Double',
									// if new added please update conditions below
								'#optgroup-border-end' => '',

								'#optgroup-pattern' => 'Pattern',
									'pattern-star' => 'Star',
									'pattern-triangle' => 'Triangle',
									'pattern-plus' => 'Plus',
									'pattern-tree' => 'Tree',
									'pattern-heart' => 'Heart',
									'pattern-question' => 'Question',
									'pattern-gear' => 'Gear',
									'pattern-parquet' => 'Parquet',
									'pattern-sun' => 'Sun',
									'pattern-fence' => 'Fence',
									'pattern-flower' => 'Flower',
									// if new added please update conditions below
								'#optgroup-pattern-end' => '',

								'#optgroup-lines' => 'Lines',
									'pattern-line-wave' => 'Waves',
									'pattern-line-zigzag' => 'Zig Zag',
									'pattern-line-zigzag2' => 'Zig Zag 2',
									'pattern-line-hearts' => 'Hearts',
									'pattern-line-triangles' => 'Triangles',
									'pattern-line-circles' => 'Circles',
									'pattern-line-circles2' => 'Circles 2',
									'pattern-line-circles3' => 'Circles 3',
									'pattern-line-stars' => 'Stars',
									'pattern-line-rhombus' => 'Rhombus',
									'pattern-line-figures' => 'Figures',
									'pattern-line-loopline' => 'Loopline',
									// if new added please update conditions below
								'#optgroup-lines-end' => '',

							),
							'std' => ''
						),

						array(
							'id' => 'align',
							'type' => 'switch',
							'title' => __('Alignment', 'mfn-opts'),
							're_render' => true,
							'options' => array(
								'start' => __('Start', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'end' => __('End', 'mfn-opts'),
							),
							'std' => 'center',
							'responsive' => 'desktop',

						),

						array(
							'id' => 'align_laptop',
							'type' => 'switch',
							'title' => __('Alignment', 'mfn-opts'),
							're_render' => true,
							'options' => array(
								'start' => __('Start', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'end' => __('End', 'mfn-opts'),
								'' => __('Default', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'laptop',

						),

						array(
							'id' => 'align_tablet',
							'type' => 'switch',
							'title' => __('Alignment', 'mfn-opts'),
							're_render' => true,
							'options' => array(
								'start' => __('Start', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'end' => __('End', 'mfn-opts'),
								'' => __('Default', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'tablet',

						),

						array(
							'id' => 'align_mobile',
							'type' => 'switch',
							'title' => __('Alignment', 'mfn-opts'),
							're_render' => true,
							'options' => array(
								'start' => __('Start', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'end' => __('End', 'mfn-opts'),
								'' => __('Default', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'mobile',

						),

						// gap

						array(
							'type' => 'header',
							'title' => __('Gap', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:--mfn-divider-gap-top',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Gap top', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
							'std' => '20px',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:--mfn-divider-gap-top_laptop',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Gap top', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:--mfn-divider-gap-top_tablet',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Gap top', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:--mfn-divider-gap-top_mobile',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Gap top', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:--mfn-divider-gap-bottom',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Gap bottom', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
							'std' => '20px',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:--mfn-divider-gap-bottom_laptop',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Gap bottom', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:--mfn-divider-gap-bottom_tablet',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Gap bottom', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:--mfn-divider-gap-bottom_mobile',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Gap bottom', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						// addon

						array(
							'type' => 'header',
							'title' => __('Addon', 'mfn-opts'),
						),

						array(
							'id' => 'addon',
							'attr_id' => 'divider-2-addon',
							'type' => 'select',
							'title' => __('Addon', 'mfn-opts'),
							're_render' => true,
							'options' => array(
								'' => 'None',
								'label' => 'Label',
								'image' => 'Image',
								'icon' => 'Icon',
							),
							'std' => ''
						),

						array(
							'id' => 'label',
							'type' => 'text',
							'title' => __('Label', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-addon', 'opt' => 'is', 'val' => 'label' ),
							're_render' => true,
							'std' => ''
						),

						array(
							'id' => 'image',
							'type' => 'upload',
							'title' => __('Image', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-addon', 'opt' => 'is', 'val' => 'image' ),
							're_render' => true,
							'std' => ''
						),

						array(
							'id' => 'icon',
							'type' => 'icon',
							'title' => __('Icon', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-addon', 'opt' => 'is', 'val' => 'icon' ),
							're_render' => true,
							'std' => ''
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render modalbox-card-style">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Container', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:width',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled narrow',
							'css_attr' => 'width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:width_laptop',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled narrow',
							'css_attr' => 'width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:width_tablet',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled narrow',
							'css_attr' => 'width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:width_mobile',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled narrow',
							'css_attr' => 'width',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Divider', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-type', 'opt' => 'isnt', 'val' => '' ),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:--mfn-divider-height',
							'type' => 'sliderbar',
							'title' => __('Pattern height', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-type', 'opt' => 'is', 'val' => 'pattern-star,pattern-triangle,pattern-plus,pattern-tree,pattern-heart,pattern-question,pattern-gear,pattern-parquet,pattern-sun,pattern-fence,pattern-flower,pattern-line-wave,pattern-line-zigzag,pattern-line-zigzag2,pattern-line-hearts,pattern-line-triangles,pattern-line-circles,pattern-line-circles2,pattern-line-circles3,pattern-line-stars,pattern-line-rhombus,pattern-line-figures,pattern-line-loopline' ),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '100',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:--mfn-divider-height_laptop',
							'type' => 'sliderbar',
							'title' => __('Pattern height', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-type', 'opt' => 'is', 'val' => 'pattern-star,pattern-triangle,pattern-plus,pattern-tree,pattern-heart,pattern-question,pattern-gear,pattern-parquet,pattern-sun,pattern-fence,pattern-flower,pattern-line-wave,pattern-line-zigzag,pattern-line-zigzag2,pattern-line-hearts,pattern-line-triangles,pattern-line-circles,pattern-line-circles2,pattern-line-circles3,pattern-line-stars,pattern-line-rhombus,pattern-line-figures,pattern-line-loopline' ),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '100',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:--mfn-divider-height_tablet',
							'type' => 'sliderbar',
							'title' => __('Pattern height', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-type', 'opt' => 'is', 'val' => 'pattern-star,pattern-triangle,pattern-plus,pattern-tree,pattern-heart,pattern-question,pattern-gear,pattern-parquet,pattern-sun,pattern-fence,pattern-flower,pattern-line-wave,pattern-line-zigzag,pattern-line-zigzag2,pattern-line-hearts,pattern-line-triangles,pattern-line-circles,pattern-line-circles2,pattern-line-circles3,pattern-line-stars,pattern-line-rhombus,pattern-line-figures,pattern-line-loopline' ),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '100',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:--mfn-divider-height_mobile',
							'type' => 'sliderbar',
							'title' => __('Pattern height', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-type', 'opt' => 'is', 'val' => 'pattern-star,pattern-triangle,pattern-plus,pattern-tree,pattern-heart,pattern-question,pattern-gear,pattern-parquet,pattern-sun,pattern-fence,pattern-flower,pattern-line-wave,pattern-line-zigzag,pattern-line-zigzag2,pattern-line-hearts,pattern-line-triangles,pattern-line-circles,pattern-line-circles2,pattern-line-circles3,pattern-line-stars,pattern-line-rhombus,pattern-line-figures,pattern-line-loopline' ),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '100',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:--mfn-divider-pattern-color',
							'type' => 'color',
							'title' => __('Pattern color', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-type', 'opt' => 'is', 'val' => 'pattern-star,pattern-triangle,pattern-plus,pattern-tree,pattern-heart,pattern-question,pattern-gear,pattern-parquet,pattern-sun,pattern-fence,pattern-flower,pattern-line-wave,pattern-line-zigzag,pattern-line-zigzag2,pattern-line-hearts,pattern-line-triangles,pattern-line-circles,pattern-line-circles2,pattern-line-circles3,pattern-line-stars,pattern-line-rhombus,pattern-line-figures,pattern-line-loopline' ),
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:--mfn-divider-border-width',
							'type' => 'sliderbar',
							'title' => __('Border width', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-type', 'opt' => 'is', 'val' => 'border-solid,border-dotted,border-dashed,border-double' ),
							'class' => 'mfn-toggled narrow',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '20',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:--mfn-divider-border-color',
							'type' => 'color',
							'title' => __('Border color', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-type', 'opt' => 'is', 'val' => 'border-solid,border-dotted,border-dashed,border-double' ),
							'class' => 'mfn-toggled',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Addon', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-addon', 'opt' => 'isnt', 'val' => '' ),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:--mfn-divider-spacing',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'title' => __('Spacing', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 0, 'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0, 'max' => 10, 'step' => 0.1),
								'rem' => array('min' => 0, 'max' => 10, 'step' => 0.1),
								'vw' 	=> array('min' => 0, 'max' => 100, 'step' => 1),
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:--mfn-divider-spacing_laptop',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'title' => __('Spacing', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 0, 'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0, 'max' => 10, 'step' => 0.1),
								'rem' => array('min' => 0, 'max' => 10, 'step' => 0.1),
								'vw' 	=> array('min' => 0, 'max' => 100, 'step' => 1),
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:--mfn-divider-spacing_tablet',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'title' => __('Spacing', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 0, 'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0, 'max' => 10, 'step' => 0.1),
								'rem' => array('min' => 0, 'max' => 10, 'step' => 0.1),
								'vw' 	=> array('min' => 0, 'max' => 100, 'step' => 1),
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:--mfn-divider-spacing_mobile',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'title' => __('Spacing', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 0, 'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0, 'max' => 10, 'step' => 0.1),
								'rem' => array('min' => 0, 'max' => 10, 'step' => 0.1),
								'vw' 	=> array('min' => 0, 'max' => 100, 'step' => 1),
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner .divider-icon i:font-size',
							'type' => 'sliderbar',
							'title' => __('Icon size', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-addon', 'opt' => 'is', 'val' => 'icon' ),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'units' => array(
								'px' 	=> array('min' => 0, 'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0, 'max' => 10, 'step' => 0.1),
								'rem' => array('min' => 0, 'max' => 10, 'step' => 0.1),
								'vw' 	=> array('min' => 0, 'max' => 100, 'step' => 1),
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner .divider-icon i:font-size_laptop',
							'type' => 'sliderbar',
							'title' => __('Icon size', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-addon', 'opt' => 'is', 'val' => 'icon' ),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'units' => array(
								'px' 	=> array('min' => 0, 'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0, 'max' => 10, 'step' => 0.1),
								'rem' => array('min' => 0, 'max' => 10, 'step' => 0.1),
								'vw' 	=> array('min' => 0, 'max' => 100, 'step' => 1),
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner .divider-icon i:font-size_tablet',
							'type' => 'sliderbar',
							'title' => __('Icon size', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-addon', 'opt' => 'is', 'val' => 'icon' ),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'units' => array(
								'px' 	=> array('min' => 0, 'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0, 'max' => 10, 'step' => 0.1),
								'rem' => array('min' => 0, 'max' => 10, 'step' => 0.1),
								'vw' 	=> array('min' => 0, 'max' => 100, 'step' => 1),
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner .divider-icon i:font-size_mobile',
							'type' => 'sliderbar',
							'title' => __('Icon size', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-addon', 'opt' => 'is', 'val' => 'icon' ),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:--mfn-divider-icon-color',
							'type' => 'color',
							'title' => __('Icon color', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-addon', 'opt' => 'is', 'val' => 'icon' ),
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner .divider-image img:height',
							'type' => 'sliderbar',
							'title' => __('Image height', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-addon', 'opt' => 'is', 'val' => 'image' ),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner .divider-image img:height_laptop',
							'type' => 'sliderbar',
							'title' => __('Image height', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-addon', 'opt' => 'is', 'val' => 'image' ),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner .divider-image img:height_tablet',
							'type' => 'sliderbar',
							'title' => __('Image height', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-addon', 'opt' => 'is', 'val' => 'image' ),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner .divider-image img:height_mobile',
							'type' => 'sliderbar',
							'title' => __('Image height', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-addon', 'opt' => 'is', 'val' => 'image' ),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner .divider-label:color',
							'type' => 'color',
							'title' => __('Text color', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-addon', 'opt' => 'is', 'val' => 'label' ),
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner .divider-label:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'title' => __('Typography', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-addon', 'opt' => 'is', 'val' => 'label' ),
							'class' => 'mfn-toggled toggle_fields',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner .divider-addon:background-color',
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner .divider-label:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-addon', 'opt' => 'is', 'val' => 'label' ),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner .divider-label:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-addon', 'opt' => 'is', 'val' => 'label' ),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner .divider-label:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-addon', 'opt' => 'is', 'val' => 'label' ),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner .divider-label:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-addon', 'opt' => 'is', 'val' => 'label' ),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:--mfn-divider-addon-width',
							'type' => 'sliderbar',
							'title' => __('Width', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-addon', 'opt' => 'isnt', 'val' => 'label' ),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:--mfn-divider-addon-width_laptop',
							'type' => 'sliderbar',
							'title' => __('Width', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-addon', 'opt' => 'isnt', 'val' => 'label' ),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:--mfn-divider-addon-width_tablet',
							'type' => 'sliderbar',
							'title' => __('Width', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-addon', 'opt' => 'isnt', 'val' => 'label' ),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:--mfn-divider-addon-width_mobile',
							'type' => 'sliderbar',
							'title' => __('Width', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-addon', 'opt' => 'isnt', 'val' => 'label' ),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:--mfn-divider-addon-height',
							'type' => 'sliderbar',
							'title' => __('Height', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-addon', 'opt' => 'isnt', 'val' => 'label' ),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:--mfn-divider-addon-height_laptop',
							'type' => 'sliderbar',
							'title' => __('Height', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-addon', 'opt' => 'isnt', 'val' => 'label' ),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:--mfn-divider-addon-height_tablet',
							'type' => 'sliderbar',
							'title' => __('Height', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-addon', 'opt' => 'isnt', 'val' => 'label' ),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner:--mfn-divider-addon-height_mobile',
							'type' => 'sliderbar',
							'title' => __('Height', 'mfn-opts'),
							'condition' => array( 'id' => 'divider-2-addon', 'opt' => 'isnt', 'val' => 'label' ),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner .divider-addon:border-style',
							'attr_id' => 'divider-2-addon-border',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner .divider-addon:border-color',
							'condition' => array( 'id' => 'divider-2-addon-border', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner .divider-addon:border-width',
		  				'condition' => array( 'id' => 'divider-2-addon-border', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'class' => 'mfn-toggled',
		  			),


						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-divider-inner .divider-addon:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'class' => 'mfn-toggled',
		  			),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

					),
				),

				// Divider Basic -------------------------------------------------------

   			'divider' => array(
   				'type' => 'divider',
   				'title' => '&bull; '. __('Divider Basic', 'mfn-opts'),
   				// 'title' => '&bull; '. __('Divider', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'other',
   				'attr' => array(
   				//'fields' => array(

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'height',
   						'type' => 'text',
   						're_render' => true,
   						'title' => __('Height', 'mfn-opts'),
							'param' => 'number',
							'class' => 'narrow',
							'after' => 'px',
   					),

   					array(
   						'id' => 'style',
							'attr_id' => 'divider_style',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Style', 'mfn-opts'),
   						'options' => array(
   							'default' => __('Default', 'mfn-opts'),
   							'dots' => __('Dots', 'mfn-opts'),
   							'zigzag' => __('ZigZag', 'mfn-opts'),
   						),
							'std' => 'default',
   					),

   					array(
   						'id' => 'line',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Line', 'mfn-opts'),
							'condition' => array( 'id' => 'divider_style', 'opt' => 'is', 'val' => 'default' ),
   						'options' => array(
								'' => __('No line', 'mfn-opts'),
   							'default' => __('Default', 'mfn-opts'),
   							'narrow' => __('Narrow', 'mfn-opts'),
   							'wide' => __('Wide', 'mfn-opts'),
   						),
							'std' => '',
   					),

   					array(
   						'id' => 'color',
   						'type' => 'color',
   						're_render' => true,
   						'title' => __('Color', 'mfn-opts'),
							'alpha' => true,
   					),

   					array(
   						'id' => 'themecolor',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Theme color', 'mfn-opts'),
   						'desc' => __('Overwrites color selected above', 'mfn-opts'),
   						'options' => array(
   							0 => __('No', 'mfn-opts'),
   							1 => __('Yes', 'mfn-opts'),
   						),
							'std' => 0,
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Placeholder ----------------------------------------------------

   			'placeholder' => array(
   				'type' => 'placeholder',
   				'title' => '&bull; '. __('Placeholder', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'other',
   				'attr' => array(
   				//'fields' => array(

   					array(
   						'id' => 'info',
   						'class' => 'mfn-alert',
   						'type' => 'info',
   						'title' => __('This item has no attributes.', 'mfn-opts'),
   					),

   				),
   			),

   			// Spacer ------------------------------------------------------

   			'spacer' => array(
   				'type' => 'spacer',
   				'title' => __('Spacer', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'boxes',
   				'attr' => array(

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:height',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'class' => 'sliderbar-units',
							'title' => __('Height', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 1000, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'vh' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:height_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'class' => 'sliderbar-units',
							'title' => __('Height', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 1000, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'vh' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:height_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'class' => 'sliderbar-units',
							'title' => __('Height', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 1000, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'vh' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:height_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'class' => 'sliderbar-units',
							'title' => __('Height', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 1000, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'vh' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),

   			),

   			// Accordion  -----------------------------------------------------

   			'accordion' => array(
   				'type' => 'accordion',
   				'title' => __('Accordion', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'blocks',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
   						'id' => 'info-key',
   						'class' => 'mfn-alert',
   						'type' => 'info',
   						'title' => __('Try the new Toggle element with unlimited customization options', 'mfn-opts'),
   					),

   					array(
   						'id' => 'title',
   						'type' => 'text',
   						'edit_tag' => '.accordion | h4',
   						'edit_position' => 'prepend',
   						'title' => __('Title', 'mfn-opts'),
   						'dynamic_data' => 'title',
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h4',
						),

						// tabs

						array(
							'type' => 'header',
   						'title' => __('Tabs', 'mfn-opts'),
   					),

   					array(
   						'id' => 'tabs',
   						'type' => 'tabs',
   						'title' => __('Accordion', 'mfn-opts'),
   						'desc' => __('<b>JavaScript</b> content like Google Maps and some plugins shortcodes do <b>not work</b> in tabs', 'mfn-opts'),
							'options' => [
								'title' => [
									'input',
									__('Title', 'mfn-opts'),
									__('Sample tab', 'mfn-opts'),
								],
								'content' => [
									'textarea',
									__('Content', 'mfn-opts'),
									'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Mauris eu massa orci.',
								],
							],
							'std' => [
								0 => [
									'title' => __('This is the 1st item', 'mfn-opts'),
									'content' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Mauris eu massa orci.',
								],
								1 => [
									'title' => __('This is the 2nd item', 'mfn-opts'),
									'content' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Mauris eu massa orci.',
								],
							],
							'preview' => 'tabs',
							'primary' => 'title', // default
   					),

   					array(
   						'type' => 'header',
   						'title' => __('Icon', 'mfn-opts'),
   					),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
   						'id' => 'icon',
   						'type' => 'icon',
   						'title' => __('Icon', 'mfn-opts'),
   					),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
   						'id' => 'icon_active',
   						'type' => 'icon',
   						'title' => __('Icon', 'mfn-opts'),
   					),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						// options

						array(
							'type' => 'header',
   						'title' => __('Options', 'mfn-opts'),
   					),

   					array(
   						'id' => 'open1st',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Open first', 'mfn-opts'),
   						'options' => array(
   							0 => __('Default', 'mfn-opts'),
   							1 => __('Open first', 'mfn-opts'),
   						),
							'std' => 0,
   					),

   					array(
   						'id' => 'openAll',
   						're_render' => true,
   						'type' => 'switch',
							'title' => __('Open all', 'mfn-opts'),
   						'options' => array(
								0 => __('Default', 'mfn-opts'),
   							1 => __('Open all', 'mfn-opts'),
							),
							'std' => 0,
   					),

   					array(
   						'id' => 'style',
   						'type' => 'switch',
   						'title' => __('Style', 'mfn-opts'),
   						'options' => array(
   							'accordion' => __('Accordion', 'mfn-opts'),
   							'toggle' => __('Toggle', 'mfn-opts'),
   						),
							'std' => 'accordion',
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .heading:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .heading:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .heading:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .heading:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .heading:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .heading:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title bar', 'mfn-opts'),
   					),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question .title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question .title:background',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question.active .title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question.active .title:background',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question .title:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question .title:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question .title:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question .title:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question .title|before:border-style',
							'attr_id' => 'border_style_accq',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question .title|before:border-color',
							'condition' => array( 'id' => 'border_style_accq', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question:border-width',
		  				'condition' => array( 'id' => 'border_style_accq', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_accq', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_accq', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_accq', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question .title:box-shadow',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow'
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question .title:box-shadow_laptop',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question .title:box-shadow_tablet',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question .title:box-shadow_mobile',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title bar icon', 'mfn-opts'),
   					),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question .title i:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question.active .title i:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question .title i:font-size',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question .title i:font-size_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question .title i:font-size_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question .title i:font-size_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question .answer:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question .answer:background',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question .answer:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question .answer:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question .answer:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question .answer:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question .answer:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'themeoptions' => 'style:simple',
   						'title' => __('Question line', 'mfn-opts'),
   					),

   					array(
							'type' => 'html',
							'themeoptions' => 'style:simple',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question:border-color',
							'type' => 'color',
							'themeoptions' => 'style:simple',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'themeoptions' => 'style:simple',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .accordion .question.active|after:background-color',
							'type' => 'color',
							'themeoptions' => 'style:simple',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'themeoptions' => 'style:simple',
							'html' => '</div></div>',
						),


						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Article box  ---------------------------------------------------

   			'article_box' => array(
   				'type' => 'article_box',
   				'title' => __('Article box', 'mfn-opts'),
   				'size' => '1/3',
   				'tablet_size' => '1/3',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'boxes',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
   						'id' => 'image',
   						'dynamic_data' => 'featured_image',
   						'type' => 'upload',
   						'title' => __('Image', 'mfn-opts'),
   						'desc' => __('Recommended image width <b>384px - 960px</b> depending on size of the item', 'mfn-opts'),
							'std' => $this->get_placeholder(),
							'preview' => 'image',
   					),

   					array(
   						'id' => 'slogan',
   						'type' => 'text',
   						'dynamic_data' => 'title',
   						'edit_tag' => '.desc_wrapper | p',
   						'edit_tagchild' => 'span',
   						'edit_position' => 'prepend',
   						'title' => __('Slogan', 'mfn-opts'),
   						'desc' => __('Allowed HTML tags: span, strong, b, em, i, u', 'mfn-opts'),
							'std' => __('This is the slogan', 'mfn-opts'),
							'preview' => 'subtitle',
   					),

   					array(
   						'id' => 'title',
   						'type' => 'text',
						'dynamic_data' => 'title',
   						'edit_tag' => '.desc_wrapper | .title',
   						'edit_position' => 'append',
   						'title' => __('Title', 'mfn-opts'),
   						'desc' => __('Allowed HTML tags: span, strong, b, em, i, u', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h4',
						),

   					// link

   					array(
   						'type' => 'header',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link',
   						'type' => 'text',
   						'dynamic_data' => 'permalink',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'target',
   						'type' => 'select',
   						'title' => __('Target', 'mfn-opts'),
   						'options' => array(
   							0 => __('Default | _self', 'mfn-opts'),
   							1 => __('New tab or window | _blank', 'mfn-opts'),
   							'lightbox' => __('Lightbox (image or embed video)', 'mfn-opts'),
   						),
   					),

            array(
							'id' => 'link_title',
							'type' => 'text',
							'dynamic_data' => 'title',
							'title' => __('Link title', 'mfn-opts'),
						),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Image', 'mfn-opts'),
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .article_box .photo_wrapper img:border-style',
							'attr_id' => 'border_style_abimg',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .article_box .photo_wrapper img:border-color',
							'condition' => array( 'id' => 'border_style_abimg', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .article_box .photo_wrapper img:border-width',
		  				'condition' => array( 'id' => 'border_style_abimg', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .article_box .photo_wrapper img:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_abimg', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .article_box .photo_wrapper img:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_abimg', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .article_box .photo_wrapper img:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_abimg', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .article_box .photo_wrapper img:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .article_box .photo_wrapper img:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .article_box .photo_wrapper img:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .article_box .photo_wrapper img:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .photo_wrapper:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .photo_wrapper:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .photo_wrapper:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .photo_wrapper:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'type' => 'header',
		  				'class' => 'toggled_header row-header',
   						'title' => __('Desc', 'mfn-opts'),
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper:background',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .article_box .desc_wrapper:border-style',
							'attr_id' => 'border_style_desc',
							'class' => 'mfn-toggled',
							'type' => 'select',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .article_box .desc_wrapper:border-color',
							'condition' => array( 'id' => 'border_style_desc', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .article_box .desc_wrapper:border-width',
		  				'condition' => array( 'id' => 'border_style_desc', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .article_box .desc_wrapper:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_desc', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .article_box .desc_wrapper:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_desc', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .article_box .desc_wrapper:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_desc', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .article_box .desc_wrapper:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .article_box .desc_wrapper:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .article_box .desc_wrapper:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .article_box .desc_wrapper:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper .title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper .title:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Slogan', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper p:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper p:border-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'desc' => __('Works only with <b>classic style</b>', 'mfn-opts'),
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper p:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper p:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper p:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper p:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper p:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),


						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Decoration', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper:--mfn-article-box-decoration',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

   					array(
							'type' => 'html',
							'html' => '</div>',
						),

						// advanced here

   				),
   			),

				// Banner Box  ---------------------------------------------------

   			'banner_box' => array(
   				'type' => 'banner_box',
   				'title' => __('Banner box', 'mfn-opts'),
   				'size' => '1/3',
   				'tablet_size' => '1/3',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'boxes',
   				'attr' => array(

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'type' => 'header',
   						'title' => __('Box', 'mfn-opts'),
   					),

   					array(
   						'id' => 'style',
   						'attr_id' => 'style',
   						'type' => 'switch',
   						'title' => __('Style', 'mfn-opts'),
   						'options' => array(
   							'' => __('Full', 'mfn-opts'),
   							'boxed' => __('Boxed', 'mfn-opts'),
   						),
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-desc:justify-content',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'title' => __('Horizontal position', 'mfn-opts'),
   						'options' => array(
   							'flex-start' => __('Left', 'mfn-opts'),
								'' => __('Center', 'mfn-opts'),
   							'flex-end' => __('Right', 'mfn-opts'),
   						),
   						'std' => '',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-desc:justify-content_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'title' => __('Horizontal position', 'mfn-opts'),
   						'options' => array(
   							'flex-start' => __('Left', 'mfn-opts'),
								'' => __('Center', 'mfn-opts'),
   							'flex-end' => __('Right', 'mfn-opts'),
   						),
   						'std' => '',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-desc:justify-content_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'title' => __('Horizontal position', 'mfn-opts'),
   						'options' => array(
   							'flex-start' => __('Left', 'mfn-opts'),
								'' => __('Center', 'mfn-opts'),
   							'flex-end' => __('Right', 'mfn-opts'),
   						),
   						'std' => '',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-desc:justify-content_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'title' => __('Horizontal position', 'mfn-opts'),
   						'options' => array(
   							'flex-start' => __('Left', 'mfn-opts'),
								'' => __('Center', 'mfn-opts'),
   							'flex-end' => __('Right', 'mfn-opts'),
   						),
   						'std' => '',
   					),



   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-desc:align-items',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'title' => __('Vertical position', 'mfn-opts'),
   						'options' => array(
   							'flex-start' => __('Top', 'mfn-opts'),
								'' => __('Center', 'mfn-opts'),
   							'flex-end' => __('Bottom', 'mfn-opts'),
   						),
   						'std' => '',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-desc:align-items_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'title' => __('Vertical position', 'mfn-opts'),
   						'options' => array(
   							'flex-start' => __('Top', 'mfn-opts'),
								'' => __('Center', 'mfn-opts'),
   							'flex-end' => __('Bottom', 'mfn-opts'),
   						),
   						'std' => '',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-desc:align-items_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'title' => __('Vertical position', 'mfn-opts'),
   						'options' => array(
   							'flex-start' => __('Top', 'mfn-opts'),
								'' => __('Center', 'mfn-opts'),
   							'flex-end' => __('Bottom', 'mfn-opts'),
   						),
   						'std' => '',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-desc:align-items_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'title' => __('Vertical position', 'mfn-opts'),
   						'options' => array(
   							'flex-start' => __('Top', 'mfn-opts'),
								'' => __('Center', 'mfn-opts'),
   							'flex-end' => __('Bottom', 'mfn-opts'),
   						),
   						'std' => '',
   					),


						array(
							'type' => 'header',
   						'title' => __('Image', 'mfn-opts'),
   					),

						array(
   						'id' => 'image',
   						'dynamic_data' => 'featured_image',
   						'type' => 'upload',
   						'title' => __('Image', 'mfn-opts'),
							'std' => $this->get_placeholder(),
   					),

   					array(
   						'id' => 'overlay',
   						'attr_id' => 'overlay',
   						'type' => 'switch',
   						'title' => __('Overlay', 'mfn-opts'),
   						'options' => array(
   							'' => __('Off', 'mfn-opts'),
   							'1' => __('On', 'mfn-opts'),
   						),
   						'std' => '',
   					),

   					array(
   						'id' => 'hover_effect',
   						'type' => 'select',
   						'title' => __('Hover effect', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'toggle' => __('Toggle', 'mfn-opts'),
   							'zoom-in' => __('Zoom In', 'mfn-opts'),
   							'zoom-out' => __('Zoom Out', 'mfn-opts'),
   							'blur' => __('Blur', 'mfn-opts'),
   						),
   						'std' => '',
   					),

						array(
							'type' => 'header',
   						'title' => __('Title', 'mfn-opts'),
   					),

						array(
   						'id' => 'title',
   						'type' => 'text',
							'dynamic_data' => 'title',
   						'edit_tag' => '.banner-wrapper | .title.banner-item',
   						'edit_position' => 'append',
   						'title' => __('Title', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h3',
						),

						array(
							'type' => 'header',
   						'title' => __('Subtitle', 'mfn-opts'),
   					),

						array(
   						'id' => 'subtitle',
   						'type' => 'text',
   						'dynamic_data' => 'title',
   						'edit_tag' => '.banner-wrapper | .subtitle.banner-item',
   						'edit_position' => 'prepend',
   						'title' => __('Subtitle', 'mfn-opts'),
							'std' => __('This is the subtitle', 'mfn-opts'),
   					),

   					array(
							'id' => 'subtitle_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Subtitle tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'p',
						),

						array(
							'type' => 'header',
   						'title' => __('Excerpt', 'mfn-opts'),
   					),

						array(
   						'id' => 'bb_excerpt',
   						'attr_id' => 'bb_excerpt',
   						'type' => 'textarea',
   						'dynamic_data' => 'content',
   						'edit_tag' => '.banner-wrapper | p.excerpt',
   						'edit_position' => 'append',
   						'title' => __('Excerpt', 'mfn-opts'),
   					),


   					// link

   					array(
   						'type' => 'header',
   						'title' => __('Call To Action', 'mfn-opts'),
   					),

   					array(
   						'id' => 'cta',
   						'attr_id' => 'cta',
   						're_render' => true,
   						'type' => 'select',
   						'title' => __('CTA type', 'mfn-opts'),
   						'options' => array(
   							'' => __('Text', 'mfn-opts'),
   							'icon' => __('Icon', 'mfn-opts'),
   							'image' => __('Image', 'mfn-opts'),
   						),
   						'std' => ''
   					),

   					array(
   						'id' => 'cta_icon',
   						'type' => 'icon',
   						'title' => __('Icon', 'mfn-opts'),
   						'condition' => array( 'id' => 'cta', 'opt' => 'is', 'val' => 'icon' ),
   					),

   					array(
   						'id' => 'cta_image',
   						'type' => 'upload',
   						'title' => __('Image', 'mfn-opts'),
   						'condition' => array( 'id' => 'cta', 'opt' => 'is', 'val' => 'image' ),
   					),

   					array(
							'id' => 'link_title',
							'type' => 'text',
							'edit_tag' => '.banner-wrapper | .cta-text.banner-item',
   						'edit_position' => 'append',
   						'std' => __('Read more', 'mfn-opts'),
   						'condition' => array( 'id' => 'cta', 'opt' => 'is', 'val' => '' ),
							'title' => __('Call to action', 'mfn-opts'),
						),

						array(
   						'id' => 'cta_hover_effect',
   						'type' => 'select',
   						'title' => __('Hover effect', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'zoom-in' => __('Zoom In', 'mfn-opts'),
   						),
   						'std' => '',
   					),

						array(
   						'type' => 'header',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link_type',
   						'attr_id' => 'link_type',
   						'type' => 'select',
   						'title' => __('On click action', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'1' => __('Open popup', 'mfn-opts'),
   						),
   						'std' => ''
   					),

   					array(
   						'id' => 'popup_id',
   						'type' => 'text',
   						'condition' => array( 'id' => 'link_type', 'opt' => 'is', 'val' => '1' ),
   						'title' => __('Popup ID', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link',
   						'type' => 'text',
   						'dynamic_data' => 'permalink',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'target',
   						'type' => 'select',
   						'title' => __('Target', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default | _self', 'mfn-opts'),
   							'1' => __('New tab or window | _blank', 'mfn-opts'),
   							'lightbox' => __('Lightbox (image or embed video)', 'mfn-opts'),
   						),
   					),

   					array(
							'id' => 'link_title_2',
							'type' => 'text',
   						'std' => __('Link title', 'mfn-opts'),
   						'condition' => array( 'id' => 'cta', 'opt' => 'is', 'val' => '' ),
							'title' => __('Link title', 'mfn-opts'),
						),

   					array(
   						'type' => 'header',
   						'title' => __('Order', 'mfn-opts'),
   					),

   					array(
							'id' => 'order',
							'type' => 'order',
							're_render' => true,
							'title' => __('Order', 'mfn-opts'),
							'std' => 'title,subtitle,excerpt,divider,cta',
						),

						array(
   						'type' => 'header',
   						'title' => __('Responsive', 'mfn-opts'),
   					),

   					array(
   						'id' => 'hidden_elements_mobile',
   						'type' => 'select',
   						'title' => __('Hidden elements on mobile', 'mfn-opts'),
   						'options' => array(
   							0 => __('Default', 'mfn-opts'),
   							1 => __('Visible on mobile', 'mfn-opts'),
   							2 => __('Visible on tablet and mobile', 'mfn-opts'),
   						),
   						'std' => '0'
   					),

   					array(
   						'type' => 'header',
   						'title' => __('Badge', 'mfn-opts'),
   					),

   					array(
   						'id' => 'bb_badge',
   						'attr_id' => 'bb_badge',
   						'type' => 'text',
   						'edit_tag' => '.mfn-banner-box | .banner-badge',
   						'edit_position' => 'append',
   						'title' => __('Badge', 'mfn-opts'),
   					),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

   					array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Box', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:--mfn-banner-box-offset',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Offset', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 0, 		'max' => 1000, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 100, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 100, 	'step' => 0.1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:--mfn-banner-box-offset_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Offset', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 0, 		'max' => 1000, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 100, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 100, 	'step' => 0.1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:--mfn-banner-box-offset_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Offset', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 0, 		'max' => 1000, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 100, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 100, 	'step' => 0.1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:--mfn-banner-box-offset_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Offset', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 0, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 100, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 100, 	'step' => 0.1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:border-style',
							'attr_id' => 'border_style_bannerbox',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:border-width',
		  				'condition' => array( 'id' => 'border_style_bannerbox', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_bannerbox', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_bannerbox', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_bannerbox', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:box-shadow',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow'
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:box-shadow_laptop',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:box-shadow_tablet',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:box-shadow_mobile',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:--mfn-banner-box-animation-duration',
							'type' => 'sliderbar',
							'class' => 'mfn-toggled',
							'title' => __('Transition duration', 'mfn-opts'),
							'param' => array(
								'min' => '0',
								'max' => '1',
								'step' => '0.1',
								'unit' => 's',
							)
						),













		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:border-color',
							'condition' => array( 'id' => 'border_style_bannerbox', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box|hover:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box|hover:border-color',
							'condition' => array( 'id' => 'border_style_bannerbox', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),







						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Image', 'mfn-opts'),
   					),

   					array(
							'id' => 'image_height',
							'attr_id' => 'image_height',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Height', 'mfn-opts'),
							'options' => [
								'' => __('Auto', 'mfn-opts'),
								'1' => __('Custom', 'mfn-opts'),
							],
							'std' => ''
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box.mfn-banner-box-height .banner-image img:height',
							'type' => 'text',
							'condition' => array( 'id' => 'image_height', 'opt' => 'is', 'val' => '1' ),
							'default_unit' => 'px',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Height', 'mfn-opts'),
							'desc' => __('Use px, %, vw, vh or auto to set content width', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box.mfn-banner-box-height .banner-image img:height_laptop',
							'type' => 'text',
							'condition' => array( 'id' => 'image_height', 'opt' => 'is', 'val' => '1' ),
							'default_unit' => 'px',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Height', 'mfn-opts'),
							'desc' => __('Use px, %, vw, vh or auto to set content width', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box.mfn-banner-box-height .banner-image img:height_tablet',
							'type' => 'text',
							'condition' => array( 'id' => 'image_height', 'opt' => 'is', 'val' => '1' ),
							'default_unit' => 'px',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Height', 'mfn-opts'),
							'desc' => __('Use px, %, vw, vh or auto to set content width', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box.mfn-banner-box-height .banner-image img:height_mobile',
							'type' => 'text',
							'condition' => array( 'id' => 'image_height', 'opt' => 'is', 'val' => '1' ),
							'default_unit' => 'px',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Height', 'mfn-opts'),
							'desc' => __('Use px, %, vw, vh or auto to set content width', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'condition' => array( 'id' => 'overlay', 'opt' => 'is', 'val' => '1' ),
							'class' => 'toggled_header row-header',
   						'title' => __('Overlay', 'mfn-opts'),
   					),


		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box-image-overlay .banner-image|after:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box-image-overlay|hover .banner-image|after:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),


						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Description wrapper', 'mfn-opts'),
   					),


   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box.mfn-banner-box-boxed .banner-wrapper:width',
							'type' => 'text',
							'condition' => array( 'id' => 'style', 'opt' => 'is', 'val' => 'boxed' ),
							'default_unit' => '%',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Width', 'mfn-opts'),
							'std' => '60%',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box.mfn-banner-box-boxed .banner-wrapper:width_laptop',
							'type' => 'text',
							'condition' => array( 'id' => 'style', 'opt' => 'is', 'val' => 'boxed' ),
							'default_unit' => '%',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Width', 'mfn-opts'),
							'std' => '60%',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box.mfn-banner-box-boxed .banner-wrapper:width_tablet',
							'type' => 'text',
							'condition' => array( 'id' => 'style', 'opt' => 'is', 'val' => 'boxed' ),
							'default_unit' => '%',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Width', 'mfn-opts'),
							'std' => '60%',
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box.mfn-banner-box-boxed .banner-wrapper:width_mobile',
							'type' => 'text',
							'condition' => array( 'id' => 'style', 'opt' => 'is', 'val' => 'boxed' ),
							'default_unit' => '%',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Width', 'mfn-opts'),
							'std' => '60%',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper:max-width',
							'type' => 'text',
							'default_unit' => '%',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Max width', 'mfn-opts'),
							'std' => '100%',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper:max-width_laptop',
							'type' => 'text',
							'default_unit' => '%',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Max width', 'mfn-opts'),
							'std' => '100%',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper:max-width_tablet',
							'type' => 'text',
							'default_unit' => '%',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Max width', 'mfn-opts'),
							'std' => '100%',
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper:max-width_mobile',
							'type' => 'text',
							'default_unit' => '%',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Max width', 'mfn-opts'),
							'std' => '100%',
						),





   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
   						'std' => '',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
   						'std' => '',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
   						'std' => '',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
   						'std' => '',
   					),



						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),


		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box.mfn-banner-box-boxed .banner-wrapper:border-style',
							'attr_id' => 'border_style_bannerdesc',
							'condition' => array( 'id' => 'style', 'opt' => 'is', 'val' => 'boxed' ),
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box.mfn-banner-box-boxed .banner-wrapper:border-width',
		  				'condition' => array( 'id' => 'border_style_bannerdesc', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box.mfn-banner-box-boxed .banner-wrapper:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_bannerdesc', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box.mfn-banner-box-boxed .banner-wrapper:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_bannerdesc', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box.mfn-banner-box-boxed .banner-wrapper:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_bannerdesc', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box.mfn-banner-box-boxed .banner-wrapper:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'condition' => array( 'id' => 'style', 'opt' => 'is', 'val' => 'boxed' ),
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box.mfn-banner-box-boxed .banner-wrapper:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'condition' => array( 'id' => 'style', 'opt' => 'is', 'val' => 'boxed' ),
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box.mfn-banner-box-boxed .banner-wrapper:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'condition' => array( 'id' => 'style', 'opt' => 'is', 'val' => 'boxed' ),
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box.mfn-banner-box-boxed .banner-wrapper:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'condition' => array( 'id' => 'style', 'opt' => 'is', 'val' => 'boxed' ),
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box.mfn-banner-box-boxed .banner-wrapper:box-shadow',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'condition' => array( 'id' => 'style', 'opt' => 'is', 'val' => 'boxed' ),
							'css_attr' => 'box-shadow'
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box.mfn-banner-box-boxed .banner-wrapper:box-shadow_laptop',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'condition' => array( 'id' => 'style', 'opt' => 'is', 'val' => 'boxed' ),
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box.mfn-banner-box-boxed .banner-wrapper:box-shadow_tablet',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'condition' => array( 'id' => 'style', 'opt' => 'is', 'val' => 'boxed' ),
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box.mfn-banner-box-boxed .banner-wrapper:box-shadow_mobile',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'mobile',
							'condition' => array( 'id' => 'style', 'opt' => 'is', 'val' => 'boxed' ),
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),


		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:--mfn-banner-box-hidden-content-gap',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Hidden content gap', 'mfn-opts'),
							'std' => '15px',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:--mfn-banner-box-hidden-content-gap_laptop',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Hidden content gap', 'mfn-opts'),
							'std' => '15px',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:--mfn-banner-box-hidden-content-gap_tablet',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Hidden content gap', 'mfn-opts'),
							'std' => '15px',
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:--mfn-banner-box-hidden-content-gap_mobile',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Hidden content gap', 'mfn-opts'),
							'std' => '15px',
						),











   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled activeif activeif-style mfn-vb-formrow mfn-vb-mfnuidhere" data-conditionid="style" data-opt="is" data-val="boxed"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box.mfn-banner-box-boxed .banner-wrapper:border-color',
							'condition' => array( 'id' => 'border_style_bannerdesc', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box.mfn-banner-box-boxed .banner-wrapper:background-color',
							'type' => 'color',
							'condition' => array( 'id' => 'style', 'opt' => 'is', 'val' => 'boxed' ),
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box.mfn-banner-box-boxed|hover .banner-wrapper:border-color',
							'condition' => array( 'id' => 'border_style_bannerdesc', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box.mfn-banner-box-boxed.mfn-banner-box-boxed|hover .banner-wrapper:background-color',
							'type' => 'color',
							'condition' => array( 'id' => 'style', 'opt' => 'is', 'val' => 'boxed' ),
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),



		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),


						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'toggle_fields mfn-toggled',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .title:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),


		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box|hover .banner-wrapper .title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),


						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Subtitle', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .subtitle:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'toggle_fields mfn-toggled',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .subtitle:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .subtitle:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .subtitle:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .subtitle:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),







		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .subtitle:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box|hover .banner-wrapper .subtitle:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),





						array(
							'type' => 'header',
							'condition' => array( 'id' => 'bb_excerpt', 'opt' => 'isnt', 'val' => '' ),
							'class' => 'toggled_header row-header',
   						'title' => __('Excerpt', 'mfn-opts'),
   					),



						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .excerpt:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'toggle_fields mfn-toggled',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .excerpt:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .excerpt:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .excerpt:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .excerpt:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),






		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .excerpt:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box|hover .banner-wrapper .excerpt:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

   					array(
							'type' => 'header',
							'condition' => array( 'id' => 'cta', 'opt' => 'is', 'val' => '' ),
							'class' => 'toggled_header row-header',
   						'title' => __('CTA', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-text:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'toggle_fields mfn-toggled',
							'title' => __('Typography', 'mfn-opts'),
						),

   					array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-text:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-text:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-text:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-text:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-text:padding',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-text:padding_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-text:padding_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-text:padding_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-text:border-style',
							'attr_id' => 'border_style_bannercta',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),



						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-text:border-width',
		  				'condition' => array( 'id' => 'border_style_bannercta', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-text:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_bannercta', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-text:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_bannercta', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-text:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_bannercta', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-text:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-text:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-text:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-text:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-text:box-shadow',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow'
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-text:box-shadow_laptop',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-text:box-shadow_tablet',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-text:box-shadow_mobile',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),













		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-text:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-text:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-text:border-color',
							'condition' => array( 'id' => 'border_style_bannercta', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box|hover .banner-wrapper .cta-text:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box|hover .banner-wrapper .cta-text:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box|hover .banner-wrapper .cta-text:border-color',
							'condition' => array( 'id' => 'border_style_bannercta', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),



		  			array(
							'type' => 'header',
							'condition' => array( 'id' => 'cta', 'opt' => 'is', 'val' => 'icon' ),
							'class' => 'toggled_header row-header',
   						'title' => __('CTA', 'mfn-opts'),
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:--mfn-banner-box-cta-icon-size',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Icon size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 1000, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:--mfn-banner-box-cta-icon-size_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Icon size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 1000, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:--mfn-banner-box-cta-icon-size_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Icon size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 1000, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:--mfn-banner-box-cta-icon-size_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Icon size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 1000, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-icon:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-icon:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-icon:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-icon:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
		  			),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-icon:width',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Width', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-icon:width_laptop',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Width', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-icon:width_tablet',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Width', 'mfn-opts'),
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-icon:width_mobile',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Width', 'mfn-opts'),
						),


   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-icon:height',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Height', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-icon:height_laptop',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Height', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-icon:height_tablet',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Height', 'mfn-opts'),
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-icon:height_mobile',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Height', 'mfn-opts'),
						),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-icon:border-style',
							'attr_id' => 'border_style_bannericoncta',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),



						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-icon:border-width',
		  				'condition' => array( 'id' => 'border_style_bannericoncta', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-icon:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_bannericoncta', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-icon:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_bannericoncta', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-icon:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_bannericoncta', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-icon:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-icon:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-icon:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-icon:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-icon:box-shadow',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow'
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-icon:box-shadow_laptop',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-icon:box-shadow_tablet',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-icon:box-shadow_mobile',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),





















		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-icon:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-icon:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-icon:border-color',
							'condition' => array( 'id' => 'border_style_bannericoncta', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box|hover .banner-wrapper .cta-icon:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box|hover .banner-wrapper .cta-icon:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box|hover .banner-wrapper .cta-icon:border-color',
							'condition' => array( 'id' => 'border_style_bannericoncta', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),


















		  			array(
							'type' => 'header',
							'condition' => array( 'id' => 'cta', 'opt' => 'is', 'val' => 'image' ),
							'class' => 'toggled_header row-header',
   						'title' => __('CTA', 'mfn-opts'),
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:--mfn-banner-box-cta-image-width',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Image size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 1920, 'step' => 1),
								'%' 	=> array('min' => 1, 	'max' => 100, 	'step' => 1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:--mfn-banner-box-cta-image-width_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Image size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 1920, 'step' => 1),
								'%' 	=> array('min' => 1, 	'max' => 100, 	'step' => 1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:--mfn-banner-box-cta-image-width_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Image size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 1920, 'step' => 1),
								'%' 	=> array('min' => 1, 	'max' => 100, 	'step' => 1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:--mfn-banner-box-cta-image-width_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Image size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 1920, 'step' => 1),
								'%' 	=> array('min' => 1, 	'max' => 100, 	'step' => 1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-image:width',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Width', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-image:width_laptop',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Width', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-image:width_tablet',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Width', 'mfn-opts'),
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-image:width_mobile',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Width', 'mfn-opts'),
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-image:height',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Height', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-image:height_laptop',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Height', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-image:height_tablet',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Height', 'mfn-opts'),
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-image:height_mobile',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Height', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-image:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-image:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-image:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-image:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-image:border-style',
							'attr_id' => 'border_style_bannerimagecta',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-image:border-width',
		  				'condition' => array( 'id' => 'border_style_bannerimagecta', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-image:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_bannerimagecta', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-image:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_bannerimagecta', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-image:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_bannerimagecta', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-image:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-image:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-image:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-image:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-image:box-shadow',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow'
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-image:box-shadow_laptop',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-image:box-shadow_tablet',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-image:box-shadow_mobile',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),




















		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-image:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-wrapper .cta-image:border-color',
							'condition' => array( 'id' => 'border_style_bannerimagecta', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box|hover .banner-wrapper .cta-image:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box|hover .banner-wrapper .cta-image:border-color',
							'condition' => array( 'id' => 'border_style_bannerimagecta', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),



















		  			array(
							'type' => 'header',
							'condition' => array( 'id' => 'bb_badge', 'opt' => 'isnt', 'val' => '' ),
							'class' => 'toggled_header row-header',
   						'title' => __('Badge', 'mfn-opts'),
   					),

		  			array(
							'id' => 'bb_badge_pos',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Position', 'mfn-opts'),
							'options' => [
								'' => __('Top Right', 'mfn-opts'),
								'top-left' => __('Top Left', 'mfn-opts'),
								'bottom-right' => __('Bottom Right', 'mfn-opts'),
								'bottom-left' => __('Bottom Left', 'mfn-opts'),
							],
						),


		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:--mfn-banner-box-badge-offset',
							'type' => 'sliderbar',
							'title' => __('Offset', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '200',
								'step' => '1',
								'unit' => 'px',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:--mfn-banner-box-badge-offset_laptop',
							'type' => 'sliderbar',
							'title' => __('Offset', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '200',
								'step' => '1',
								'unit' => 'px',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:--mfn-banner-box-badge-offset_tablet',
							'type' => 'sliderbar',
							'title' => __('Offset', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '200',
								'step' => '1',
								'unit' => 'px',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box:--mfn-banner-box-badge-offset_mobile',
							'type' => 'sliderbar',
							'title' => __('Offset', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '200',
								'step' => '1',
								'unit' => 'px',
							),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-badge:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-badge:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-badge:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-badge:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-badge:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'toggle_fields mfn-toggled',
							'title' => __('Typography', 'mfn-opts'),
						),



		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-badge:border-style',
							'attr_id' => 'border_style_bannerimagecta',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),



						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-badge:border-width',
		  				'condition' => array( 'id' => 'border_style_bannerimagecta', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-badge:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_bannerimagecta', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-badge:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_bannerimagecta', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-badge:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_bannerimagecta', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-badge:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-badge:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-badge:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-badge:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-badge:box-shadow',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow'
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-badge:box-shadow_laptop',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-badge:box-shadow_tablet',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-badge:box-shadow_mobile',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),













		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-badge:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-badge:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box .banner-badge:border-color',
							'condition' => array( 'id' => 'border_style_bannerimagecta', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box|hover .banner-badge:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box|hover .banner-badge:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-banner-box|hover .banner-badge:border-color',
							'condition' => array( 'id' => 'border_style_bannerimagecta', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),











						array(
							'type' => 'html',
							'html' => '</div>',
						),


					),

   			),

   			// Before After  ---------------------------------------------------

   			'before_after' => array(
   				'type' => 'before_after',
   				'title' => __('Before After', 'mfn-opts'),
   				'size' => '1/3',
   				'tablet_size' => '1/3',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'boxes',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						// image

						array(
							'type' => 'header',
   						'title' => __('Image', 'mfn-opts'),
   					),

   					array(
   						'id' => 'image_before',
   						'dynamic_data' => 'featured_image',
   						'type' => 'upload',
   						'title' => __('Before', 'mfn-opts'),
   						'desc' => __('Recommended image width <b>768px - 1920px</b> depending on size of the item', 'mfn-opts'),
							'std' => $this->get_placeholder(),
							'preview' => 'image',
   					),

   					array(
   						'id' => 'image_after',
   						'dynamic_data' => 'featured_image',
   						'type' => 'upload',
   						'title' => __('After', 'mfn-opts'),
   						'desc' => __('Both images <b>must have the same size</b>', 'mfn-opts'),
							'std' => $this->get_placeholder(),
   					),

					// label

						array(
							'type' => 'header',
   						'title' => __('Label', 'mfn-opts'),
   					),

						array(
		  				'id' => 'label_before',
		  				'type' => 'text',
		  				'title' => __('Before', 'mfn-opts'),
		  			),

						array(
		  				'id' => 'label_after',
		  				'type' => 'text',
		  				'title' => __('After', 'mfn-opts'),
		  			),

						// deprecated

						array(
							'class' => 'mfn-deprecated',
							'type' => 'header',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Slider', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .twentytwenty-wrapper:--mfn-before-after-slider',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Label', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .twentytwenty-wrapper:--mfn-before-after-label',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .twentytwenty-wrapper:--mfn-before-after-label-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Blockquote -----------------------------------------------------

   			'blockquote' => array(
   				'type' => 'blockquote',
   				'title' => __('Blockquote', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'used_fonts' => '',
   				'cat' => 'typography',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'content',
   						'type' => 'textarea',
   						'title' => __('Content', 'mfn-opts'),
   						'std' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut elit tellus, luctus nec ullamcorper mattis, pulvinar dapibus leo.',
							'class' => 'form-content-full-width',
							'editor' => 'basic',
							'preview' => 'content',
   					),

   					array(
   						'id' => 'icon',
   						'type' => 'icon',
   						'themeoptions' => 'style:',
   						'title' => __('Icon', 'mfn-opts'),
   					),

   					array(
   						'id' => 'author',
   						'type' => 'text',
   						'edit_tag' => '.blockquote .author | span',
   						'edit_position' => 'append',
   						'title' => __('Author', 'mfn-opts'),
   						'std' => __('This is the author', 'mfn-opts'),
   					),

   					array(
   						'id' => 'icon_author',
   						'type' => 'icon',
   						'title' => __('Icon author', 'mfn-opts'),
   					),

						// link

						array(
							'type' => 'header',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link',
   						'type' => 'text',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'target',
   						'type' => 'select',
   						'title' => __('Target', 'mfn-opts'),
   						'options' => array(
   							0 => __('Default | _self', 'mfn-opts'),
   							1 => __('New tab or window | _blank', 'mfn-opts'),
   							'lightbox' => __('Lightbox (image or embed video)', 'mfn-opts'),
   						),
   					),

            array(
							'id' => 'link_title',
							'type' => 'text',
							'title' => __('Link title', 'mfn-opts'),
						),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

						array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote blockquote:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote blockquote:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote blockquote:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote blockquote:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote blockquote:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote blockquote:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote blockquote:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote blockquote:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote blockquote:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote blockquote:text-shadow',
							'type' => 'text_shadow',
							'class' => 'mfn-toggled',
							'title' => __('Text shadow', 'mfn-opts'),
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote blockquote:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'toggle_fields mfn-toggled',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'themeoptions' => 'style:',
   						'title' => __('Content icon', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .mfn-blockquote-icon:color',
							'type' => 'color',
							'themeoptions' => 'style:',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .mfn-blockquote-icon:font-size',
							'type' => 'sliderbar',
							'themeoptions' => 'style:',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .mfn-blockquote-icon:font-size_laptop',
							'type' => 'sliderbar',
							'themeoptions' => 'style:',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .mfn-blockquote-icon:font-size_tablet',
							'type' => 'sliderbar',
							'themeoptions' => 'style:',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .mfn-blockquote-icon:font-size_mobile',
							'type' => 'sliderbar',
							'themeoptions' => 'style:',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .mfn-blockquote-icon:top',
							'type' => 'text',
							'themeoptions' => 'style:',
							'responsive' => 'desktop',
							'default_unit' => 'px',
							'class' => 'mfn-toggled',
							'title' => __('Top', 'mfn-opts')
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .mfn-blockquote-icon:top_laptop',
							'type' => 'text',
							'themeoptions' => 'style:',
							'responsive' => 'laptop',
							'default_unit' => 'px',
							'class' => 'mfn-toggled',
							'title' => __('Top', 'mfn-opts')
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .mfn-blockquote-icon:top_tablet',
							'type' => 'text',
							'themeoptions' => 'style:',
							'responsive' => 'tablet',
							'default_unit' => 'px',
							'class' => 'mfn-toggled',
							'title' => __('Top', 'mfn-opts')
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .mfn-blockquote-icon:top_mobile',
							'type' => 'text',
							'themeoptions' => 'style:',
							'responsive' => 'mobile',
							'default_unit' => 'px',
							'class' => 'mfn-toggled',
							'title' => __('Top', 'mfn-opts')
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .mfn-blockquote-icon:left',
							'type' => 'text',
							'themeoptions' => 'style:',
							'responsive' => 'desktop',
							'default_unit' => 'px',
							'class' => 'mfn-toggled',
							'title' => __('Left', 'mfn-opts')
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .mfn-blockquote-icon:left_laptop',
							'type' => 'text',
							'themeoptions' => 'style:',
							'responsive' => 'laptop',
							'default_unit' => 'px',
							'class' => 'mfn-toggled',
							'title' => __('Left', 'mfn-opts')
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .mfn-blockquote-icon:left_tablet',
							'type' => 'text',
							'themeoptions' => 'style:',
							'responsive' => 'tablet',
							'default_unit' => 'px',
							'class' => 'mfn-toggled',
							'title' => __('Left', 'mfn-opts')
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .mfn-blockquote-icon:left_mobile',
							'type' => 'text',
							'themeoptions' => 'style:',
							'responsive' => 'mobile',
							'default_unit' => 'px',
							'class' => 'mfn-toggled',
							'title' => __('Left', 'mfn-opts')
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .mfn-blockquote-icon:bottom',
							'type' => 'text',
							'themeoptions' => 'style:',
							'responsive' => 'desktop',
							'default_unit' => 'px',
							'class' => 'mfn-toggled',
							'title' => __('Bottom', 'mfn-opts')
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .mfn-blockquote-icon:bottom_laptop',
							'type' => 'text',
							'themeoptions' => 'style:',
							'responsive' => 'laptop',
							'default_unit' => 'px',
							'class' => 'mfn-toggled',
							'title' => __('Bottom', 'mfn-opts')
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .mfn-blockquote-icon:bottom_tablet',
							'type' => 'text',
							'themeoptions' => 'style:',
							'responsive' => 'tablet',
							'default_unit' => 'px',
							'class' => 'mfn-toggled',
							'title' => __('Bottom', 'mfn-opts')
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .mfn-blockquote-icon:bottom_mobile',
							'type' => 'text',
							'themeoptions' => 'style:',
							'responsive' => 'mobile',
							'default_unit' => 'px',
							'class' => 'mfn-toggled',
							'title' => __('Bottom', 'mfn-opts')
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .mfn-blockquote-icon:right',
							'type' => 'text',
							'themeoptions' => 'style:',
							'responsive' => 'desktop',
							'default_unit' => 'px',
							'class' => 'mfn-toggled',
							'title' => __('Right', 'mfn-opts')
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .mfn-blockquote-icon:right_laptop',
							'type' => 'text',
							'themeoptions' => 'style:',
							'responsive' => 'laptop',
							'default_unit' => 'px',
							'class' => 'mfn-toggled',
							'title' => __('Right', 'mfn-opts')
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .mfn-blockquote-icon:right_tablet',
							'type' => 'text',
							'themeoptions' => 'style:',
							'responsive' => 'tablet',
							'default_unit' => 'px',
							'class' => 'mfn-toggled',
							'title' => __('Right', 'mfn-opts')
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .mfn-blockquote-icon:right_mobile',
							'type' => 'text',
							'themeoptions' => 'style:',
							'responsive' => 'mobile',
							'default_unit' => 'px',
							'class' => 'mfn-toggled',
							'title' => __('Right', 'mfn-opts')
						),


						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Author', 'mfn-opts'),
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .author:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .author:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .author:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),
   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .author:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .author span,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .author a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .author:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'toggle_fields mfn-toggled',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .author:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .author:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .author:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .author:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Author icon', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .author i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Icon color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .author i:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .author i:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .author i:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote .author i:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'themeoptions' => 'style:',
   						'title' => __('Lines', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote blockquote:text-decoration-color',
							'type' => 'color',
							'themeoptions' => 'style:',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote blockquote:text-underline-offset',
							'type' => 'text',
							'themeoptions' => 'style:',
							'class' => 'mfn-toggled',
							'default_unit' => 'px',
							'title' => __('Offset', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blockquote blockquote:text-decoration-thickness',
							'type' => 'text',
							'themeoptions' => 'style:',
							'class' => 'mfn-toggled',
							'default_unit' => 'px',
							'title' => __('Thickness', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Blog -----------------------------------------------------------

   			'blog' => array(
   				'type' => 'blog',
   				'title' => __('Blog', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'loops',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'count',
   						're_render' => true,
   						'type' => 'text',
   						'title' => __('Posts number', 'mfn-opts'),
   						'std' => '3',
							'after' => 'posts',
							'param' => 'number',
							'class' => 'narrow',
							'preview' => 'number',
   					),

   					array(
   						'id' => 'style',
   						're_render' => true,
   						'type' => 'select',
   						'title' => __('Style', 'mfn-opts'),
   						'options' => array(
   							'classic' => __('Classic - 1 column', 'mfn-opts'),
   							'grid' => __('Grid - 2-4 columns', 'mfn-opts'),
   							'masonry' => __('Masonry blog style - 2-4 columns', 'mfn-opts'),
   							'masonry tiles' => __('Masonry tiles - 2-4 columns', 'mfn-opts'),
   							'photo' => __('Photo - 1 column', 'mfn-opts'),
   							'photo2' => __('Photo 2 - 1-3 columns', 'mfn-opts'),
   							'timeline' => __('Timeline - 1 column', 'mfn-opts'),
   						),
   						'std' => 'grid',
   						'preview' => 'style',
   					),

   					array(
   						'id' => 'columns',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Columns', 'mfn-opts'),
   						'desc' => __('for style: Grid, Masonry, Photo 2', 'mfn-opts'),
   						'options' => array(
   							2	=> 2,
   							3	=> 3,
   							4	=> 4,
   							5	=> 5,
   							6	=> 6,
   						),
   						'std' => 3,
   					),

						array(
   						'id' => 'title_tag',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Title tag', 'mfn-opts'),
   						'options' => array(
   							'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
   						),
   						'std' => 'h2'
   					),

   					array(
   						'id' => 'images',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Post image', 'mfn-opts'),
   						'desc' => __('for all styles except Masonry tiles', 'mfn-opts'),
   						'options' => array(
   							'' => 'Default',
   							'images-only' => 'Featured images only',
   						),
   					),

   					// options

   					array(
   						'type' => 'header',
   						'title' => __('Options', 'mfn-opts'),
   					),

   					array(
   						'id' => 'category',
   						're_render' => true,
   						'type' => 'select',
   						'title' => __('Category', 'mfn-opts'),
   						'php_options' => mfn_get_categories('category'),
   						'js_hierarchical_options' => 'blog_categories',
   						'preview' => 'category',
   					),

   					array(
   						'id' => 'category_multi',
   						're_render' => true,
   						'type' => 'text',
   						'title' => __('Multiple categories', 'mfn-opts'),
   						'desc' => __('Slugs should be separated with <b>coma</b> ( , )', 'mfn-opts'),
							'preview' => 'category-all',
   					),

   					array(
   						'id' => 'orderby',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Order by', 'mfn-opts'),
   						'desc' => __('Do <b>not</b> use random order with pagination or load more', 'mfn-opts'),
   						'options' => array(
   							'date' => __('Date', 'mfn-opts'),
   							'title' => __('Title', 'mfn-opts'),
   							'rand' => __('Random', 'mfn-opts'),
   						),
   						'std' => 'date'
   					),

   					array(
   						'id' => 'order',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Order', 'mfn-opts'),
   						'options' => array(
								'DESC' => __('Descending', 'mfn-opts'),
   							'ASC' => __('Ascending', 'mfn-opts'),
   						),
   						'std' => 'DESC'
   					),

   					array(
   						'id' => 'sticky_posts',
   						'type' => 'switch',
   						'title' => __('Sticky posts', 'mfn-opts'),
   						'options' => array(
								'' => __('Include', 'mfn-opts'),
   							'1' => __('Exclude', 'mfn-opts'),
   						),
   						'std' => ''
   					),


						// advanced

   					array(
   						'type' => 'header',
   						'title' => __('Advanced', 'mfn-opts'),
   					),

   					array(
   						'id' => 'exclude_id',
   						're_render' => true,
   						'type' => 'text',
   						'title' => __('Exclude posts', 'mfn-opts'),
   						'desc' => __('IDs should be separated with <b>coma</b> ( , )', 'mfn-opts'),
   					),

						array(
   						'id' => 'related',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Use as related posts', 'mfn-opts'),
   						'desc' => __('Exclude current post on single post page. This option will overwrite exclude posts option above.', 'mfn-opts'),
   						'options' => array(
   							0 => __('Disable', 'mfn-opts'),
   							1 => __('Enable', 'mfn-opts'),
   						),
							'std' => 0,
   					),

   					array(
   						'id' => 'filters',
   						'attr_id' => 'filters',
   						're_render' => true,
   						'type' => 'select',
   						'title' => __('Filters', 'mfn-opts'),
   						'desc' => __('for style: Masonry and category: All<br />Does <b>not</b> work with pagination or load more', 'mfn-opts'),
   						'options' => array(
   							'0' => __('Hide', 'mfn-opts'),
   							'1' => __('Show', 'mfn-opts'),
   							'only-categories' => __('Show only categories', 'mfn-opts'),
   							'only-tags' => __('Show only tags', 'mfn-opts'),
   							'only-authors' => __('Show only authors', 'mfn-opts'),
   						),
   						'std' => '0'
   					),

						array(
   						'id' => 'excerpt',
   						're_render' => true,
   						'type' => 'switch',
							'title' => __('Excerpt', 'mfn-opts'),
   						'options' => array(
								0 => __('Hide', 'mfn-opts'),
   							1 => __('Show', 'mfn-opts'),
							),
   						'std' => 1,
   					),

   					array(
   						'id' => 'more',
   						're_render' => true,
   						'type' => 'switch',
							'title' => __('Read more', 'mfn-opts'),
   						'options' => array(
								0 => __('Hide', 'mfn-opts'),
   							1 => __('Show', 'mfn-opts'),
							),
   						'std' => 1,
   					),

						// pagination

   					array(
   						'type' => 'header',
   						'title' => __('Pagination', 'mfn-opts'),
   					),

   					array(
   						'id' => 'pagination',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Pagination', 'mfn-opts'),
   						'desc' => __('Does <b>not</b> work on WMPL homepage', 'mfn-opts'),
							'options' => array(
								0 => __('Hide', 'mfn-opts'),
   							1 => __('Show', 'mfn-opts'),
							),
							'std' => 0,
   					),

   					array(
   						'id' => 'load_more',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Load more', 'mfn-opts'),
   						'desc' => __('Sliders will be replaced with featured images', 'mfn-opts'),
   						'options' => array(
								0 => __('Hide', 'mfn-opts'),
   							1 => __('Show', 'mfn-opts'),
   						),
							'std' => 0,
   					),

						// style

   					array(
   						'type' => 'header',
   						'title' => __('Style', 'mfn-opts'),
   					),

   					array(
   						'id' => 'greyscale',
   						'type' => 'switch',
   						'title' => __('Grayscale', 'mfn-opts'),
							'options' => array(
   							0 => __('Disable', 'mfn-opts'),
   							1 => __('Enable', 'mfn-opts'),
   						),
							'std' => 0,
   					),

   					array(
   						'id' => 'margin',
   						'desc' => __('For style: Masonry tiles', 'mfn-opts'),
   						'type' => 'switch',
   						'title' => __('Margin', 'mfn-opts'),
							'options' => array(
   							0 => __('Disable', 'mfn-opts'),
   							1 => __('Enable', 'mfn-opts'),
   						),
							'std' => 0,
   					),

						// plugins

						array(
							'type' => 'header',
							'title' => __('Plugins', 'mfn-opts'),
						),

   					array(
   						'id' => 'events',
   						'type' => 'switch',
   						'title' => __('Events', 'mfn-opts'),
   						'desc' => __('for category: All<br />requires free The Events Calendar plugin', 'mfn-opts'),
   						'options' => array(
								0 => __('Exclude', 'mfn-opts'),
   							1 => __('Include', 'mfn-opts'),
   						),
							'std' => 0,
   					),

						// deprecated

						array(
							'class' => 'mfn-deprecated',
							'type' => 'header',
							'title' => __('Deprecated', 'mfn-opts'),
						),

   					array(
   						'id' => 'margin',
   						'type' => 'switch',
   						'class' => 'mfn-deprecated',
   						'title' => __('Margin', 'mfn-opts'),
   						'desc' => __('for style: Masonry tiles', 'mfn-opts'),
							'options' => array(
								0 => __('Hide', 'mfn-opts'),
   							1 => __('Show', 'mfn-opts'),
   						),
							'std' => 0,
   					),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Container', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item:border-style',
							'attr_id' => 'border_style_blog_item',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item:border-color',
							'condition' => array( 'id' => 'border_style_blog_item', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item:border-width',
		  				'condition' => array( 'id' => 'border_style_blog_item', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_blog_item', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_blog_item', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_blog_item', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Image', 'mfn-opts'),
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .image_frame:border-style',
							'attr_id' => 'border_style_blogitem',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .image_frame:border-color',
							'condition' => array( 'id' => 'border_style_blogitem', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .image_frame:border-width',
		  				'condition' => array( 'id' => 'border_style_blogitem', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .image_frame:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_blogitem', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .image_frame:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_blogitem', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .image_frame:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_blogitem', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .image_frame:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .image_frame:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .image_frame:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .image_frame:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc-wrapper:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc-wrapper:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc-wrapper:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc-wrapper:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Post header', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-head:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-head .post-meta:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-head:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-head:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-head:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-head:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-head:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-head:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-head:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-head:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-head:border-style',
							'attr_id' => 'border_style_bloghead',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-head:border-color',
							'condition' => array( 'id' => 'border_style_bloghead', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-head:border-width',
		  				'condition' => array( 'id' => 'border_style_bloghead', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-head:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_bloghead', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-head:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_bloghead', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-head:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_bloghead', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-head:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-head:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-head:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-head:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-head:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-head .post-meta a:color',
							'type' => 'color',
							'title' => __('Link color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-head .post-meta a|hover:color',
							'type' => 'color',
							'title' => __('Link color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-title .entry-title a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-title .entry-title:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-title .entry-title:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-title .entry-title:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-title .entry-title:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-title .entry-title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Excerpt', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-excerpt:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-excerpt:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-excerpt:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-excerpt:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-excerpt:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-excerpt:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Post footer', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-footer:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-footer,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-footer .button-love a .label:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-footer:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-footer:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-footer:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-footer:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-footer:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-footer:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-footer:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-footer:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-footer:border-style',
							'attr_id' => 'border_style_blogpf',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-footer:border-color',
							'condition' => array( 'id' => 'border_style_blogpf', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-footer:border-width',
		  				'condition' => array( 'id' => 'border_style_blogpf', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-footer:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_blogpf', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-footer:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_blogpf', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-footer:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_blogpf', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-footer:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-footer:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-footer:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-footer:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-footer:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-footer a:color',
							'type' => 'color',
							'title' => __('Link color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-desc .post-footer a|hover:color',
							'type' => 'color',
							'title' => __('Link color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Masonry tiles line', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .post-title|after:background',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Link color', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Timeline', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .date_label:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Date color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .date_label:--mfn-blog-date-bg',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Date background', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .date_label:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .date_label:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .date_label:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item .date_label:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_wrapper .post-item|before:border-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Dot color', 'mfn-opts'),
						),







						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'condition' => array( 'id' => 'filters', 'opt' => 'isnt', 'val' => '0' ),
   						'title' => __('Filters', 'mfn-opts'),
   					),


						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Nav', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_buttons:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_buttons:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_buttons:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_buttons:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_buttons:background',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_buttons:border-style',
							'attr_id' => 'border_style_blog_filters',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_buttons:border-color',
							'condition' => array( 'id' => 'border_style_blog_filters', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_buttons:border-width',
		  				'condition' => array( 'id' => 'border_style_blog_filters', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_buttons:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_blog_filters', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_buttons:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_blog_filters', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_buttons:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_blog_filters', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_buttons:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_buttons:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_buttons:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_buttons:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_buttons li:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Label color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_buttons li:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),


						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_buttons li a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Links color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_buttons li a|hover:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Links color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Filters', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:background',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:border-style',
							'attr_id' => 'border_blog_filters_wrapper',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:border-width',
		  				'condition' => array( 'id' => 'border_blog_filters_wrapper', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:border-width_laptop',
		  				'condition' => array( 'id' => 'border_blog_filters_wrapper', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:border-width_tablet',
		  				'condition' => array( 'id' => 'border_blog_filters_wrapper', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:border-width_mobile',
		  				'condition' => array( 'id' => 'border_blog_filters_wrapper', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),


		  			array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Filters items', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li:width',
							'attr_id' => 'filters_items_width_switcher',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Items width', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'auto' => __('Auto', 'mfn-opts'),
							],
							'std' => ''
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),



		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:border-style',
							'attr_id' => 'border_blog_filters_wrapper_li',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:border-width',
		  				'condition' => array( 'id' => 'border_blog_filters_wrapper_li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:border-width_laptop',
		  				'condition' => array( 'id' => 'border_blog_filters_wrapper_li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:border-width_tablet',
		  				'condition' => array( 'id' => 'border_blog_filters_wrapper_li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:border-width_mobile',
		  				'condition' => array( 'id' => 'border_blog_filters_wrapper_li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),




		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),


						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Links color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:border-color',
							'condition' => array( 'id' => 'border_blog_filters_wrapper_li', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a|hover:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Links color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a|hover:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a|hover:border-color',
							'condition' => array( 'id' => 'border_blog_filters_wrapper_li', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li.current-cat a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Links color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li.current-cat a:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li.current-cat a:border-color',
							'condition' => array( 'id' => 'border_blog_filters_wrapper_li', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),


						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Filters exit', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li.close a:width',
							'attr_id' => 'filters_items_width_switcher',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Width', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'auto' => __('Auto', 'mfn-opts'),
							],
							'std' => ''
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li.close a:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li.close a:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li.close a:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li.close a:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),


		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li.close a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li.close a:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li.close a:border-color',
							'condition' => array( 'id' => 'border_blog_filters_wrapper_li', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li.close a|hover:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li.close a|hover:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li.close a|hover:border-color',
							'condition' => array( 'id' => 'border_blog_filters_wrapper_li', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),










						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Blog News ------------------------------------------------------

   			'blog_news' => array(
   				'type' => 'blog_news',
   				'title' => __('Blog News', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'loops',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'title',
   						'type' => 'text',
   						'dynamic_data' => 'title',
   						'edit_tag' => '.Latest_news | .title',
   						'edit_position' => 'prepend',
   						'title' => __('Title', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h3',
						),


						array(
							'id' => 'items_title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Items title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h5',
						),


						array(
   						'id' => 'count',
   						'type' => 'text',
   						're_render' => true,
   						'title' => __('Posts number', 'mfn-opts'),
   						'std' => '5',
							'after' => 'posts',
							'param' => 'number',
							'class' => 'narrow',
							'preview' => 'number',
   					),

   					array(
   						'id' => 'style',
   						'attr_id' => 'style',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Style', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'featured' => __('Featured 1st', 'mfn-opts'),
   						),
							'preview' => 'style',
   					),

   					array(
							'id' => 'featured_title_tag',
							'condition' => array( 'id' => 'style', 'opt' => 'is', 'val' => 'featured' ),
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Featured title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h4',
						),

   					// options

   					array(
   						'type' => 'header',
   						'title' => __('Options', 'mfn-opts'),
   					),

   					array(
   						'id' => 'category',
   						're_render' => true,
   						'type' => 'select',
   						'title' => __('Category', 'mfn-opts'),
   						'php_options' => mfn_get_categories('category'),
   						'js_hierarchical_options' => 'blog_categories',
   						'preview' => 'category',
   					),

   					array(
   						'id' => 'category_multi',
   						're_render' => true,
   						'type' => 'text',
   						'title' => __('Multiple categories', 'mfn-opts'),
   						'desc' => __('Slugs should be separated with <b>coma</b> ( , )', 'mfn-opts'),
							'preview' => 'category-all',
   					),

   					array(
   						'id' => 'orderby',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Order by', 'mfn-opts'),
   						'desc' => __('Do <b>not</b> use random order with pagination or load more', 'mfn-opts'),
   						'options' => array(
   							'date' => __('Date', 'mfn-opts'),
   							'title' => __('Title', 'mfn-opts'),
   							'rand' => __('Random', 'mfn-opts'),
   						),
   						'std' => 'date',
   					),

   					array(
   						'id' => 'order',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Order', 'mfn-opts'),
   						'options' => array(
								'DESC' => __('Descending', 'mfn-opts'),
   							'ASC' => __('Ascending', 'mfn-opts'),
   						),
   						'std' => 'DESC',
   					),

   					array(
   						'id' => 'sticky_posts',
   						'type' => 'switch',
   						'title' => __('Sticky posts', 'mfn-opts'),
   						'options' => array(
								'' => __('Include', 'mfn-opts'),
   							'1' => __('Exclude', 'mfn-opts'),
   						),
   						'std' => ''
   					),

   					// advanced

   					array(
   						'type' => 'header',
   						'title' => __('Advanced', 'mfn-opts'),
   					),

   					array(
   						'id' => 'excerpt',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Excerpt', 'mfn-opts'),
   						'options' => array(
   							0 => __('Hide', 'mfn-opts'),
   							1 => __('Show', 'mfn-opts'),
   							'featured' => __('Show for featured only', 'mfn-opts'),
   						),
							'std' => 0,
   					),

   					array(
   						'id' => 'link',
   						'type' => 'text',
   						'dynamic_data' => 'permalink',
   						're_render' => true,
   						'title' => __('Button link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link_title',
   						'type' => 'text',
   						'dynamic_data' => 'title',
   						're_render' => true,
   						'title' => __('Button title', 'mfn-opts'),
   					),

						// deprecated

						array(
							'class' => 'mfn-deprecated',
							'type' => 'header',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Item title', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news .title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news .title:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Image', 'mfn-opts'),
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news li .photo:border-style',
							'attr_id' => 'border_style_lnp',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news li .photo:border-color',
							'condition' => array( 'id' => 'border_style_lnp', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news li .photo:border-width',
		  				'condition' => array( 'id' => 'border_style_lnp', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news li .photo:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_lnp', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news li .photo:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_lnp', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news li .photo:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_lnp', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news li .photo:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news li .photo:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news li .photo:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news li .photo:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Featured title', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news.featured .desc h4 a:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news.featured .desc h4:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news.featured .desc h4:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news.featured .desc h4:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news.featured .desc h4:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news.featured .desc h4 a:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news .desc h5 a:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news .desc h5:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news .desc h5:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news .desc h5:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news .desc h5:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news .desc h5 a:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Excerpt', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news .desc .post-excerpt:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news .desc .post-excerpt:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news .desc .post-excerpt:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news .desc .post-excerpt:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news .desc .post-excerpt:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news .desc .post-excerpt:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),


		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Post footer', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news ul li .desc_footer:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news ul li .desc_footer,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news ul li .desc_footer .button-love a .label:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news ul li .desc_footer:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news ul li .desc_footer:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news ul li .desc_footer:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news ul li .desc_footer:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news ul li .desc_footer:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news ul li .desc_footer:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news ul li .desc_footer:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news ul li .desc_footer:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news ul li .desc_footer:border-style',
							'attr_id' => 'border_style_lndf',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news ul li .desc_footer:border-color',
							'condition' => array( 'id' => 'border_style_lndf', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news ul li .desc_footer:border-width',
		  				'condition' => array( 'id' => 'border_style_lndf', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news ul li .desc_footer:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_lndf', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news ul li .desc_footer:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_lndf', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news ul li .desc_footer:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_lndf', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news ul li .desc_footer:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news ul li .desc_footer:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news ul li .desc_footer:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news ul li .desc_footer:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news ul li .desc_footer:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news ul li .desc_footer a:color',
							'type' => 'color',
							'title' => __('Link color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .Latest_news ul li .desc_footer a|hover:color',
							'type' => 'color',
							'title' => __('Link color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Blog Slider ----------------------------------------------------

   			'blog_slider' => array(
   				'type' => 'blog_slider',
   				'title' => __('Blog Slider', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'loops',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
   						'id' => 'style',
   						'attr_id' => 'blog_slider_style',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Style', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'flat' => __('Flat', 'mfn-opts'),
   						),
							'std' => '',
   					),

   					array(
   						'id' => 'title',
   						'condition' => array( 'id' => 'blog_slider_style', 'opt' => 'is', 'val' => '' ),
   						'type' => 'text',
   						'title' => __('Title', 'mfn-opts'),
   						'edit_tag' => '.blog_slider_header | .title',
   						'edit_class' => 'title',
						'dynamic_data' => 'title',
   						'edit_position' => 'prepend',
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h4',
						),

						array(
   						'id' => 'count',
   						'type' => 'text',
   						're_render' => true,
   						'title' => __('Posts number', 'mfn-opts'),
   						'std' => '5',
							'after' => 'posts',
							'param' => 'number',
							'class' => 'narrow',
							'preview' => 'number',
   					),

						// options

   					array(
   						'type' => 'header',
   						'title' => __('Options', 'mfn-opts'),
   					),

   					array(
   						'id' => 'category',
   						'type' => 'select',
   						're_render' => true,
   						'title' => __('Category', 'mfn-opts'),
   						'php_options' => mfn_get_categories('category'),
   						'js_hierarchical_options' => 'blog_categories',
							'preview' => 'category',
   					),

   					array(
   						'id' => 'category_multi',
   						'type' => 'text',
   						're_render' => true,
   						'title' => __('Multiple categories', 'mfn-opts'),
   						'desc' => __('Slugs should be separated with <b>coma</b> ( , )', 'mfn-opts'),
							'preview' => 'category-all',
   					),

   					array(
   						'id' => 'orderby',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Order by', 'mfn-opts'),
   						'desc' => __('Do <b>not</b> use random order with pagination or load more', 'mfn-opts'),
   						'options' => array(
   							'date' => __('Date', 'mfn-opts'),
   							'title' => __('Title', 'mfn-opts'),
   							'rand' => __('Random', 'mfn-opts'),
   						),
   						'std' => 'date',
   					),

   					array(
   						'id' => 'order',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Order', 'mfn-opts'),
   						'options' => array(
								'DESC' => __('Descending', 'mfn-opts'),
   							'ASC' => __('Ascending', 'mfn-opts'),

   						),
   						'std' => 'DESC'
   					),

   					array(
   						'id' => 'sticky_posts',
   						'type' => 'switch',
   						'title' => __('Sticky posts', 'mfn-opts'),
   						'options' => array(
								'' => __('Include', 'mfn-opts'),
   							'1' => __('Exclude', 'mfn-opts'),
   						),
   						'std' => ''
   					),

						// advanced

   					array(
   						'type' => 'header',
   						'title' => __('Advanced', 'mfn-opts'),
   					),

						array(
							'id' => 'one_post_per_slide',
							'type' => 'switch',
							're_render' => true,
							'title' => __('One post per slide', 'mfn-opts'),
							'desc' => __('Display single post per slide, instead few of them', 'mfn-opts'),
							'options' => array(
								0 => __('Disable', 'mfn-opts'),
								1 => __('Enable', 'mfn-opts'),
							),
							'std' => 0,
						),

						array(
   						'id' => 'excerpt',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Excerpt', 'mfn-opts'),
   						'options' => array(
								'' => __('Hide', 'mfn-opts'),
   							'1' => __('Show', 'mfn-opts'),

   						),
   						'std' => ''
   					),

   					array(
   						'id' => 'more',
   						'condition' => array( 'id' => 'blog_slider_style', 'opt' => 'is', 'val' => '' ),
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Read more', 'mfn-opts'),
   						'options' => array(
   							0 => __('Hide', 'mfn-opts'),
   							1 => __('Show', 'mfn-opts'),
   						),
   						'std' => 1,
   					),

   					array(
   						'id' => 'navigation',
   						'condition' => array( 'id' => 'blog_slider_style', 'opt' => 'is', 'val' => '' ),
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Navigation', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'hide-arrows' => __('Hide arrows', 'mfn-opts'),
   							'hide-dots' => __('Hide dots', 'mfn-opts'),
   							'hide-nav' => __('Hide nav', 'mfn-opts'),
   						),
							'std' => '',
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Item title', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_header .title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_header .title:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_header .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_header .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_header .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_header .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Image', 'mfn-opts'),
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .item_wrapper .image_frame:border-style',
							'attr_id' => 'border_style_bs',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .item_wrapper .image_frame:border-color',
							'condition' => array( 'id' => 'border_style_bs', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .item_wrapper .image_frame:border-width',
		  				'condition' => array( 'id' => 'border_style_bs', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .item_wrapper .image_frame:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_bs', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .item_wrapper .image_frame:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_bs', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .item_wrapper .image_frame:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_bs', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .item_wrapper .image_frame:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .item_wrapper .image_frame:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .item_wrapper .image_frame:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .item_wrapper .image_frame:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .item_wrapper .image_frame:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .item_wrapper .image_frame:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .item_wrapper .image_frame:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .item_wrapper .image_frame:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .desc h4 a:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .desc h4:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .desc h4:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .desc h4:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .desc h4:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .desc h4:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Excerpt', 'mfn-opts'),
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .desc .post_excerpt:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .desc .post_excerpt:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .desc .post_excerpt:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .desc .post_excerpt:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .desc .post_excerpt:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .desc .post_excerpt:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Button', 'mfn-opts'),
   					),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .button:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .button:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .button:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .button:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .button:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .button:border-style',
							'attr_id' => 'border_style_bsb',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .button:border-width',
		  				'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .button:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .button:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .button:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .button:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .button:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .button:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .button:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .button:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .button:border-color',
							'condition' => array( 'id' => 'border_style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher',
							'attr_id' => 'background_switcher',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .button:background-color',
							'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .button:gradient',
		  				'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .button|hover:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .button|hover:border-color',
							'condition' => array( 'id' => 'border_style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher_hover',
							'attr_id' => 'background_switcher_hover',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .button|hover:background',
							'condition' => array( 'id' => 'background_switcher_hover', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .button|hover:gradient',
		  				'condition' => array( 'id' => 'background_switcher_hover', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

























						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Arrows', 'mfn-opts'),
   					),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .blog_slider_header .button:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .blog_slider_header .button:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .blog_slider_header .button:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .blog_slider_header .button:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .blog_slider_header .button:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .blog_slider_header .button:border-style',
							'attr_id' => 'border_style_bsb',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .blog_slider_header .button:border-width',
		  				'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .blog_slider_header .button:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .blog_slider_header .button:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .blog_slider_header .button:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .blog_slider_header .button:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .blog_slider_header .button:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .blog_slider_header .button:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .blog_slider_header .button:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .blog_slider_header .button, .mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .blog_slider_header .button .button_icon i:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .blog_slider_header .button:border-color',
							'condition' => array( 'id' => 'border_style_bsb', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher',
							'attr_id' => 'background_switcher',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .blog_slider_header .button:background-color',
							'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .blog_slider_header .button:gradient',
		  				'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .blog_slider_header .button|hover:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .blog_slider_header .button|hover:border-color',
							'condition' => array( 'id' => 'border_style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher_hover',
							'attr_id' => 'background_switcher_hover',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .blog_slider_header .button|hover:background',
							'condition' => array( 'id' => 'background_switcher_hover', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .blog_slider_header .button|hover:gradient',
		  				'condition' => array( 'id' => 'background_switcher_hover', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),






























						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Dots', 'mfn-opts'),
   					),

   					array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .slider_pagination li a:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .slider_pagination li a:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .slider_pagination li a:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .slider_pagination li a:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
		  			),


						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .slider_pagination li a,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .slider_pagination li a|after:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .slider_pagination li|hover a,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .slider_pagination li|hover a|after:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .slider_pagination li.slick-active a,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider .slider_pagination li.slick-active a|after:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),


































						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Date', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .item_wrapper:--mfn-blog-slider-date-bg',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .date_label:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .date_label:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .date_label:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .date_label:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .date_label:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .date_label:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .date_label:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .date_label:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .date_label:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .date_label:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Line', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_ul li .desc hr:background',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

   					array(
							'type' => 'html',
							'html' => '</div>',
						),

						// advanced here

   				),
   			),

   			// Blog Teaser ------------------------------------------------------

   			'blog_teaser' => array(
   				'type' => 'blog_teaser',
   				'title' => __('Blog Teaser', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'loops',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'title',
   						'type' => 'text',
   						'edit_tag' => '.blog-teaser | .title',
   						'edit_position' => 'prepend',
						'dynamic_data' => 'title',
   						'title' => __('Heading', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
   						'id' => 'heading_tag',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Heading tag', 'mfn-opts'),
   						'options' => array(
   							'h1' => 'H1',
   							'h2' => 'H2',
   							'h3' => 'H3',
   							'h4' => 'H4',
   							'h5' => 'H5',
   							'h6' => 'H6',
   							'p' => 'p',
								'span' => 'span',
								'div' => 'div',
   						),
   						'std' => 'h3'
   					),

   					// options

   					array(
   						'type' => 'header',
   						'title' => __('Options', 'mfn-opts'),
   					),

   					array(
   						'id' => 'title_tag',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Title tag', 'mfn-opts'),
   						'desc' => __('Title tag for 1st item, others use a smaller one', 'mfn-opts'),
   						'options' => array(
   							'h1' => 'H1',
   							'h2' => 'H2',
   							'h3' => 'H3',
   							'h4' => 'H4',
   							'h5' => 'H5',
   							'h6' => 'H6',
   							'p' => 'p',
								'span' => 'span',
								'div' => 'div',
   						),
   						'std' => 'h3'
   					),

   					array(
   						'id' => 'category',
   						'type' => 'select',
   						're_render' => true,
   						'title' => __('Category', 'mfn-opts'),
   						'php_options' => mfn_get_categories('category'),
   						'js_hierarchical_options' => 'blog_categories',
							'preview' => 'category',
   					),

   					array(
   						'id' => 'category_multi',
   						'type' => 'text',
   						're_render' => true,
   						'title' => __('Multiple categories', 'mfn-opts'),
   						'desc' => __('Slugs should be separated with <b>coma</b> ( , )', 'mfn-opts'),
							'preview' => 'category-all',
   					),

   					array(
   						'id' => 'orderby',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Order by', 'mfn-opts'),
   						'desc' => __('Do <b>not</b> use random order with pagination or load more', 'mfn-opts'),
   						'options' => array(
   							'date' => __('Date', 'mfn-opts'),
   							'title' => __('Title', 'mfn-opts'),
   							'rand' => __('Random', 'mfn-opts'),
   						),
   						'std' => 'date'
   					),

   					array(
   						'id' => 'order',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Order', 'mfn-opts'),
   						'options' => array(
								'DESC' => __('Descending', 'mfn-opts'),
   							'ASC' => __('Ascending', 'mfn-opts'),

   						),
   						'std' => 'DESC'
   					),

   					array(
   						'id' => 'sticky_posts',
   						'type' => 'switch',
   						'title' => __('Sticky posts', 'mfn-opts'),
   						'options' => array(
								'' => __('Include', 'mfn-opts'),
   							'1' => __('Exclude', 'mfn-opts'),
   						),
   						'std' => ''
   					),

   					// advanced

   					array(
   						'type' => 'header',
   						'title' => __('Advanced', 'mfn-opts'),
   					),

   					array(
   						'id' => 'margin',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Margin', 'mfn-opts'),
   						'options' => array(
								'0' => __('Disable', 'mfn-opts'),
   							'1' => __('Enable', 'mfn-opts'),
   						),
							'std' => '0'
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Item title', 'mfn-opts'),
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .title:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Image', 'mfn-opts'),
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .teaser-wrapper li:border-style',
							'attr_id' => 'border_style_tw',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .teaser-wrapper li:border-color',
							'condition' => array( 'id' => 'border_style_tw', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .teaser-wrapper li:border-width',
		  				'condition' => array( 'id' => 'border_style_tw', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .teaser-wrapper li:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_tw', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .teaser-wrapper li:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_tw', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .teaser-wrapper li:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_tw', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .teaser-wrapper li:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .teaser-wrapper li:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .teaser-wrapper li:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .teaser-wrapper li:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .teaser-wrapper li .desc .post-title a:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .teaser-wrapper li .desc .post-title a:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Post header', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .teaser-wrapper li .desc .post-meta:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .teaser-wrapper li .desc .post-meta:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .teaser-wrapper li .desc .post-meta:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .teaser-wrapper li .desc .post-meta:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .teaser-wrapper li .desc .post-meta:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .teaser-wrapper li .desc .post-meta:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .teaser-wrapper li .desc .post-meta:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .teaser-wrapper li .desc .post-meta:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .teaser-wrapper li .desc .post-meta:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .teaser-wrapper li .desc .post-meta:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .teaser-wrapper li .desc .post-meta:border-style',
							'attr_id' => 'border_style_twpm',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .teaser-wrapper li .desc .post-meta:border-color',
							'condition' => array( 'id' => 'border_style_twpm', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .teaser-wrapper li .desc .post-meta:border-width',
		  				'condition' => array( 'id' => 'border_style_twpm', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .teaser-wrapper li .desc .post-meta:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_twpm', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .teaser-wrapper li .desc .post-meta:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_twpm', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .teaser-wrapper li .desc .post-meta:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_twpm', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .teaser-wrapper li .desc .post-meta:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .teaser-wrapper li .desc .post-meta:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .teaser-wrapper li .desc .post-meta:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .teaser-wrapper li .desc .post-meta:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .teaser-wrapper li .desc .post-meta:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .teaser-wrapper li .desc .post-meta a:color',
							'type' => 'color',
							'title' => __('Link color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser .teaser-wrapper li .desc .post-meta a|hover:color',
							'type' => 'color',
							'title' => __('Link color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Line', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog-teaser li .desc-wrapper .desc .post-title|after:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

				// Breadcrumbs ------------------------------------------------------

				'breadcrumbs' => array(
   				'type' => 'breadcrumbs',
   				'title' => __('Breadcrumbs', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'elements',
   				'attr' => array(

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
   						'id' => 'separator',
   						'type' => 'text',
   						'title' => __('Separator', 'mfn-opts'),
							'std' => __('>', 'mfn-opts'),
   					),


						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Container', 'mfn-opts'),
   					),

						array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .breadcrumbs:justify-content',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .breadcrumbs:justify-content_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .breadcrumbs:justify-content_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .breadcrumbs:justify-content_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Items', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .breadcrumbs:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),


   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .breadcrumbs:gap',
							'type' => 'sliderbar',
							'title' => __('Gap', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .breadcrumbs:gap_laptop',
							'type' => 'sliderbar',
							'title' => __('Gap', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .breadcrumbs:gap_tablet',
							'type' => 'sliderbar',
							'title' => __('Gap', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .breadcrumbs:gap_mobile',
							'type' => 'sliderbar',
							'title' => __('Gap', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .breadcrumbs li:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Text color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .breadcrumbs li a:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Link color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .breadcrumbs li|hover a:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Link color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),


						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Separator', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .breadcrumbs li .mfn-breadcrumbs-separator:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .breadcrumbs li .mfn-breadcrumbs-separator:font-size',
							'type' => 'sliderbar',
							'title' => __('Size', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .breadcrumbs li .mfn-breadcrumbs-separator:font-size_laptop',
							'type' => 'sliderbar',
							'title' => __('Size', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .breadcrumbs li .mfn-breadcrumbs-separator:font-size_tablet',
							'type' => 'sliderbar',
							'title' => __('Size', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .breadcrumbs li .mfn-breadcrumbs-separator:font-size_mobile',
							'type' => 'sliderbar',
							'title' => __('Size', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							),
						),


						array(
							'type' => 'html',
							'html' => '</div>',
						)

   				)
   			),

   			// Button ----------------------------------------------------

   			'button' => array(
   				'type' => 'button',
   				'title' => __('Button', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'typography',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'title',
   						'type' => 'text',
   						'edit_tag' => '.button | span.button_label',
   						'edit_position' => 'append',
   						'dynamic_data' => 'title',
   						'title' => __('Title', 'mfn-opts'),
							'std' => __('Click here', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
   						'id' => 'link',
   						'type' => 'text',
   						'dynamic_data' => 'permalink',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'target',
   						'type' => 'select',
   						'title' => __('Link target', 'mfn-opts'),
   						'options' => array(
   							0 => __('Default | _self', 'mfn-opts'),
   							1 => __('New tab or window | _blank', 'mfn-opts'),
   							'lightbox' => __('Lightbox (image or embed video)', 'mfn-opts'),
   						),
   					),

            array(
							'id' => 'link_title',
							'type' => 'text',
							'title' => __('Link title', 'mfn-opts'),
						),

   					// icon

   					array(
   						'type' => 'header',
   						'title' => __('Icon', 'mfn-opts'),
   					),

   					array(
   						'id' => 'icon',
   						'type' => 'icon',
   						'title' => __('Icon', 'mfn-opts'),
   					),

   					array(
   						'id' => 'icon_position',
   						'type' => 'switch',
   						'title' => __('Position', 'mfn-opts'),
   						'options' => array(
   							'left' => __('Left', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'std' => 'left',
   					),

   					// style

   					array(
   						'type' => 'header',
   						'title' => __('Style', 'mfn-opts'),
   					),

   					array(
   						'id' => 'size',
   						'type' => 'switch',
   						'title' => __('Size', 'mfn-opts'),
   						'options' => array(
   							1 => __('Small', 'mfn-opts'),
   							2 => __('Default', 'mfn-opts'),
   							3 => __('Large', 'mfn-opts'),
   							4 => __('XL', 'mfn-opts'),
   						),
   						'std' => 2,
   					),

   					array(
   						'id' => 'full_width',
   						'type' => 'switch',
   						'title' => __('Width', 'mfn-opts'),
   						'options' => array(
   							0 => __('Default', 'mfn-opts'),
   							1 => __('Full width', 'mfn-opts'),
   						),
							'std' => 0,
   					),

   					array(
   						'id' => 'button_style',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Style', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'button_theme' => __('Highlighted', 'mfn-opts'),
   						),
   						'std' => '',
   					),

   					// advanced

   					array(
   						'type' => 'header',
   						'title' => __('Advanced', 'mfn-opts'),
   					),

   					array(
   						'id' => 'button_function',
   						'attr_id' => 'button_function',
   						'type' => 'select',
   						'title' => __('Functional button', 'mfn-opts'),
   						'options' => array(
   							'' => __('None', 'mfn-opts'),
   							'exit-mfn-popup' => __('Close popup', 'mfn-opts'),
   							'open-mfn-popup' => __('Open popup', 'mfn-opts'),
   							'mfn-go-to' => __('Go to Section', 'mfn-opts'),
   						),
   						'std' => '',
   					),

   					array(
		  				'id' => 'button_function_go_to',
		  				'condition' => array( 'id' => 'button_function', 'opt' => 'is', 'val' => 'mfn-go-to' ),
		  				'type' => 'select',
		  				'title' => __('Section to go', 'mfn-opts'),
		  				'options' => array(
		  					'' => __('Next', 'mfn-opts'),
		  					'last' => __('Last', 'mfn-opts'),
		  					'prev' => __('Prev', 'mfn-opts'),
		  				),
		  				'std' => ''
		  			),

   					array(
		  				'id' => 'button_function_popupid',
		  				'condition' => array( 'id' => 'button_function', 'opt' => 'is', 'val' => 'open-mfn-popup' ),
		  				'type' => 'text',
		  				'title' => __('Popup ID', 'mfn-opts'),
		  			),

						array(
							'id' => 'class',
							'type' => 'text',
							'title' => __('Class', 'mfn-opts'),
							'desc' => __('This option is useful when you want to use <b>scroll</b>', 'mfn-opts'),
						),

						array(
							'id' => 'button_id',
							'type' => 'text',
							'title' => __('ID', 'mfn-opts'),
							'desc' => __('This option is useful when you want to use <b>GTM</b>', 'mfn-opts'),
						),

						array(
   						'id' => 'download',
   						'type' => 'text',
   						'title' => __('Download', 'mfn-opts'),
   						'desc' => __('Enter the new filename for the downloaded file', 'mfn-opts'),
   					),

   					array(
   						'id' => 'rel',
   						'type' => 'text',
   						'title' => __('Rel', 'mfn-opts'),
   						'before' => 'rel=',
   					),

   					array(
   						'id' => 'onclick',
   						'type' => 'text',
   						'title' => __('Onclick', 'mfn-opts'),
							'before' => 'onclick=',
   					),

						// deprecated

   					array(
   						'type' => 'header',
   						'class' => 'mfn-deprecated',
   						'title' => __('Deprecated', 'mfn-opts'),
   					),

   					array(
   						'id' => 'align',
   						'class' => 'mfn-deprecated',
   						'type' => 'switch',
   						'title' => __('Alignment', 'mfn-opts'),
   						'options' => array(
   							'' => __('Left', 'mfn-opts'),
   							'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'std' => '',
							'preview' => 'align',
   					),

   					array(
   						'id' => 'color',
   						'class' => 'mfn-deprecated',
   						'type' => 'color',
   						'title' => __('Background color', 'mfn-opts'),
   						'desc' => __('For theme color button please enter <b>theme</b> in color filed', 'mfn-opts'),
   					),

   					array(
   						'id' => 'font_color',
   						'class' => 'mfn-deprecated',
   						'type' => 'color',
   						'title' => __('Text color', 'mfn-opts'),
   					),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

   					array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Container', 'mfn-opts'),
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Button', 'mfn-opts'),
   					),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .button:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .button:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .button:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .button:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .button:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .button:border-style',
							'attr_id' => 'border_style_button',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .button:border-width',
		  				'condition' => array( 'id' => 'border_style_button', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .button:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_button', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .button:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_button', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .button:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_button', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .button:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .button:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .button:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .button:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .button:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .button:border-color',
							'condition' => array( 'id' => 'border_style_button', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher',
							'attr_id' => 'background_switcher_button',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .button:background-color',
							'condition' => array( 'id' => 'background_switcher_button', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .button:gradient',
		  				'condition' => array( 'id' => 'background_switcher_button', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),


						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .button:transition',
							'type' => 'sliderbar',
							'title' => __('Transition duration', 'mfn-opts'),
							'param' => array(
								'min' => '0',
								'max' => '3',
								'step' => '0.1',
								'unit' => 's',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .button|hover:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .button|hover:border-color',
							'condition' => array( 'id' => 'border_style_button', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher_hover',
							'attr_id' => 'background_switcher_hover_button',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .button|hover:background',
							'condition' => array( 'id' => 'background_switcher_hover_button', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .button|hover:gradient',
		  				'condition' => array( 'id' => 'background_switcher_hover_button', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),












						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Call to Action -------------------------------------------------

   			'call_to_action' => array(
   				'type' => 'call_to_action',
   				'title' => __('Call to Action', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'elements',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'title',
   						'edit_tag' => '.call_left | .title',
   						'edit_position' => 'prepend',
   						'type' => 'text',
						'dynamic_data' => 'title',
   						'title' => __('Title', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h3',
						),

   					array(
   						'id' => 'icon',
   						'type' => 'icon',
   						'title' => __('Icon', 'mfn-opts'),
   						'std' => 'icon-check',
   						'preview' => 'icon',
   					),

   					array(
   						'id' => 'content',
   						'type' => 'textarea',
   						'edit_tag' => '.call_right | div.desc',
   						'edit_position' => 'prepend',
   						'dynamic_data' => 'content',
   						'title' => __('Content', 'mfn-opts'),
							'std' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut elit tellus, luctus nec ullamcorper mattis, pulvinar dapibus leo.',
   						'preview' => 'content',
							'class' => 'form-content-full-width',
   					),

   					// link

   					array(
   						'type' => 'header',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link_type',
   						'attr_id' => 'link_type',
   						'type' => 'select',
							'title' => __('On click action', 'mfn-opts'),
   						'options' => array(
								'' => __('Default', 'mfn-opts'),
								'1' => __('Open popup', 'mfn-opts'),
							),
							'std' => '',
   					),

   					array(
   						'id' => 'popup_id',
   						'condition' => array( 'id' => 'link_type', 'opt' => 'is', 'val' => '1' ),
   						'type' => 'text',
   						'title' => __('Popup ID', 'mfn-opts'),
   					),

						array(
   						'id' => 'button_title',
   						'type' => 'text',
   						'dynamic_data' => 'title',
   						'title' => __('Button title', 'mfn-opts'),
   						'desc' => __('Leave this field blank if you want Call to action with big icon', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link',
   						'type' => 'text',
   						'dynamic_data' => 'permalink',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'target',
   						'type' => 'select',
   						'title' => __('Target', 'mfn-opts'),
   						'options' => array(
   							0 => __('Default | _self', 'mfn-opts'),
   							1 => __('New tab or window | _blank', 'mfn-opts'),
   							'lightbox' => __('Lightbox (image or embed video)', 'mfn-opts'),
   						),
   					),

            array(
							'id' => 'link_title',
							'dynamic_data' => 'title',
							'type' => 'text',
							'title' => __('Link title', 'mfn-opts'),
						),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

		  			array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_left .title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_left .title:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_left .title:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_left .title:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_left .title:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_left .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Desc', 'mfn-opts'),
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_right .desc:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_right .desc:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_right .desc:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_right .desc:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),
   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_right .desc:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_right .desc:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Icon', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .icon_wrapper i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .icon_wrapper:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .icon_wrapper i:font-size',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .icon_wrapper i:font-size_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .icon_wrapper i:font-size_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .icon_wrapper i:font-size_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .icon_wrapper:border-style',
							'attr_id' => 'border_style_ctaiw',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .icon_wrapper:border-color',
							'condition' => array( 'id' => 'border_style_ctaiw', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .icon_wrapper:border-width',
		  				'condition' => array( 'id' => 'border_style_ctaiw', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .icon_wrapper:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_ctaiw', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .icon_wrapper:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_ctaiw', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .icon_wrapper:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_ctaiw', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .icon_wrapper:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .icon_wrapper:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .icon_wrapper:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .icon_wrapper:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Button', 'mfn-opts'),
   					),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .button:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .button:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .button:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .button:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .button:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .button:border-style',
							'attr_id' => 'border_style_ctab',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .button:border-width',
		  				'condition' => array( 'id' => 'border_style_ctab', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .button:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_ctab', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .button:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_ctab', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .button:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_ctab', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .button:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .button:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .button:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .button:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .button:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .button:border-color',
							'condition' => array( 'id' => 'border_style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher',
							'attr_id' => 'background_switcher',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .button:background-color',
							'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .button:gradient',
		  				'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .button|hover:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .button|hover:border-color',
							'condition' => array( 'id' => 'border_style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher_hover',
							'attr_id' => 'background_switcher_hover',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .button|hover:background',
							'condition' => array( 'id' => 'background_switcher_hover', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .call_to_action .call_center .button|hover:gradient',
		  				'condition' => array( 'id' => 'background_switcher_hover', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),




















				// cf7 contact form 7  ---------------------------------------------------------

   			'cf7' => array(
   				'type' => 'cf7',
   				'title' => __('Contact Form 7', 'mfn-opts'),
   				'size' => '1/1',
   				'laptop_size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'boxes',
   				'attr' => array(

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'form',
   						'type' => 'select',
   						're_render' => true,
   						'title' => __('Contact form', 'mfn-opts'),
   						'php_options' => mfna_cf7(),
   						'js_options' => 'cf7',
   					),



						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),


						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Labels', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form label:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form p,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form label:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form p,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form label:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form p,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form label:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form p,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form label:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form label:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form label:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',

   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form label:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',

   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form label:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',

   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
   					),
   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form label:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',

   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
   					),

   					array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Inputs', 'mfn-opts'),
   					),


   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|not(.wpcf7-submit),.mcb-section .mcb-wrap .mcb-item-mfnuidelement form select, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|not(.wpcf7-submit),.mcb-section .mcb-wrap .mcb-item-mfnuidelement form select, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',

   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|not(.wpcf7-submit),.mcb-section .mcb-wrap .mcb-item-mfnuidelement form select, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',

   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|not(.wpcf7-submit),.mcb-section .mcb-wrap .mcb-item-mfnuidelement form select, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',

   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
   					),
   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|not(.wpcf7-submit),.mcb-section .mcb-wrap .mcb-item-mfnuidelement form select, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',

   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
   					),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|not(.wpcf7-submit),.mcb-section .mcb-wrap .mcb-item-mfnuidelement form select, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|not(.wpcf7-submit),.mcb-section .mcb-wrap .mcb-item-mfnuidelement form select, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|not(.wpcf7-submit),.mcb-section .mcb-wrap .mcb-item-mfnuidelement form select, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|not(.wpcf7-submit),.mcb-section .mcb-wrap .mcb-item-mfnuidelement form select, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),


						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|not(.wpcf7-submit),.mcb-section .mcb-wrap .mcb-item-mfnuidelement form select, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|not(.wpcf7-submit),.mcb-section .mcb-wrap .mcb-item-mfnuidelement form select, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|not(.wpcf7-submit),.mcb-section .mcb-wrap .mcb-item-mfnuidelement form select, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|not(.wpcf7-submit),.mcb-section .mcb-wrap .mcb-item-mfnuidelement form select, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|not(.wpcf7-submit),.mcb-section .mcb-wrap .mcb-item-mfnuidelement form select, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:border-style',
							'attr_id' => 'border_style_cf7inputs',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|not(.wpcf7-submit),.mcb-section .mcb-wrap .mcb-item-mfnuidelement form select, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:border-width',
		  				'condition' => array( 'id' => 'border_style_cf7inputs', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|not(.wpcf7-submit),.mcb-section .mcb-wrap .mcb-item-mfnuidelement form select, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_cf7inputs', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|not(.wpcf7-submit),.mcb-section .mcb-wrap .mcb-item-mfnuidelement form select, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_cf7inputs', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|not(.wpcf7-submit),.mcb-section .mcb-wrap .mcb-item-mfnuidelement form select, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_cf7inputs', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|not(.wpcf7-submit),.mcb-section .mcb-wrap .mcb-item-mfnuidelement form select, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|not(.wpcf7-submit),.mcb-section .mcb-wrap .mcb-item-mfnuidelement form select, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|not(.wpcf7-submit),.mcb-section .mcb-wrap .mcb-item-mfnuidelement form select, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|not(.wpcf7-submit),.mcb-section .mcb-wrap .mcb-item-mfnuidelement form select, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Focus</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|not(.wpcf7-submit),.mcb-section .mcb-wrap .mcb-item-mfnuidelement form select, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input||placeholder, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea||placeholder:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Placeholder color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|not(.wpcf7-submit),.mcb-section .mcb-wrap .mcb-item-mfnuidelement form select, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|not(.wpcf7-submit),.mcb-section .mcb-wrap .mcb-item-mfnuidelement form select, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:border-color',
							'condition' => array( 'id' => 'border_style_cf7inputs', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|not(.wpcf7-submit)|focus,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form select|focus, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea|focus:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|focus||placeholder, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea|focus||placeholder:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Placeholder color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|not(.wpcf7-submit)|focus,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form select|focus, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea|focus:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input|not(.wpcf7-submit)|focus,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form select|focus, .mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea|focus:border-color',
							'condition' => array( 'id' => 'border_style_cf7inputs', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Textarea', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:height',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Height', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '700',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:height_laptop',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Height', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '700',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:height_tablet',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Height', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '700',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form textarea:height_mobile',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Height', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '700',
								'step' => '1',
							),
						),








						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Fields wrapper', 'mfn-opts'),
   					),



						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form .column:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
		  				'desc' => __('Modifies div.column that is included by default in the Betheme forms.', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form .column:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'desc' => __('Modifies div.column that is included by default in the Betheme forms.', 'mfn-opts'),
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form .column:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'desc' => __('Modifies div.column that is included by default in the Betheme forms.', 'mfn-opts'),
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form .column:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'desc' => __('Modifies div.column that is included by default in the Betheme forms.', 'mfn-opts'),
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),





   					array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Button', 'mfn-opts'),
   					),
















   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.wpcf7-submit,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form button:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.wpcf7-submit,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form button:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.wpcf7-submit,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form button:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.wpcf7-submit,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form button:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.wpcf7-submit,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form button:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),


						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.wpcf7-submit,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form button:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.wpcf7-submit,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form button:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.wpcf7-submit,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form button:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.wpcf7-submit,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form button:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.wpcf7-submit,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form button:border-style',
							'attr_id' => 'border_style_cf7button',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.wpcf7-submit,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form button:border-width',
		  				'condition' => array( 'id' => 'border_style_cf7button', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.wpcf7-submit,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form button:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_cf7button', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.wpcf7-submit,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form button:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_cf7button', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.wpcf7-submit,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form button:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_cf7button', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.wpcf7-submit,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form button:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.wpcf7-submit,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form button:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.wpcf7-submit,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form button:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.wpcf7-submit,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form button:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.wpcf7-submit,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form button:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.wpcf7-submit,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form button:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.wpcf7-submit,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form button:border-color',
							'condition' => array( 'id' => 'border_style_cf7button', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.wpcf7-submit|hover,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form button|hover:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.wpcf7-submit|hover,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form button|hover:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement form input.wpcf7-submit|hover,.mcb-section .mcb-wrap .mcb-item-mfnuidelement form button|hover:border-color',
							'condition' => array( 'id' => 'border_style_cf7button', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),






















						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),














   			// Chart  ---------------------------------------------------------

   			'chart' => array(
   				'type' => 'chart',
   				'title' => __('Chart', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'boxes',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'title',
   						'type' => 'text',
   						'dynamic_data' => 'title',
   						'title' => __('Title', 'mfn-opts'),
   						'edit_tag' => '.chart_box | big',
   						'edit_position' => 'append',
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'p',
						),

   					// chart

   					array(
   						'type' => 'header',
   						'title' => __('Chart', 'mfn-opts'),
   					),

   					array(
   						'id' => 'percent',
   						'type' => 'text',
   						're_render' => true,
   						'title' => __('Percent', 'mfn-opts'),
   						'desc' => __('0-100', 'mfn-opts'),
							'param' => 'number',
							'after' => '%',
							'class' => 'narrow',
							'preview' => 'number',
							'std' => 50,
   					),

   					array(
   						'id' => 'label',
   						'type' => 'text',
   						'title' => __('Label', 'mfn-opts'),
							'preview' => 'subtitle',
   					),

   					array(
   						'id' => 'icon',
   						'type' => 'icon',
   						'title' => __('Icon', 'mfn-opts'),
							'std' => 'icon-check',
   					),

   					array(
   						'id' => 'image',
   						'type' => 'upload',
   						'title' => __('Image', 'mfn-opts'),
   						'desc' => __('Recommended image size <b>70px x 70px</b>', 'mfn-opts'),
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

   					array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .chart_box .chart-desc:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .chart_box .chart-desc:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Label', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .chart_box .num:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .chart_box .num:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Style Icon', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .chart_box .icon i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Icon', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .chart_box .icon i:font-size',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .chart_box .icon i:font-size_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .chart_box .icon i:font-size_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .chart_box .icon i:font-size_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Chart', 'mfn-opts'),
   					),

   					array(
   						'id' => 'color',
   						'type' => 'color',
   						're_render' => true,
   						'class' => 'mfn-toggled',
   						'title' => __('Color', 'mfn-opts'),
   						'desc' => __('Overrides color set in Theme Options', 'mfn-opts'),
   					),

   					array(
   						'id' => 'line_width',
   						'type' => 'text',
   						'title' => __('Line thickness', 'mfn-opts'),
							'param' => 'number',
							'after' => 'px',
							'class' => 'narrow mfn-toggled',
							'placeholder' => 4,
   					),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Clients  -------------------------------------------------------

   			'clients' => array(
   				'type' => 'clients',
   				'title' => __('Clients', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'loops',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'in_row',
   						'type' => 'text',
   						're_render' => true,
   						'title' => __('Items in row', 'mfn-opts'),
   						'desc' => __('Recommended: 3-6', 'mfn-opts'),
							'after' => 'items',
							'param' => 'number',
							'class' => 'narrow',
   						'std' => 6,
   						'preview' => 'number',
   					),

						array(
   						'id' => 'style',
   						'type' => 'switch',
   						'attr_id' => 'clients_style',
   						're_render' => true,
   						'title' => __('Style', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'tiles' => __('Tiles', 'mfn-opts'),
   						),
							'std' => '',
							'preview' => 'style',
   					),

						// options

   					array(
   						'type' => 'header',
   						'title' => __('Options', 'mfn-opts'),
   					),

   					array(
   						'id' => 'category',
   						're_render' => true,
   						'type' => 'select',
   						'title' => __('Category', 'mfn-opts'),
   						'php_options' => mfn_get_categories('client-types'),
   						'js_hierarchical_options' => 'client_types',
							'preview' => 'category',
   					),

   					array(
   						'id' => 'orderby',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Order by', 'mfn-opts'),
   						'options' => array(
   							'date' => __('Date', 'mfn-opts'),
   							'menu_order' => __('Menu order', 'mfn-opts'),
   							'title' => __('Title', 'mfn-opts'),
   							'rand' => __('Random', 'mfn-opts'),
   						),
   						'std' => 'menu_order'
   					),

   					array(
   						'id' => 'order',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Order', 'mfn-opts'),
   						'options' => array(
								'DESC' => __('Descending', 'mfn-opts'),
   							'ASC' => __('Ascending', 'mfn-opts'),
   						),
   						'std' => 'ASC',
   					),

						// advanced

   					array(
   						'type' => 'header',
   						'title' => __('Advanced', 'mfn-opts'),
   					),

   					array(
   						'id' => 'greyscale',
   						'type' => 'switch',
   						'title' => 'Grayscale',
   						'options' => array(
   							0 => __('Disable', 'mfn-opts'),
   							1 => __('Enable', 'mfn-opts'),
   						),
							'std' => 0,
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Container', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.clients li .client_wrapper:height',
							'type' => 'text',
							'default_unit' => 'px',
							'class' => 'mfn-toggled',
							'title' => __('Height', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.clients li .client_wrapper:padding',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.clients li .client_wrapper:padding_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.clients li .client_wrapper:padding_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.clients li .client_wrapper:padding_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.clients li .client_wrapper:background-color',
							'type' => 'color',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.clients li .client_wrapper|hover:background-color',
							'type' => 'color',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Style tiles', 'mfn-opts'),
   					),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#tilenormal" data-tab="tilenormal">Normal</a></li><li><a href="#tilehover" data-tab="tilehover">Hover</a></li></ul><div class="mfn-sft mfn-sft-tilenormal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.clients.clients_tiles:--mfn-clients-tiles',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-tilehover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement ul.clients.clients_tiles:--mfn-clients-tiles-hover',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Clients Slider -------------------------------------------------

   			'clients_slider' => array(
   				'type' => 'clients_slider',
   				'title' => __('Clients Slider', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'loops',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'title',
   						'type' => 'text',
   						'dynamic_data' => 'title',
   						'edit_tag' => '.blog_slider_header | .title',
   						'edit_position' => 'append',
   						'title' => __('Title', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h4',
						),

						// options

   					array(
   						'type' => 'header',
   						'title' => __('Options', 'mfn-opts'),
   					),

   					array(
   						'id' => 'category',
   						'type' => 'select',
   						're_render' => true,
   						'title' => __('Category', 'mfn-opts'),
   						'php_options' => mfn_get_categories('client-types'),
   						'js_hierarchical_options' => 'client_types',
							'preview' => 'category',
   					),

   					array(
   						'id' => 'orderby',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Order by', 'mfn-opts'),
   						'options' => array(
   							'date' => __('Date', 'mfn-opts'),
   							'menu_order' => __('Menu order', 'mfn-opts'),
   							'title' => __('Title', 'mfn-opts'),
   							'rand' => __('Random', 'mfn-opts'),
   						),
   						'std' => 'menu_order'
   					),

   					array(
   						'id' => 'order',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Order', 'mfn-opts'),
   						'options' => array(
								'DESC' => __('Descending', 'mfn-opts'),
   							'ASC' => __('Ascending', 'mfn-opts'),
   						),
   						'std' => 'ASC'
   					),

						array(
							'id' => 'per_slide',
							'type' => 'sliderbar',
							're_render' => true,
							'title' => __('Clients per slide', 'mfn-opts'),
							'desc' => __('The amount will be automatically reduced in the narrow screen', 'mfn-opts'),
							'param' => array(
								'min' => '1',
								'max' => '6',
								'step' => '1',
							),
							'std' => '3',
						),

						array(
							'id' => 'scroll',
							'type' => 'switch',
							're_render' => true,
							'title' => __('Slides to scroll', 'mfn-opts'),
							'options' => array(
								'' => __('One slide', 'mfn-opts'),
								'1' => __('One client', 'mfn-opts'),
							),
							'std' => '',
						),

						array(
   						'id' => 'navigation',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Navigation position', 'mfn-opts'),
   						'options' => array(
								'' => __('Header', 'mfn-opts'),
   							'content' => __('Content', 'mfn-opts'),
   						),
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						// custom

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Item title', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_header .title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_header .title:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_header .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_header .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_header .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_header .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Container', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider_ul li .client_wrapper:height',
							'type' => 'text',
							'default_unit' => 'px',
							'class' => 'mfn-toggled',
							'title' => __('Height', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider_ul li .client_wrapper:padding',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider_ul li .client_wrapper:padding_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider_ul li .client_wrapper:padding_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider_ul li .client_wrapper:padding_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider_ul li .client_wrapper:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider_ul li .client_wrapper|hover:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Button', 'mfn-opts'),
   					),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider .button:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider .button:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider .button:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider .button:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider .button:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider .button:border-style',
							'attr_id' => 'border_style_csb',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider .button:border-width',
		  				'condition' => array( 'id' => 'border_style_csb', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider .button:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_csb', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider .button:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_csb', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider .button:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_csb', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider .button:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider .button:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider .button:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider .button:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider .button .button_icon i:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider .button:border-color',
							'condition' => array( 'id' => 'border_style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher',
							'attr_id' => 'background_switcher',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider .button:background-color',
							'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider .button:gradient',
		  				'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider .button|hover .button_icon i:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider .button|hover:border-color',
							'condition' => array( 'id' => 'border_style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher_hover',
							'attr_id' => 'background_switcher_hover',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider .button|hover:background',
							'condition' => array( 'id' => 'background_switcher_hover', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .clients_slider .button|hover:gradient',
		  				'condition' => array( 'id' => 'background_switcher_hover', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Code  ----------------------------------------------------------

   			'code' => array(
   				'type' => 'code',
   				'title' => __('Code', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'other',
   				'attr' => array(
   				//'fields' => array(

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'content',
   						'type' => 'textarea',
   						'edit_tag' => '.mcb-item-code-inner | pre.mfn-code',
   						'edit_position' => 'append',
   						'title' => __('Content', 'mfn-opts'),
   						'std' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit.',
							'class' => 'form-content-full-width',
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Column  --------------------------------------------------------

   			'column' => array(
   				'type' => 'column',
   				'title' => __('Column Text', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'used_fonts' => '',
   				'cat' => 'typography',
   				'attr' => array(
   				//'fields' => array(

						// HTML content

						/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'content',
   						'type' => $this->get_column_editor(), // textarea, visual
   						'title' => __('Content', 'mfn-opts'),
   						'class' => 'form-content-full-width',
   						'dynamic_data' => 'content',
							'editor' => 'full', // basic (bold, i, etc), full (media, shortcodes)
							'preview' => 'content',
							'vbstd' => '<p>Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.</p>',
   					),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'id' => 'title',
							'type' => 'text',
							'title' => __('Label', 'mfn-opts'),
							'desc' => __('This field is only used as label in the builder', 'mfn-opts'),
							'preview' => 'title',
							'class' => 'mfn-hidden-form-row',
						),

						array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .column_attr:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',

   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .column_attr:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',

   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .column_attr:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',

   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),
   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .column_attr:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',

   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .column_attr:color',
							'type' => 'color',
							'desc' => __('May be overwritten by individual tags or inline CSS', 'mfn-opts'),
							'title' => __('Color', 'mfn-opts'),
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .column_attr a:color',
							'type' => 'color',
							'title' => __('Link color', 'mfn-opts'),
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .column_attr a|hover:color',
							'type' => 'color',
							'title' => __('Link hover', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .column_attr:text-shadow',
							'type' => 'text_shadow',
							'title' => __('Text shadow', 'mfn-opts'),
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .column_attr:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
   						'title' => __('Deprecated', 'mfn-opts'),
   					),

   					array(
   						'id' => 'align',
   						'class' => 'mfn-deprecated',
   						'type' => 'switch',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'align-mobile',
   						'class' => 'mfn-deprecated',
   						'type' => 'switch',
   						'title' => __('Mobile text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('As above', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
   					),

   					array(
   						'id' => 'column_bg',
   						'class' => 'mfn-deprecated',
   						'type' => 'color',
   						'title' => __('Background color', 'mfn-opts'),
   						'alpha' => true,
   					),

   					array(
   						'id' => 'bg_image',
   						'class' => 'mfn-deprecated',
   						'type' => 'upload',
   						'title' => __('Background image', 'mfn-opts'),
   					),

   					array(
   						'id' => 'bg_position',
   						'class' => 'mfn-deprecated',
   						'type' => 'select',
   						'title' => __('Background position', 'mfn-opts'),
   						'options' => mfna_bg_position('column'),
   						'std' => 'center top no-repeat',
   					),

   					array(
   						'id' => 'bg_size',
   						'class' => 'mfn-deprecated',
   						'type' => 'select',
   						'title' => __('Background size', 'mfn-opts'),
   						'options' => mfna_bg_size(),
   					),

   					array(
   						'id' => 'margin_bottom',
   						'class' => 'mfn-deprecated',
   						'type' => 'select',
   						'title' => __('Margin bottom', 'mfn-opts'),
   						'options' => array(
   							'' => __('- Default -', 'mfn-opts'),
   							'0px' => '0px',
   							'10px' => '10px',
   							'20px' => '20px',
   							'30px' => '30px',
   							'40px' => '40px',
   							'50px' => '50px',
   						),
   					),

   					array(
   						'id' => 'padding',
   						'class' => 'mfn-deprecated',
   						'type' => 'text',
   						'title' => __('Padding', 'mfn-opts'),
   						'desc' => __('Use value with <b>px</b> or <b>%</b>. Example: <b>20px</b> or <b>20px 10px 20px 10px</b> or <b>20px 1%</b>', 'mfn-opts'),
   					),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'id' => 'style',
							'type' => 'textarea',
							'title' => __('Column inline CSS', 'mfn-opts'),
							'desc' => __('Example: <b>border: 1px solid #999;</b>', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'helper',
							'class' => 'mfn-deprecated',
							'title' => __('Need help', 'mfn-opts'),
							'link' => 'https://support.muffingroup.com/faq/what-deprecated-in-the-bebuilder-means/',
						),

						// HTML end: settings

						array(
   						'type' => 'html',
   						'html' => '</div>',
   					),

   				),
   			),

   			// Contact box ----------------------------------------------------

   			'contact_box' => array(
   				'type' => 'contact_box',
   				'title' => __('Contact Box', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'elements',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'title',
						'dynamic_data' => 'title',
   						'type' => 'text',
   						'edit_tag' => '.get_in_touch | .title',
   						'edit_position' => 'prepend',
   						'title' => __('Title', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h3',
						),

   					array(
   						'id' => 'address',
   						'type' => 'textarea',
   						'dynamic_data' => 'content',
   						'title' => __('Address', 'mfn-opts'),
   						'desc' => __('HTML tags allowed', 'mfn-opts'),
							'std' => 'This is the address',
							'preview' => 'content',
							'class' => 'form-content-full-width',
   					),

   					array(
   						'id' => 'telephone',
   						'type' => 'text',
   						'title' => __('Phone', 'mfn-opts'),
   					),

   					array(
   						'id' => 'telephone_2',
   						'type' => 'text',
   						'title' => __('Second phone', 'mfn-opts'),
   					),

   					array(
   						'id' => 'fax',
   						'type' => 'text',
   						'title' => __('Fax', 'mfn-opts'),
   					),

   					array(
   						'id' => 'email',
   						'type' => 'text',
   						'title' => __('Email', 'mfn-opts'),
   					),

   					array(
   						'id' => 'www',
   						'type' => 'text',
   						'title' => __('WWW', 'mfn-opts'),
   					),

						// advanced

   					array(
   						'class' => 'mfn-deprecated',
   						'type' => 'header',
   						'title' => __('Deprecated', 'mfn-opts'),
   					),

						array(
   						'id' => 'image',
   						'class' => 'mfn-deprecated',
   						'type' => 'upload',
   						'title' => __('Background image', 'mfn-opts'),
   						'desc' => __('Recommended image width <b>768px - 1920px</b> depending on size of the item', 'mfn-opts'),
   					),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Item title', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .get_in_touch .title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .get_in_touch .title:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .get_in_touch .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .get_in_touch .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .get_in_touch .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .get_in_touch .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Icon', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .get_in_touch ul li .icon:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('List', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .get_in_touch .get_in_touch_wrapper ul li:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .get_in_touch ul li a:color',
							'type' => 'color',
							'title' => __('Links color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .get_in_touch ul li a|hover:color',
							'type' => 'color',
							'title' => __('Links color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Lines', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .get_in_touch ul li:--mfn-contactbox-line',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Line color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Content  -------------------------------------------------------

   			'content' => array(
   				'type' => 'content',
   				'title' => __('Content WP', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'typography',
   				'attr' => array(
   				//'fields' => array(

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
							'id' => 'info',
							'class' => 'mfn-alert',
   						'type' => 'info',
   						'title' => __('Content from WordPress editor. Can be used once per page.', 'mfn-opts'),
   					),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Countdown  -----------------------------------------------------

   			'countdown' => array(
   				'type' => 'countdown',
   				'title' => __('Countdown', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'boxes',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'date',
   						'type' => 'text',
   						're_render' => true,
   						'title' => __('Launch date', 'mfn-opts'),
   						'desc' => __('month/day/year hour:minute:second', 'mfn-opts'),
   						'std' => '12/30/2023 12:00:00',
							'preview' => 'title',
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Label tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h3',
						),

   					array(
   						'id' => 'timezone',
   						'type' => 'select',
   						're_render' => true,
   						'title' => __('Timezone', 'mfn-opts'),
   						'options' => mfna_utc(),
   						'std' => '0',
   					),

   					// options

   					array(
   						'type' => 'header',
   						'title' => __('Options', 'mfn-opts'),
   					),

   					array(
   						'id' => 'show',
   						're_render' => true,
   						'type' => 'select',
   						'title' => __('Show', 'mfn-opts'),
   						'options' => array(
   							'' => __('days hours minutes seconds', 'mfn-opts'),
   							'dhm' => __('days hours minutes', 'mfn-opts'),
   							'dh' => __('days hours', 'mfn-opts'),
   							'd' => __('days', 'mfn-opts'),
   						),
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

   					array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Container', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact:border-style',
							'attr_id' => 'border_style_dcqf',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact:border-color',
							'condition' => array( 'id' => 'border_style_dcqf', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact:border-width',
		  				'condition' => array( 'id' => 'border_style_dcqf', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_dcqf', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_dcqf', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_dcqf', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Number', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact .number-wrapper .number:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact .number-wrapper:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact .number-wrapper:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact .number-wrapper:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact .number-wrapper:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact .number-wrapper:typography',
							'type' => 'typography_vb',
							'typography' => 'simple',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

   					array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Label', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact .title:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact .title:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact .title:typography',
							'type' => 'typography_vb',
							'typography' => 'simple',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

   					array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'themeoptions' => 'style:',
   						'title' => __('Divider', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact hr:width',
							'type' => 'text',
							'themeoptions' => 'style:',
							'default_unit' => '%',
							'class' => 'mfn-toggled',
							'title' => __('Width', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .downcount .column .quick_fact hr:background-color',
							'themeoptions' => 'style:',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Counter  -------------------------------------------------------

   			'counter' => array(
   				'type' => 'counter',
   				'title' => __('Counter', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'boxes',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'title',
   						'type' => 'text',
   						'dynamic_data' => 'title',
   						'edit_tag' => '.desc_wrapper | .title',
   						'edit_position' => 'append',
   						'title' => __('Title', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'p',
						),

   					// counter

   					array(
   						'type' => 'header',
   						'title' => __('Counter', 'mfn-opts'),
   					),

   					array(
   						'id' => 'icon',
   						'type' => 'icon',
   						'title' => __('Icon', 'mfn-opts'),
   						'std' => 'icon-lamp',
   						'preview' => 'icon',
   					),

   					array(
   						'id' => 'image',
   						'dynamic_data' => 'featured_image',
   						'type' => 'upload',
   						'title' => __('Image', 'mfn-opts'),
   						'desc' => __('Replaces icon if uploaded', 'mfn-opts'),
							'preview' => 'image',
   					),

   					array(
   						'id' => 'prefix',
   						'type' => 'text',
   						'title' => __('Prefix', 'mfn-opts'),
							'class' => 'narrow',
   					),

   					array(
   						'id' => 'number',
   						'type' => 'text',
   						'title' => __('Number', 'mfn-opts'),
							'param' => 'number',
							'class' => 'narrow',
							'std' => '50',
							'preview' => 'number',
   					),

   					array(
   						'id' => 'label',
   						'type' => 'text',
   						'title' => __('Postfix', 'mfn-opts'),
							'class' => 'narrow',
   					),

            array(
   						'id' => 'duration',
   						'type' => 'sliderbar',
   						'title' => __('Duration [ms]', 'mfn-opts'),
   						'desc' => __('1000 = 1 second', 'mfn-opts'),
							'param' => 'number',
							'class' => 'narrow',
							'std' => '1000',
							'param' => array(
								'min' => '0',
								'max' => '9500',
								'step' => '500',
								'unit' => 'ms',
							),
   					),

   					// options

   					array(
   						'type' => 'header',
   						'title' => __('Options', 'mfn-opts'),
   					),

   					array(
   						'id' => 'type',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Style', 'mfn-opts'),
   						'desc' => __('Vertical style works for column widths: 1/4, 1/3 & 1/2', 'mfn-opts'),
   						'options' => array(
   							'vertical' => __('Vertical', 'mfn-opts'),
								'horizontal' => __('Horizontal', 'mfn-opts'),
   						),
   						'std' => 'vertical',
   					),

						// deprecated

   					array(
   						'type' => 'header',
   						'class' => 'mfn-deprecated',
   						'title' => __('Deprecated', 'mfn-opts'),
   					),

   					array(
   						'id' => 'color',
   						'class' => 'mfn-deprecated',
   						'type' => 'color',
   						'title' => __('Icon color', 'mfn-opts'),
   					),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

		  			array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Container', 'mfn-opts'),
   					),

						array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter:padding',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter:padding_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter:padding_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter:padding_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Icon', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .icon_wrapper i:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .icon_wrapper:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .icon_wrapper:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .icon_wrapper:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .icon_wrapper:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Image', 'mfn-opts'),
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .icon_wrapper img:border-style',
							'attr_id' => 'border_style_ciwi',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .icon_wrapper img:border-color',
							'condition' => array( 'id' => 'border_style_ciwi', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .icon_wrapper img:border-width',
		  				'condition' => array( 'id' => 'border_style_ciwi', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .icon_wrapper img:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_ciwi', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .icon_wrapper img:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_ciwi', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .icon_wrapper img:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_ciwi', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .icon_wrapper img:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .icon_wrapper img:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .icon_wrapper img:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .icon_wrapper img:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .icon_wrapper:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .icon_wrapper:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .icon_wrapper:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .icon_wrapper:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .icon_wrapper img:width',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'title' => __('Width', 'mfn-opts'),
							'desc' => __('Use px, %, vw, vh or auto to set content width', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .icon_wrapper img:width_laptop',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'title' => __('Width', 'mfn-opts'),
							'desc' => __('Use px, %, vw, vh or auto to set content width', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .icon_wrapper img:width_tablet',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'title' => __('Width', 'mfn-opts'),
							'desc' => __('Use px, %, vw, vh or auto to set content width', 'mfn-opts'),
						),
						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .icon_wrapper img:width_mobile',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'title' => __('Width', 'mfn-opts'),
							'desc' => __('Use px, %, vw, vh or auto to set content width', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .icon_wrapper img:height',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'title' => __('Height', 'mfn-opts'),
							'desc' => __('Use px, %, vw, vh or auto to set content width', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .icon_wrapper img:height_laptop',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'title' => __('Height', 'mfn-opts'),
							'desc' => __('Use px, %, vw, vh or auto to set content width', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .icon_wrapper img:height_tablet',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'title' => __('Height', 'mfn-opts'),
							'desc' => __('Use px, %, vw, vh or auto to set content width', 'mfn-opts'),
						),
						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .icon_wrapper img:height_mobile',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'title' => __('Height', 'mfn-opts'),
							'desc' => __('Use px, %, vw, vh or auto to set content width', 'mfn-opts'),
						),

						array(
		  				'type' => 'header',
		  				'class' => 'toggled_header row-header',
   						'title' => __('Pre', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .number-wrapper .prefix:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .number-wrapper .prefix:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .number-wrapper .prefix:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .number-wrapper .prefix:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .number-wrapper .prefix:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .number-wrapper .prefix:typography',
							'type' => 'typography_vb',
							'typography' => 'simple',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'type' => 'header',
		  				'class' => 'toggled_header row-header',
   						'title' => __('Number', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .number-wrapper .number:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .number-wrapper:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .number-wrapper:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .number-wrapper:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .number-wrapper:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .number-wrapper .number:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),


						array(
		  				'type' => 'header',
		  				'class' => 'toggled_header row-header',
   						'title' => __('Post', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .number-wrapper .postfix:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .number-wrapper .postfix:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .number-wrapper .postfix:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .number-wrapper .postfix:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .number-wrapper .postfix:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .number-wrapper .postfix:typography',
							'type' => 'typography_vb',
							'typography' => 'simple',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'type' => 'header',
		  				'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .title:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .counter .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Fancy Divider  -------------------------------------------------

   			'fancy_divider' => array(
   				'type' => 'fancy_divider',
   				'title' => __('Fancy Divider', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'elements',
   				'attr' => array(
   				//'fields' => array(

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
							'id' => 'info',
							'class' => 'mfn-alert',
							'type' => 'info',
   						'title' => __('Please use on pages without sidebar.', 'mfn-opts'),
   					),

   					array(
   						'id' => 'style',
   						'type' => 'select',
   						're_render' => true,
   						'title' => __('Style', 'mfn-opts'),
   						'options' => array(
   							'circle up' => __('Circle up', 'mfn-opts'),
   							'circle down' => __('Circle down', 'mfn-opts'),
   							'curve up' => __('Curve up', 'mfn-opts'),
   							'curve down' => __('Curve down', 'mfn-opts'),
   							'stamp' => __('Stamp', 'mfn-opts'),
   							'triangle up' => __('Triangle up', 'mfn-opts'),
   							'triangle down' => __('Triangle down', 'mfn-opts'),
   						),
							'std' => 'circle up',
							'preview' => 'style',
   					),

   					array(
   						'id' => 'color_top',
   						'type' => 'color',
   						'title' => __('Color top', 'mfn-opts'),
   					),

   					array(
   						'id' => 'color_bottom',
   						'type' => 'color',
   						'title' => __('Color bottom', 'mfn-opts'),
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Fancy Heading --------------------------------------------------

   			'fancy_heading' => array(
   				'type' => 'fancy_heading',
   				'title' => __('Fancy Heading', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'elements',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'title',
   						'type' => 'text',
   						'dynamic_data' => 'title',
   						'edit_tag' => '.fancy_heading | .title',
   						'edit_position' => 'append',
   						'title' => __('Title', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
   						'id' => 'h1',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Tag', 'mfn-opts'),
   						'kl_options' => array( // prevent js sort
   							(object) array('key' => 'h1', 'label' => 'H1'),
								(object) array('key' => 0, 'label' => 'H2'),
								(object) array('key' => 'h3', 'label' => 'H3'),
								(object) array('key' => 'h4', 'label' => 'H4'),
								(object) array('key' => 'h5', 'label' => 'H5'),
								(object) array('key' => 'h6', 'label' => 'H6'),
								(object) array('key' => 'p', 'label' => 'p'),
								(object) array('key' => 'span', 'label' => 'span'),
								(object) array('key' => 'div', 'label' => 'div'),
   						),
							'std' => 0
   					),

   					array(
   						'id' => 'content',
   						'type' => 'textarea',
   						'edit_tag' => '.fancy_heading | div.inside',
   						'edit_position' => 'append',
   						'title' => __('Content', 'mfn-opts'),
							'std' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut elit tellus, luctus nec ullamcorper mattis, pulvinar dapibus leo.',
							'preview' => 'content',
							'class' => 'form-content-full-width',
   					),

   					// style

   					array(
   						'type' => 'header',
   						'title' => __('Style', 'mfn-opts'),
   					),

   					array(
   						'id' => 'style',
   						'type' => 'switch',
   						'attr_id' => 'fancy_header_style',
   						're_render' => true,
   						'title' => __('Style', 'mfn-opts'),
   						'options' => array(
   							'icon' => __('Icon', 'mfn-opts'),
   							'line' => __('Line', 'mfn-opts'),
   							'arrows' => __('Arrows', 'mfn-opts'),
   						),
							'std' => 'icon',
   					),

   					array(
   						'id' => 'icon',
   						'condition' => array( 'id' => 'fancy_header_style', 'opt' => 'is', 'val' => 'icon' ),
   						'type' => 'icon',
   						'title' => __('Icon', 'mfn-opts'),
   						'desc' => __('for style: icon', 'mfn-opts'),
   					),

   					array(
   						'id' => 'slogan',
   						'condition' => array( 'id' => 'fancy_header_style', 'opt' => 'is', 'val' => 'line' ),
   						'edit_tag' => '.fh-top | span.slogan',
   						'edit_position' => 'append',
   						'type' => 'text',
   						'title' => __('Slogan', 'mfn-opts'),
   						'desc' => __('for style: line', 'mfn-opts'),
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Container', 'mfn-opts'),
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .fancy_heading:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .fancy_heading:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .fancy_heading:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .fancy_heading:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .fancy_heading .title:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .fancy_heading .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .fancy_heading .title:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .fancy_heading .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .fancy_heading .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .fancy_heading .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .fancy_heading .inside:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .fancy_heading .inside:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .fancy_heading .inside:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .fancy_heading .inside:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .fancy_heading .inside:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .fancy_heading .inside:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Style Icon', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .fancy_heading_icon .icon_top:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .fancy_heading_icon .icon_top:font-size',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .fancy_heading_icon .icon_top:font-size_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .fancy_heading_icon .icon_top:font-size_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .fancy_heading_icon .icon_top:font-size_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .fancy_heading_icon .icon_top:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .fancy_heading_icon .icon_top:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .fancy_heading_icon .icon_top:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .fancy_heading_icon .icon_top:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

   					array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Style Line', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .fancy_heading.fancy_heading_line .slogan:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Slogan Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .fancy_heading.fancy_heading_line .slogan:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Slogan Typography', 'mfn-opts'),
						),

   					array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Style Arrows', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .fancy_heading .title i:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Arrows Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// FAQ  -----------------------------------------------------------

   			'faq' => array(
   				'type' => 'faq',
   				'title' => __('FAQ', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'blocks',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
   						'id' => 'info-key',
   						'class' => 'mfn-alert',
   						'type' => 'info',
   						'title' => __('Try the new Toggle element with unlimited customization options', 'mfn-opts'),
   					),

   					array(
   						'id' => 'title',
   						'type' => 'text',
						'dynamic_data' => 'title',
   						'edit_tag' => '.faq | .heading',
   						'edit_position' => 'prepend',
   						'title' => __('Title', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h4',
						),

   					array(
   						'id' => 'tabs',
   						'type' => 'tabs',
   						'title' => __('FAQ', 'mfn-opts'),
   						'desc' => __('<b>JavaScript</b> content like Google Maps and some plugins shortcodes do <b>not work</b> in tabs', 'mfn-opts'),
							'options' => [
								'title' => [
									'input',
									__('Question', 'mfn-opts'),
									__('Sample question', 'mfn-opts'),
								],
								'content' => [
									'textarea',
									__('Answer', 'mfn-opts'),
									'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Mauris eu massa orci.',
								],
							],
							'std' => [
								0 => [
									'title' => __('This is the 1st question', 'mfn-opts'),
									'content' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Mauris eu massa orci.',
								],
								1 => [
									'title' => __('This is the 2nd question', 'mfn-opts'),
									'content' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Mauris eu massa orci.',
								],
							],
							'primary' => 'title',
							'preview' => 'tabs',
   					),

						array(
   						'id' => 'open1st',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Open first', 'mfn-opts'),
   						'options' => array(
   							0 => __('Default', 'mfn-opts'),
   							1 => __('Open first', 'mfn-opts'),
   						),
							'std' => '0',
   					),

   					array(
   						'id' => 'openAll',
   						're_render' => true,
   						'type' => 'switch',
							'title' => __('Open all', 'mfn-opts'),
   						'options' => array(
								0 => __('Default', 'mfn-opts'),
   							1 => __('Open all', 'mfn-opts'),
							),
							'std' => '0',
   					),

						array(
   						'id' => 'style',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Style', 'mfn-opts'),
   						'options' => array(
   							'accordion' => __('Accordion', 'mfn-opts'),
   							'toggle' => __('Toggle', 'mfn-opts'),
   						),
							'std' => 'accordion',
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .heading:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .heading:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .heading:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .heading:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .heading:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .heading:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Bar', 'mfn-opts'),
   					),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question .title|before:border-style',
							'attr_id' => 'border_style_faqt',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question .title|before:border-color',
							'condition' => array( 'id' => 'border_style_faqt', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question:border-width',
		  				'condition' => array( 'id' => 'border_style_faqt', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_faqt', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_faqt', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_faqt', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question .title:box-shadow',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow'
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question .title:box-shadow_laptop',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question .title:box-shadow_tablet',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question .title:box-shadow_mobile',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq:--mfn-faq-line',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Line color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question:background',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question.active:background',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title bar', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#titlenormal" data-tab="titlenormal">Normal</a></li><li><a href="#titlehover" data-tab="titlehover">Active</a></li></ul><div class="mfn-sft mfn-sft-titlenormal mfn-tabs-fields-active">',
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question .title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question .title i:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Icon color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-titlehover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question.active .title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question.active .title i:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Icon color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question .answer:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question .answer:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'themeoptions' => 'style:simple',
   						'title' => __('Bar line', 'mfn-opts'),
   					),

   					array(
							'type' => 'html',
							'themeoptions' => 'style:simple',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#barlinenormal" data-tab="barlinenormal">Normal</a></li><li><a href="#barlinehover" data-tab="barlinehover">Active</a></li></ul><div class="mfn-sft mfn-sft-barlinenormal mfn-tabs-fields-active">',
						),

   					array(
							'id' => 'style:.style-simple .mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question:border-color',
							'type' => 'color',
							'themeoptions' => 'style:simple',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'themeoptions' => 'style:simple',
							'html' => '</div><div class="mfn-sft mfn-sft-barlinehover">',
						),

						array(
							'id' => 'style:.style-simple .mcb-section .mcb-wrap .mcb-item-mfnuidelement .faq .question.active|after:background-color',
							'type' => 'color',
							'themeoptions' => 'style:simple',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'themeoptions' => 'style:simple',
							'html' => '</div></div>',
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Feature Box -------------------------------------------------------

   			'feature_box' => array(
   				'type' => 'feature_box',
   				'title' => __('Feature Box', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'boxes',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'image',
   						'dynamic_data' => 'featured_image',
   						'type' => 'upload',
   						'title' => __('Image', 'mfn-opts'),
   						'desc' => __('Recommended image width <b>384px - 960px</b> depending on size of the item', 'mfn-opts'),
							'std' => $this->get_placeholder(),
							'preview' => 'image',
   					),

   					array(
   						'id' => 'title',
   						'type' => 'text',
						'dynamic_data' => 'title',
   						'edit_tag' => '.desc_wrapper | .title',
   						'edit_position' => 'prepend',
   						'title' => __('Title', 'mfn-opts'),
   						'desc' => __('Allowed HTML tags: span, strong, b, em, i, u', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h4',
						),

   					array(
   						'id' => 'content',
   						'type' => 'textarea',
   						'dynamic_data' => 'content',
   						'edit_tag' => '.desc_wrapper | div.desc',
   						'edit_position' => 'append',
   						'title' => __('Content', 'mfn-opts'),
							'std' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut elit tellus, luctus nec ullamcorper mattis, pulvinar dapibus leo.',
							'preview' => 'content',
							'class' => 'form-content-full-width',
   					),

   					// link

   					array(
   						'type' => 'header',
   						'title' => __('Link', 'mfn-opts'),
   					),


   					array(
   						'id' => 'link_type',
   						'attr_id' => 'link_type',
   						'type' => 'select',
							'title' => __('On click action', 'mfn-opts'),
   						'options' => array(
								'' => __('Default', 'mfn-opts'),
								'1' => __('Open popup', 'mfn-opts'),
							),
							'std' => '',
   					),

   					array(
   						'id' => 'popup_id',
   						'condition' => array( 'id' => 'link_type', 'opt' => 'is', 'val' => '1' ),
   						'type' => 'text',
   						'title' => __('Popup ID', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link',
   						'type' => 'text',
   						'dynamic_data' => 'permalink',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'target',
   						'type' => 'select',
   						'title' => __('Target', 'mfn-opts'),
   						'options' => array(
   							0 => __('Default | _self', 'mfn-opts'),
   							1 => __('New tab or window | _blank', 'mfn-opts'),
   							'lightbox' => __('Lightbox (image or embed video)', 'mfn-opts'),
   						),
   					),

            array(
							'id' => 'link_title',
							'type' => 'text',
							'dynamic_data' => 'title',
							'title' => __('Link title', 'mfn-opts'),
						),

   					array(
   						'type' => 'header',
   						'class' => 'mfn-deprecated',
   						'title' => __('Deprecated', 'mfn-opts'),
   					),

   					array(
   						'id' => 'background',
   						'class' => 'mfn-deprecated',
   						'type' => 'color',
   						'title' => __('Background color', 'mfn-opts'),
   					),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Image', 'mfn-opts'),
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_box .photo_wrapper img:border-style',
							'attr_id' => 'border_style_fbimg',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_box .photo_wrapper img:border-color',
							'condition' => array( 'id' => 'border_style_fbimg', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_box .photo_wrapper img:border-width',
		  				'condition' => array( 'id' => 'border_style_fbimg', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_box .photo_wrapper img:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_fbimg', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_box .photo_wrapper img:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_fbimg', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_box .photo_wrapper img:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_fbimg', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_box .photo_wrapper img:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_box .photo_wrapper img:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_box .photo_wrapper img:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_box .photo_wrapper img:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper .title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper .title:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_box_wrapper .desc_wrapper:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_box_wrapper .desc_wrapper:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_box_wrapper .desc_wrapper:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_box_wrapper .desc_wrapper:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper .desc:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper .desc:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Feature List ---------------------------------------------------

   			'feature_list' => array(
   				'type' => 'feature_list',
   				'title' => __('Feature List', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'elements',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
   						'id' => 'tabs',
   						'type' => 'tabs',
   						'title' => __('Items', 'mfn-opts'),
							'options' => [
								'title' => [
									'input',
									__('Title', 'mfn-opts'),
									__('List item', 'mfn-opts'),
								],
								'icon' => [
									'icon',
									__('Icon', 'mfn-opts'),
									'icon-lamp',
								],
								'link' => [
									'input',
									__('Link', 'mfn-opts'),
									'',
								],
								'target' => [
									'input',
									__('Target', 'mfn-opts'),
									'',
								],
								'animate' => [
									'input',
									__('Animate', 'mfn-opts'),
									'',
								],
							],
							'std' => [
								0 => [
									'title' => 'This is the 1st item',
									'icon' => 'icon-book',
									'link' => '',
									'target' => '',
									'animate' => '',
								],
								1 => [
									'title' => 'This is the 2nd item',
									'icon' => 'icon-bucket',
									'link' => '',
									'target' => '',
									'animate' => '',
								],
							],
							'primary' => 'title',
							'preview' => 'tabs',
   					),

   					array(
   						'id' => 'content',
   						'dynamic_data' => 'content',
   						'type' => 'textarea',
   						'edit_tag' => '.feature_list | ul.fl-content',
   						'edit_position' => 'append',
   						'title' => __('Content', 'mfn-opts'),
   						'preview' => 'content',
							'class' => 'form-content-full-width',
   					),

   					array(
   						'id' => 'columns',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Columns', 'mfn-opts'),
   						'options' => array(
   							2	=> 2,
   							3	=> 3,
   							4	=> 4,
   							5	=> 5,
   							6	=> 6,
   						),
   						'std' => 4,
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Container', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_list ul li p:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_list ul li,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_list ul li a:background-color',
							'type' => 'color',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_list ul li,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_list ul li a:color',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_list ul li|hover,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_list ul li|hover a:background-color',
							'type' => 'color',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_list ul li|hover,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_list ul li|hover a:color',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Icon', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_list ul li .icon i:font-size',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_list ul li .icon i:font-size_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_list ul li .icon i:font-size_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_list ul li .icon i:font-size_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#iconnormal" data-tab="iconnormal">Normal</a></li><li><a href="#iconhover" data-tab="iconhover">Hover</a></li></ul><div class="mfn-sft mfn-sft-iconnormal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_list ul li .icon i:color',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-iconhover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_list ul li|hover .icon i:color',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'themeoptions' => 'style:',
   						'title' => __('Line', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .feature_list hr:background-color',
							'type' => 'color',
							'themeoptions' => 'style:',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Flat Box -------------------------------------------------------

   			'flat_box' => array(
   				'type' => 'flat_box',
   				'title' => __('Flat Box', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'boxes',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'image',
   						'dynamic_data' => 'featured_image',
   						'type' => 'upload',
   						'title' => __('Image', 'mfn-opts'),
   						'desc' => __('Recommended image width <b>768px - 1920px</b> depending on size of the item', 'mfn-opts'),
							'std' => $this->get_placeholder(),
							'preview' => 'image',
   					),

   					array(
   						'id' => 'title',
   						'type' => 'text',
						'dynamic_data' => 'title',
   						'edit_tag' => '.desc_wrapper | .title',
   						'edit_position' => 'prepend',
   						'title' => __('Title', 'mfn-opts'),
   						'desc' => __('Allowed HTML tags: span, strong, b, em, i, u', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h4',
						),

   					array(
   						'id' => 'content',
   						'type' => 'textarea',
   						'dynamic_data' => 'content',
   						'edit_tag' => '.desc_wrapper | div.desc',
   						'edit_position' => 'append',
   						'title' => __('Content', 'mfn-opts'),
							'std' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut elit tellus, luctus nec ullamcorper mattis, pulvinar dapibus leo.',
							'preview' => 'content',
							'class' => 'form-content-full-width',
   					),

   					// icon

   					array(
   						'type' => 'header',
   						'title' => __('Icon', 'mfn-opts'),
   					),

   					array(
   						'id' => 'icon',
   						'type' => 'icon',
   						'title' => __('Icon', 'mfn-opts'),
   						'std' => 'icon-lamp',
   					),

   					array(
   						'id' => 'icon_image',
   						'type' => 'upload',
   						'title' => __('Image', 'mfn-opts'),
   						'desc' => __('Replaces icon if uploaded', 'mfn-opts'),
   					),

   					// link

   					array(
   						'type' => 'header',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link',
   						'type' => 'text',
   						'dynamic_data' => 'permalink',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'target',
   						'type' => 'select',
   						'title' => __('Target', 'mfn-opts'),
   						'options' => array(
   							0 => __('Default | _self', 'mfn-opts'),
   							1 => __('New tab or window | _blank', 'mfn-opts'),
   							'lightbox' => __('Lightbox (image or embed video)', 'mfn-opts'),
   						),
   					),

            array(
							'id' => 'link_title',
							'dynamic_data' => 'title',
							'type' => 'text',
							'title' => __('Link title', 'mfn-opts'),
						),

   					array(
   						'type' => 'header',
   						'class' => 'mfn-deprecated',
   						'title' => __('Deprecated', 'mfn-opts'),
   					),

   					array(
   						'id' => 'background',
   						'class' => 'mfn-deprecated',
   						'type' => 'color',
   						'title' => __('Background color', 'mfn-opts'),
   					),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Image', 'mfn-opts'),
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flat_box .photo_wrapper:border-style',
							'attr_id' => 'border_style_fbpw',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flat_box .photo_wrapper:border-color',
							'condition' => array( 'id' => 'border_style_fbpw', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flat_box .photo_wrapper:border-width',
		  				'condition' => array( 'id' => 'border_style_fbpw', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flat_box .photo_wrapper:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_fbpw', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flat_box .photo_wrapper:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_fbpw', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flat_box .photo_wrapper:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_fbpw', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flat_box .photo_wrapper:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flat_box .photo_wrapper:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flat_box .photo_wrapper:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flat_box .photo_wrapper:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flat_box .photo_wrapper:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flat_box .photo_wrapper:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flat_box .photo_wrapper:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flat_box .photo_wrapper:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper .title:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper .title:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Icon', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flat_box .photo_wrapper .icon:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flat_box .photo_wrapper .icon:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flat_box .photo_wrapper .icon i:font-size',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flat_box .photo_wrapper .icon i:font-size_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flat_box .photo_wrapper .icon i:font-size_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .flat_box .photo_wrapper .icon i:font-size_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper .desc:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper .desc:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

				// Heading  --------------------------------------------------------

   			'heading' => array(
   				'type' => 'heading',
   				'title' => __('Heading', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'typography',
   				'attr' => array(
   				//'fields' => array(

						// HTML content

						/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'title',
   						'type' => 'textarea',
						'dynamic_data' => 'title',
   						'class' => 'form-content-full-width',
   						'edit_tag' => '.mcb-item-heading-inner | .title',
   						'edit_position' => 'append',
   						'edit_tag_var' => 'preview-header_taginput',
   						'title' => __('Title', 'mfn-opts'),
							'std' => __('This is the heading', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
							'id' => 'header_tag',
							're_render' => true,
							'type' 	=> 'switch',
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h2',
						),

						// link

   					array(
   						'type' => 'header',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link',
   						'dynamic_data' => 'permalink',
   						'type' => 'text',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'target',
   						'type' => 'select',
   						'title' => __('Target', 'mfn-opts'),
   						'options' => array(
   							0 => __('Default | _self', 'mfn-opts'),
   							1 => __('New tab or window | _blank', 'mfn-opts'),
   							'lightbox' => __('Lightbox (image or embed video)', 'mfn-opts'),
   						),
   					),

            array(
							'id' => 'link_title',
							'type' => 'text',
							'title' => __('Link title', 'mfn-opts'),
						),

            array(
   						'id' => 'onclick',
   						'type' => 'text',
   						'title' => __('Onclick', 'mfn-opts'),
							'before' => 'onclick=',
   					),

            // mask shape

            array(
            	'type' => 'header',
  						'title' => __('Mask shape', 'mfn-opts'),
  					),

  					array(
  						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:background-image',
  						'type' => 'upload',
  						'title' => __('Background image', 'mfn-opts'),
  					),

  					array(
  						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:background-position',
  						'type' => 'select',
  						'title' => __('Background position', 'mfn-opts'),
  						'options' => array(
  							'center center' => __('Center Center', 'mfn-opts'),
  							'center left' => __('Center Left', 'mfn-opts'),
  							'center right' => __('Center Right', 'mfn-opts'),
  							'top left' => __('Top Left', 'mfn-opts'),
  							'top right' => __('Top Right', 'mfn-opts'),
  							'top center' => __('Top Center', 'mfn-opts'),
  							'bottom left' => __('Bottom Left', 'mfn-opts'),
  							'bottom right' => __('Bottom Right', 'mfn-opts'),
  							'bottom center' => __('Bottom Center', 'mfn-opts'),
  						),
							'std' => 'center center',
  					),

  					array(
  						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:-webkit-text-stroke-width',
  						'type' => 'select',
  						'title' => __('Text stroke width', 'mfn-opts'),
  						'options' => [
  							'initial' => __('Default', 'mfn-opts'),
  							'thin' => __('Thin', 'mfn-opts'),
  							'medium' => __('Medium', 'mfn-opts'),
  							'thick' => __('Thick', 'mfn-opts'),
  						],
  					),

  					array(
  						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:-webkit-text-stroke-color',
  						'type' => 'color',
  						'title' => __('Text stroke color', 'mfn-opts'),
  						'std' => '',
  					),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Heading', 'mfn-opts'),
   					),

						array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-toggled mfn-sidebar-fields-tabs mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title a:color',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title|hover,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title a|hover:color',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:text-shadow',
							'type' => 'text_shadow',
							'class' => 'mfn-toggled',
							'title' => __('Text shadow', 'mfn-opts'),
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'toggle_fields mfn-toggled',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:-webkit-line-clamp',
   						'type' => 'text',
   						'title' => __('Line clamp', 'mfn-opts'),
   						'desc' => __('Limit the number of lines to', 'mfn-opts'),
   						'param' => 'number',
   						'class' => 'narrow mfn-line-clamp-field',
							'preview' => 'number',
   					),

						// HTML end: settings

						array(
   						'type' => 'html',
   						'html' => '</div>',
   					),

   				),
   			),

   			// Helper -------------------------------------------------------

   			'helper' => array(
   				'type' => 'helper',
   				'title' => __('Helper', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'blocks',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'title',
   						'type' => 'text',
   						'dynamic_data' => 'title',
   						'edit_tag' => '.helper_header | .title',
   						'edit_position' => 'prepend',
   						'title' => __('Title', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
   						'id' => 'title_tag',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Title tag', 'mfn-opts'),
   						'options' => array(
   							'h1' => 'H1',
   							'h2' => 'H2',
   							'h3' => 'H3',
   							'h4' => 'H4',
   							'h5' => 'H5',
   							'h6' => 'H6',
   							'p' => 'p',
								'span' => 'span',
								'div' => 'div',
   						),
   						'std' => 'h4',
   					),

						// item 1

   					array(
   						'type' => 'header',
   						'title' => __('Item 1', 'mfn-opts'),
   					),

   					array(
   						'id' => 'title1',
   						'dynamic_data' => 'title',
   						'type' => 'text',
   						'title' => __('Title', 'mfn-opts'),
							'std' => __('This is the 1st item title', 'mfn-opts'),
   					),

   					array(
   						'id' => 'content1',
   						'dynamic_data' => 'content',
   						'type' => 'textarea',
   						'title' => __('Content', 'mfn-opts'),
   						'desc' => __('Some shortcodes and HTML tags allowed', 'mfn-opts'),
							'std' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut elit tellus, luctus nec ullamcorper mattis, pulvinar dapibus leo.',
							'class' => 'form-content-full-width',
   					),

   					array(
   						'id' => 'link1',
   						'dynamic_data' => 'permalink',
   						'type' => 'text',
   						'title' => __('Link', 'mfn-opts'),
   						'desc' => __('Use if you want to link to another page instead of showing the content', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link_type',
   						'attr_id' => 'link_type',
   						'type' => 'select',
							'title' => __('On click action', 'mfn-opts'),
   						'options' => array(
								'' => __('Default', 'mfn-opts'),
								'1' => __('Open popup', 'mfn-opts'),
							),
							'std' => '',
   					),

   					array(
   						'id' => 'popup_id',
   						'condition' => array( 'id' => 'link_type', 'opt' => 'is', 'val' => '1' ),
   						'type' => 'text',
   						'title' => __('Popup ID', 'mfn-opts'),
   					),

   					array(
   						'id' => 'target1',
   						'type' => 'switch',
							'title' => __('Link target', 'mfn-opts'),
   						'options' => array(
								0 => __('_self', 'mfn-opts'),
								1 => __('_blank', 'mfn-opts'),
							),
							'std' => 0,
   					),

   					array(
   						'id' => 'class1',
   						'type' => 'text',
   						'title' => __('Link class', 'mfn-opts'),
   						'desc' => __('e.g. <b>prettyphoto</b> or <b>scroll</b>', 'mfn-opts'),
   					),

						// item 2

   					array(
   						'type' => 'header',
   						'title' => __('Item 2', 'mfn-opts'),
   					),

   					array(
   						'id' => 'title2',
   						'type' => 'text',
   						'dynamic_data' => 'title',
   						'title' => __('Title', 'mfn-opts'),
   					),

   					array(
   						'id' => 'content2',
   						'type' => 'textarea',
   						'dynamic_data' => 'content',
   						'title' => __('Content', 'mfn-opts'),
   						'desc' => __('Some shortcodes and HTML tags allowed', 'mfn-opts'),
							'class' => 'form-content-full-width',
   					),

   					array(
   						'id' => 'link2',
   						'type' => 'text',
   						'dynamic_data' => 'permalink',
   						'title' => __('Link', 'mfn-opts'),
   						'desc' => __('Use if you want to link to another page instead of showing the content', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link_type_2',
   						'attr_id' => 'link_type_2',
   						'type' => 'select',
							'title' => __('On click action', 'mfn-opts'),
   						'options' => array(
								'' => __('Default', 'mfn-opts'),
								'1' => __('Open popup', 'mfn-opts'),
							),
							'std' => '',
   					),

   					array(
   						'id' => 'popup_id_2',
   						'condition' => array( 'id' => 'link_type_2', 'opt' => 'is', 'val' => '1' ),
   						'type' => 'text',
   						'title' => __('Popup ID', 'mfn-opts'),
   					),

   					array(
   						'id' => 'target2',
   						'type' => 'switch',
							'title' => __('Link target', 'mfn-opts'),
   						'options' => array(
								0 => __('_self', 'mfn-opts'),
								1 => __('_blank', 'mfn-opts'),
							),
							'std' => 0,
   					),

   					array(
   						'id' => 'class2',
   						'type' => 'text',
   						'title' => __('Link class', 'mfn-opts'),
   						'desc' => __('e.g. <b>prettyphoto</b> or <b>scroll</b>', 'mfn-opts'),
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .helper_header .title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .helper_header .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Button', 'mfn-opts'),
   					),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .helper_header .links .link:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .helper_header .links .link:background',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .helper_header .links .link|hover,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .helper_header .links .link.active:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .helper_header .links .link|hover,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .helper_header .links .link.active:background',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .helper_content .item:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .helper_content .item:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Hover Box ------------------------------------------------------

   			'hover_box' => array(
   				'type' => 'hover_box',
   				'title' => __('Hover Box', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'boxes',
   				'attr' => array(
   				//'fields' => array(

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'image',
   						'dynamic_data' => 'featured_image',
   						'type' => 'upload',
   						'title' => __('Image', 'mfn-opts'),
   						'desc' => __('Recommended image width <b>768px - 1920px</b> depending on size of the item', 'mfn-opts'),
							'std' => $this->get_placeholder(),
							'preview' => 'image',
   					),

   					array(
   						'id' => 'image_hover',
   						'dynamic_data' => 'featured_image',
   						'type' => 'upload',
   						'title' => __('Hover image', 'mfn-opts'),
   						'desc' => __('Images must have the same size', 'mfn-opts'),
							'std' => $this->get_placeholder(),
   					),

						// link

   					array(
   						'type' => 'header',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link_type',
   						'attr_id' => 'link_type',
   						'type' => 'select',
							'title' => __('On click action', 'mfn-opts'),
   						'options' => array(
								'' => __('Default', 'mfn-opts'),
								'1' => __('Open popup', 'mfn-opts'),
							),
							'std' => '',
   					),

   					array(
   						'id' => 'popup_id',
   						'condition' => array( 'id' => 'link_type', 'opt' => 'is', 'val' => '1' ),
   						'type' => 'text',
   						'title' => __('Popup ID', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link',
   						'dynamic_data' => 'permalink',
   						'type' => 'text',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'target',
   						'type' => 'select',
   						'title' => __('Target', 'mfn-opts'),
   						'options' => array(
   							0 => __('Default | _self', 'mfn-opts'),
   							1 => __('New tab or window | _blank', 'mfn-opts'),
   							'lightbox' => __('Lightbox (image or embed video)', 'mfn-opts'),
   						),
   					),

            array(
							'id' => 'link_title',
							'type' => 'text',
							'dynamic_data' => 'title',
							'title' => __('Link title', 'mfn-opts'),
						),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Hover Color ----------------------------------------------------

   			'hover_color' => array(
   				'type' => 'hover_color',
   				'title' => __('Hover Color', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'elements',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'content',
   						'edit_tag' => '.hover_color_bg | div.hover_color_wrapper',
   						'edit_position' => 'prepend',
   						'type' => 'textarea',
   						'title' => __('Content', 'mfn-opts'),
							'std' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut elit tellus, luctus nec ullamcorper mattis, pulvinar dapibus leo.',
							'preview' => 'content',
							'dynamic_data' => 'content',
							'class' => 'form-content-full-width',
   					),

   					// link

   					array(
   						'type' => 'header',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link_type',
   						'attr_id' => 'link_type',
   						'type' => 'select',
							'title' => __('On click action', 'mfn-opts'),
   						'options' => array(
								'' => __('Default', 'mfn-opts'),
								'1' => __('Open popup', 'mfn-opts'),
							),
							'std' => '',
   					),

   					array(
   						'id' => 'popup_id',
   						'condition' => array( 'id' => 'link_type', 'opt' => 'is', 'val' => '1' ),
   						'type' => 'text',
   						'title' => __('Popup ID', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link',
   						'dynamic_data' => 'permalink',
   						'type' => 'text',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'target',
   						'type' => 'select',
   						'title' => __('Target', 'mfn-opts'),
   						'options' => array(
   							0 => __('Default | _self', 'mfn-opts'),
   							1 => __('New tab or window | _blank', 'mfn-opts'),
   							'lightbox' => __('Lightbox (image or embed video)', 'mfn-opts'),
   						),
   					),

            array(
							'id' => 'link_title',
							'type' => 'text',
							'dynamic_data' => 'title',
							'title' => __('Link title', 'mfn-opts'),
						),

   					array(
   						'id' => 'class',
   						'type' => 'text',
   						'title' => __('Class', 'mfn-opts'),
   						'desc' => __('e.g. <b>scroll</b>', 'mfn-opts'),
   					),

						// deprecated

   					array(
   						'type' => 'header',
   						'class' => 'mfn-deprecated',
   						'title' => __('Deprecated', 'mfn-opts'),
   					),

						array(
   						'id' => 'align',
   						'class' => 'mfn-deprecated',
   						'type' => 'switch',
   						'title' => __('Text alignment', 'mfn-opts'),
   						'options' => array(
   							'left' => __('Left', 'mfn-opts'),
   							'' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
   					),

   					array(
   						'id' => 'padding',
   						'class' => 'mfn-deprecated',
   						'type' => 'text',
   						'title' => __('Padding', 'mfn-opts'),
   						'desc' => __('Use value with <b>px</b> or <b>%</b>. Example: <b>20px</b> or <b>20px 10px 20px 10px</b> or <b>20px 1%</b>', 'mfn-opts'),
   					),

   					array(
   						'id' => 'background',
   						'class' => 'mfn-deprecated',
   						'type' => 'color',
   						'title' => __('Background color', 'mfn-opts'),
   					),

   					array(
   						'id' => 'background_hover',
   						'class' => 'mfn-deprecated',
   						'type' => 'color',
   						'title' => __('Background color hover', 'mfn-opts'),
   					),

   					array(
   						'id' => 'border',
   						'class' => 'mfn-deprecated',
   						'type' => 'color',
   						'title' => __('Border color', 'mfn-opts'),
   					),

   					array(
   						'id' => 'border_hover',
   						'class' => 'mfn-deprecated',
   						'type' => 'color',
   						'title' => __('Border color hover', 'mfn-opts'),
   					),

   					array(
   						'id' => 'border_width',
   						'class' => 'mfn-deprecated',
   						'type' => 'text',
   						'title' => __('Border width', 'mfn-opts'),
   						'desc' => __('Use value with <b>px</b>. Example: <b>1px</b> or <b>2px 5px 2px 5px</b>', 'mfn-opts'),
   					),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'id' => 'style',
							'type' => 'textarea',
							'title' => __('Element inline CSS', 'mfn-opts'),
							'desc' => __('Example: <b>opacity: 0.5;</b>', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

						array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .hover_color_wrapper:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .hover_color_wrapper:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .hover_color_wrapper:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .hover_color_wrapper:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner .hover_color_wrapper:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner .hover_color_wrapper:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner .hover_color_wrapper:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner .hover_color_wrapper:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .hover_color_wrapper:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .hover_color_wrapper:border-style',
							'type' => 'select',
							'attr_id' => 'border_style_hc',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .hover_color_wrapper:border-width',
		  				'condition' => array( 'id' => 'border_style_hc', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .hover_color_wrapper:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_hc', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .hover_color_wrapper:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_hc', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .hover_color_wrapper:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_hc', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .hover_color_wrapper:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .hover_color_wrapper:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .hover_color_wrapper:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .hover_color_wrapper:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .hover_color_wrapper:box-shadow',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow'
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .hover_color_wrapper:box-shadow_laptop',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .hover_color_wrapper:box-shadow_tablet',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .hover_color_wrapper:box-shadow_mobile',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .hover_color_wrapper:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .hover_color_wrapper:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .hover_color_wrapper:border-color',
							'condition' => array( 'id' => 'border_style_hc', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement|hover .hover_color_wrapper:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement|hover .hover_color_wrapper:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement|hover .hover_color_wrapper:border-color',
							'condition' => array( 'id' => 'border_style_hc', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),


   				),
   			),

   			// How It Works ---------------------------------------------------

   			'how_it_works' => array(
   				'type' => 'how_it_works',
   				'title' => __('How It Works', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'elements',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
   						'id' => 'title',
   						'type' => 'text',
						'dynamic_data' => 'title',
   						'edit_tag' => '.how_it_works | .title',
   						'edit_position' => 'append',
   						'title' => __('Title', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h4',
						),

   					array(
   						'id' => 'image',
   						'type' => 'upload',
   						'title' => __('Background', 'mfn-opts'),
   						'desc' => __('Recommended: Square Image with transparent background.', 'mfn-opts'),
							'preview' => 'image',
   					),

   					array(
   						'id' => 'number',
   						'type' => 'text',
   						'title' => __('Number', 'mfn-opts'),
   						'param' => 'number',
   						'class' => 'narrow',
   						'std' => '1',
							'preview' => 'number',
   					),

   					array(
   						'id' => 'content',
   						'type' => 'textarea',
   						'edit_tag' => '.how_it_works | div.desc',
   						'dynamic_data' => 'content',
   						'edit_position' => 'append',
   						'title' => __('Content', 'mfn-opts'),
							'std' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut elit tellus, luctus nec ullamcorper mattis, pulvinar dapibus leo.',
							'preview' => 'content',
							'class' => 'form-content-full-width',
   					),

   					// style

   					array(
   						'type' => 'header',
   						'title' => __('Style', 'mfn-opts'),
   					),

   					array(
   						'id' => 'border',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Line', 'mfn-opts'),
   						'options' => array(
   							0 => __('Hide', 'mfn-opts'),
   							1 => __('Show', 'mfn-opts'),
   						),
							'std' => 0,
   					),

   					array(
   						'id' => 'style',
   						'type' => 'select',
   						're_render' => true,
   						'title' => __('Style', 'mfn-opts'),
   						'options' => array(
   							'' => __('Small centered image (image size: max 116px)', 'mfn-opts'),
   							'fill' => __('Fill the circle (image size: 200px x 200px)', 'mfn-opts'),
   						),
   					),

   					// link

   					array(
   						'type' => 'header',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link',
   						'type' => 'text',
   						'dynamic_data' => 'permalink',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'target',
   						'type' => 'select',
   						'title' => __('Target', 'mfn-opts'),
   						'options' => array(
   							0 => __('Default | _self', 'mfn-opts'),
   							1 => __('New tab or window | _blank', 'mfn-opts'),
   							'lightbox' => __('Lightbox (image or embed video)', 'mfn-opts'),
   						),
   					),

            array(
							'id' => 'link_title',
							'dynamic_data' => 'title',
							'type' => 'text',
							'title' => __('Link title', 'mfn-opts'),
						),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

   					array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .desc:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .desc:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Number', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .number:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .number:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .number:border-style',
							'attr_id' => 'border_style_hiw_number',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .number:border-color',
							'condition' => array( 'id' => 'border_style_hiw_number', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .number:border-width',
		  				'condition' => array( 'id' => 'border_style_hiw_number', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .number:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_hiw_number', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .number:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_hiw_number', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .number:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_hiw_number', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .number:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .number:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .number:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .number:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .number:box-shadow',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow'
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .number:box-shadow_laptop',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .number:box-shadow_tablet',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .number:box-shadow_mobile',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Image', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .image:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .image:border-style',
							'attr_id' => 'border_style_hiw_img',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .image:border-color',
							'condition' => array( 'id' => 'border_style_hiw_img', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .image:border-width',
		  				'condition' => array( 'id' => 'border_style_hiw_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .image:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_hiw_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .image:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_hiw_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .image:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_hiw_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .image:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .image:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .image:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .image:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .image_wrapper:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .image_wrapper:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .image_wrapper:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .image_wrapper:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Line', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works.has_border|after:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'themeoptions' => 'style:simple',
   						'title' => __('Divider', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .how_it_works .image:border-color',
							'type' => 'color',
							'themeoptions' => 'style:simple',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

				// HTML ----------------------------------------------------

   			'html' => array(
   				'type' => 'html',
   				'title' => __('HTML', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'elements',
   				'attr' => array(

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'id' => 'content',
							'type' => 'ace',
							'class' => 'ace-html-editor',
							'title' => __('Html code', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render modalbox-card-style-mfnuidhere">',
						),

						array(
							'id' => 'info',
							'class' => 'mfn-alert',
							'type' => 'info',
							'title' => __('This element has no attributes. Please check <b>advanced</b> tab for more customisation options.', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

					),
   			),

				// Icon Box 2  ------------------------------------------------------

				'icon_box_2' => array(
					'type' => 'icon_box_2',
					'title' => __('Icon Box', 'mfn-opts'),
					'size' => '1/4',
					'tablet_size' => '1/4',
					'mobile_size' => '1/1',
					'tablet_resized' => '0',
					'cat' => 'boxes',
					'attr' => array(
					//'fields' => array(

						/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

						array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'id' => 'title',
							'attr_id' => 'title',
							'type' => 'text',
							'edit_tag' => '.mfn-icon-box .desc-wrapper | .title',
							'edit_position' => 'prepend',
							'dynamic_data' => 'title',
							'title' => __('Title', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
						),

						array(
							'id' => 'title_tag',
							'condition' => array( 'id' => 'title', 'opt' => 'isnt', 'val' => '' ),
							'type' => 'switch',
							'title' => __('Tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'H1',
								'h2' => 'H2',
								'h3' => 'H3',
								'h4' => 'H4',
								'h5' => 'H5',
								'h6' => 'H6',
								'div' => 'div',
								'p' => 'p',
								'span' => 'span',
							),
							'std' => 'h4'
						),

						array(
							'id' => 'content',
							'type' => 'textarea',
							'edit_tag' => '.mfn-icon-box .desc-wrapper | .desc',
							'edit_position' => 'prepend',
							'dynamic_data' => 'content',
							'title' => __('Content', 'mfn-opts'),
							'std' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut elit tellus, luctus nec ullamcorper mattis, pulvinar dapibus leo.',
							'preview' => 'content',
							'class' => 'form-content-full-width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box .desc-wrapper .desc:order',
							'type' => 'select',
							'title' => __('Title / Content order', 'mfn-opts'),
							'options' => array(
								'' => __('Title first', 'mfn-opts'),
								'1' => __('Content first', 'mfn-opts'),
							),
						),

						// icon

						array(
							'type' => 'header',
							'title' => __('Icon', 'mfn-opts'),
						),

						array(
							'id' => 'icon',
							'condition' => array( 'id' => 'title', 'opt' => 'isnt', 'val' => '{postmeta:mfn-post-love}' ),
							'attr_id' => 'ib2-icon',
							'type' => 'icon',
							'title' => __('Icon', 'mfn-opts'),
							'std' => 'icon-lamp',
							'preview' => 'icon',
						),

						array(
							'id' => 'icon_position',
							'type' => 'switch',
							'title' => __('Icon position', 'mfn-opts'),
							'options' => array(
								'top' => __('Top', 'mfn-opts'),
								'bottom' => __('Bottom', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							),
							'std' => 'top',
							'responsive' => 'desktop',

						),

						array(
							'id' => 'icon_position_laptop',
							'type' => 'switch',
							'title' => __('Icon position', 'mfn-opts'),
							'options' => array(
								'top' => __('Top', 'mfn-opts'),
								'bottom' => __('Bottom', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
								'' => __('Default', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'laptop',

						),

						array(
							'id' => 'icon_position_tablet',
							'type' => 'switch',
							'title' => __('Icon position', 'mfn-opts'),
							'options' => array(
								'top' => __('Top', 'mfn-opts'),
								'bottom' => __('Bottom', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
								'' => __('Default', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'tablet',

						),

						array(
							'id' => 'icon_position_mobile',
							'type' => 'switch',
							'title' => __('Icon position', 'mfn-opts'),
							'options' => array(
								'top' => __('Top', 'mfn-opts'),
								'bottom' => __('Bottom', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
								'' => __('Default', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'mobile',

						),

						array(
							'id' => 'icon_align',
							'type' => 'switch',
							'title' => __('Icon alignment', 'mfn-opts'),
							'options' => array(
								'start' => __('Start', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'end' => __('End', 'mfn-opts'),
							),
							'std' => 'center',
							'responsive' => 'desktop',

						),

						array(
							'id' => 'icon_align_laptop',
							'type' => 'switch',
							'title' => __('Icon alignment', 'mfn-opts'),
							'options' => array(
								'start' => __('Start', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'end' => __('End', 'mfn-opts'),
								'' => __('Default', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'laptop',

						),

						array(
							'id' => 'icon_align_tablet',
							'type' => 'switch',
							'title' => __('Icon alignment', 'mfn-opts'),
							'options' => array(
								'start' => __('Start', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'end' => __('End', 'mfn-opts'),
								'' => __('Default', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'tablet',

						),

						array(
							'id' => 'icon_align_mobile',
							'type' => 'switch',
							'title' => __('Icon alignment', 'mfn-opts'),
							'options' => array(
								'start' => __('Start', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'end' => __('End', 'mfn-opts'),
								'' => __('Default', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'mobile',

						),

						array(
							'id' => 'image',
							'condition' => array( 'id' => 'title', 'opt' => 'isnt', 'val' => '{postmeta:mfn-post-love}' ),
							'type' => 'upload',
							'dynamic_data' => 'featured_image',
							'title' => __('Image', 'mfn-opts'),
							'attr_id' => 'ib2-image',
							'desc' => __('Image instead of an icon', 'mfn-opts'),
							'preview' => 'image',
						),

						array(
							'id' => 'label',
							'condition' => array( 'id' => 'title', 'opt' => 'isnt', 'val' => '{postmeta:mfn-post-love}' ),
							'type' => 'text',
							'attr_id' => 'ib2-label',
							'title' => __('Label', 'mfn-opts'),
						),

						// link

						array(
							'type' => 'header',
							'title' => __('Link', 'mfn-opts'),
						),

						array(
   						'id' => 'link_type',
   						'attr_id' => 'link_type',
   						'type' => 'select',
							'title' => __('On click action', 'mfn-opts'),
   						'options' => array(
								'' => __('Default', 'mfn-opts'),
								'1' => __('Open popup', 'mfn-opts'),
							),
							'std' => '',
   					),

   					array(
   						'id' => 'popup_id',
   						'condition' => array( 'id' => 'link_type', 'opt' => 'is', 'val' => '1' ),
   						'type' => 'text',
   						'title' => __('Popup ID', 'mfn-opts'),
   					),

						array(
							'id' => 'link',
							'dynamic_data' => 'permalink',
							'type' => 'text',
							'title' => __('Link', 'mfn-opts'),
						),

						array(
							'id' => 'target',
							'type' => 'select',
							'title' => __('Target', 'mfn-opts'),
							'options' => array(
								0 => __('Default | _self', 'mfn-opts'),
								1 => __('New tab or window | _blank', 'mfn-opts'),
								'lightbox' => __('Lightbox (image or embed video)', 'mfn-opts'),
							),
						),

            array(
							'id' => 'link_title',
							'type' => 'text',
							'title' => __('Link title', 'mfn-opts'),
						),

						// Hover

						array(
							'type' => 'header',
							'title' => __('Hover effect', 'mfn-opts'),
						),

						array(
							'id' => 'hover',
							'type' => 'select',
							'title' => __('Hover effect', 'mfn-opts'),
							'options' => array(
								'' => __('None', 'mfn-opts'),
								'move-up' => __('Move box up', 'mfn-opts'),
								'box-scale' => __('Zoom box', 'mfn-opts'),
								'icon-scale' => __('Zoom icon', 'mfn-opts'),
							),
							'std' => '',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render modalbox-card-style">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Container', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box:padding',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box:padding_laptop',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box:padding_tablet',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box:padding_mobile',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box:border-style',
							'attr_id' => 'ib2-border-style',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box:border-color',
							'condition' => array( 'id' => 'ib2-border-style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box:border-width',
							'condition' => array( 'id' => 'ib2-border-style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'dimensions',
							'responsive' => 'desktop',
							'title' => __('Border width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box:border-width_tablet',
							'condition' => array( 'id' => 'ib2-border-style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'dimensions',
							'responsive' => 'tablet',
							'title' => __('Border width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box:border-width_mobile',
							'condition' => array( 'id' => 'ib2-border-style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'dimensions',
							'responsive' => 'mobile',
							'title' => __('Border width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box:border-radius',
							'type' => 'dimensions',
							'title' => __('Border radius', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'desktop',
							'css_attr' => 'border-radius',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box:border-radius_tablet',
							'type' => 'dimensions',
							'title' => __('Border radius', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'tablet',
							'css_attr' => 'border-radius',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box:border-radius_mobile',
							'type' => 'dimensions',
							'title' => __('Border radius', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'mobile',
							'css_attr' => 'border-radius',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box:box-shadow',
							'type' => 'box_shadow',
							'title' => __('Box shadow', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'desktop',
							'css_attr' => 'box-shadow',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box:box-shadow_tablet',
							'type' => 'box_shadow',
							'title' => __('Box shadow', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'tablet',
							'css_attr' => 'box-shadow',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box:box-shadow_mobile',
							'type' => 'box_shadow',
							'title' => __('Box shadow', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'mobile',
							'css_attr' => 'box-shadow',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box|hover:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box|hover:border-color',
							'condition' => array( 'id' => 'ib2-border-style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box|hover:box-shadow',
							'type' => 'box_shadow',
							'title' => __('Box shadow', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Icon wrapper', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:width',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled narrow',
							'css_attr' => 'width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:width_laptop',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled narrow',
							'css_attr' => 'width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:width_tablet',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled narrow',
							'css_attr' => 'width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:width_mobile',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled narrow',
							'css_attr' => 'width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:height',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Height', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled narrow',
							'css_attr' => 'height',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:height_laptop',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Height', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled narrow',
							'css_attr' => 'height',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:height_tablet',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Height', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled narrow',
							'css_attr' => 'height',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:height_mobile',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Height', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled narrow',
							'css_attr' => 'height',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:margin',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:margin_laptop',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:margin_tablet',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:margin_mobile',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-label:typography',
							'condition' => array( 'id' => 'ib2-label', 'opt' => 'isnt', 'val' => '' ),
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper i:font-size',
							'condition' => array( 'id' => 'ib2-icon', 'opt' => 'isnt', 'val' => '' ),
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'title' => __('Icon size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper i:font-size_laptop',
							'condition' => array( 'id' => 'ib2-icon', 'opt' => 'isnt', 'val' => '' ),
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'title' => __('Icon size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper i:font-size_tablet',
							'condition' => array( 'id' => 'ib2-icon', 'opt' => 'isnt', 'val' => '' ),
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'title' => __('Icon size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper i:font-size_mobile',
							'condition' => array( 'id' => 'ib2-icon', 'opt' => 'isnt', 'val' => '' ),
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'title' => __('Icon size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper img:max-width',
							'condition' => array( 'id' => 'ib2-image', 'opt' => 'isnt', 'val' => '' ),
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled narrow',
							'title' => __('Image width', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper img:max-width_laptop',
							'condition' => array( 'id' => 'ib2-image', 'opt' => 'isnt', 'val' => '' ),
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled narrow',
							'title' => __('Image width', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper img:max-width_tablet',
							'condition' => array( 'id' => 'ib2-image', 'opt' => 'isnt', 'val' => '' ),
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled narrow',
							'title' => __('Image width', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper img:max-width_mobile',
							'condition' => array( 'id' => 'ib2-image', 'opt' => 'isnt', 'val' => '' ),
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled narrow',
							'title' => __('Image width', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-label:color',
							'condition' => array( 'id' => 'ib2-label', 'opt' => 'isnt', 'val' => '' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:border-style',
							'attr_id' => 'ib2-border-style-iw',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:border-color',
							'condition' => array( 'id' => 'ib2-border-style-iw', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:border-width',
							'condition' => array( 'id' => 'ib2-border-style-iw', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'dimensions',
							'title' => __('Border width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'desktop',
							'css_attr' => 'border-width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:border-width_tablet',
							'condition' => array( 'id' => 'ib2-border-style-iw', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'dimensions',
							'title' => __('Border width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'tablet',
							'css_attr' => 'border-width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:border-width_mobile',
							'condition' => array( 'id' => 'ib2-border-style-iw', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'dimensions',
							'title' => __('Border width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'mobile',
							'css_attr' => 'border-width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:border-radius',
							'type' => 'dimensions',
							'title' => __('Border radius', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'desktop',
							'css_attr' => 'border-radius',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:border-radius_tablet',
							'type' => 'dimensions',
							'title' => __('Border radius', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'tablet',
							'css_attr' => 'border-radius',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:border-radius_mobile',
							'type' => 'dimensions',
							'title' => __('Border radius', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'mobile',
							'css_attr' => 'border-radius',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:box-shadow',
							'type' => 'box_shadow',
							'title' => __('Box shadow', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'desktop',
							'css_attr' => 'box-shadow',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:box-shadow_tablet',
							'type' => 'box_shadow',
							'title' => __('Box shadow', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'tablet',
							'css_attr' => 'box-shadow',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper:box-shadow_mobile',
							'type' => 'box_shadow',
							'title' => __('Box shadow', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'responsive' => 'mobile',
							'css_attr' => 'box-shadow',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon-wrapper i:color',
							'condition' => array( 'id' => 'ib2-icon', 'opt' => 'isnt', 'val' => '' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Icon color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box|hover .icon-label:color',
							'condition' => array( 'id' => 'ib2-label', 'opt' => 'isnt', 'val' => '' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box|hover .icon-wrapper:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box|hover .icon-wrapper:border-color',
							'condition' => array( 'id' => 'ib2-border-style-iw', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box|hover .icon-wrapper:box-shadow',
							'type' => 'box_shadow',
							'title' => __('Box shadow', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box|hover .icon-wrapper i:color',
							'condition' => array( 'id' => 'ib2-icon', 'opt' => 'isnt', 'val' => '' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Icon color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Description wrapper', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc-wrapper:text-align',
							'responsive' => 'desktop',
							'type' => 'switch',
							'class' => 'mfn-toggled',
							'title' => __('Text align', 'mfn-opts'),
							'options' => array(
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
								'justify' => __('Justify', 'mfn-opts'),
							),
							'std' => '',
							'preview' => 'align',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc-wrapper:text-align_laptop',
							'responsive' => 'laptop',
							'type' => 'switch',
							'class' => 'mfn-toggled',
							'title' => __('Text align', 'mfn-opts'),
							'options' => array(
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
								'justify' => __('Justify', 'mfn-opts'),
							),
							'std' => '',
							'preview' => 'align',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc-wrapper:text-align_tablet',
							'responsive' => 'tablet',
							'type' => 'switch',
							'class' => 'mfn-toggled',
							'title' => __('Text align', 'mfn-opts'),
							'options' => array(
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
								'justify' => __('Justify', 'mfn-opts'),
							),
							'std' => '',
							'preview' => 'align',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc-wrapper:text-align_mobile',
							'responsive' => 'mobile',
							'type' => 'switch',
							'class' => 'mfn-toggled',
							'title' => __('Text align', 'mfn-opts'),
							'options' => array(
								'' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
								'justify' => __('Justify', 'mfn-opts'),
							),
							'std' => '',
							'preview' => 'align',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc-wrapper:padding',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc-wrapper:padding_laptop',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc-wrapper:padding_tablet',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc-wrapper:padding_mobile',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Title', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box|hover .title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:margin',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:margin_laptop',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:margin_tablet',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:margin_mobile',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Description', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-icon-box|hover .desc:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc:margin',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc:margin_laptop',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc:margin_tablet',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc:margin_mobile',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

					),
				),

   			// Icon Box  ------------------------------------------------------

   			'icon_box' => array(
   				'type' => 'icon_box',
   				'title' => __('Icon Box Basic', 'mfn-opts'),
   				// 'deprecated' => 'Icon Box',
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'boxes',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'title',
   						'type' => 'text',
   						'edit_tag' => '.icon_box .desc_wrapper | .title',
   						'edit_position' => 'prepend',
   						'title' => __('Title', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
   						'id' => 'title_tag',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Tag', 'mfn-opts'),
   						'options' => array(
   							'h1' => 'H1',
   							'h2' => 'H2',
   							'h3' => 'H3',
   							'h4' => 'H4',
   							'h5' => 'H5',
   							'h6' => 'H6',
   							'p' => 'p',
								'span' => 'span',
								'div' => 'div',
   						),
   						'std' => 'h4'
   					),

   					array(
   						'id' => 'content',
   						'type' => 'textarea',
   						'edit_tag' => '.icon_box .desc_wrapper | .desc',
   						'edit_position' => 'prepend',
   						'title' => __('Content', 'mfn-opts'),
							'std' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut elit tellus, luctus nec ullamcorper mattis, pulvinar dapibus leo.',
							'preview' => 'content',
							'class' => 'form-content-full-width',
   					),

   					// icon

   					array(
   						'type' => 'header',
   						'title' => __('Icon', 'mfn-opts'),
   					),

   					array(
   						'id' => 'icon',
   						'type' => 'icon',
   						'title' => __('Icon', 'mfn-opts'),
   						'std' => 'icon-lamp',
   						'preview' => 'icon',
   					),

						array(
   						'id' => 'icon_position',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Icon position', 'mfn-opts'),
   						'desc' => __('Left position works only for column widths: 1/4, 1/3 & 1/2', 'mfn-opts'),
   						'options' => array(
   							'left' => __('Left', 'mfn-opts'),
   							'top' => __('Top', 'mfn-opts'),
   						),
   						'std' => 'top',
   					),

   					array(
   						'id' => 'image',
   						'type' => 'upload',
   						'dynamic_data' => 'featured_image',
   						'title' => __('Image', 'mfn-opts'),
							'preview' => 'image',
   					),

   					// link

   					array(
   						'type' => 'header',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link',
   						'type' => 'text',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'target',
   						'type' => 'select',
   						'title' => __('Target', 'mfn-opts'),
   						'options' => array(
   							0 => __('Default | _self', 'mfn-opts'),
   							1 => __('New tab or window | _blank', 'mfn-opts'),
   							'lightbox' => __('Lightbox (image or embed video)', 'mfn-opts'),
   						),
   					),

            array(
							'id' => 'link_title',
							'type' => 'text',
							'title' => __('Link title', 'mfn-opts'),
						),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

		  			array(
   						'id' => 'border',
   						'class' => 'mfn-deprecated',
   						'type' => 'switch',
   						'title' => __('Border', 'mfn-opts'),
   						'options' => array(
   							0 => __('Hide', 'mfn-opts'),
   							1 => __('Show', 'mfn-opts'),
   						),
							'std' => 0,
   					),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

		  			array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),


						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Icon', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .icon_wrapper i:font-size',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .icon_wrapper i:font-size_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .icon_wrapper i:font-size_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .icon_wrapper i:font-size_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .icon_wrapper:border-style',
							'attr_id' => 'border_style_ib_icon',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .icon_wrapper:border-color',
							'condition' => array( 'id' => 'border_style_ib_icon', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .icon_wrapper:border-width',
		  				'condition' => array( 'id' => 'border_style_ib_icon', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .icon_wrapper:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_ib_icon', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .icon_wrapper:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_ib_icon', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .icon_wrapper:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_ib_icon', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .icon_wrapper:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .icon_wrapper:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .icon_wrapper:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .icon_wrapper:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .icon_wrapper .icon:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .icon_wrapper:background',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

		  			array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box|hover .icon_wrapper .icon:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box|hover .icon_wrapper,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box|hover .icon_wrapper|before:background',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box|hover .icon_wrapper:border-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Image', 'mfn-opts'),
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .image_wrapper img:border-style',
							'attr_id' => 'border_style_ib_img',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .image_wrapper img:border-color',
							'condition' => array( 'id' => 'border_style_ib_img', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .image_wrapper img:border-width',
		  				'condition' => array( 'id' => 'border_style_ib_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .image_wrapper img:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_ib_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .image_wrapper img:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_ib_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .image_wrapper img:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_ib_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .image_wrapper img:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .image_wrapper img:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .image_wrapper img:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .image_wrapper img:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .desc_wrapper .title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .desc_wrapper .title:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .desc_wrapper .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .desc_wrapper .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .desc_wrapper .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .desc_wrapper .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Desc', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .desc_wrapper .desc:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .desc_wrapper .desc:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .desc_wrapper .desc:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .desc_wrapper .desc:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .desc_wrapper .desc:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box .desc_wrapper .desc:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'type' => 'header',
		  				'class' => 'toggled_header row-header',
		  				'themeoptions' => 'style:simple',
   						'title' => __('Divider', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box|hover .desc_wrapper .title|before:background-color',
							'themeoptions' => 'style:simple',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .icon_box|hover .desc_wrapper .title|before:width',
		  				'themeoptions' => 'style:simple',
		  				'type' => 'text',
		  				'default_unit' => '%',
		  				'title' => __('Width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'width',
		  			),




						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Image  ---------------------------------------------------------

   			'image' => array(
   				'type' => 'image',
   				'title' => __('Image', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'typography',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'src',
   						'type' => 'upload',
							'dynamic_data' => 'featured_image',
   						're_render' => true,
   						'title' => __('Image', 'mfn-opts'),
							'std' => $this->get_placeholder(),
   						'preview' => 'image',
   					),

   					array(
   						'id' => 'size',
   						'type' => 'select',
   						're_render' => true,
   						'title' => __('Size', 'mfn-opts'),
   						'desc' => __('Select image size from <a target="_blank" href="options-media.php">Settings > Media > Image sizes</a> (Media Library images only)<br />or use below fields for HTML resize', 'mfn-opts'),
   						'php_options' => array(
   							'full' => __('Full size', 'mfn-opts'),
   							'large' => __('Large', 'mfn-opts') .' | '. mfn_get_image_sizes('large', 1),
   							'medium' => __('Medium', 'mfn-opts') .' | '. mfn_get_image_sizes('medium', 1),
   							'thumbnail' => __('Thumbnail', 'mfn-opts') .' | '. mfn_get_image_sizes('thumbnail', 1),
   						),
   						'js_options' => 'media_sizes',
							'std' => 'full',
   					),

						array(
   						'id' => 'stretch',
   						'type' => 'select',
   						're_render' => true,
   						'title' => __('Stretch', 'mfn-opts'),
   						'desc' => __('Stretch image to column width, height will be changed proportionally', 'mfn-opts'),
   						'options' => array(
   							'0' => __('No', 'mfn-opts'),
   							'1' => __('Yes', 'mfn-opts'),
   							'ultrawide' => __('Yes, on ultrawide screens only > 1920px', 'mfn-opts'),
   						),
   					),

						// lazy load

   					array(
   						'type' => 'header',
   						'title' => __('Lazy load', 'mfn-opts'),
   					),

   					array(
   						'id' => 'lazy_load',
   						'type' => 'switch',
   						'title' => __('Lazy load', 'mfn-opts'),
							'options' => array(
								'' => __('Default', 'mfn-opts'),
								'disable' => __('Disable', 'mfn-opts'),
								'lazy' => __('Enable', 'mfn-opts'),
							),
							'std' => '',
   					),

   					// link

   					array(
   						'type' => 'header',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link_type',
   						'attr_id' => 'link_type',
   						'type' => 'select',
							'title' => __('On click action', 'mfn-opts'),
   						'options' => array(
								'' => __('Default', 'mfn-opts'),
								'1' => __('Open popup', 'mfn-opts'),
							),
							'std' => '',
   					),

   					array(
   						'id' => 'popup_id',
   						'condition' => array( 'id' => 'link_type', 'opt' => 'is', 'val' => '1' ),
   						'type' => 'text',
   						'title' => __('Popup ID', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link_image',
   						'attr_id' => 'link_image',
   						'condition' => array( 'id' => 'link_type', 'opt' => 'is', 'val' => '' ),
   						'dynamic_data' => 'featured_image',
   						'type' => 'upload',
   						'title' => __('Popup image', 'mfn-opts'),
   						'desc' => __('This <b>image or embed video</b> will be opened in lightbox', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link',
   						'attr_id' => 'link',
   						'dynamic_data' => 'permalink',
   						'type' => 'text',
   						'title' => __('Link', 'mfn-opts'),
   					),


   					array(
							'id' => 'rel',
							'type' => 'text',
							'title' => __('Rel', 'mfn-opts'),
							'desc' => __('"Rel" attribute to the link', 'mfn-opts'),
							'before' => 'rel='
						),

						array(
   						'id' => 'target',
   						'type' => 'switch',
							'title' => __('Target', 'mfn-opts'),
   						'options' => array(
								0 => __('_self', 'mfn-opts'),
								1 => __('_blank', 'mfn-opts'),
							),
							'std' => 0,
   					),

            array(
							'id' => 'link_title',
							'type' => 'text',
							'title' => __('Link title', 'mfn-opts'),
						),

   					array(
   						'id' => 'hover',
   						'type' => 'switch',
   						'title' => __('Hover effect', 'mfn-opts'),
   						'desc' => __('Hover effect selected in Theme Options', 'mfn-opts'),
   						'options' => array(
   							'disable' => __('Disable', 'mfn-opts'),
								'' => __('Enable', 'mfn-opts'),
   						),
							'std' => '',
   					),

            array(
   						'id' => 'onclick',
   						'type' => 'text',
   						'title' => __('Onclick', 'mfn-opts'),
							'before' => 'onclick=',
   					),

   					// description

   					array(
   						'type' => 'header',
   						'title' => __('Description', 'mfn-opts'),
   					),

   					array(
							'id' => 'info',
							'type' => 'info',
							'class' => 'mfn-alert',
							'title' => __('Works with external images only.', 'mfn-opts'),
						),

   					array(
   						'id' => 'alt',
   						'type' => 'text',
   						'title' => __('Alternate text', 'mfn-opts'),
   					),

   					array(
   						'id' => 'caption',
   						'attr_id' => 'caption',
   						'type' => 'text',
   						'edit_tag' => '.image_frame | p.wp-caption-text',
   						'edit_position' => 'append',
   						'title' => __('Caption', 'mfn-opts'),
   					),

						// mask shape

						array(
							'type' => 'header',
							'title' => __('Mask shape', 'mfn-opts'),
						),

						array(
							'id' => 'mask_shape_type',
							'attr_id' => 'mask_type',
							'type' => 'select',
							'title' => __('Mask type', 'mfn-opts'),
							'options' => array(
								'0' => __('None', 'mfn-opts'),
								'blob' => __('Blob', 'mfn-opts'),
								'blob-2' => __('Blob 2', 'mfn-opts'),
								'brush' => __('Brush', 'mfn-opts'),
								'brush-2' => __('Brush 2', 'mfn-opts'),
								'circle' =>  __('Circle', 'mfn-opts'),
								'cross' => __('Cross', 'mfn-opts'),
								'irregular-circle' => __('Irregular Circle', 'mfn-opts'),
								'stain' => __('Stain', 'mfn-opts'),
								'triangle' => __('Triangle', 'mfn-opts'),
								'custom' => __('Custom', 'mfn-opts'),
							),
							'std' => '0'
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .image_wrapper:background-color',
							'type' => 'color',
							'title' => __('Mask color', 'mfn-opts'),
							'condition' => array( 'id' => 'mask_type', 'opt' => 'isnt', 'val' => '0' ),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .image_frame.mfn-mask-shape .image_wrapper img:-webkit-mask-image',
							'type' => 'upload',
							'title' => __('Mask image', 'mfn-opts'),
							'desc' => 'Only SVG type of image works',
							'condition' => array( 'id' => 'mask_type', 'opt' => 'is', 'val' => 'custom' ),
						),

						array(
							'id' => 'mask_shape_size',
							'attr_id' => 'mask_shape_size',
							'type' => 'select',
							'title' => __('Size', 'mfn-opts'),
							'condition' => array( 'id' => 'mask_type', 'opt' => 'isnt', 'val' => '0' ),
							'options' => array(
								'cover' => __('Cover', 'mfn-opts'),
								'contain' => __('Contain', 'mfn-opts'),
								'custom' => __('Custom', 'mfn-opts'),
							),
							'std' => 'contain',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .image_frame.mfn-mask-shape .image_wrapper img:-webkit-mask-size',
							'type' => 'sliderbar',
							'condition' => array( 'id' => 'mask_shape_size', 'opt' => 'is', 'val' => 'custom' ),
							'title' => __('Scale', 'mfn-opts'),
							'param' => array(
								'min' => '0',
								'max' => '200',
								'step' => '1',
								'unit' => '%',
							),
						),

						array(
							'id' => 'mask_shape_position',
							'attr_id' => 'mask_shape_position',
							'type' => 'select',
							'title' => __('Position', 'mfn-opts'),
							'condition' => array( 'id' => 'mask_type', 'opt' => 'isnt', 'val' => '0' ),
							'options' => array(
								'center center' => __('Center Center', 'mfn-opts'),
								'center left' => __('Center Left', 'mfn-opts'),
								'center right' => __('Center Right', 'mfn-opts'),
								'top left' => __('Top Left', 'mfn-opts'),
								'top right' => __('Top Right', 'mfn-opts'),
								'top center' => __('Top Center', 'mfn-opts'),
								'bottom left' => __('Bottom Left', 'mfn-opts'),
								'bottom right' => __('Bottom Right', 'mfn-opts'),
								'bottom center' => __('Bottom Center', 'mfn-opts'),
								'custom' => __('Custom', 'mfn-opts'),
							),
							'std' => 'center center',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .image_frame.mfn-mask-shape .image_wrapper img:-webkit-mask-position-x',
							'type' => 'sliderbar',
							'condition' => array( 'id' => 'mask_shape_position', 'opt' => 'is', 'val' => 'custom' ),
							'title' => __('Position X', 'mfn-opts'),
							'param' => array(
								'min' => '-100',
								'max' => '100',
								'step' => '1',
								'unit' => '%',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .image_frame.mfn-mask-shape .image_wrapper img:-webkit-mask-position-y',
							'type' => 'sliderbar',
							'condition' => array( 'id' => 'mask_shape_position', 'opt' => 'is', 'val' => 'custom' ),
							'title' => __('Position Y', 'mfn-opts'),
							'param' => array(
								'min' => '-100',
								'max' => '100',
								'step' => '1',
								'unit' => '%',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .image_frame.mfn-mask-shape .image_wrapper img:-webkit-mask-repeat',
							'type' => 'select',
							'title' => __('Repeat', 'mfn-opts'),
							'condition' => array( 'id' => 'mask_type', 'opt' => 'isnt', 'val' => '0' ),
							'options' => array(
								'' => __('No repeat', 'mfn-opts'),
								'repeat-x' => __('Repeat X', 'mfn-opts'),
								'repeat-y' => __('Repeat Y', 'mfn-opts'),
								'repeat' => __('Repeat', 'mfn-opts'),
							),
						),

						// deprecated

   					array(
   						'type' => 'header',
   						'class' => 'mfn-deprecated',
   						'title' => __('Deprecated', 'mfn-opts'),
   					),

   					array(
   						'id' => 'border',
   						'class' => 'mfn-deprecated',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Border', 'mfn-opts'),
   						'options' => array(
   							0 => __('Hide', 'mfn-opts'),
   							1 => __('Show', 'mfn-opts'),
   						),
							'std' => 0,
   					),

   					array(
   						'id' => 'align',
   						'class' => 'mfn-deprecated',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Align', 'mfn-opts'),
   						'desc' => __('If you want image to be <b>resized</b> to column width use <b>align none</b>', 'mfn-opts'),
   						'options' => array(
   							'' => __('None', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
   							'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
   						),
							'std' => '',
							'preview' => 'align',
   					),

   					array(
   						'id' => 'margin',
   						'type' => 'text',
   						'title' => __('Margin top', 'mfn-opts'),
   						'param' => 'number',
   						'after' => 'px',
   						'class' => 'narrow mfn-deprecated',
   					),

   					array(
   						'id' => 'margin_bottom',
   						'type' => 'text',
   						'title' => __('Margin bottom', 'mfn-opts'),
							'param' => 'number',
   						'after' => 'px',
   						'class' => 'narrow mfn-deprecated',
   					),

   					array(
   						'id' => 'width',
   						'type' => 'text',
   						'title' => __('Width', 'mfn-opts'),
   						'desc' => __('HTML resize, optional', 'mfn-opts'),
   						'param' => 'number',
   						'class' => 'narrow mfn-deprecated',
   						'after' => 'px',
   					),

   					array(
   						'id' => 'height',
   						'type' => 'text',
   						'title' => __('Height', 'mfn-opts'),
							'param' => 'number',
   						'class' => 'narrow mfn-deprecated',
   						'after' => 'px',
   					),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Image', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .image_frame:width',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Width', 'mfn-opts'),
							'desc' => __('Use px, %, vw, vh or auto to set content width', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .image_frame:width_laptop',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Width', 'mfn-opts'),
							'desc' => __('Use px, %, vw, vh or auto to set content width', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .image_frame:width_tablet',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Width', 'mfn-opts'),
							'desc' => __('Use px, %, vw, vh or auto to set content width', 'mfn-opts'),
						),
						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .image_frame:width_mobile',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Width', 'mfn-opts'),
							'desc' => __('Use px, %, vw, vh or auto to set content width', 'mfn-opts'),
						),

						array(
   						'id' => 'image_height',
   						'attr_id' => 'image_height',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Height', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'custom' => __('Custom', 'mfn-opts'),
   						),
   						'std' => ''
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .image_frame.mfn-coverimg .image_wrapper img:height',
							'condition' => array( 'id' => 'image_height', 'opt' => 'is', 'val' => 'custom' ),
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Height value', 'mfn-opts'),
							'desc' => __('Use px, %, vw, vh or auto to set image height', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .image_frame.mfn-coverimg .image_wrapper img:height_laptop',
							'condition' => array( 'id' => 'image_height', 'opt' => 'is', 'val' => 'custom' ),
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Height value', 'mfn-opts'),
							'desc' => __('Use px, %, vw, vh or auto to set image height', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .image_frame.mfn-coverimg .image_wrapper img:height_tablet',
							'condition' => array( 'id' => 'image_height', 'opt' => 'is', 'val' => 'custom' ),
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Height value', 'mfn-opts'),
							'desc' => __('Use px, %, vw, vh or auto to set image height', 'mfn-opts'),
						),
						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .image_frame.mfn-coverimg .image_wrapper img:height_mobile',
							'condition' => array( 'id' => 'image_height', 'opt' => 'is', 'val' => 'custom' ),
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Height value', 'mfn-opts'),
							'desc' => __('Use px, %, vw, vh or auto to set image height', 'mfn-opts'),
						),







   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .image_frame:border-style',
							'attr_id' => 'border_style_imgif',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .image_frame:border-color',
							'condition' => array( 'id' => 'border_style_imgif', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .image_frame:border-width',
		  				'condition' => array( 'id' => 'border_style_imgif', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .image_frame:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_imgif', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .image_frame:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_imgif', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .image_frame:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_imgif', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .image_frame:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .image_frame:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .image_frame:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .image_frame:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
   						'id' => 'greyscale',
   						'type' => 'switch',
   						'title' => 'Grayscale',
   						'class' => 'mfn-toggled',
   						'options' => array(
   							0 => __('Disable', 'mfn-opts'),
   							1 => __('Enable', 'mfn-opts'),
   						),
							'std' => 0,
   					),


		  			array(
							'type' => 'header',
							'condition' => array( 'id' => 'caption', 'opt' => 'isnt', 'val' => '' ),
							'class' => 'toggled_header row-header',
   						'title' => __('Caption', 'mfn-opts'),
   					),


		  			array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .wp-caption-text:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .wp-caption-text:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .wp-caption-text:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .wp-caption-text:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .wp-caption-text:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .wp-caption-text:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .wp-caption-text:padding',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .wp-caption-text:padding_laptop',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .wp-caption-text:padding_tablet',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .wp-caption-text:padding_mobile',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .wp-caption-text:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .wp-caption-text:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .wp-caption-text:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .wp-caption-text:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),


		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .wp-caption-text:border-style',
							'attr_id' => 'border_style_img_capt',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .wp-caption-text:border-color',
							'condition' => array( 'id' => 'border_style_img_capt', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .wp-caption-text:border-width',
		  				'condition' => array( 'id' => 'border_style_img_capt', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .wp-caption-text:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_img_capt', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .wp-caption-text:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_img_capt', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .wp-caption-text:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_img_capt', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .wp-caption-text:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .wp-caption-text:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .wp-caption-text:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .wp-caption-text:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),


						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .wp-caption-text:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

   					array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Image Gallery  ---------------------------------------------------------

   			'image_gallery' => array(
   				'type' => 'image_gallery',
   				'title' => __('Image Gallery', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'typography',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'ids',
   						'type' => 'upload_multi',
   						'title' => __('Images', 'mfn-opts'),
   						'preview' => 'images',
   					),

   					// options

   					array(
   						'type' => 'header',
   						'title' => __('Options', 'mfn-opts'),
   					),

   					array(
   						'id' => 'size',
   						're_render' => true,
   						'type' => 'select',
   						'title' => __('Size', 'mfn-opts'),
   						'php_options' => array(
								'thumbnail' => __('Thumbnail', 'mfn-opts') .' | '. mfn_get_image_sizes('thumbnail', 1),
								'medium' => __('Medium', 'mfn-opts') .' | '. mfn_get_image_sizes('medium', 1),
								'large' => __('Large', 'mfn-opts') .' | '. mfn_get_image_sizes('large', 1),
								'full' => __('Full size', 'mfn-opts'),
   						),
   						'js_options' => 'media_sizes',
							'std' => 'full',
   					),

   					array(
   						'id' => 'style',
   						'attr_id' => 'style',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Style', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'flat' => __('Flat', 'mfn-opts'),
   							'fancy' => __('Fancy', 'mfn-opts'),
   							'masonry' => __('Masonry', 'mfn-opts'),
   						),
							'std' => '',
   					),

   					array(
   						'id' => 'layout',
   						'attr_id' => 'layout',
   						'condition' => array( 'id' => 'style', 'opt' => 'isnt', 'val' => 'masonry' ),
   						'type' => 'switch',
   						'title' => __('Layout', 'mfn-opts'),
   						'options' => array(
   							'' 	=> __('Equal columns', 'mfn-opts'),
   							'1' => __('Equal heights', 'mfn-opts'),
   						),
							'std' => '',
   					),

   					array(
   						'id' => 'columns',
   						're_render' => true,
   						'condition' => array( 'id' => 'layout', 'opt' => 'is', 'val' => '' ),
   						'type' => 'text',
   						'title' => __('Columns', 'mfn-opts'),
   						'desc' => __('min: <b>1</b> | max: <b>9</b>', 'mfn-opts'),
   						'after' => 'columns',
   						'param' => 'number',
   						'class' => 'narrow',
   						'std' => '3',
   					),

   					// advanced

   					array(
   						'type' => 'header',
   						'title' => __('Advanced', 'mfn-opts'),
   					),

   					array(
   						'id' => 'greyscale',
   						'type' => 'switch',
   						'title' => __('Grayscale', 'mfn-opts'),
   						'options' => array(
   							0 => __('Disable', 'mfn-opts'),
   							1 => __('Enable', 'mfn-opts'),
   						),
							'std' => 0,
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Image', 'mfn-opts'),
   					),

   					array(
   						'id' => 'image_height',
   						'attr_id' => 'image_height',
   						'type' => 'switch',
   						'title' => __('Height', 'mfn-opts'),
   						'class' => 'gallery-msnry-rerender-item',
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'mfn_custom_img_height' => __('Custom', 'mfn-opts'),
   						),
							'std' => '',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery.mfn_custom_img_height .image_frame .image_wrapper img:--mfn-gal-img-height',
   						'condition' => array( 'id' => 'image_height', 'opt' => 'is', 'val' => 'mfn_custom_img_height' ),
   						'type' => 'text',
   						'class' => 'gallery-msnry-rerender-item',
   						'default_unit' => 'px',
   						'responsive' => 'desktop',
   						'title' => __('Height', 'mfn-opts'),
   						'std' => '300px',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery.mfn_custom_img_height .image_frame .image_wrapper img:--mfn-gal-img-height_laptop',
   						'condition' => array( 'id' => 'image_height', 'opt' => 'is', 'val' => 'mfn_custom_img_height' ),
   						'type' => 'text',
   						'class' => 'gallery-msnry-rerender-item',
   						'default_unit' => 'px',
   						'responsive' => 'laptop',
   						'title' => __('Height', 'mfn-opts'),
   						'std' => '300px',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery.mfn_custom_img_height .image_frame .image_wrapper img:--mfn-gal-img-height_tablet',
   						'condition' => array( 'id' => 'image_height', 'opt' => 'is', 'val' => 'mfn_custom_img_height' ),
   						'type' => 'text',
   						'class' => 'gallery-msnry-rerender-item',
   						'default_unit' => 'px',
   						'responsive' => 'tablet',
   						'title' => __('Height', 'mfn-opts'),
   						'std' => '300px',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery.mfn_custom_img_height .image_frame .image_wrapper img:--mfn-gal-img-height_mobile',
   						'condition' => array( 'id' => 'image_height', 'opt' => 'is', 'val' => 'mfn_custom_img_height' ),
   						'type' => 'text',
   						'class' => 'gallery-msnry-rerender-item',
   						'default_unit' => 'px',
   						'responsive' => 'mobile',
   						'title' => __('Height', 'mfn-opts'),
   						'std' => '300px',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery.equal-heights .image_frame .image_wrapper img:--mfn-gal-img-max-width',
   						'condition' => array( 'id' => 'layout', 'opt' => 'is', 'val' => '1' ),
   						'type' => 'sliderbar',
   						'default_unit' => 'px',
   						'responsive' => 'desktop',
   						'title' => __('Max width', 'mfn-opts'),
   						'param' => array(
								'unit' => 'px',
								'min' => '50',
								'max' => '1200',
								'step' => '1',
							),
							'std' => '300px'
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery.equal-heights .image_frame .image_wrapper img:--mfn-gal-img-max-width_tablet',
   						'condition' => array( 'id' => 'layout', 'opt' => 'is', 'val' => '1' ),
   						'type' => 'sliderbar',
   						'default_unit' => 'px',
   						'responsive' => 'tablet',
   						'title' => __('Max width', 'mfn-opts'),
   						'param' => array(
								'unit' => 'px',
								'min' => '50',
								'max' => '1200',
								'step' => '1',
							),
							'std' => '300px'
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery.equal-heights .image_frame .image_wrapper img:--mfn-gal-img-max-width_mobile',
   						'condition' => array( 'id' => 'layout', 'opt' => 'is', 'val' => '1' ),
   						'type' => 'sliderbar',
   						'default_unit' => 'px',
   						'responsive' => 'mobile',
   						'title' => __('Max width', 'mfn-opts'),
   						'param' => array(
								'unit' => 'px',
								'min' => '50',
								'max' => '1200',
								'step' => '1',
							),
							'std' => '300px'
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .image_frame:border-style',
							'attr_id' => 'border_style_ig',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .image_frame:border-color',
							'condition' => array( 'id' => 'border_style_ig', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .image_frame:border-width',
		  				'condition' => array( 'id' => 'border_style_ig', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .image_frame:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_ig', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .image_frame:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_ig', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .image_frame:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_ig', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .image_frame:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .image_frame:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .image_frame:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .image_frame:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .image_frame:margin',
		  				//'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .gallery-item:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled gallery-msnry-rerender-item',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .image_frame:margin_tablet',
		  				//'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .gallery-item:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled gallery-msnry-rerender-item',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .image_frame:margin_mobile',
		  				//'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .gallery-item:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled gallery-msnry-rerender-item',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Caption', 'mfn-opts'),
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .gallery-item .gallery-caption:display',
   						'attr_id' => 'image_caption',
   						'class' => 'mfn-layout-modifier',
   						'type' => 'switch',
   						'title' => __('Visibility', 'mfn-opts'),
   						'options' => array(
   							'' => __('Visible', 'mfn-opts'),
   							'none' => __('Hidden', 'mfn-opts'),
   						),
							'std' => '',
   					),

   					array(
   						'id' => 'image_caption_style',
   						'condition' => array( 'id' => 'image_caption', 'opt' => 'is', 'val' => '' ),
   						'type' => 'switch',
   						'class' => 'mfn-layout-modifier',
   						'title' => __('Style', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'img_caption_overlay' => __('Overlay', 'mfn-opts'),
   						),
							'std' => '',
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .gallery-item .gallery-caption:background-color',
							'condition' => array( 'id' => 'image_caption', 'opt' => 'is', 'val' => '' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .gallery-item .gallery-caption:color',
							'condition' => array( 'id' => 'image_caption', 'opt' => 'is', 'val' => '' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .gallery-item .gallery-caption:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .gallery-item .gallery-caption:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'std' => '',
   					),
   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .gallery-item .gallery-caption:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'std' => '',
   					),
   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .gallery-item .gallery-caption:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'std' => '',
   					),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .gallery-item .gallery-caption:padding',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled mfn-layout-modifier',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .gallery-item .gallery-caption:padding_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled mfn-layout-modifier',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .gallery-item .gallery-caption:padding_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled mfn-layout-modifier',
		  			),

		 				array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Item', 'mfn-opts'),
   					),

   					array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .gallery-item:padding',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled gallery-msnry-rerender-item',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .gallery-item:padding_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled gallery-msnry-rerender-item',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .gallery-item:padding_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled gallery-msnry-rerender-item',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .gallery .gallery-item:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Info box -------------------------------------------------------

   			'info_box' => array(
   				'type' => 'info_box',
   				'title' => __('Info Box', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'elements',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'title',
   						'type' => 'text',
						'dynamic_data' => 'title',
   						'edit_tag' => '.infobox | .title',
   						'edit_position' => 'prepend',
   						'title' => __('Title', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h3',
						),

						array(
   						'id' => 'tabs',
   						'type' => 'tabs',
   						'title' => __('Items', 'mfn-opts'),
							'options' => [
								'content' => [
									'input',
									__('Title', 'mfn-opts'),
									__('List item', 'mfn-opts'),
								],
							],
							'std' => [
								0 => [
									'content' => 'This is the 1st item',
								],
								1 => [
									'content' => 'This is the 2nd item',
								],
							],
							'primary' => 'content',
							'preview' => 'tabs',
   					),

   					array(
   						'id' => 'content',
   						'dynamic_data' => 'content',
   						'type' => 'textarea',
   						'edit_tag' => '.infobox_wrapper | span.ib-desc',
   						'edit_position' => 'append',
   						'title' => __('Content', 'mfn-opts'),
							'class' => 'form-content-full-width',
   					),

   					array(
   						'type' => 'header',
							'class' => 'mfn-deprecated',
   						'title' => __('Deprecated', 'mfn-opts'),
   					),

   					array(
   						'id' => 'image',
   						'class' => 'mfn-deprecated',
   						'type' => 'upload',
   						'title' => __('Background image', 'mfn-opts'),
   					),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Item title', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .infobox .title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .infobox .title:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .infobox .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .infobox .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .infobox .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .infobox .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Icon', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .infobox ul li|before:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('List', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .infobox ul:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .infobox .ib-desc:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .infobox .ib-desc:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .infobox .ib-desc:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .infobox .ib-desc:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .infobox .ib-desc:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Lines', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .infobox:--mfn-infobox-line',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Line color', 'mfn-opts'),
						),


						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// List NEW -----------------------------------------------------------

   			'list_2' => array(
   				'type' => 'list_2',
   				'title' => __('List', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'blocks',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
   						'id' => 'tabs',
   						'type' => 'tabs',
   						'title' => __('Items', 'mfn-opts'),
							'options' => [

								'content' => [
									'textarea',
									__('Content', 'mfn-opts'),
									'This is the list item',
								],
								'link' => [
									'input',
									__('Link', 'mfn-opts'),
									'',
								],
								'target' => [
									'select',
									__('Target', 'mfn-opts'),
									'',
									[
										'' => __('Default | _self', 'mfn-opts'),
										'_blank' => __('New tab or window | _blank', 'mfn-opts'),
									],
								],
								'icon' => [
									'icon',
									__('Icon', 'mfn-opts'),
									'',
								],
								'image' => [
									'image',
									__('Image', 'mfn-opts'),
									'',
								],
								'color' => [
									'color',
									__('Icon color', 'mfn-opts'),
									'',
								],
								'background' => [
									'color',
									__('Icon background', 'mfn-opts'),
									'',
								],

							],
							'std' => [
								0 => [
									'content' => __('This is the 1st item', 'mfn-opts'),
								],
								1 => [
									'content' => __('This is the 2nd item', 'mfn-opts'),
								],
							],
							'preview' => 'tabs',
							'primary' => 'content',
   					),

						// options

   					array(
   						'type' => 'header',
   						'title' => __('Options', 'mfn-opts'),
   					),

						array(
							'id' => 'type',
							'attr_id' => 'list-2-type',
							'type' => 'switch',
							'title' => __('Type', 'mfn-opts'),
							're_render' => true,
							'options' => array(
								'' => __('Unordered', 'mfn-opts'),
								'ordered' => __('Ordered', 'mfn-opts'),
							),
							'std' => ''
						),

						array(
							'id' => 'starting',
							'type' => 'text',
							'title' => __('Starting number', 'mfn-opts'),
							'condition' => array( 'id' => 'list-2-type', 'opt' => 'is', 'val' => 'ordered' ),
							're_render' => true,
							'std' => '1'
						),

						array(
							'id' => 'align',
							'type' => 'switch',
							'title' => __('Alignment', 'mfn-opts'),
							're_render' => true,
							'options' => array(
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							),
							'std' => 'left',
							'responsive' => 'desktop',

						),

						array(
							'id' => 'align_laptop',
							'type' => 'switch',
							'title' => __('Alignment', 'mfn-opts'),
							're_render' => true,
							'options' => array(
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
								'' => __('Default', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'laptop',

						),

						array(
							'id' => 'align_tablet',
							'type' => 'switch',
							'title' => __('Alignment', 'mfn-opts'),
							're_render' => true,
							'options' => array(
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
								'' => __('Default', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'tablet',

						),

						array(
							'id' => 'align_mobile',
							'type' => 'switch',
							'title' => __('Alignment', 'mfn-opts'),
							're_render' => true,
							'options' => array(
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
								'' => __('Default', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'mobile',

						),

						array(
							'id' => 'valign',
							'type' => 'switch',
							'title' => __('Vertical alignment', 'mfn-opts'),
							'attr_id' => 'list-2-valign',
							're_render' => true,
							'options' => array(
								'top' => __('Top', 'mfn-opts'),
								'middle' => __('Middle', 'mfn-opts'),
								'bottom' => __('Bottom', 'mfn-opts'),
							),
							'std' => 'middle',
							'responsive' => 'desktop',

						),

						array(
							'id' => 'valign_laptop',
							'type' => 'switch',
							'title' => __('Vertical alignment', 'mfn-opts'),
							'attr_id' => 'list-2-valign_laptop',
							're_render' => true,
							'options' => array(
								'top' => __('Top', 'mfn-opts'),
								'middle' => __('Middle', 'mfn-opts'),
								'bottom' => __('Bottom', 'mfn-opts'),
								'' => __('Default', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'laptop',

						),

						array(
							'id' => 'valign_tablet',
							'type' => 'switch',
							'title' => __('Vertical alignment', 'mfn-opts'),
							'attr_id' => 'list-2-valign_tablet',
							're_render' => true,
							'options' => array(
								'top' => __('Top', 'mfn-opts'),
								'middle' => __('Middle', 'mfn-opts'),
								'bottom' => __('Bottom', 'mfn-opts'),
								'' => __('Default', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'tablet',

						),

						array(
							'id' => 'valign_mobile',
							'type' => 'switch',
							'title' => __('Vertical alignment', 'mfn-opts'),
							'attr_id' => 'list-2-valign_mobile',
							're_render' => true,
							'options' => array(
								'top' => __('Top', 'mfn-opts'),
								'middle' => __('Middle', 'mfn-opts'),
								'bottom' => __('Bottom', 'mfn-opts'),
								'' => __('Default', 'mfn-opts'),
							),
							'std' => '',
							'responsive' => 'mobile',

						),

						array(
							'id' => 'divider',
							'type' => 'switch',
							'title' => __('Divider', 'mfn-opts'),
							're_render' => true,
							'options' => array(
								'' => __('Disable', 'mfn-opts'),
								'enable' => __('Enable', 'mfn-opts'),
							),
							'std' => '',
						),

						// icon

   					array(
   						'type' => 'header',
   						'title' => __('Icon', 'mfn-opts'),
   					),

   					array(
   						'id' => 'icon',
   						'type' => 'icon',
   						'title' => __('Icon', 'mfn-opts'),
							're_render' => true,
							'preview' => 'icon',
							'std' => 'icon-dot',
   					),

   					array(
   						'id' => 'image',
   						'type' => 'upload',
							're_render' => true,
   						'title' => __('Image', 'mfn-opts'),
   						'desc' => __('Image replaces icon selected above', 'mfn-opts'),
							'preview' => 'image',
   					),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						// list item

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('List item', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list .mfn-list-item:padding',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list .mfn-list-item:padding_laptop',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list .mfn-list-item:padding_tablet',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list .mfn-list-item:padding_mobile',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
						),

						// icon wrapper

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Icon wrapper', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list-icon:width',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled mfn-slider-input',
							'css_attr' => 'width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list-icon:width_laptop',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled mfn-slider-input',
							'css_attr' => 'width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list-icon:width_tablet',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled mfn-slider-input',
							'css_attr' => 'width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list-icon:width_mobile',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled mfn-slider-input',
							'css_attr' => 'width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list-icon:height',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Height', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled mfn-slider-input',
							'css_attr' => 'height',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list-icon:height_laptop',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Height', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled mfn-slider-input',
							'css_attr' => 'height',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list-icon:height_tablet',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Height', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled mfn-slider-input',
							'css_attr' => 'height',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list-icon:height_mobile',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Height', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled mfn-slider-input',
							'css_attr' => 'height',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list-icon:top',
							'condition' => array( 'id' => 'list-2-valign', 'opt' => 'is', 'val' => 'top' ),
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Top', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list-icon:top_laptop',
							'condition' => array( 'id' => 'list-2-valign', 'opt' => 'is', 'val' => 'top' ),
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Top', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list-icon:top_tablet',
							'condition' => array( 'id' => 'list-2-valign', 'opt' => 'is', 'val' => 'top' ),
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Top', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list-icon:top_mobile',
							'condition' => array( 'id' => 'list-2-valign', 'opt' => 'is', 'val' => 'top' ),
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Top', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list:--mfn-list-icon-spacing',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Spacing', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list:--mfn-list-icon-spacing_laptop',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Spacing', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list:--mfn-list-icon-spacing_tablet',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Spacing', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list:--mfn-list-icon-spacing_mobile',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Spacing', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list:--mfn-list-icon-size',
							'condition' => array( 'id' => 'list-2-type', 'opt' => 'isnt', 'val' => 'ordered' ),
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'title' => __('Icon size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list:--mfn-list-icon-size_laptop',
							'condition' => array( 'id' => 'list-2-type', 'opt' => 'isnt', 'val' => 'ordered' ),
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'title' => __('Icon size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list:--mfn-list-icon-size_tablet',
							'condition' => array( 'id' => 'list-2-type', 'opt' => 'isnt', 'val' => 'ordered' ),
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'title' => __('Icon size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list:--mfn-list-icon-size_mobile',
							'condition' => array( 'id' => 'list-2-type', 'opt' => 'isnt', 'val' => 'ordered' ),
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'title' => __('Icon size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list .mfn-list-icon|after:typography',
							'condition' => array( 'id' => 'list-2-type', 'opt' => 'is', 'val' => 'ordered' ),
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list:--mfn-list-icon-color',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'background_switcher',
							'attr_id' => 'background_switcher_list2',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher mfn-toggled disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list-icon:background-color',
							'condition' => array( 'id' => 'background_switcher_list2', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list-icon:gradient',
		  				'condition' => array( 'id' => 'background_switcher_list2', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list-icon:border-style',
							'attr_id' => 'list2-border-style',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list-icon:border-color',
							'condition' => array( 'id' => 'list2-border-style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list-icon:border-width',
							'condition' => array( 'id' => 'list2-border-style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'dimensions',
							'title' => __('Border width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list-icon:border-radius',
							'type' => 'dimensions',
							'title' => __('Border radius', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
						),

						// description wrapper

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Description wrapper', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list-desc:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list-desc a:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Link color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list-desc a|hover:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Link color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list-desc:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						// divider

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Divider', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list:--mfn-list-divider-width',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Width', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 'max' => 1000, 'step' => 1),
								'%' 	=> array('min' => 1, 'max' => 100, 	'step' => 1),
								'vw' 	=> array('min' => 1, 'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list:--mfn-list-divider-width_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Width', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 'max' => 1000, 'step' => 1),
								'%' 	=> array('min' => 1, 'max' => 100, 	'step' => 1),
								'vw' 	=> array('min' => 1, 'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list:--mfn-list-divider-width_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Width', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 'max' => 1000, 'step' => 1),
								'%' 	=> array('min' => 1, 'max' => 100, 	'step' => 1),
								'vw' 	=> array('min' => 1, 'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list:--mfn-list-divider-width_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Width', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 'max' => 1000, 'step' => 1),
								'%' 	=> array('min' => 1, 'max' => 100, 	'step' => 1),
								'vw' 	=> array('min' => 1, 'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list:--mfn-list-divider-height',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Border width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '20',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-list:--mfn-list-divider-color',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
							'class' => 'mfn-toggled',
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// List -----------------------------------------------------------

   			'list' => array(
   				'type' => 'list',
   				'title' => __('List Basic', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'blocks',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'icon',
   						'type' => 'icon',
   						'title' => __('Icon', 'mfn-opts'),
   						'std' => 'icon-lamp',
							'preview' => 'icon',
   					),

   					array(
   						'id' => 'image',
   						'type' => 'upload',
   						'dynamic_data' => 'featured_image',
   						'title' => __('Image', 'mfn-opts'),
   						'desc' => __('Image replaces icon selected above', 'mfn-opts'),
							'preview' => 'image',
   					),

   					array(
   						'id' => 'title',
   						'type' => 'text',
   						'dynamic_data' => 'title',
   						'title' => __('Title', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h4',
						),

   					array(
   						'id' => 'content',
   						'edit_tag' => '.list_right | div.desc',
   						'edit_position' => 'append',
   						'type' => 'textarea',
   						'dynamic_data' => 'content',
   						'title' => __('Content', 'mfn-opts'),
							'std' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut elit tellus, luctus nec ullamcorper mattis, pulvinar dapibus leo.',
							'preview' => 'content',
							'class' => 'form-content-full-width',
   					),

						array(
   						'id' => 'style',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Style', 'mfn-opts'),
   						'options' => array(
   							1 => __('With bg', 'mfn-opts'),
   							2 => __('Transparent', 'mfn-opts'),
   							3 => __('Vertical', 'mfn-opts'),
   							4 => __('Ordered list', 'mfn-opts'),
   						),
							'std' => 1,
   					),

						// link

   					array(
   						'type' => 'header',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link',
   						'dynamic_data' => 'permalink',
   						'type' => 'text',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'target',
   						'type' => 'switch',
   						'title' => __('Target', 'mfn-opts'),
   						'options' => array(
   							0 => __('_self', 'mfn-opts'),
   							1 => __('_blank', 'mfn-opts'),
   						),
							'std' => 0,
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .list_right .title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .list_right .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .list_right .desc:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .list_right .desc:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Icon', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .list_left i,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .circle:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .list_left,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .circle:background',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .list_left i,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .circle:font-size',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .list_left i,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .circle:font-size_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .list_left i,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .circle:font-size_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .list_left i,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .circle:font-size_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .list_left,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .circle:border-style',
							'attr_id' => 'border_style_llc',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .list_left,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .circle:border-color',
							'condition' => array( 'id' => 'border_style_llc', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .list_left,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .circle:border-width',
		  				'condition' => array( 'id' => 'border_style_llc', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .list_left,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .circle:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_llc', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .list_left,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .circle:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_llc', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .list_left,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .circle:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_llc', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .list_left,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .circle:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .list_left,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .circle:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .list_left,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .circle:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .list_left,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .circle:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

				// Live search -----------------------------------------------------------

				'livesearch' => array(

					'type' => 'livesearch',
					'title' => __('Live search', 'mfn-opts'),
					'desc' => __('Display search form and browse your item live', 'mfn-opts'),
					'size' => '1/3',
					'cat' => '',

					'attr' => array(
					//'fields' => array(

						array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'id' => 'item_info',
							'class' => 'mfn-alert',
							'type' => 'info',
							'title' => __('You can only use one <b>Live search</b> element per page', 'mfn-opts'),
						),

						array(
							'id' => 'min_characters',
							'type' => 'text',
							'title' => __('Minimal characters', 'mfn-opts'),
							'desc' => __('Minimal amount of characters in input to load posts', 'mfn-opts'),
							'std' => '3',
							'param' => 'number',
							'class' => 'narrow',
							'after' => __('characters', 'mfn-opts'),
						),

						array(
							'id' => 'container_height',
							'type' => 'text',
							'title' => __('Search results container height', 'mfn-opts'),
							'std' => '300',
							'param' => 'number',
							'class' => 'narrow',
							'after' => __('px', 'mfn-opts'),
						),

						array(
							'id' => 'featured_image',
							'type' => 'switch',
							'title' => __('Featured image', 'mfn-opts'),
							'options' => array(
								'0' => __('Hide', 'mfn-opts'),
								'1' => __('Show', 'mfn-opts'),
							),
							'std' => '1',
						),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						)
					),
				),

   			// Map Basic ------------------------------------------------------------

   			'map_basic' => array(
   				'type' => 'map_basic',
   				'title' => __('Map Basic', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'elements',
   				'attr' => array(
   				//'fields' => array(

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'id' => 'info-iframe',
							'class' => 'mfn-alert',
							'type' => 'info',
							'title' => __('Number of <b>iframe map</b> loads is unlimited.', 'mfn-opts'),
						),

						array(
							'type' => 'header',
   						'title' => __('Iframe', 'mfn-opts'),
   					),

   					array(
   						'id' => 'iframe',
   						'type' => 'textarea',
   						're_render' => true,
   						'title' => __('Iframe', 'mfn-opts'),
   						'desc' => __('Visit <a target="_blank" href="https://google.com/maps">Google Maps</a> and follow these instructions:<br />1. Find place. 2. Click the share button in the left panel. 3. Select "embed a map" 4. Choose size. 5. Click "copy HTML" and paste it above', 'mfn-opts'),
   					),

   					// embed

						array(
							'id' => 'info-embed',
							'class' => 'mfn-alert',
							'type' => 'info',
							'title' => __('Number of <b>embed map</b> loads is unlimited. Google Maps API key is required.', 'mfn-opts'),
							'label' => __('Enter key', 'mfn-opts'),
							'link' => 'admin.php?page='. apply_filters('betheme_slug','be'). '-options#advanced&options',
						),

   					array(
   						'type' => 'header',
   						'title' => __('Embed', 'mfn-opts'),
   					),

   					array(
   						'id' => 'address',
   						'type' => 'text',
   						're_render' => true,
   						'title' => __('Address or place name', 'mfn-opts'),
   					),

   					array(
   						'id' => 'zoom',
   						'type' => 'text',
   						're_render' => true,
   						'title' => __('Zoom', 'mfn-opts'),
   						'param' => 'number',
   						'class' => 'narrow',
   						'std' => 13,
   					),

   					array(
   						'id' => 'height',
   						'type' => 'text',
   						're_render' => true,
   						'title' => __('Height', 'mfn-opts'),
							'param' => 'number',
   						'class' => 'narrow',
   						'after' => 'px',
   						'std' => 300,
   					),

   					array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Map Advanced ------------------------------------------------------------

   			'map' => array(
   				'type' => 'map',
   				'title' => __('Map Advanced', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/4',
   				'tablet_resized' => '0',
   				'cat' => 'elements',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'info-key',
   						'class' => 'mfn-alert',
   						'type' => 'info',
   						'title' => __('Number of free dynamic map loads is limited. Google Maps API key is required.', 'mfn-opts'),
							'label' => __('Enter key', 'mfn-opts'),
							'link' => 'admin.php?page='. apply_filters('betheme_slug','be'). '-options#advanced&options',
   					),

   					array(
   						'id' => 'lat',
   						'type' => 'text',
   						're_render' => true,
   						'title' => __('Latitude', 'mfn-opts'),
   						'before' => 'LAT',
   						'placeholder' => '-33.87',
   						'std' => '-33.87',
   					),

   					array(
   						'id' => 'lng',
   						'type' => 'text',
   						're_render' => true,
   						'title' => __('Longitude', 'mfn-opts'),
							'before' => 'LNG',
							'placeholder' => '151.21',
							'std' => '151.21',
   					),

            array(
   						'id' => 'info_window',
   						'type' => 'textarea',
   						're_render' => true,
   						'title' => __('Info window (optional)', 'mfn-opts'),
							'desc' => __('Additional information that would appear in Info window on marker click', 'mfn-opts'),
   					),

   					array(
   						'id' => 'zoom',
   						'type' => 'text',
   						're_render' => true,
   						'title' => __('Zoom', 'mfn-opts'),
   						'param' => 'number',
   						'class' => 'narrow',
   						'std' => 13,
   					),

   					// options

   					array(
   						'type' => 'header',
   						'title' => __('Options', 'mfn-opts'),
   					),

   					array(
   						'id' => 'type',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Type', 'mfn-opts'),
   						'options' => array(
   							'ROADMAP' => __('Map', 'mfn-opts'),
   							'SATELLITE' => __('Satellite', 'mfn-opts'),
   							'HYBRID' => __('Satellite + Map', 'mfn-opts'),
   							'TERRAIN' => __('Terrain', 'mfn-opts'),
   						),
							'std' => 'ROADMAP',
   					),

   					array(
   						'id' => 'controls',
   						'type' => 'select',
   						're_render' => true,
   						'title' => __('Controls', 'mfn-opts'),
   						'options' => array(
   							'' => __('Zoom', 'mfn-opts'),
   							'mapType' => __('Map Type', 'mfn-opts'),
   							'streetView' => __('Street View', 'mfn-opts'),
   							'zoom mapType' => __('Zoom & Map Type', 'mfn-opts'),
   							'zoom streetView' => __('Zoom & Street View', 'mfn-opts'),
   							'mapType streetView' => __('Map Type & Street View', 'mfn-opts'),
   							'zoom mapType streetView' => __('Zoom, Map Type & Street View', 'mfn-opts'),
   							'hide' => __('Hide All', 'mfn-opts'),
   						),
   					),

   					array(
   						'id' => 'draggable',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Draggable', 'mfn-opts'),
   						'options' => array(
   							'disable' => __('Disable', 'mfn-opts'),
   							'disable-mobile'=> __('Disable on mobile', 'mfn-opts'),
								'' => __('Enable', 'mfn-opts'),
   						),
							'std' => '',
   					),

   					array(
   						'id' => 'border',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Border', 'mfn-opts'),
   						'options' => array(
   							0 => __('Hide', 'mfn-opts'),
   							1 => __('Show', 'mfn-opts'),
   						),
							'std' => 0,
   					),

   					// advanced

   					array(
   						'type' => 'header',
   						'title' => __('Advanced', 'mfn-opts'),
   					),

   					array(
   						'id' => 'icon',
   						're_render' => true,
   						'type' => 'upload',
   						'title' => __('Marker icon', 'mfn-opts'),
   						'desc' => __('.png', 'mfn-opts'),
   					),

   					array(
   						'id' => 'color',
   						're_render' => true,
   						'type' => 'color',
   						'title' => __('Map color', 'mfn-opts'),
   					),

   					array(
   						'id' => 'styles',
   						're_render' => true,
   						'type' => 'textarea',
   						'title' => __('Styles', 'mfn-opts'),
   						'desc' => __('You can get predefined styles from <a target="_blank" href="https://snazzymaps.com/explore">snazzymaps.com/explore</a> or generate your own <a target="_blank" href="https://snazzymaps.com/editor">snazzymaps.com/editor</a>', 'mfn-opts'),
							'class' => 'form-content-full-width',
   					),

						// additional markers

   					array(
   						'type' => 'header',
   						'title' => __('Additional markers', 'mfn-opts'),
   					),

						array(
   						'id' => 'tabs',
   						'type' => 'tabs',
   						'title' => __('Markers', 'mfn-opts'),
							'options' => [
								// Add Title
								'title' => [
									'input',
									__('Title (optional)', 'mfn-opts'),
									'',
								],
								'lat' => [
									'input',
									__('Lat', 'mfn-opts'),
									'-33.88',
								],
								'lng' => [
									'input',
									__('Lng', 'mfn-opts'),
									'151.21',
								],
								'icon' => [
									'input',
									__('Icon URL (optional)', 'mfn-opts'),
									'',
								],
								'content' => [
									'textarea',
									__('Info window (optional)', 'mfn-opts'),
									'',
								],
							],
							'std' => [],
							'primary' => 'title',
							'secondary' => 'lat',
   					),

   					// contact

   					array(
   						'type' => 'header',
   						'title' => __('Contact box', 'mfn-opts'),
   					),

   					array(
   						'id' => 'title',
   						're_render' => true,
   						'type' => 'text',
   						'title' => __('Title', 'mfn-opts'),
   					),

   					array(
   						'id' => 'content',
   						're_render' => true,
   						'type' => 'textarea',
   						'title' => __('Address', 'mfn-opts'),
   						'desc' => __('HTML tags allowed', 'mfn-opts'),
   					),

   					array(
   						'id' => 'telephone',
   						're_render' => true,
   						'type' => 'text',
   						'title' => __('Telephone', 'mfn-opts'),
   					),

   					array(
   						'id' => 'email',
   						're_render' => true,
   						'type' => 'text',
   						'title' => __('Email', 'mfn-opts'),
   					),

   					array(
   						'id' => 'www',
   						're_render' => true,
   						'type' => 'text',
   						'title' => __('WWW', 'mfn-opts'),
   					),

   					array(
   						'id' => 'style',
   						're_render' => true,
   						'type' => 'select',
   						'title' => __('Style', 'mfn-opts'),
   						'options' => array(
   							'box' => __('Box on the map (for full width column/wrap)', 'mfn-opts'),
   							'bar' => __('Bar at the top', 'mfn-opts'),
   						),
   					),

						array(
   						'id' => 'info-pricing',
   						'class' => 'mfn-alert',
   						'type' => 'info',
   						'title' => __('If you need more than 28500 map loads per month please check current Google Maps Pricing or choose Map Basic instead.', 'mfn-opts'),
							'label' => __('Google Maps Pricing', 'mfn-opts'),
							'link' => 'https://cloud.google.com/maps-platform/pricing/',
   					),

						 // deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
   						'title' => __('Deprecated', 'mfn-opts'),
   					),

   					array(
   						'id' => 'height',
   						'type' => 'text',
   						'title' => __('Height', 'mfn-opts'),
							'param' => 'number',
   						'class' => 'narrow mfn-deprecated',
   						'after' => 'px',
   					),

   					array(
   						'id' => 'latlng',
   						'type' => 'text',
   						'title' => __('Lat,Lng,IconURL', 'mfn-opts'),
   						'class' => 'mfn-deprecated',
   					),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

   					array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Map', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner .google-map:height',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'title' => __('Height', 'mfn-opts')
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner .google-map:height_laptop',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'title' => __('Height', 'mfn-opts')
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner .google-map:height_tablet',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'title' => __('Height', 'mfn-opts')
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner .google-map:height_mobile',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'title' => __('Height', 'mfn-opts')
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Container', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .google-map-contact-wrapper .get_in_touch:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

   					array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .get_in_touch h3:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .get_in_touch h3:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .get_in_touch h3:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .get_in_touch h3:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .get_in_touch h3:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .get_in_touch h3:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Icon', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .get_in_touch ul li .icon:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('List', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .get_in_touch .get_in_touch_wrapper ul li:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .get_in_touch ul li a:color',
							'type' => 'color',
							'title' => __('Links color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .get_in_touch ul li a|hover:color',
							'type' => 'color',
							'title' => __('Links color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Lines', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .get_in_touch ul li:--mfn-contactbox-line',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Line color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Offer Slider Full ----------------------------------------------

   			'offer' => array(
   				'type' => 'offer',
   				'title' => __('Offer Slider Full', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'loops',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'info',
   						'type' => 'info',
   						'class' => 'mfn-alert',
   						'title' => __('Please use on pages <strong>without sidebar</strong> and within <strong>full width sections</strong>. One item per section.', 'mfn-opts'),
   					),

   					array(
   						'id' => 'category',
   						'type' => 'select',
   						're_render' => true,
   						'title' => __('Category', 'mfn-opts'),
   						'php_options' => mfn_get_categories('offer-types'),
   						'js_hierarchical_options' => 'offer_types',
							'preview' => 'category',
   					),

						// deprecated

   					array(
   						'type' => 'header',
   						'class' => 'mfn-deprecated',
   						'title' => __('Deprecated', 'mfn-opts'),
   					),

   					array(
   						'id' => 'align',
   						'type' => 'select',
   						'class' => 'mfn-deprecated',
   						'title' => __('Text align', 'mfn-opts'),
   						'desc' => __('Text align center does not affect title if button is active', 'mfn-opts'),
   						'options' => array(
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'std' => 'left',
   					),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Image', 'mfn-opts'),
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .image_wrapper:border-style',
							'attr_id' => 'border_style_os_img',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .image_wrapper:border-color',
							'condition' => array( 'id' => 'border_style_os_img', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .image_wrapper:border-width',
		  				'condition' => array( 'id' => 'border_style_os_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .image_wrapper:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_os_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .image_wrapper:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_os_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .image_wrapper:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_os_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .image_wrapper:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .image_wrapper:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .image_wrapper:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .image_wrapper:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .desc_wrapper .title h3:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .desc_wrapper .title h3:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

						array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'std' => '',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'std' => '',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'std' => '',
   					),
   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'std' => '',
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .desc_wrapper .desc:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .desc_wrapper .desc:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Nav', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .slider_pagination:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Label background', 'mfn-opts'),
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .slider_pagination, .mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .slider_pagination span:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Label color', 'mfn-opts'),
						),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .slick-arrow:background-color',
							'type' => 'color',
							'title' => __('Arrows background', 'mfn-opts'),
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .slick-arrow .button_icon i:color',
							'type' => 'color',
							'title' => __('Arrows color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .slick-arrow|hover:background-color',
							'type' => 'color',
							'title' => __('Arrows background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .slick-arrow|hover .button_icon i:color',
							'type' => 'color',
							'title' => __('Arrows color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Button', 'mfn-opts'),
   					),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .desc_wrapper .title a.button:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .desc_wrapper .title a.button:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .desc_wrapper .title a.button:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .desc_wrapper .title a.button:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .desc_wrapper .title a.button:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .desc_wrapper .title a.button:border-style',
							'attr_id' => 'border_style_osf_border',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .desc_wrapper .title a.button:border-width',
		  				'condition' => array( 'id' => 'border_style_osf_border', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .desc_wrapper .title a.button:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_osf_border', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .desc_wrapper .title a.button:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_osf_border', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .desc_wrapper .title a.button:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_osf_border', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .desc_wrapper .title a.button:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .desc_wrapper .title a.button:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .desc_wrapper .title a.button:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .desc_wrapper .title a.button:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .desc_wrapper .title a.button:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .desc_wrapper .title a.button:border-color',
							'condition' => array( 'id' => 'border_style_osf_border', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher',
							'attr_id' => 'background_switcher_osf',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .desc_wrapper .title a.button:background-color',
							'condition' => array( 'id' => 'background_switcher_osf', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .desc_wrapper .title a.button:gradient',
		  				'condition' => array( 'id' => 'background_switcher_osf', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .desc_wrapper .title a.button|hover:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .desc_wrapper .title a.button|hover:border-color',
							'condition' => array( 'id' => 'border_style_osf_border', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher_hover',
							'attr_id' => 'background_switcher_hover_osf',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .desc_wrapper .title a.button|hover:background',
							'condition' => array( 'id' => 'background_switcher_hover_osf', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer .offer_li .desc_wrapper .title a.button|hover:gradient',
		  				'condition' => array( 'id' => 'background_switcher_hover_osf', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Offer Slider Thumb ---------------------------------------------

   			'offer_thumb' => array(
   				'type' => 'offer_thumb',
   				'title' => __('Offer Slider Thumb', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'loops',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'category',
   						're_render' => true,
   						'type' => 'select',
   						'title' => __('Category', 'mfn-opts'),
   						'php_options' => mfn_get_categories('offer-types'),
   						'js_hierarchical_options' => 'offer_types',
							'preview' => 'category',
   					),

   					array(
   						'id' => 'style',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Thumbnails position', 'mfn-opts'),
   						'options' => array(
   							'bottom' => __('Bottom', 'mfn-opts'),
   							'' => __('Left', 'mfn-opts'),
   						),
   						'std' => 'bottom',
   					),

						array(
							'type' => 'header',
   						'class' => 'mfn-deprecated',
   						'title' => __('Deprecated', 'mfn-opts'),
   					),

   					array(
   						'id' => 'align',
   						'type' => 'switch',
							'class' => 'mfn-deprecated',
   						'title' => __('Text align', 'mfn-opts'),
   						'desc' => __('Text align center does not affect title if button is active', 'mfn-opts'),
   						'options' => array(
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'std' => 'left',
   					),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Image', 'mfn-opts'),
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .image_wrapper:border-style',
							'attr_id' => 'border_style_os_img',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .image_wrapper:border-color',
							'condition' => array( 'id' => 'border_style_os_img', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .image_wrapper:border-width',
		  				'condition' => array( 'id' => 'border_style_os_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .image_wrapper:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_os_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .image_wrapper:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_os_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .image_wrapper:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_os_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .image_wrapper:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .image_wrapper:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .image_wrapper:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .image_wrapper:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .desc_wrapper .title h3:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .desc_wrapper .title h3:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

						array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'std' => '',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'std' => '',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'std' => '',
   					),
   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc_wrapper:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'std' => '',
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .desc_wrapper .desc:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .desc_wrapper .desc:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Nav', 'mfn-opts'),
   					),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .slider_pagination li a:background-color',
							'type' => 'color',
							'title' => __('Background', 'mfn-opts'),
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .slider_pagination li a:border-color',
							'type' => 'color',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .slider_pagination li.slick-active a:background-color',
							'type' => 'color',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .slider_pagination li.slick-active a:border-color',
							'type' => 'color',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Button', 'mfn-opts'),
   					),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .desc_wrapper .title a.button:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .desc_wrapper .title a.button:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .desc_wrapper .title a.button:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .desc_wrapper .title a.button:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .desc_wrapper .title a.button:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .desc_wrapper .title a.button:border-style',
							'attr_id' => 'border_style_osf_border',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .desc_wrapper .title a.button:border-width',
		  				'condition' => array( 'id' => 'border_style_osf_border', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .desc_wrapper .title a.button:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_osf_border', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .desc_wrapper .title a.button:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_osf_border', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .desc_wrapper .title a.button:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_osf_border', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .desc_wrapper .title a.button:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .desc_wrapper .title a.button:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .desc_wrapper .title a.button:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .desc_wrapper .title a.button:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .desc_wrapper .title a.button:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .desc_wrapper .title a.button:border-color',
							'condition' => array( 'id' => 'border_style_osf_border', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher',
							'attr_id' => 'background_switcher_osf',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .desc_wrapper .title a.button:background-color',
							'condition' => array( 'id' => 'background_switcher_osf', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .desc_wrapper .title a.button:gradient',
		  				'condition' => array( 'id' => 'background_switcher_osf', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .desc_wrapper .title a.button|hover:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .desc_wrapper .title a.button|hover:border-color',
							'condition' => array( 'id' => 'border_style_osf_border', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher_hover',
							'attr_id' => 'background_switcher_hover_osf',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .desc_wrapper .title a.button|hover:background',
							'condition' => array( 'id' => 'background_switcher_hover_osf', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .offer_thumb_li .desc_wrapper .title a.button|hover:gradient',
		  				'condition' => array( 'id' => 'background_switcher_hover_osf', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'themeoptions' => 'style:',
   						'title' => __('Decoration', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .offer_thumb .slider_pagination li:--mfn-offer-thumbs-nav',
							'themeoptions' => 'style:',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Active', 'mfn-opts'),
						),


						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Opening Hours --------------------------------------------------

   			'opening_hours' => array(
   				'type' => 'opening_hours',
   				'title' => __('Opening Hours', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'elements',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'title',
   						'edit_tag' => '.opening_hours | .title',
   						'edit_position' => 'prepend',
   						'type' => 'text',
						'dynamic_data' => 'title',
   						'title' => __('Title', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h3',
						),

						array(
   						'id' => 'tabs',
   						'type' => 'tabs',
   						'title' => __('Days', 'mfn-opts'),
							'options' => [
								'days' => [
									'input',
									__('Days', 'mfn-opts'),
									'Monday - Friday',
								],
								'hours' => [
									'input',
									__('Hours', 'mfn-opts'),
									'8am - 4pm',
								],
							],
							'std' => [
								0 => [
									'days' => 'Monday - Friday',
									'hours' => '8am - 4pm',
								],
								1 => [
									'days' => 'Saturday',
									'hours' => '10am - 2pm',
								],
							],
							'primary' => 'days',
							'preview' => 'tabs',
   					),

   					array(
   						'id' => 'content',
   						'type' => 'textarea',
   						'edit_tag' => '.opening_hours_wrapper | span.ohw-desc',
   						'edit_position' => 'prepend',
   						'dynamic_data' => 'content',
   						'title' => __('Content', 'mfn-opts'),
							'preview' => 'content',
							'class' => 'form-content-full-width',
							'std' => __('Vitae adipiscing turpis. Aenean ligula nibh, molestie id viverra a, dapibus at dolor.', 'mfn-opts')
   					),

   					array(
   						'type' => 'header',
							'class' => 'mfn-deprecated',
   						'title' => __('Deprecated', 'mfn-opts'),
   					),

   					array(
   						'id' => 'image',
   						'class' => 'mfn-deprecated',
   						'type' => 'upload',
   						'title' => __('Background image', 'mfn-opts'),
   						'desc' => __('Recommended image width <b>768px - 1920px</b> depending on size of the item', 'mfn-opts'),
   					),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
		  				'type' => 'header',
		  				'class' => 'toggled_header row-header',
   						'title' => __('Container', 'mfn-opts'),
   					),

						array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),
   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
		  				'type' => 'header',
		  				'class' => 'toggled_header row-header',
   						'title' => __('Row', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li:border-style',
							'attr_id' => 'border_style_oh_row',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li:border-width',
		  				'condition' => array( 'id' => 'border_style_oh_row', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_oh_row', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_oh_row', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_oh_row', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li:border-color',
							'condition' => array( 'id' => 'border_style_oh_row', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

   					array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

   					array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

						array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),
   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .title:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours_wrapper .ohw-desc:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours_wrapper .ohw-desc:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours_wrapper .ohw-desc:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours_wrapper .ohw-desc:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours_wrapper .ohw-desc:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours_wrapper .ohw-desc:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Days', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li label:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li label:background',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li label:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li label:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li label:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li label:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li label:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li label:border-style',
							'attr_id' => 'border_style_oh',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li label:border-width',
		  				'condition' => array( 'id' => 'border_style_oh', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li label:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_oh', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li label:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_oh', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li label:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_oh', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li label:border-color',
							'condition' => array( 'id' => 'border_style_oh', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li label:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li label:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li label:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li label:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Hours', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li span:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .opening_hours .opening_hours_wrapper li span:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Our team -------------------------------------------------------

   			'our_team' => array(
   				'type' => 'our_team',
   				'title' => __('Our Team', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'elements',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'heading',
						'dynamic_data' => 'title',
   						'type' => 'text',
   						'edit_tag' => '.team | .title',
   						'edit_position' => 'prepend',
   						'title' => __('Heading', 'mfn-opts'),
							'std' => __('This is the heading', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
							'id' => 'heading_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Heading tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h4',
						),

   					array(
   						'id' => 'image',
   						'dynamic_data' => 'featured_image',
   						'type' => 'upload',
   						'title' => __('Photo', 'mfn-opts'),
   						'desc' => __('Recommended image width <b>768px - 1920px</b> depending on size of the item', 'mfn-opts'),
							'std' => $this->get_placeholder(),
							'preview' => 'image',
   					),

   					array(
   						'id' => 'title',
						'dynamic_data' => 'title',
   						'type' => 'text',
   						'edit_tag' => '.desc_wrapper | .desc_wrappper_title',
   						'edit_position' => 'prepend',
   						'title' => __('Title', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'subtitle',
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h4',
						),

   					array(
   						'id' => 'subtitle',
   						'type' => 'text',
   						'edit_tag' => '.desc_wrapper | p.subtitle',
   						'edit_position' => 'prepend',
   						'title' => __('Subtitle', 'mfn-opts'),
							'std' => __('This is the subtitle', 'mfn-opts'),
   					),

   					// description

   					array(
   						'type' => 'header',
   						'title' => __('Description', 'mfn-opts'),
   					),

   					array(
   						'id' => 'phone',
   						'type' => 'text',
   						'title' => __('Phone', 'mfn-opts'),
   					),

   					array(
   						'id' => 'content',
   						'type' => 'textarea',
   						'dynamic_data' => 'content',
   						'edit_tag' => '.desc_wrapper | div.desc',
   						'edit_position' => 'append',
   						'title' => __('Content', 'mfn-opts'),
   						'desc' => __('Some shortcodes and HTML tags allowed', 'mfn-opts'),
							'class' => 'form-content-full-width',
   					),

   					array(
   						'id' => 'email',
   						'type' => 'text',
   						'title' => __('Email', 'mfn-opts'),
   					),

   					// social

   					array(
   						'type' => 'header',
   						'title' => __('Social', 'mfn-opts'),
   					),

   					array(
   						'id' => 'facebook',
   						'type' => 'text',
   						'title' => __('Facebook', 'mfn-opts'),
   					),

   					array(
   						'id' => 'twitter',
   						'type' => 'text',
   						'title' => __('Twitter', 'mfn-opts'),
   					),

   					array(
   						'id' => 'linkedin',
   						'type' => 'text',
   						'title' => __('LinkedIn', 'mfn-opts'),
   					),

   					array(
   						'id' => 'vcard',
   						'type' => 'text',
   						'title' => __('vCard', 'mfn-opts'),
   					),

   					// other

   					array(
   						'type' => 'header',
   						'title' => __('Other', 'mfn-opts'),
   					),

   					array(
   						'id' => 'blockquote',
   						'type' => 'textarea',
   						'title' => __('Blockquote', 'mfn-opts'),
							'class' => 'form-content-full-width',
   					),

   					array(
   						'id' => 'style',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Style', 'mfn-opts'),
   						'options' => array(
   							'circle' => __('Circle', 'mfn-opts'),
   							'vertical' => __('Vertical', 'mfn-opts'),
   							'horizontal' => __('Horizontal', 'mfn-opts'),
   						),
   						'std' => 'vertical',
   					),

   					// link

   					array(
   						'type' => 'header',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link_type',
   						'attr_id' => 'link_type',
   						'type' => 'select',
							'title' => __('On click action', 'mfn-opts'),
   						'options' => array(
								'' => __('Default', 'mfn-opts'),
								'1' => __('Open popup', 'mfn-opts'),
							),
							'std' => '',
   					),

   					array(
   						'id' => 'popup_id',
   						'condition' => array( 'id' => 'link_type', 'opt' => 'is', 'val' => '1' ),
   						'type' => 'text',
   						'title' => __('Popup ID', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link',
   						'type' => 'text',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'target',
   						'type' => 'select',
   						'title' => __('Target', 'mfn-opts'),
   						'options' => array(
   							0 => __('Default | _self', 'mfn-opts'),
   							1 => __('New tab or window | _blank', 'mfn-opts'),
   							'lightbox' => __('Lightbox (image or embed video)', 'mfn-opts'),
   						),
   					),

            array(
							'id' => 'link_title',
							'type' => 'text',
							'title' => __('Link title', 'mfn-opts'),
						),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

   					array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Heading', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Image', 'mfn-opts'),
   					),

   					array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .image_frame:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .image_frame:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .image_frame:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .image_frame:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .image_frame:border-style',
							'attr_id' => 'border_style_os_img',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .image_frame:border-color',
							'condition' => array( 'id' => 'border_style_os_img', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .image_frame:border-width',
		  				'condition' => array( 'id' => 'border_style_os_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .image_frame:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_os_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .image_frame:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_os_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .image_frame:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_os_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .image_frame:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .image_frame:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .image_frame:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .image_frame:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Desc', 'mfn-opts'),
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),
   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper:border-style',
							'attr_id' => 'border_style_os_img',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper:border-color',
							'condition' => array( 'id' => 'border_style_os_img', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper:border-width',
		  				'condition' => array( 'id' => 'border_style_os_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_os_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_os_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_os_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .desc_wrappper_title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .desc_wrappper_title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Subtitle', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .subtitle:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .subtitle:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Phone', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .phone a:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .phone i:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Icon color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .phone:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .desc:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .desc:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Blockquote', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper blockquote:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper blockquote:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Blockquote icon', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .mfn-blockquote-icon:color',
							'type' => 'color',
							'themeoptions' => 'style:',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .mfn-blockquote-icon:font-size',
							'type' => 'sliderbar',
							'themeoptions' => 'style:',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .mfn-blockquote-icon:font-size_laptop',
							'type' => 'sliderbar',
							'themeoptions' => 'style:',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .mfn-blockquote-icon:font-size_tablet',
							'type' => 'sliderbar',
							'themeoptions' => 'style:',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .mfn-blockquote-icon:font-size_mobile',
							'type' => 'sliderbar',
							'themeoptions' => 'style:',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Blockquote lines', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper blockquote:text-decoration-color',
							'type' => 'color',
							'themeoptions' => 'style:',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper blockquote:text-underline-offset',
							'type' => 'text',
							'themeoptions' => 'style:',
							'class' => 'mfn-toggled',
							'default_unit' => 'px',
							'title' => __('Offset', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper blockquote:text-decoration-thickness',
							'type' => 'text',
							'themeoptions' => 'style:',
							'class' => 'mfn-toggled',
							'default_unit' => 'px',
							'title' => __('Thickness', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Icons', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .links a:border-style',
							'attr_id' => 'border_style_ot_icon',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .links a:border-width',
		  				'condition' => array( 'id' => 'border_style_ot_icon', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .links a:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_ot_icon', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .links a:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_ot_icon', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .links a:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_ot_icon', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .links a:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .links a:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .links a:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .links a:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .links a span i:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .links a:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .links a:border-color',
							'condition' => array( 'id' => 'border_style_ot_icon', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .links a|hover span.b i:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .links a|hover:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .links a|hover:border-color',
							'condition' => array( 'id' => 'border_style_ot_icon', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Divider', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper hr:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper hr:width',
							'type' => 'text',
							'default_unit' => 'px',
							'class' => 'mfn-toggled',
							'title' => __('Width', 'mfn-opts'),
						),

   					array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper hr:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper hr:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper hr:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper hr:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Our team list --------------------------------------------------

   			'our_team_list' => array(
   				'type' => 'our_team_list',
   				'title' => __('Our Team List', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'elements',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'image',
   						'dynamic_data' => 'featured_image',
   						'type' => 'upload',
   						'title' => __('Photo', 'mfn-opts'),
   						'desc' => __('Recommended minimum image width <b>768px</b>', 'mfn-opts'),
							'std' => $this->get_placeholder(),
							'preview' => 'image',
   					),

   					array(
   						'id' => 'title',
   						'type' => 'text',
						'dynamic_data' => 'title',
   						'edit_tag' => '.desc_wrapper | .title',
   						'edit_position' => 'prepend',
   						'title' => __('Title', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h4',
						),

   					array(
   						'id' => 'subtitle',
   						'edit_tag' => '.desc_wrapper | p.subtitle',
   						'edit_position' => 'append',
   						'type' => 'text',
   						'title' => __('Subtitle', 'mfn-opts'),
							'std' => __('This is the subtitle', 'mfn-opts'),
							'preview' => 'subtitle',
   					),

   					// description

   					array(
   						'type' => 'header',
   						'title' => __('Description', 'mfn-opts'),
   					),

   					array(
   						'id' => 'phone',
   						'type' => 'text',
   						'title' => __('Phone', 'mfn-opts'),
   					),

   					array(
   						'id' => 'content',
   						'type' => 'textarea',
   						'dynamic_data' => 'content',
   						'edit_tag' => '.desc_wrapper | div.desc',
   						'edit_position' => 'append',
   						'title' => __('Content', 'mfn-opts'),
   						'desc' => __('Some shortcodes and HTML tags allowed', 'mfn-opts'),
							'preview' => 'content',
							'class' => 'form-content-full-width',
   					),

   					array(
   						'id' => 'blockquote',
   						'type' => 'textarea',
   						'title' => __('Blockquote', 'mfn-opts'),
							'class' => 'form-content-full-width',
   					),

   					array(
   						'id' => 'email',
   						'type' => 'text',
   						'title' => __('Email', 'mfn-opts'),
   					),

   					// social

   					array(
   						'type' => 'header',
   						'title' => __('Social', 'mfn-opts'),
   					),

   					array(
   						'id' => 'facebook',
   						'type' => 'text',
   						'title' => __('Facebook', 'mfn-opts'),
   					),

   					array(
   						'id' => 'twitter',
   						'type' => 'text',
   						'title' => __('Twitter', 'mfn-opts'),
   					),

   					array(
   						'id' => 'linkedin',
   						'type' => 'text',
   						'title' => __('LinkedIn', 'mfn-opts'),
   					),

   					array(
   						'id' => 'vcard',
   						'type' => 'text',
   						'title' => __('vCard', 'mfn-opts'),
   					),

   					// link

   					array(
   						'type' => 'header',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link',
   						'type' => 'text',
   						'dynamic_data' => 'permalink',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'target',
   						'type' => 'select',
   						'title' => __('Target', 'mfn-opts'),
   						'options' => array(
   							0 => __('Default | _self', 'mfn-opts'),
   							1 => __('New tab or window | _blank', 'mfn-opts'),
   							'lightbox' => __('Lightbox (image or embed video)', 'mfn-opts'),
   						),
   					),

            array(
							'id' => 'link_title',
							'type' => 'text',
							'dynamic_data' => 'title',
							'title' => __('Link title', 'mfn-opts'),
						),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Image', 'mfn-opts'),
   					),

   					array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .image_frame:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .image_frame:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .image_frame:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .image_frame:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .image_frame:border-style',
							'attr_id' => 'border_style_team_list_img',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .image_frame:border-color',
							'condition' => array( 'id' => 'border_style_team_list_img', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .image_frame:border-width',
		  				'condition' => array( 'id' => 'border_style_team_list_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .image_frame:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_team_list_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .image_frame:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_team_list_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .image_frame:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_team_list_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .image_frame:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .image_frame:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .image_frame:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .image_frame:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Right content', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .bq_wrapper:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .bq_wrapper:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .bq_wrapper:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .bq_wrapper:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .bq_wrapper:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .bq_wrapper:border-style',
							'attr_id' => 'border_style_team_list_desc',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .bq_wrapper:border-color',
							'condition' => array( 'id' => 'border_style_team_list_desc', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .bq_wrapper:border-width',
		  				'condition' => array( 'id' => 'border_style_team_list_desc', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .bq_wrapper:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_team_list_desc', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .bq_wrapper:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_team_list_desc', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .bq_wrapper:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_team_list_desc', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .bq_wrapper:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .bq_wrapper:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .bq_wrapper:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .bq_wrapper:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Subtitle', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .subtitle:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .subtitle:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Phone', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .phone a:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .phone i:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Icon color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .phone:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .desc:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .desc_wrapper .desc:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Blockquote', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team blockquote:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team blockquote:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'themeoptions' => 'style:',
   						'title' => __('Blockquote icon', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .mfn-blockquote-icon:color',
							'type' => 'color',
							'themeoptions' => 'style:',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .mfn-blockquote-icon:font-size',
							'type' => 'sliderbar',
							'themeoptions' => 'style:',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .mfn-blockquote-icon:font-size_laptop',
							'type' => 'sliderbar',
							'themeoptions' => 'style:',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .mfn-blockquote-icon:font-size_tablet',
							'type' => 'sliderbar',
							'themeoptions' => 'style:',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .mfn-blockquote-icon:font-size_mobile',
							'type' => 'sliderbar',
							'themeoptions' => 'style:',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled sliderbar-units',
							'title' => __('Size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Blockquote lines', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team blockquote:text-decoration-color',
							'type' => 'color',
							'themeoptions' => 'style:',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team blockquote:text-underline-offset',
							'type' => 'text',
							'themeoptions' => 'style:',
							'class' => 'mfn-toggled',
							'default_unit' => 'px',
							'title' => __('Offset', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team blockquote:text-decoration-thickness',
							'type' => 'text',
							'themeoptions' => 'style:',
							'class' => 'mfn-toggled',
							'default_unit' => 'px',
							'title' => __('Thickness', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Icons', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .links a:border-style',
							'attr_id' => 'border_style_team_list_icon',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .links a:border-width',
		  				'condition' => array( 'id' => 'border_style_team_list_icon', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .links a:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_team_list_icon', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .links a:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_team_list_icon', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .links a:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_team_list_icon', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .links a:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .links a:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .links a:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .links a:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .links a span i:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .links a:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .links a:border-color',
							'condition' => array( 'id' => 'border_style_team_list_icon', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .links a|hover span.b i:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .links a|hover:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team .links a|hover:border-color',
							'condition' => array( 'id' => 'border_style_team_list_icon', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Divider', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team hr:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team hr:width',
							'type' => 'text',
							'default_unit' => 'px',
							'class' => 'mfn-toggled',
							'title' => __('Width', 'mfn-opts'),
						),

   					array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team hr:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team hr:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team hr:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .team hr:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

				// Lottie --------------------------------------------------

   			'lottie' => array(
   				'type' => 'lottie',
   				'title' => __('Lottie', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'elements',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

						array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'id' => 'source_switcher',
							'attr_id' => 'lottie_source_switcher',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher',
							'title' => __('Lottie Source', 'mfn-opts'),
							'options' => array(
		  					'file' => __('Media File', 'mfn-opts'),
		  					'url' => __('External URL', 'mfn-opts'),
		  				),
		  				'std' => 'file',
						),

						array(
   						'id' => 'file',
   						'condition' => array( 'id' => 'lottie_source_switcher', 'opt' => 'is', 'val' => 'file' ),
   						'type' => 'upload',
   						're_render' => true,
   						'title' => __('Lottie File', 'mfn-opts'),
   						'desc' => __('Upload self-hosted JSON file', 'mfn-opts'),
							'preview' => 'image',
   					),

   					array(
   						'id' => 'src',
   						'condition' => array( 'id' => 'lottie_source_switcher', 'opt' => 'is', 'val' => 'url' ),
   						'type' => 'text',
   						're_render' => true,
   						'title' => __('Lottie URL', 'mfn-opts'),
   						'desc' => __('Your Lottie JSON url', 'mfn-opts'),
   					),

   					array(
   						'type' => 'header',
   						'title' => __('Settings', 'mfn-opts'),
   					),

   					array(
   						'id' => 'trigger',
   						'attr_id' => 'lottie_trigger',
   						'type' => 'select',
   						're_render' => true,
   						'title' => __('Trigger', 'mfn-opts'),
   						'options' => array(
   							'default' => __('Default', 'mfn-opts'),
   							'hover' => __('On hover', 'mfn-opts'),
   							'click' => __('On click', 'mfn-opts'),
   							'scroll' => __('On scroll', 'mfn-opts'),
   							'viewport' => __('Viewport', 'mfn-opts'),
   						),
							'std' => 'default',
   					),

   					array(
   						'id' => 'loop',
   						'condition' => array( 'id' => 'lottie_trigger', 'opt' => 'isnt', 'val' => 'scroll' ),
   						'attr_id' => 'lottie_loop',
   						'type' => 'switch',
   						'title' => __('Loop', 'mfn-opts'),
   						'options' => array(
   							"0" => __('No', 'mfn-opts'),
   							"1" => __('Yes', 'mfn-opts'),
   						),
							'std' => "0",
   					),

   					array(
		  				'id' => 'speed',
		  				'type' => 'sliderbar',
							'title' => __('Speed', 'mfn-opts'),
							'std' => '1',
							'param' => array(
								'min' => '0.1',
								'max' => '5',
								'step' => '0.1',
							)
		  			),

		  			array(
		  				'id' => 'viewport',
		  				'condition' => array( 'id' => 'lottie_trigger', 'opt' => 'is', 'val' => 'viewport,scroll' ),
		  				'type' => 'sliderbar',
		  				're_render' => true,
							'title' => __('Viewport bottom offset', 'mfn-opts'),
							'std' => '10',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
							)
		  			),

		  			array(
		  				'id' => 'frame_start',
		  				'type' => 'sliderbar',
							'title' => __('Start point', 'mfn-opts'),
							'std' => '1',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
							)
		  			),

		  			array(
		  				'id' => 'frame_end',
		  				'type' => 'sliderbar',
							'title' => __('End point', 'mfn-opts'),
							'std' => '100',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
							)
		  			),

		  			array(
   						'id' => 'direction',
   						'condition' => array( 'id' => 'lottie_trigger', 'opt' => 'isnt', 'val' => 'scroll' ),
   						'type' => 'switch',
   						'title' => __('Direction', 'mfn-opts'),
   						'options' => array(
   							'1' => __('Forward', 'mfn-opts'),
   							'-1' => __('Backward', 'mfn-opts'),
   						),
							'std' => '1',
   					),

		  			array(
   						'id' => 'link',
   						'condition' => array( 'id' => 'lottie_trigger', 'opt' => 'isnt', 'val' => 'click' ),
   						'type' => 'text',
   						'placeholder' => __('Enter your custom URL', 'mfn-opts'),
   						'title' => __('Link', 'mfn-opts'),
   					),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
		  				'type' => 'header',
		  				'class' => 'toggled_header row-header',
   						'title' => __('Container', 'mfn-opts'),
   					),

						array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-lottie-wrapper:justify-content',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-lottie-wrapper:justify-content_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-lottie-wrapper:justify-content_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-lottie-wrapper:justify-content_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-lottie-wrapper .lottie:width',
							'responsive' => 'desktop',
   						'type' => 'text',
   						'default_unit' => 'px',
   						'class' => 'mfn-toggled',
   						'title' => __('Width', 'mfn-opts'),
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-lottie-wrapper .lottie:width_laptop',
							'responsive' => 'laptop',
   						'type' => 'text',
   						'default_unit' => 'px',
   						'class' => 'mfn-toggled',
   						'title' => __('Width', 'mfn-opts'),
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-lottie-wrapper .lottie:width_tablet',
							'responsive' => 'tablet',
   						'type' => 'text',
   						'default_unit' => 'px',
   						'class' => 'mfn-toggled',
   						'title' => __('Width', 'mfn-opts'),
   					),
   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-lottie-wrapper .lottie:width_mobile',
							'responsive' => 'mobile',
   						'type' => 'text',
   						'default_unit' => 'px',
   						'class' => 'mfn-toggled',
   						'title' => __('Width', 'mfn-opts'),
   					),


						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

				// Payments methods ----------------------------------------------------

   			'payment_methods' => array(
   				'type' => 'payment_methods',
   				'title' => __('Payment methods', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'other',
   				'attr' => array(
   				//'fields' => array(

						/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
   						'id' => 'dynamic_items',
   						'type' => 'dynamic_items',
   						'title' => __('Payment methods', 'mfn-opts'),
							'options' => array(
								'var' => array(
									'predefined' 	=> __('Predefined', 'mfn-opts'),
									'custom' 			=> __('Custom', 'mfn-opts')
								),
								'add_button' => false,
								'preview' => 'img',
								'options' => array(
									'predefined' => array(
										'type' => 'select-img',
										'label' => __('Predefined', 'mfn-opts'),
										'options' => array(
											'Affirm' => get_template_directory_uri().'/images/payment-methods/Affirm.svg',
					  					'Alipay' => get_template_directory_uri().'/images/payment-methods/Alipay.svg',
					  					'AmazonPay' => get_template_directory_uri().'/images/payment-methods/AmazonPay.svg',
					  					'Amex' => get_template_directory_uri().'/images/payment-methods/Amex.svg',
					  					'ApplePay' => get_template_directory_uri().'/images/payment-methods/ApplePay.svg',
					  					'Bancontact' => get_template_directory_uri().'/images/payment-methods/Bancontact.svg',
					  					'Bitcoin' => get_template_directory_uri().'/images/payment-methods/Bitcoin.svg',
					  					'BitcoinCash' => get_template_directory_uri().'/images/payment-methods/BitcoinCash.svg',
					  					'Bitpay' => get_template_directory_uri().'/images/payment-methods/Bitpay.svg',
					  					'Citadele' => get_template_directory_uri().'/images/payment-methods/Citadele.svg',
					  					'DinersClub' => get_template_directory_uri().'/images/payment-methods/DinersClub.svg',
					  					'Discover' => get_template_directory_uri().'/images/payment-methods/Discover.svg',
					  					'Elo' => get_template_directory_uri().'/images/payment-methods/Elo.svg',
					  					'Etherium' => get_template_directory_uri().'/images/payment-methods/Etherium.svg',
					  					'FacebookPay' => get_template_directory_uri().'/images/payment-methods/FacebookPay.svg',
					  					'Forbrugsforeningen' => get_template_directory_uri().'/images/payment-methods/Forbrugsforeningen.svg',
					  					'Giropay' => get_template_directory_uri().'/images/payment-methods/Giropay.svg',
					  					'GooglePay' => get_template_directory_uri().'/images/payment-methods/GooglePay.svg',
					  					'Ideal' => get_template_directory_uri().'/images/payment-methods/Ideal.svg',
					  					'Interac' => get_template_directory_uri().'/images/payment-methods/Interac.svg',
					  					'JCB' => get_template_directory_uri().'/images/payment-methods/JCB.svg',
					  					'Klarna' => get_template_directory_uri().'/images/payment-methods/Klarna.svg',
					  					'Lightcoin' => get_template_directory_uri().'/images/payment-methods/Lightcoin.svg',
					  					'Maestro' => get_template_directory_uri().'/images/payment-methods/Maestro.svg',
					  					'Mastercard' => get_template_directory_uri().'/images/payment-methods/Mastercard.svg',
					  					'PayPal' => get_template_directory_uri().'/images/payment-methods/PayPal.svg',
					  					'Payoneer' => get_template_directory_uri().'/images/payment-methods/Payoneer.svg',
					  					'Paysafe' => get_template_directory_uri().'/images/payment-methods/Paysafe.svg',
					  					'Poli' => get_template_directory_uri().'/images/payment-methods/Poli.svg',
					  					'Qiwi' => get_template_directory_uri().'/images/payment-methods/Qiwi.svg',
					  					'SEPA' => get_template_directory_uri().'/images/payment-methods/SEPA.svg',
					  					'ShopPay' => get_template_directory_uri().'/images/payment-methods/ShopPay.svg',
					  					'Skrill' => get_template_directory_uri().'/images/payment-methods/Skrill.svg',
					  					'Sofort' => get_template_directory_uri().'/images/payment-methods/Sofort.svg',
					  					'Stripe' => get_template_directory_uri().'/images/payment-methods/Stripe.svg',
					  					'UnionPay' => get_template_directory_uri().'/images/payment-methods/UnionPay.svg',
					  					'Venmo' => get_template_directory_uri().'/images/payment-methods/Venmo.svg',
					  					'Verifone' => get_template_directory_uri().'/images/payment-methods/Verifone.svg',
					  					'Visa' => get_template_directory_uri().'/images/payment-methods/Visa.svg',
					  					'WeChat' => get_template_directory_uri().'/images/payment-methods/WeChat.svg',
					  					'Webmoney' => get_template_directory_uri().'/images/payment-methods/Webmoney.svg',
					  					'Yandex' => get_template_directory_uri().'/images/payment-methods/Yandex.svg',
										)
									),
									'custom' => array(
										'type' => 'upload',
										'label' => __('Custom', 'mfn-opts'),
									)
								)
							),
   					),

						array(
   						'id' => 'greyscale',
   						'type' => 'switch',
   						'title' => __('Grayscale', 'mfn-opts'),
   						'options' => array(
   							0 => __('Disable', 'mfn-opts'),
   							1 => __('Enable', 'mfn-opts'),
   						),
							'std' => 0,
   					),

   					array(
   						'id' => 'invert',
   						'type' => 'switch',
   						'title' => __('Invert', 'mfn-opts'),
   						'options' => array(
   							0 => __('Disable', 'mfn-opts'),
   							1 => __('Enable', 'mfn-opts'),
   						),
							'std' => 0,
   					),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
		  				'type' => 'header',
		  				'class' => 'toggled_header row-header',
   						'title' => __('Items', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .payment-methods-list:justify-content',
							'type' => 'select',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'title' => __('Alignment', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'flex-start' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'flex-end' => __('Right', 'mfn-opts'),
								'space-between' => __('Space between', 'mfn-opts'),
								'space-around' => __('Space around', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .payment-methods-list:justify-content_laptop',
							'type' => 'select',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'title' => __('Alignment', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'flex-start' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'flex-end' => __('Right', 'mfn-opts'),
								'space-between' => __('Space between', 'mfn-opts'),
								'space-around' => __('Space around', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .payment-methods-list:justify-content_tablet',
							'type' => 'select',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'title' => __('Alignment', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'flex-start' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'flex-end' => __('Right', 'mfn-opts'),
								'space-between' => __('Space between', 'mfn-opts'),
								'space-around' => __('Space around', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .payment-methods-list:justify-content_mobile',
							'type' => 'select',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'title' => __('Alignment', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'flex-start' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'flex-end' => __('Right', 'mfn-opts'),
								'space-between' => __('Space between', 'mfn-opts'),
								'space-around' => __('Space around', 'mfn-opts'),
							],
							'std' => '',
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .payment-methods-list li img:width',
							'type' => 'sliderbar',
							'responsive' => 'desktop',
							'title' => __('Size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '10',
								'max' => '300',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .payment-methods-list li:opacity',
							'type' => 'sliderbar',
							'title' => __('Opacity', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '1',
								'step' => '0.1',
							),
							'std' => '1'
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .payment-methods-list li img:width_laptop',
							'type' => 'sliderbar',
							'responsive' => 'laptop',
							'title' => __('Size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '10',
								'max' => '300',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .payment-methods-list li img:width_tablet',
							'type' => 'sliderbar',
							'responsive' => 'tablet',
							'title' => __('Size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '10',
								'max' => '300',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .payment-methods-list li img:width_mobile',
							'type' => 'sliderbar',
							'responsive' => 'mobile',
							'title' => __('Size', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '10',
								'max' => '300',
								'step' => '1',
								'unit' => 'px',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .payment-methods-list li:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

   					array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .payment-methods-list li:padding',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .payment-methods-list li:padding_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .payment-methods-list li:padding_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .payment-methods-list li:padding_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .payment-methods-list li:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .payment-methods-list li:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .payment-methods-list li:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .payment-methods-list li:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .payment-methods-list li:border-style',
							'attr_id' => 'border_style_paymet',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .payment-methods-list li:border-color',
							'condition' => array( 'id' => 'border_style_paymet', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .payment-methods-list li:border-width',
		  				'condition' => array( 'id' => 'border_style_paymet', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .payment-methods-list li:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_paymet', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .payment-methods-list li:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_paymet', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .payment-methods-list li:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_paymet', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .payment-methods-list li:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .payment-methods-list li:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .payment-methods-list li:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .payment-methods-list li:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				)
   			),

   			// Photo Box ------------------------------------------------------

   			'photo_box' => array(
   				'type' => 'photo_box',
   				'title' => __('Photo Box', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/4',
   				'tablet_resized' => '0',
   				'cat' => 'boxes',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'title',
   						'type' => 'text',
						'dynamic_data' => 'title',
   						'edit_tag' => '.photo_box | .title',
   						'edit_position' => 'prepend',
   						'title' => __('Title', 'mfn-opts'),
   						'desc' => __('Allowed HTML tags: span, strong, b, em, i, u', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h4',
						),

   					array(
   						'id' => 'image',
   						'dynamic_data' => 'featured_image',
   						'type' => 'upload',
   						'title' => __('Image', 'mfn-opts'),
   						'desc' => __('Recommended image width <b>768px - 1920px</b> depending on size of the item', 'mfn-opts'),
							'std' => $this->get_placeholder(),
							'preview' => 'image',
   					),

   					array(
   						'id' => 'content',
   						'type' => 'textarea',
   						'dynamic_data' => 'content',
   						'edit_tag' => '.photo_box | div.desc',
   						'edit_position' => 'append',
   						'title' => __('Content', 'mfn-opts'),
   						'desc' => __('Some shortcodes and HTML tags allowed', 'mfn-opts'),
							'std' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut elit tellus, luctus nec ullamcorper mattis, pulvinar dapibus leo.',
							'preview' => 'content',
							'class' => 'form-content-full-width',
   					),

   					// link

   					array(
   						'type' => 'header',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link',
   						'dynamic_data' => 'permalink',
   						'type' => 'text',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'target',
   						'type' => 'select',
   						'title' => __('Target', 'mfn-opts'),
   						'options' => array(
   							0 => __('Default | _self', 'mfn-opts'),
   							1 => __('New tab or window | _blank', 'mfn-opts'),
   							'lightbox' => __('Lightbox (image or embed video)', 'mfn-opts'),
   						),
   					),

            array(
							'id' => 'link_title',
							'type' => 'text',
							'dynamic_data' => 'title',
							'title' => __('Link title', 'mfn-opts'),
						),

   					// advanced

   					array(
   						'title' => __('Advanced', 'mfn-opts'),
   					),

   					array(
   						'id' => 'greyscale',
   						'type' => 'switch',
   						'title' => __('Grayscale', 'mfn-opts'),
   						'desc' => __('Works only for images with link', 'mfn-opts'),
   						'options' => array(
   							0 => __('Disable', 'mfn-opts'),
   							1 => __('Enable', 'mfn-opts'),
   						),
							'std' => 0,
   					),

						// deprecated

   					array(
   						'type' => 'header',
   						'class' => 'mfn-deprecated',
   						'title' => __('Deprecated', 'mfn-opts'),
   					),

   					array(
   						'id' => 'align',
   						'class' => 'mfn-deprecated',
   						'type' => 'switch',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'left' => __('Left', 'mfn-opts'),
								'' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'std' => '',
   					),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
		  				'type' => 'header',
		  				'class' => 'toggled_header row-header',
   						'title' => __('Container', 'mfn-opts'),
   					),

						array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .photo_box:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .photo_box:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .photo_box:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),
   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .photo_box:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .photo_box .title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .photo_box .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Image', 'mfn-opts'),
   					),

   					array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .photo_box .image_frame:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .photo_box .image_frame:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .photo_box .image_frame:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .photo_box .image_frame:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .photo_box .image_frame:border-style',
							'attr_id' => 'border_style_pb_img',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .photo_box .image_frame:border-color',
							'condition' => array( 'id' => 'border_style_pb_img', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .photo_box .image_frame:border-width',
		  				'condition' => array( 'id' => 'border_style_pb_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .photo_box .image_frame:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_pb_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .photo_box .image_frame:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_pb_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .photo_box .image_frame:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_pb_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .photo_box .image_frame:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .photo_box .image_frame:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .photo_box .image_frame:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .photo_box .image_frame:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .photo_box .desc:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .photo_box .desc:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

				// Plain text

				'plain_text' => array(
   				'type' => 'plain_text',
   				'title' => __('Plain Text', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/4',
   				'tablet_resized' => '0',
   				'cat' => 'typography',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'content',
   						'type' => 'textarea',
   						'dynamic_data' => 'content',
   						'edit_tag' => '.mcb-column-inner | div.desc',
   						'edit_position' => 'append',
   						'title' => __('Text', 'mfn-opts'),
   						'desc' => __('Some shortcodes and HTML tags allowed', 'mfn-opts'),
							'std' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut elit tellus, luctus nec ullamcorper mattis, pulvinar dapibus leo.',
							'vbstd' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut elit tellus, luctus nec ullamcorper mattis, pulvinar dapibus leo.',
							'preview' => 'content',
							'class' => 'form-content-full-width',
   					),

   					array(
   						'id' => 'shortcodes_parser',
   						'type' => 'switch',
   						'title' => __('Shortcodes parser', 'mfn-opts'),
   						'options' => array(
   							'' => __('Disabled', 'mfn-opts'),
   							'1' => __('Enabled', 'mfn-opts'),
   						),
							'preview' => '1',
   					),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',

   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',

   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',

   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),
   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',

   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc a:color',
							'type' => 'color',
							'desc' => __('May be overwritten by individual tags or inline CSS', 'mfn-opts'),
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc:text-shadow',
							'type' => 'text_shadow',
							'title' => __('Text shadow', 'mfn-opts'),
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .desc:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

   					array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),


   			// Portfolio ------------------------------------------------------

   			'portfolio' => array(
   				'type' => 'portfolio',
   				'title' => __('Portfolio', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'loops',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h5',
						),

   					array(
   						'id' => 'count',
   						're_render' => true,
   						'type' => 'text',
   						'title' => __('Projects number', 'mfn-opts'),
   						'param' => 'number',
   						'after' => 'projects',
   						'class' => 'narrow',
   						'std' => 3,
							'preview' => 'number',
   					),

   					array(
   						'id' => 'style',
   						're_render' => true,
   						'type' => 'select',
   						'title' => __('Style', 'mfn-opts'),
   						'options' => array(
   							'flat' => __('Flat', 'mfn-opts'),
   							'grid' => __('Grid', 'mfn-opts'),
   							'masonry' => __('Masonry blog style', 'mfn-opts'),
   							'masonry-hover' => __('Masonry hover description', 'mfn-opts'),
   							'masonry-minimal' => __('Masonry minimal', 'mfn-opts'),
   							'masonry-flat' => __('Masonry flat', 'mfn-opts'),
   							'list' => __('List', 'mfn-opts'),
   							'exposure' => __('Exposure', 'mfn-opts'),
   						),
   						'std' => 'grid',
							'preview' => 'style',
   					),

   					array(
   						'id' => 'columns',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Columns', 'mfn-opts'),
   						'desc' => __('for styles: Flat, Grid, Masonry blog style, Masonry hover description', 'mfn-opts'),
   						'options' => array(
   							2	=> 2,
   							3	=> 3,
   							4	=> 4,
   							5	=> 5,
   							6	=> 6,
   						),
   						'std' => 3,
   					),

   					// options

   					array(
   						'type' => 'header',
   						'title' => __('Options', 'mfn-opts'),
   					),

   					array(
   						'id' => 'category',
   						're_render' => true,
   						'type' => 'select',
   						'title' => __('Category', 'mfn-opts'),
   						'php_options' => mfn_get_categories('portfolio-types'),
   						'js_hierarchical_options' => 'portfolio_types',
   						'wpml' => 'portfolio-types',
							'preview' => 'category',
   					),

   					array(
   						'id' => 'category_multi',
   						're_render' => true,
   						'type' => 'text',
   						'title' => __('Multiple categories', 'mfn-opts'),
   						'desc' => __('Slugs should be separated with <b>coma</b> ( , )', 'mfn-opts'),
							'preview' => 'category-all',
   					),

   					array(
   						'id' => 'orderby',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Order by', 'mfn-opts'),
   						'desc' => __('Do <b>not</b> use random order with pagination or load more', 'mfn-opts'),
   						'options' => array(
   							'date' => __('Date', 'mfn-opts'),
   							'menu_order' => __('Menu order', 'mfn-opts'),
   							'title' => __('Title', 'mfn-opts'),
   							'rand' => __('Random', 'mfn-opts'),
   						),
   						'std' => 'date'
   					),

   					array(
   						'id' => 'order',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Order', 'mfn-opts'),
   						'options' => array(
   							'ASC' => __('Ascending', 'mfn-opts'),
   							'DESC' => __('Descending', 'mfn-opts'),
   						),
   						'std' => 'DESC'
   					),

   					// advanced

   					array(
   						'type' => 'header',
   						'title' => __('Advanced', 'mfn-opts'),
   					),


   					array(
							'id' => 'portfolio-link',
							'type' => 'select',
							'title' => __('Project link', 'mfn-opts'),
							'options' => array(
								''	 => __('Default (from Theme Options)', 'mfn-opts'),
								'details' => __('Details', 'mfn-opts'),
								'popup' => __('Popup Image', 'mfn-opts'),
								'disable'	=> __('Disable Details | Only Popup Image', 'mfn-opts'),
								'_self' => __('Project Website | Open in the same window', 'mfn-opts'),
								'_blank'	=> __('Project Website | Open in new window', 'mfn-opts'),
							)
						),

   					array(
   						'id' => 'exclude_id',
   						're_render' => true,
   						'type' => 'text',
   						'title' => __('Exclude posts', 'mfn-opts'),
   						'desc' => __('IDs should be separated with <b>coma</b> ( , )', 'mfn-opts'),
   					),

   					array(
   						'id' => 'related',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Use as related projects', 'mfn-opts'),
   						'desc' => __('Exclude current project on single project page. This option will overwrite exclude posts option above.', 'mfn-opts'),
   						'options' => array(
   							0 => __('Disable', 'mfn-opts'),
   							1 => __('Enable', 'mfn-opts'),
   						),
							'std' => 0,
   					),

   					array(
   						'id' => 'filters',
   						'attr_id' => 'filters',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Filters', 'mfn-opts'),
   						'desc' => __('for category: all or multiple categories (only selected categories show in filters)', 'mfn-opts'),
   						'options' => array(
   							0 => __('Hide', 'mfn-opts'),
   							1 => __('Show', 'mfn-opts'),
   						),
							'std' => 0,
   					),

						// pagination

						array(
							'type' => 'header',
   						'title' => __('Pagination', 'mfn-opts'),
   					),

   					array(
   						'id' => 'pagination',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Pagination', 'mfn-opts'),
   						'desc' => __('Does <b>not</b> work on WMPL homepage', 'mfn-opts'),
   						'options' => array(
   							0 => __('Hide', 'mfn-opts'),
   							1 => __('Show', 'mfn-opts'),
   						),
							'std' => 0,
   					),

   					array(
   						'id' => 'load_more',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Load More button', 'mfn-opts'),
   						'desc' => __('Sliders will be replaced with featured images. Please use with pagination enabled.', 'mfn-opts'),
   						'options' => array(
   							0 => __('No', 'mfn-opts'),
   							1 => __('Yes', 'mfn-opts'),
   						),
							'std' => 0,
   					),

						// style

						array(
							'type' => 'header',
   						'title' => __('Style', 'mfn-opts'),
   					),

   					array(
   						'id' => 'greyscale',
   						'type' => 'switch',
   						'title' => __('Grayscale', 'mfn-opts'),
   						'options' => array(
   							0 => __('Disable', 'mfn-opts'),
   							1 => __('Enable', 'mfn-opts'),
   						),
							'std' => 0,
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Container', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

   					array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item:padding',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item:padding_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item:padding_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item:padding_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item:border-style',
							'attr_id' => 'border_style_portfolio',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item:border-color',
							'condition' => array( 'id' => 'border_style_portfolio', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item:border-width',
		  				'condition' => array( 'id' => 'border_style_portfolio', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_portfolio', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_portfolio', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_portfolio', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

   					array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Image', 'mfn-opts'),
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .image_frame:border-style',
							'attr_id' => 'border_style_portfolio_img',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .image_frame:border-color',
							'condition' => array( 'id' => 'border_style_portfolio_img', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .image_frame:border-width',
		  				'condition' => array( 'id' => 'border_style_portfolio_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .image_frame:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_portfolio_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .image_frame:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_portfolio_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .image_frame:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_portfolio_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .image_frame:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .image_frame:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .image_frame:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .image_frame:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .desc:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

   					array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .desc:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

   					array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .desc:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .desc:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .desc:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

   					array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .desc:padding',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .desc:padding_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .desc:padding_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .desc:padding_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
		  				'version' => 'separated-fields',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

						array(
							'id' => 'style:body .mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .entry-title a,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .entry-title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .entry-title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Excerpt', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .desc-wrapper:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .desc-wrapper:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Icons', 'mfn-opts'),
   					),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item a i:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item a|hover i:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Masonry hover', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group.masonry-hover .portfolio-item .masonry-hover-wrapper .hover-desc:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group.masonry-hover .portfolio-item .masonry-hover-wrapper .hover-desc h3|after:background',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Decoration', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Meta', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .details-wrapper,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .details-wrapper h5:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .details-wrapper a:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Link color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .details-wrapper,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .details-wrapper .column:border-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .details-wrapper:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Exposure', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .desc-inner .line:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Line', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Button', 'mfn-opts'),
   					),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .list_style_header .links_wrapper .button:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .list_style_header .links_wrapper .button:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .list_style_header .links_wrapper .button:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .list_style_header .links_wrapper .button:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .list_style_header .links_wrapper .button:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .list_style_header .links_wrapper .button:border-style',
							'attr_id' => 'border_style',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .list_style_header .links_wrapper .button:border-width',
		  				'condition' => array( 'id' => 'border_style', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .list_style_header .links_wrapper .button:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .list_style_header .links_wrapper .button:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .list_style_header .links_wrapper .button:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .list_style_header .links_wrapper .button:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .list_style_header .links_wrapper .button:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .list_style_header .links_wrapper .button:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .list_style_header .links_wrapper .button:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .list_style_header .links_wrapper .button,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .list_style_header .links_wrapper .button i:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .list_style_header .links_wrapper .button:border-color',
							'condition' => array( 'id' => 'border_style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher',
							'attr_id' => 'background_switcher',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .list_style_header .links_wrapper .button:background-color',
							'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .list_style_header .links_wrapper .button:gradient',
		  				'condition' => array( 'id' => 'background_switcher', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .list_style_header .links_wrapper .button|hover,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .list_style_header .links_wrapper .button|hover i:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .list_style_header .links_wrapper .button|hover:border-color',
							'condition' => array( 'id' => 'border_style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher_hover',
							'attr_id' => 'background_switcher_hover',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .list_style_header .links_wrapper .button|hover:background',
							'condition' => array( 'id' => 'background_switcher_hover', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_group .portfolio-item .list_style_header .links_wrapper .button|hover:gradient',
		  				'condition' => array( 'id' => 'background_switcher_hover', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),
						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'condition' => array( 'id' => 'filters', 'opt' => 'is', 'val' => '1' ),
   						'title' => __('Filters', 'mfn-opts'),
   					),

						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Container', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:background',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:border-style',
							'attr_id' => 'border_blog_filters_wrapper',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:border-width',
		  				'condition' => array( 'id' => 'border_blog_filters_wrapper', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:border-width_laptop',
		  				'condition' => array( 'id' => 'border_blog_filters_wrapper', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:border-width_tablet',
		  				'condition' => array( 'id' => 'border_blog_filters_wrapper', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:border-width_mobile',
		  				'condition' => array( 'id' => 'border_blog_filters_wrapper', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),


		  			array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Items', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li:width',
							'attr_id' => 'filters_items_width_switcher',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Items width', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'auto' => __('Auto', 'mfn-opts'),
							],
							'std' => ''
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),



		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:border-style',
							'attr_id' => 'border_blog_filters_wrapper_li',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:border-width',
		  				'condition' => array( 'id' => 'border_blog_filters_wrapper_li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:border-width_laptop',
		  				'condition' => array( 'id' => 'border_blog_filters_wrapper_li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:border-width_tablet',
		  				'condition' => array( 'id' => 'border_blog_filters_wrapper_li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:border-width_mobile',
		  				'condition' => array( 'id' => 'border_blog_filters_wrapper_li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),




		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),


						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Links color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a:border-color',
							'condition' => array( 'id' => 'border_blog_filters_wrapper_li', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a|hover:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Links color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a|hover:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li a|hover:border-color',
							'condition' => array( 'id' => 'border_blog_filters_wrapper_li', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li.current-cat a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Links color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li.current-cat a:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li.current-cat a:border-color',
							'condition' => array( 'id' => 'border_blog_filters_wrapper_li', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),


						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Filters exit', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li.close a:width',
							'attr_id' => 'filters_items_width_switcher',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Width', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'auto' => __('Auto', 'mfn-opts'),
							],
							'std' => ''
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li.close a:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li.close a:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li.close a:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li.close a:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),


		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li.close a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li.close a:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li.close a:border-color',
							'condition' => array( 'id' => 'border_blog_filters_wrapper_li', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li.close a|hover:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li.close a|hover:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement #Filters .filters_wrapper li.close a|hover:border-color',
							'condition' => array( 'id' => 'border_blog_filters_wrapper_li', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),






















						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Portfolio Grid -------------------------------------------------

   			'portfolio_grid' => array(
   				'type' => 'portfolio_grid',
   				'title' => __('Portfolio Grid', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'loops',
   				'attr' => array(
   				//'fields' => array(

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
   						'id' => 'count',
   						're_render' => true,
   						'type' => 'text',
   						'title' => __('Projects number', 'mfn-opts'),
   						'param' => 'number',
   						'after' => 'projects',
   						'class' => 'narrow',
   						'std' => 4,
							'preview' => 'number',
   					),

   					// options

   					array(
   						'type' => 'header',
   						'title' => __('Options', 'mfn-opts'),
   					),

   					array(
   						'id' => 'category',
   						're_render' => true,
   						'type' => 'select',
   						'title' => __('Category', 'mfn-opts'),
   						'php_options' => mfn_get_categories('portfolio-types'),
   						'js_hierarchical_options' => 'portfolio_types',
   						'wpml' => 'portfolio-types',
							'preview' => 'category',
   					),

   					array(
   						'id' => 'category_multi',
   						're_render' => true,
   						'type' => 'text',
   						'title' => __('Multiple categories', 'mfn-opts'),
   						'desc' => __('Slugs should be separated with <b>coma</b> ( , )', 'mfn-opts'),
							'preview' => 'category-all',
   					),

   					array(
   						'id' => 'orderby',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Order by', 'mfn-opts'),
   						'options' => array(
   							'date' => __('Date', 'mfn-opts'),
   							'menu_order' => __('Menu order', 'mfn-opts'),
   							'title' => __('Title', 'mfn-opts'),
   							'rand' => __('Random', 'mfn-opts'),
   						),
   						'std' => 'date',
   					),

   					array(
   						'id' => 'order',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Order', 'mfn-opts'),
   						'options' => array(
   							'ASC' => __('Ascending', 'mfn-opts'),
   							'DESC' => __('Descending', 'mfn-opts'),
   						),
   						'std' => 'DESC',
   					),

   					// advanced

   					array(
   						'type' => 'header',
   						'title' => __('Advanced', 'mfn-opts'),
   					),

   					array(
   						'id' => 'greyscale',
   						'type' => 'switch',
   						'title' => 'Grayscale',
   						'options' => array(
   							0 => __('Disable', 'mfn-opts'),
   							1 => __('Enable', 'mfn-opts'),
   						),
							'std' => 0,
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-re_render modalbox-card-style modalbox-re_render">',
						),











						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Box', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li:width',
							'type' => 'text',
							'default_unit' => '%',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'title' => __('Width', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li:width_laptop',
							'type' => 'text',
							'default_unit' => '%',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'title' => __('Width', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li:width_tablet',
							'type' => 'text',
							'default_unit' => '%',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'title' => __('Width', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li:width_mobile',
							'type' => 'text',
							'default_unit' => '%',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'title' => __('Width', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li:border-style',
							'type' => 'select',
							'attr_id' => 'border_style_portgrid',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li:border-width',
		  				'condition' => array( 'id' => 'border_style_portgrid', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_portgrid', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_portgrid', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_portgrid', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),


		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li .image_frame .image_wrapper .mask|after:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Mask', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li:border-color',
							'condition' => array( 'id' => 'border_style_portgrid', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li|hover:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li|hover .image_frame .image_wrapper .mask|after:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Mask', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li|hover:border-color',
							'condition' => array( 'id' => 'border_style_portgrid', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),


		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Icons', 'mfn-opts'),
   					),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li .image_links a:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li .image_links a .path:stroke',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li .image_links a:border-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li .image_links a|hover:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li .image_links a|hover .path:stroke',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_grid li .image_links a|hover:border-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),


						array(
							'type' => 'html',
							'html' => '</div></div>',
						),








						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Portfolio Photo ------------------------------------------------

   			'portfolio_photo' => array(
   				'type' => 'portfolio_photo',
   				'title' => __('Portfolio Photo', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'loops',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h3',
						),

						array(
   						'id' => 'count',
   						're_render' => true,
   						'type' => 'text',
   						'title' => __('Projects number', 'mfn-opts'),
   						'param' => 'number',
   						'after' => 'projects',
   						'class' => 'narrow',
   						'std' => 5,
							'preview' => 'number',
   					),

   					// options

   					array(
   						'type' => 'header',
   						'title' => __('Options', 'mfn-opts'),
   					),

   					array(
   						'id' => 'category',
   						're_render' => true,
   						'type' => 'select',
   						'title' => __('Category', 'mfn-opts'),
   						'php_options' => mfn_get_categories('portfolio-types'),
   						'js_hierarchical_options' => 'portfolio_types',
   						'wpml' => 'portfolio-types',
							'preview' => 'category',
   					),

   					array(
   						'id' => 'category_multi',
   						're_render' => true,
   						'type' => 'text',
   						'title' => __('Multiple categories', 'mfn-opts'),
   						'desc' => __('Slugs should be separated with <b>coma</b> ( , )', 'mfn-opts'),
							'preview' => 'category-all',
   					),

   					array(
   						'id' => 'orderby',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Order by', 'mfn-opts'),
   						'options' => array(
   							'date' => __('Date', 'mfn-opts'),
   							'menu_order' => __('Menu order', 'mfn-opts'),
   							'title' => __('Title', 'mfn-opts'),
   							'rand' => __('Random', 'mfn-opts'),
   						),
   						'std' => 'date',
   					),

   					array(
   						'id' => 'order',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Order', 'mfn-opts'),
   						'options' => array(
   							'ASC' => __('Ascending', 'mfn-opts'),
   							'DESC' => __('Descending', 'mfn-opts'),
   						),
   						'std' => 'DESC',
   					),

   					// advanced

   					array(
   						'type' => 'header',
   						'title' => __('Advanced', 'mfn-opts'),
   					),

   					array(
   						'id' => 'target',
   						'type' => 'switch',
   						'title' => __('Link target', 'mfn-opts'),
   						'options' => array(
   							0 => __('_self', 'mfn-opts'),
   							1 => __('_blank', 'mfn-opts'),
   						),
							'std' => 0,
   					),

   					array(
   						'id' => 'greyscale',
   						'type' => 'switch',
   						'title' => __('Grayscale', 'mfn-opts'),
   						'options' => array(
   							0 => __('Disable', 'mfn-opts'),
   							1 => __('Enable', 'mfn-opts'),
   						),
							'std' => 0,
   					),

   					array(
   						'id' => 'margin',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Margin', 'mfn-opts'),
   						'options' => array(
   							0 => __('Hide', 'mfn-opts'),
   							1 => __('Show', 'mfn-opts'),
   						),
							'std' => 0,
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Box', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .details:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .details:border-style',
							'type' => 'select',
							'attr_id' => 'border_style_port_photo',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .details:border-color',
							'condition' => array( 'id' => 'border_style_port_photo', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .details:border-width',
		  				'condition' => array( 'id' => 'border_style_port_photo', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .details:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_port_photo', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .details:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_port_photo', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .details:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_port_photo', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .details:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .details:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .details:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .details:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .details .title:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .details .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Decoration', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .details .title:border-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Meta', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .details .categories:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .details .categories:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Button', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .more:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .more h4:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .more:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .more:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .more:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .more:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .more:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .more:border-style',
							'type' => 'select',
							'attr_id' => 'border_style_port_photo',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .more:border-color',
							'condition' => array( 'id' => 'border_style_port_photo', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .more:border-width',
		  				'condition' => array( 'id' => 'border_style_port_photo', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .more:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_port_photo', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .more:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_port_photo', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .more:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_port_photo', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .more:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .more:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .more:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio-photo .portfolio-item .portfolio-details .more:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Portfolio Slider -----------------------------------------------

   			'portfolio_slider' => array(
   				'type' => 'portfolio_slider',
   				'title' => __('Portfolio Slider', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'loops',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
   						'id' => 'count',
   						're_render' => true,
   						'type' => 'text',
   						'title' => __('Projects number', 'mfn-opts'),
   						'param' => 'number',
   						'after' => 'projects',
   						'class' => 'narrow',
   						'std' => 6,
							'preview' => 'number',
   					),

   					// options

   					array(
   						'type' => 'header',
   						'title' => __('Options', 'mfn-opts'),
   					),

   					array(
   						'id' => 'category',
   						're_render' => true,
   						'type' => 'select',
   						'title' => __('Category', 'mfn-opts'),
   						'php_options' => mfn_get_categories('portfolio-types'),
   						'js_hierarchical_options' => 'portfolio_types',
   						'wpml' => 'portfolio-types',
							'preview' => 'category',
   					),

   					array(
   						'id' => 'category_multi',
   						're_render' => true,
   						'type' => 'text',
   						'title' => __('Multiple categories', 'mfn-opts'),
   						'desc' => __('Slugs should be separated with <strong>coma</strong> (,).', 'mfn-opts'),
							'preview' => 'category-all',
   					),

   					array(
   						'id' => 'orderby',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Order by', 'mfn-opts'),
   						'options' => array(
   							'date' => __('Date', 'mfn-opts'),
   							'menu_order' => __('Menu order', 'mfn-opts'),
   							'title' => __('Title', 'mfn-opts'),
   							'rand' => __('Random', 'mfn-opts'),
   						),
   						'std' => 'date',
   					),

   					array(
   						'id' => 'order',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Order', 'mfn-opts'),
   						'options' => array(
   							'ASC' => __('Ascending', 'mfn-opts'),
   							'DESC' => __('Descending', 'mfn-opts'),
   						),
   						'std' => 'DESC',
   					),

   					// advanced

   					array(
   						'type' => 'header',
   						'title' => __('Advanced', 'mfn-opts'),
   					),

   					array(
   						'id' => 'arrows',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Navigation', 'mfn-opts'),
   						'options' => array(
   							'' => __('Hide', 'mfn-opts'),
   							'always' => __('Show', 'mfn-opts'),
								'hover' => __('Show on hover', 'mfn-opts'),
   						),
							'std' => '',
   					),

   					array(
   						'id' => 'size',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Image size', 'mfn-opts'),
   						'options' => array(
   							'small' => __('Small', 'mfn-opts'),
   							'medium' => __('Medium', 'mfn-opts'),
   							'large' => __('Large', 'mfn-opts'),
   						),
							'std' => 'small',
   					),

   					array(
   						'id' => 'scroll',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Slides to scroll', 'mfn-opts'),
   						'options' => array(
   							'page' => __('One page', 'mfn-opts'),
   							'slide' => __('Single slide', 'mfn-opts'),
   						),
							'std' => 'page',
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Nav', 'mfn-opts'),
   					),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_slider .slider_nav:background-color',
							'type' => 'color',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_slider .slider_nav:color',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_slider .slider_nav|hover:background-color',
							'type' => 'color',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .portfolio_slider .slider_nav|hover:color',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Pricing item ---------------------------------------------------

   			'pricing_item' => array(
   				'type' => 'pricing_item',
   				'title' => __('Pricing Item', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/4',
   				'tablet_resized' => '0',
   				'cat' => 'blocks',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'image',
   						'dynamic_data' => 'featured_image',
   						'type' => 'upload',
   						'title' => __('Image', 'mfn-opts'),
							'std' => $this->get_placeholder(),
							'preview' => 'image',
   					),

   					array(
   						'id' => 'title',
   						'type' => 'text',
						'dynamic_data' => 'title',
   						'edit_tag' => '.plan-header | .title',
   						'title' => __('Title', 'mfn-opts'),
   						'edit_position' => 'before | .price',
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h2',
						),

						// price

   					array(
   						'type' => 'header',
   						'title' => __('Price', 'mfn-opts'),
   					),

   					array(
   						'id' => 'price',
   						'type' => 'text',
   						'title' => __('Price', 'mfn-opts'),
   						'class' => 'narrow',
   						'edit_tag' => '.price | span',
   						'edit_position' => 'append',
							'std' => '99',
							'preview' => 'number',
   					),

   					array(
   						'id' => 'currency',
   						'edit_tag' => '.price | sup.currency',
   						'edit_position' => 'prepend',
   						'type' => 'text',
   						'title' => __('Currency', 'mfn-opts'),
   						'class' => 'narrow',
   					),

   					array(
   						'id' => 'currency_pos',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Currency position', 'mfn-opts'),
   						'options' => array(
   							'' => __('Left', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'std' => '',
   					),

   					array(
   						'id' => 'period',
   						'type' => 'text',
   						'edit_tag' => '.price | .period',
   						'edit_position' => 'append',
   						'title' => __('Period', 'mfn-opts'),
							'class' => 'narrow',
   					),

   					// description

   					array(
   						'type' => 'header',
   						'title' => __('Description', 'mfn-opts'),
   					),

   					array(
   						'id' => 'subtitle',
   						'edit_tag' => '.plan-header | p.subtitle',
   						'edit_position' => 'append',
   						'type' => 'text',
   						'title' => __('Subtitle', 'mfn-opts'),
   					),

						array(
   						'id' => 'content',
   						'type' => 'textarea',
   						'dynamic_data' => 'content',
   						'edit_tag' => '.plan-inside | span.pi-content',
   						'edit_position' => 'prepend',
   						'title' => __('Content', 'mfn-opts'),
   						'desc' => __('HTML tags allowed', 'mfn-opts'),
   						'preview' => 'content',
							'class' => 'form-content-full-width',
   					),

						array(
   						'id' => 'tabs',
   						'type' => 'tabs',
   						'title' => __('List', 'mfn-opts'),
							'options' => [
								'title' => [
									'input',
									__('Title', 'mfn-opts'),
									'List item',
								],
							],
							'std' => [
								0 => [
									'title' => __('This is the 1st item', 'mfn-opts'),
								],
								1 => [
									'title' => __('This is the 2nd item', 'mfn-opts'),
								],
							],
							'primary' => 'title',
							'preview' => 'tabs',
   					),

   					// button

   					array(
   						'type' => 'header',
   						'title' => __('Button', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link_type',
   						'attr_id' => 'link_type',
   						'type' => 'select',
							'title' => __('On click action', 'mfn-opts'),
   						'options' => array(
								'' => __('Default', 'mfn-opts'),
								'1' => __('Open popup', 'mfn-opts'),
							),
							'std' => '',
   					),

   					array(
   						'id' => 'popup_id',
   						'condition' => array( 'id' => 'link_type', 'opt' => 'is', 'val' => '1' ),
   						'type' => 'text',
   						'title' => __('Popup ID', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link_title',
   						're_render' => true,
   						'dynamic_data' => 'title',
   						'type' => 'text',
   						'title' => __('Title', 'mfn-opts'),
   					),

   					array(
   						'id' => 'icon',
   						're_render' => true,
   						'type' => 'icon',
   						'title' => __('Icon', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link',
   						're_render' => true,
   						'dynamic_data' => 'permalink',
   						'type' => 'text',
   						'title' => __('Link', 'mfn-opts'),
   						'desc' => __('Button will appear only if this field is filled.', 'mfn-opts'),
   					),

   					array(
   						'id' => 'target',
   						'type' => 'select',
   						'title' => __('Link target', 'mfn-opts'),
   						'options' => array(
   							0 => __('Default | _self', 'mfn-opts'),
   							1 => __('New tab or window | _blank', 'mfn-opts'),
   							'lightbox' => __('Lightbox (image or embed video)', 'mfn-opts'),
   						),
   					),

            array(
							'id' => 'link_title',
							'type' => 'text',
							'title' => __('Link title', 'mfn-opts'),
						),

   					// style

   					array(
   						'type' => 'header',
   						'title' => __('Style', 'mfn-opts'),
   					),

   					array(
   						'id' => 'style',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Style', 'mfn-opts'),
   						'options' => array(
   							'box' => __('Box', 'mfn-opts'),
   							'table' => __('Table', 'mfn-opts'),
								'label' => __('Table label', 'mfn-opts'),
   						),
							'std' => 'box',
   					),

						// deprecated

   					array(
   						'type' => 'header',
   						'class' => 'mfn-deprecated',
   						'title' => __('Deprecated', 'mfn-opts'),
   					),

   					array(
   						'id' => 'featured',
   						'class' => 'mfn-deprecated',
   						'type' => 'switch',
   						'title' => __('Featured', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							1 => __('Featured', 'mfn-opts'),
   						),
   					),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Container', 'mfn-opts'),
   					),

						array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Image', 'mfn-opts'),
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header .image:border-style',
							'attr_id' => 'border_style_pricing_box',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header .image:border-color',
							'condition' => array( 'id' => 'border_style_pricing_box', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header .image:border-width',
		  				'condition' => array( 'id' => 'border_style_pricing_box', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header .image:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_pricing_box', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header .image:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_pricing_box', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header .image:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_pricing_box', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header .image:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header .image:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header .image:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header .image:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header .image:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header .image:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header .image:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header .image:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header .title:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header .title:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Price', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement  .pricing-box .plan-header .price span:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header .price span:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header .price:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header .price:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header .price:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header .price:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Price currency', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header .price sup.currency:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner .pricing-box .plan-header .price sup.currency:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Price period', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header .price sup.period:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner .pricing-box .plan-header .price sup.period:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Decoration', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header hr:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header hr:width',
							'type' => 'text',
							'default_unit' => 'px',
							'class' => 'mfn-toggled',
							'title' => __('Width', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Subtitle', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header p.subtitle:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-header p.subtitle:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside .pi-content:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside .pi-content:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside .pi-content:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside .pi-content:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside .pi-content:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside .pi-content:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('List', 'mfn-opts'),
   					),

   					array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside ul:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside ul:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside ul:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside ul:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside ul li:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('List row', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside ul li:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside ul li:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside ul li:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside ul li:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside ul li:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside ul li:border-style',
							'attr_id' => 'border_style_pricing_li',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside ul li:border-color',
							'condition' => array( 'id' => 'border_style_pricing_li', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside ul li:border-width',
		  				'condition' => array( 'id' => 'border_style_pricing_li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside ul li:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_pricing_li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside ul li:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_pricing_li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside ul li:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_pricing_li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside ul li:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside ul li:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside ul li:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside ul li:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

   					array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside ul li:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside ul li:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside ul li:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside ul li:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside ul li:padding',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside ul li:padding_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside ul li:padding_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-inside ul li:padding_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Button', 'mfn-opts'),
   					),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-footer a.button:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-footer a.button:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-footer a.button:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-footer a.button:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-footer a.button:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-footer a.button:border-style',
							'attr_id' => 'border_style_pricing_button',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-footer a.button:border-width',
		  				'condition' => array( 'id' => 'border_style_pricing_button', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-footer a.button:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_pricing_button', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-footer a.button:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_pricing_button', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-footer a.button:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_pricing_button', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-footer a.button:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-footer a.button:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-footer a.button:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-footer a.button:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-footer a.button:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-footer a.button:border-color',
							'condition' => array( 'id' => 'border_style_pricing_button', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher',
							'attr_id' => 'background_switcher_pricing_bg',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-footer a.button:background-color',
							'condition' => array( 'id' => 'background_switcher_pricing_bg', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-footer a.button:gradient',
		  				'condition' => array( 'id' => 'background_switcher_pricing_bg', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-footer a.button|hover:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-footer a.button|hover:border-color',
							'condition' => array( 'id' => 'background_switcher_pricing_bg', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher_hover',
							'attr_id' => 'background_switcher_hover_pricing_bg',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-footer a.button|hover:background',
							'condition' => array( 'id' => 'background_switcher_hover_pricing_bg', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .pricing-box .plan-footer a.button|hover:gradient',
		  				'condition' => array( 'id' => 'background_switcher_hover_pricing_bg', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Progress Bars  -------------------------------------------------

   			'progress_bars' => array(
   				'type' => 'progress_bars',
   				'title' => __('Progress Bars', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'boxes',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'title',
   						'type' => 'text',
   						'edit_tag' => '.progress_bars | .title',
						'dynamic_data' => 'title',
   						'edit_position' => 'prepend',
   						'title' => __('Title', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h4',
						),

						array(
   						'id' => 'tabs',
   						'type' => 'tabs',
   						'title' => __('Bars', 'mfn-opts'),
							'options' => [
								'title' => [
									'input',
									__('Title', 'mfn-opts'),
									__('Bar item', 'mfn-opts'),
								],
								'value' => [
									'input',
									__('Value', 'mfn-opts'),
									'50',
								],
								'size' => [
									'input',
									__('Size', 'mfn-opts'),
									'20',
								],
								'color' => [
									'input',
									__('Color (optional)', 'mfn-opts'),
									'',
								],
							],
							'std' => [
								0 => [
									'title' => 'This is the 1st bar',
									'value' => '50',
									'size' => '10',
									'color' => '#72a5d8',
								],
								1 => [
									'title' => 'This is the 2nd bar',
									'value' => '30',
									'size' => '15',
									'color' => 'grey',
								],
							],
							'primary' => 'title',
							'preview' => 'tabs',
   					),

   					array(
   						'id' => 'content',
   						'edit_tag' => '.bars_list | ul.pb-desc',
   						'edit_position' => 'append',
   						'dynamic_data' => 'content',
   						'type' => 'textarea',
   						'title' => __('Content', 'mfn-opts'),
							'class' => 'form-content-full-width',
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .progress_bars .title:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .progress_bars .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .progress_bars .title:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .progress_bars .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .progress_bars .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .progress_bars .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .progress_bars .bars_list .pb-desc:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .progress_bars .bars_list .pb-desc:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .progress_bars .bars_list .pb-desc:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .progress_bars .bars_list .pb-desc:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .progress_bars .bars_list .pb-desc:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .progress_bars .bars_list .pb-desc:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Bar title', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .progress_bars .bars_list li h6:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .progress_bars .bars_list li h6:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Bar value', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .progress_bars .bars_list li h6 .label:display',
							'attr_id' => 'progress_bar_percents',
							'type' => 'switch',
							'class' => 'mfn-toggled',
							'title' => __('Visibility', 'mfn-opts'),
							'options' => array(
								'' => __('Visible', 'mfn-opts'),
								'none' => __('Hidden', 'mfn-opts'),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .progress_bars .bars_list li h6 .label:color',
							'condition' => array( 'id' => 'progress_bar_percents', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .progress_bars .bars_list li h6 .label:background-color',
							'condition' => array( 'id' => 'progress_bar_percents', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .progress_bars .bars_list li h6 .label:typography',
							'condition' => array( 'id' => 'progress_bar_percents', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Bar', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .progress_bars .bars_list li .bar:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .progress_bars .bars_list li .bar .progress:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background active', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .progress_bars .bars_list li .bar:box-shadow',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow'
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .progress_bars .bars_list li .bar:box-shadow_laptop',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .progress_bars .bars_list li .bar:box-shadow_tablet',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .progress_bars .bars_list li .bar:box-shadow_mobile',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),


		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .progress_bars .bars_list li .bar:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .progress_bars .bars_list li .bar:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .progress_bars .bars_list li .bar:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .progress_bars .bars_list li .bar:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),


						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Promo Box ------------------------------------------------------

   			'promo_box' => array(
   				'type' => 'promo_box',
   				'title' => __('Promo Box', 'mfn-opts'),
   				'size' => '1/2',
   				'tablet_size' => '1/2',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'boxes',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'image',
   						'dynamic_data' => 'featured_image',
   						'type' => 'upload',
   						'title' => __('Image', 'mfn-opts'),
   						'desc' => __('Recommended minimum image width <b>768px</b>', 'mfn-opts'),
							'std' => $this->get_placeholder(),
							'preview' => 'image',
   					),

   					array(
   						'id' => 'title',
   						'type' => 'text',
						'dynamic_data' => 'title',
   						'edit_tag' => '.desc_wrapper | .title',
   						'edit_position' => 'prepend',
   						'title' => __('Title', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h2',
						),

   					array(
   						'id' => 'content',
   						'type' => 'textarea',
   						'dynamic_data' => 'content',
   						'edit_tag' => '.desc_wrapper | div.desc',
   						'edit_position' => 'prepend',
   						'title' => __('Content', 'mfn-opts'),
							'std' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut elit tellus, luctus nec ullamcorper mattis, pulvinar dapibus leo.',
							'preview' => 'content',
							'class' => 'form-content-full-width',
   					),

   					// button

   					array(
   						'type' => 'header',
   						'title' => __('Button', 'mfn-opts'),
   					),

   					array(
   						'id' => 'btn_text',
   						'type' => 'text',
   						'dynamic_data' => 'title',
   						're_render' => true,
   						'title' => __('Text', 'mfn-opts'),
   					),
   					array(
   						'id' => 'btn_link',
   						'dynamic_data' => 'link',
   						'type' => 'text',
   						're_render' => true,
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'target',
   						'type' => 'select',
   						'title' => __('Link target', 'mfn-opts'),
   						'options' => array(
   							0 => __('Default | _self', 'mfn-opts'),
   							1 => __('New tab or window | _blank', 'mfn-opts'),
   							'lightbox' => __('Lightbox (image or embed video)', 'mfn-opts'),
   						),
   					),

            array(
							'id' => 'link_title',
							'type' => 'text',
							'dynamic_data' => 'title',
							'title' => __('Link title', 'mfn-opts'),
						),

   					// options

   					array(
   						'type' => 'header',
   						'title' => __('Options', 'mfn-opts'),
   					),

   					array(
   						'id' => 'position',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Image position', 'mfn-opts'),
   						'options' => array(
   							'left' => __('Left', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
   						'std' => 'left',
   					),

   					array(
   						'id' => 'border',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Border right', 'mfn-opts'),
   						'options' => array(
   							0 => __('Hide', 'mfn-opts'),
   							1 => __('Show', 'mfn-opts'),
   						),
							'std' => 0,
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Image', 'mfn-opts'),
   					),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .photo_wrapper:border-style',
							'attr_id' => 'border_style_pricing_box',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .photo_wrapper:border-color',
							'condition' => array( 'id' => 'border_style_pricing_box', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .photo_wrapper:border-width',
		  				'condition' => array( 'id' => 'border_style_pricing_box', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .photo_wrapper:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_pricing_box', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .photo_wrapper:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_pricing_box', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .photo_wrapper:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_pricing_box', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .photo_wrapper:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .photo_wrapper:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .photo_wrapper:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .photo_wrapper:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Desc', 'mfn-opts'),
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .title:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .title:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .desc:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .desc:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .desc:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .desc:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .desc:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .desc:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Button', 'mfn-opts'),
   					),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .button:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .button:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .button:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .button:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .button:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .button:border-style',
							'attr_id' => 'border_style_promo',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .button:border-width',
		  				'condition' => array( 'id' => 'border_style_promo', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .button:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_promo', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .button:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_promo', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .button:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_promo', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .button:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .button:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .button:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .button:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .button:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .button:border-color',
							'condition' => array( 'id' => 'border_style_promo', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher',
							'attr_id' => 'background_switcher_promo',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .button:background-color',
							'condition' => array( 'id' => 'background_switcher_promo', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .button:gradient',
		  				'condition' => array( 'id' => 'background_switcher_promo', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .button|hover:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .button|hover:border-color',
							'condition' => array( 'id' => 'border_style_promo', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher_hover',
							'attr_id' => 'background_switcher_hover_promo',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .button|hover:background',
							'condition' => array( 'id' => 'background_switcher_hover_promo', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box_wrapper .desc_wrapper .button|hover:gradient',
		  				'condition' => array( 'id' => 'background_switcher_hover_promo', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Line', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .promo_box.has_border|after:border-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Quick Fact -----------------------------------------------------

   			'quick_fact' => array(
   				'type' => 'quick_fact',
   				'title' => __('Quick Fact', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'boxes',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'heading',
   						'type' => 'text',
   						'dynamic_data' => 'title',
   						'title' => __('Heading', 'mfn-opts'),
   						'edit_tag' => '.quick_fact | .heading_tag',
   						'edit_position' => 'prepend',
   						'edit_tag_var' => 'preview-heading_taginput',
							'std' => __('This is the heading', 'mfn-opts'),
							'preview' => 'title',
   					),

						array(
							'id' => 'heading_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Heading tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'H1',
								'h2' => 'H2',
								'h3' => 'H3',
								'h4' => 'H4',
								'h5' => 'H5',
								'h6' => 'H6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h4'
						),

   					array(
   						'id' => 'title',
   						'type' => 'text',
   						'title' => __('Title', 'mfn-opts'),
   						'dynamic_data' => 'title',
							'std' => __('This is the title', 'mfn-opts'),
							'edit_tag' => '.quick_fact | .title_tag',
							'edit_tag_var' => 'preview-title_taginput',
							'edit_position' => 'before | .hr_narrow',
							'preview' => 'subtitle',
	   				),

						array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'H1',
								'h2' => 'H2',
								'h3' => 'H3',
								'h4' => 'H4',
								'h5' => 'H5',
								'h6' => 'H6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h3'
						),

   					array(
   						'id' => 'content',
   						'type' => 'textarea',
   						'title' => __('Content', 'mfn-opts'),
							'std' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut elit tellus, luctus nec ullamcorper mattis, pulvinar dapibus leo.',
							'preview' => 'content',
							'dynamic_data' => 'content',
							'edit_tag' => '.quick_fact | div.desc',
							'class' => 'form-content-full-width',
   					),

   					// quick fact

   					array(
   						'type' => 'header',
   						'title' => __('Quick fact', 'mfn-opts'),
   					),

   					array(
   						'id' => 'number',
   						'type' => 'text',
   						'title' => __('Number', 'mfn-opts'),
   						'param' => 'number',
   						'class' => 'narrow',
   						'std' => '99',
   						'preview' => 'number',
   					),

   					array(
   						'id' => 'prefix',
   						'type' => 'text',
   						'title' => __('Prefix', 'mfn-opts'),
   						'class' => 'narrow',
   					),

   					array(
   						'id' => 'label',
   						'type' => 'text',
   						'title' => __('Postfix', 'mfn-opts'),
   						'class' => 'narrow',
   					),

   					array(
   						'type' => 'header',
   						'class' => 'mfn-deprecated',
   						'title' => __('Deprecated', 'mfn-opts'),
   					),

   					array(
   						'id' => 'align',
   						'class' => 'mfn-deprecated',
   						'type' => 'switch',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
								'left' => __('Left', 'mfn-opts'),
   							'' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'std' => '',
   					),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

		  			array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
		  				'type' => 'header',
		  				'class' => 'toggled_header row-header',
   						'title' => __('Container', 'mfn-opts'),
   					),

						array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),
   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

		  			array(
		  				'type' => 'header',
		  				'class' => 'toggled_header row-header',
   						'title' => __('Heading', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .heading_tag:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .heading_tag:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .heading_tag:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .heading_tag:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .heading_tag:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .heading_tag:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'type' => 'header',
		  				'class' => 'toggled_header row-header',
   						'title' => __('Pre', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .number-wrapper .prefix:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .number-wrapper .prefix:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .number-wrapper .prefix:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .number-wrapper .prefix:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .number-wrapper .prefix:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .number-wrapper .prefix:typography',
							'type' => 'typography_vb',
							'typography' => 'simple',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'type' => 'header',
		  				'class' => 'toggled_header row-header',
   						'title' => __('Number', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .number-wrapper .number:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .number-wrapper:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .number-wrapper:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .number-wrapper:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .number-wrapper:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .number-wrapper .number:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),


						array(
		  				'type' => 'header',
		  				'class' => 'toggled_header row-header',
   						'title' => __('Post', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .number-wrapper .postfix:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .number-wrapper .postfix:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .number-wrapper .postfix:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .number-wrapper .postfix:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .number-wrapper .postfix:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .number-wrapper .postfix:typography',
							'type' => 'typography_vb',
							'typography' => 'simple',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'type' => 'header',
		  				'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .title_tag:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .title_tag:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .title_tag:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .title_tag:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .title_tag:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .title_tag:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'type' => 'header',
		  				'class' => 'toggled_header row-header',
   						'title' => __('Desc', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .desc:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .desc:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .desc:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .desc:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .desc:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .desc:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'type' => 'header',
		  				'class' => 'toggled_header row-header',
		  				'themeoptions' => 'style:',
   						'title' => __('Divider', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .hr_narrow:background-color',
							'themeoptions' => 'style:',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .hr_narrow:margin',
		  				'themeoptions' => 'style:',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .hr_narrow:margin_laptop',
		  				'themeoptions' => 'style:',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .hr_narrow:margin_tablet',
		  				'themeoptions' => 'style:',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .hr_narrow:margin_mobile',
		  				'themeoptions' => 'style:',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .quick_fact .hr_narrow:width',
		  				'themeoptions' => 'style:',
		  				'type' => 'text',
		  				'default_unit' => '%',
		  				'title' => __('Width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'width',
		  			),


						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

				// share

				'share' => array(
   				'type' => 'share',
   				'title' => __('Share', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'elements',
   				'attr' => array(

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'type' => 'header',
							'title' => __('Copy link', 'mfn-opts'),
						),

						array(
							'id' => 'copy_link',
							'attr_id' => 'copy_link',
							're_render' => true,
							'type' => 'switch',
							'title' => __('Copy link', 'mfn-opts'),
							'options' => array(
								'1' => 'Show',
								'' 	=> 'Hide',
							),
							'std' => '1',
						),

						array(
							'id' => 'copy_link_icon',
							'condition' => array( 'id' => 'copy_link', 'opt' => 'is', 'val' => '1' ),
							'type' => 'icon',
							'title' => __('Icon', 'mfn-opts'),
							'preview' => 'icon',
							'std' => 'far fa-copy'
						),

						array(
							'id' => 'copy_link_label',
							'condition' => array( 'id' => 'copy_link', 'opt' => 'is', 'val' => '1' ),
							'type' => 'text',
							'title' => __('Label', 'mfn-opts'),
							'std' => __('Copy link', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'title' => __('Facebook', 'mfn-opts'),
						),

						array(
							'id' => 'facebook',
							'attr_id' => 'facebook',
							're_render' => true,
							'type' => 'switch',
							'title' => __('Facebook', 'mfn-opts'),
							'options' => array(
								'1' => 'Show',
								'' 	=> 'Hide',
							),
							'std' => '1',
						),

						array(
							'id' => 'facebook_icon',
							'condition' => array( 'id' => 'facebook', 'opt' => 'is', 'val' => '1' ),
							'type' => 'icon',
							'title' => __('Icon', 'mfn-opts'),
							'preview' => 'icon',
							'std' => 'icon-facebook'
						),

						array(
							'id' => 'facebook_label',
							'condition' => array( 'id' => 'facebook', 'opt' => 'is', 'val' => '1' ),
							'type' => 'text',
							'title' => __('Label', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'title' => __('X', 'mfn-opts'),
						),

						array(
							'id' => 'twitter',
							'attr_id' => 'twitter',
							're_render' => true,
							'type' => 'switch',
							'title' => __('X', 'mfn-opts'),
							'options' => array(
								'1' => 'Show',
								'' 	=> 'Hide',
							),
							'std' => '1',
						),

						array(
							'id' => 'twitter_icon',
							'condition' => array( 'id' => 'twitter', 'opt' => 'is', 'val' => '1' ),
							'type' => 'icon',
							'title' => __('Icon', 'mfn-opts'),
							'preview' => 'icon',
							'std' => 'icon-x-twitter'
						),

						array(
							'id' => 'twitter_label',
							'condition' => array( 'id' => 'twitter', 'opt' => 'is', 'val' => '1' ),
							'type' => 'text',
							'title' => __('Label', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'title' => __('LinkedIn', 'mfn-opts'),
						),

						array(
							'id' => 'linkedin',
							'attr_id' => 'linkedin',
							're_render' => true,
							'type' => 'switch',
							'title' => __('LinkedIn', 'mfn-opts'),
							'options' => array(
								'1' => 'Show',
								'' 	=> 'Hide',
							),
							'std' => '1',
						),

						array(
							'id' => 'linkedin_icon',
							'condition' => array( 'id' => 'linkedin', 'opt' => 'is', 'val' => '1' ),
							'type' => 'icon',
							'title' => __('Icon', 'mfn-opts'),
							'preview' => 'icon',
							'std' => 'icon-linkedin'
						),

						array(
							'id' => 'linkedin_label',
							'condition' => array( 'id' => 'linkedin', 'opt' => 'is', 'val' => '1' ),
							'type' => 'text',
							'title' => __('Label', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render modalbox-card-style-mfnuidhere">',
						),


						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Container', 'mfn-opts'),
   					),

						array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post:justify-content',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post:justify-content_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post:justify-content_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post:justify-content_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Items', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),


   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post:gap',
							'type' => 'sliderbar',
							'title' => __('Gap', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post:gap_laptop',
							'type' => 'sliderbar',
							'title' => __('Gap', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post:gap_tablet',
							'type' => 'sliderbar',
							'title' => __('Gap', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post:gap_mobile',
							'type' => 'sliderbar',
							'title' => __('Gap', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button:border-style',
							'attr_id' => 'border_sharepost_button',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button:border-width',
		  				'condition' => array( 'id' => 'border_sharepost_button', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button:border-width_laptop',
		  				'condition' => array( 'id' => 'border_sharepost_button', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button:border-width_tablet',
		  				'condition' => array( 'id' => 'border_sharepost_button', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button:border-width_mobile',
		  				'condition' => array( 'id' => 'border_sharepost_button', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post-button:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Link color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post-button:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post-button:border-color',
							'type' => 'color',
							'condition' => array( 'id' => 'border_sharepost_button', 'opt' => 'isnt', 'val' => 'none' ),
							'class' => 'main-color mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post-button|hover:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Link color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post-button|hover:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post-button|hover:border-color',
							'type' => 'color',
							'condition' => array( 'id' => 'border_sharepost_button', 'opt' => 'isnt', 'val' => 'none' ),
							'class' => 'main-color mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),



						array(
							'type' => 'header',
							'condition' => array( 'id' => 'facebook', 'opt' => 'is', 'val' => '1' ),
							'class' => 'toggled_header row-header',
   						'title' => __('Facebook', 'mfn-opts'),
   					),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button.mfn-share-post-facebook:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Link color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button.mfn-share-post-facebook:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button.mfn-share-post-facebook:border-color',
							'type' => 'color',
							'condition' => array( 'id' => 'border_sharepost_button', 'opt' => 'isnt', 'val' => 'none' ),
							'class' => 'main-color mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button.mfn-share-post-facebook|hover:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Link color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button.mfn-share-post-facebook|hover:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button.mfn-share-post-facebook|hover:border-color',
							'type' => 'color',
							'condition' => array( 'id' => 'border_sharepost_button', 'opt' => 'isnt', 'val' => 'none' ),
							'class' => 'main-color mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

   					array(
							'type' => 'header',
							'condition' => array( 'id' => 'twitter', 'opt' => 'is', 'val' => '1' ),
							'class' => 'toggled_header row-header',
   						'title' => __('X', 'mfn-opts'),
   					),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button.mfn-share-post-twitter:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Link color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button.mfn-share-post-twitter:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button.mfn-share-post-twitter:border-color',
							'type' => 'color',
							'condition' => array( 'id' => 'border_sharepost_button', 'opt' => 'isnt', 'val' => 'none' ),
							'class' => 'main-color mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button.mfn-share-post-twitter|hover:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Link color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button.mfn-share-post-twitter|hover:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button.mfn-share-post-twitter|hover:border-color',
							'type' => 'color',
							'condition' => array( 'id' => 'border_sharepost_button', 'opt' => 'isnt', 'val' => 'none' ),
							'class' => 'main-color mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

   					array(
							'type' => 'header',
							'condition' => array( 'id' => 'linkedin', 'opt' => 'is', 'val' => '1' ),
							'class' => 'toggled_header row-header',
   						'title' => __('LinkedIn', 'mfn-opts'),
   					),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button.mfn-share-post-linkedin:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Link color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button.mfn-share-post-linkedin:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button.mfn-share-post-linkedin:border-color',
							'type' => 'color',
							'condition' => array( 'id' => 'border_sharepost_button', 'opt' => 'isnt', 'val' => 'none' ),
							'class' => 'main-color mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button.mfn-share-post-linkedin|hover:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Link color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button.mfn-share-post-linkedin|hover:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-share-post .mfn-share-post-button.mfn-share-post-linkedin|hover:border-color',
							'type' => 'color',
							'condition' => array( 'id' => 'border_sharepost_button', 'opt' => 'isnt', 'val' => 'none' ),
							'class' => 'main-color mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),


   				)
   			),



   			// Shop Categories ----------------------------------------------------

				'shop_categories' => array(
					'type' => 'shop_categories',
					'title' => __('Shop categories', 'mfn-opts'),
					'size' => '1/1',
					'tablet_size' => '1/1',
					'mobile_size' => '1/1',
					'tablet_resized' => '0',
					'cat' => 'loops',
					'attr' => array(
					//'fields' => array(

						/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						// options

						array(
							'type' => 'header',
							'title' => __('Options', 'mfn-opts'),
						),

						array(
   						'id' => 'columns',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Columns', 'mfn-opts'),
							'options' => array(
								2 => 2,
								3 => 3,
								4 => 4,
							),
   						'std' => '3',
   					),

   					array(
   						'id' => 'display',
   						'attr_id' => 'shop_cat_display',
   						'type' => 'select',
   						'title' => __('Display', 'mfn-opts'),
							'options' => array(
								'' => __('Default', 'mfn-opts'),
								'1' => __('Main categories', 'mfn-opts'),
							),
							'std' => '',
   					),

						array(
   						'id' => 'category',
   						'condition' => array( 'id' => 'shop_cat_display', 'opt' => 'isnt', 'val' => '1' ),
   						're_render' => true,
   						'type' => 'select',
   						'title' => __('Parent category', 'mfn-opts'),
   						'php_options' => mfn_get_categories('product_cat'),
   						'js_hierarchical_options' => 'product_cat',
							'preview' => 'category',
   					),

   					array(
   						'id' => 'subcategory',
   						'type' => 'select',
   						'title' => __('Subcatories display', 'mfn-opts'),
							'options' => array(
								'' => __('Default', 'mfn-opts'),
								'1' => __('Subcatories', 'mfn-opts'),
							),
							'std' => '',
   					),

						array(
   						'id' => 'empty',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Empty categories', 'mfn-opts'),
   						'desc' => __('Show categories without products', 'mfn-opts'),
							'options' => array(
								1 => __('Hide', 'mfn-opts'),
								0 => __('Show', 'mfn-opts'),
							),
   						'std' => 1,
   					),

						// advanced

						array(
							'type' => 'header',
							'title' => __('Advanced', 'mfn-opts'),
						),

						array(
   						'id' => 'image',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Image', 'mfn-opts'),
							'options' => array(
								0 => __('Hide', 'mfn-opts'),
								1 => __('Show', 'mfn-opts'),
							),
   						'std' => 1,
   					),

						array(
   						'id' => 'title',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Title', 'mfn-opts'),
							'options' => array(
								0 => __('Hide', 'mfn-opts'),
								1 => __('Show', 'mfn-opts'),
							),
   						'std' => 1,
   					),

						array(
   						'id' => 'count',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Count', 'mfn-opts'),
							'desc' => __('Number of products in category', 'mfn-opts'),
							'options' => array(
								0 => __('Hide', 'mfn-opts'),
								1 => __('Show', 'mfn-opts'),
							),
   						'std' => 0,
   					),

   					array(
							'id' => 'title_tag',
							'type' => 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => [
								'h1' => 'H1',
								'h2' => 'H2',
								'h3' => 'H3',
								'h4' => 'H4',
								'h5' => 'H5',
								'h6' => 'H6',
								'p' => 'p',
								'span' => 'span',
							],
							'std' => 'h2',
						),

   					// order

						array(
							'type' => 'header',
							'title' => __('Order', 'mfn-opts'),
						),

						array(
							'id' => 'order',
							'type' => 'order',
							're_render' => true,
							'title' => __('Order', 'mfn-opts'),
							'std' => 'image,title',
						),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Container', 'mfn-opts'),
   					),

						array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:text-align',
   						'oldid' => 'style:.woocommerce-loop-category__title:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:border-style',
							'attr_id' => 'border_style_sc-li',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:border-color',
							'condition' => array( 'id' => 'border_style_sc-li', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:border-width',
		  				'condition' => array( 'id' => 'border_style_sc-li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_sc-li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_sc-li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_sc-li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Image', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product img:border-style',
							'attr_id' => 'border_style_sc-img',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product img:border-color',
							'condition' => array( 'id' => 'border_style_sc-img', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product img:border-width',
		  				'condition' => array( 'id' => 'border_style_sc-img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product img:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_sc-img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product img:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_sc-img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product img:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_sc-img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product img:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product img:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product img:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product img:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .woocommerce-loop-category__title:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .woocommerce-loop-category__title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .woocommerce-loop-category__title:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .woocommerce-loop-category__title:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .woocommerce-loop-category__title:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .woocommerce-loop-category__title:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Count', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .woocommerce-loop-category__title .count:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .woocommerce-loop-category__title .count:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
   						'type' => 'html',
   						'html' => '</div>',
   					),

					),
				),

   			// Shop ----------------------------------------------------

   			'shop' => array(
   				'type' => 'shop',
   				'title' => __('Shop', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'loops',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'limit',
   						're_render' => true,
   						'type' => 'text',
   						'title' => __('Products number', 'mfn-opts'),
   						'std' => '6',
   						'after' => 'products',
   						'param' => 'number',
   						'class' => 'narrow',
							'preview' => 'number',
   					),

   					array(
   						'id' => 'columns',
   						're_render' => true,
   						//'responsive' => 'desktop',
   						'type' => 'switch',
   						'title' => __('Columns', 'mfn-opts'),
							'options' => array(
								2 => 2,
								3 => 3,
								4 => 4,
							),
   						'std' => '3',
   					),

						array(
   						'id' => 'type',
   						're_render' => true,
   						'type' => 'select',
   						'title' => __('Display', 'mfn-opts'),
   						'options' => array(
   							'products' => __('- Default -', 'mfn-opts'),
   							'sale_products' => __('On sale', 'mfn-opts'),
   							'best_selling_products' => __('Best selling (order by: Sales)', 'mfn-opts'),
   							'top_rated_products' => __('Top-rated (order by: Rating)', 'mfn-opts'),
   						),
   					),

						// options

   					array(
   						'type' => 'header',
   						'title' => __('Options', 'mfn-opts'),
   					),

   					array(
   						'id' => 'category',
   						're_render' => true,
   						'type' => 'select',
   						'title' => __('Category', 'mfn-opts'),
   						'php_options' => mfn_get_categories('product_cat'),
   						'js_hierarchical_options' => 'product_cat',
							'preview' => 'category',
   					),

   					array(
   						'id' => 'orderby',
   						're_render' => true,
   						'type' => 'select',
   						'title' => __('Order by', 'mfn-opts'),
   						'options' => array(
   							'date' => __('Date the product was published', 'mfn-opts'),
   							'id' => __('ID of the product', 'mfn-opts'),
   							'menu_order' => __('Menu order (if set)', 'mfn-opts'),
   							'popularity' => __('Popularity (number of purchases)', 'mfn-opts'),
   							'rating' => __('Rating', 'mfn-opts'),
   							'title' => __('Title', 'mfn-opts'),
								'rand' => __('Random (do not use with pagination)', 'mfn-opts'),
   						),
   						'std' => 'title'
   					),

   					array(
   						'id' => 'order',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Order', 'mfn-opts'),
   						'options' => array(
   							'ASC' => __('Ascending', 'mfn-opts'),
   							'DESC' => __('Descending', 'mfn-opts'),
   						),
   						'std' => 'ASC'
   					),

						// advanced

						array(
							'type' => 'header',
							'title' => __('Advanced', 'mfn-opts'),
						),

						array(
   						'id' => 'paginate',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Pagination', 'mfn-opts'),
   						'options' => array(
   							0 => __('Hide', 'mfn-opts'),
   							1 => __('Show', 'mfn-opts'),
   						),
   						'std' => 0,
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Container', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:border-style',
							'attr_id' => 'border_style_sc-li',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:border-color',
							'condition' => array( 'id' => 'border_style_sc-li', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:border-width',
		  				'condition' => array( 'id' => 'border_style_sc-li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_sc-li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_sc-li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_sc-li', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Image', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .image_frame:border-style',
							'attr_id' => 'border_style_sc-img',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .image_frame:border-color',
							'condition' => array( 'id' => 'border_style_sc-img', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .image_frame:border-width',
		  				'condition' => array( 'id' => 'border_style_sc-img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .image_frame:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_sc-img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .image_frame:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_sc-img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .image_frame:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_sc-img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .image_frame:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .image_frame:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .image_frame:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .image_frame:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .desc:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

		  			array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .desc:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .desc:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .desc:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .desc:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .desc:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .desc:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .desc:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .desc:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .desc:padding',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .desc:padding_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .desc:padding_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .desc:padding_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product h4 a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product h4:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product h4:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product h4:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product h4:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product h4:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Price', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .price:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .price:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .price:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .price:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .price:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .price:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Regular price', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .price del:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .price del:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Excerpt', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .excerpt:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .excerpt:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .excerpt:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .excerpt:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .excerpt:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .excerpt:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Button', 'mfn-opts'),
   					),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .button:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .button:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .button:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .button:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .button:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .button:border-style',
							'attr_id' => 'border_style_shop',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .button:border-width',
		  				'condition' => array( 'id' => 'border_style_shop', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .button:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_shop', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .button:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_shop', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .button:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_shop', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .button:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .button:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .button:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .button:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .button:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .button:border-color',
							'condition' => array( 'id' => 'border_style_shop', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher',
							'attr_id' => 'background_switcher_shop',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .button:background-color',
							'condition' => array( 'id' => 'background_switcher_shop', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .button:gradient',
		  				'condition' => array( 'id' => 'background_switcher_shop', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .button|hover:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .button|hover:border-color',
							'condition' => array( 'id' => 'border_style_shop', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher_hover',
							'attr_id' => 'background_switcher_hover_shop',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .button|hover:background',
							'condition' => array( 'id' => 'background_switcher_hover_shop', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .woocommerce ul.products li.product .button|hover:gradient',
		  				'condition' => array( 'id' => 'background_switcher_hover_shop', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Shop Slider ----------------------------------------------------

   			'shop_slider' => array(
   				'type' => 'shop_slider',
   				'title' => __('Shop Slider', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'loops',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'title',
   						'type' => 'text',
						'dynamic_data' => 'title',
   						'edit_tag' => '.blog_slider_header | .title',
   						'edit_position' => 'prepend',
   						'title' => __('Title', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h4',
						),

   					array(
   						'id' => 'count',
   						're_render' => true,
   						'type' => 'text',
   						'title' => __('Products number', 'mfn-opts'),
   						'std' => '5',
							'after' => 'products',
   						'param' => 'number',
   						'class' => 'narrow',
							'preview' => 'number',
   					),

   					array(
   						'id' => 'show',
   						're_render' => true,
   						'type' => 'select',
   						'title' => __('Display', 'mfn-opts'),
   						'options' => array(
   							'' => __('All (or category selected below)', 'mfn-opts'),
   							'featured' => __('Featured', 'mfn-opts'),
   							'onsale' => __('Onsale', 'mfn-opts'),
   							'best-selling' => __('Best Selling (order by: Sales)', 'mfn-opts'),
   						),
   					),

   					array(
   						'id' => 'out_of_stock',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Out of stock products', 'mfn-opts'),
   						'options' => array(
								'hide' => __('Hide', 'mfn-opts'),
   							'' => __('Show', 'mfn-opts'),
   						),
							'std' => '',
   					),

						// options

   					array(
   						'type' => 'header',
   						'title' => __('Options', 'mfn-opts'),
   					),

						array(
   						'id' => 'category',
   						're_render' => true,
   						'type' => 'select',
   						'title' => __('Category', 'mfn-opts'),
   						'php_options' => mfn_get_categories('product_cat'),
   						'js_hierarchical_options' => 'product_cat',
							'preview' => 'category',
   					),

   					array(
   						'id' => 'orderby',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Order by', 'mfn-opts'),
   						'options' => array(
   							'date' => __('Date', 'mfn-opts'),
   							'title' => __('Title', 'mfn-opts'),
   						),
   						'std' => 'date'
   					),

   					array(
   						'id' => 'order',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Order', 'mfn-opts'),
   						'options' => array(
   							'ASC' => __('Ascending', 'mfn-opts'),
   							'DESC' => __('Descending', 'mfn-opts'),
   						),
   						'std' => 'DESC'
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Item title', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_header .title:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_header .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_header .title:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_header .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_header .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .blog_slider_header .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Image', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .image_frame:border-style',
							'attr_id' => 'border_style_sc-img',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .image_frame:border-color',
							'condition' => array( 'id' => 'border_style_sc-img', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .image_frame:border-width',
		  				'condition' => array( 'id' => 'border_style_sc-img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .image_frame:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_sc-img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .image_frame:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_sc-img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .image_frame:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_sc-img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .image_frame:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .image_frame:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .image_frame:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .image_frame:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .desc:background-color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

		  			array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .desc:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .desc:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .desc:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .desc:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .desc:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .desc:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .desc:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .desc:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .desc:padding',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .desc:padding_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .desc:padding_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .desc:padding_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li h4 a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li h4:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li h4:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li h4:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li h4:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li h4:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Price', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .price:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .price:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .price:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .price:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .price:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .price:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Regular price', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .price del:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .shop_slider_ul li .price del:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Arrows', 'mfn-opts'),
   					),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .button:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .button:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .button:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .button:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .button:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .button:border-style',
							'attr_id' => 'border_style_shopslider',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .button:border-width',
		  				'condition' => array( 'id' => 'border_style_shopslider', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .button:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_shopslider', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .button:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_shopslider', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .button:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_shopslider', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .button:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .button:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .button:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .button:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .button,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .button .button_icon i:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .button:border-color',
							'condition' => array( 'id' => 'border_style_shopslider', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher',
							'attr_id' => 'background_switcher_shopslider',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .button:background-color',
							'condition' => array( 'id' => 'background_switcher_shopslider', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .button:gradient',
		  				'condition' => array( 'id' => 'background_switcher_shopslider', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .button|hover .button_icon i:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .button|hover:border-color',
							'condition' => array( 'id' => 'border_style_shopslider', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher_hover_shopslider',
							'attr_id' => 'background_switcher_hover_shopslider',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .button|hover:background',
							'condition' => array( 'id' => 'background_switcher_hover_shopslider', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .shop_slider .button|hover:gradient',
		  				'condition' => array( 'id' => 'background_switcher_hover_shopslider', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Sidebar Widget -------------------------------------------------

   			'sidebar_widget' => array(
   				'type' => 'sidebar_widget',
   				'title' => __('Sidebar Widget', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/4',
   				'tablet_resized' => '0',
   				'cat' => 'other',
   				'attr' => array(
   				//'fields' => array(

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'sidebar',
   						'type' => 'select',
   						'title' => __('Select Sidebar', 'mfn-opts'),
   						'desc' => __('1. Create Sidebar in <a target="_blank" href="admin.php?page=be-options#sidebars">Theme Options > Sidebars</a><br />2. Add Widgets<br />3. Select your sidebar.', 'mfn-opts'),
   						'php_options' => mfn_opts_get('sidebars'),
   						'js_options' => 'sidebars',
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Slider ---------------------------------------------------------

   			'slider' => array(
   				'type' => 'slider',
   				'title' => __('Slider', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'blocks',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'category',
   						're_render' => true,
   						'type' => 'select',
   						'title' => __('Category', 'mfn-opts'),
   						'php_options' => mfn_get_categories('slide-types'),
   						'js_hierarchical_options' => 'slide_types',
							'preview' => 'category',
   					),

   					array(
   						'id' => 'orderby',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Order by', 'mfn-opts'),
   						'options' => array(
   							'date' => __('Date', 'mfn-opts'),
   							'menu_order' => __('Menu order', 'mfn-opts'),
   							'title' => __('Title', 'mfn-opts'),
   						),
   						'std' => 'date'
   					),

   					array(
   						'id' => 'order',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Order', 'mfn-opts'),
   						'options' => array(
   							'ASC' => __('Ascending', 'mfn-opts'),
   							'DESC' => __('Descending', 'mfn-opts'),
   						),
   						'std' => 'DESC'
   					),

   					// advanced

   					array(
   						'type' => 'header',
   						'title' => __('Advanced', 'mfn-opts'),
   					),

   					array(
   						'id' => 'style',
   						're_render' => true,
   						'type' => 'select',
							'title' => __('Style', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'flat' => __('Flat', 'mfn-opts'),
   							'description' => __('Flat with title and description', 'mfn-opts'),
   							'carousel' => __('Flat carousel with titles', 'mfn-opts'),
   							'center' => __('Center mode', 'mfn-opts'),
   						),
							'preview' => 'style',
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h3',
						),

   					array(
   						'id' => 'navigation',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Navigation', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'hide-arrows' => __('Hide Arrows', 'mfn-opts'),
   							'hide-dots' => __('Hide Dots', 'mfn-opts'),
   							'hide' => __('Hide', 'mfn-opts'),
   						),
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Image', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider.default .content_slider_ul .slick-list,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider|not(.default) .content_slider_ul li img:border-style',
							'attr_id' => 'border_style_slider_img',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider.default .content_slider_ul .slick-list,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider|not(.default) .content_slider_ul li img:border-color',
							'condition' => array( 'id' => 'border_style_slider_img', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider.default .content_slider_ul .slick-list,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider|not(.default) .content_slider_ul li img:border-width',
		  				'condition' => array( 'id' => 'border_style_slider_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider.default .content_slider_ul .slick-list,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider|not(.default) .content_slider_ul li img:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_slider_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider.default .content_slider_ul .slick-list,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider|not(.default) .content_slider_ul li img:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_slider_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider.default .content_slider_ul .slick-list,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider|not(.default) .content_slider_ul li img:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_slider_img', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider.default .content_slider_ul .slick-list,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider|not(.default) .content_slider_ul li img:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider.default .content_slider_ul .slick-list,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider|not(.default) .content_slider_ul li img:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider.default .content_slider_ul .slick-list,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider|not(.default) .content_slider_ul li img:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider.default .content_slider_ul .slick-list,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider|not(.default) .content_slider_ul li img:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider.default .content_slider_ul,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider|not(.default) .content_slider_ul li img:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider.default .content_slider_ul,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider|not(.default) .content_slider_ul li img:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider.default .content_slider_ul,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider|not(.default) .content_slider_ul li img:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider.default .content_slider_ul,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider|not(.default) .content_slider_ul li img:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider ul li .title:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider ul li .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Excerpt', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider ul li .desc:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider ul li .desc:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Arrows', 'mfn-opts'),
   					),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider .content_slider_ul .button:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider .content_slider_ul .button:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider .content_slider_ul .button:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider .content_slider_ul .button:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider .content_slider_ul .button:border-style',
							'attr_id' => 'border_style_slider',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider .content_slider_ul .button:border-width',
		  				'condition' => array( 'id' => 'border_style_slider', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider .content_slider_ul .button:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_slider', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider .content_slider_ul .button:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_slider', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider .content_slider_ul .button:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_slider', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider .content_slider_ul .button:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider .content_slider_ul .button:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider .content_slider_ul .button:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider .content_slider_ul .button:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider .content_slider_ul .button:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider .content_slider_ul .button:border-color',
							'condition' => array( 'id' => 'border_style_slider', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher',
							'attr_id' => 'background_switcher_slider',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider .content_slider_ul .button:background-color',
							'condition' => array( 'id' => 'background_switcher_slider', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider .content_slider_ul .button:gradient',
		  				'condition' => array( 'id' => 'background_switcher_slider', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider .content_slider_ul .button|hover:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider .content_slider_ul .button|hover:border-color',
							'condition' => array( 'id' => 'border_style_slider', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher_hover',
							'attr_id' => 'background_switcher_hover_slider',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider .content_slider_ul .button|hover:background',
							'condition' => array( 'id' => 'background_switcher_hover_slider', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_slider .content_slider_ul .button|hover:gradient',
		  				'condition' => array( 'id' => 'background_switcher_hover_slider', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Dots', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .slider_pagination a,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .slider_pagination a|after:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .slider_pagination .slick-active a,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .slider_pagination .slick-active a|after:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Active color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Slider Plugin --------------------------------------------------

   			'slider_plugin' => array(
   				'type' => 'slider_plugin',
   				'title' => __('Slider Plugin', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'other',
   				'attr' => array(
   				//'fields' => array(

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'type' => 'header',
   						'title' => __('Slider Revolution', 'mfn-opts'),
   					),

   					array(
   						'id' => 'rev',
   						'type' => 'select',
   						'title' => __('Slider', 'mfn-opts'),
   						'js_options' => 'rev_slider',
							'php_options' => $this->sliders['rev'],
   						'preview' => 'slider-rev',
   					),

						// layer slider

   					array(
   						'type' => 'header',
   						'title' => __('Layer Slider', 'mfn-opts'),
   					),

   					array(
   						'id' => 'layer',
   						'type' => 'select',
   						'title' => __('Slider', 'mfn-opts'),
   						'js_options' => 'layer_slider',
							'php_options' => $this->sliders['layer'],
							'preview' => 'slider-layer',
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

   					array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Sliding Box ----------------------------------------------------

   			'sliding_box' => array(
   				'type' => 'sliding_box',
   				'title' => __('Sliding Box', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'boxes',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'image',
   						'dynamic_data' => 'featured_image',
   						'type' => 'upload',
   						'title' => __('Image', 'mfn-opts'),
   						'desc' => __('Recommended image width <b>768px - 1920px</b> depending on size of the item', 'mfn-opts'),
							'std' => $this->get_placeholder(),
							'preview' => 'image',
   					),

   					array(
   						'id' => 'title',
   						'type' => 'text',
						'dynamic_data' => 'title',
   						'edit_tag' => '.desc_wrapper | .title',
   						'edit_position' => 'prepend',
   						'title' => __('Title', 'mfn-opts'),
   						'desc' => __('Allowed HTML tags: span, strong, b, em, i, u', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h4',
						),

						// link

						array(
							'type' => 'header',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link',
   						'type' => 'text',
   						'dynamic_data' => 'permalink',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'target',
   						'type' => 'select',
   						'title' => __('Target', 'mfn-opts'),
   						'options' => array(
   							0 => __('Default | _self', 'mfn-opts'),
   							1 => __('New tab or window | _blank', 'mfn-opts'),
   							'lightbox' => __('Lightbox (image or embed video)', 'mfn-opts'),
   						),
   					),

            array(
							'id' => 'link_title',
							'type' => 'text',
							'dynamic_data' => 'title',
							'title' => __('Link title', 'mfn-opts'),
						),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .sliding_box .desc_wrapper .title:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .sliding_box .desc_wrapper:--mfn-sliding-box-bg',
							'themeoptions' => 'style:',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .sliding_box .desc_wrapper .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .sliding_box .desc_wrapper:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .sliding_box .desc_wrapper:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .sliding_box .desc_wrapper:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .sliding_box .desc_wrapper:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'themeoptions' => 'style:simple',
   						'title' => __('Divider', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .sliding_box .desc_wrapper|after:background-color',
							'themeoptions' => 'style:simple',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .sliding_box .desc_wrapper|after:margin',
		  				'type' => 'dimensions',
		  				'themeoptions' => 'style:simple',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .sliding_box .desc_wrapper|after:margin_laptop',
		  				'type' => 'dimensions',
		  				'themeoptions' => 'style:simple',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .sliding_box .desc_wrapper|after:margin_tablet',
		  				'type' => 'dimensions',
		  				'themeoptions' => 'style:simple',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .sliding_box .desc_wrapper|after:margin_mobile',
		  				'type' => 'dimensions',
		  				'themeoptions' => 'style:simple',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),


   			// Story Box ------------------------------------------------------

   			'story_box' => array(
   				'type' => 'story_box',
   				'title' => __('Story Box', 'mfn-opts'),
   				'size' => '1/2',
   				'tablet_size' => '1/2',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'boxes',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'image',
   						'dynamic_data' => 'featured_image',
   						'type' => 'upload',
   						'title' => __('Image', 'mfn-opts'),
   						'desc' => __('Recommended image width <b>750px - 1500px</b> depending on size of the item', 'mfn-opts'),
							'std' => $this->get_placeholder(),
							'preview' => 'image',
   					),

   					array(
   						'id' => 'style',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Style', 'mfn-opts'),
   						'options' => array(
   							'' => __('Horizontal Image', 'mfn-opts'),
   							'vertical' => __('Vertical Image', 'mfn-opts'),
   						),
							'std' => '',
   					),

   					array(
   						'id' => 'title',
   						'type' => 'text',
   						'edit_tag' => '.desc_wrapper | .title',
   						'edit_position' => 'prepend',
   						'dynamic_data' => 'title',
   						'title' => __('Title', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h3',
						),

   					array(
   						'id' => 'content',
   						'type' => 'textarea',
   						'edit_tag' => '.story_box | div.desc',
   						'dynamic_data' => 'content',
   						'edit_position' => 'append',
   						'title' => __('Content', 'mfn-opts'),
							'std' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut elit tellus, luctus nec ullamcorper mattis, pulvinar dapibus leo.',
							'preview' => 'content',
							'class' => 'form-content-full-width',
   					),

						// link

   					array(
   						'type' => 'header',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link',
   						'type' => 'text',
   						'dynamic_data' => 'permalink',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'target',
   						'type' => 'select',
   						'title' => __('Target', 'mfn-opts'),
   						'options' => array(
   							0 => __('Default | _self', 'mfn-opts'),
   							1 => __('New tab or window | _blank', 'mfn-opts'),
   							'lightbox' => __('Lightbox (image or embed video)', 'mfn-opts'),
   						),
   					),

            array(
							'id' => 'link_title',
							'type' => 'text',
							'dynamic_data' => 'title',
							'title' => __('Link title', 'mfn-opts'),
						),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Image', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .photo_wrapper:border-style',
							'attr_id' => 'border_style_slider',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .photo_wrapper:border-color',
							'condition' => array( 'id' => 'border_style_slider', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .photo_wrapper:border-width',
		  				'condition' => array( 'id' => 'border_style_slider', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .photo_wrapper:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_slider', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .photo_wrapper:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_slider', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .photo_wrapper:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_slider', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .photo_wrapper:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .photo_wrapper:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .photo_wrapper:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .photo_wrapper:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .photo_wrapper:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .photo_wrapper:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .photo_wrapper:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .photo_wrapper:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .desc_wrapper .title:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .desc_wrapper .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .desc_wrapper .title:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .desc_wrapper .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .desc_wrapper .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .desc_wrapper .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .desc:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .desc:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .desc:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .desc:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .desc:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .desc:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Line', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .desc_wrapper hr:background',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .desc_wrapper hr:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .desc_wrapper hr:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .desc_wrapper hr:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .story_box .desc_wrapper hr:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Tabs -----------------------------------------------------------

   			'tabs' => array(
   				'type' => 'tabs',
   				'title' => __('Tabs', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'blocks',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'title',
   						'type' => 'text',
						'dynamic_data' => 'title',
   						'edit_tag' => '.mcb-item-tabs-inner | .title',
   						'edit_position' => 'prepend',
   						'title' => __('Title', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

   					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h4',
						),

   					// tabs

   					array(
   						'type' => 'header',
   						'title' => __('Tabs', 'mfn-opts'),
   					),

   					array(
   						'id' => 'tabs',
   						'type' => 'tabs',
   						'title' => __('Tabs', 'mfn-opts'),
   						'desc' => __('<b>JavaScript</b> content like Google Maps and some plugins shortcodes do <b>not work</b> in tabs. You can use Drag & Drop to set the order', 'mfn-opts'),
							'options' => [
								'title' => [
									'input',
									__('Title', 'mfn-opts'),
									__('Sample tab', 'mfn-opts'),
								],
								'content' => [
									'textarea',
									__('Content', 'mfn-opts'),
									'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Mauris eu massa orci.',
								],
							],
							'std' => [
								0 => [
									'title' => __('This is the 1st tab', 'mfn-opts'),
									'content' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Mauris eu massa orci.',
								],
								1 => [
									'title' => __('This is the 2nd tab', 'mfn-opts'),
									'content' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Mauris eu massa orci.',
								],
							],
							'primary' => 'title',
							'preview' => 'tabs',
   					),

   					// options

   					array(
   						'type' => 'header',
   						'title' => __('Options', 'mfn-opts'),
   					),

   					array(
   						'id' => 'type',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Style', 'mfn-opts'),
   						'options' => array(
   							'horizontal' => __('Horizontal', 'mfn-opts'),
   							'centered' => __('Centered', 'mfn-opts'),
   							'vertical' => __('Vertical', 'mfn-opts'),
   						),
							'std' => 'horizontal',
   					),

   					// custom

   					array(
   						'type' => 'header',
   						'title' => __('Custom', 'mfn-opts'),
   					),

   					array(
   						'id' => 'uid',
   						'type' => 'text',
   						'title' => __('Unique ID [optional]', 'mfn-opts'),
   						'desc' => __('Use if you want to open specified tab from link (does not work on the same page).<br />For example: Your Unique ID is <b>offer</b> and you want to open 2nd tab, please use link: <b>your-url/#offer-2</b>', 'mfn-opts'),
   					),

   					array(
   						'type' => 'header',
   						'class' => 'mfn-deprecated',
   						'title' => __('Deprecated', 'mfn-opts'),
   					),

   					array(
   						'id' => 'padding',
   						'class' => 'mfn-deprecated',
   						'type' => 'text',
   						'title' => __('Content padding', 'mfn-opts'),
   						'desc' => __('Use value with <b>px</b> or <b>%</b><br />Example: <b>20px</b> or <b>15px 20px 20px</b> or <b>20px 1%</b>', 'mfn-opts'),
   						'placeholder' => '20px',
   					),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:text-align',
							'responsive' => 'desktop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:text-align_laptop',
							'responsive' => 'laptop',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:text-align_tablet',
							'responsive' => 'tablet',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),
   					array(
   						'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:text-align_mobile',
							'responsive' => 'mobile',
   						'type' => 'switch',
   						'class' => 'mfn-toggled',
   						'title' => __('Text align', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
   							'right' => __('Right', 'mfn-opts'),
   							'justify' => __('Justify', 'mfn-opts'),
   						),
							'preview' => 'align',
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Container', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .tabs_wrapper:--mfn-tabs-border-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .tabs_wrapper:--mfn-tabs-border-width',
		  				'type' => 'sliderbar',
		  				'class' => 'mfn-toggled',
							'title' => __('Border width', 'mfn-opts'),
							'param' => array(
								'min' => '0',
								'max' => '5',
								'step' => '1',
								'unit' => 'px',
							)
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .tabs_wrapper:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .tabs_wrapper:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .tabs_wrapper:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .tabs_wrapper:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title bar', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .tabs_wrapper .ui-tabs-nav:background',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title tab', 'mfn-opts'),
   					),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .tabs_wrapper ul li a:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .tabs_wrapper ul li a:background',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .tabs_wrapper ul li.ui-state-active a:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .tabs_wrapper ul li.ui-state-active a:background',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .ui-tabs .ui-tabs-nav li.ui-state-active a|after:background',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Line', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .tabs_wrapper ul li a:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .tabs_wrapper ul li a:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .tabs_wrapper ul li a:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .tabs_wrapper ul li a:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .tabs_wrapper ul li a:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .ui-tabs .ui-tabs-panel:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .ui-tabs .ui-tabs-panel,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .ui-tabs .ui-tabs-nav li.ui-state-active|after:background',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .ui-tabs .ui-tabs-panel:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .ui-tabs .ui-tabs-panel:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .ui-tabs .ui-tabs-panel:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .ui-tabs .ui-tabs-panel:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .ui-tabs .ui-tabs-panel:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

				// Tag cloud -----------------------------------------------------------

   			'tag_cloud' => array(
   				'type' => 'tag_cloud',
   				'title' => __('Tag cloud', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'blocks',
   				'attr' => array(

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'category',
   						're_render' => true,
   						'type' => 'select',
   						'title' => __('Taxonomy', 'mfn-opts'),
   						'php_options' => mfna_taxonomies(),
   						'js_options' => 'taxonomies',
   						'std' => 'category'
   					),

   					array(
   						'id' => 'reference',
   						'attr_id' => 'reference',
   						're_render' => true,
   						'type' => 'select',
   						'title' => __('Reference', 'mfn-opts'),
   						'options' => array(
   							'' => __('All items', 'mfn-opts'),
   							'not_empty' => __('Not empty items', 'mfn-opts'),
   							'post' => __('Current post terms', 'mfn-opts'),
   						),
   						'std' => ''
   					),

   					array(
   						'id' => 'design',
   						'attr_id' => 'design',
   						're_render' => true,
   						'type' => 'select',
   						'title' => __('Design', 'mfn-opts'),
   						'options' => array(
   							'' => __('Simple list', 'mfn-opts'),
   							'pills' => __('Pills', 'mfn-opts'),
   						),
   						'std' => ''
   					),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Item', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-tag-cloud:justify-content',
							'type' => 'select',
							'title' => __('Align', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'options' => array(
								'' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'flex-end' => __('Right', 'mfn-opts'),
								'space-between' => __('Space between', 'mfn-opts'),
								'space-around' => __('Space around', 'mfn-opts')
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-tag-cloud:justify-content_laptop',
							'type' => 'select',
							'title' => __('Align', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'options' => array(
								'flex-start' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'flex-end' => __('Right', 'mfn-opts'),
								'space-between' => __('Space between', 'mfn-opts'),
								'space-around' => __('Space around', 'mfn-opts')
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-tag-cloud:justify-content_tablet',
							'type' => 'select',
							'title' => __('Align', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'options' => array(
								'flex-start' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'flex-end' => __('Right', 'mfn-opts'),
								'space-between' => __('Space between', 'mfn-opts'),
								'space-around' => __('Space around', 'mfn-opts')
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-tag-cloud:justify-content_mobile',
							'type' => 'select',
							'title' => __('Align', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'options' => array(
								'flex-start' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'flex-end' => __('Right', 'mfn-opts'),
								'space-between' => __('Space between', 'mfn-opts'),
								'space-around' => __('Space around', 'mfn-opts')
							)
						),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-tag-cloud:--mfn-tag-cloud-offset',
							'type' => 'sliderbar',
							'title' => __('Gap', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-tag-cloud:--mfn-tag-cloud-offset_laptop',
							'type' => 'sliderbar',
							'title' => __('Gap', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-tag-cloud:--mfn-tag-cloud-offset_tablet',
							'type' => 'sliderbar',
							'title' => __('Gap', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-tag-cloud:--mfn-tag-cloud-offset_mobile',
							'type' => 'sliderbar',
							'title' => __('Gap', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-tag-cloud li a:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-tag-cloud.mfn-tag-cloud-pills li a:padding',
		  				'condition' => array( 'id' => 'design', 'opt' => 'is', 'val' => 'pills' ),
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-tag-cloud.mfn-tag-cloud-pills li a:padding_laptop',
		  				'condition' => array( 'id' => 'design', 'opt' => 'is', 'val' => 'pills' ),
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-tag-cloud.mfn-tag-cloud-pills li a:padding_tablet',
		  				'condition' => array( 'id' => 'design', 'opt' => 'is', 'val' => 'pills' ),
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-tag-cloud.mfn-tag-cloud-pills li a:padding_mobile',
		  				'condition' => array( 'id' => 'design', 'opt' => 'is', 'val' => 'pills' ),
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-tag-cloud.mfn-tag-cloud-pills li a:border-style',
							'attr_id' => 'border_style_tags',
							'condition' => array( 'id' => 'design', 'opt' => 'is', 'val' => 'pills' ),
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-tag-cloud.mfn-tag-cloud-pills li a:border-width',
		  				'condition' => array( 'id' => 'border_style_tags', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-tag-cloud.mfn-tag-cloud-pills li a:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_tags', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-tag-cloud.mfn-tag-cloud-pills li a:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_tags', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-tag-cloud.mfn-tag-cloud-pills li a:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_tags', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-tag-cloud.mfn-tag-cloud-pills li a:border-radius',
		  				'type' => 'dimensions',
		  				'condition' => array( 'id' => 'design', 'opt' => 'is', 'val' => 'pills' ),
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-tag-cloud.mfn-tag-cloud-pills li a:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'condition' => array( 'id' => 'design', 'opt' => 'is', 'val' => 'pills' ),
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-tag-cloud.mfn-tag-cloud-pills li a:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'condition' => array( 'id' => 'design', 'opt' => 'is', 'val' => 'pills' ),
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-tag-cloud.mfn-tag-cloud-pills li a:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'condition' => array( 'id' => 'design', 'opt' => 'is', 'val' => 'pills' ),
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-tag-cloud li a:color',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-tag-cloud li a:background-color',
							'type' => 'color',
							'condition' => array( 'id' => 'design', 'opt' => 'is', 'val' => 'pills' ),
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-tag-cloud li a:border-color',
							'type' => 'color',
							'condition' => array( 'id' => 'border_style_tags', 'opt' => 'isnt', 'val' => 'none' ),
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-tag-cloud li a|hover:color',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-tag-cloud li a|hover:background-color',
							'type' => 'color',
							'condition' => array( 'id' => 'design', 'opt' => 'is', 'val' => 'pills' ),
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-tag-cloud li a|hover:border-color',
							'type' => 'color',
							'condition' => array( 'id' => 'border_style_tags', 'opt' => 'isnt', 'val' => 'none' ),
							'title' => __('Border color', 'mfn-opts'),
						),


						array(
							'type' => 'html',
							'html' => '</div></div>',
						),


						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				)
   			),

   			// Testimonials ---------------------------------------------------

   			'testimonials' => array(
   				'type' => 'testimonials',
   				'title' => __('Testimonials', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'loops',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h5',
						),

   					array(
   						'id' => 'category',
   						're_render' => true,
   						'type' => 'select',
   						'title' => __('Category', 'mfn-opts'),
   						'php_options' => mfn_get_categories('testimonial-types'),
   						'js_hierarchical_options' => 'testimonial_types',
							'preview' => 'category',
   					),

   					array(
   						'id' => 'orderby',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Order by', 'mfn-opts'),
   						'options' => array(
   							'date' => __('Date', 'mfn-opts'),
   							'menu_order' => __('Menu order', 'mfn-opts'),
   							'title' => __('Title', 'mfn-opts'),
   						),
   						'std' => 'date',
   					),

   					array(
   						'id' => 'order',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Order', 'mfn-opts'),
   						'options' => array(
   							'ASC' => __('Ascending', 'mfn-opts'),
   							'DESC' => __('Descending', 'mfn-opts'),
   						),
   						'std' => 'DESC',
   					),

   					// options

   					array(
   						'type' => 'header',
   						'title' => __('Options', 'mfn-opts'),
   					),

   					array(
   						'id' => 'style',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Style', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'single-photo' => __('Single photo', 'mfn-opts'),
   						),
							'std' => '',
							'preview' => 'style',
   					),

   					array(
   						'id' => 'hide_photos',
   						're_render' => true,
   						'type' => 'switch',
   						'title' => __('Photos', 'mfn-opts'),
   						'options' => array(
								1 => __('Hide', 'mfn-opts'),
   							0 => __('Show', 'mfn-opts'),
   						),
							'std' => 0,
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Container', 'mfn-opts'),
   					),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider .testimonials_slider_ul li:padding',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider .testimonials_slider_ul li:padding_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider .testimonials_slider_ul li:padding_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider .testimonials_slider_ul li:padding_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Blockquote', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement  .testimonials_slider .testimonials_slider_ul li .bq_wrapper blockquote:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement  .testimonials_slider .testimonials_slider_ul li .bq_wrapper:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement  .testimonials_slider .testimonials_slider_ul li .bq_wrapper blockquote:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider .testimonials_slider_ul li .bq_wrapper:padding',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider .testimonials_slider_ul li .bq_wrapper:padding_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider .testimonials_slider_ul li .bq_wrapper:padding_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider .testimonials_slider_ul li .bq_wrapper:padding_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider .testimonials_slider_ul li .bq_wrapper:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider .testimonials_slider_ul li .bq_wrapper:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider .testimonials_slider_ul li .bq_wrapper:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider .testimonials_slider_ul li .bq_wrapper:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Blockquote icon', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider .blockquote .mfn-blockquote-icon:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),
						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Blockquote lines', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider .blockquote blockquote:text-decoration-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider .blockquote blockquote:text-underline-offset',
							'type' => 'text',
							'class' => 'mfn-toggled',
							'default_unit' => 'px',
							'title' => __('Offset', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider .blockquote blockquote:text-decoration-thickness',
							'type' => 'text',
							'class' => 'mfn-toggled',
							'default_unit' => 'px',
							'title' => __('Thickness', 'mfn-opts'),
						),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Dots', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .slider_pagination a:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .slider_pagination li.slick-active a,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .slider_pagination li.slick-active a|after:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color active', 'mfn-opts'),
						),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Author', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider .testimonials_slider_ul li .author .title a,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider .testimonials_slider_ul li .author .title:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider .testimonials_slider_ul li .author .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Company', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider .testimonials_slider_ul li .author .company:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider .testimonials_slider_ul li .author .company:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Arrows', 'mfn-opts'),
   					),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider a.button:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider a.button:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider a.button:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider a.button:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider a.button:border-style',
							'attr_id' => 'border_style_testi',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider a.button:border-width',
		  				'condition' => array( 'id' => 'border_style_testi', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider a.button:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_testi', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider a.button:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_testi', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider a.button:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_testi', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider a.button:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider a.button:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider a.button:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider a.button:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider a.button i:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider a.button:border-color',
							'condition' => array( 'id' => 'border_style_testi', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher',
							'attr_id' => 'background_switcher_testi',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider a.button:background-color',
							'condition' => array( 'id' => 'background_switcher_testi', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider a.button:gradient',
		  				'condition' => array( 'id' => 'background_switcher_testi', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider a.button|hover i:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider a.button|hover:border-color',
							'condition' => array( 'id' => 'border_style_testi', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'background_switcher_hover',
							'attr_id' => 'background_switcher_hover_testi',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider a.button|hover:background',
							'condition' => array( 'id' => 'background_switcher_hover_testi', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider a.button|hover:gradient',
		  				'condition' => array( 'id' => 'background_switcher_hover_testi', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Decoration', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider .testimonials_slider_ul li .hr_dots span:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Line', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider .slider_images li a|after:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'themeoptions' => 'style:',
   						'title' => __('Photos background', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider .slider_images,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_slider .slider_images|before:background-color',
							'type' => 'color',
							'themeoptions' => 'style:',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Testimonials List ----------------------------------------------

   			'testimonials_list' => array(
   				'type' => 'testimonials_list',
   				'title' => __('Testimonials List', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'loops',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h4',
						),

   					array(
   						'id' => 'category',
   						're_render' => true,
   						'type' => 'select',
   						'title' => __('Category', 'mfn-opts'),
   						'php_options' => mfn_get_categories('testimonial-types'),
   						'js_hierarchical_options' => 'testimonial_types',
							'preview' => 'category',
   					),

   					array(
   						'id' => 'orderby',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Order by', 'mfn-opts'),
   						'options' => array(
   							'date' => __('Date', 'mfn-opts'),
   							'menu_order' => __('Menu order', 'mfn-opts'),
   							'title' => __('Title', 'mfn-opts'),
   						),
   						'std' => 'date',
   					),

   					array(
   						'id' => 'order',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Order', 'mfn-opts'),
   						'options' => array(
   							'ASC' => __('Ascending', 'mfn-opts'),
   							'DESC' => __('Descending', 'mfn-opts'),
   						),
   						'std' => 'DESC',
   					),

						// options

   					array(
   						'type' => 'header',
   						'title' => __('Options', 'mfn-opts'),
   					),

   					array(
   						'id' => 'style',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Style', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'quote' => __('Quote above the author', 'mfn-opts'),
   						),
							'std' => '',
							'preview' => 'style',
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Image', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_list .image_frame:border-style',
							'attr_id' => 'border_style_testi_list',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_list .image_frame:border-color',
							'condition' => array( 'id' => 'border_style_testi_list', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_list .image_frame:border-width',
		  				'condition' => array( 'id' => 'border_style_testi_list', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_list .image_frame:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_testi_list', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_list .image_frame:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_testi_list', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_list .image_frame:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_testi_list', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_list .image_frame:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_list .image_frame:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_list .image_frame:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_list .image_frame:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Author', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_list .item .desc .title,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_list .item .desc .title a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_list .item .desc .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Company', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_list .item .desc p.subtitle:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_list .item .desc p.subtitle:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_list .item .desc p.subtitle:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_list .item .desc p.subtitle:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_list .item .desc p.subtitle:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_list .item .desc p.subtitle:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Divider', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_list .item .desc hr.hr_color:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Blockquote', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_list .blockquote blockquote:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_list .blockquote blockquote:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Blockquote icon', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_list .blockquote .mfn-blockquote-icon:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),
						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Blockquote lines', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_list .blockquote blockquote:text-decoration-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_list .blockquote blockquote:text-underline-offset',
							'type' => 'text',
							'class' => 'mfn-toggled',
							'default_unit' => 'px',
							'title' => __('Offset', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .testimonials_list .blockquote blockquote:text-decoration-thickness',
							'type' => 'text',
							'class' => 'mfn-toggled',
							'default_unit' => 'px',
							'title' => __('Thickness', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

				// Table of contents ---------------------------------------------

				'table_of_contents' => array(
					'type' => 'table_of_contents',
					'title' => __('Table of Contents', 'mfn-opts'),
					'size' => '1/1',
					'tablet_size' => '1/1',
					'mobile_size' => '1/1',
					'tablet_resized' => '0',
					'cat' => 'blocks',
					'attr' => array(
					//'fields' => array(

						// HTML content

						/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'id' => 'info',
							'type' => 'info',
							'class' => 'mfn-alert',
							'title' => __('This item will check all of the <b>Column Text</b> and <b>Heading</b> items to match tags stated in <b>Anchor by HTML tags</b> field below', 'mfn-opts'),
						),

						array(
							'id' => 'title',
							'attr_id' => 'toc_title',
							'type' => 'text',
							'dynamic_data' => 'title',
							'edit_tag' => '.table_of_content | .title-inner',
   						'edit_position' => 'prepend',
							'std' => 'Table of contents',
							'title' => __('Title', 'mfn-opts'),
							'preview' => 'title',
						),

						// options

   					array(
   						'type' => 'header',
   						'title' => __('Options', 'mfn-opts'),
   					),

						array(
							'id' => 'title_tag',
							'type' => 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => [
								'h1' => 'H1',
								'h2' => 'H2',
								'h3' => 'H3',
								'h4' => 'H4',
								'h5' => 'H5',
								'h6' => 'H6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							],
							'std' => 'h4',
						),

						array(
			  			'id' => 'tags_anchors',
		  				'type' => 'pills',
		  				're_render' => true,
							'std' => 'H1 H2 H3 H4 H5 H6',
							'desc' => 'Separated with space button.<br/>Maximal depth level: <b>3</b>',
		  				'title' => __('Anchor by HTML tags', 'mfn-opts'),
   					),

						array(
							'id' => 'marker_view',
							're_render' => true,
							'title' => __('Marker view', 'mfn-opts'),
							'type' => 'switch',
							'options' => [
								'numbers' => __('Numbers', 'mfn-opts'),
								'bullets' => __('Bullets', 'mfn-opts'),
							],
							'std' => 'numbers',
						),

						array(
							'id' => 'icon',
							'type' => 'icon',
							're_render' => true,
							'title' => __('Bullets icon', 'mfn-opts'),
							'desc' => __('Only for <b>Bullets</b> type of marker', 'mfn-opts'),
						),

            array(
							'id' => 'url_format',
							're_render' => true,
							'title' => __('Link format', 'mfn-opts'),
							'desc' => __('Use <b>Simple</b> for languages based on non URL friendly characters', 'mfn-opts'),
							'type' => 'switch',
							'options' => [
								'' => __('SEO friendly', 'mfn-opts'),
								'simple' => __('Simple', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'allow_hide',
							'attr_id' => 'toc_allow_hide',
							'type' => 'select',
							're_render' => true,
							'title' => __('Toggle the Visibility', 'mfn-opts'),
							'options' => [
								'' => __('Disable', 'mfn-opts'),
								'enable' => __('Enable', 'mfn-opts'),
								'hide' => __('Enable and hide initially', 'mfn-opts'),
							],
							'std' => '',
						),

						array(
							'id' => 'text_show',
							'type' => 'text',
							're_render' => true,
							'title' => __('Show text', 'mfn-opts'),
							'condition' => array( 'id' => 'toc_allow_hide', 'opt' => 'isnt', 'val' => '' ),
							'std' => __('Show', 'mfn-opts'),
						),

						array(
							'id' => 'text_hide',
							'type' => 'text',
							're_render' => true,
							'title' => __('Hide text', 'mfn-opts'),
							'condition' => array( 'id' => 'toc_allow_hide', 'opt' => 'isnt', 'val' => '' ),
							'std' => __('Hide', 'mfn-opts'),
						),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .table_of_content .title .title-inner:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .table_of_content .title .title-inner:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .table_of_content .title:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .table_of_content .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .table_of_content .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .table_of_content .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('List row', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .table_of_content .table_of_content_wrapper li:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .table_of_content .table_of_content_wrapper li a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Link color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .table_of_content .table_of_content_wrapper li a|hover:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Link hover', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .table_of_content .table_of_content_wrapper li:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

					),
				),

   			// Timeline -------------------------------------------------------

   			'timeline' => array(
   				'type' => 'timeline',
   				'title' => __('Timeline', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'elements',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'tabs',
   						'type' => 'tabs',
   						'title' => __('Timeline', 'mfn-opts'),
							'desc' => __('<b>JavaScript</b> content like Google Maps and some plugins shortcodes do <b>not work</b> in tabs', 'mfn-opts'),
							'options' => [
								'title' => [
									'input',
									__('Title', 'mfn-opts'),
									__('Sample event', 'mfn-opts'),
								],
								'date' => [
									'input',
									__('Date', 'mfn-opts'),
									'2021',
								],
								'content' => [
									'textarea',
									__('Content', 'mfn-opts'),
									'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Mauris eu massa orci.',
								],
							],
							'std' => [
								0 => [
									'title' => __('This is the 1st event', 'mfn-opts'),
									'date' => '2021',
									'content' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Mauris eu massa orci.',
								],
								1 => [
									'title' => __('This is the 2nd event', 'mfn-opts'),
									'date' => '2022',
									'content' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Mauris eu massa orci.',
								],
							],
							'primary' => 'title',
							'preview' => 'tabs',
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .timeline_items > li h3:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .timeline_items > li h3:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Date', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .timeline_items li h3 span:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .timeline_items li h3:--mfn-timeline-date-bg',
							'type' => 'color',
							'themeoptions' => 'style:',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .timeline_items li h3 span:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .timeline_items li h3 span:padding',
		  				'type' => 'dimensions',
		  				'themeoptions' => 'style:',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .timeline_items li h3 span:padding_laptop',
		  				'type' => 'dimensions',
		  				'themeoptions' => 'style:',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .timeline_items li h3 span:padding_tablet',
		  				'type' => 'dimensions',
		  				'themeoptions' => 'style:',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .timeline_items li h3 span:padding_mobile',
		  				'type' => 'dimensions',
		  				'themeoptions' => 'style:',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .timeline_items li .desc:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .timeline_items li .desc:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Dot', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .timeline_items li h3|before:border-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'themeoptions' => 'style:',
   						'title' => __('Placeholder', 'mfn-opts'),
   					),

   					array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .timeline_items li .desc|before:background-image',
		  				'oldid' => 'bg_color',
		  				'type' => 'upload',
		  				'title' => __('Image', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'background-image',
		  			),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'themeoptions' => 'style:',
   						'title' => __('Content lines', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .timeline_items li .desc:text-decoration-color',
							'type' => 'color',
							'themeoptions' => 'style:',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .timeline_items li .desc:text-underline-offset',
							'type' => 'text',
							'themeoptions' => 'style:',
							'class' => 'mfn-toggled',
							'default_unit' => 'px',
							'title' => __('Offset', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .timeline_items li .desc:text-decoration-thickness',
							'type' => 'text',
							'themeoptions' => 'style:',
							'class' => 'mfn-toggled',
							'default_unit' => 'px',
							'title' => __('Thickness', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'themeoptions' => 'style:simple',
   						'title' => __('Line', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .timeline_items li h3,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .timeline_items li .desc:border-color',
							'themeoptions' => 'style:simple',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .timeline_items li|hover h3,.mcb-section .mcb-wrap .mcb-item-mfnuidelement .timeline_items li|hover .desc:border-color',
							'themeoptions' => 'style:simple',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Hover color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

				// Toggle -----------------------------------------------------------

   			'toggle' => array(
   				'type' => 'toggle',
   				'title' => __('Toggle', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'blocks',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
   						'id' => 'tabs',
   						'type' => 'tabs',
   						'title' => __('Items', 'mfn-opts'),
							'options' => [

								'title' => [
									'input',
									__('Title', 'mfn-opts'),
									'This is the title',
								],
								'content' => [
									'textarea',
									__('Content', 'mfn-opts'),
									'This is the content',
								],
								'icon' => [
									'icon',
									__('Icon', 'mfn-opts'),
									'',
								],
								'image' => [
									'image',
									__('Image', 'mfn-opts'),
									'',
								],

							],
							'std' => [
								0 => [
									'title' => __('This is the 1st title', 'mfn-opts'),
									'content' => __('This is the 1st content', 'mfn-opts'),
								],
								1 => [
									'title' => __('This is the 2nd title', 'mfn-opts'),
									'content' => __('This is the 2nd content', 'mfn-opts'),
								],
							],
							'preview' => 'tabs',
							'primary' => 'title',
   					),

						array(
							'id' => 'tag',
							'type' => 'switch',
							'title' => __('Question tag', 'mfn-opts'),
							're_render' => true,
							'options' => array(
								'h1' => 'H1',
								'h2' => 'H2',
								'h3' => 'H3',
								'h4' => 'H4',
								'h5' => 'H5',
								'h6' => 'H6',
								'p' => 'p',
							),
							'std' => 'h5',
						),

						// options

   					array(
   						'type' => 'header',
   						'title' => __('Options', 'mfn-opts'),
   					),

						array(
							'id' => 'type',
							'attr_id' => 'toggle-type',
							'type' => 'switch',
							'title' => __('Type', 'mfn-opts'),
							're_render' => true,
							'options' => array(
								'unordered' => __('Unordered', 'mfn-opts'),
								'ordered' => __('Ordered', 'mfn-opts'),
							),
							'std' => 'unordered'
						),

						array(
							'id' => 'starting',
							'type' => 'text',
							'title' => __('Starting number', 'mfn-opts'),
							'condition' => array( 'id' => 'toggle-type', 'opt' => 'is', 'val' => 'ordered' ),
							're_render' => true,
							'std' => '1'
						),

						array(
							'id' => 'divider',
							'attr_id' => 'toggle-divider',
							'type' => 'switch',
							'title' => __('Divider', 'mfn-opts'),
							're_render' => true,
							'options' => array(
								'disable' => __('Disable', 'mfn-opts'),
								'enable' => __('Enable', 'mfn-opts'),
							),
							'std' => 'disable',
						),

						array(
							'id' => 'open_first',
							'type' => 'switch',
							'title' => __('Open first', 'mfn-opts'),
							're_render' => true,
							'options' => array(
								'disable' => __('Disable', 'mfn-opts'),
								'enable' => __('Enable', 'mfn-opts'),
							),
							'std' => 'disable',
						),

						array(
							'id' => 'open_all',
							'type' => 'switch',
							'title' => __('Open all', 'mfn-opts'),
							're_render' => true,
							'options' => array(
								'disable' => __('Disable', 'mfn-opts'),
								'enable' => __('Enable', 'mfn-opts'),
							),
							'std' => 'disable',
						),

						array(
							'id' => 'open_more',
							'type' => 'switch',
							'title' => __('Open more than one at a time', 'mfn-opts'),
							'options' => array(
								'disable' => __('Disable', 'mfn-opts'),
								'enable' => __('Enable', 'mfn-opts'),
							),
							'std' => 'disable',
						),

						// icon

   					array(
   						'type' => 'header',
   						'title' => __('Icon', 'mfn-opts'),
   					),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
   						'id' => 'icon',
   						'type' => 'icon',
   						'title' => __('Icon', 'mfn-opts'),
							're_render' => true,
							'preview' => 'icon',
							'std' => 'fas fa-plus',
   					),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
   						'id' => 'active_icon',
   						'type' => 'icon',
   						'title' => __('Icon', 'mfn-opts'),
							're_render' => true,
							'preview' => 'icon',
							'std' => 'fas fa-minus',
   					),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'id' => 'icon_animation',
							'type' => 'switch',
							'title' => __('Animation', 'mfn-opts'),
							'options' => array(
								'disable' => __('Disable', 'mfn-opts'),
								'zoom' => __('Zoom', 'mfn-opts'),
								'rotate' => __('Rotate', 'mfn-opts'),
							),
							'std' => 'disable',
						),

						// style ---

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						// item

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Toggle item', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle:--mfn-toggle-gap',
							'type' => 'sliderbar',
							'title' => __('Gap', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle:--mfn-toggle-gap_laptop',
							'type' => 'sliderbar',
							'title' => __('Gap', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle:--mfn-toggle-gap_tablet',
							'type' => 'sliderbar',
							'title' => __('Gap', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle:--mfn-toggle-gap_mobile',
							'type' => 'sliderbar',
							'title' => __('Gap', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '100',
								'step' => '1',
								'unit' => 'px',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item:border-style',
							'attr_id' => 'toggle-border-style',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item:border-width',
							'condition' => array( 'id' => 'toggle-border-style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'dimensions',
							'title' => __('Border width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item:border-radius',
							'type' => 'dimensions',
							'title' => __('Border radius', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-vb-formrow mfn-vb-mfnuidhere mfn-toggled"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item:border-color',
							'condition' => array( 'id' => 'toggle-border-style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item:background-color',
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
							'class' => 'mfn-toggled',
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item:box-shadow',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow'
		  			),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item|hover:border-color',
							'condition' => array( 'id' => 'toggle-border-style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item|hover:background-color',
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
							'class' => 'mfn-toggled',
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item|hover:box-shadow',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow'
		  			),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle .mfn-toggle-item.active:border-color',
							'condition' => array( 'id' => 'toggle-border-style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle .mfn-toggle-item.active:background-color',
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
							'class' => 'mfn-toggled',
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle .mfn-toggle-item.active:box-shadow',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow'
		  			),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						// divider

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Divider', 'mfn-opts'),
							'condition' => array( 'id' => 'toggle-divider', 'opt' => 'is', 'val' => 'enable' ),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle .toggle-divider:height',
							'condition' => array( 'id' => 'toggle-divider', 'opt' => 'is', 'val' => 'enable' ),
							'type' => 'sliderbar',
							'title' => __('Border width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'param' => array(
								'min' => '0',
								'max' => '20',
								'step' => '1',
								'unit' => 'px',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle:--mfn-toggle-divider-color',
							'condition' => array( 'id' => 'toggle-divider', 'opt' => 'is', 'val' => 'enable' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						// title bar

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title bar', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item .toggle-bar:padding',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item .toggle-bar:padding_laptop',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item .toggle-bar:padding_tablet',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item .toggle-bar:padding_mobile',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item .toggle-heading:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item .toggle-bar:border-style',
							'attr_id' => 'toggle-title-border-style',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item .toggle-bar:border-width',
							'condition' => array( 'id' => 'toggle-title-border-style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'dimensions',
							'title' => __('Border width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item .toggle-bar:border-radius',
							'type' => 'dimensions',
							'title' => __('Border radius', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-vb-formrow mfn-vb-mfnuidhere mfn-toggled"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item .toggle-bar:border-color',
							'condition' => array( 'id' => 'toggle-title-border-style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item .toggle-heading:color',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item .toggle-bar:background-color',
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
							'class' => 'mfn-toggled',
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item .toggle-bar:box-shadow',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow'
		  			),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item|hover .toggle-bar:border-color',
							'condition' => array( 'id' => 'toggle-title-border-style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item|hover .toggle-heading:color',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item|hover .toggle-bar:background-color',
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
							'class' => 'mfn-toggled',
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item|hover .toggle-bar:box-shadow',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow'
		  			),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle .mfn-toggle-item.active .toggle-bar:border-color',
							'condition' => array( 'id' => 'toggle-title-border-style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle .mfn-toggle-item.active .toggle-heading:color',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle .mfn-toggle-item.active .toggle-bar:background-color',
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
							'class' => 'mfn-toggled',
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle .mfn-toggle-item.active .toggle-bar:box-shadow',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow'
		  			),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						// content

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle .toggle-content:padding',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle .toggle-content:padding_laptop',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle .toggle-content:padding_tablet',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle .toggle-content:padding_mobile',
							'type' => 'dimensions',
							'version' => 'separated-fields',
							'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle .toggle-content:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle .toggle-content:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-vb-formrow mfn-vb-mfnuidhere mfn-toggled"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle .toggle-content a:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Link color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle .toggle-content a|hover:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Link color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle .toggle-content:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						// title icon

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title icon', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .toggle-bar-icon:width',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled mfn-slider-input',
							'css_attr' => 'width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .toggle-bar-icon:width_laptop',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled mfn-slider-input',
							'css_attr' => 'width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .toggle-bar-icon:width_tablet',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled mfn-slider-input',
							'css_attr' => 'width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .toggle-bar-icon:width_mobile',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled mfn-slider-input',
							'css_attr' => 'width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .toggle-bar-icon:height',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Height', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled mfn-slider-input',
							'css_attr' => 'height',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .toggle-bar-icon:height_laptop',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Height', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled mfn-slider-input',
							'css_attr' => 'height',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .toggle-bar-icon:height_tablet',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Height', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled mfn-slider-input',
							'css_attr' => 'height',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .toggle-bar-icon:height_mobile',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Height', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled mfn-slider-input',
							'css_attr' => 'height',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle:--mfn-toggle-bar-icon-spacing',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Spacing', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle:--mfn-toggle-bar-icon-spacing_laptop',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Spacing', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle:--mfn-toggle-bar-icon-spacing_tablet',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Spacing', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle:--mfn-toggle-bar-icon-spacing_mobile',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Spacing', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle:--mfn-toggle-bar-icon-size',
							'condition' => array( 'id' => 'toggle-type', 'opt' => 'isnt', 'val' => 'ordered' ),
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'title' => __('Icon size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle:--mfn-toggle-bar-icon-size_laptop',
							'condition' => array( 'id' => 'toggle-type', 'opt' => 'isnt', 'val' => 'ordered' ),
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'title' => __('Icon size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle:--mfn-toggle-bar-icon-size_tablet',
							'condition' => array( 'id' => 'toggle-type', 'opt' => 'isnt', 'val' => 'ordered' ),
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'title' => __('Icon size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle:--mfn-toggle-bar-icon-size_mobile',
							'condition' => array( 'id' => 'toggle-type', 'opt' => 'isnt', 'val' => 'ordered' ),
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'title' => __('Icon size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .toggle-bar-icon|after:typography',
							'condition' => array( 'id' => 'toggle-type', 'opt' => 'is', 'val' => 'ordered' ),
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .toggle-bar-icon:border-style',
							'attr_id' => 'toggle-ti-border-style',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .toggle-bar-icon:border-width',
							'condition' => array( 'id' => 'toggle-ti-border-style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'dimensions',
							'title' => __('Border width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .toggle-bar-icon:border-radius',
							'type' => 'dimensions',
							'title' => __('Border radius', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-vb-formrow mfn-vb-mfnuidhere mfn-toggled"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .toggle-bar-icon:border-color',
							'condition' => array( 'id' => 'toggle-ti-border-style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle:--mfn-toggle-bar-icon-color',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .toggle-bar-icon:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item|hover .toggle-bar-icon:border-color',
							'condition' => array( 'id' => 'toggle-ti-border-style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item|hover:--mfn-toggle-bar-icon-color',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item|hover .toggle-bar-icon:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item.active .toggle-bar-icon:border-color',
							'condition' => array( 'id' => 'toggle-ti-border-style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item.active:--mfn-toggle-bar-icon-color',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item.active .toggle-bar-icon:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						// toggle icon

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Toggle icon', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .toggle-icon:width',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled mfn-slider-input',
							'css_attr' => 'width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .toggle-icon:width_laptop',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled mfn-slider-input',
							'css_attr' => 'width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .toggle-icon:width_tablet',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled mfn-slider-input',
							'css_attr' => 'width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .toggle-icon:width_mobile',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled mfn-slider-input',
							'css_attr' => 'width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .toggle-icon:height',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Height', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled mfn-slider-input',
							'css_attr' => 'height',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .toggle-icon:height_laptop',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Height', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled mfn-slider-input',
							'css_attr' => 'height',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .toggle-icon:height_tablet',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Height', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled mfn-slider-input',
							'css_attr' => 'height',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .toggle-icon:height_mobile',
							'type' => 'text',
							'default_unit' => 'px',
							'title' => __('Height', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled mfn-slider-input',
							'css_attr' => 'height',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle:--mfn-toggle-icon-spacing',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Spacing', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle:--mfn-toggle-icon-spacing_laptop',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Spacing', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle:--mfn-toggle-icon-spacing_tablet',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Spacing', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle:--mfn-toggle-icon-spacing_mobile',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Spacing', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '200',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle:--mfn-toggle-icon-size',
							'condition' => array( 'id' => 'toggle-type', 'opt' => 'isnt', 'val' => 'ordered' ),
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'title' => __('Icon size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle:--mfn-toggle-icon-size_laptop',
							'condition' => array( 'id' => 'toggle-type', 'opt' => 'isnt', 'val' => 'ordered' ),
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'title' => __('Icon size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle:--mfn-toggle-icon-size_tablet',
							'condition' => array( 'id' => 'toggle-type', 'opt' => 'isnt', 'val' => 'ordered' ),
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'title' => __('Icon size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle:--mfn-toggle-icon-size_mobile',
							'condition' => array( 'id' => 'toggle-type', 'opt' => 'isnt', 'val' => 'ordered' ),
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled narrow sliderbar-units',
							'title' => __('Icon size', 'mfn-opts'),
							'units' => array(
								'px' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
								'em' 	=> array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'rem' => array('min' => 0.1, 	'max' => 10, 	'step' => 0.1),
								'vw' 	=> array('min' => 1, 		'max' => 100, 'step' => 1),
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .toggle-icon|after:typography',
							'condition' => array( 'id' => 'toggle-type', 'opt' => 'is', 'val' => 'ordered' ),
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .toggle-icon:border-style',
							'attr_id' => 'toggle-ti-border-style',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .toggle-icon:border-width',
							'condition' => array( 'id' => 'toggle-ti-border-style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'dimensions',
							'title' => __('Border width', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .toggle-icon:border-radius',
							'type' => 'dimensions',
							'title' => __('Border radius', 'mfn-opts'),
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
						),

						array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-vb-formrow mfn-vb-mfnuidhere mfn-toggled"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li><li><a href="#active" data-tab="active">Active</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .toggle-icon:border-color',
							'condition' => array( 'id' => 'toggle-ti-border-style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle:--mfn-toggle-icon-color',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .toggle-icon:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item|hover .toggle-icon:border-color',
							'condition' => array( 'id' => 'toggle-ti-border-style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item|hover:--mfn-toggle-icon-color',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item|hover .toggle-icon:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item.active .toggle-icon:border-color',
							'condition' => array( 'id' => 'toggle-ti-border-style', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item.active:--mfn-toggle-icon-color',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
							'class' => 'mfn-toggled',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-toggle-item.active .toggle-icon:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Trailer Box ----------------------------------------------------

   			'trailer_box' => array(
   				'type' => 'trailer_box',
   				'title' => __('Trailer Box', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'boxes',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'image',
   						'type' => 'upload',
   						'dynamic_data' => 'featured_image',
   						'title' => __('Image', 'mfn-opts'),
   						'desc' => __('Recommended image width <b>768px - 1920px</b> depending on size of the item', 'mfn-opts'),
							'std' => $this->get_placeholder(),
							'preview' => 'image',
   					),

						array(
   						'id' => 'orientation',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Image orientation', 'mfn-opts'),
   						'options' => array(
   							'' => __('Vertical', 'mfn-opts'),
   							'horizontal' => __('Horizontal', 'mfn-opts'),
   						),
							'std' => '',
   					),

   					array(
   						'id' => 'slogan',
   						'type' => 'text',
   						'edit_tag' => '.desc | div.subtitle',
   						'edit_position' => 'prepend',
   						'dynamic_data' => 'content',
   						'title' => __('Slogan', 'mfn-opts'),
							'std' => __('This is the slogan', 'mfn-opts'),
							'preview' => 'subtitle',
   					),

   					array(
   						'id' => 'title',
   						'type' => 'text',
   						'edit_tag' => '.desc | .title',
   						'dynamic_data' => 'title',
   						'edit_position' => 'append',
   						'title' => __('Title', 'mfn-opts'),
							'std' => __('This is the title', 'mfn-opts'),
							'preview' => 'title',
   					),

					array(
							'id' => 'title_tag',
							'type' 	=> 'switch',
							're_render' => true,
							'title' => __('Title tag', 'mfn-opts'),
							'options' => array(
								'h1' => 'h1',
								'h2' => 'h2',
								'h3' => 'h3',
								'h4' => 'h4',
								'h5' => 'h5',
								'h6' => 'h6',
								'p' => 'p',
								'span' => 'span',
								'div' => 'div',
							),
							'std' => 'h2',
						),

						// link

   					array(
   						'type' => 'header',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link_type',
   						'attr_id' => 'link_type',
   						'type' => 'select',
							'title' => __('On click action', 'mfn-opts'),
   						'options' => array(
								'' => __('Default', 'mfn-opts'),
								'1' => __('Open popup', 'mfn-opts'),
							),
							'std' => '',
   					),

   					array(
   						'id' => 'popup_id',
   						'condition' => array( 'id' => 'link_type', 'opt' => 'is', 'val' => '1' ),
   						'type' => 'text',
   						'title' => __('Popup ID', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link',
   						'dynamic_data' => 'permalink',
   						'type' => 'text',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'target',
   						'type' => 'select',
   						'title' => __('Target', 'mfn-opts'),
   						'options' => array(
   							0 => __('Default | _self', 'mfn-opts'),
   							1 => __('New tab or window | _blank', 'mfn-opts'),
   							'lightbox' => __('Lightbox (image or embed video)', 'mfn-opts'),
   						),
   					),

            array(
							'id' => 'link_title',
							'type' => 'text',
							'title' => __('Link title', 'mfn-opts'),
						),

   					// advanced

   					array(
   						'type' => 'header',
   						'title' => __('Advanced', 'mfn-opts'),
   					),

   					array(
   						'id' => 'style',
   						'type' => 'switch',
   						're_render' => true,
   						'title' => __('Style', 'mfn-opts'),
   						'options' => array(
   							'' => __('Default', 'mfn-opts'),
   							'plain' => __('Plain', 'mfn-opts'),
   						),
							'std' => '',
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

   					array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Slogan', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .trailer_box .desc .subtitle:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .trailer_box .desc .subtitle:background',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .trailer_box .desc .subtitle:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .trailer_box .desc .subtitle:padding',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .trailer_box .desc .subtitle:padding_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .trailer_box .desc .subtitle:padding_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .trailer_box .desc .subtitle:padding_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Title', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .trailer_box .desc .title:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .trailer_box .desc .title:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .trailer_box .desc .title:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .trailer_box .desc .title:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .trailer_box .desc .title:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .trailer_box .desc .title:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'themeoptions' => 'style:simple',
   						'title' => __('Content', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.style-simple .mcb-section .mcb-wrap .mcb-item-mfnuidelement .trailer_box|hover .desc:background-color',
							'type' => 'color',
							'themeoptions' => 'style:simple',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'themeoptions' => 'style:',
   						'title' => __('Line', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .trailer_box .desc .line:background-color',
							'type' => 'color',
							'themeoptions' => 'style:',
							'class' => 'mfn-toggled',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Video  --------------------------------------------

   			'video' => array(
   				'type' => 'video',
   				'title' => __('Video', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'elements',
   				'attr' => array(
   				//'fields' => array(

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'type' => 'header',
   						'title' => __('YouTube or Vimeo', 'mfn-opts'),
   					),

   					array(
   						'id' => 'video',
   						're_render' => true,
   						'type' => 'text',
   						'title' => __('Video ID', 'mfn-opts'),
   						'desc' => __('<b>YouTube:</b> http://www.youtube.com/watch?v=<u>WoJhnRczeNg</u><br /><b>Vimeo:</b> http://vimeo.com/<u>62954028</u>', 'mfn-opts'),
							'preview' => 'title',
							'std' => 'XQfqevYQQIQ',
   					),

   					array(
   						'id' => 'parameters',
   						're_render' => true,
   						'type' => 'text',
   						'title' => __('Parameters', 'mfn-opts'),
   						'desc' => __('Multiple parameters should be connected with "&"<br />Example: <b>autoplay=1&loop=1</b><br />Notice: Vimeo authors may disable some parameters for their videos', 'mfn-opts'),
   					),

						// html5

						array(
							'type' => 'header',
   						'title' => __('HTML5', 'mfn-opts'),
   					),

   					array(
   						'id' => 'mp4',
   						're_render' => true,
   						'type' => 'upload',
   						'title' => __('MP4 video', 'mfn-opts'),
   						'desc' => __('Please add both mp4 and ogv for cross-browser compatibility', 'mfn-opts'),
   						'data' => 'video',
   					),

   					array(
   						'id' => 'ogv',
   						're_render' => true,
   						'type' => 'upload',
   						'title' => __('OGV video', 'mfn-opts'),
   						'data' => 'video',
   					),

   					array(
   						'id' => 'placeholder',
   						're_render' => true,
   						'type' => 'upload',
   						'title' => __('Placeholder image', 'mfn-opts'),
   						'desc' => __('Placeholder Image will be used as video placeholder before video loads and on mobile devices', 'mfn-opts'),
   					),

   					array(
   						'id' => 'html5_parameters',
   						're_render' => true,
   						'type' => 'select',
   						'title' => __('Parameters', 'mfn-opts'),
   						'desc' => __('WebKit browsers and iOS do not support autoplay', 'mfn-opts'),
   						'options' => array(
   							'' => __('autoplay controls loop muted', 'mfn-opts'),
   							'a;c;l;m;i' => __('autoplay controls loop muted playsinline', 'mfn-opts'),
   							'a;c;l;' => __('autoplay controls loop', 'mfn-opts'),
   							'a;c;l;;i' => __('autoplay controls loop playsinline', 'mfn-opts'),
   							'a;c;;m' => __('autoplay controls muted', 'mfn-opts'),
   							'a;c;;m;i' => __('autoplay controls muted playsinline', 'mfn-opts'),
   							'a;;l;m' => __('autoplay loop muted', 'mfn-opts'),
   							'a;;l;m;i' => __('autoplay loop muted playsinline', 'mfn-opts'),
   							'a;c;;' => __('autoplay controls', 'mfn-opts'),
   							'a;c;;;i' => __('autoplay controls playsinline', 'mfn-opts'),
   							'a;;l;' => __('autoplay loop', 'mfn-opts'),
   							'a;;l;;i' => __('autoplay loop playsinline', 'mfn-opts'),
   							'a;;;m' => __('autoplay muted', 'mfn-opts'),
   							'a;;;m;p' => __('autoplay muted playsinline', 'mfn-opts'),
   							'a;;;' => __('autoplay', 'mfn-opts'),
   							'a;;;;p' => __('autoplay playsinline', 'mfn-opts'),
   							';c;l;m' => __('controls loop muted', 'mfn-opts'),
   							';c;l;m;p' => __('controls loop muted playsinline', 'mfn-opts'),
   							';c;l;' => __('controls loop', 'mfn-opts'),
   							';c;l;;p' => __('controls loop playsinline', 'mfn-opts'),
   							';c;;m' => __('controls muted', 'mfn-opts'),
   							';c;;m;p' => __('controls muted playsinline', 'mfn-opts'),
   							';c;;' => __('controls', 'mfn-opts'),
   							';c;;;p' => __('controls playsinline', 'mfn-opts'),
   						),
   					),

						// advanced

						array(
							'type' => 'header',
   						'title' => __('Advanced', 'mfn-opts'),
   					),

   					array(
   						'id' => 'width',
   						're_render' => true,
   						'type' => 'text',
   						'title' => __('Width', 'mfn-opts'),
   						'param' => 'number',
   						'after' => 'px',
   						'class' => 'narrow',
   						'placeholder' => 700,
   					),

   					array(
   						'id' => 'height',
   						're_render' => true,
   						'type' => 'text',
   						'title' => __('Height', 'mfn-opts'),
							'param' => 'number',
   						'after' => 'px',
   						'class' => 'narrow',
   						'placeholder' => 400,
   					),

						// Mask Shape

						array(
							'type' => 'header',
							'title' => __('Mask shape', 'mfn-opts'),
						),

						array(
							'id' => 'mask_shape_type',
							'attr_id' => 'mask_type',
							'type' => 'select',
							'title' => __('Mask type', 'mfn-opts'),
							'options' => array(
								'0' => __('None', 'mfn-opts'),
								'blob' => __('Blob', 'mfn-opts'),
								'blob-2' => __('Blob 2', 'mfn-opts'),
								'brush' => __('Brush', 'mfn-opts'),
								'brush-2' => __('Brush 2', 'mfn-opts'),
								'circle' =>  __('Circle', 'mfn-opts'),
								'cross' => __('Cross', 'mfn-opts'),
								'irregular-circle' => __('Irregular Circle', 'mfn-opts'),
								'stain' => __('Stain', 'mfn-opts'),
								'triangle' => __('Triangle', 'mfn-opts'),
								'custom' => __('Custom', 'mfn-opts'),
							),
							'std' => '0'
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_video.mfn-mask-shape iframe, .mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_video.mfn-mask-shape video:-webkit-mask-image',
							'type' => 'upload',
							'title' => __('Mask image', 'mfn-opts'),
							'desc' => 'Only SVG type of image works',
							'condition' => array( 'id' => 'mask_type', 'opt' => 'is', 'val' => 'custom' ),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_video.mfn-mask-shape:background-color',
							'type' => 'color',
							'title' => __('Mask color', 'mfn-opts'),
							'condition' => array( 'id' => 'mask_type', 'opt' => 'isnt', 'val' => '0' ),
						),

						array(
							'id' => 'mask_shape_size',
							'attr_id' => 'mask_shape_size',
							'type' => 'select',
							'title' => __('Size', 'mfn-opts'),
							'condition' => array( 'id' => 'mask_type', 'opt' => 'isnt', 'val' => '0' ),
							'options' => array(
								'auto' => __('Default', 'mfn-opts'),
								'cover' => __('Cover', 'mfn-opts'),
								'contain' => __('Contain', 'mfn-opts'),
								'custom' => __('Custom', 'mfn-opts'),
							),
							'std' => 'contain',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_video.mfn-mask-shape iframe, .mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_video.mfn-mask-shape video:-webkit-mask-size',
							'type' => 'sliderbar',
							'condition' => array( 'id' => 'mask_shape_size', 'opt' => 'is', 'val' => 'custom' ),
							'title' => __('Scale', 'mfn-opts'),
							'param' => array(
								'min' => '0',
								'max' => '200',
								'step' => '1',
								'unit' => '%',
							),
						),

						array(
							'id' => 'mask_shape_position',
							'attr_id' => 'mask_shape_position',
							'type' => 'select',
							'title' => __('Position', 'mfn-opts'),
							'condition' => array( 'id' => 'mask_type', 'opt' => 'isnt', 'val' => '0' ),
							'options' => array(
								'center center' => __('Center Center', 'mfn-opts'),
								'center left' => __('Center Left', 'mfn-opts'),
								'center right' => __('Center Right', 'mfn-opts'),
								'top left' => __('Top Left', 'mfn-opts'),
								'top right' => __('Top Right', 'mfn-opts'),
								'top center' => __('Top Center', 'mfn-opts'),
								'bottom left' => __('Bottom Left', 'mfn-opts'),
								'bottom right' => __('Bottom Right', 'mfn-opts'),
								'bottom center' => __('Bottom Center', 'mfn-opts'),
								'custom' => __('Custom', 'mfn-opts'),
							),
							'std' => 'center center'
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_video.mfn-mask-shape iframe, .mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_video.mfn-mask-shape video:-webkit-mask-position-x',
							'type' => 'sliderbar',
							'condition' => array( 'id' => 'mask_shape_position', 'opt' => 'is', 'val' => 'custom' ),
							'title' => __('Position X', 'mfn-opts'),
							'param' => array(
								'min' => '-100',
								'max' => '100',
								'step' => '1',
								'unit' => '%',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_video.mfn-mask-shape iframe, .mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_video.mfn-mask-shape video:-webkit-mask-position-y',
							'type' => 'sliderbar',
							'condition' => array( 'id' => 'mask_shape_position', 'opt' => 'is', 'val' => 'custom' ),
							'title' => __('Position Y', 'mfn-opts'),
							'param' => array(
								'min' => '-100',
								'max' => '100',
								'step' => '1',
								'unit' => '%',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_video.mfn-mask-shape iframe, .mcb-section .mcb-wrap .mcb-item-mfnuidelement .content_video.mfn-mask-shape video:-webkit-mask-repeat',
							'type' => 'select',
							'title' => __('Repeat', 'mfn-opts'),
							'condition' => array( 'id' => 'mask_type', 'opt' => 'isnt', 'val' => '0' ),
							'options' => array(
								'' => __('No repeat', 'mfn-opts'),
								'repeat-x' => __('Repeat X', 'mfn-opts'),
								'repeat-y' => __('Repeat Y', 'mfn-opts'),
								'repeat' => __('Repeat', 'mfn-opts'),
							),
						),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

				// Header WPML Language Switcher ----------------------------------------------------

   			'header_language_switcher' => array(
   				'type' => 'header_language_switcher',
   				'title' => __('WPML switcher', 'mfn-opts'),
   				'size' => '1/1',
   				'tablet_size' => '1/1',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'other',
   				'attr' => array(

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

						array(
							'id' => 'style',
							'attr_id' => 'style',
							'type' => 'select',
							're_render' => true,
							'title' => __('Style', 'mfn-opts'),
							'options' => array(
								'' => __('Default', 'mfn-opts'),
								'dropdown' => __('Dropdown', 'mfn-opts'),
							),
							'std' => '',
						),

						array(
							'id' => 'flags',
							'type' => 'switch',
							're_render' => true,
							'title' => __('Flags', 'mfn-opts'),
							'options' => array(
								'1' => __('Show', 'mfn-opts'),
								'0' => __('Hide', 'mfn-opts'),
							),
							'std' => 1,
						),

						array(
							'id' => 'dropdown_icon',
							'attr_id' => 'dropdown_icon',
							'condition' => array( 'id' => 'style', 'opt' => 'is', 'val' => 'dropdown' ),
							'type' => 'switch',
							're_render' => true,
							'title' => __('Dropdown icon', 'mfn-opts'),
							'options' => array(
								'1' => __('Show', 'mfn-opts'),
								'' => __('Hide', 'mfn-opts'),
							),
							'std' => 1,
						),

						array(
							'id' => 'dropdown_icon_html',
							'condition' => array( 'id' => 'dropdown_icon', 'opt' => 'is', 'val' => '1' ),
							'type' => 'icon',
							're_render' => true,
							'title' => __('Dropdown icon', 'mfn-opts'),
							'std' => 'icon-down-open',
						),

						array(
							'id' => 'dropdown_icon_image',
							'condition' => array( 'id' => 'dropdown_icon', 'opt' => 'is', 'val' => '1' ),
							'type' => 'upload',
							're_render' => true,
							'title' => __('Dropdown icon', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render modalbox-card-style-mfnuidhere">',
						),

						array(
							'class' => 'toggled_header row-header',
							'type' => 'header',
   						'title' => __('Item', 'mfn-opts'),
   					),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher ul li a:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher ul li a:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher ul li a:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher ul li a:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher ul li a:border-style',
							'attr_id' => 'border_style_wpml_swi',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher ul li a:border-width',
		  				'condition' => array( 'id' => 'border_style_wpml_swi', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher ul li a:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_wpml_swi', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher ul li a:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_wpml_swi', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher ul li a:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_wpml_swi', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher ul li a:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher ul li a:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher ul li a:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher ul li a:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),


		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher ul li a:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),


   					array(
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher ul li a:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher ul li a:border-color',
							'condition' => array( 'id' => 'border_style_wpml_swi', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),


						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher ul li a:background-color',
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher ul li a|hover:color',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher ul li a|hover:border-color',
							'condition' => array( 'id' => 'border_style_wpml_swi', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher ul li a|hover:background',
							'type' => 'color',
							'class' => 'main-color mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

						array(
							'type' => 'html',
							'html' => '</div></div>',
						),




		  			array(
							'class' => 'toggled_header row-header',
							'type' => 'header',
   						'title' => __('Flag', 'mfn-opts'),
   					),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher ul li a img:width',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Flag size', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'narrow mfn-toggled',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '100',
								'step' => '1',
							),
							'std' => '18px',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher ul li a img:width_laptop',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Flag size', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'narrow mfn-toggled',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '100',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher ul li a img:width_tablet',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Flag size', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'narrow mfn-toggled',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '100',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher ul li a img:width_mobile',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Flag size', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'narrow mfn-toggled',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '100',
								'step' => '1',
							),
						),


		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher ul li a img:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher ul li a img:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher ul li a img:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher ul li a img:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
							'class' => 'toggled_header row-header',
							'type' => 'header',
							'condition' => array( 'id' => 'style', 'opt' => 'is', 'val' => 'dropdown' ),
   						'title' => __('Dropdown icon', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher-dropdown.mfn-language-switcher-dropdown-icon ul li a .mfn-arrow-icon:--mfn-wpml-arrow-size',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Icon size', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'narrow mfn-toggled',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '100',
								'step' => '1',
							),
							'std' => '10px',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher-dropdown.mfn-language-switcher-dropdown-icon ul li a .mfn-arrow-icon:--mfn-wpml-arrow-size_laptop',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Icon size', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'narrow mfn-toggled',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '100',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher-dropdown.mfn-language-switcher-dropdown-icon ul li a .mfn-arrow-icon:--mfn-wpml-arrow-size_tablet',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Icon size', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'narrow mfn-toggled',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '100',
								'step' => '1',
							),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher-dropdown.mfn-language-switcher-dropdown-icon ul li a .mfn-arrow-icon:--mfn-wpml-arrow-size_mobile',
							'type' => 'sliderbar',
							'default_unit' => 'px',
							'title' => __('Icon size', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'narrow mfn-toggled',
							'param' => array(
								'unit' => 'px',
								'min' => '0',
								'max' => '100',
								'step' => '1',
							),
						),

   					array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher-dropdown ul li a .mfn-arrow-icon:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher-dropdown ul li a .mfn-arrow-icon:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher-dropdown ul li a .mfn-arrow-icon:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher-dropdown ul li a .mfn-arrow-icon:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),


		  			array(
							'class' => 'toggled_header row-header',
							'type' => 'header',
							'condition' => array( 'id' => 'style', 'opt' => 'is', 'val' => 'dropdown' ),
   						'title' => __('Submenu', 'mfn-opts'),
   					),

   					array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher-dropdown ul li ul:width',
		  				'type' => 'text',
		  				'title' => __('Width', 'mfn-opts'),
		  				'default_unit' => 'px',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled mfn-slider-input',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher-dropdown ul li ul:width_laptop',
		  				'type' => 'text',
		  				'title' => __('Width', 'mfn-opts'),
		  				'default_unit' => 'px',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled mfn-slider-input',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher-dropdown ul li ul:width_tablet',
		  				'type' => 'text',
		  				'title' => __('Width', 'mfn-opts'),
		  				'default_unit' => 'px',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled mfn-slider-input',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher-dropdown ul li ul:width_mobile',
		  				'type' => 'text',
		  				'title' => __('Width', 'mfn-opts'),
		  				'default_unit' => 'px',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled mfn-slider-input',
		  			),

   					array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher-dropdown ul li ul:background-color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Background color', 'mfn-opts'),
						),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher-dropdown ul li ul:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher-dropdown ul li ul:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher-dropdown ul li ul:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher-dropdown ul li ul:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'padding',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher-dropdown ul li ul:border-style',
							'attr_id' => 'border_style_wpml_submenu',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher-dropdown ul li ul:border-width',
		  				'condition' => array( 'id' => 'border_style_wpml_submenu', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher-dropdown ul li ul:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_wpml_submenu', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher-dropdown ul li ul:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_wpml_submenu', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher-dropdown ul li ul:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_wpml_submenu', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher-dropdown ul li ul:border-color',
							'condition' => array( 'id' => 'border_style_wpml_submenu', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher-dropdown ul li ul:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher-dropdown ul li ul:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher-dropdown ul li ul:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mfn-language-switcher-dropdown ul li ul:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

						array(
							'type' => 'html',
							'html' => '</div>',
						),


					),
   			),

   			// Visual Editor  -------------------------------------------------

   			'visual' => array(
   				'type' => 'visual',
   				'title' => __('Visual Editor', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'used_fonts' => '',
   				'cat' => 'other',
   				'attr' => array(
   				//'fields' => array(

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'content',
   						'type' => 'visual',
   						'title' => __('Editor', 'mfn-opts'),
							'class' => 'form-content-full-width',
							'dynamic_data' => 'content',
							'preview' => 'content',
							'vbstd' => '<p>Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.</p>',
   					),

						// deprecated

						array(
							'type' => 'header',
							'class' => 'mfn-deprecated',
							'title' => __('Deprecated', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   			// Zoom Box -------------------------------------------------------

   			'zoom_box' => array(
   				'type' => 'zoom_box',
   				'title' => __('Zoom Box', 'mfn-opts'),
   				'size' => '1/4',
   				'tablet_size' => '1/4',
   				'mobile_size' => '1/1',
   				'tablet_resized' => '0',
   				'cat' => 'boxes',
   				'attr' => array(
   				//'fields' => array(

   					/*array(
							'type' => 'html',
							'html' => $this->itemtabs,
						),*/

   					array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-re_render modalbox-card-content active">',
						),

   					array(
   						'id' => 'image',
   						'dynamic_data' => 'featured_image',
   						'type' => 'upload',
   						'title' => __('Image', 'mfn-opts'),
   						'desc' => __('Recommended image width <b>768px - 1920px</b> depending on size of the item', 'mfn-opts'),
							'std' => $this->get_placeholder(),
							'preview' => 'image',
   					),

   					array(
   						'id' => 'content_image',
   						'dynamic_data' => 'featured_image',
   						'type' => 'upload',
   						'title' => __('Content image', 'mfn-opts'),
   					),

   					array(
   						'id' => 'content',
   						'dynamic_data' => 'content',
   						'type' => 'textarea',
   						'title' => __('Content', 'mfn-opts'),
							'std' => 'Lorem ipsum dolor',
							'preview' => 'content',
							'class' => 'form-content-full-width',
   					),

						// link

						array(
							'type' => 'header',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link_type',
   						'attr_id' => 'link_type',
   						'type' => 'select',
							'title' => __('On click action', 'mfn-opts'),
   						'options' => array(
								'' => __('Default', 'mfn-opts'),
								'1' => __('Open popup', 'mfn-opts'),
							),
							'std' => '',
   					),

   					array(
   						'id' => 'popup_id',
   						'condition' => array( 'id' => 'link_type', 'opt' => 'is', 'val' => '1' ),
   						'type' => 'text',
   						'title' => __('Popup ID', 'mfn-opts'),
   					),

   					array(
   						'id' => 'link',
   						'type' => 'text',
   						'dynamic_data' => 'permalink',
   						'title' => __('Link', 'mfn-opts'),
   					),

   					array(
   						'id' => 'target',
   						'type' => 'select',
   						'title' => __('Target', 'mfn-opts'),
   						'options' => array(
   							0 => __('Default | _self', 'mfn-opts'),
   							1 => __('New tab or window | _blank', 'mfn-opts'),
   							'lightbox' => __('Lightbox (image or embed video)', 'mfn-opts'),
   						),
   					),

            array(
							'id' => 'link_title',
							'type' => 'text',
							'dynamic_data' => 'title',
							'title' => __('Link title', 'mfn-opts'),
						),

   					array(
   						'type' => 'header',
   						'class' => 'mfn-deprecated',
   						'title' => __('Deprecated', 'mfn-opts'),
   					),

   					array(
   						'id' => 'bg_color',
   						'class' => 'mfn-deprecated',
   						'type' => 'color',
   						'title' => __('Overlay background', 'mfn-opts'),
   					),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Element classes', 'mfn-opts'),
							'class' => 'mfn-deprecated',
						),

						array(
							'type' => 'html',
							'html' => '</div><div class="modalbox-card modalbox-card-style modalbox-re_render">',
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Overlay', 'mfn-opts'),
   					),

						array(
							'id' => 'background_switcher',
							'attr_id' => 'background_switcher_zoom',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher mfn-toggled disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .zoom_box .desc:background-color',
							'condition' => array( 'id' => 'background_switcher_zoom', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .zoom_box .desc:gradient',
		  				'condition' => array( 'id' => 'background_switcher_zoom', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .zoom_box .desc:background-image',
		  				'condition' => array( 'id' => 'background_switcher_zoom', 'opt' => 'is', 'val' => 'default' ),
		  				'oldid' => 'bg_color',
		  				'type' => 'upload',
		  				'title' => __('Image', 'mfn-opts'),
		  				'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-image',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .zoom_box .desc:background-image_laptop',
		  				'condition' => array( 'id' => 'background_switcher_zoom', 'opt' => 'is', 'val' => 'default' ),
		  				'oldid' => 'bg_image',
		  				'type' => 'upload',
		  				'title' => __('Image', 'mfn-opts'),
		  				'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-image',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .zoom_box .desc:background-image_tablet',
		  				'condition' => array( 'id' => 'background_switcher_zoom', 'opt' => 'is', 'val' => 'default' ),
		  				'oldid' => 'bg_image',
		  				'type' => 'upload',
		  				'title' => __('Image', 'mfn-opts'),
		  				'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-image',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .zoom_box .desc:background-image_mobile',
		  				'condition' => array( 'id' => 'background_switcher_zoom', 'opt' => 'is', 'val' => 'default' ),
		  				'type' => 'upload',
		  				'title' => __('Image', 'mfn-opts'),
		  				'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-image',
		  			),

		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .zoom_box .desc:background-repeat',
							'condition' => array( 'id' => 'background_switcher_zoom', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'select',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-repeat',
							'title' => __('Repeat', 'mfn-opts'),
							'options' => [
								'repeat' => __('Repeat', 'mfn-opts'),
								'no-repeat' => __('No repeat', 'mfn-opts'),
								'repeat-x' => __('Repeat X', 'mfn-opts'),
								'repeat-y' => __('Repeat Y', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .zoom_box .desc:background-repeat_laptop',
							'condition' => array( 'id' => 'background_switcher_zoom', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'select',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-repeat',
							'title' => __('Repeat', 'mfn-opts'),
							'options' => [
								'repeat' => __('Repeat', 'mfn-opts'),
								'no-repeat' => __('No repeat', 'mfn-opts'),
								'repeat-x' => __('Repeat X', 'mfn-opts'),
								'repeat-y' => __('Repeat Y', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .zoom_box .desc:background-repeat_tablet',
							'condition' => array( 'id' => 'background_switcher_zoom', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'select',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-repeat',
							'title' => __('Repeat', 'mfn-opts'),
							'options' => [
								'repeat' => __('Repeat', 'mfn-opts'),
								'no-repeat' => __('No repeat', 'mfn-opts'),
								'repeat-x' => __('Repeat X', 'mfn-opts'),
								'repeat-y' => __('Repeat Y', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .zoom_box .desc:background-repeat_mobile',
							'condition' => array( 'id' => 'background_switcher_zoom', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'select',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-repeat',
							'title' => __('Repeat', 'mfn-opts'),
							'options' => [
								'repeat' => __('Repeat', 'mfn-opts'),
								'no-repeat' => __('No repeat', 'mfn-opts'),
								'repeat-x' => __('Repeat X', 'mfn-opts'),
								'repeat-y' => __('Repeat Y', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .zoom_box .desc:background-position',
							'condition' => array( 'id' => 'background_switcher_zoom', 'opt' => 'is', 'val' => 'default' ),
							'oldid' => 'bg_position',
							'type' => 'select',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-position',
							'title' => __('Position', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'center' => __('Center Center', 'mfn-opts'),
								'center left' => __('Center Left', 'mfn-opts'),
								'center right' => __('Center Right', 'mfn-opts'),
								'top left' => __('Top Left', 'mfn-opts'),
								'top right' => __('Top Right', 'mfn-opts'),
								'top center' => __('Top Center', 'mfn-opts'),
								'bottom left' => __('Bottom Left', 'mfn-opts'),
								'bottom right' => __('Bottom Right', 'mfn-opts'),
								'bottom center' => __('Bottom Center', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .zoom_box .desc:background-position_laptop',
							'condition' => array( 'id' => 'background_switcher_zoom', 'opt' => 'is', 'val' => 'default' ),
							'oldid' => 'bg_position',
							'type' => 'select',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-position',
							'title' => __('Position', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'center' => __('Center Center', 'mfn-opts'),
								'center left' => __('Center Left', 'mfn-opts'),
								'center right' => __('Center Right', 'mfn-opts'),
								'top left' => __('Top Left', 'mfn-opts'),
								'top right' => __('Top Right', 'mfn-opts'),
								'top center' => __('Top Center', 'mfn-opts'),
								'bottom left' => __('Bottom Left', 'mfn-opts'),
								'bottom right' => __('Bottom Right', 'mfn-opts'),
								'bottom center' => __('Bottom Center', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .zoom_box .desc:background-position_tablet',
							'condition' => array( 'id' => 'background_switcher_zoom', 'opt' => 'is', 'val' => 'default' ),
							'oldid' => 'bg_position',
							'type' => 'select',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-position',
							'title' => __('Position', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'center' => __('Center Center', 'mfn-opts'),
								'center left' => __('Center Left', 'mfn-opts'),
								'center right' => __('Center Right', 'mfn-opts'),
								'top left' => __('Top Left', 'mfn-opts'),
								'top right' => __('Top Right', 'mfn-opts'),
								'top center' => __('Top Center', 'mfn-opts'),
								'bottom left' => __('Bottom Left', 'mfn-opts'),
								'bottom right' => __('Bottom Right', 'mfn-opts'),
								'bottom center' => __('Bottom Center', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .zoom_box .desc:background-position_mobile',
							'condition' => array( 'id' => 'background_switcher_zoom', 'opt' => 'is', 'val' => 'default' ),
							'oldid' => 'bg_position',
							'type' => 'select',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-position',
							'title' => __('Position', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'center' => __('Center Center', 'mfn-opts'),
								'center left' => __('Center Left', 'mfn-opts'),
								'center right' => __('Center Right', 'mfn-opts'),
								'top left' => __('Top Left', 'mfn-opts'),
								'top right' => __('Top Right', 'mfn-opts'),
								'top center' => __('Top Center', 'mfn-opts'),
								'bottom left' => __('Bottom Left', 'mfn-opts'),
								'bottom right' => __('Bottom Right', 'mfn-opts'),
								'bottom center' => __('Bottom Center', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .zoom_box .desc:background-attachment',
							'condition' => array( 'id' => 'background_switcher_zoom', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'select',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-attachment',
							'title' => __('Attachment', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'scroll' => __('Scroll', 'mfn-opts'),
								'fixed' => __('Fixed', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .zoom_box .desc:background-attachment_laptop',
							'condition' => array( 'id' => 'background_switcher_zoom', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'select',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-attachment',
							'title' => __('Attachment', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'scroll' => __('Scroll', 'mfn-opts'),
								'fixed' => __('Fixed', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .zoom_box .desc:background-attachment_tablet',
							'condition' => array( 'id' => 'background_switcher_zoom', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'select',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-attachment',
							'title' => __('Attachment', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'scroll' => __('Scroll', 'mfn-opts'),
								'fixed' => __('Fixed', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .zoom_box .desc:background-attachment_mobile',
							'condition' => array( 'id' => 'background_switcher_zoom', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'select',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-attachment',
							'title' => __('Attachment', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'scroll' => __('Scroll', 'mfn-opts'),
								'fixed' => __('Fixed', 'mfn-opts'),
							],
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .zoom_box .desc:background-size',
							'condition' => array( 'id' => 'background_switcher_zoom', 'opt' => 'is', 'val' => 'default' ),
							'oldid' => 'bg_size',
							'type' => 'select',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-size',
							'title' => __('Size', 'mfn-opts'),
							'options' => mfna_bg_size()
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .zoom_box .desc:background-size_laptop',
							'condition' => array( 'id' => 'background_switcher_zoom', 'opt' => 'is', 'val' => 'default' ),
							'oldid' => 'bg_size',
							'type' => 'select',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-size',
							'title' => __('Size', 'mfn-opts'),
							'options' => mfna_bg_size()
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .zoom_box .desc:background-size_tablet',
							'condition' => array( 'id' => 'background_switcher_zoom', 'opt' => 'is', 'val' => 'default' ),
							'oldid' => 'bg_size',
							'type' => 'select',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-size',
							'title' => __('Size', 'mfn-opts'),
							'options' => mfna_bg_size()
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .zoom_box .desc:background-size_mobile',
							'condition' => array( 'id' => 'background_switcher_zoom', 'opt' => 'is', 'val' => 'default' ),
							'oldid' => 'bg_size',
							'type' => 'select',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'background-size',
							'title' => __('Size', 'mfn-opts'),
							'options' => mfna_bg_size()
						),

						array(
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Content', 'mfn-opts'),
   					),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .zoom_box .desc .desc_txt:color',
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .zoom_box .desc .desc_txt:typography',
							'type' => 'typography_vb',
							'label_after' => '<a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Typography', 'mfn-opts'),
						),

						array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .zoom_box .desc .desc_txt:margin',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .zoom_box .desc .desc_txt:margin_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .zoom_box .desc .desc_txt:margin_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .zoom_box .desc .desc_txt:margin_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'version' => 'separated-fields',
							'class' => 'mfn-toggled',
							'css_attr' => 'margin',
		  			),

						array(
							'type' => 'html',
							'html' => '</div>',
						),

   				),
   			),

   		);

   	}

		/**
		 * GET inline shortcodes for CodeMirror
		 */

		public static function get_inline_shortcode(){

			$shortcode_inline = array(

				'alert' => array(
					'type' => 'alert',
					'title' => __('Alert', 'mfn-opts'),
					'attr' => array(
					//'fields' => array(

						array(
							'id' => 'style',
							'type' => 'select',
							'title' => __('Style', 'mfn-opts'),
							'desc' => __('Please select the alert style above', 'mfn-opts'),
							'options' => array(
								'warning' => __('Warning', 'mfn-opts'),
								'error' => __('Error', 'mfn-opts'),
								'info' => __('Info', 'mfn-opts'),
								'success' => __('Success', 'mfn-opts'),
							),
							'std' => 'warning'
						),

						array(
							'id' => 'content',
							'type' => 'textarea',
							'title' => __('Content', 'mfn-opts'),
						)

					),
				),

				'blockquote' => array(
					'type' => 'blockquote',
					'title' => __('Blockquote', 'mfn-opts'),
					'attr' => array(
					//'fields' => array(

						array(
							'id' => 'author',
							'type' => 'text',
							'title' => __('Author', 'mfn-opts'),
							'desc' => __('Blockquote author', 'mfn-opts'),
						),

						array(
							'id' => 'link',
							'type' => 'text',
							'title' => __('Link', 'mfn-opts'),
							'desc' => __("Link to author's page", 'mfn-opts'),
						),

						array(
							'id' => 'target',
							'type' => 'select',
							'title' => __('Target', 'mfn-opts'),
							'desc' => __('Link target', 'mfn-opts'),
							'options' => array(
								'0' => __('Default', 'mfn-opts'),
								'_blank' => __('New tab or window', 'mfn-opts'),
								'lightbox' => __('Lightbox', 'mfn-opts'),
							),
							'std' => '0'
						),

						array(
							'id' => 'content',
							'type' => 'textarea',
							'title' => __('Content', 'mfn-opts'),
						),

					),
				),

				'button' => array(
					'type' => 'button',
					'title' => __('Button', 'mfn-opts'),
					'attr' => array(
					//'fields' => array(

						array(
							'id' => 'title',
							'type' => 'text',
							'title' => __('Button', 'mfn-opts'),
							'desc' => __('Text on the button', 'mfn-opts'),
							'std' => 'Button'
						),

						array(
							'id' => 'link',
							'type' => 'text',
							'title' => __('Link', 'mfn-opts'),
							'desc' => __('Link (with http://)', 'mfn-opts')
						),

						array(
							'id' => 'target',
							'type' => 'select',
							'title' => __('Target', 'mfn-opts'),
							'desc' => __('Link target', 'mfn-opts'),
							'options' => array(
								'0' => __('Default', 'mfn-opts'),
								'_blank' => __('New tab or window', 'mfn-opts'),
								'lightbox' => __('Lightbox', 'mfn-opts'),
							),
							'std' => '0'
						),

						array(
							'id' => 'align',
							'type' => 'switch',
							'title' => __('Align', 'mfn-opts'),
							'desc' => __('Button align', 'mfn-opts'),
							'options' => array(
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts')
							),
						),

						array(
							'id' => 'icon',
							'type' => 'icon',
							'title' => __('Icon', 'mfn-opts'),
							'desc' => __('To use a button with an icon, type the icon name here', 'mfn-opts'),
						),

						array(
							'id' => 'icon_position',
							'type' => 'switch',
							'title' => __('Icon position', 'mfn-opts'),
							'desc' => __('Icon position', 'mfn-opts'),
							'options' => array(
								'left' => __('Left', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts')
							),
						),

						array(
							'id' => 'color',
							'type' => 'color',
							'title' => __('Button background color', 'mfn-opts'),
							'alpha' => true
						),

						array(
							'id' => 'font_color',
							'type' => 'color',
							'title' => __('Button text color', 'mfn-opts'),
						),

						array(
							'id' => 'size',
							'type' => 'switch',
							'title' => __('Button Size', 'mfn-opts'),
							'options' => array(
								'1' => __('Small', 'mfn-opts'),
								'2' => __('Default', 'mfn-opts'),
								'3' => __('Large', 'mfn-opts'),
								'4' => __('Very Large', 'mfn-opts')
							),
							'std' => '2'
						),

						array(
							'id' => 'full_width',
							'type' => 'switch',
							'title' => __('Full Width', 'mfn-opts'),
							'desc' => __('Stretch to the width of the column', 'mfn-opts'),
							'options' => array(
								'' => __('No', 'mfn-opts'),
								'1' => __('Yes', 'mfn-opts')
							)
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Class', 'mfn-opts'),
							'desc' => __('CSS Classes', 'mfn-opts')
						),

						array(
							'id' => 'button_id',
							'type' => 'text',
							'title' => __('ID', 'mfn-opts'),
							'desc' => __('This option is useful when you want to use <b>GTM</b>', 'mfn-opts'),
						),

						array(
							'id' => 'rel',
							'type' => 'text',
							'title' => __('Rel', 'mfn-opts'),
							'desc' => __('"Rel" attribute to the link', 'mfn-opts'),
							'before' => 'rel='
						),

						array(
							'id' => 'download',
							'type' => 'text',
							'title' => __('Download', 'mfn-opts'),
							'desc' => __('Enter the new filename (if click on button downloads a file)', 'mfn-opts')
						),

						array(
							'id' => 'onclick',
							'type' => 'text',
							'title' => __('onclick', 'mfn-opts'),
							'desc' => __('"Onclick" attribute to the link', 'mfn-opts'),
							'before' => 'onclick='
						),

					),
				),

				'code' => array(
					'type' => 'code',
					'title' => __('Code', 'mfn-opts'),
					'attr' => array(
					//'fields' => array(

						array(
							'id' => 'content',
							'type' => 'textarea',
							'title' => __('Content', 'mfn-opts'),
							'desc' => __('Code inside the box', 'mfn-opts'),
						),

					),
				),

				'content_link' => array(
					'type' => 'content_link',
					'title' => __('Content link', 'mfn-opts'),
					'attr' => array(
					//'fields' => array(

						array(
							'id' => 'title',
							'type' => 'text',
							'title' => __('Title', 'mfn-opts'),
							'desc' => __('Item title text', 'mfn-opts'),
						),

						array(
							'id' => 'icon',
							'type' => 'icon',
							'title' => __('Icon', 'mfn-opts'),
							'std' => 'icon-lamp'
						),

						array(
							'id' => 'link',
							'type' => 'text',
							'title' => 'Link'
						),

						array(
							'id' => 'target',
							'type' => 'select',
							'title' => __('Target', 'mfn-opts'),
							'desc' => __('Link target', 'mfn-opts'),
							'options' => array(
								'0' => __('Default', 'mfn-opts'),
								'_blank' => __('New tab or window', 'mfn-opts')
							),
							'std' => '0'
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Class', 'mfn-opts'),
							'desc' => __('CSS classess', 'mfn-opts')
						),

						array(
							'id' => 'download',
							'type' => 'text',
							'title' => __('Download', 'mfn-opts'),
							'desc' => __('Enter the new filename (if click on button downloads a file)', 'mfn-opts')
						),

					),
				),

				'countdown_inline' => array(
					'type' => 'countdown_inline',
					'title' => __('Countdown Inline', 'mfn-opts'),
					'attr' => array(
					//'fields' => array(

						array(
   						'id' => 'date',
   						'type' => 'text',
   						'title' => __('Launch date', 'mfn-opts'),
   						'desc' => __('month/day/year hour:minute:second', 'mfn-opts'),
   						'std' => '12/30/2022 12:00:00',
							'preview' => 'title',
   					),

   					array(
   						'id' => 'timezone',
   						'type' => 'select',
   						'title' => __('Timezone', 'mfn-opts'),
   						'options' => mfna_utc(),
   						'std' => '0',
   					),

   					array(
   						'id' => 'show',
   						'type' => 'select',
   						'title' => __('Show', 'mfn-opts'),
   						'options' => array(
   							'' => __('days hours minutes seconds', 'mfn-opts'),
   							'dhm' => __('days hours minutes', 'mfn-opts'),
   							'dh' => __('days hours', 'mfn-opts'),
   							'd' => __('days', 'mfn-opts'),
   						),
   					),

					),
				),

				'counter_inline' => array(
					'type' => 'counter_inline',
					'title' => __('Counter Inline', 'mfn-opts'),
					'attr' => array(
					//'fields' => array(

						array(
							'id' => 'value',
							'type' => 'text',
							'title' => __('Value', 'mfn-opts'),
							'std' => 500
						),

						array(
							'id' => 'duration',
							'type' => 'text',
							'title' => __('Duration [ms]', 'mfn-opts'),
							'std' => ''
						),

					),
				),

				'divider' => array(
					'type' => 'divider',
					'title' => __('Divider', 'mfn-opts'),
					'attr' => array(
					//'fields' => array(

						array(
							'id' => 'height',
							'type' => 'text',
							'title' => __('Height', 'mfn-opts'),
							'param' => 'number',
							'after' => 'px',
							'std' => 30,
						),

						array(
							'id' => 'style',
							'type' => 'switch',
							'title' => __('Style', 'mfn-opts'),
							'options' => array(
								'default' => __('Default', 'mfn-opts'),
								'dots' => __('Dots', 'mfn-opts'),
								'zigzag' => __('ZigZag', 'mfn-opts'),
							),
							'std' => 'default',
						),

						array(
							'id' => 'line',
							'type' => 'switch',
							'title' => __('Line', 'mfn-opts'),
							'desc' => __('for style: default', 'mfn-opts'),
							'options' => array(
								'' => __('No line', 'mfn-opts'),
								'default' => __('Default', 'mfn-opts'),
								'narrow' => __('Narrow', 'mfn-opts'),
								'wide' => __('Wide', 'mfn-opts'),
							),
							'std' => '',
						),

						array(
							'id' => 'color',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
							'alpha' => true,
						),

						array(
							'id' => 'themecolor',
							'type' => 'switch',
							'title' => __('Theme color', 'mfn-opts'),
							'desc' => __('Overwrites color selected above', 'mfn-opts'),
							'options' => array(
								0 => __('No', 'mfn-opts'),
								1 => __('Yes', 'mfn-opts'),
							),
							'std' => 0
						),

						array(
		  				'id' => 'classes',
		  				'type' => 'pills',
		  				'title' => __('CSS classes', 'mfn-opts'),
		  			),

					),
				),

				'dropcap' => array(
					'type' => 'dropcap',
					'title' => __('Dropcap', 'mfn-opts'),
					'attr' => array(
					//'fields' => array(

						array(
							'id' => 'background',
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
							'alpha' => true
						),

						array(
							'id' => 'color',
							'type' => 'color',
							'title' => __('Text color', 'mfn-opts'),
						),

						array(
							'id' => 'circle',
							'type' => 'switch',
							'title' => __('Style', 'mfn-opts'),
							'options' => array(
								'0' => __('Square', 'mfn-opts'),
								'1' => __('Circle', 'mfn-opts')
							),
							'std' => '0'
						),

						array(
							'id' => 'size',
							'type' => 'text',
							'title' => __('Font size', 'mfn-opts'),
							'desc' => __('Size in px or 1, 2, 3 for predefined sizes', 'mfn-opts'),
							'class' => 'narrow',
							'param' => 'number',
							'after' => 'px',
							'std' => '1',
						),

						array(
							'id' => 'font',
							'type' => 'font_select',
							'title' => __('Font family', 'mfn-opts'),
							'param' => [
								'allow-empty' => true,
							],
							'std' => '',
						),

						array(
							'id' => 'content',
							'type' => 'textarea',
							'title' => __('Text of dropcap', 'mfn-opts')
						),

					),
				),

				'fancy_link' => array(
					'type' => 'fancy_link',
					'title' => __('Fancy Link', 'mfn-opts'),
					'attr' => array(
					//'fields' => array(

						array(
							'id' => 'title',
							'type' => 'text',
							'title' => __('Link text', 'mfn-opts'),
						),

						array(
							'id' => 'link',
							'type' => 'text',
							'title' => __('Link (with https://)', 'mfn-opts'),
						),

						array(
							'id' => 'target',
							'type' => 'select',
							'title' => __('Target', 'mfn-opts'),
							'desc' => __('Link target', 'mfn-opts'),
							'options' => array(
								'0' => __('Default', 'mfn-opts'),
								'_blank' => __('New tab or window', 'mfn-opts')
							)
						),

						array(
							'id' => 'style',
							'type' => 'select',
							'title' => __('Style', 'mfn-opts'),
							'desc' => __('Style of fancy link', 'mfn-opts'),
							'options' => array(
								'1' => __('Brackets', 'mfn-opts'),
								'2' => __('Block flipping', 'mfn-opts'),
								'3' => __('Border bottom', 'mfn-opts'),
								'4' => __('Border clone', 'mfn-opts'),
								'5' => __('Background shift', 'mfn-opts'),
								'6' => __('Color and border shift', 'mfn-opts'),
								'7' => __('Border position change', 'mfn-opts'),
								'8' => __('Borders to cross', 'mfn-opts'),
								'9' => __('Icon below link', 'mfn-opts'),
							),
							'std' => '1'
						),

						array(
							'id' => 'font',
							'type' => 'font_select',
							'title' => __('Font family', 'mfn-opts'),
							'param' => [
								'allow-empty' => true,
							],
						),

						array(
							'id' => 'font_size',
							'type' => 'text',
							'title' => __('Font size', 'mfn-opts'),
							'after' => 'px',
							'class' => 'narrow',
							'param' => 'number',
						),

						array(
							'id' => 'margin',
							'type' => 'text',
							'title' => __('Margin', 'mfn-opts'),
						),

						array(
							'id' => 'icon',
							'type' => 'icon',
							'title' => __('Icon', 'mfn-opts'),
							'desc' => __('for style <b>Icon below link</b> only', 'mfn-opts'),
						),

						array(
							'id' => 'class',
							'type' => 'pills',
							'title' => __('Class', 'mfn-opts'),
							'desc' => __('CSS classess', 'mfn-opts')
						),

						array(
							'id' => 'download',
							'type' => 'text',
							'title' => __('Download', 'mfn-opts'),
							'desc' => __('Enter the new filename (if click on button downloads a file)', 'mfn-opts')
						),

					),
				),

				'google_font' => array(
					'type' => 'google_font',
					'title' => __('Google Font', 'mfn-opts'),
					'attr' => array(
					//'fields' => array(

						array(
							'id' => 'font',
							'type' => 'text',
							'title' => __('Font', 'mfn-opts'),
							'desc' => __('Google font name<br><b>Important</b>: Works only with Google Fonts loaded from Google or Local for fonts selected in Theme Options', 'mfn-opts'),
							'std' => 'Open Sans'
						),

						array(
							'id' => 'size',
							'type' => 'text',
							'title' => __('Size', 'mfn-opts'),
							'desc' => __('Font size in px', 'mfn-opts')
						),

						array(
							'id' => 'weight',
							'type' => 'select',
							'title' => __('Weight', 'mfn-opts'),
							'desc' => __('Font weight (some fonts only)', 'mfn-opts'),
							'options' => array(
								'100' => __('100', 'mfn-opts'),
								'200' => __('200', 'mfn-opts'),
								'300' => __('300', 'mfn-opts'),
								'400' => __('400', 'mfn-opts'),
								'500' => __('500', 'mfn-opts'),
								'600' => __('600', 'mfn-opts'),
								'700' => __('700', 'mfn-opts'),
								'800' => __('800', 'mfn-opts')
							),
							'std' => '400'
						),

						array(
							'id' => 'italic',
							'type' => 'switch',
							'title' => __('Italic', 'mfn-opts'),
							'desc' => __('Font style: italic (some fonts only)', 'mfn-opts'),
							'options' => array(
								'0' => __('No', 'mfn-opts'),
								'1' => __('Yes', 'mfn-opts')
							),
							'std' => '0'
						),

						array(
							'id' => 'letter_spacing',
							'type' => 'text',
							'title' => __('Letter Spacing', 'mfn-opts'),
							'desc' => __('Letter spacing in px', 'mfn-opts')
						),

						array(
							'id' => 'color',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
							'desc' => __('Color of font', 'mfn-opts'),
							'std' => '#626262'
						),

						array(
							'id' => 'subset',
							'type' => 'text',
							'title' => __('Subset', 'mfn-opts'),
							'desc' => __('Subset for font (multiple separate with comma)', 'mfn-opts')
						),

						array(
							'id' => 'content',
							'type' => 'textarea',
							'title' => __('Content', 'mfn-opts'),
						),

					),
				),

				'heading' => array(
					'type' => 'heading',
					'title' => __('Heading', 'mfn-opts'),
					'attr' => array(
					//'fields' => array(

						array(
							'id' => 'tag',
							'type' => 'switch',
							'title' => __('Heading size', 'mfn-opts'),
							'options' => array(
								'h1' => __('H1', 'mfn-opts'),
								'h2' => __('H2', 'mfn-opts'),
								'h3' => __('H3', 'mfn-opts'),
								'h4' => __('H4', 'mfn-opts'),
								'h5' => __('H5', 'mfn-opts'),
								'h6' => __('H6', 'mfn-opts')
							),
							'std' => 'h2'
						),

						array(
							'id' => 'align',
							'type' => 'switch',
							'title' => __('Text align', 'mfn-opts'),
							'options' => array(
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts')
							),
							'std' => 'center'
						),

						array(
							'id' => 'color',
							'type' => 'color',
							'title' => 'Color',
							'std' => '#000'
						),

						array(
							'id' => 'style',
							'type' => 'switch',
							'title' => __('Style', 'mfn-opts'),
							'options' => array(
								'none' => __('Default', 'mfn-opts'),
								'lines' => __('Lines', 'mfn-opts')
							),
							'std' => 'lines'
						),

						array(
							'id' => 'color2',
							'type' => 'color',
							'title' => 'Color 2',
							'std' => '#000'
						),

						array(
							'id' => 'content',
							'type' => 'textarea',
							'title' => 'Content'
						),

					),
				),

				'highlight' => array(
					'type' => 'highlight',
					'title' => __('Highlight', 'mfn-opts'),
					'attr' => array(
					//'fields' => array(

						array(
							'id' => 'color',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'id' => 'background',
							'type' => 'color',
							'title' => __('Background color', 'mfn-opts'),
							'alpha' => true
						),

						array(
							'id' => 'style',
							'type' => 'switch',
							'title' => __('Style', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'underline' => __('Underline', 'mfn-opts'),
							],
						),

						array(
							'id' => 'content',
							'type' => 'textarea',
							'title' => __('Content', 'mfn-opts'),
						),

					)
				),

				'hr' => array(
					'type' => 'hr',
					'title' => __('Hr', 'mfn-opts'),
					'attr' => array(
					//'fields' => array(

						array(
							'id' => 'height',
							'type' => 'text',
							'title' => __('Height', 'mfn-opts'),
							'std' => 30
						),

						array(
							'id' => 'style',
							'type' => 'switch',
							'title' => __('Line style', 'mfn-opts'),
							'options' => array(
								'0' => __('Default', 'mfn-opts'),
								'dots' => __('Dots', 'mfn-opts'),
								'zigzag' => __('ZigZag', 'mfn-opts')
							),
							'std' => '0'
						),

						array(
							'id' => 'line',
							'type' => 'switch',
							'title' => __('Line width', 'mfn-opts'),
							'desc' => __('for style: default', 'mfn-opts'),
							'options' => array(
								'0' => __('Default', 'mfn-opts'),
								'narrow' => __('Narrow', 'mfn-opts'),
								'wide' => __('Wide', 'mfn-opts')
							),
							'std' => '0'
						),

						array(
							'id' => 'color',
							'type' => 'color',
							'title' => __('Line color', 'mfn-opts'),
						),

						array(
							'id' => 'themecolor',
							'type' => 'switch',
							'title' => __('Theme color', 'mfn-opts'),
							'desc' => 'Use the same color as theme color',
							'options' => array(
								'0' => __('No', 'mfn-opts'),
								'1' => __('Yes', 'mfn-opts')
							),
							'std' => '0'
						),

					),
				),

				'icon' => array(
					'type' => 'icon',
					'title' => __('Icon', 'mfn-opts'),
					'attr' => array(
					//'fields' => array(

						array(
							'id' => 'type',
							'type' => 'icon',
							'title' => 'Icon',
							'std' => 'icon-lamp'
						),

						array(
							'id' => 'color',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
						),

					),
				),

				'icon_bar' => array(
					'type' => 'icon_bar',
					'title' => __('Icon Bar', 'mfn-opts'),
					'attr' => array(
					//'fields' => array(

						array(
							'id' => 'icon',
							'type' => 'icon',
							'title' => __('Icon', 'mfn-opts'),
							'std' => 'icon-lamp'
						),

						array(
							'id' => 'link',
							'type' => 'text',
							'title' => __('Link', 'mfn-opts'),
						),

						array(
							'id' => 'target',
							'type' => 'select',
							'title' => __('Target', 'mfn-opts'),
							'options' => array(
								'0' => __('Default', 'mfn-opts'),
								'_blank' => __('New tab or window', 'mfn-opts'),
							),
							'std' => '0'
						),

						array(
							'id' => 'size',
							'type' => 'text',
							'title' => __('Size', 'mfn-opts'),
							'desc' => __('If you want large icon, then type large, leave field empty if you want default size', 'mfn-opts')
						),

						array(
							'id' => 'social',
							'type' => 'select',
							'title' => __('Social', 'mfn-opts'),
							'options' => array(
								'' => __('No', 'mfn-opts'),
								'dribbble' => __('Dribbble', 'mfn-opts'),
								'facebook' => __('Facebook', 'mfn-opts'),
								'flickr' => __('Flickr', 'mfn-opts'),
								'google' => __('Google', 'mfn-opts'),
								'instagram' => __('Instagram', 'mfn-opts'),
								'linkedin' => __('LinkedIn', 'mfn-opts'),
								'pinterest' => __('Pinterest', 'mfn-opts'),
								'twitter' => __('Twitter', 'mfn-opts'),
								'vimeo' => __('Vimeo', 'mfn-opts'),
								'youtube' => __('Youtube', 'mfn-opts'),
							),
						),

					),
				),

				'icon_block' => array(
					'type' => 'icon_block',
					'title' => __('Icon Block', 'mfn-opts'),
					'attr' => array(
					//'fields' => array(

						array(
							'id' => 'icon',
							'type' => 'icon',
							'title' => __('Icon', 'mfn-opts'),
							'std' => 'icon-lamp'
						),

						array(
							'id' => 'align',
							'type' => 'switch',
							'title' => __('Align', 'mfn-opts'),
							'options' => array(
								'0' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							),
						),

						array(
							'id' => 'color',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts')
						),

						array(
							'id' => 'size',
							'type' => 'text',
							'title' => __('Icon size', 'mfn-opts'),
							'std' => 25
						),

					),
				),

				'idea' => array(
					'type' => 'idea',
					'title' => __('Idea', 'mfn-opts'),
					'attr' => array(
					//'fields' => array(

						array(
							'id' => 'content',
							'type' => 'textarea',
							'title' => __('Content', 'mfn-opts')
						),

						array(
							'id' => 'icon',
							'type' => 'icon',
							'title' => __('Icon', 'mfn-opts'),
							'std' => 'icon-lamp'
						),

						array(
   						'id' => 'border_radius',
   						'type' => 'text',
   						'title' => __('Border radius', 'mfn-opts'),
   						'std' => '0',
   						'class' => 'narrow',
   						'param' => 'number',
   						'after' => 'px',
   					),

					),
				),

				'image' => array(
					'type' => 'image',
					'title' => __('Image', 'mfn-opts'),
					'attr' => array(
					//'fields' => array(

						array(
							'id' => 'src',
							'type' => 'upload',
							'title' => __('Image', 'mfn-opts')
						),

						array(
							'id' => 'align',
							'type' => 'switch',
							'title' => __('Align', 'mfn-opts'),
							'options' => array(
								'0' => __('Default', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
							)
						),

						array(
							'id' => 'border',
							'type' => 'switch',
							'title' => __('Border', 'mfn-opts'),
							'options' => array(
								'0' => __('No', 'mfn-opts'),
								'1' => __('Yes', 'mfn-opts'),
							),
							'std' => ''
						),

						array(
							'id' => 'margin_top',
							'type' => 'text',
							'title' => __('Margin Top in px', 'mfn-opts')
						),

						array(
							'id' => 'margin_bottom',
							'type' => 'text',
							'title' => __('Margin Bottom in px', 'mfn-opts')
						),

						array(
							'id' => 'link_image',
							'type' => 'upload',
							'title' => __('Image on click', 'mfn-opts'),
							'desc' => __('Link to image to open after click', 'mfn-opts')
						),

						array(
							'id' => 'link',
							'type' => 'text',
							'title' => __('Link', 'mfn-opts'),
							'desc' => __('URL to open on click instead of image url', 'mfn-opts')
						),

						array(
							'id' => 'target',
							'type' => 'select',
							'title' => __('Target', 'mfn-opts'),
							'desc' => __('Link target', 'mfn-opts'),
							'options' => array(
								'0' => __('Default', 'mfn-opts'),
								'_blank' => __('New tab or window', 'mfn-opts')
							)
						),

						array(
							'id' => 'hover',
							'type' => 'switch',
							'title' => __('Hover effect', 'mfn-opts'),
							'options' => array(
								'' => __('ON', 'mfn-opts'),
								'disable' => __('OFF', 'mfn-opts')
							)
						),

						array(
							'id' => 'alt',
							'type' => 'text',
							'title' => __('Alternate text', 'mfn-opts')
						),

						array(
							'id' => 'caption',
							'type' => 'text',
							'title' => __('Caption', 'mfn-opts'),
							'desc' => __('Short text under image', 'mfn-opts'),
						),

						array(
							'id' => 'greyscale',
							'type' => 'switch',
							'title' => __('Grayscale', 'mfn-opts'),
							'options' => array(
								'' => __('OFF', 'mfn-opts'),
								'1' => __('ON', 'mfn-opts')
							)
						),

						array(
							'id' => 'animate',
							'type' => 'select',
							'title' => __('Animation', 'mfn-opts'),
							'desc' => __('Entrance animation', 'mfn-opts'),
							'options' => array(
								'' => esc_html__('- Not Animated -', 'mfn-opts'),
								'fadeIn' => esc_html__('Fade in', 'mfn-opts'),
								'fadeInUp' => esc_html__('Fade in up', 'mfn-opts'),
								'fadeInDown' => esc_html__('Fade in down ', 'mfn-opts'),
								'fadeInLeft' => esc_html__('Fade in left', 'mfn-opts'),
								'fadeInRight' => esc_html__('Fade in right ', 'mfn-opts'),
								'fadeInUpLarge' => esc_html__('Fade in up large', 'mfn-opts'),
								'fadeInDownLarge' => esc_html__('Fade in down large', 'mfn-opts'),
								'fadeInLeftLarge' => esc_html__('Fade in left large', 'mfn-opts'),
								'fadeInRightLarge' => esc_html__('Fade in right large', 'mfn-opts'),
								'zoomIn' => esc_html__('Zoom in', 'mfn-opts'),
								'zoomInUp' => esc_html__('Zoom in up', 'mfn-opts'),
								'zoomInDown' => esc_html__('Zoom in down', 'mfn-opts'),
								'zoomInLeft' => esc_html__('Zoom in left', 'mfn-opts'),
								'zoomInRight' => esc_html__('Zoom in right', 'mfn-opts'),
								'zoomInUpLarge' => esc_html__('Zoom in up large', 'mfn-opts'),
								'zoomInDownLarge' => esc_html__('Zoom in down large', 'mfn-opts'),
								'zoomInLeftLarge' => esc_html__('Zoom in left large', 'mfn-opts'),
								'bounceIn' => esc_html__('Bounce in', 'mfn-opts'),
								'bounceInUp' => esc_html__('Bounce in up', 'mfn-opts'),
								'bounceInDown' => esc_html__('Bounce in down', 'mfn-opts'),
								'bounceInLeft' => esc_html__('Bounce in left', 'mfn-opts'),
								'bounceInRight' => esc_html__('Bounce in right', 'mfn-opts'),
							),
						),

					),
				),

				'popup' => array(
					'type' => 'popup',
					'title' => __('Popup', 'mfn-opts'),
					'attr' => array(
					//'fields' => array(

						array(
							'id' => 'title',
							'type' => 'text',
							'title' => __('Title', 'mfn-opts'),
							'desc' => __('Popup link text', 'mfn-opts')
						),

						array(
							'id' => 'padding',
							'type' => 'text',
							'title' => __('Padding', 'mfn-opts'),
							'desc' => __('Inner padding in px', 'mfn-opts')
						),

						array(
							'id' => 'button',
							'type' => 'switch',
							'title' => __('Button', 'mfn-opts'),
							'desc' => __('Use button instead of text link', 'mfn-opts'),
							'options' => array(
								'0' => __('No', 'mfn-opts'),
								'1' => __('Yes', 'mfn-opts')
							)
						),

						array(
							'id' => 'content',
							'type' => 'textarea',
							'title' => __('Content', 'mfn-opts')
						),

					),
				),

				'progress_icons' => array(
					'type' => 'progress_icons',
					'title' => __('Popup', 'mfn-opts'),
					'attr' => array(
					//'fields' => array(

						array(
							'id' => 'icon',
							'type' => 'icon',
							'title' => __('Icon', 'mfn-opts'),
							'std' => 'icon-lamp'
						),

						array(
							'id' => 'image',
							'type' => 'upload',
							'title' => __('Image url', 'mfn-opts'),
							'desc' => __('If you dont want to use icon you can enter image URL', 'mfn-opts')
						),

						array(
							'id' => 'count',
							'type' => 'text',
							'title' => __('Number of icons', 'mfn-opts'),
							'std' => 5
						),

						array(
							'id' => 'active',
							'type' => 'text',
							'title' => __('Number of active icons', 'mfn-opts'),
							'std' => 3
						),

						array(
							'id' => 'background',
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
							'alpha' => true
						),

						array(
							'id' => 'transparent',
							'type' => 'switch',
							'title' => __('Transparent background', 'mfn-opts'),
							'alpha' => true,
							'options' => [
								'0' => __('Disable', 'mfn-opts'),
								'1' => __('Enable', 'mfn-opts'),
							],
							'std' => '0',
						),

					),
				),

				'tooltip' => array(
					'type' => 'tooltip',
					'title' => __('Tooltip', 'mfn-opts'),
					'attr' => array(
					//'fields' => array(

						array(
							'id' => 'hint',
							'type' => 'text',
							'title' => __('Popup text', 'mfn-opts'),
							'std' => 'Text'
						),

						array(
							'id' => 'position',
							'type' => 'select',
							'title' => __('Position', 'mfn-opts'),
							'options' => [
								'top' => __('Top', 'mfn-opts'),
								'right' => __('Right', 'mfn-opts'),
								'bottom' => __('Bottom', 'mfn-opts'),
								'left' => __('Left', 'mfn-opts'),
							],
							'std' => 'top'
						),

						array(
							'id' => 'content',
							'type' => 'textarea',
							'title' => __('Content', 'mfn-opts'),
						),

					),
				),

				'tooltip_image' => array(
					'type' => 'tooltip_image',
					'title' => __('Tooltip_Image', 'mfn-opts'),
					'attr' => array(
					//'fields' => array(

						array(
							'id' => 'hint',
							'type' => 'text',
							'title' => __('Popup text', 'mfn-opts'),
							'std' => 'Text'
						),

						array(
							'id' => 'image',
							'type' => 'upload',
							'title' => __('Image URL', 'mfn-opts'),
						),

						array(
							'id' => 'content',
							'type' => 'textarea',
							'title' => __('Content', 'mfn-opts'),
						),

					),
				),

				'lorem' => array(
					'type' => 'lorem',
					'title' => __('Lorem ipsum', 'mfn-opts'),
					'attr' => array(
					//'fields' => array(

						array(
							'id' => 'type',
							'type' => 'switch',
							'title' => __('Type of listing', 'mfn-opts'),
							'options' => array(
								'paragraphs' => __('Paragraphs', 'mfn-opts'),
								'lists' => __('Lists', 'mfn-opts')
							),
							'std' => 'paragraphs'
						),

						array(
							'id' => 'rows_amount',
							'type' => 'text',
							'title' => __('Amount', 'mfn-opts'),
							'std' => 3
						),

						array(
							'id' => 'min_words_amount',
							'type' => 'text',
							'title' => __('Minimal words amount', 'mfn-opts'),
							'std' => 10
						),

						array(
							'id' => 'max_words_amount',
							'type' => 'text',
							'title' => __('Maximal words amount', 'mfn-opts'),
							'std' => 15
						),

					),
				),

			);

			return $shortcode_inline;
		}

		/**
		 * SET item entrance animations
		 */

		private function set_animations(){

			$this->animations = array(
				'' => esc_html__('- Not Animated -', 'mfn-opts'),
				'fadeIn' => esc_html__('Fade in', 'mfn-opts'),
				'fadeInUp' => esc_html__('Fade in up', 'mfn-opts'),
				'fadeInDown' => esc_html__('Fade in down ', 'mfn-opts'),
				'fadeInLeft' => esc_html__('Fade in left', 'mfn-opts'),
				'fadeInRight' => esc_html__('Fade in right ', 'mfn-opts'),
				'fadeInUpLarge' => esc_html__('Fade in up large', 'mfn-opts'),
				'fadeInDownLarge' => esc_html__('Fade in down large', 'mfn-opts'),
				'fadeInLeftLarge' => esc_html__('Fade in left large', 'mfn-opts'),
				'fadeInRightLarge' => esc_html__('Fade in right large', 'mfn-opts'),
				'zoomIn' => esc_html__('Zoom in', 'mfn-opts'),
				'zoomInUp' => esc_html__('Zoom in up', 'mfn-opts'),
				'zoomInDown' => esc_html__('Zoom in down', 'mfn-opts'),
				'zoomInLeft' => esc_html__('Zoom in left', 'mfn-opts'),
				'zoomInRight' => esc_html__('Zoom in right', 'mfn-opts'),
				'zoomInUpLarge' => esc_html__('Zoom in up large', 'mfn-opts'),
				'zoomInDownLarge' => esc_html__('Zoom in down large', 'mfn-opts'),
				'zoomInLeftLarge' => esc_html__('Zoom in left large', 'mfn-opts'),
				'bounceIn' => esc_html__('Bounce in', 'mfn-opts'),
				'bounceInUp' => esc_html__('Bounce in up', 'mfn-opts'),
				'bounceInDown' => esc_html__('Bounce in down', 'mfn-opts'),
				'bounceInLeft' => esc_html__('Bounce in left', 'mfn-opts'),
				'bounceInRight' => esc_html__('Bounce in right', 'mfn-opts'),
			);

		}

		/**
		 *
		 * ADVANCED section
		 *
		 * */

		public function set_advanced() {
    	$this->advanced = array(
      			array(
							'type' => 'html',
							'html' => '<div class="modalbox-card modalbox-card-advanced">',
						),

   					array(
   						'form' => 'advanced',
   						'type' => 'header',
   						'class' => 'toggled_header row-header',
   						'title' => __('Dimensions', 'mfn-opts'),
   					),

   					array(
   						'form' => 'advanced',
							'id' => 'width_switcher',
							'attr_id' => 'item_width_switcher_adv',
							'type' => 'select',
							'class' => 'mfn-toggled mfn-layout-modifier',
							'title' => __('Width', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'inline' => __('Inline', 'mfn-opts'),
		  					'custom' 	=> __('Custom', 'mfn-opts'),
		  				),
		  				'std' => 'default',
						),

   					array(
   						'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:flex',
							'type' => 'text',
							'default_unit' => 'px',
							'condition' => array( 'id' => 'item_width_switcher_adv', 'opt' => 'is', 'val' => 'custom' ),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled advanced_flex mfn-slider-input mfn-layout-modifier',
							'title' => __('Width', 'mfn-opts'),
							'desc' => __('Use px, %, vw, vh or auto to set content width', 'mfn-opts'),
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:flex_laptop',
							'type' => 'text',
							'default_unit' => 'px',
							'condition' => array( 'id' => 'item_width_switcher_adv', 'opt' => 'is', 'val' => 'custom' ),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled advanced_flex mfn-slider-input mfn-layout-modifier',
							'title' => __('Width', 'mfn-opts'),
							'desc' => __('Use px, %, vw, vh or auto to set content width', 'mfn-opts'),
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:flex_tablet',
							'type' => 'text',
							'default_unit' => 'px',
							'condition' => array( 'id' => 'item_width_switcher_adv', 'opt' => 'is', 'val' => 'custom' ),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled advanced_flex mfn-slider-input mfn-layout-modifier',
							'title' => __('Width', 'mfn-opts'),
							'desc' => __('Use px, %, vw, vh or auto to set content width', 'mfn-opts'),
						),
						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:flex_mobile',
							'type' => 'text',
							'default_unit' => 'px',
							'condition' => array( 'id' => 'item_width_switcher_adv', 'opt' => 'is', 'val' => 'custom' ),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled advanced_flex mfn-slider-input mfn-layout-modifier',
							'title' => __('Width', 'mfn-opts'),
							'desc' => __('Use px, %, vw, vh or auto to set content width', 'mfn-opts'),
						),

						array(
							'form' => 'advanced',
							'id' => 'height_switcher',
							'attr_id' => 'item_height_switcher_adv',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Height', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'custom' 	=> __('Custom', 'mfn-opts'),
		  				),
		  				'std' => 'default',
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:height',
							'type' => 'text',
							'default_unit' => 'px',
							'condition' => array( 'id' => 'item_height_switcher_adv', 'opt' => 'is', 'val' => 'custom' ),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Height', 'mfn-opts')
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:height_laptop',
							'type' => 'text',
							'default_unit' => 'px',
							'condition' => array( 'id' => 'item_height_switcher_adv', 'opt' => 'is', 'val' => 'custom' ),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Height', 'mfn-opts')
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:height_tablet',
							'type' => 'text',
							'default_unit' => 'px',
							'condition' => array( 'id' => 'item_height_switcher_adv', 'opt' => 'is', 'val' => 'custom' ),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Height', 'mfn-opts')
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:height_mobile',
							'type' => 'text',
							'default_unit' => 'px',
							'condition' => array( 'id' => 'item_height_switcher_adv', 'opt' => 'is', 'val' => 'custom' ),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled mfn-slider-input',
							'title' => __('Height', 'mfn-opts')
						),

		  			array(
		  				'form' => 'advanced',
		  				'type' => 'header',
		  				'class' => 'toggled_header row-header',
   						'title' => __('Spacing', 'mfn-opts'),
   					),

   					array(
   						'form' => 'advanced',
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:padding',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled mfn-layout-modifier',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'form' => 'advanced',
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:padding_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled mfn-layout-modifier',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'form' => 'advanced',
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:padding_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled mfn-layout-modifier',
							'css_attr' => 'padding',
		  			),

		  			array(
		  				'form' => 'advanced',
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:padding_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Padding', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled mfn-layout-modifier',
							'css_attr' => 'padding',
		  			),

						array(
							'form' => 'advanced',
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:margin',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled advanced_margin mfn-layout-modifier',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'form' => 'advanced',
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:margin_laptop',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled advanced_margin mfn-layout-modifier',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'form' => 'advanced',
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:margin_tablet',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled advanced_margin mfn-layout-modifier',
							'css_attr' => 'margin',
		  			),

		  			array(
		  				'form' => 'advanced',
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:margin_mobile',
		  				'type' => 'dimensions',
		  				'version' => 'separated-fields',
		  				'title' => __('Margin', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled advanced_margin mfn-layout-modifier',
							'css_attr' => 'margin',
		  			),

						array(
							'form' => 'advanced',
		  				'type' => 'header',
		  				'class' => 'toggled_header hide_if_ql row-header',
   						'title' => __('Positioning', 'mfn-opts'),
   					),

   					array(
   						'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:position',
							'attr_id' => 'item_position',
							'type' => 'select',
							'responsive' => 'desktop',
							'class' => 'mfn-toggled absolute-pos-watcher custom-element-position',
							'title' => __('Position', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'relative' => __('Offset', 'mfn-opts'),
								'absolute' => __('Absolute', 'mfn-opts'),
							],
						),

						array(
   						'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:position_laptop',
							'attr_id' => 'item_position_laptop',
							'type' => 'select',
							'responsive' => 'laptop',
							'class' => 'mfn-toggled absolute-pos-watcher custom-element-position',
							'title' => __('Position', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'unset' => __('Static', 'mfn-opts'),
								'relative' => __('Offset', 'mfn-opts'),
								'absolute' => __('Absolute', 'mfn-opts'),
							],
						),

						array(
   						'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:position_tablet',
							'attr_id' => 'item_position_tablet',
							'type' => 'select',
							'responsive' => 'tablet',
							'class' => 'mfn-toggled absolute-pos-watcher custom-element-position',
							'title' => __('Position', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'unset' => __('Static', 'mfn-opts'),
								'relative' => __('Offset', 'mfn-opts'),
								'absolute' => __('Absolute', 'mfn-opts'),
							],
						),

						array(
   						'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:position_mobile',
							'attr_id' => 'item_position_mobile',
							'type' => 'select',
							'responsive' => 'mobile',
							'class' => 'mfn-toggled absolute-pos-watcher custom-element-position',
							'title' => __('Position', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'unset' => __('Static', 'mfn-opts'),
								'relative' => __('Offset', 'mfn-opts'),
								'absolute' => __('Absolute', 'mfn-opts'),
							],
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:top',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'desktop',
							'condition' => array( 'id' => 'item_position', 'opt' => 'is', 'val' => 'relative,absolute' ),
							'title' => __('Top', 'mfn-opts'),
   						'class' => 'mfn-toggled custom-abs-offset-values',
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:bottom',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'desktop',
							'condition' => array( 'id' => 'item_position', 'opt' => 'is', 'val' => 'relative,absolute' ),
							'title' => __('Bottom', 'mfn-opts'),
   						'class' => 'mfn-toggled custom-abs-offset-values',
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:left',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'desktop',
							'condition' => array( 'id' => 'item_position', 'opt' => 'is', 'val' => 'relative,absolute' ),
							'title' => __('Left', 'mfn-opts'),
   						'class' => 'mfn-toggled custom-abs-offset-values',
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:right',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'desktop',
							'condition' => array( 'id' => 'item_position', 'opt' => 'is', 'val' => 'relative,absolute' ),
							'title' => __('Right', 'mfn-opts'),
   						'class' => 'mfn-toggled custom-abs-offset-values',
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:top_laptop',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'laptop',
							'condition' => array( 'id' => 'item_position_laptop', 'opt' => 'is', 'val' => 'relative,absolute' ),
							'title' => __('Top', 'mfn-opts'),
   						'class' => 'mfn-toggled custom-abs-offset-values',
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:bottom_laptop',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'laptop',
							'condition' => array( 'id' => 'item_position_laptop', 'opt' => 'is', 'val' => 'relative,absolute' ),
							'title' => __('Bottom', 'mfn-opts'),
   						'class' => 'mfn-toggled custom-abs-offset-values',
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:left_laptop',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'laptop',
							'condition' => array( 'id' => 'item_position_laptop', 'opt' => 'is', 'val' => 'relative,absolute' ),
							'title' => __('Left', 'mfn-opts'),
   						'class' => 'mfn-toggled custom-abs-offset-values',
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:right_laptop',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'laptop',
							'condition' => array( 'id' => 'item_position_laptop', 'opt' => 'is', 'val' => 'relative,absolute' ),
							'title' => __('Right', 'mfn-opts'),
   						'class' => 'mfn-toggled custom-abs-offset-values',
						),





						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:top_tablet',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'tablet',
							'condition' => array( 'id' => 'item_position_tablet', 'opt' => 'is', 'val' => 'relative,absolute' ),
							'title' => __('Top', 'mfn-opts'),
   						'class' => 'mfn-toggled custom-abs-offset-values',
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:bottom_tablet',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'tablet',
							'condition' => array( 'id' => 'item_position_tablet', 'opt' => 'is', 'val' => 'relative,absolute' ),
							'title' => __('Bottom', 'mfn-opts'),
   						'class' => 'mfn-toggled custom-abs-offset-values',
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:left_tablet',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'tablet',
							'condition' => array( 'id' => 'item_position_tablet', 'opt' => 'is', 'val' => 'relative,absolute' ),
							'title' => __('Left', 'mfn-opts'),
   						'class' => 'mfn-toggled custom-abs-offset-values',
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:right_tablet',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'tablet',
							'condition' => array( 'id' => 'item_position_tablet', 'opt' => 'is', 'val' => 'relative,absolute' ),
							'title' => __('Right', 'mfn-opts'),
   						'class' => 'mfn-toggled custom-abs-offset-values',
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:top_mobile',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'mobile',
							'condition' => array( 'id' => 'item_position_mobile', 'opt' => 'is', 'val' => 'relative,absolute' ),
							'title' => __('Top', 'mfn-opts'),
   						'class' => 'mfn-toggled custom-abs-offset-values',
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:bottom_mobile',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'mobile',
							'condition' => array( 'id' => 'item_position_mobile', 'opt' => 'is', 'val' => 'relative,absolute' ),
							'title' => __('Bottom', 'mfn-opts'),
   						'class' => 'mfn-toggled custom-abs-offset-values',
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:left_mobile',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'mobile',
							'condition' => array( 'id' => 'item_position_mobile', 'opt' => 'is', 'val' => 'relative,absolute' ),
							'title' => __('Left', 'mfn-opts'),
   						'class' => 'mfn-toggled custom-abs-offset-values',
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:right_mobile',
							'type' => 'text',
							'default_unit' => 'px',
							'responsive' => 'mobile',
							'condition' => array( 'id' => 'item_position_mobile', 'opt' => 'is', 'val' => 'relative,absolute' ),
							'title' => __('Right', 'mfn-opts'),
   						'class' => 'mfn-toggled custom-abs-offset-values',
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:z-index',
							'type' => 'text',
							'title' => __('Z-index', 'mfn-opts'),
							'param' => 'number',
   						'class' => 'narrow mfn-toggled',
							'preview' => 'number',
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:order',
							'type' => 'text',
							'title' => __('Order', 'mfn-opts'),
							'param' => 'number',
   						'class' => 'narrow mfn-toggled',
   						'responsive' => 'desktop',
							'preview' => 'number',
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:order_laptop',
							'type' => 'text',
							'title' => __('Order', 'mfn-opts'),
							'param' => 'number',
   						'class' => 'narrow mfn-toggled',
   						'responsive' => 'laptop',
							'preview' => 'number',
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:order_tablet',
							'type' => 'text',
							'title' => __('Order', 'mfn-opts'),
							'param' => 'number',
   						'class' => 'narrow mfn-toggled',
   						'responsive' => 'tablet',
							'preview' => 'number',
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:order_mobile',
							'type' => 'text',
							'title' => __('Order', 'mfn-opts'),
							'param' => 'number',
   						'class' => 'narrow mfn-toggled',
   						'responsive' => 'mobile',
							'preview' => 'number',
						),

						array(
							'form' => 'advanced',
		  				'type' => 'header',
		  				'class' => 'toggled_header row-header',
							'title' => __('Background', 'mfn-opts'),
						),

						array(
							'form' => 'advanced',
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:transition',
							'type' => 'sliderbar',
							'title' => __('Transition duration', 'mfn-opts'),
							'param' => array(
								'min' => '0',
								'max' => '3',
								'step' => '0.1',
								'unit' => 's',
							)
						),

						array(
							'form' => 'advanced',
							'id' => 'background_switcher',
							'attr_id' => 'background_switcher_adv',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-color',
							'condition' => array( 'id' => 'background_switcher_adv', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'form' => 'advanced',
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:gradient',
		  				'condition' => array( 'id' => 'background_switcher_adv', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'form' => 'advanced',
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-image',
		  				'condition' => array( 'id' => 'background_switcher_adv', 'opt' => 'is', 'val' => 'default' ),
		  				'oldid' => 'bg_color',
		  				'type' => 'upload',
		  				'title' => __('Image', 'mfn-opts'),
		  				'responsive' => 'desktop',

							'css_attr' => 'background-image',
		  			),

		  			array(
		  				'form' => 'advanced',
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-image_laptop',
		  				'condition' => array( 'id' => 'background_switcher_adv', 'opt' => 'is', 'val' => 'default' ),
		  				'oldid' => 'bg_image',
		  				'type' => 'upload',
		  				'title' => __('Image', 'mfn-opts'),
		  				'responsive' => 'laptop',

							'css_attr' => 'background-image',
		  			),

		  			array(
		  				'form' => 'advanced',
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-image_tablet',
		  				'condition' => array( 'id' => 'background_switcher_adv', 'opt' => 'is', 'val' => 'default' ),
		  				'oldid' => 'bg_image',
		  				'type' => 'upload',
		  				'title' => __('Image', 'mfn-opts'),
		  				'responsive' => 'tablet',

							'css_attr' => 'background-image',
		  			),

		  			array(
		  				'form' => 'advanced',
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-image_mobile',
		  				'condition' => array( 'id' => 'background_switcher_adv', 'opt' => 'is', 'val' => 'default' ),
		  				'type' => 'upload',
		  				'title' => __('Image', 'mfn-opts'),
		  				'responsive' => 'mobile',

							'css_attr' => 'background-image',
		  			),

		  			array(
		  				'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-repeat',
							'condition' => array( 'id' => 'background_switcher_adv', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'select',
							'responsive' => 'desktop',

							'css_attr' => 'background-repeat',
							'title' => __('Repeat', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'repeat' => __('Repeat', 'mfn-opts'),
								'no-repeat' => __('No repeat', 'mfn-opts'),
								'repeat-x' => __('Repeat X', 'mfn-opts'),
								'repeat-y' => __('Repeat Y', 'mfn-opts'),
							],
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-repeat_laptop',
							'condition' => array( 'id' => 'background_switcher_adv', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'select',
							'responsive' => 'laptop',

							'css_attr' => 'background-repeat',
							'title' => __('Repeat', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'repeat' => __('Repeat', 'mfn-opts'),
								'no-repeat' => __('No repeat', 'mfn-opts'),
								'repeat-x' => __('Repeat X', 'mfn-opts'),
								'repeat-y' => __('Repeat Y', 'mfn-opts'),
							],
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-repeat_tablet',
							'condition' => array( 'id' => 'background_switcher_adv', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'select',
							'responsive' => 'tablet',

							'css_attr' => 'background-repeat',
							'title' => __('Repeat', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'repeat' => __('Repeat', 'mfn-opts'),
								'no-repeat' => __('No repeat', 'mfn-opts'),
								'repeat-x' => __('Repeat X', 'mfn-opts'),
								'repeat-y' => __('Repeat Y', 'mfn-opts'),
							],
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-repeat_mobile',
							'condition' => array( 'id' => 'background_switcher_adv', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'select',
							'responsive' => 'mobile',

							'css_attr' => 'background-repeat',
							'title' => __('Repeat', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'repeat' => __('Repeat', 'mfn-opts'),
								'no-repeat' => __('No repeat', 'mfn-opts'),
								'repeat-x' => __('Repeat X', 'mfn-opts'),
								'repeat-y' => __('Repeat Y', 'mfn-opts'),
							],
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-position',
							'attr_id' => 'bgposopt',
							'condition' => array( 'id' => 'background_switcher_adv', 'opt' => 'is', 'val' => 'default' ),
							'oldid' => 'bg_position',
							'type' => 'select',
							'responsive' => 'desktop',
							'class' => 'bgposopt',
							'css_attr' => 'background-position',
							'title' => __('Position', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'center' => __('Center Center', 'mfn-opts'),
								'center left' => __('Center Left', 'mfn-opts'),
								'center right' => __('Center Right', 'mfn-opts'),
								'top left' => __('Top Left', 'mfn-opts'),
								'top right' => __('Top Right', 'mfn-opts'),
								'top center' => __('Top Center', 'mfn-opts'),
								'bottom left' => __('Bottom Left', 'mfn-opts'),
								'bottom right' => __('Bottom Right', 'mfn-opts'),
								'bottom center' => __('Bottom Center', 'mfn-opts'),
								'custom' => __('Custom', 'mfn-opts'),
							],
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-position_laptop',
							'attr_id' => 'bgposopt_laptop',
							'condition' => array( 'id' => 'background_switcher_adv', 'opt' => 'is', 'val' => 'default' ),
							'oldid' => 'bg_position',
							'type' => 'select',
							'responsive' => 'laptop',
							'class' => 'bgposopt',
							'css_attr' => 'background-position',
							'title' => __('Position', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'center' => __('Center Center', 'mfn-opts'),
								'center left' => __('Center Left', 'mfn-opts'),
								'center right' => __('Center Right', 'mfn-opts'),
								'top left' => __('Top Left', 'mfn-opts'),
								'top right' => __('Top Right', 'mfn-opts'),
								'top center' => __('Top Center', 'mfn-opts'),
								'bottom left' => __('Bottom Left', 'mfn-opts'),
								'bottom right' => __('Bottom Right', 'mfn-opts'),
								'bottom center' => __('Bottom Center', 'mfn-opts'),
								'custom' => __('Custom', 'mfn-opts'),
							],
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-position_tablet',
							'attr_id' => 'bgposopt_tablet',
							'condition' => array( 'id' => 'background_switcher_adv', 'opt' => 'is', 'val' => 'default' ),
							'oldid' => 'bg_position',
							'type' => 'select',
							'responsive' => 'tablet',
							'class' => 'bgposopt',
							'css_attr' => 'background-position',
							'title' => __('Position', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'center' => __('Center Center', 'mfn-opts'),
								'center left' => __('Center Left', 'mfn-opts'),
								'center right' => __('Center Right', 'mfn-opts'),
								'top left' => __('Top Left', 'mfn-opts'),
								'top right' => __('Top Right', 'mfn-opts'),
								'top center' => __('Top Center', 'mfn-opts'),
								'bottom left' => __('Bottom Left', 'mfn-opts'),
								'bottom right' => __('Bottom Right', 'mfn-opts'),
								'bottom center' => __('Bottom Center', 'mfn-opts'),
								'custom' => __('Custom', 'mfn-opts'),
							],
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-position_mobile',
							'attr_id' => 'bgposopt_mobile',
							'condition' => array( 'id' => 'background_switcher_adv', 'opt' => 'is', 'val' => 'default' ),
							'oldid' => 'bg_position',
							'type' => 'select',
							'responsive' => 'mobile',
							'class' => 'bgposopt',
							'css_attr' => 'background-position',
							'title' => __('Position', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'center' => __('Center Center', 'mfn-opts'),
								'center left' => __('Center Left', 'mfn-opts'),
								'center right' => __('Center Right', 'mfn-opts'),
								'top left' => __('Top Left', 'mfn-opts'),
								'top right' => __('Top Right', 'mfn-opts'),
								'top center' => __('Top Center', 'mfn-opts'),
								'bottom left' => __('Bottom Left', 'mfn-opts'),
								'bottom right' => __('Bottom Right', 'mfn-opts'),
								'bottom center' => __('Bottom Center', 'mfn-opts'),
								'custom' => __('Custom', 'mfn-opts'),
							],
						),





						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-position_v2',
							'type' => 'text',
							'condition' => array( 'id' => 'bgposopt', 'opt' => 'is', 'val' => 'custom' ),
							'title' => __('Custom background position', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled mfn-custom-bg-pos',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-position_laptop_v2',
							'type' => 'text',
							'condition' => array( 'id' => 'bgposopt_laptop', 'opt' => 'is', 'val' => 'custom' ),
							'title' => __('Custom background position', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled mfn-custom-bg-pos',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-position_tablet_v2',
							'type' => 'text',
							'condition' => array( 'id' => 'bgposopt_tablet', 'opt' => 'is', 'val' => 'custom' ),
							'title' => __('Custom background position', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled mfn-custom-bg-pos',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-position_mobile_v2',
							'type' => 'text',
							'condition' => array( 'id' => 'bgposopt_mobile', 'opt' => 'is', 'val' => 'custom' ),
							'title' => __('Custom background position', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled mfn-custom-bg-pos',
						),



























						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-attachment',
							'condition' => array( 'id' => 'background_switcher_adv', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'select',
							'responsive' => 'desktop',

							'css_attr' => 'background-attachment',
							'title' => __('Attachment', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'scroll' => __('Scroll', 'mfn-opts'),
								'fixed' => __('Fixed', 'mfn-opts'),
							],
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-attachment_laptop',
							'condition' => array( 'id' => 'background_switcher_adv', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'select',
							'responsive' => 'laptop',

							'css_attr' => 'background-attachment',
							'title' => __('Attachment', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'scroll' => __('Scroll', 'mfn-opts'),
								'fixed' => __('Fixed', 'mfn-opts'),
							],
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-attachment_tablet',
							'condition' => array( 'id' => 'background_switcher_adv', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'select',
							'responsive' => 'tablet',

							'css_attr' => 'background-attachment',
							'title' => __('Attachment', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'scroll' => __('Scroll', 'mfn-opts'),
								'fixed' => __('Fixed', 'mfn-opts'),
							],
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-attachment_mobile',
							'condition' => array( 'id' => 'background_switcher_adv', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'select',
							'responsive' => 'mobile',

							'css_attr' => 'background-attachment',
							'title' => __('Attachment', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'scroll' => __('Scroll', 'mfn-opts'),
								'fixed' => __('Fixed', 'mfn-opts'),
							],
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-size',
							'attr_id' => 'custombgsize',
							'condition' => array( 'id' => 'background_switcher_adv', 'opt' => 'is', 'val' => 'default' ),
							'oldid' => 'bg_size',
							'type' => 'select',
							'responsive' => 'desktop',
							'class' => 'custombgsize',
							'css_attr' => 'background-size',
							'title' => __('Size', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'auto' => __('Auto', 'mfn-opts'),
								'contain' => __('Contain', 'mfn-opts'),
								'cover' => __('Cover', 'mfn-opts'),
								'cover-ultrawide' => __('Cover, on ultrawide screens only > 1920px', 'mfn-opts'),
								'custom' => __('Custom', 'mfn-opts'),
							],
							//'options' => mfna_bg_size()
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-size_laptop',
							'attr_id' => 'custombgsize_laptop',
							'condition' => array( 'id' => 'background_switcher_adv', 'opt' => 'is', 'val' => 'default' ),
							'oldid' => 'bg_size',
							'type' => 'select',
							'responsive' => 'laptop',
							'class' => 'custombgsize',
							'css_attr' => 'background-size',
							'title' => __('Size', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'auto' => __('Auto', 'mfn-opts'),
								'contain' => __('Contain', 'mfn-opts'),
								'cover' => __('Cover', 'mfn-opts'),
								'cover-ultrawide' => __('Cover, on ultrawide screens only > 1920px', 'mfn-opts'),
								'custom' => __('Custom', 'mfn-opts'),
							],
							//'options' => mfna_bg_size()
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-size_tablet',
							'attr_id' => 'custombgsize_tablet',
							'condition' => array( 'id' => 'background_switcher_adv', 'opt' => 'is', 'val' => 'default' ),
							'oldid' => 'bg_size',
							'type' => 'select',
							'responsive' => 'tablet',
							'class' => 'custombgsize',
							'css_attr' => 'background-size',
							'title' => __('Size', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'auto' => __('Auto', 'mfn-opts'),
								'contain' => __('Contain', 'mfn-opts'),
								'cover' => __('Cover', 'mfn-opts'),
								'cover-ultrawide' => __('Cover, on ultrawide screens only > 1920px', 'mfn-opts'),
								'custom' => __('Custom', 'mfn-opts'),
							],
							//'options' => mfna_bg_size()
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-size_mobile',
							'attr_id' => 'custombgsize_mobile',
							'condition' => array( 'id' => 'background_switcher_adv', 'opt' => 'is', 'val' => 'default' ),
							'oldid' => 'bg_size',
							'type' => 'select',
							'responsive' => 'mobile',
							'class' => 'custombgsize',
							'css_attr' => 'background-size',
							'title' => __('Size', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'auto' => __('Auto', 'mfn-opts'),
								'contain' => __('Contain', 'mfn-opts'),
								'cover' => __('Cover', 'mfn-opts'),
								'cover-ultrawide' => __('Cover, on ultrawide screens only > 1920px', 'mfn-opts'),
								'custom' => __('Custom', 'mfn-opts'),
							],
							//'options' => mfna_bg_size()
						),






						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-size_v2',
							'type' => 'text',
							'default_unit' => 'px',
							'condition' => array( 'id' => 'custombgsize', 'opt' => 'is', 'val' => 'custom' ),
							'title' => __('Custom background size', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled mfn-custom-bg-size',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-size_laptop_v2',
							'type' => 'text',
							'default_unit' => 'px',
							'condition' => array( 'id' => 'custombgsize_laptop', 'opt' => 'is', 'val' => 'custom' ),
							'title' => __('Custom background size', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled mfn-custom-bg-size',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-size_tablet_v2',
							'type' => 'text',
							'default_unit' => 'px',
							'condition' => array( 'id' => 'custombgsize_tablet', 'opt' => 'is', 'val' => 'custom' ),
							'title' => __('Custom background size', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled mfn-custom-bg-size',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:background-size_mobile_v2',
							'type' => 'text',
							'default_unit' => 'px',
							'condition' => array( 'id' => 'custombgsize_mobile', 'opt' => 'is', 'val' => 'custom' ),
							'title' => __('Custom background size', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled mfn-custom-bg-size',
						),


						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:backdrop-filter',
							'type' => 'backdrop_filter',
							'label_after' => '<a href="#" class="mfn-option-btn mfn-option-blank btn-medium reset-big-field reset-backdrop-filter" data-tooltip="Reset"><span class="mfn-icon mfn-icon-reset"></span></a><a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Backdrop filter', 'mfn-opts'),
						),




						array(
							'form' => 'advanced',
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner|hover:transition',
							'type' => 'sliderbar',
							'title' => __('Transition duration', 'mfn-opts'),
							'param' => array(
								'min' => '0',
								'max' => '3',
								'step' => '0.1',
								'unit' => 's',
							)
						),

						array(
							'form' => 'advanced',
							'id' => 'background_switcher_hover',
							'attr_id' => 'background_switcher_hover_adv',
							'type' => 'switch',
							'class' => 'mfn-fields-switcher disable-history',
							'title' => __('Background type', 'mfn-opts'),
							'options' => array(
		  					'default' => __('Default', 'mfn-opts'),
		  					'gradient' 	=> __('Gradient', 'mfn-opts'),
		  				),
							'visual_options' => array(
		  					'default' => '',
		  					'gradient' 	=> '',
		  				),
		  				'std' => 'default',
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner|hover:background-color',
							'condition' => array( 'id' => 'background_switcher_hover_adv', 'opt' => 'is', 'val' => 'default' ),
							'type' => 'color',
							'title' => __('Color', 'mfn-opts'),
						),

						array(
							'form' => 'advanced',
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner|hover:gradient',
		  				'condition' => array( 'id' => 'background_switcher_hover_adv', 'opt' => 'is', 'val' => 'gradient' ),
		  				'type' => 'gradient',
		  				'title' => __('Gradient', 'mfn-opts'),
		  				'class' => 'mfn-toggled',
							'css_attr' => 'gradient',
		  			),

						array(
							'form' => 'advanced',
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner|hover:background-image',
		  				'condition' => array( 'id' => 'background_switcher_hover_adv', 'opt' => 'is', 'val' => 'default' ),
		  				'oldid' => 'bg_color',
		  				'type' => 'upload',
		  				'title' => __('Image', 'mfn-opts'),
		  				'responsive' => 'desktop',

							'css_attr' => 'background-image',
		  			),


		  			array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner|hover:backdrop-filter',
							'type' => 'backdrop_filter',
							'label_after' => '<a href="#" class="mfn-option-btn mfn-option-blank btn-medium reset-big-field reset-backdrop-filter" data-tooltip="Reset"><span class="mfn-icon mfn-icon-reset"></span></a><a href="#" title="Edit" class="mfn-option-btn mfn-option-blank btn-medium mfn-vb-label-button"><span class="mfn-icon mfn-icon-edit"></span></a>',
							'class' => 'mfn-toggled toggle_fields',
							'title' => __('Backdrop filter', 'mfn-opts'),
						),

						array(
							'form' => 'advanced',
							'type' => 'html',
							'html' => '</div></div>',
						),



						array(
							'form' => 'advanced',
							'type' => 'header',
							'class' => 'toggled_header row-header',
   						'title' => __('Border', 'mfn-opts'),
   					),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:border-style',
							'type' => 'select',
							'attr_id' => 'border_style_adv',
							'class' => 'mfn-toggled',
							'title' => __('Border style', 'mfn-opts'),
							'options' => [
								'none' => __('None', 'mfn-opts'),
								'solid' => __('Solid', 'mfn-opts'),
								'dashed' => __('Dashed', 'mfn-opts'),
								'dotted' => __('Dotted', 'mfn-opts'),
								'double' => __('Double', 'mfn-opts'),
							],
						),

						array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:border-color',
							'condition' => array( 'id' => 'border_style_adv', 'opt' => 'isnt', 'val' => 'none' ),
							'type' => 'color',
							'class' => 'mfn-toggled',
							'title' => __('Border color', 'mfn-opts'),
						),

						array(
							'form' => 'advanced',
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:border-width',
		  				'condition' => array( 'id' => 'border_style_adv', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'form' => 'advanced',
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:border-width_laptop',
		  				'condition' => array( 'id' => 'border_style_adv', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'form' => 'advanced',
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:border-width_tablet',
		  				'condition' => array( 'id' => 'border_style_adv', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

		  			array(
		  				'form' => 'advanced',
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:border-width_mobile',
		  				'condition' => array( 'id' => 'border_style_adv', 'opt' => 'isnt', 'val' => 'none' ),
		  				'type' => 'dimensions',
		  				'title' => __('Border width', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-width',
		  			),

						array(
							'form' => 'advanced',
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:border-radius',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'form' => 'advanced',
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:border-radius_laptop',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'form' => 'advanced',
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:border-radius_tablet',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'form' => 'advanced',
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:border-radius_mobile',
		  				'type' => 'dimensions',
		  				'title' => __('Border radius', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'border-radius',
		  			),

		  			array(
		  				'form' => 'advanced',
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:box-shadow',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'desktop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow'
		  			),

		  			array(
		  				'form' => 'advanced',
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:box-shadow_laptop',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'laptop',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'form' => 'advanced',
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:box-shadow_tablet',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'tablet',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
		  				'form' => 'advanced',
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:box-shadow_mobile',
		  				'type' => 'box_shadow',
		  				'title' => __('Box shadow', 'mfn-opts'),
							'responsive' => 'mobile',
							'class' => 'mfn-toggled',
							'css_attr' => 'box-shadow',
		  			),

		  			array(
							'form' => 'advanced',
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:overflow',
							'type' => 'select',
							'class' => 'mfn-toggled',
							'title' => __('Overflow', 'mfn-opts'),
							'options' => [
								'' => __('Default', 'mfn-opts'),
								'hidden' => __('Hidden', 'mfn-opts'),
								'auto' => __('Auto', 'mfn-opts'),
							],
						),

						array(
							'form' => 'advanced',
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Responsive', 'mfn-opts'),
						),

						array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Device type', 'mfn-opts'),
						),

						array(
							'form' => 'advanced',
		  				'id' => 'visibility',
		  				'iconinfo' => 'https://support.muffingroup.com/video-tutorials/section-responsive-visibility/',
		  				'type' => 'switch',
		  				'invert' => true,
		  				'version' => 'multiple',
		  				'class' => 'mfn-toggled',
		  				'title' => __('Responsive visibility', 'mfn-opts'),
		  				'options' => array(
		  					'hide-desktop' => __('Hide on Desktop', 'mfn-opts'),
		  					'hide-laptop' => __('Hide on Laptop', 'mfn-opts'),
		  					'hide-tablet' => __('Hide on Tablet', 'mfn-opts'),
		  					'hide-mobile' => __('Hide on Mobile', 'mfn-opts'),
		  				),
		  				'active_tooltip' => array(
		  					'hide-desktop' => __('Show on Desktop', 'mfn-opts'),
		  					'hide-laptop' => __('Show on Laptop', 'mfn-opts'),
		  					'hide-tablet' => __('Show on Tablet', 'mfn-opts'),
		  					'hide-mobile' => __('Show on Mobile', 'mfn-opts'),
		  				),
		  				'visual_options' => array(
		  					'hide-desktop' => '<svg id="Layer_1" data-name="Layer 1" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 22 22"><defs><style>.cls-1{fill:none;stroke:#304050;stroke-linecap:round;stroke-linejoin:round;stroke-width:1.5px;}.cls-2{fill:#304050;}</style></defs><path class="cls-1" d="M4.94,5H17.06a.93.93,0,0,1,.94.91v7.18a.93.93,0,0,1-.94.91H4.94A.93.93,0,0,1,4,13.09V5.91A.93.93,0,0,1,4.94,5Z"/><path class="cls-1" d="M7,17h8"/><path class="cls-2" d="M13,15.51a1.58,1.58,0,0,0,.7,1.26L14,17H8l.3-.23A1.58,1.58,0,0,0,9,15.51V14h4Z"/></svg>',			// 960 +
		  					'hide-laptop' => '<svg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"viewBox="0 0 22 22" style="enable-background:new 0 0 22 22;" xml:space="preserve"> <style type="text/css"> .st0{fill:none;stroke:currentColor;stroke-width:1.5;stroke-linecap:round;stroke-linejoin:round;} </style> <path class="st0" d="M5,15V6.9C5,6.4,5.4,5.9,5.9,6h10.1c0.5,0,0.9,0.4,0.9,0.9V15"/> <path class="st0" d="M4,17h14"/> </svg>',		// 768 - 959
		  					'hide-tablet' => '<svg id="Layer_1" data-name="Layer 1" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 22 22"><defs><style>.cls-1{fill:none;stroke:#304050;stroke-linecap:round;stroke-linejoin:round;stroke-width:1.5px;}</style></defs><path class="cls-1" d="M7,15.83V7.17A1.21,1.21,0,0,1,8.25,6h5.5A1.21,1.21,0,0,1,15,7.17v8.66A1.21,1.21,0,0,1,13.75,17H8.25A1.21,1.21,0,0,1,7,15.83Z"/></svg>',		// 768 - 959
		  					'hide-mobile' => '<svg id="Layer_1" data-name="Layer 1" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 22 22"><defs><style>.cls-1{fill:none;stroke:#304050;stroke-linecap:round;stroke-linejoin:round;stroke-width:1.5px;}.cls-2{fill:#304050;}</style></defs><path class="cls-1" d="M8,15.82V8.18A1.08,1.08,0,0,1,8.94,7h4.12A1.08,1.08,0,0,1,14,8.18v7.64A1.08,1.08,0,0,1,13.06,17H8.94A1.08,1.08,0,0,1,8,15.82Z"/><circle class="cls-2" cx="11" cy="14" r="1"/></svg>',			// - 768
		  				),
		  			),

		  			array(
							'type' => 'subheader',
							'class' => 'mfn-builder-subheader row-header mfn-toggled',
							'title' => __('Custom', 'mfn-opts'),
						),

						array(
							'form' => 'advanced',
		  				'id' => 'custom-responsive',
		  				'attr_id' => 'custom-responsive',
		  				'type' => 'switch',
		  				'class' => 'mfn-toggled',
		  				'title' => __('Custom visibility', 'mfn-opts'),
		  				'options' => array(
		  					'hide' => __('Hide under', 'mfn-opts'),
		  					'show' => __('Show under', 'mfn-opts'),
		  				),
		  				'std' => 'hide'
		  			),

						array(
		  				'form' => 'advanced',
		  				'condition' => array( 'id' => 'custom-responsive', 'opt' => 'is', 'val' => 'hide' ),
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:hide_under_custom',
		  				'type' => 'sliderbar',
		  				'class' => 'mfn-toggled responsive-custom-visibility',
							'title' => __('Hide under', 'mfn-opts'),
							'param' => array(
								'min' => '400',
								'max' => '1920',
								'step' => '1',
								'unit' => 'px',
							),
		  			),

		  			array(
		  				'form' => 'advanced',
		  				'condition' => array( 'id' => 'custom-responsive', 'opt' => 'is', 'val' => 'show' ),
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:show_under_custom',
		  				'type' => 'sliderbar',
		  				'title' => __('Show under', 'mfn-opts'),
							'class' => 'mfn-toggled responsive-custom-visibility',
							'param' => array(
								'min' => '400',
								'max' => '1920',
								'step' => '1',
								'unit' => 'px',
							),
		  			),

		  			array(
		  				'form' => 'advanced',
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Animation', 'mfn-opts'),
						),

		  			array(
		  				'form' => 'advanced',
   						'id' => 'animate',
   						'attr_id' => 'animate',
   						'type' => 'select',
   						'title' => __('Animation', 'mfn-opts'),
   						'desc' => __('Entrance animation', 'mfn-opts'),
   						'class' => 'mfn-toggled',
   						'options' => $this->get_animations(),
   					),

   					array(
		  				'form' => 'advanced',
		  				'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement:animation-delay',
		  				'condition' => array( 'id' => 'animate', 'opt' => 'isnt', 'val' => '' ),
		  				'type' => 'sliderbar',
		  				'class' => 'mfn-toggled',
							'title' => __('Animation delay', 'mfn-opts'),
							'after' => 'ms',
							'param' => array(
								'min' => '0',
								'max' => '3000',
								'step' => '1',
								'unit' => 'ms'
							),
		  			),

						// Transform

            array(
							'form' => 'advanced',
							'type' => 'header',
							'class' => 'toggled_header row-header transform_options',
							'title' => __('Transform', 'mfn-opts'),
						),

						array(
							'form' => 'advanced',
							'type' => 'html',
							'html' => '<div class="mfn-form-row mfn-sidebar-fields-tabs mfn-toggled mfn-vb-formrow mfn-vb-mfnuidhere"><ul class="mfn-sft-nav"><li class="active"><a href="#normal" data-tab="normal">Normal</a></li><li><a href="#hover" data-tab="hover">Hover</a></li></ul><div class="mfn-sft mfn-sft-normal mfn-tabs-fields-active">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:transform',
							'type' => 'transform',
							'js_options' => 'transform_options',
							'std' => '0'
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:transform-origin',
							'type' => 'select',
							'title' => __('Transform origin', 'mfn-opts'),
							'options' => array(
								'' => __('Default', 'mfn-opts'),
								'top left' => __('Top Left', 'mfn-opts'),
								'top center' => __('Top Center', 'mfn-opts'),
								'top right' => __('Top Right', 'mfn-opts'),
								'center left' => __('Center Left', 'mfn-opts'),
								'center' => __('Center', 'mfn-opts'),
								'center right' => __('Center Right', 'mfn-opts'),
								'bottom left' => __('Bottom Left', 'mfn-opts'),
								'bottom center' => __('Bottom Center', 'mfn-opts'),
								'bottom right' => __('Bottom Right', 'mfn-opts'),
							),
						),

						array(
							'form' => 'advanced',
							'type' => 'html',
							'html' => '</div><div class="mfn-sft mfn-sft-hover">',
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner:transition',
							'type' => 'sliderbar',
							'title' => __('Transition duration', 'mfn-opts'),
							'js_options' => 'transform_options',
							'param' => array(
								'min' => '0',
								'max' => '3',
								'step' => '0.1',
								'unit' => 's',
							)
						),

						array(
							'id' => 'style:.mcb-section .mcb-wrap .mcb-item-mfnuidelement .mcb-column-inner|hover:transform',
							'type' => 'transform',
							'class' => 'transform_hover',
							'title' => __('Transform options', 'mfn-opts'),
							'js_options' => 'transform_options',
							'std' => '0'
						),

		  			array(
							'type' => 'html',
							'html' => '</div></div>',
						),

						// Conditions

						array(
							'form' => 'advanced',
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Conditional logic', 'mfn-opts'),
						),

						array(
							'form' => 'advanced',
		  				'id' => 'conditions',
		  				'class' => 'mfn-toggled conditional_logic',
		  				'type' => 'logic',
		  				'title' => __('Conditional logic', 'mfn-opts'),
		  			),

						// Custom

						array(
							'form' => 'advanced',
							'type' => 'header',
							'class' => 'toggled_header row-header',
							'title' => __('Custom', 'mfn-opts'),
						),

						array(
							'form' => 'advanced',
		  				'id' => 'classes',
		  				'class' => 'mfn-toggled',
		  				'type' => 'pills',
		  				'title' => __('CSS classes', 'mfn-opts'),
		  			),

		  			array(
		  				'form' => 'advanced',
		  				'id' => 'custom_id',
		  				'class' => 'mfn-toggled',
		  				'type' => 'text',
		  				'title' => __('Custom ID', 'mfn-opts'),
		  			),

		  			array(
		  				'form' => 'advanced',
		  				'id' => 'custom_css', // changed from style: because of another style inputs in widgets
		  				'oldid' => 'style',
		  				'class' => 'mfn-toggled',
		  				'type' => 'textarea',
		  				'title' => __('Additional CSS', 'mfn-opts'),
		  			),

		  			array(
							'type' => 'html',
							'html' => '</div>',
						),
      );
    }

  }
}
